<?php

namespace Teknisa\FIN\Modules\ContaPagar\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Model\Financeiro\Entities\Adiantapagar;
use Teknisa\Model\Financeiro\Entities\Devolucaopag;
use Teknisa\Model\Util\Novocodigo;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Geral\Entities\Fornecedor;
use Teknisa\Model\Fiscal\Entities\Notafiscal;
use Teknisa\Model\Financeiro\Entities\Ajctpagar;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Rattpctp;
use Teknisa\Model\Financeiro\Entities\Ratctpagar;
use Teknisa\Model\Financeiro\Entities\Movfluxo;
use Teknisa\Model\Financeiro\Entities\Ccorforn;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\FIN\Modules\ContaPagar\Factory\TitulosAPagarFactory;
use Teknisa\FIN\Modules\FluxoCaixa\Services\FluxoDeCaixaService;
use Teknisa\FIN\Modules\ContaPagar\Services\ProrrogacaoTituloService;
use Teknisa\FIN\Modules\ContaPagar\Services\RemessaTituloService;
use Teknisa\FIN\Modules\ContaPagar\Services\BaixaTituloService;
use Teknisa\FIN\Modules\ContaPagar\Services\AssociacaoTituloService;
use Teknisa\FIN\Modules\ContaPagar\Services\ParcelamentoTituloService;
use Teknisa\FIN\Services\ContabilizacaoService;
use Teknisa\Libs\Util\NewCode;
//use Zeedhi\FileServerAPI\API;

/**
 * Class TitulosAPagarService
 * @package Teknisa\FIN\Modules\ContaPagar\Services
 */
class TitulosAPagarService {

    const BOLETO_ALTERADO = '3';
    const PERMITE_ALTERACAO_BOLETO = '2';
    const EM_ABERTO = 'A';
    const SIM = 'S';

    protected $entityManager;
    protected $environment;
    protected $titulosAPagarFactory;
    protected $fluxoDeCaixaService;
    protected $contabilizacaoService;
    protected $prorrogacaoTituloService;
    protected $remessaTituloService;
    protected $baixaTituloService;
    protected $associacaoTituloService;
    protected $ajusteTituloService;
    protected $parcelamentoTituloService;
    protected $newCode;
    protected $fileUploader;
    protected $_fileServerUrl;
    
    public $returnFileServer;
    

    /**
     * @param EntityManager            $entityManager
     * @param TitulosAPagarFactory     $titulosAPagarFactory
     * @param FluxoDeCaixaService      $fluxoDeCaixaService
     * @param ContabilizacaoService    $contabilizacaoService
     * @param ProrrogacaoTituloService $prorrogacaoTituloService
     * @param RemessaTituloService     $remessaTituloService
     * @param BaixaTituloService       $baixaTituloService
     * @param AssociacaoTituloService  $associacaoTituloService
     * @param AjusteTituloService  $ajusteTituloService
     * @param NewCode                  $newCode
     */
    public function __construct(
        EntityManager $entityManager, Environment $environment, TitulosAPagarFactory $titulosAPagarFactory, FluxoDeCaixaService $fluxoDeCaixaService,
        ContabilizacaoService $contabilizacaoService, ProrrogacaoTituloService $prorrogacaoTituloService,
        RemessaTituloService $remessaTituloService, BaixaTituloService $baixaTituloService, AssociacaoTituloService $associacaoTituloService,
        AjusteTituloService $ajusteTituloService, ParcelamentoTituloService $parcelamentoTituloService, NewCode $newCode,
        $fileUploader, $fileServerUrl)
    {
        $this->entityManager             = $entityManager;
        $this->environment               = $environment;
        $this->titulosAPagarFactory      = $titulosAPagarFactory;
        $this->fluxoDeCaixaService       = $fluxoDeCaixaService;
        $this->contabilizacaoService     = $contabilizacaoService;
        $this->prorrogacaoTituloService  = $prorrogacaoTituloService;
        $this->remessaTituloService      = $remessaTituloService;
        $this->baixaTituloService        = $baixaTituloService;
        $this->associacaoTituloService   = $associacaoTituloService;
        $this->ajusteTituloService       = $ajusteTituloService;
        $this->parcelamentoTituloService = $parcelamentoTituloService;
        $this->newCode                   = $newCode;
        $this->fileUploader              = $fileUploader;
        $this->_fileServerUrl            = $fileServerUrl;
    }

    /**
     * Gera Título a Pagar
     * @param $row
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTituloAPagar($row, $response, $permiteParcela = true){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::geraTituloAPag($row, $response, $permiteParcela);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Busca Todos Ajustes Boletos
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public function buscaTodosAjustesBoletos($boletos) {
        $sql = $this->montaSqlAjuste($boletos);
        return $this->titulosAPagarFactory->buscaTodosAjustesBoletos($sql);
    }
    
    /**
     * Monta SQL Ajuste
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    private function montaSqlAjuste($cdFilialLanBolPag) {
        $sql = '(';
        if($cdFilialLanBolPag) {
            foreach($cdFilialLanBolPag as $value) {
                $value = $value["CDFILIAL"].$value["NRLANBOLPAG"];
                $sql.= "'".$value."'".',';
            }
        }
        $size = strlen($sql);
        return substr($sql,0, $size-1).')';
    }
    
    

    /**
     * Gera Título a Pagar
     * @param $row
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTituloAPag($row, $response, $permiteParcela = true){
        try {
            $nrorg = $row['NRORG']; $cdempresa = $row['CDEMPRESA'];
            $cdinscesta = $row['CDINSCESTA']; $cdinscmuni = $row['CDINSCMUNI'];
            $cdfilirece = $row['CDFILIRECE']; $idtpijurnf = $row['IDTPIJURNF'];
            $nrinsjurnf = $row['NRINSJURNF'];$cdforneced = $row['CDFORNECED'];
            $idbloqueado = $row['IDBLOQUEADO']; $iddocfiscal = $row['IDDOCFISCAL'];
            $iddocfinanc = $row['IDDOCFINANC']; $vrprodutos = $row['VRPRODUTOS'];
            $dtliberacaofin = $row['DTLIBERACAOFIN'];  $vrnotafisc = $row['VRNOTAFISC'];
            $nrnotafisc = $row['NRNOTAFISC']; $dtemissao = $row['DTEMISSAO'];
            $dtentrsaid = $row['DTENTRSAID']; $identrsaidop = $row['IDENTRSAIDOP'];
            $cdtipooper = $row['CDTIPOOPER']; $cdserie = $row['CDSERIE'];
            $iddoccontab = $row['IDDOCCONTAB']; $idstatus = $row['IDSTATUS'];
            $idsistema = $row['IDSISTEMA']; $dtlancamennf = $row['DTLANCAMENNF'];
            $nrlanbolpag = $row['NRLANBOLPAG']; $vrbolpag = $row['VRBOLPAG'];
            $vrfinalbpag = $row['VRFINALBPAG']; $dtvenbolpag = $row['DTVENBOLPAG'];
            $cdclassfina = $row['CDCLASSFINA']; $dtmovfluxo = $row['DTMOVFLUXO'];
            $idmovfluxo = $row['IDMOVFLUXO']; $nrseqvenpag = $row['NRSEQVENPAG'];
            $nrparcelapag = $row['NRPARCELAPAG']; $dtorigvenpag = $row['DTORIGVENPAG'];
            $vratuapag = $row['VRATUAPAG']; $cdtpctpagar = $row['CDTPCTPAGAR'];
            $dtatuavenpag = $row['DTATUAVENPAG'];  $vrorigpag = $row['VRORIGPAG'];
            $cdfilibpag = $row['CDFILIBPAG']; $idstatpag = $row['IDSTATPAG'];
            $qtdiasvenpag = $row['QTDIASVENPAG']; $cdfimovflpag = $row['CDFIMOVFLPAG'];
            $dscontapag = $row['DSCONTAPAG']; $cdoperactpg = $row['CDOPERACTPG'];
            $cdfililanc = null;

            $parcela = $this->titulosAPagarFactory->buscaParcela($cdfilirece, $nrnotafisc, $cdserie, $nrinsjurnf, $vrnotafisc, $dtemissao->format("d/m/Y H:i:s"), $nrorg);
            if($permiteParcela && $parcela){
                $cdfilial = $parcela['CDFILIAL'];
                $nrlanctonf = $parcela['NRLANCTONF'];
                $sequecialNotaFiscal = $this->titulosAPagarFactory->getSequencialNotaFiscal($cdfilial, $nrlanctonf, $nrorg);
                $nrseqvenpag = strval($sequecialNotaFiscal["QUANT"] + 1);
                if(strlen($nrseqvenpag) < 2){
                    $nrseqvenpag = '0'.$nrseqvenpag;
                }
                if(strlen($nrseqvenpag) > 2){
                    throw new \Exception("O número limite de parcelas é 99");
                }
            }
            else{
                $cdfilial = $row['CDFILIAL'];
                $nrlanctonf = $this->geraNovoCodigo($nrorg, 'LANCNF/FILIAL'.$cdfilial);
                // Gera Nota Fiscal Pagar
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->insereNotaFiscalPagar(
                    $nrorg, $cdfilial, $nrlanctonf, $cdempresa, $cdinscesta, $cdinscmuni, $cdfilirece,
                    $idtpijurnf, $nrinsjurnf, $cdforneced, $idbloqueado, $iddocfiscal, $iddocfinanc,
                    $vrprodutos, $dtliberacaofin, $vrnotafisc, $nrnotafisc, $dtemissao, $dtentrsaid, $identrsaidop,
                    $cdtipooper, $cdserie, $iddoccontab, $idstatus, $idsistema, $dtlancamennf, $cdoperactpg
                );
                $this->entityManager->persist($notafiscal);
            }

            // Atualiza Fluxo de Caixa
            /** @var Movfluxo $movfluxo */
            $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                $nrorg, $cdempresa, $cdfilirece, $cdclassfina, $dtatuavenpag, $vratuapag, $idmovfluxo, 'S'
            );
            $this->entityManager->persist($movfluxo);
            
            /** @var GeralPag $geralpag */
            $geralpag = $this->titulosAPagarFactory->buscaGeralPag($nrorg);
            
            $idhabibpag = $geralpag->getIdhabibole() == 'S'?$geralpag->getIdutilpadrbolhab():'S';
            
            // Gera Boleto Pagar
            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->geraBoletoPagar(
                $nrorg, $cdfilibpag, $nrlanbolpag, $cdforneced, $cdempresa, $vrbolpag, $vrfinalbpag, $dtvenbolpag, $idhabibpag, $cdfililanc, $cdfilirece
            );
            $this->entityManager->persist($boletpagar);
            $this->entityManager->flush();

            // Gera Conta Pagar
            /** @var Contapagar $contapagar */
            $contapagar = $this->titulosAPagarFactory->insereContaPagar(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $dtorigvenpag,
                $vratuapag, $cdtpctpagar, $dtatuavenpag, $vrorigpag, $cdfilibpag, $nrlanbolpag,
                $idstatpag, $qtdiasvenpag, $cdclassfina, $cdempresa, $dtmovfluxo,
                $idmovfluxo, $cdfimovflpag, $dscontapag
            );
            $this->entityManager->persist($contapagar);
            $this->entityManager->flush();

            // Verifica Status da Nota Fiscal
            if($idstatus != 'V') {

                // Gera Rateio Conta Pagar
                /** @var Ratctpagar $ratctpagar */
                $ratctpagar = $this->geraRateioPag(
                    $nrorg, $cdempresa, $cdfilial, $cdfilirece, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdtpctpagar, $vrorigpag
                );

                foreach($ratctpagar as $rateio)
                    $this->entityManager->persist($rateio);

                // Gera Contabilização Conta Pagar
                $this->contabilizacaoService->contabilizaPagamento(
                    $cdfilial, $cdempresa, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $idstatpag, $cdtpctpagar
                );
            }

            $this->entityManager->flush();
        } catch (\Exception $e) {
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Gera Títulos a Pagar
     * @param $rows
     * @param $response
     * @param bool $permiteParcela
     * @throws \Exception
     */
    public function geraTitulosAPagar($rows, $response, $permiteParcela = true){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try{
            foreach ($rows as $row) {
                self::geraTituloAPag($row, $response, $permiteParcela);
            }
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Atualiza Titulo Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrnotafisc
     * @param $dtemissao
     * @param $dtentrsaid
     * @param $nrlanctonf
     * @param $cdserieentr
     * @param $idstatus
     * @param $idmovfluxo
     * @param $cdclassfina
     * @param $cdtpctpagar
     * @param $dscontapag
     * @param $response
     * @throws \Exception
     */
    public function atualizaTituloAPagar(
        $nrorg, $cdfilial, $nrnotafisc, $dtemissao, $dtentrsaid, $nrlanctonf,
        $cdserieentr, $idstatus, $idmovfluxo, $cdclassfina,
        $cdtpctpagar, $vrnotafisc, $dscontapag, $response
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Busca Nota Fiscal
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAPagarFactory->buscaNotaFiscal(
                $nrorg, $cdfilial, $nrlanctonf
            );

            // Atualiza Nota Fiscal
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAPagarFactory->atualizaNotaFiscal(
                $notafiscal, $nrnotafisc, $dtemissao, $dtentrsaid, $cdserieentr, $idstatus, null, $vrnotafisc, $vrnotafisc
            );
            $this->entityManager->persist($notafiscal);

            // Busca Contas a Pagar
            /** @var Notafiscal $notafiscal */
            $contaspagar = $this->titulosAPagarFactory->buscaContaspagar(
                $nrorg, $notafiscal->getCdfilial(), $notafiscal->getNrlanctonf()
            );

            // Percorre Contas a Pagar
            /** @var Contapagar $contapagar */
            foreach($contaspagar as $contapagar) {
                
                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $contapagar->getCdempresa(), $contapagar->getCdfimovflpag(),
                    $contapagar->getCdclassfina(), $contapagar->getDtmovfluxo(), $contapagar->getVratuapag() * -1, $contapagar->getIdmovfluxo(), 'S'
                );
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();

                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $contapagar->getCdempresa(), $contapagar->getCdfimovflpag(),
                    $cdclassfina, $contapagar->getDtmovfluxo(), $vrnotafisc, $idmovfluxo, 'S'
                );
                $this->entityManager->persist($movfluxo);
                
                // Atualiza Conta Pagar
                /** @var Contapagar $contapagar */
                $contapagar = $this->titulosAPagarFactory->atualizaContaPagar(
                    $contapagar, $cdtpctpagar, $contapagar->getDtatuavenpag(), $cdclassfina, $contapagar->getDtmovfluxo(), $idmovfluxo, $vrnotafisc, $vrnotafisc, $dscontapag
                );
                $this->entityManager->persist($contapagar);
                $this->entityManager->flush();

                /** Boletpagar $boletpagar **/
                // Busca Título a Pagar
                $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar(
                    $contapagar->getNrorg(), $contapagar->getCdfilibpag(), $contapagar->getNrlanbolpag()
                );

                // Atualiza Valor Título a Pagar
                $boletpagar = $this->titulosAPagarFactory->atualizaValorBoletPagar(
                    $boletpagar, $vrnotafisc, $vrnotafisc
                );

                $this->entityManager->persist($boletpagar);

                /** @var Ratctpagar $ratctpagar */
                $ratsctpagar = $this->titulosAPagarFactory->buscaRatCtPagar(
                    $nrorg, $notafiscal->getNrlanctonf(), $notafiscal->getCdfilial(), $contapagar->getNrseqvenpag(),
                    $contapagar->getNrparcelapag(), $notafiscal->getCdfilirece()
                );

                // Verifica Existencia de Rateio(s)
                if($ratsctpagar)
                    //Percorre Rateio(s)
                    /** @var Ratctpagar $ratctpagar */
                    foreach($ratsctpagar as $ratctpagar)
                        // Remove Rateio
                        $this->entityManager->remove($ratctpagar);
                $this->entityManager->flush();
                
                // Verifica Status - Nota Fiscal
                if($idstatus != 'V') {

                    // Gera Rateio Conta Pagar
                    /** @var Ratctpagar $ratctpagar */
                    $ratctpagar = $this->geraRateioPag(
                        $nrorg, $notafiscal->getCdempresa(), $notafiscal->getCdfilial(), $notafiscal->getCdfilirece(), $notafiscal->getNrlanctonf(),
                        $contapagar->getNrseqvenpag(), $contapagar->getNrparcelapag(), $cdtpctpagar,
                        $contapagar->getVrorigpag()
                    );

                    // Verifica Existência de Novos Rateio(s)
                    if($ratctpagar)
                        //Percorre Novos Rateio(s)
                        foreach($ratctpagar as $rateio)
                            $this->entityManager->persist($rateio);

                    // Gera Contabilização Conta Pagar
                    $this->contabilizacaoService->contabilizaPagamento(
                        $cdfilial, $notafiscal->getCdempresa(), $notafiscal->getCdfilirece(), $contapagar->getNrparcelapag(), $contapagar->getNrseqvenpag(),
                        $contapagar->getIdstatpag(), $cdtpctpagar
                    );
                }else{
                    // Busca Lançamento Contábil
                    /** @var Lanccont $lanccont */
                    $lancconts = $this->titulosAPagarFactory->buscaLanccont(
                        $nrorg, $notafiscal->getCdempresa(), $notafiscal->getNrlanccont()
                    );
                    // Verifica Existência de Lançamento Contábil
                    if($lancconts)
                        foreach($lancconts as $lanccont)
                            // Remove Lançamento Contabil
                            $this->entityManager->remove($lanccont);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdforneced
     * @param $cdempresa
     * @param $vrbolpag
     * @param $vrfinalbpag
     * @param $dtvenbolpag
     * @return Boletpagar|null
     * @throws \Exception
     */
    public function geraBoletoPagar(
        $nrorg, $cdfilial, $nrlanbolpag, $cdforneced, $cdempresa, $vrbolpag, $vrfinalbpag, $dtvenbolpag, $idhabibpag, $cdfililanc, $cdfilirece
    ){
        try {
            // Calcula Multa / Mora Boleto
            $multamora = $this->calcMultaMoraBolPag($nrorg, $cdempresa, $vrbolpag);
            
            // Busca Tipo de Baixa Preferencial
            $tipobaixapag = $this->buscaTipoBaixaPagPrefBolPag($cdforneced, $cdfilirece, $cdempresa);
            
            $cdtpbaixapag = $tipobaixapag['CDTPBAIXAPAG'];
            $idbaixautilp = $tipobaixapag['IDBAIXAUTILP'];
            $idbaixautilpPermitidos = array('D', 'C', 'T', 'S');
            
            if(in_array($idbaixautilp, $idbaixautilpPermitidos)){
                // Busca Banco Preferencial
                $bancopref = $this->buscaBancoPrefBolPag($nrorg, $cdforneced);
            }else{
                $bancopref = null;
            }

            // Operador de Lançamento
            $cdoperinclusao = $this->environment->getCdOperador();
            
            // Gera Boleto Pagar
            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->titulosAPagarFactory->insereBoletoPagar(
                $nrorg, $cdfilial, $nrlanbolpag, $cdforneced, $cdempresa, $vrbolpag, $vrfinalbpag, $dtvenbolpag,
                $multamora['VRPEMULBPAG'], $multamora['VRMULTABPAG'] , $multamora['VRPEMORBPAG'], $multamora['VRMORDIABPAG'],
                $bancopref['CDBANCFORN'], $bancopref['CDAGENFORN'], $bancopref['CDCCORFORN'], $idhabibpag, $cdoperinclusao, $cdfililanc, $cdtpbaixapag
            );

            return $boletpagar;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Calcula Multa|Mora Boleto Pagar
     * @param $nrorg
     * @param $cdempresa
     * @param $vrbolpag
     * @return array|null
     * @throws \Exception
     */
    public function calcMultaMoraBolPag($nrorg, $cdempresa, $vrbolpag){
        try{

            // Busca Empresa
            $empresa     = $this->titulosAPagarFactory->buscaEmpresa($nrorg, $cdempresa);
            $vrpemulempr = $empresa->getVrpemulempr();
            $vrpemorempr = $empresa->getVrpemorempr();

            $vrmultabpag  = $vrbolpag * ($vrpemulempr / 100);
            $vrmordiabpag = $vrbolpag * ($vrpemorempr / 100);
            $vrpemorbpag  = $vrpemorempr;
            $vrpemulbpag  = $vrpemulempr;

            return $multamora = [
                'VRMULTABPAG'  => $vrmultabpag,
                'VRMORDIABPAG' => $vrmordiabpag,
                'VRPEMORBPAG'  => $vrpemorbpag,
                'VRPEMULBPAG'  => $vrpemulbpag,
            ];
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Busca Banco Preferencial Boleto Pagar
     * @param $nrorg
     * @param $cdforneced
     * @return array|null
     * @throws \Exception
     */
    public function buscaTipoBaixaPagPrefBolPag($cdforneced, $cdfilirece, $cdempresa){
        try{
            
            // Busca Tipo de Baixa a Pagar Preferencial do Boleto a Pagar
            $tipobaixapag = $this->titulosAPagarFactory->buscaTipoBaixaPagPrefBolPag($cdforneced, $cdfilirece, $cdempresa);
            
            return $tipobaixapag;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Busca Banco Preferencial Boleto Pagar
     * @param $nrorg
     * @param $cdforneced
     * @return array|null
     * @throws \Exception
     */
    public function buscaBancoPrefBolPag($nrorg, $cdforneced){
        try{

            // Busca Conta Corrente Fornecedor
            /** @var Ccorforn $ccorforn */
            $ccorforn = $this->titulosAPagarFactory->buscaCcorforn($nrorg, $cdforneced, 'S');

            // Verifica Existencia Banco Preferencial
            if($ccorforn)
                $bancopref = [
                    'CDBANCFORN' => $ccorforn->getCdbancforn(),
                    'CDAGENFORN' => $ccorforn->getCdagenforn(),
                    'CDCCORFORN' => $ccorforn->getCdccorforn()
                ];
            else $bancopref = null;

            return $bancopref;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Gera Rateio
     * @param $nrorg
     * @param $cdfilial
     * @param $cdfilirece
     * @param $nrlanctonf
     * @param $nrseqvenpag
     * @param $nrparcelapag
     * @param $cdtpctpagar
     * @param $vrorigpag
     * @return Ratctpagar|null
     * @throws \Exception
     */
    public function geraRateioPag(
        $nrorg, $cdempresa, $cdfilial, $cdfilirece, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdtpctpagar, $vrorigpag
    ){
        try{

            // Rateio Conta Pagar
            $ratctpagar = $this->titulosAPagarFactory->buscaRatCtPagar(
                $nrorg, $nrlanctonf, $cdfilial, $nrseqvenpag, $nrparcelapag, $cdfilirece
            );
            
            
            // Verifica Existência Rateio Conta Pagar
            if(!$ratctpagar){

                // Rateios Padrão Conta Pagar
                $rattpctr = $this->titulosAPagarFactory->buscaRatTpCtP($nrorg, $cdtpctpagar);
                
                // Verifica Não Existência Rateio Padrão Conta Pagar
                if(!$rattpctr){
                    
                    // Centro de Custo Filial
                    // Linha comentada, pois não estava fazendo a verificação do centro de custo corretamente.
                   // $centcustfil = $this->titulosAPagarFactory->buscaCentCustFil($nrorg, $cdfilirece);
                    
                    // Adicionado busca igual a rotina do contas a receber, pois, quando a consulta retornar null não se gera rateio.
                    // A busca retorna tanto o centro de custo por empresa, quanto por filial.
                    // Centro de Custo Filial
                     $centcustfil = $this->titulosAPagarFactory->buscaCentCustFili($cdfilirece, $cdempresa);
                    
                    // Verifica Existência Centro de Custo Filial
                    if($centcustfil){
                        if ($centcustfil[0]['CDCENTCUST']){
                            // Insere Rateio Tipo Conta Pagar[Temporário]
                            $rattpctr[0] = $this->titulosAPagarFactory->insereRatTpCtP(
                                $nrorg, $cdfilial, $cdfilirece, $centcustfil[0]['CDCENTCUST'], 100
                            );
                        }
                    }
                }

                // Verifica Existência Rateio Padrão Conta Pagar
                if($rattpctr){
                    // Variáveis de Inicialização
                    $vrtotratcnt  = 0;
                    $vrratccctpag = 0;
                    $chave        = 0;
                    
                    // Percorre Rateios Padrões Conta Pagar
                    /** @var Rattpctp $rateio */
                    foreach($rattpctr as $chave => $rateio){

                        // Verifica Filial de Rateio Conta Pagar
                        $cdfiratctpag = $rateio->getCdfilial()?$rateio->getCdfirattpctp():$cdfilirece;
                        
                        // Calcula Valor Rateio Conta Pagar
                        $vrratccctpag = round((round($vrorigpag, 2) * round($rateio->getVrperattpctp(), 2))/100, 2);

                        // Insere Rateio Conta Pagar
                        $ratctpagar[$chave] = $this->titulosAPagarFactory->insereRatCtPagar(
                            $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $rateio->getCdcentcust(), $cdfiratctpag, $vrratccctpag
                        );
                        
                        // Soma Valor Total Rateio Conta Pagar
                        $vrtotratcnt = round($vrtotratcnt, 2) + $vrratccctpag;
                    }

                    // Verifica Existência de Diferença do Valor Conta/Rateio
                    if (round($vrtotratcnt, 2) <> round($vrorigpag, 2)){

                        // Calcula Valor Ajustado Última Conta Pagar
                        $vrratccctpag = round($vrratccctpag, 2) + round($vrorigpag, 2) - round($vrtotratcnt, 2);

                        // Atualiza Último Rateio Conta Pagar
                        $ratctpagar = $this->titulosAPagarFactory->atualizaRatCtPagar(
                            $ratctpagar[$chave], $vrratccctpag
                        );
                    }
                }
            } //else {

            //     // Variáveis de Inicialização
            //     $vrtotratcnt  = 0;
            //     $vrratccctpag = 0;
            //     $chave        = 0;

            //     // Percorre Rateios Conta Pagar
            //     /** @var Rattpctp $rateio */
            //     foreach($ratctpagar as $chave => $rateio){

            //         // Calcula Valor Rateio Conta Pagar
            //         $vrratccctpag = round((round($vrorigpag, 2) * round($rateio->getVrperattpctp(), 2))/100, 2);

            //         // Atualiza Rateio Conta Pagar
            //         $ratctpagar[$chave] = $this->titulosAPagarFactory->atualizaRatCtPagar(
            //             $rateio, $vrratccctpag
            //         );

            //         // Soma Valor Total Rateio Conta Pagar
            //         $vrtotratcnt = round($vrtotratcnt, 2) + $vrratccctpag;
            //     }

            //     // Verifica Existência de Diferença do Valor Conta/Rateio
            //     if (round($vrtotratcnt, 2) <> round($vrorigpag, 2)){

            //         // Calcula Valor Ajustado Última Conta Pagar
            //         $vrratccctpag = round($vrratccctpag, 2) + round($vrorigpag, 2) - round($vrtotratcnt, 2);

            //         // Atualiza Último Rateio Conta Pagar
            //         $ratctpagar = $this->titulosAPagarFactory->atualizaRatCtPagar(
            //             $ratctpagar[$chave], $vrratccctpag
            //         );
            //     }
            // }

            return $ratctpagar;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Edita Titulo A Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @param $cdreducont
     * @param $cdreducont
     * @param $cdtpbaixapag
     * @param $cdbancbolp
     * @param $cdagenbolp
     * @param $cdccorbolp
     * @param $vrpemulbpag
     * @param $vrpemorbpag
     * @param $vrmultabpag
     * @param $vrmordiabpag
     * @param $changedsbol
     * @param $chavepix
     * @param $codeqrpix 
     * @throws \Exception
     */
    public function editaTituloAPagar(
        $nrorg, $cdfilial, $nrlanbolpag, $cdreducont, $cdreducont1, $cdtpbaixapag, $cdbancbolp,
        $cdagenbolp, $cdccorbolp, $vrpemulbpag, $vrpemorbpag, $vrmultabpag, $vrmordiabpag,
        $dslinhbarpag, $cdbarrapag, $dsboletpag, $changedsbol, $chavepix, $codeqrpix, $idchavepix, $dschavepix
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar($nrorg, $cdfilial, $nrlanbolpag);

            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->titulosAPagarFactory->atualizaBoletpagar(
                $boletpagar, $cdreducont, $cdtpbaixapag, $cdbancbolp, $cdagenbolp, $cdccorbolp,
                $vrpemulbpag, $vrpemorbpag, $vrmultabpag, $vrmordiabpag, $dslinhbarpag, $cdbarrapag,
                $chavepix, $codeqrpix, $idchavepix, $dschavepix
            );

            $this->entityManager->persist($boletpagar);

            // Verifica Alteração de Descrição
            if($changedsbol){
                /** @var Contapagar $contapagar **/
                $contaspagar = $this->titulosAPagarFactory->buscaContaspagarAssoc(
                    $boletpagar->getNrorg(), $boletpagar->getCdfilial(), $boletpagar->getNrlanbolpag()
                );

                /** @var ContaPagar $contaspagar */
                foreach ($contaspagar as $contapagar) {
                    $contapagar->setDscontapag($dsboletpag);
                    $this->entityManager->persist($contapagar);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Deleta Vencimento Títulos a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @throws \Exception
     */
    public function deletaVencTituloPagar($nrorg, $cdfilial, $nrlanbolpag, $response)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Conta a Pagar
            /** @var Contapagar $contapagar */
            $contapagar = $this->titulosAPagarFactory->buscaContapagar(
                $nrorg, $cdfilial, $nrlanbolpag
            );

            // Validação Nota Fiscal
            $this->validaNotaSistema($nrorg, $contapagar->getCdfilial(), $contapagar->getNrlanctonf());

            // Validação Conta a Pagar
            $this->validaStatusConta($contapagar);

            // Validação Ajuste Conta Pagar
            $this->validaAjuste($nrorg, $contapagar->getCdfilial(), $contapagar->getNrlanctonf(), $contapagar->getNrseqvenpag(), $contapagar->getNrparcelapag());

            // Validação Ajuste Boleto Pagar
            $this->verificaAjusteBoleto($nrorg, $contapagar->getCdfilial(), $nrlanbolpag);

            // Validação Boleto Pagar CNAB
            $this->validaBoletoCnab($nrorg, $cdfilial, $nrlanbolpag);

            // Busca Contas a Pagar
            /** @var Contapagar $contapagar */
            $contaspagar = $this->titulosAPagarFactory->buscaContaspagar(
                $nrorg, $contapagar->getCdfilial(), $contapagar->getNrlanctonf()
            );

            // Verifica Existência de outro(s) Vencimento(s)
            if (count($contaspagar) > 1) {

                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal(
                    $contapagar->getNrorg(), $contapagar->getCdfilial(), $contapagar->getNrlanctonf()
                );

                // Atualiza Valor Nota Fiscal/Produto
                $notafiscal->setVrnotafisc($notafiscal->getVrnotafisc() - $contapagar->getVratuapag());
                $notafiscal->setVrprodutos($notafiscal->getVrprodutos() - $contapagar->getVratuapag());

                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $notafiscal->getCdempresa(), $notafiscal->getCdfilirece(), $contapagar->getCdclassfina(), $contapagar->getDtatuavenpag(), $contapagar->getVratuapag() * -1, $contapagar->getIdmovfluxo(), 'S'
                );

                $this->entityManager->persist($notafiscal);
                $this->entityManager->flush();

                // Busca Rateio(s) Conta a Pagar
                /** @var Ratctpagar $ratctpagar*/
                $ratctpagar = $this->titulosAPagarFactory->buscaRatCtPagar(
                    $contapagar->getNrorg(), $contapagar->getNrlanctonf(), $contapagar->getCdfilial(),
                    $contapagar->getNrseqvenpag(), $contapagar->getNrparcelapag(), $notafiscal->getCdfilirece()
                );

                // Deleta Rateio(s) Conta Pagar
                if ($ratctpagar)
                    foreach($ratctpagar as $rateio)
                        $this->entityManager->remove($rateio);

                //Busca Ajuste(s) Boleto Pagar
                /** @var Ajbolpag $ajbolpag */
                $ajbolpag = $this->titulosAPagarFactory->buscaAjbolpags(
                    $contapagar->getNrorg(), $contapagar->getCdfilibpag(), $contapagar->getNrlanbolpag()
                );

                // Deleta Ajuste(s) Boleto Pagar
                if ($ajbolpag)
                    foreach($ajbolpag as $ajuste)
                        $this->entityManager->remove($ajuste);

                // Busca Boleto Pagar
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->titulosAPagarFactory->buscaBoletpagar(
                    $contapagar->getNrorg(), $contapagar->getCdfilibpag(), $contapagar->getNrlanbolpag()
                );

                $this->entityManager->persist($movfluxo);

                // Deleta Conta Pagar
                $this->entityManager->remove($contapagar);

                // Deleta Boleto Pagar
                $this->entityManager->remove($boletpagar);

            }else throw new \Exception('Operação bloqueada. O título possui apenas um vencimento.');

            $this->entityManager->flush();

            /** @var Contapagar $conta */
            foreach ($contaspagar as $conta) {

                // Gera Contabilização Conta Pagar
                $this->contabilizacaoService->contabilizaPagamento(
                    $conta->getCdfilial(), $conta->getCdempresa(), $conta->getNrlanctonf(), $conta->getNrparcelapag(), $conta->getNrseqvenpag(), $conta->getIdstatpag(), $conta->getCdtpctpagar()
                );

            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            $this->contabilizacaoService->saveContabilizacaoData($response);
            throw $e;
        }
    }

    /**
     * Deleta Documento Títulos a Pag
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @throws \Exception
     */
    public function deletaDocTituloPagar($nrorg, $cdfilial, $nrlanbolpag)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Busca Conta a Pagar
            /** @var Contapagar $contaPagar */
            $contaPagar = $this->titulosAPagarFactory->buscaContapagar(
                $nrorg, $cdfilial, $nrlanbolpag
            );
            
            // Validação Nota Fiscal Sistema PAG
            $this->validaNotaSistema($contaPagar->getNrorg(), $contaPagar->getCdfilial(), $contaPagar->getNrlanctonf());

            // Busca Contas a Pagar
            /** @var Contapagar $contasPagar */
            $contasPagar = $this->titulosAPagarFactory->buscaContaspagar(
                $contaPagar->getNrorg(), $contaPagar->getCdfilial(), $contaPagar->getNrlanctonf()
            );
            
            /** @var ContaPagar $conta */
            foreach ($contasPagar as $conta) {

                // Validação Conta a Pagar
                $this->validaStatusConta($conta);

                // Validação Ajuste Conta Pagar
                $this->validaAjuste($conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(),  $conta->getNrseqvenpag(), $conta->getNrparcelapag());

                // Validação Ajuste Título Pagar
                $this->verificaAjusteBoleto($conta->getNrorg(), $conta->getCdfilibpag(), $conta->getNrlanbolpag());

                // Validação Título Pagar CNAB
                $this->validaBoletoCnab($conta->getNrorg(), $conta->getCdfilibpag(), $conta->getNrlanbolpag());

                // Exclui Rateio Conta a Pagar
                /** @var Ratctpagar $ratctpagar*/
                $ratctpagar = $this->titulosAPagarFactory->buscaRatCtPagar(
                    $conta->getNrorg(), $conta->getNrlanctonf(), $conta->getCdfilial(),
                    $conta->getNrseqvenpag(), $conta->getNrparcelapag(), $conta->getCdfilibpag()
                );
                if ($ratctpagar)
                    foreach($ratctpagar as $ratctpag)
                        $this->entityManager->remove($ratctpag);

                // Busca Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
                );

                // Atualiza Fluxo de Caixa
                /** @var Movfluxo $movfluxo */
                $movfluxo = $this->fluxoDeCaixaService->atualizaFluxoCaixa(
                    $nrorg, $notafiscal->getCdempresa(), $notafiscal->getCdfilirece(), $conta->getCdclassfina(), $conta->getDtatuavenpag(), $conta->getVratuapag() * -1, $conta->getIdmovfluxo(), 'S'
                );
                $this->entityManager->persist($movfluxo);
                $this->entityManager->flush();

                // Exclui Conta a Pagar
                /** @var ContaPagar $contaPagar */
                $contaPagar = $this->titulosAPagarFactory->buscaContaPagarParcela(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf(), $conta->getNrparcelapag(), $conta->getNrseqvenpag()
                );
                $this->entityManager->remove($contaPagar);
                $this->entityManager->flush();

                // Exclui Título a Pagar
                /** @var BoletPagar $boletPagar */
                $boletPagar = $this->titulosAPagarFactory->buscaBoletPagar(
                    $conta->getNrorg(), $conta->getCdfilibpag(), $conta->getNrlanbolpag()
                );
                $this->entityManager->remove($boletPagar);


                // Exclui Contabilização Nota Fiscal
                /** @var Lanccont $lanccont */
                $lanccont = $this->titulosAPagarFactory->buscaLanccont(
                    $conta->getNrorg(), $notafiscal->getCdempresa(), $notafiscal->getNrlanctbdoc()
                );
                if ($lanccont)
                    foreach ($lanccont as $lancctb)
                        $this->entityManager->remove($lancctb);
            }
            
            /** @var ContaPagar $conta */
            foreach ($contasPagar as $conta) {
                // Exclui Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal(
                    $conta->getNrorg(), $conta->getCdfilial(), $conta->getNrlanctonf()
                );
                if($notafiscal)
                    $this->entityManager->remove($notafiscal);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Valida Nota Fiscal Sistema
     * @param $cdfilial
     * @param $nrlanctonf
     * @throws \Exception
     */
    public function validaNotaSistema($nrorg, $cdfilial, $nrlanctonf)
    {
        try {
            /** @var Notafiscal $notafiscal */
            $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal(
                $nrorg, $cdfilial, $nrlanctonf
            );
            if ($notafiscal->getIdsistema() != 'PAG')
                throw new \Exception('Operação bloqueada. Apenas títulos lançados no módulo financeiro podem ser excluídos.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Ajuste Conta a Pagar
     * @param $cdfilial
     * @param $nrlanctonf
     * @throws \Exception
     */
    public function validaAjuste($nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag)
    {
        try {
            /** @var Ajctpagar $ajctpagar */
            $ajctpagar = $this->titulosAPagarFactory->buscaAjctpagar(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag
            );
            if ($ajctpagar)
                throw new \Exception('Operação bloqueada. Título possui Ajuste(s) na Conta.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica Ajuste Boleto Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @throws \Exception
     */
    public function verificaAjusteBoleto($nrorg, $cdfilial, $nrlanbolpag)
    {
        try {
            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->titulosAPagarFactory->buscaAjbolpag(
                $nrorg, $cdfilial, $nrlanbolpag
            );
            if ($ajbolpag)
                throw new \Exception('Operação bloqueada. Título possui Ajuste(s).');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Boleto Pagar CNAB
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @throws \Exception
     */
    public function validaBoletoCnab($nrorg, $cdfilial, $nrlanbolpag)
    {
        try {
            /** @var Boletpagar $boletpagar */
            $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar(
                $nrorg, $cdfilial, $nrlanbolpag
            );
            if ($boletpagar->getNrcnabpagrem())
                throw new \Exception('Operação bloqueada. Título bloqueado via CNAB.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Status Conta Pagar
     * @param Contapagar $contapagar
     * @throws \Exception
     */
    public static function validaStatusConta(Contapagar $contapagar)
    {
        try {
            if ($contapagar->getIdstatpag() == 'B')
                throw new \Exception('Operação bloqueada. Título baixado não pode ser excluído.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Prorroga Título a Pagar
     * @param $nrorg
     * @param $dtvenbolpag
     * @param $cdmotipror
     * @param $boletos
     * @throws \Exception
     */
    public function prorrogaTituloAPagar($nrorg, $dtvenbolpag, $cdmotipror, $boletos, $idtppror, $nrdiasvenc)
    {
        try {
            $this->prorrogacaoTituloService->prorrogaTituloAPagar($nrorg, $dtvenbolpag, $cdmotipror, $boletos, $idtppror, $nrdiasvenc);
        }catch(\Exception $e){
            throw $e;
        }
    }


    /**
     * Remessa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $remessatitulo
     * @param $boletosmarcados
     * @return null|string
     * @throws \Exception
     */
    public function remessaTituloPagar($nrorg, $cdfilial, $remessatitulo, $boletosmarcados)
    {
        try{
            return $this->remessaTituloService->geraRemessaPagar(
                $nrorg, $cdfilial, $remessatitulo, $boletosmarcados
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Exclui Título a Pagar da Remessa
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @throws \Exception
     */
    public function excluiTituloPagRemessa($nrorg, $cdfilial, $cdoperador, $boletos)
    {
        try{
            $this->remessaTituloService->excluiTituloPagRemessa($nrorg, $cdfilial, $cdoperador, $boletos);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Exclui Remessa Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $cdoperador
     * @param $boletos
     * @return null
     * @throws \Exception
     */
    public function excluiRemessaPag($nrorg, $cdfilial, $cdoperador, $boletos)
    {
        try{
            $this->remessaTituloService->excluiRemessaPag($nrorg, $cdfilial, $cdoperador, $boletos);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Baixa Titulo Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @param $response
     * @return null
     * @throws \Exception
     */
    public function baixaTituloPagar($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $ajustesboletos, $contabiliza, $response, $boletostotais = [])
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if(count($boletostotais) > 1){
                $boletosparcelados = [];
                // Agrupa Baixa Parcial
                foreach($boletostotais as $boletomarcados) {
                    if($boletomarcados['NRORIGBOLPAG'] && isset($boletomarcados['PARCELADO'])){
                        if (!isset($boletosparcelados[$boletomarcados['NRORIGBOLPAG']])) {
                            $boletosparcelados[$boletomarcados['NRORIGBOLPAG']] = [$boletomarcados];
                        } else {
                            array_push($boletosparcelados[$boletomarcados['NRORIGBOLPAG']], $boletomarcados);
                        }
                    }
                }
                
                
                
                // Parcela Boleto da Baixa Parcial
                foreach($boletosparcelados as $boletoparcelado){
                    if(count($boletoparcelado) > 1){
                        $nrparcbolpag_aux = [];
                        
                        // Armazena as parcelas que ja estao na baixa
                        foreach($boletoparcelado as $boleto){
                            array_push($nrparcbolpag_aux, $boleto['NRPARCBOLPAGVISU']);
                        }
                        
                        // Monta Array Boletos Originais
                        $boletosOriginais = $this->titulosAPagarFactory->getBoletosOriginais($boletoparcelado[0]['NRORG'], $boletoparcelado[0]['CDFILIAL'], $boletoparcelado[0]['NRORIGBOLPAG']);
                        // Adiciona as parcelas que ainda não estao na baixa para o parcelamento
                        foreach($boletosOriginais as $boletoOriginal){
                            if(!in_array($boletoOriginal->getNrparcbolpagvisu(), $nrparcbolpag_aux)){
                                $aux =    array(
                                    'NRORG' => (string)$boletoOriginal->getNrorg(), 
                                    'CDFILIAL' => $boletoOriginal->getCdfilial(),
                                    'NRLANBOLPAG' => $boletoOriginal->getNrlanbolpag(),
                                    'NRORIGBOLPAG' => $boletoOriginal->getNrorigbolpag(), 
                                    'NRPARCBOLPAG' => $boletoOriginal->getNrparcbolpag(), 
                                    'NRPARCBOLPAGVISU' => $boletoOriginal->getNrparcbolpagvisu(), 
                                    'DTVENBOLPAG' => DateUtil::truncateString($boletoOriginal->getDtvenbolpag()), 
                                    'VRFINALBPAG' => $boletoOriginal->getVrfinalbpag(), 
                                    'CDMOTIPARC' => $boletoOriginal->getCdmotiparc() 
                                );
                                array_push($boletoparcelado, $aux);
                            }
                        }
                        
                        // Ordena Parcelas por NRPARCBOLPAGVISU
                        uasort($boletoparcelado, function ($a, $b) {
                            return $a['NRPARCBOLPAGVISU'] > $b['NRPARCBOLPAGVISU'];
                        });
                        
                        $boletoparcelado_final = [];
                        
                        foreach($boletoparcelado as $boleto){
                            array_push($boletoparcelado_final, $boleto);
                        }
                        
                        $this->parcelamentoTituloService->geraParcTitulo($boletoparcelado_final[0]['NRORG'], $boletoparcelado_final[0]['CDFILIAL'], $boletoparcelado_final[0]['NRORIGBOLPAG'], $boletoparcelado_final);
                    }
                }
            }
            
            // Busca Numero de Lançamento do Boleto Parcelado
            foreach($boletosmarcados as $key => $boletomarcados){
                
                if(!$boletomarcados['NRLANBOLPAG']){
                    $boletpagar = $this->titulosAPagarFactory->buscaBoletoParcelado($boletomarcados['NRORG'], $boletomarcados['CDFILIAL'], $boletomarcados['NRORIGBOLPAG'], $boletomarcados['NRPARCBOLPAG']);
                    
                    $boletosmarcados[$key]['NRLANBOLPAG'] = $boletpagar->getNrlanbolpag();
                }
            }
            
            // Gera Ajuste Boleto
            if($ajustesboletos)
                 $this->geraAjusteTitulo($ajustesboletos);
            
            // Baixa Boletos
            if($baixaboleto['IDUNICABAIXA'] == 'N'){
                foreach ($boletosmarcados as $boleto){
                    $baixaboleto['CDTPBAIXAPAG'] = $boleto['CDTPBAIXAPAG'];
                    $this->baixaTituloService->baixaTituloPagar($nrorg, $cdfilial, $baixaboleto, [$boleto], $contabiliza, $response);
                }
            }else{
                $boletosPorTipoDeBaixa = [];
                
                // Agrupa Tipos de Baixa
                foreach($boletosmarcados as $boletomarcados) {
                    if($boletomarcados['CDTPBAIXAPAG']){
                        if (!isset($boletosPorTipoDeBaixa[$boletomarcados['CDTPBAIXAPAG']])) {
                            $boletosPorTipoDeBaixa[$boletomarcados['CDTPBAIXAPAG']] = [$boletomarcados];
                        } else {
                            array_push($boletosPorTipoDeBaixa[$boletomarcados['CDTPBAIXAPAG']], $boletomarcados);
                        }
                    }
                }
                
                // Baixa por Grupo de Tipo de Baixa
                foreach($boletosPorTipoDeBaixa as $key => $boletoPorTipoDeBaixa){
                    $baixaboleto['CDTPBAIXAPAG'] = $key;
                    $this->baixaTituloService->baixaTituloPagar($nrorg, $cdfilial, $baixaboleto, $boletoPorTipoDeBaixa, $contabiliza , $response);
                }
            }
            
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Gera Ajuste Título
     * @param $ajustesboletos
     * @throws \Exception
     */
    public function geraAjusteTitulo($ajustestitulo){
        try {

            // Exclui Ajustes Título
            foreach($ajustestitulo as $ajustetitulo){
                if($ajustetitulo['IDOPERACAO'] == 'E'){
                    $ajbolpag = $this->titulosAPagarFactory->buscaAjusteTitulo(
                        $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLPAG'],
                        $ajustetitulo['CDAJUCTPAG']
                    );
                    
                    if($ajbolpag)
                        $this->ajusteTituloService->deletaAjusteBoleto(
                            $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLPAG'],
                            $ajustetitulo['CDAJUCTPAG'], $ajustetitulo['IDAJUCTPAG'], $ajustetitulo['VRAJBOLPAG']
                        );
                }
            }

            // Inclui Ajustes Título
            foreach($ajustestitulo as $ajustetitulo){
                if($ajustetitulo['IDOPERACAO'] == 'I'){
                                    
                    $ajbolpag = $this->titulosAPagarFactory->buscaAjusteTitulo(
                        $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLPAG'],
                        $ajustetitulo['CDAJUCTPAG']
                    );
                    
                    if(!$ajbolpag)
                        $this->ajusteTituloService->salvaAjusteBoleto(
                            $ajustetitulo['NRORG'], $ajustetitulo['CDFILIAL'], $ajustetitulo['NRLANBOLPAG'],
                            $ajustetitulo['IDAJUCTPAG'], $ajustetitulo['CDAJUCTPAG'], $ajustetitulo['VRAJBOLPAG'],
                            $ajustetitulo['IDORIGAJBPAG']
                        );
                }
            }
            
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Exclui Baixa Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @return null
     * @throws \Exception
     */
    public function excluiBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $cheque)
    {
        try{
            $this->baixaTituloService->excluiBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $cheque);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     *  Exclui Titulo Baixa Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $boletosmarcados
     * @param $response
     * @return null
     * @throws \Exception
     */
    public function excluiTituloBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $response)
    {
        try{
            $this->baixaTituloService->excluiTituloBaixaPagar($nrorg, $cdfilial, $boletosmarcados, $response);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @param $cheque
     * @param $response
     * @throws \Exception
     */
    public function transformaBaixaPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $cheque, $response)
    {
        try{
            $this->baixaTituloService->transformaBaixaPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $cheque, $response);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $nrorg
     * @param $cdfilial
     * @param $baixaboleto
     * @param $boletosmarcados
     * @param $response
     * @throws \Exception
     */
    public function transformaTituloPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response)
    {
        try{
            $this->baixaTituloService->transformaTituloPlanejado($nrorg, $cdfilial, $baixaboleto, $boletosmarcados, $response);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Associação Titulo
     * @param $boletos
     * @throws \Exception
     */
    public function validaAssociacaoTitulo($boletos)
    {
        try{
            $this->associacaoTituloService->validaAssociacaoTitulo($boletos);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida Data de Vencimento
     * @param $dtvenbolpag
     * @param $boletos
     * @throws \Exception
     */
    public function validaDataVencimento($dtvenbolpag, $boletos)
    {
        try{
            $datavencimento = $this->formataData($dtvenbolpag);
            foreach($boletos as $boleto){

                // Busca Conta a Pagar
                /** @var Contapagar $contapagar */
                $contapagar = $this->titulosAPagarFactory->buscaContapagar($boleto['NRORG'], $boleto['CDFILIAL'],$boleto['NRLANBOLPAG']);

                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal($contapagar->getNrorg(), $contapagar->getCdfilial(), $contapagar->getNrlanctonf());
                if ($datavencimento < $this->formataData($notafiscal->getDtemissao()))
                    throw new \Exception('Data de vencimento deve ser maior que a data de emissão da nota fiscal.');
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Confirma Associação Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $boletos
     * @param $dtvenbolpag
     * @param $cdreducont
     * @param $cdtpbaixapag
     * @throws \Exception
     */
    public function geraAssociacaoTitulo($nrorg, $cdfilial, $boletos, $dtvenbolpag, $cdreducont, $cdtpbaixapag)
    {
        try{
            $this->associacaoTituloService->geraAssociacaoTitulo($nrorg, $cdfilial, $boletos, $dtvenbolpag, $cdreducont, $cdtpbaixapag);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Cancela Associacao Título
     * @param $boletos
     * @throws \Exception
     */
    public function cancelaAssociacaoTitulo($boletos)
    {
        try{
            $this->associacaoTituloService->cancelaAssociacaoTitulo($boletos);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Gera Adiantamento Fornecedor
     * @param $tituloGrid
     * @param $adiantamentoTabbed
     * @param $adiantamentoGrid
     * @throws \Exception
     */
    public function geraAdiantamentoFornecedor($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid, $cdoperador){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Valída Ajuste Adiantamento Fornecedor
            self::validaAjusteAdiantDevolFornecedor($tituloGrid, $adiantamentoTabbed);

            // Insere Ajuste Adiantamento Fornecedor
            self::insereAjusteAdiantDevolFornecedor($tituloGrid, $adiantamentoTabbed);

            $this->entityManager->flush();

            // Insere Adiantamento Fornecedor
            self::insereAdiantamentoFornecedor($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid, $cdoperador);

            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Devolução Fornecedor
     * @param $tituloGrid
     * @param $devolucaoTabbed
     * @param $devolucaoGrid
     * @param $cdoperador
     * @throws \Exception
     */
    public function geraDevolucaoFornecedor($tituloGrid, $devolucaoTabbed, $devolucaoGrid, $cdoperador){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {

            // Valída Ajuste Adiantamento Fornecedor
            self::validaAjusteAdiantDevolFornecedor($tituloGrid, $devolucaoTabbed);

            // Insere Ajuste Adiantamento Fornecedor
            self::insereAjusteAdiantDevolFornecedor($tituloGrid, $devolucaoTabbed);

            $this->entityManager->flush();

            // Insere Devolução Fornecedor
            self::insereDevolucaoFornecedor($tituloGrid, $devolucaoTabbed, $devolucaoGrid, $cdoperador);

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Valída Adiantamento/Devolução Fornecedor
     * @param $tituloGrid
     * @param $adiantDevolTabbed
     * @throws \Exception
     */
    public function validaAjusteAdiantDevolFornecedor($tituloGrid, $adiantDevolTabbed){
        try {
            /** @var Ajbolpag $ajbolpag */
            $ajbolpag = $this->titulosAPagarFactory->buscaAjbolpagCod(
                $tituloGrid['NRORG'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLPAG'], $adiantDevolTabbed['CDAJUCTPAG']
            );

            if($ajbolpag)
                throw new \Exception('Operação bloqueada. Ajuste selecionado já cadastrado para o título, informe outro para realizar a operação.');

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Adiantamento/Devolução Fornecedor
     * @param $tituloGrid
     * @param $adiantDevolTabbed
     * @throws \Exception
     */
    public function insereAjusteAdiantDevolFornecedor($tituloGrid, $adiantDevolTabbed){
        try {
            // Insere Ajuste Ajuste Titulo
            $this->ajusteTituloService->insereAjusteAdiantDevolFornecedor(
                $tituloGrid['NRORG'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLPAG'], $adiantDevolTabbed['IDAJUCTPAG'],
                $adiantDevolTabbed['CDAJUCTPAG'], $adiantDevolTabbed['TOTALDESCONTO']
            );
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Adiantamento Fornecedor
     * @param $tituloGrid
     * @param $adiantamentoTabbed
     * @param $adiantamentoGrid
     * @throws \Exception
     */
    public function insereAdiantamentoFornecedor($tituloGrid, $adiantamentoTabbed, $adiantamentoGrid, $cdoperador){
        try {
            $operador = $this->environment->getCdOperador();
            
            // Percorre Adiantamento
            foreach($adiantamentoGrid as $adiantamento) {

                // Gera Novo Código - Adiantamento Pagar
                $nradiantapagar = $this->geraNovoCodigo($tituloGrid['NRORG'], 'ADIANTAPAGAR');

                /** @var Adiantapagar $adiantapagar */
                $adiantapagar = $this->titulosAPagarFactory->insereAdiantaPagar(
                    $nradiantapagar, $adiantamento['CDFILIAL'], $tituloGrid['CDFILIAL'], $adiantamento['NRLANCTONF'],
                    $tituloGrid['NRLANBOLPAG'], $adiantamento['NRSEQVENPAG'], $adiantamento['NRPARCELAPAG'],
                    floatval($adiantamento['VRDESCONTO']), $tituloGrid['NRORG'], $adiantamentoTabbed['CDAJUCTPAG'], DateUtil::getDataAtual(),
                    $cdoperador
                );

                $this->entityManager->persist($adiantapagar);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Insere Devolução Fornecedor
     * @param $tituloGrid
     * @param $devolucaoTabbed
     * @param $devolucaoGrid
     * @param $cdoperador
     * @throws \Exception
     */
    public function insereDevolucaoFornecedor($tituloGrid, $devolucaoTabbed, $devolucaoGrid, $cdoperador){
        try {
            // Percorre Devolução
            foreach($devolucaoGrid as $devolucao) {

                // Busca Nota Fiscal
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->buscaNotafiscal(
                    $tituloGrid['NRORG'],  $devolucao['FILIDEVO'], $devolucao['NRNFDEVO']
                );

                // Valor de Devolução
                $vrdevolnfutl = $notafiscal->getVrdevolnfutl()+floatval($devolucao['VRDESCONTO']);

                // Atualiza Nota Fiscal de Devolução
                /** @var Notafiscal $notafiscal */
                $notafiscal = $this->titulosAPagarFactory->atualizaNotaFiscalDev(
                    $notafiscal, $vrdevolnfutl
                );

                $this->entityManager->persist($notafiscal);

                // Gera Novo Código - Devolução Pagar
                $nrdevolucaopag =  $this->geraNovoCodigo($tituloGrid['NRORG'], 'DEVOLUCAOPAG');

                /** @var Devolucaopag $devolucaopag */
                $devolucaopag = $this->titulosAPagarFactory->insereDevolucaoPag(
                    $tituloGrid['NRORG'], $devolucao['FILIDEVO'], $tituloGrid['CDFILIAL'], $tituloGrid['NRLANBOLPAG'],
                    $nrdevolucaopag, DateUtil::getDataAtual(), $devolucao['NRNFDEVO'], $devolucaoTabbed['CDAJUCTPAG'],
                    floatval($devolucao['VRDESCONTO']), $cdoperador
                );

                $this->entityManager->persist($devolucaopag);
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Salva Conta Documento
     * @param $conta
     * @throws \Exception
     */
    public function salvaContaDocumento($conta){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $contapagar = $this->titulosAPagarFactory->buscaContaPagarParcela($conta['NRORG'], $conta['CDFILIAL'], $conta['NRLANCTONF'], $conta['NRPARCELAPAG'], $conta['NRSEQVENPAG']);

            if($contapagar) {
                $contapagar->setCdtpctpagar($conta['CDTPCTPAGAR']);
                $contapagar->setDscontapag($conta['DSCONTAPAG']);
                $contapagar->setCdclassfina($conta['CDCLASSFINA']);
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Gera Novo Código
     * @param $nrorg
     * @param $cdcontador
     * @return null|Novocodigo
     * @throws \Exception
     */
    public function geraNovoCodigo(
        $nrorg, $cdcontador
    ){
        try {
            /** @var Novocodigo $novocodigo */
            $novocodigo = $this->newCode->getCodeByTableName($cdcontador, $nrorg);
            return $novocodigo;
        } catch (\Exception $e) {
            throw $e;
            return null;
        }
    }

    /**
     * Finaliza Antecipação de Fornecedor
     * @param $antecipacao
     * @param $titulos
     * @throws \Exception
     */
    public function finalizaAntecipacaoFornecedor(
        $antecipacao, $titulos
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Gera Novo Código
            $nrlancantecipa = $this->geraNovoCodigo($antecipacao['NRORG'], 'BOLETPAGAR'.$antecipacao['CDFILIAL']);

            /** @var Antecipaforne $antecipaforne */
            $antecipaforne = $this->titulosAPagarFactory->insereAntecipaforne(
                $antecipacao['NRORG'], $antecipacao['CDFILIAL'], $nrlancantecipa, $antecipacao['CDFORNECED'],
                $antecipacao['CDFILIORIG'], $antecipacao['CDMOTIPROR'], $antecipacao['CDREDUCONTPAG'],
                $antecipacao['CDTPBAIXAPAG'], SELF::EM_ABERTO, $this->formataData($antecipacao['DTLANCANTECIPA']),
                $this->formataData($antecipacao['DTVENCANTECIPA']), $antecipacao['VRCONTAANTECIPA'], $antecipacao['VRLIQUANTECIPA'],
                $antecipacao['VRPEMULANTECIPA'], $antecipacao['VRPEMORAANTECIP'],
                $antecipacao['VRMULTAANTECIPA'], $antecipacao['VRMORAANTECIPA'], $antecipacao['DSLINHBARANTECI'],
                $antecipacao['CDBARRAANTECIPA'], $antecipacao['DSANTECIPAFORN'], $antecipacao['CDEMPRESA'],
                $this->formataData($antecipacao['DTEMISANTECIPA']), $antecipacao['CDBANCANTECIPA'], $antecipacao['CDAGENANTECIPA'],
                $antecipacao['CDCCORANTECIPA'], $antecipacao['CDEMPRESA']
            );
            $this->entityManager->persist($antecipaforne);

            foreach($titulos as $titulo){
                // Busca Título a Pagar
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar(
                    $titulo['NRORG'], $titulo['CDFILIAL'], $titulo['NRLANBOLPAG']
                );

                // Atualiza Título a Pagar Antecipado
                $boletpagar = $this->titulosAPagarFactory->atualizaBoletoAntecipado(
                    $boletpagar, $nrlancantecipa, $antecipacao['CDTPBAIXAPAG'], $antecipacao['CDREDUCONTPAG'],
                    $antecipacao['DSLINHBARANTECI'], $antecipacao['CDBARRAANTECIPA'], $antecipacao['CDBANCANTECIPA'],
                    $antecipacao['CDAGENANTECIPA'], $antecipacao['CDCCORANTECIPA'], $antecipacao['VRPEMULANTECIPA'],
                    $antecipacao['VRPEMORAANTECIP'], $antecipacao['VRMULTAANTECIPA'], $antecipacao['VRMORAANTECIPA'],
                    $antecipacao['CDFILIAL'], SELF::SIM, SELF::SIM
                );

                $this->entityManager->persist($boletpagar);

                /** @var Boletpagar $boletpagar */
                $this->prorrogacaoTituloService->geraProrrogacaoTitulo(
                    $titulo['NRORG'], $titulo['CDFILIAL'], $titulo['NRLANBOLPAG'],
                    $antecipacao['DTVENCANTECIPA'], $antecipacao['CDMOTIPROR'], $antecipacao['DTEMISANTECIPA']
                );
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    /**
     * Exclui Antecipação de Fornecedor
     * @param $antecipacao
     * @param $titulos
     * @throws \Exception
     */
    public function cancelaAntecipacaoFornecedor(
        $titulos
    ){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach($titulos as $titulo){
                /** @var Antecipaforne $antecipaforne */
                $antecipaforne = $this->titulosAPagarFactory->buscaAntecipaforne(
                    $titulo['NRORG'], $titulo['CDFILILANC'], $titulo['NRLANCANTECIPA']
                );

                if($antecipaforne){
                    // Busca Títulos Antecipados
                    /** @var Boletpagar $boletspagar */
                    $boletspagar = $this->titulosAPagarFactory->buscaBoletosAntecipados(
                        $antecipaforne->getNrorg(), $antecipaforne->getCdfilial(), $antecipaforne->getNrlancantecipa()
                    );

                    /** @var Boletpagar $boletpagar */
                    foreach($boletspagar as $boletpagar){
                        // Atualiza Título Antecipado
                        $boletpagar->setNrlancantecipa('');
                        $boletpagar->setIdbpagreal('N');
                        $boletpagar->setIdhabibpag('N');

                        $this->entityManager->persist($boletpagar);
                    }

                    // Exclui Antecipação
                    $this->entityManager->remove($antecipaforne);
                }
            }

            $this->entityManager->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Remessa Titulo a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $remessatitulo
     * @param $boletosmarcados
     * @return null|string
     * @throws \Exception
     */
    public function geraTituloAPagarDoc($row, $response, $permiteParcela, $nrorg, $cdfilial, $nrlanbolpag, $parcelas)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            self::geraTituloAPag($row, $response, $permiteParcela);
            
            // Gera Parcelamento
            $this->parcelamentoTituloService->geraParcTitulo($nrorg, $cdfilial, $nrlanbolpag, $parcelas);
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Valida Filial Logada Por Empresa Filtrada
     * @param $cdempresa
     * @param $cdfilial
     * @param $nrorg
     */
    public function validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial){
        try {
            return $this->titulosAPagarFactory->validaFilialLogadaPorEmpresaFiltrada($cdempresa, $cdfilial);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    /**
     * Habilitado Título a Pagar
     * @param $boletos
     * @param $idhabibpag
     * @throws \Exception
     */
    public function habilitaTituloAPagar($boletos, $idhabibpag)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            foreach($boletos as $boleto){
            
                // Busca Título a Pagar
                $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar($boleto['NRORG'], $boleto['CDFILIAL'], $boleto['NRLANBOLPAG']);
            
                // Atualiza Título a Pagar - IDHABIBPAG
                $boletpagar = $this->titulosAPagarFactory->atualizaBoletPagarHabilitado($boletpagar, $idhabibpag);
                
                $this->entityManager->persist($boletpagar);
                $this->entityManager->flush();
                
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Cadastrar Guia de Tributo
     * @param $dadosTributo
     * @param $titulosGuiaTributo
     * @throws \Exception
     */
    public function cadastrarGuiaTributo($dadosTributo, $titulosGuiaTributo){
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            foreach($titulosGuiaTributo as $titulo) {

                // Busca Boleto a Pagar
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->titulosAPagarFactory->buscaBoletPagar(
                    $titulo['NRORG'], $titulo['CDFILIAL'], $titulo['NRLANBOLPAG']
                );
                
                // Troca a vírgula pelo ponto dos valores dos campos VROUTRENTGUIA e VRPERCENTTTRIB para salvar
                $vroutrentguia = str_replace(',', '.', $dadosTributo['VROUTRENTGUIA']);
                $vrpercenttrib = str_replace(',', '.', $dadosTributo['VRPERCENTTTRIB']);
                
                $dadosTributo['NRREFEGUIA'] = isset($dadosTributo['NRREFEGUIA']) ? $dadosTributo['NRREFEGUIA'] : null;

                // Atualiza Boleto a Pagar - Guia de Tributo
                /** @var Boletpagar $boletpagar */
                $boletpagar = $this->titulosAPagarFactory->atualizaBoletPagarGuiaTributo(
                    $boletpagar, $dadosTributo['CDRECEITAGUIA'], $this->formataData($dadosTributo['DTAPURACGUIA']), $dadosTributo['NRREFEGUIA'], $vroutrentguia,
                    $vrpercenttrib
                );

                $this->entityManager->persist($boletpagar);
                $this->entityManager->flush();
            }
            
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Salva Anexo do Titulo a Pagar antes de salvar no FileServer, criando um arquivo na pasta /tmp/
     * @param $dadosTributo
     * @param $titulosGuiaTributo
     * @throws \Exception
     */
    public function salvaAnexoTitulo($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row['NRANEXOLANCBOL'] = $this->geraNovoCodigo($row['NRORG'], 'ANEXOLANCBOL');
            $row['CDTELA'] = 'PAG00001';
            //Salva o arquivo sempre como txt independente do seu formato
            $row['NMPATHANEXO'] = sys_get_temp_dir() . DIRECTORY_SEPARATOR . $row['CDTELA'] . $row['NRANEXOLANCBOL'] .'.txt';
            $row['DSINFOANEXO'] = $row['NMANEXOLANCBOL'][0]["lastModified"] . '_' . $row['NMANEXOLANCBOL'][0]["lastModifiedDate"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["webkitRelativePath"]. '_' . $row['NMANEXOLANCBOL'][0]["type"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["size"] . '_' . $row['NMANEXOLANCBOL'][0]["name"];
            
            $this->titulosAPagarFactory->salvaAnexoTitulo($row);
            
            file_put_contents($row['NMPATHANEXO'], $row['NMANEXOLANCBOL'][0]["b64File"]);
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    /**
     * Salva Anexo do Titulo a Pagar no FileServer, e atualiza dados do banco
     * @param $dadosTributo
     * @param $titulosGuiaTributo
     * @throws \Exception
     */
    public function salvaAnexoGrid($row){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $row['NRANEXOLANCBOL'] = $this->geraNovoCodigo($row['NRORG'], 'ANEXOLANCBOL');
            //Salva o arquivo sempre como txt independente do seu formato
            $row['DSINFOANEXO'] = $row['NMANEXOLANCBOL'][0]["lastModified"] . '_' . $row['NMANEXOLANCBOL'][0]["lastModifiedDate"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["webkitRelativePath"]. '_' . $row['NMANEXOLANCBOL'][0]["type"] . '_' . 
                                $row['NMANEXOLANCBOL'][0]["size"] . '_' . $row['NMANEXOLANCBOL'][0]["name"];
            
            
            $row['NMPATHANEXO'] = $this->saveXmlInFileUplouder($row['NMANEXOLANCBOL'][0]);
            
            $this->titulosAPagarFactory->salvaAnexoGrid($row);
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deletaAnexoTitulo($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                if(file_exists($row['NMPATHANEXO'])){
                    unlink($row['NMPATHANEXO']);
                }
                
                $this->titulosAPagarFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteAnexoGrid($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                
                $this->fileUploader->deleteFile($row['NMPATHANEXO']);
                
                $this->titulosAPagarFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deletaAnexoInFileServer($rows){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row){
                
                $this->deletaAnexoTitulo($row);
                
                $this->titulosAPagarFactory->deletaAnexoTitulo($row);
            }
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function deleteAnexoIncompletos(){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $anexosIncompletos = $this->titulosAPagarFactory->selectDeleteAnexoIncompletos();
            
            foreach ($anexosIncompletos as $anexo){
                if(file_exists($anexo['NMPATHANEXO'])){
                    unlink($anexo['NMPATHANEXO']);
                }
                
                $this->titulosAPagarFactory->deleteAnexoIncompletos($anexo);
            }
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function salvaAnexoInFileServer($anexos, $cdfilial, $nrlanbolpag){
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($anexos as $anexo){
                if(file_exists($anexo['NMPATHANEXO'])){
                    $arquivo = array();
                    $arquivo["b64File"] = file_get_contents($anexo['NMPATHANEXO']);
                    $arquivo["lastModified"] =  explode('_', $anexo['DSINFOANEXO'])[0];
                    $arquivo["lastModifiedDate"] = explode('_', $anexo['DSINFOANEXO'])[1];
                    $arquivo["webkitRelativePath"] = explode('_', $anexo['DSINFOANEXO'])[2];
                    $arquivo["type"] = explode('_', $anexo['DSINFOANEXO'])[3];
                    $arquivo["size"] = explode('_', $anexo['DSINFOANEXO'])[4];
                    $arquivo["name"] = explode('_', $anexo['DSINFOANEXO'])[5];
                    
                    unlink($anexo['NMPATHANEXO']);
                    
                    $anexo['NMPATHANEXO'] = $this->saveXmlInFileUplouder($arquivo);
                    
                    $anexo['CDFILIAL'] = $cdfilial;
                    $anexo['NRLANBOLPAG'] = $nrlanbolpag;
                    
                    $this->titulosAPagarFactory->uptadeAnexos($anexo);
                    
                } /*else {
                    DELETAR ARQUIVO CASO ELE NAO EXISTA ?
                }*/
            }
            
            $connection->commit();
            
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }
    
    public function saveXmlInFileUplouder($arquivo) {
        
        if (isset($arquivo)) {
            
            $file = array(
                "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                "lastModified" => $arquivo["lastModified"],
                "lastModifiedDate" => $arquivo["lastModifiedDate"],
                "webkitRelativePath" => $arquivo["webkitRelativePath"],
                "type" => $arquivo["type"],
                "size"  => $arquivo["size"], 
                "b64File" => $arquivo["b64File"]);
                
            $row['ANEXOLANCBOLPAG']    = array();
            $row['ANEXOLANCBOLPAG'][0] = $file;
            
            $this->fileUploader->uploadFilesInRow($row, 'ANEXOLANCBOLPAG', 'ANEXOLANCBOLPAG');
            
            return $row['ANEXOLANCBOLPAG'][0]['path'];
            
            /*
            
            $row = [
                'FILE' => [
                    [
                        "name" => time().'.'.explode('/', $arquivo["type"])[1], 
                        "type" => $arquivo["type"], 
                        "size" => $arquivo["size"], 
                        "path" => '', 
                        "b64File" => $arquivo["b64File"]
                    ]
                ],
                'FOLDER' => 'ANEXOLANCBOLPAG'
            ];
            
            return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
            */
            
        } else {
            return false;
        }
    }
    
    public function getFileServerUrl(){
        return $this->_fileServerUrl;
    }
    
    /**
     * Formata Data
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataData(
        $data
    ){
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
    
    public function verificaDadosGuiaTributo($row){
        try{
             
             $params = array(
                'NRLANBOLPAG' => $row['NRLANBOLPAG'],
             );
             
             $guiaCadastrada = $this->titulosAPagarFactory->verificaDadosGuiaTributo($params);
             return $guiaCadastrada;
            
        } catch (\Exception $e) {
            throw $e;
        }
    }

}