IF EXISTS (SELECT *
           FROM SYSOBJECTS O, SYSUSERS U
           WHERE O.NAME = 'CONSISTE_PEDIDO' AND
	         TYPE = 'P' 		    AND
      	         U.UID = O.UID              AND
      	         U.NAME = USER)
DROP PROCEDURE CONSISTE_PEDIDO
GO
CREATE PROCEDURE CONSISTE_PEDIDO @P_CDFILIAL   VARCHAR(4),
                                 @P_NRPEDIDO   VARCHAR(10),
                                 @P_NRITEMPED  VARCHAR(10),
                                 @P_CDCLIENTE  VARCHAR(21) = NULL,
                                 @P_TXMENSAGEM VARCHAR(200) OUTPUT,
                                 @P_PEDIDOOK   VARCHAR(1) OUTPUT
AS
   DECLARE @V_DIASEMANA    VARCHAR(1);
   DECLARE @V_DTINICIAL    DATETIME;
   DECLARE @V_CTPED        INTEGER;
   DECLARE @V_CDCLIENTE    VARCHAR(21);
   DECLARE @V_CDPRODUTO    VARCHAR(10);
   DECLARE @V_NRINCONSPED  VARCHAR(3);
   DECLARE @V_IDINCONSLIB  VARCHAR(1);

   DECLARE @V_CDCONSPEDIDO VARCHAR(3);
   DECLARE @V_NMCONSPEDIDO VARCHAR(50);
   DECLARE @V_NRITEMPED    VARCHAR(10);
/* Novas variaveis */
   DECLARE  @V_CTNOTA      INTEGER;
   DECLARE  @V_CTCLIE      INTEGER;
   DECLARE  @V_SGESTADO    VARCHAR(2);
   DECLARE  @V_NMMUNICIPIO VARCHAR(30);

BEGIN
/* P_PEDIDOOK
   A - OK
   B - INCONSISTENTE
*/
   SET @P_PEDIDOOK   = 'A';
   SET @P_TXMENSAGEM = '' ;

   DECLARE C_CONS CURSOR FOR
      SELECT CDCONSPEDIDO, NMCONSPEDIDO
      FROM CONSPEDIDO
      WHERE DTIMPCONSPED <= GETDATE()
        AND IDCONPEDATIV = 'S'
   OPEN C_CONS;
   FETCH NEXT FROM C_CONS INTO @V_CDCONSPEDIDO, @V_NMCONSPEDIDO;

   WHILE (@@FETCH_STATUS = 0) BEGIN
      /* PEDIDOS EM DUPLICIDADE POR CLIENTE PRODUTO */
      IF @V_CDCONSPEDIDO = '001' BEGIN
         /* DEFINICAO DO PERIODO PARA PESQUISA GENERICA
            Se o dia atual for sexta feira: o periodo sera do dia atual ate a (ULTIMA) sexta feira
            se nao sera: do dia atual ate a (PENULTIMA) sexta feira */
         SET @V_DIASEMANA = CAST(DATEPART(WEEKDAY, GETDATE()) AS VARCHAR(1));
         IF @V_DIASEMANA IN ('6', '7') BEGIN
            IF @V_DIASEMANA = '6' BEGIN
               SET @V_DTINICIAL = GETDATE() - 7;
            END
            ELSE BEGIN
               SET @V_DTINICIAL = GETDATE() - 8;
            END;
         END
         ELSE BEGIN
            SET @V_DTINICIAL = (GETDATE() + (6 - DATEPART(WEEKDAY, GETDATE()))) - 14;
         END;

         SELECT @V_CDCLIENTE = CDCLIENTE
         FROM PEDIDO
         WHERE CDFILIAL = @P_CDFILIAL
           AND NRPEDIDO = @P_NRPEDIDO;

         IF @P_NRITEMPED IS NULL BEGIN
            DECLARE C_ITEM CURSOR FOR
               SELECT NRITEMPED, CDPRODUTO
               FROM ITEMPEDIDO
               WHERE CDFILIAL  = @P_CDFILIAL
                 AND NRPEDIDO  = @P_NRPEDIDO
            OPEN C_ITEM;
            FETCH NEXT FROM C_ITEM INTO @V_NRITEMPED, @V_CDPRODUTO;

            WHILE (@@FETCH_STATUS = 0) BEGIN

               SELECT @V_CTPED = COUNT(*)
               FROM PEDIDO PE, ITEMPEDIDO IT
               WHERE PE.CDFILIAL  = @P_CDFILIAL
                 AND PE.DTPEDIDO BETWEEN @V_DTINICIAL AND GETDATE()
                 AND PE.CDCLIENTE = @V_CDCLIENTE
                 AND PE.IDSITPEDIDO IN ('1', '2')
                 AND IT.CDFILIAL  = PE.CDFILIAL
                 AND IT.NRPEDIDO  = PE.NRPEDIDO
                 AND IT.CDPRODUTO = @V_CDPRODUTO
                 AND IT.NRPEDIDO <> @P_NRPEDIDO;

               IF @V_CTPED > 0 BEGIN
                EXECUTE GRAVA_INCONSPEDIDO @P_CDFILIAL, @P_NRPEDIDO, @V_NRITEMPED, @V_CDCONSPEDIDO,             @P_IDINCONSLIB = @V_IDINCONSLIB OUTPUT ;
                  IF @V_IDINCONSLIB <> 'L' BEGIN
                     SET @P_PEDIDOOK = 'B';
                  END;
               END;
               FETCH NEXT FROM C_ITEM INTO @V_NRITEMPED, @V_CDPRODUTO;
            END;
            CLOSE C_ITEM;
            DEALLOCATE C_ITEM;
         END
         ELSE BEGIN
            SELECT @V_CDPRODUTO = CDPRODUTO
            FROM ITEMPEDIDO
            WHERE CDFILIAL  = @P_CDFILIAL
              AND NRPEDIDO  = @P_NRPEDIDO
              AND NRITEMPED = @P_NRITEMPED;

            SELECT @V_CTPED = COUNT(*)
            FROM PEDIDO PE, ITEMPEDIDO IT
            WHERE PE.CDFILIAL  = @P_CDFILIAL
              AND PE.DTPEDIDO BETWEEN @V_DTINICIAL AND GETDATE()
              AND PE.CDCLIENTE = @V_CDCLIENTE
              AND PE.IDSITPEDIDO IN ('1', '2')
              AND IT.CDFILIAL  = PE.CDFILIAL
              AND IT.NRPEDIDO  = PE.NRPEDIDO
              AND IT.CDPRODUTO = @V_CDPRODUTO;

            IF @V_CTPED > 0 BEGIN
               EXECUTE GRAVA_INCONSPEDIDO @P_CDFILIAL, @P_NRPEDIDO, @V_NRITEMPED, @V_CDCONSPEDIDO, @P_IDINCONSLIB = @V_IDINCONSLIB OUTPUT ;
               IF @V_IDINCONSLIB <> 'L' BEGIN
                  SET @P_PEDIDOOK = 'B';
               END;
            END;
         END;
      END;

      IF @P_CDCLIENTE IS NOT NULL BEGIN
         /********************************************************/
         /*****  Mensagem de Primeira Compra na Aprovao ********/
         /********************************************************/
         IF @V_CDCONSPEDIDO = '002' BEGIN
            /* Verifica se  a primeira compra do cliente e manda a mensagem na aprovao */
            SELECT @V_CTNOTA = COUNT(*)
            FROM NOTAFISCAL N, TIPOOPERAC T
            WHERE N.CDCLIENTE    = @P_CDCLIENTE
              AND N.IDENTRSAIDOP = 'S'
              AND T.CDTIPOOPER   = N.CDTIPOOPER
              AND T.IDENTRSAIDOP = N.IDENTRSAIDOP;

            IF @V_CTNOTA = 0 BEGIN
               SET @P_TXMENSAGEM = @P_TXMENSAGEM  + 'Primeira compra do cliente._';
               IF @P_PEDIDOOK = 'A' BEGIN
                  SET @P_PEDIDOOK = 'C';
               END
            END;
         END

         /********************************************************/
         /****** Cliente  fora do estado de Minas Gerais ********/
         /********************************************************/
         IF @V_CDCONSPEDIDO = '003' BEGIN
            SELECT @V_CTCLIE = COUNT(*)
            FROM ENDECLIE E
            WHERE E.CDCLIENTE    = @P_CDCLIENTE
              AND E.IDTPENDECLIE = 'T'
              AND E.SGESTADO     <> 'MG';

            IF @V_CTCLIE > 0 BEGIN
               SELECT @V_SGESTADO = M.SGESTADO, @V_NMMUNICIPIO = M.NMMUNICIPIO
               FROM ENDECLIE E, MUNICIPIO M
               WHERE E.CDCLIENTE    = @P_CDCLIENTE
                 AND E.IDTPENDECLIE = 'T'
                 AND M.CDPAIS       = E.CDPAIS
                 AND M.SGESTADO     = E.SGESTADO
                 AND M.CDMUNICIPIO  = E.CDMUNICIPIO
                 AND M.SGESTADO     <> 'MG';

               SET @P_TXMENSAGEM = @P_TXMENSAGEM +  'Entrega solicitada para o estado de [' + @V_SGESTADO + '] - [' + @V_NMMUNICIPIO + ']_';
               IF @P_PEDIDOOK = 'A' BEGIN
                  SET @P_PEDIDOOK = 'D';
               END
            END;
         END;
      END;
      FETCH NEXT FROM C_CONS INTO @V_CDCONSPEDIDO, @V_NMCONSPEDIDO;
   END;
   CLOSE C_CONS;
   DEALLOCATE C_CONS;
END;
GO


-- VERSAO --
EXEC ATUVEROBJ 'CONSISTE_PEDIDO','4.05.126';
GO

