// FILE: ./assets/js/utilities/VersionDataLibs.js
function VersionDataLibs(templateManager, eventAggregator, requestDataSourceEngine, ZHPromise){
    var self = this;
    
    self.showErrorMessage = function(error) {
        var message = error;
        if(typeof error === 'object') {
            if(error.message) {
                message = error.message;
            } else if(error.data && error.data.error){
                message = error.data.error;
            }
        }

        if(!message) {
            message = 'Ocorreu um erro inesperado. Entre em contato com o administrador do sistema.';
        } else if(message.indexOf('Exception: Exception: ') != -1) {
            message = message.replace('Exception: Exception: ', '');
        }

        ScreenService.showMessage(message, 'error');
    };

    self.buildVersion = function () {
        var deferred = ZHPromise.defer();
        if(!templateManager.versionData) {
            templateManager.versionData = true;
            var filter = [ {
                name: 'CURRENT_MODULE',
                operator: '=',
                value: 'fin'
            } ];
            return requestDataSourceEngine.getDataSource('fin#/lib_getVersionInfo', filter)
            .then(function (response) {
                templateManager.versionData = response.dataset.versionInfo;
                templateManager.versionData.dependencies.changeCurrentVersion = function(newVersion) {
                    templateManager.versionData.dependencies.current = templateManager.versionData.dependencies[newVersion];
                };
            })
            .catch(function(){
                deferred.reject(self.showErrorMessage);
            });
        }else{
            deferred.resolve();
        }
        return deferred.promise;
    };
    
    /*Esta função foi temporariamente criada para corrigir erros na geração do
    reporte em ambiente de produção para corrigir erros de incompatibilidade 
    entre o framework e a libs. Devendo posteriormente ser retirada.*/
    eventAggregator.subscribe("AboutOnClick", this.buildVersion);

    eventAggregator.subscribe("beforeProjectStartup", function() {
        templateManager.updateVersionData = self.buildVersion;
    });
}
Configuration(function(ContextRegister) {
    ContextRegister.register("VersionDataLibs", VersionDataLibs);
});

// FILE: ./assets/js/repositories/Ger03000_historicoIntegracao_Repository.js
var Ger03000_historicoIntegracao_Repository = function(restEngine){
	this.salvar = function(data){
		return restEngine.post('fin#/crud/histintegra_histpadr/save', {
            "requestType": "DataSet",
            "dataset": data
        });
	};

    this.find = function(){
        return restEngine.post('fin#/crud/histintegra_histpadr', {
            "requestType": "FilterData",
            "filter": []
        });
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03000_historicoIntegracao_Repository', Ger03000_historicoIntegracao_Repository);
});

// FILE: ./assets/js/repositories/Ger03200_ParametroFinanceiro_Repository.js
var Ger03200_ParametroFinanceiro_Repository = function(restEngine){
	this.save = function(dataSet){
		return restEngine.post('fin#/paramgeral_geralrec/save', {
			"requestType": "DataSet",
			"dataset": dataSet
		});
	};
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03200_ParametroFinanceiro_Repository', Ger03200_ParametroFinanceiro_Repository);
});

// FILE: ./assets/js/repositories/Ger06700_FormaPagamento_Repository.js
var Ger06700_FormaPagamento_Repository = function(restEngine, localStorageEngine){

    this.getPeriodo = function(cdformpgto, nrorg) {
        var filter = [
            {
                "name": "CDFORMPGTO",
                "operator": "=",
                "value": cdformpgto
            },
            {
                "name": "NRORG",
                "operator": "=",
                "value": nrorg
            }
        ];
        
        return restEngine.post('fin#/crud/diapgto', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

	this.excluiPeriodo = function(row) {
        return restEngine.post('fin#/crud/diapgto/remove', {
            "requestType": "Row",
            "row": row
        });
    };
    
};
 Configuration(function (ContextRegister){
    ContextRegister.register('Ger06700_FormaPagamento_Repository', Ger06700_FormaPagamento_Repository);
});

// FILE: ./assets/js/repositories/Ger08300_banco_Repository.js
function Ger08300_banco_Repository(restEngine) {

	this.save = function(row) {
        return restEngine.post('fin#/empresa_cnab/save', {
            "requestType": "Row",
            "row": row
        });
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger08300_banco_Repository', Ger08300_banco_Repository);
});

// FILE: ./assets/js/repositories/Ger08500_ContaCorrente_Repository.js
function Ger08500_ContaCorrente_Repository(restEngine, localStorageEngine) {
	this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("fin#/crud/filial", filter);
    };
	this.recalculaValor = function(row) {
        return restEngine.post('fin#/recalcularValor', {
            "requestType": "Row",
            "row": row
        });
	};
	this.validarContaCorrente = function(row) {
        return restEngine.post('fin#/contcorr/validarContaCorrente', {
            "requestType": "Row",
            "row": row
        });
	};
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Ger08500_ContaCorrente_Repository', Ger08500_ContaCorrente_Repository);
});

// FILE: ./assets/js/repositories/Ger25940_TipoRecebimento_Repository.js
var Ger25940_TipoRecebimento_Repository = function(restEngine, localStorageEngine){

	this.verificaRecebimentos = function(){
        return restEngine.post('fin#/tiporece_tiporece', {
            "requestType": "FilterData",
            "filter": ""
        });
    };
};
 Configuration(function (ContextRegister){
    ContextRegister.register('Ger25940_TipoRecebimento_Repository', Ger25940_TipoRecebimento_Repository);
});

// FILE: ./assets/js/repositories/Ger38000_TipoBaixaReceber_Repository.js
var Ger38000_TipoBaixaReceber_Repository = function(restEngine){

	this.salvar = function(data){
		return restEngine.post('fin#/tipobaixarec_histpadr/save', {
            "requestType": "DataSet",
            "dataset": data
        });
	};

    this.find = function(){
        return restEngine.post('fin#/tipobaixarec_histpadr', {
            "requestType": "FilterData",
            "filter": []
        });
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger38000_TipoBaixaReceber_Repository', Ger38000_TipoBaixaReceber_Repository);
});

// FILE: ./assets/js/repositories/Ger38100_ImportacaoTitulosAberto_Repository.js
var Ger38100_ImportacaoTitulosAberto_Repository = function(restEngine){
    
    this.importTituloPagar = function(row){
        return restEngine.post('fin#/importTituloPagar', {
            "requestType": "Row",
            "row": row
        });
    };
    
    this.importTituloReceber = function(row){
        return restEngine.post('fin#/importTituloReceber', {
            "requestType": "Row",
            "row": row
        });
    };
};

Configuration(function (ContextRegister){
 ContextRegister.register('Ger38100_ImportacaoTitulosAberto_Repository', Ger38100_ImportacaoTitulosAberto_Repository);
});

// FILE: ./assets/js/repositories/Lib00101_Contabilizacao_Factory.js
var Lib00101_Contabilizacao_Factory = function(restEngine) {

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContabilizacaoLancamentos = function(lancamentos) {
        return restEngine.post('fin#/getContabilizacaoLancamentos', {
            "requestType": "DataSet",
            "dataset": lancamentos
        });
    };

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContopectblParams = function(filter) {
        return restEngine.post('fin#/crud/contopectbl', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    /*
     * Função para retornar a parametrização geral.
     */
    this.getParamGeral = function() {
        return restEngine.post('fin#/crud/paramgeral', {
            "requestType": "FilterData",
            "filter": []
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('contabilizacaoFactory', Lib00101_Contabilizacao_Factory);
});

// FILE: ./assets/js/repositories/Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.js
var Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository = function(restEngine) {
    
  this.getCDPICTCCTB = function() {
      return restEngine.post("fin#/crud/paramgeral", {
                              "requestType": "FilterData",
                              "filter": []
      });        

  };

  this.getPARAMFILIAL = function() {
      return restEngine.post("fin#/crud/paramfilial", {
                              "requestType": "FilterData",
                              "filter": [{
                                name: "CDFILIAL",
                                operator: "=",
                                value: "FILIAL_LOGADA"
                              }]
      });        

  };
  
  this.getGERALREC = function() {
    return restEngine.post("fin#/crud/geral_rec", {
      "requestType": "FilterData",
      "filter": []
    });
  };
  
  this.getGERALPAG = function() {
    return restEngine.post("fin#/crud/geral_pag", {
      "requestType": "FilterData",
      "filter": []
    });
  };
  
  this.validateCheckedRow = function(row) {
      return restEngine.post("fin#/recalculo_titulos/validate_selected_row",{
                              "requestType": "Row",
                              "row": row
                              });
  };
  
  this.validateCheckedRows = function(checkedRows) {
  
    return restEngine.post('fin#/recalculo_titulos/validate_multiple_rows', {
            "requestType": "DataSet",
            "dataset": checkedRows
          });
  
  };
  
  this.validateCheckedRowBaixaPagar = function(row) {
      return restEngine.post("fin#/recalculo_baixas/validate_selected_row_pagar",{
                              "requestType": "Row",
                              "row": row
                              });
  };
  
  this.validateCheckedRowsBaixasPagar = function(checkedRows) {
  
    return restEngine.post('fin#/recalculo_baixas/validate_multiple_rows_pagar', {
            "requestType": "DataSet",
            "dataset": checkedRows
          });
  
  };
  
  this.validateCheckedRowBaixaReceber = function(row) {
      return restEngine.post("fin#/recalculo_baixas/validate_selected_row_receber",{
                              "requestType": "Row",
                              "row": row
                              });
  };
  
  this.validateCheckedRowsBaixasReceber = function(checkedRows) {
  
    return restEngine.post('fin#/recalculo_baixas/validate_multiple_rows_receber', {
            "requestType": "DataSet",
            "dataset": checkedRows
          });
  
  };
  
  this.recontabilizarLancamentos = function(rows) {
    
    return restEngine.post('fin#/recalculo_titulos/recontabilizar_lancamentos', {
              "requestType": "DataSet",
              "dataset": rows
            });
    
  };
  
  this.recontabilizarRecebimentos = function(rows) {
    
    return restEngine.post('fin#/recalculo_titulos/recontabilizar_recebimentos', {
              "requestType": "DataSet",
              "dataset": rows
            });
    
  };

  this.recontabilizarBaixasPagar = function(rows) {
    
    return restEngine.post('fin#/recalculo_baixas/recontabilizar_a_pagar', {
              "requestType": "DataSet",
              "dataset": rows
            });
    
  };

  this.recontabilizarBaixasReceber = function(rows) {
    
    return restEngine.post('fin#/recalculo_baixas/recontabilizar_a_receber', {
              "requestType": "DataSet",
              "dataset": rows
            });
    
  };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository', Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository);
}); 

// FILE: ./assets/modules/contaPagar/assets/js/repositories/Pag00001_TitulosAPagar_Repository.js
var pag00001_TitulosAPagar_Repository = function (restEngine, localStorageEngine) {
    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        
        //return localStorageEngine.download("fin#/crud/filial", filter);
        
        return restEngine.post("fin#/crud/filial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("fin#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.buscaEmpresa = function(cdempresa){
        return restEngine.post('fin#/crud/empresa', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };
    
    this.buscaPadraoSerieNF = function(cdfilial, cdfilirece) {
        return restEngine.post('fin_contaPagar#/crud/notafiscal_serieentr', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDFILIRECE",
                    "operator": "=",
                    "value": cdfilirece
                }
            ]
        });
    };

    this.buscaParamfilialFilialEmprcntrfil = function(cdempresa, cdfilial){
        return restEngine.post('fin_contaPagar#/crud/paramfilial_filial_emprcntrfilpag', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "P_CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

    this.buscaTipooperac = function(cdtipooper){
        return restEngine.post('fin#/crud/tipooperac', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "IDENTRSAIDOP",
                    "operator": "=",
                    "value": "E"
                },
                {
                    "name": "IDUTILOPER",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "CDTIPOOPER",
                    "operator": "=",
                    "value": cdtipooper
                }
            ]
        });
    };

    this.buscaGeralPag = function(){
        return restEngine.post('fin#/crud/geral_pag', {
            "requestType": "FilterData",
            "filter": []

        });
    };

    this.buscaFornecedor = function(cdforneced){
        return restEngine.post('fin#/crud/fornecedor', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFORNECED",
                    "operator": "=",
                    "value": cdforneced
                }
            ]
        });
    };

    this.buscaMunicipio = function(cdfilial){
        return restEngine.post('fin_contaPagar#/crud/municipio_endefili', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

    this.buscaContOpeCtbl = function(cdempresa){
        return restEngine.post('fin#/crud/contopectbl', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };

    this.buscaTipoctpagar = function(cdtpctpagar){
        return restEngine.post('fin#/crud/tipoctpagar', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDTPCTPAGAR",
                    "operator": "=",
                    "value": cdtpctpagar
                }
            ]
        });
    };

    this.buscaSerieentr = function(cdserieentr){
        return restEngine.post('fin#/crud/serieentr', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDSERIEENTR",
                    "operator": "=",
                    "value": cdserieentr
                }
            ]
        });
    };

    this.buscaAjusteTitulo = function(cdfilial, nrlanbolpag){
        return restEngine.post('fin#/crud/ajbolpag', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "NRLANBOLPAG",
                    "operator": "=",
                    "value": nrlanbolpag
                }
            ]
        });
    };

    this.buscaContaPagar = function(cdfilial, nrlanctonf){
        return restEngine.post('fin#/crud/contapagar', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "NRLANCTONF",
                    "operator": "=",
                    "value": nrlanctonf
                }
            ]
        });
    };

    this.transformaBaixaPlanejado = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/transformaBaixaPlanejado', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.geraCadastramentoAutomatico = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/geraCadastramentoAutomatico', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.transformaTituloPlanejado = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/transformaTituloPlanejado', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.atualizaTituloAPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/atualizaTituloAPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.salvaContaDocumento = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/salvaContaDocumento', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.geraNovoCodigo = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/geraNovoCodigo', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.editaTituloAPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/editaTituloAPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaVencTituloPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/deletaVencTituloPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaDocTituloPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/deletaDocTituloPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiTituloPagRemessa = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/excluiTituloPagRemessa', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.salvaAjusteBoleto = function(currentRow){
        return restEngine.post('fin_contaPagar#/ajbolpag_tipoajpagar/salvaAjusteBoleto', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaAjusteBoleto = function(currentRow){
        return restEngine.post('fin_contaPagar#/ajbolpag_tipoajpagar/deletaAjusteBoleto', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaRateioConta = function(currentRow){
        return restEngine.post('fin_contaPagar#/ratctpagar_filial_centcust/deletaRateioConta', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.remessaTituloPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/remessaTituloPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiTituloPagRemessa = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/excluiTituloPagRemessa', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.baixaTituloPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/baixaTituloPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiBaixaPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/excluiBaixaPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiTituloBaixaPagar = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/excluiTituloBaixaPagar', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiRemessaPag = function(currentRow){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/excluiRemessaPag', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaAjusteConta = function(currentRow){
        return restEngine.post('fin_contaPagar#/ajctpagar_tipoajpagar/deletaAjusteConta', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.geraTituloAPagarDoc = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/geraTituloAPagarDoc', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };
    
    this.saveAnexoDocGrif = function(currentRow){
        return restEngine.post('fin_contaPagar#/salvaAnexoTitulo', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.salvaAnexoGrid = function(currentRow){
        return restEngine.post('fin_contaPagar#/salvaAnexoGrid', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.deleteAnexoDocGrif = function(dataSet){
        return restEngine.post('fin_contaPagar#/deletaAnexoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    this.deleteAnexoGrid = function(dataSet){
        return restEngine.post('fin_contaPagar#/deleteAnexoGrid', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    this.deleteAnexoIncompletos = function(){
        return restEngine.post('fin_contaPagar#/deleteAnexoIncompletos');
    };
    
    this.getFileServerUrl = function(){
        return restEngine.post('fin_contaPagar#/getFileServerUrl', {
            "requestType": "Row",
            "row": {}
        });
    };
    
    this.validaAssociacaoTitulo = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/validaAssociacaoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.confirmaAssociacaoTitulo = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/confirmaAssociacaoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.cancelaAssociacaoTitulo = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/cancelaAssociacaoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.finalizaRateioConta = function(dataSet){
        return restEngine.post('fin_contaPagar#/ratctpagar_filial_centcust/finalizaRateioConta', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.finalizaParcelamentoTitulo = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_motivoparc_contcorr/finalizaParcelamentoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.deletaParcelamentoTitulo = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_motivoparc_contcorr/deletaParcelamentoTitulo', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.prorrogaTituloAPagar = function(dataSet){
        return restEngine.post('fin_contaPagar#/boletpagar_tipobaixapag_contcorr/prorrogaTituloAPagar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };
    
    this.existeParametros = function(){
        return restEngine.post('fin#/crud/geral_pag', {
            "requestType": "FilterData",
            "filter": []
        });
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Pag00001_TitulosAPagar_Repository', pag00001_TitulosAPagar_Repository);
});


// FILE: ./assets/modules/contaPagar/assets/js/repositories/Pag00200_RetornoPagamento_Repository.js
var Pag00200_RetornoPagamento_Repository = function (restEngine, localStorageEngine) {
    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("fin#/crud/filial", filter);
    };
    
    this.buscaEmpresa = function(cdempresa){
        return restEngine.post('fin#/crud/empresa', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };
};
Configuration(function(ContextRegister){
    ContextRegister.register('Pag00200_RetornoPagamento_Repository', Pag00200_RetornoPagamento_Repository);
});


// FILE: ./assets/modules/contaPagar/assets/js/repositories/Pag41200_TipoAjusteContaPagarRepository.js
var Pag41200_TipoAjusteContaPagarRepository = function(restEngine) {

  this.getContctblMask = function() {
    return restEngine.post("fin#/crud/paramgeral", {
                    "requestType": "FilterData",
                    "filter": []});
  };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag41200_TipoAjusteContaPagarRepository', Pag41200_TipoAjusteContaPagarRepository);
});

// FILE: ./assets/modules/contaReceber/assets/js/repositories/Rec00001_TitulosAReceber_Repository.js
var rec00001_TitulosAReceber_Repository = function (restEngine, localStorageEngine) {

    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        
        //return localStorageEngine.download("fin#/crud/filial", filter);
        
        return restEngine.post("fin#/crud/filial", {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.buscaPadraoSerieNF = function(cdfilial, cdfiliemit) {
        return restEngine.post('fin_contaReceber#/crud/notafiscal_serie', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDFILIEMIT",
                    "operator": "=",
                    "value": cdfiliemit
                }
            ]
        });
    };

    this.getDtprocessa = function(cdFilialOpcional) {
        var cdFilialFiltrada = cdFilialOpcional || "FILIAL_LOGADA";
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": cdFilialFiltrada
            }
        ];
        return restEngine.post("fin#/crud/paramestoque_processamento", {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.buscaEmpresa = function(cdempresa){
        return restEngine.post('fin#/crud/empresa', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };

    this.buscaParamfilialFilialEmprcntrfil = function(cdempresa, cdfilial){
        return restEngine.post('fin_contaReceber#/crud/paramfilial_filial_emprcntrfil', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "P_CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

    this.buscaTipooperac = function(cdtipooper){
        return restEngine.post('fin#/crud/tipooperac', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "IDENTRSAIDOP",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "IDUTILOPER",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "CDTIPOOPER",
                    "operator": "=",
                    "value": cdtipooper
                }
            ]
        });
    };

    this.buscaSerienfFilial = function(cdempresa, cdfilial, cdserie){
        return restEngine.post('fin_contaReceber#/serienf_filial', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                },
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "CDSERIE",
                    "operator": "=",
                    "value": cdserie
                }
            ]
        });
    };

    this.buscaCliente = function(cdcliente){
        return restEngine.post('fin#/crud/empresa', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDCLIENTE",
                    "operator": "=",
                    "value": cdcliente
                }
            ]
        });
    };

    this.buscaAjusteTitulo = function(cdfilial, nrlanbolrec){
        return restEngine.post('fin#/crud/ajbolrec', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "NRLANBOLREC",
                    "operator": "=",
                    "value": nrlanbolrec
                }
            ]
        });
    };

    this.buscaEndefili = function(cdfilial, idtpendefili){
        return restEngine.post('fin#/crud/endefili', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "IDTPENDEFILI",
                    "operator": "=",
                    "value": idtpendefili
                }
            ]
        });
    };

    this.buscaContaReceber = function(cdfilial, nrlanctonf){
        return restEngine.post('fin#/crud/contareceber', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "NRLANCTONF",
                    "operator": "=",
                    "value": nrlanctonf
                }
            ]
        });
    };

    this.buscaCliente = function(cdcliente){
        return restEngine.post('fin#/crud/cliente', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDCLIENTE",
                    "operator": "=",
                    "value": cdcliente
                }
            ]
        });
    };

    this.buscaTipoctreceber = function(cdtpctreceb){
        return restEngine.post('fin#/crud/tipoctreceber', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "IDUTLANCCREC",
                    "operator": "=",
                    "value": "S"
                },
                {
                    "name": "CDTPCTRECEB",
                    "operator": "=",
                    "value": cdtpctreceb
                }
            ]
        });
    };

    this.buscaGeralRec = function(){
        return restEngine.post('fin#/crud/geral_rec', {
            "requestType": "FilterData",
            "filter": []
        });
    };

    this.buscaParExpedicao = function(cdfilial){
        return restEngine.post('fin#/crud/parexpedicao', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                }
            ]
        });
    };

    this.buscaContOpeCtbl = function(cdempresa){
        return restEngine.post('fin#/crud/contopectbl', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };

    this.atualizaTituloAReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/atualizaTituloAReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.geraTituloAReceberDoc = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/geraTituloAReceberDoc', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };
    
    this.saveAnexoDocGrid = function(currentRow){
        return restEngine.post('fin_contaReceber#/salvaAnexoTituloRec', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.salvaAnexoGrid = function(currentRow){
        return restEngine.post('fin_contaReceber#/salvaAnexoGridRec', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.deleteAnexoDocGrid = function(dataSet){
        return restEngine.post('fin_contaReceber#/deletaAnexoTituloRec', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    this.deleteAnexoGrid = function(dataSet){
        return restEngine.post('fin_contaReceber#/deleteAnexoGridRec', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    this.deleteAnexoIncompletos = function(){
        return restEngine.post('fin_contaReceber#/deleteAnexoIncompletosRec');
    };
    
    this.getFileServerUrl = function(){
        return restEngine.post('fin_contaReceber#/getFileServerUrlRec', {
            "requestType": "Row",
            "row": {}
        });
    };

    this.geraNovoCodigo = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/geraNovoCodigo', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaVencTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/deletaVencTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.baixaTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/baixaTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.geraCadastramentoAutomatico = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/geraCadastramentoAutomatico', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.remessaTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/remessaTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiBaixaTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/excluiBaixaTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiBoletoBaixaTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/excluiBoletoBaixaTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiTituloRecRemessa = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/excluiTituloRecRemessa', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.excluiRemessaRec = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/excluiRemessaRec', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaDocTituloReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/deletaDocTituloReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.editarTituloAReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/edit', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaAjusteConta = function(currentRow){
        return restEngine.post('fin_contaReceber#/ajctreceber_tipoajreceber/remove', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaRateioConta = function(currentRow){
        return restEngine.post('fin_contaReceber#/ratctreceber_filial_centcust/remove', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.salvaDocumentoBoleto = function(currentRow){
        return restEngine.post('fin_contaReceber#/notafiscal_contareceber_boletreceber/save', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.salvarAjusteBoleto = function(dataSet){
        return restEngine.post('fin_contaReceber#/salvarAjusteBoleto/save', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.finalizaParcelamentoTitulos = function(dataSet){
        return restEngine.post('fin_contaReceber#/finalizaParcelamentoTitulos/save', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.finalizaTipoRecebimento = function(dataSet){
        return restEngine.post('fin_contaReceber#/tprecboleto_tiporece_cartao/save', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.transformaPlanejadoTitulo = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/transformaPlanejadoTitulo', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.transformaBoletoPlanejadoTitulo = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/transformaBoletoPlanejadoTitulo', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.downloadRemessaReceber = function(currentRow){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/downloadRemessaReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletaParcelamentoTitulos = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_motivoparc_contcorr/remove', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.deletaTipoRecebimento = function(dataSet){
        return restEngine.post('fin_contaReceber#/tprecboleto_tiporece_cartao/remove', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.confirmarAssociacaoTitulo = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/associar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.validaAssociarTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/validaAssociar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.validaProrrogarTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/validaProrrogar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.confirmarProrrogacaoTitulo = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/prorrogar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.emitirTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/emitir', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.cancelaAssociarTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/cancelaAssociar', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.cancelaEmissaoTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/cancelaEmissao', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.verificaCnabTituloAReceber = function(dataSet){
        return restEngine.post('fin_contaReceber#/boletreceber_tipobaixarec_contcorr/verificaCnab', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.finalizaRateioConta = function(dataSet){
        return restEngine.post('fin_contaReceber#/ratctreceber_filial_centcust/save', {
            "requestType": "DataSet",
            "dataset": dataSet,
            "params": []
        });
    };

    this.buscaEmprBanc = function(cdempresa, cdbanco) {
        var filter = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": cdempresa
            },
            {
                "name": "CDBANCO",
                "operator": "=",
                "value": cdbanco
            }
        ];
        return restEngine.post('fin#/crud/emprbanc', {
            "requestType": "FilterData",
            "filter": filter
        });
    };

    this.buscaInstCobranca = function(cdbanco, cdinstcobr) {
        var filter = [
            {
                "name": "CDBANCO",
                "operator": "=",
                "value": cdbanco
            },
            {
                "name": "CDINSTCOBR",
                "operator": "=",
                "value": cdinstcobr
            }
        ];
        return restEngine.post('fin#/crud/instcobranca', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    
    this.existeParametros = function(){
        return restEngine.post('fin#/crud/geral_rec', {
            "requestType": "FilterData",
            "filter": []
        });
    };
    
    this.buscaContaCorrenteVinculada = function(contaCorrente){
        return restEngine.post('fin_contaReceber#/buscaContaCorrenteVinculada', {
            "requestType": "Row",
            "row": contaCorrente
        });
    };
};
Configuration(function(ContextRegister){
    ContextRegister.register('Rec00001_TitulosAReceber_Repository', rec00001_TitulosAReceber_Repository);
});


// FILE: ./assets/modules/contaReceber/assets/js/repositories/Rec00200_RetornoCobranca_Repository.js
var rec00200_RetornoCobranca_Repository = function (restEngine, localStorageEngine) {

    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("fin#/crud/filial", filter);
    };

    this.buscaEmpresa = function(cdempresa){
        return restEngine.post('fin#/crud/empresa', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": cdempresa
                }
            ]
        });
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Rec00200_RetornoCobranca_Repository', rec00200_RetornoCobranca_Repository);
});


// FILE: ./assets/modules/contaReceber/assets/js/repositories/Rec15000_ChequeRecebidos_Repository.js
var Rec15000_ChequeRecebidos_Repository = function(restEngine){

};
Configuration(function (ContextRegister){
    ContextRegister.register('Rec15000_ChequeRecebidos_Repository', Rec15000_ChequeRecebidos_Repository);
});


// FILE: ./assets/modules/contaReceber/assets/js/repositories/Rec18000_CompensacaoContaReceber_Repositories.js
var Rec18000_CompensacaoContaReceber_Repositories = function(restEngine, localStorageEngine) {
    this.retornaUnidadeLogada = function() {
        var filter = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        return localStorageEngine.download("fin#/crud/filial", filter);
    };
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Rec18000_CompensacaoContaReceber_Repositories', Rec18000_CompensacaoContaReceber_Repositories);
});

// FILE: ./assets/modules/contaReceber/assets/js/repositories/Rec43000_ParametroContasReceber_Repository.js
var Rec43000_ParametroContasReceber_Repository = function(restEngine){

    //Indice 0 e 1 corresponde a row que vai ser salva na tabela Geral_Rec
    this.saveWidgets = function(dataSet){
    	return restEngine.post('fin_contaReceber#/geralrec_tipoajreceber/save', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
    
    this.saveTabedsOperadores = function(dataSet){
        return restEngine.post('fin_contaReceber#/saveTabedsOperadores', {
            "requestType": "DataSet",
            "dataset": dataSet
        });
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Rec43000_ParametroContasReceber_Repository', Rec43000_ParametroContasReceber_Repository);
});

// FILE: ./assets/modules/contaReceber/assets/js/repositories/rec41200_tipoAjusteReceberRepository.js
/**
 * Created by leonardooliveira on 08/03/2016.
 */
var tipoAjusteReceberRepository = function (restEngine) {

    this.salvarTipoAjuste = function(currentRow){
        return restEngine.post('\/dtsSaveTipoAjusteReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };

    this.deletarTipoAjuste = function(currentRow){
        return restEngine.post('\/dtsDeleteTipoAjusteReceber', {
            "requestType": "Row",
            "row": currentRow
        });
    };
    
    this.getContctblMask = function() {
        return restEngine.post("fin#/crud/paramgeral", {
                        "requestType": "FilterData", 
                        "filter": []});
    };
  
    this.getNvMaxContctbl = function() {
        return restEngine.post("fin_contaReceber#/dtsFindNvMaxContctblTipoAjusteReceber", {
                          "requestType": "FilterData",
                          "filter": []});
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('TipoAjusteReceberRepository', tipoAjusteReceberRepository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco01400_RecalculoLancamentoContaCorrente_Repository.js
var Bco01400_RecalculoLancamentoContaCorrente_Repository = function(restEngine) {
  
  this.validateCheckedRow = function(row) {
    
    return restEngine.post('fin_movimentacaoBancaria#/recalculo_lanc_contcorr/validate_selected_row', {
            "requestType": "Row",
            "row": row
          });
    
  };
  
  this.validateCheckedRows = function(checkedRows) {
    
    return restEngine.post('fin_movimentacaoBancaria#/recalculo_lanc_contcorr/validate_multiple_rows', {
            "requestType": "DataSet",
            "dataset": checkedRows
          });
    
  };
  
  this.recontabilizarLancamentos = function(rows) {
    
    return restEngine.post('fin_movimentacaoBancaria#/recalculo_lanc_contcorr/recontabilizar_lancamentos', {
              "requestType": "DataSet",
              "dataset": rows
            });
    
  };
  
  
};

Configuration(function (ContextRegister) {
  ContextRegister.register('Bco01400_RecalculoLancamentoContaCorrente_Repository', Bco01400_RecalculoLancamentoContaCorrente_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco01500_RecontabilizacaoDeTransferencia_Repository.js
var Bco01500_RecontabilizacaoDeTransferencia_Repository = function(restEngine) {
    
    this.recontabilizarTransferencias = function(rows) {
    
        return restEngine.post('fin_movimentacaoBancaria#/recontabilizacao_de_transferencia/recontabilizar_transferencias', {
                  "requestType": "DataSet",
                  "dataset": rows
                });
    
    };
    
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Bco01500_RecontabilizacaoDeTransferencia_Repository', Bco01500_RecontabilizacaoDeTransferencia_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco11100_movimentacaoContaCorrente_Repository.js
var Bco11100_movimentacaoContaCorrente_Repository = function(restEngine){
    this.buscaMunicipio = function(cdfilial, nrorg){
        return restEngine.post('fin_movimentacaoBancaria#/crud/municipio_endefili', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDFILIAL",
                    "operator": "=",
                    "value": cdfilial
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": nrorg
                }
            ]
        });
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_movimentacaoContaCorrente_Repository', Bco11100_movimentacaoContaCorrente_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco51100_TipodeDebitoBancario_Repository.js
function Bco51100_TipodeDebitoBancario_Repository(restEngine) {

	this.validarCodigo = function(row){
	    return restEngine.post("fin_movimentacaoBancaria#/tpdebibanc/validarCodigo", {
            "requestType": "Row",
            "row": row 
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Bco51100_TipodeDebitoBancario_Repository', Bco51100_TipodeDebitoBancario_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco51200_TipoCreditoBancario_Repository.js
function Bco51200_TipoDeCreditoBancario_Repository(restEngine) {

	this.validarCodigo = function(row){
	    return restEngine.post("fin_movimentacaoBancaria#/tpcredbanc/validarCodigo", {
            "requestType": "Row",
            "row": row 
        });
	};

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Bco51200_TipoDeCreditoBancario_Repository', Bco51200_TipoDeCreditoBancario_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco53100_ParametrosDeControleBancario_Repository.js
var Bco53100_ParametrosDeControleBancario_Repository = function(restEngine){
    this.consultaGeralBco = function(){
    	return restEngine.post('fin_movimentacaoBancaria#/crud/geral_bco', {
			"requestType": "FilterData",
			"filter": []
		});
    };

    this.consultaParamGeral = function(){
    	return restEngine.post('fin#/crud/paramgeral', {
			"requestType": "FilterData",
			"filter": []
		});
    };

    this.consultaGeralFcx = function(){
        return restEngine.post('fin#/crud/geral_fcx', {
            "requestType": "FilterData",
            "filter": []
        });
    };

    this.consultaClassfinanc = function(cdclassfina){
        return restEngine.post('fin#/crud/classfinanc', {
            "requestType": "FilterData",
            "filter": [
                {
                    "name": "CDCLASSFINA",
                    "operator": "=",
                    "value": cdclassfina
                }
            ]
        });
    };

    this.salvarGeralBco = function(data){
    	return restEngine.post('fin_movimentacaoBancaria#/savegeralbcoparamgeral/save', {
			"requestType": "DataSet",
			"dataset": data
		});
    };

};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco53100_ParametrosDeControleBancario_Repository', Bco53100_ParametrosDeControleBancario_Repository);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco54200_ExtratoBancario_Repository.js
var Bco54200_ExtratoBancario_Repository = function(restEngine){
    /*****************************Extrato Bancário**************************************/

    this.geraReport = function(filtro) {
    	return restEngine.post('fin_movimentacaoBancaria#/extrato_gera_report', {
			"requestType": "Row",
			"row": filtro
		});
    };

    /**************************Importação de Extrato Bancário*****************************/

    this.uploadFile = function(row) {
        return restEngine.post('fin_movimentacaoBancaria#/importaxao_extrato_readFile', {
		    "requestType": "Row",
			"row": row
		});
    };

    this.saveImport = function(data) {
        return restEngine.post('fin_movimentacaoBancaria#/importaxao_extrato_save', {
		    "requestType": "DataSet",
			"dataset": data
		});
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54200_ExtratoBancario_Repository', Bco54200_ExtratoBancario_Repository);
});


// FILE: ./assets/modules/movimentacaoBancaria/assets/js/repositories/Bco54500_Conciliacao_Repository.js
var Bco54500_Conciliacao_Repository = function(restEngine, localStorageEngine){
    
    this.getSaldoContaCorr = function(currentRow){
        return restEngine.post('fin_movimentacaoBancaria#/getSaldoContaCorr', {
            "requestType": "Row",
            "row": currentRow
        });
    };

};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54500_Conciliacao_Repository', Bco54500_Conciliacao_Repository);
});


// FILE: ./assets/js/services/Ger01600_IndiceFinanceiro_Service.js
var Ger01600_IndiceFinanceiro_Service = function(restEngine, localStorageEngine) {
	this.maxDtrefind = function(filter) {
        return restEngine.post('/max_dtrefind', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    this.minDtrefind = function(filter) {
        return restEngine.post('/min_dtrefind', {
            "requestType": "FilterData",
            "filter": filter
        });
    };
    this.saveMovindice = function(data) {
        return restEngine.post('fin#/crud/movindice/save', {
            "requestType": "DataSet",
            "dataset": data
        });
    };
    this.deleteMovindice = function(data) {
        return restEngine.post('fin#/crud/movindice/remove', {
            "requestType": "DataSet",
            "dataset": data
        });
    };

    this.deleteIndice = function(data){
        var dataSave =[];
        dataSave.push(data);
        return restEngine.post('/indice/remove', {
            "requestType": "DataSet",
            "dataset": dataSave
        });
    };
};
Configuration(function(ContextRegister) {
	ContextRegister.register("Ger01600_IndiceFinanceiro_Service", Ger01600_IndiceFinanceiro_Service);
});


// FILE: ./assets/js/services/Ger03000_historicoIntegracao_Service.js
var Ger03000_historicoIntegracao_Service = function(ScreenService, Ger03000_historicoIntegracao_Repository){
	this.insertInit = function(){
		return Ger03000_historicoIntegracao_Repository.find().then(function(banco){
			var consulta = banco.dataset.histintegra_histpadr;
			var row = [
				{
					__is_new : true,
					CDOPERACAO : 'CTAPAG'
				},
				{
					__is_new : true,
					CDOPERACAO : 'BAIXAP'
				},
				{
					__is_new : true,
					CDOPERACAO : 'CTAREC'
				},
				{
					__is_new : true,
					CDOPERACAO : 'BAIXAR'
				},
				{
					__is_new : true,
					CDOPERACAO : 'TRANSF'
				},
				{
					__is_new : true,
					CDOPERACAO : 'BXPADI'
				}
			];
			for(var i = 0; i < consulta.length; i++){
				var codigo = consulta[i].CDOPERACAO;
				if(codigo == 'CTAPAG'||codigo == 'BAIXAP'||codigo == 'CTAREC' || codigo == 'BAIXAR'||codigo == 'TRANSF'||codigo == 'BXPADI'){
					for(var j = 0; j < row.length; j++){
			   			if(codigo === row[j].CDOPERACAO){
			   				row.splice(j, 1);
			   				j = row.length - 1;
			   				if(!row[0]){
			   					i = consulta.length -1;
			   				}
			   			}
					}
				}
			}
			if(row[0]){
				return Ger03000_historicoIntegracao_Repository.salvar(row);
			}
		});
	};
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03000_historicoIntegracao_Service', Ger03000_historicoIntegracao_Service);
});

// FILE: ./assets/js/services/Ger03200_ParametroFinanceiro_Service.js
var Ger03200_ParametroFinanceiro_Service = function(Ger03200_ParametroFinanceiro_Repository){
    this.save = function(row){
        var dataSet = [];
        dataSet.push({
            IDCONSORCFIN: row.IDCONSORCFIN,
            IDTPORCAFINA: row.IDTPORCAFINA,
            IDDTBINFDTE: row.IDDTBINFDTE,
            IDDTBINFDTESREC: row.IDDTBINFDTESREC,
            IDCOFERIBANC: row.IDCOFERIBANC,
            CDPARGERAL: row.CDPARGERAL,
            DSURLHCM: row.DSURLHCM,
            DSIDHCM: row.DSIDHCM,
            DSSECRETHCM: row.DSSECRETHCM,
            NRORG: row.NRORG,
            __is_new: false
        });
        dataSet.push({
            IDORDIMPRBOL: row.IDORDIMPRBOL,
            CDGERALREC: row.CDGERALREC,
            NRORG: row.NRORG,
            __is_new: false
        });

        return Ger03200_ParametroFinanceiro_Repository.save(dataSet);
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03200_ParametroFinanceiro_Service', Ger03200_ParametroFinanceiro_Service);
});

// FILE: ./assets/js/services/Ger06700_FormaPagamento_Service.js
var Ger06700_FormaPagamento_Service = function(Ger06700_FormaPagamento_Repository){

    this.getPeriodo = function(cdformpgto, nrorg) {
        return Ger06700_FormaPagamento_Repository.getPeriodo(cdformpgto, nrorg);
    };
 
	this.excluiPeriodo = function(row) {
        return Ger06700_FormaPagamento_Repository.excluiPeriodo(row);
    };
};
 Configuration(function (ContextRegister){
    ContextRegister.register('Ger06700_FormaPagamento_Service', Ger06700_FormaPagamento_Service);
});

// FILE: ./assets/js/services/Ger08300_banco_Service.js
function Ger08300_banco_Service(Ger08300_banco_Repository) {

	this.salvar = function(row){
			return Ger08300_banco_Repository.save(row);
	};
}
Configuration(function (ContextRegister) {
	ContextRegister.register('Ger08300_banco_Service', Ger08300_banco_Service);
});

// FILE: ./assets/js/services/Ger08500_ContaCorrente_Service.js
var Ger08500_ContaCorrente_Service = function(Ger08500_ContaCorrente_Repository){

	this.retornaFilial = function(){
		return Ger08500_ContaCorrente_Repository.retornaUnidadeLogada();
	};
	this.recalcularValorUtilizado = function(row){
		return Ger08500_ContaCorrente_Repository.recalculaValor(row);
	};
	this.validarContaCorrente = function(row){
		return Ger08500_ContaCorrente_Repository.validarContaCorrente(row);
	};
};
 Configuration(function (ContextRegister){
    ContextRegister.register('Ger08500_ContaCorrente_Service', Ger08500_ContaCorrente_Service);
});

// FILE: ./assets/js/services/Ger25940_TipoRecebimento_Service.js
var Ger25940_TipoRecebimento_Service = function(Ger25940_TipoRecebimento_Repository){

	this.validateDelete = function(widget){
        // var rows = Ger25940_TipoRecebimento_Repository.verificaRecebimentos();
        return Ger25940_TipoRecebimento_Repository.verificaRecebimentos().then(function(callback){
            var dataset = callback.dataset.tiporece_tiporece;
            var valida = false;
            for (var i = 0; i < dataset.length; i++) {
                if (widget.currentRow.CDTIPORECE == dataset[i].CDTIPOTROCO){
                    valida = true;
                }
            }
            if (valida) {
                return true;
            } else {
                return false;
            }
        });
    };
};
 Configuration(function (ContextRegister){
    ContextRegister.register('Ger25940_TipoRecebimento_Service', Ger25940_TipoRecebimento_Service);
});

// FILE: ./assets/js/services/Ger38000_TipoBaixaReceber_Service.js
var Ger38000_TipoBaixaReceber_Service = function(Ger38000_TipoBaixaReceber_Repository){

	this.insertInit = function(widget){
		return Ger38000_TipoBaixaReceber_Repository.find().then(function(banco){
			var consulta = banco.dataset.tipobaixarec_histpadr;
			var row = [
				{
					__is_new : true,
					CDTPBAIXAREC : '01',
					NMTPBAIXAREC : 'Cobrança Bancária'
				},
				{
					__is_new : true,
					CDTPBAIXAREC : '02',
					NMTPBAIXAREC : 'Crédito em Conta'
				},
				{
					__is_new : true,
					CDTPBAIXAREC : '03',
					NMTPBAIXAREC : 'Cartão de Crédito'
				}
			];
			for(var i = 0; i < consulta.length; i++){
				var codigo = consulta[i].CDTPBAIXAREC;
				if(codigo == '01'||codigo == '02'||codigo == '03'){
					for(var j = 0; j < row.length; j++){
			   			if(codigo === row[j].CDTPBAIXAREC){
			   				row.splice(j, 1);
			   				j = row.length - 1;
			   				if(!row[0]){
			   					i = consulta.length -1;
			   				}
			   			}
					}
				}
			}
			if(row[0]){
				return Ger38000_TipoBaixaReceber_Repository.salvar(row);
			}
		});
	};
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger38000_TipoBaixaReceber_Service', Ger38000_TipoBaixaReceber_Service);
});

// FILE: ./assets/js/services/Ger38100_ImportacaoTitulosAberto_Service.js
var Ger38100_ImportacaoTitulosAberto_Service = function(Ger38100_ImportacaoTitulosAberto_Repository){
    
    this.importTituloPagar = function(row){
        return Ger38100_ImportacaoTitulosAberto_Repository.importTituloPagar(row);
    };
    
    this.importTituloReceber = function(row){
        return Ger38100_ImportacaoTitulosAberto_Repository.importTituloReceber(row);
    };
};

Configuration(function (ContextRegister){
 ContextRegister.register('Ger38100_ImportacaoTitulosAberto_Service', Ger38100_ImportacaoTitulosAberto_Service);
});

// FILE: ./assets/js/services/Lib00101_Contabilizacao_Service.js
var Lib00101_Contabilizacao_Service = function(ZHPromise, localStorageEngine, contabilizacaoFactory) {

    /*
     * Função para remover os dataSets locais que foram setados para a tela.
     * Esta deve ser executada ao sair da tela.
     */
    this.clearLocalData = function() {
        localStorageEngine.remove("fin#/erros_contabilizacao");
        localStorageEngine.remove("fin#/partidas_contabilizacao");
        localStorageEngine.remove("fin#/rateios_contabilizacao");
    };

    this.getContabilizacaoLancamentos = function(lancamentos) {
        return contabilizacaoFactory.getContabilizacaoLancamentos(lancamentos);
    };

    /*
     * Função para retornar a parametrização de contabilidade de acordo
     * com o filtro dado.
     */
    this.getContopectblParams = function(filter) {
        return contabilizacaoFactory.getContopectblParams(filter);
    };

    /*
     * Função que retorna os dados locais dos erros de contabilização.
     */
    this.getErros = function() {
        return localStorageEngine.getDataSource("fin#/erros_contabilizacao");
    };

    /*
     * Função para retornar a parametrização geral.
     */
    this.getParamGeral = function() {
        return contabilizacaoFactory.getParamGeral();
    };

    /*
     * Função que retorna os dados locais das partidas.
     */
    this.getPartidas = function() {
        return localStorageEngine.getDataSource("fin#/partidas_contabilizacao");
    };
    
    this.getRateios = function() {
        return localStorageEngine.getDataSource("fin#/rateios_contabilizacao");
    };

    /*
     * Função para setar os dataSets localmente, retornando uma promise
     * cujo callback será executado somente após salvar todos os dataSets.
     */
    this.setLocalData = function(dataSets) {
        var promises = [
            setLocalErrosContabilizacao(dataSets.errosContabilizacao),
            setLocalPartidasContabilizacao(dataSets.partidasContabilizacao),
            setLocalRateiosContabilizacao(dataSets.rateiosContabilizacao)
        ];
        return ZHPromise.all(promises);
    };

    /*
     * Função para setar o dataSet de erros localmente.
     */
    var setLocalErrosContabilizacao = function(errosContabilizacao) {
        errosContabilizacao = errosContabilizacao || [];
        return localStorageEngine.setDataSource("fin#/erros_contabilizacao", errosContabilizacao);
    };

    /*
     * Função para setar o dataSet de partidas localmente.
     */
     this.setPartidas = function(dataSet){
         var promises = [
            setLocalPartidasContabilizacao(dataSet),
        ];
        return ZHPromise.all(promises);
     };
     
    var setLocalPartidasContabilizacao = function(partidasContabilizacao) {
        partidasContabilizacao = partidasContabilizacao || [];
        return localStorageEngine.setDataSource("fin#/partidas_contabilizacao", partidasContabilizacao);
    };

    /*
     * Função para setar o dataSet de rateios localmente.
     */
     this.setRateio = function(dataSet){
         var promises = [
            setLocalRateiosContabilizacao(dataSet),
        ];
        return ZHPromise.all(promises);
     };
     
    var setLocalRateiosContabilizacao = function(rateiosContabilizacao) {
        rateiosContabilizacao = rateiosContabilizacao || [];
        return localStorageEngine.setDataSource("fin#/rateios_contabilizacao", rateiosContabilizacao);
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('contabilizacaoService', Lib00101_Contabilizacao_Service);
});

// FILE: ./assets/js/services/Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.js
var Pag00800_RecalculoContabilizacaoTitulosBaixas_Service = function(Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository) {

    this.getCDPICTCCTB = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getCDPICTCCTB().then(function(response) {
            var paramgeral = response.dataset.paramgeral[0];
            return paramgeral.CDPICTCCTB;
        });
    };
    
    this.getNRTAMNUMNFUT = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getPARAMFILIAL().then(function(response) {
            var paramfilial = response.dataset.paramfilial[0];
            return paramfilial.NRTAMNUMNFUT ? paramfilial.NRTAMNUMNFUT : 7;
        });   
    };
    
    this.getDiasTituloPagar = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getGERALPAG().then(function(response) {
            var geral_pag = response.dataset.geral_pag[0];
            return [geral_pag.QTDIAANTLANP, geral_pag.QTDIAAPOLANP];
        });
    };
    
    this.getDiasTituloReceber = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getGERALREC().then(function(response) {
            var geral_rec = response.dataset.geral_rec[0];
            return [geral_rec.QTDIAANTLANR, geral_rec.QTDIAAPOLANR];
        });
    };
    
    this.getDiasBaixaPagar = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getGERALPAG().then(function(response) {
            var geral_pag = response.dataset.geral_pag[0];
            return [geral_pag.QTDIAANTBAIP, geral_pag.QTDIAAPOBAIP];
        });
    };
    
    this.getDiasBaixaReceber = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.getGERALREC().then(function(response) {
            var geral_rec = response.dataset.geral_rec[0];
            return [geral_rec.QTDIAANTBAIR, geral_rec.QTDIAAPOBAIR];
        });
    };

    this.buildMainGridDataSourceFilter = function(widget) {
        widget.filterApplied = widget.dataSourceFilter.filter(function(filtro){return filtro.isCustomFilter;});
        var params = Util.clone(widget.filterApplied);
        var dataSourceFilter = [];
        
        params.forEach(function(criterion) {
            if (criterion.value instanceof Array && criterion.name !== 'CDFORNECED' && criterion.name !== 'CDCLIENTE') {
				if (/^DT.*$/.test(criterion.name)) {
					dataSourceFilter.unshift({
						name: "P_"+criterion.name+"INI",
						operator: "=",
						value: criterion.value[0]
					}, 
					{
						name: "P_"+criterion.name+"FIN",
						operator: "=",
						value: criterion.value[1]
					});
				} else {
					dataSourceFilter.unshift({
						name: "P_"+criterion.name,
						operator: "=",
						value: "_"+criterion.value.join("_")+"_"
					});
				}
            } else if(criterion.name === 'CDFORNECED' || criterion.name === 'CDCLIENTE') {
                 dataSourceFilter.unshift({
                	name: "P_"+criterion.name,
                	operator: "IN",
                	value: criterion.value
                });
            }
            
            else {
                dataSourceFilter.unshift({
                	name: "P_"+criterion.name,
                	operator: "=",
                	value: criterion.value
                });
            }
            
        });
        
        return dataSourceFilter;
        
    };


    this.buildAjustesDataSourceFilterFromRow = function(row, dataSourceFilter) {
        return buildAjustesDataSourceFilterFromParams(buildAjustesParamsFromRow(row), dataSourceFilter);
    };

    this.validateCheckedRow = function(row) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRow(row);
    };

    this.validateCheckedRows = function(checkedRows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRows(checkedRows);
    };
    
    this.validateCheckedRowBaixaPagar = function(row) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRowBaixaPagar(row);
    };

    this.validateCheckedRowsBaixasPagar = function(checkedRows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRowsBaixasPagar(checkedRows);
    };
    
    this.validateCheckedRowBaixaReceber = function(row) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRowBaixaReceber(row);
    };

    this.validateCheckedRowsBaixasReceber = function(checkedRows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.validateCheckedRowsBaixasReceber(checkedRows);
    };

    this.recontabilizarLancamentos = function(rows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.recontabilizarLancamentos(rows);
    };
    
    this.recontabilizarRecebimentos = function(rows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.recontabilizarRecebimentos(rows);
    };

    this.recontabilizarBaixasPagar = function(rows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.recontabilizarBaixasPagar(rows);
    };

    this.recontabilizarBaixasReceber = function(rows) {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Repository.recontabilizarBaixasReceber(rows);
    };

    var buildAjustesParamsFromRow = function(row) {
        var result = {};
        for (var key in row) {
            if (key == 'CDFILIAL' || key == 'NRLANCTONF' || key.match(/NRSEQVEN.*/) || key.match(/NRPARCELA.*/))
                result[key] = row[key];
        }
        return result;
    };

    var buildAjustesDataSourceFilterFromParams = function(params, dataSourceFilter) {
        for (var key in params) {
            if (params.hasOwnProperty(key)) {
                dataSourceFilter.push({
                    name: 'P_' + key,
                    operator: "=",
                    value: params[key]
                });
            }
        }
        return dataSourceFilter;
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register('Pag00800_RecalculoContabilizacaoTitulosBaixas_Service', Pag00800_RecalculoContabilizacaoTitulosBaixas_Service);
}); 

// FILE: ./assets/modules/contaPagar/assets/js/services/Pag00001_TitulosAPagar_Service.js
var pag00001_TitulosAPagar_Service = function (Pag00001_TitulosAPagar_Repository) {

    this.buscaFilial = function(){
        return Pag00001_TitulosAPagar_Repository.retornaUnidadeLogada();
    };

    this.buscaParamEstoque = function(){
        return Pag00001_TitulosAPagar_Repository.getDtprocessa();
    };

    this.buscaEmpresa = function(cdempresa){
        return Pag00001_TitulosAPagar_Repository.buscaEmpresa(cdempresa);
    };

    this.buscaMunicipio = function(cdfilial){
        return Pag00001_TitulosAPagar_Repository.buscaMunicipio(cdfilial);
    };

    this.buscaGeralPag = function(){
        return Pag00001_TitulosAPagar_Repository.buscaGeralPag();
    };
    
    this.buscaPadraoSerieNF = function(cdfilial, cdfilirece){
        return Pag00001_TitulosAPagar_Repository.buscaPadraoSerieNF(cdfilial, cdfilirece);
    };

    this.buscaParamfilialFilialEmprcntrfil = function(cdempresa, cdfilial){
        return Pag00001_TitulosAPagar_Repository.buscaParamfilialFilialEmprcntrfil(cdempresa, cdfilial);
    };

    this.buscaTipooperac = function(cdtipooper){
        return Pag00001_TitulosAPagar_Repository.buscaTipooperac(cdtipooper);
    };

    this.buscaFornecedor = function(cdforneced){
        return Pag00001_TitulosAPagar_Repository.buscaFornecedor(cdforneced);
    };

    this.buscaContOpeCtbl = function(cdempresa){
        return Pag00001_TitulosAPagar_Repository.buscaContOpeCtbl(cdempresa);
    };

    this.buscaSerieentr = function(cdserieentr){
        return Pag00001_TitulosAPagar_Repository.buscaSerieentr(cdserieentr);
    };

    this.buscaTipoctpagar = function(cdtpctpagar){
        return Pag00001_TitulosAPagar_Repository.buscaTipoctpagar(cdtpctpagar);
    };

    this.geraNovoCodigo = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.geraNovoCodigo(currentRow);
    };

    this.geraTituloAPagarDoc = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.geraTituloAPagarDoc(dataSet);
    };
    
    this.saveAnexoDocGrif = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.saveAnexoDocGrif(currentRow);
    };
    
    this.salvaAnexoGrid = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.salvaAnexoGrid(currentRow);
    };
    
    this.deleteAnexoDocGrif = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.deleteAnexoDocGrif(dataSet);
    };
    
    this.deleteAnexoGrid = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.deleteAnexoGrid(dataSet);
    };
    
    this.deleteAnexoIncompletos = function(){
        return Pag00001_TitulosAPagar_Repository.deleteAnexoIncompletos();
    };

    this.getFileServerUrl = function(){
        return Pag00001_TitulosAPagar_Repository.getFileServerUrl();
    };

    this.geraCadastramentoAutomatico = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.geraCadastramentoAutomatico(currentRow);
    };

    this.transformaBaixaPlanejado = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.transformaBaixaPlanejado(currentRow);
    };

    this.transformaTituloPlanejado = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.transformaTituloPlanejado(currentRow);
    };

    this.atualizaTituloAPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.atualizaTituloAPagar(currentRow);
    };

    this.editaTituloAPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.editaTituloAPagar(currentRow);
    };

    this.salvaContaDocumento = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.salvaContaDocumento(currentRow);
    };

    this.deletaVencTituloPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.deletaVencTituloPagar(currentRow);
    };

    this.deletaDocTituloPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.deletaDocTituloPagar(currentRow);
    };

    this.excluiTituloPagRemessa = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.excluiTituloPagRemessa(currentRow);
    };

    this.salvaAjusteBoleto = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.salvaAjusteBoleto(currentRow);
    };

    this.deletaAjusteBoleto = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.deletaAjusteBoleto(currentRow);
    };

    this.deletaRateioConta = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.deletaRateioConta(currentRow);
    };

    this.remessaTituloPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.remessaTituloPagar(currentRow);
    };

    this.excluiTituloPagRemessa = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.excluiTituloPagRemessa(currentRow);
    };

    this.excluiRemessaPag = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.excluiRemessaPag(currentRow);
    };

    this.baixaTituloPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.baixaTituloPagar(currentRow);
    };

    this.excluiBaixaPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.excluiBaixaPagar(currentRow);
    };

    this.excluiTituloBaixaPagar = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.excluiTituloBaixaPagar(currentRow);
    };

    this.deletaAjusteConta = function(currentRow){
        return Pag00001_TitulosAPagar_Repository.deletaAjusteConta(currentRow);
    };

    this.validaAssociacaoTitulo = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.validaAssociacaoTitulo(dataSet);
    };

    this.confirmaAssociacaoTitulo = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.confirmaAssociacaoTitulo(dataSet);
    };

    this.cancelaAssociacaoTitulo = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.cancelaAssociacaoTitulo(dataSet);
    };

    this.finalizaRateioConta = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.finalizaRateioConta(dataSet);
    };

    this.buscaAjusteTitulo = function(cdfilial, nrlanbolpag){
        return Pag00001_TitulosAPagar_Repository.buscaAjusteTitulo(cdfilial, nrlanbolpag);
    };

    this.buscaContaPagar = function(cdfilial, nrlanctonf){
        return Pag00001_TitulosAPagar_Repository.buscaContaPagar(cdfilial, nrlanctonf);
    };

    this.finalizaParcelamentoTitulo = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.finalizaParcelamentoTitulo(dataSet);
    };

    this.deletaParcelamentoTitulo = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.deletaParcelamentoTitulo(dataSet);
    };

    this.prorrogaTituloAPagar = function(dataSet){
        return Pag00001_TitulosAPagar_Repository.prorrogaTituloAPagar(dataSet);
    };
    
    this.existeParametros = function(){
        return Pag00001_TitulosAPagar_Repository.existeParametros();
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Pag00001_TitulosAPagar_Service', pag00001_TitulosAPagar_Service);
});


// FILE: ./assets/modules/contaPagar/assets/js/services/Pag00200_RetornoPagamento_Service.js
var Pag00200_RetornoPagamento_Service = function (Pag00200_RetornoPagamento_Repository) {

    this.buscaFilial = function(){
        return Pag00200_RetornoPagamento_Repository.retornaUnidadeLogada();
    };

    this.buscaEmpresa = function(cdempresa){
        return Pag00200_RetornoPagamento_Repository.buscaEmpresa(cdempresa);
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Pag00200_RetornoPagamento_Service', Pag00200_RetornoPagamento_Service);
});


// FILE: ./assets/modules/contaPagar/assets/js/services/Pag16000_EmprestimoLeasing_Service.js
var Pag16000_EmprestimoLeasing_Service = function(ScreenService, templateManager, General, ZHPromise) {
    this.efetuaCalculoParcelas = function(numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement, sistemaAmortizacao, taxaJuros){
        if (sistemaAmortizacao == 'O'){
            return this.amortizacaoPadrao(numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement);
        }else if (sistemaAmortizacao == 'P'){
            if (taxaJuros == 0){
                return this.amortizacaoPadrao(numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement);
            }else{
                return this.amortizacaoPrice(taxaJuros, numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement);
            }
        }else if (sistemaAmortizacao == 'S'){
            if (taxaJuros == 0){
                return this.amortizacaoPadrao(numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement);
            }else{
                return  this.amortizacaoSac(taxaJuros, numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement);
            }
        }
    };

    this.amortizacaoPadrao = function(numeroParcelas, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement){
        var listaParcelas = [];
        var valorAjusteParcela = 0;
        var valorAjusteUltimaParcela = 0;
        if (valorAjusteEmp){
            valorAjusteParcela = parseFloat(valorAjusteEmp / numeroParcelas).toFixed(2);
            valorAjusteUltimaParcela = parseFloat(valorAjusteParcela) + parseFloat((valorAjusteEmp - valorAjusteParcela * numeroParcelas).toFixed(2));
        }
        var parcelasEmprestimo = (totalEmprestimo/numeroParcelas).toFixed(2);
        var parcelasJuros = (parseFloat(parcelasEmprestimo) + parseFloat(valorAjusteParcela)).toFixed(2);
        var restoDivisao  = parseFloat(totalEmprestimo - (parcelasEmprestimo * numeroParcelas)).toFixed(2);
        var ultimaParcela = parseFloat(parcelasEmprestimo) + parseFloat(restoDivisao);
        var ultimaParcelaJuros = ultimaParcela + valorAjusteUltimaParcela;
        var valorDivida   = totalEmprestimo;
        for(var i = 1; i <= numeroParcelas; i++){
            var parcela  = i.toString().padStart(2, "0");
            valorDivida -= parcelasEmprestimo;
            var novaData = (dataIncrement && listaParcelas.length == 0) ? dataIncrement.format('DD/MM/YYYY') : dataIncrement.add(1, 'month').format('DD/MM/YYYY');
            if (i < numeroParcelas){
                listaParcelas.push(
                    {
                        "NRSEQVENPAG"  : parcela,
                        "NRPARCELAPAG" : "00",
                        "DTATUAVENPAG" : novaData,
                        "VRATUAPAG"    : parcelasEmprestimo,
                        "DSCONTAPAG"   : tipoConta + "  " + parcela + "/" + numeroParcelas.toString().padStart(2, "0"),
                        "VRFINALBPAG"  : parcelasJuros,
                        "VRAJUSTE"     : parseFloat(valorAjusteParcela).toFixed(2),
                        "VRDIVIDA"     : valorDivida,
                        "IDSTATPAG"    : 'A'
                    }
                );
            }else{
                listaParcelas.push(
                    {
                        "NRSEQVENPAG"  : parcela,
                        "NRPARCELAPAG" : "00",
                        "DTATUAVENPAG" : novaData,
                        "VRATUAPAG"    : ultimaParcela,
                        "DSCONTAPAG"   : tipoConta + "  " + parcela + "/" + numeroParcelas.toString().padStart(2, "0"),
                        "VRFINALBPAG"  : ultimaParcelaJuros,
                        "VRAJUSTE"     : parseFloat(valorAjusteUltimaParcela).toFixed(2),
                        "VRDIVIDA"     : valorDivida,
                        "IDSTATPAG"    : 'A'
                    }
                );
            }
        }
        return listaParcelas;
    };

    /**
     * [amortizacaoSac description]
     * @param  {[type]} i               [Juros]
     * @param  {[type]} n               [Número de Parcelas]
     * @param  {[type]} totalEmprestimo [description]
     * @param  {[type]} tipoConta       [description]
     * @param  {[type]} valorAjusteEmp  [description]
     * @param  {[type]} dataIncrement   [description]
     * @return {[type]}                 [description]
     */
    this.amortizacaoSac = function(i, n, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement){
        n = parseFloat(n);
        var listaParcelas = [];
        var valorAmortizado = (totalEmprestimo/n).toFixed(2);
        var restoDivisao = (totalEmprestimo - (valorAmortizado * n)).toFixed(2);
        var ultimaParcelaAmortizada = parseFloat(valorAmortizado) + parseFloat(restoDivisao);
        var valorBoleto  = 0;
        var saldoDevedor = totalEmprestimo;
        var valorDivida  = totalEmprestimo;
        var valorJuros   = 0;
        for(var j = 1; j <= n; j++){
            valorJuros   = (parseFloat(saldoDevedor) * parseFloat(i) / 100).toFixed(2);
            saldoDevedor = saldoDevedor - valorAmortizado;
            valorDivida -= valorAmortizado;
            var parcela  = j.toString().padStart(2, "0");
            var novaData = (dataIncrement && listaParcelas.length == 0) ? dataIncrement.format('DD/MM/YYYY') : dataIncrement.add(1, 'month').format('DD/MM/YYYY');
            if (j < n){
                valorBoleto = parseFloat(valorAmortizado) + parseFloat(valorJuros);
                listaParcelas.push(
                    {
                        "NRSEQVENPAG"  : parcela,
                        "NRPARCELAPAG" : "00",
                        "DTATUAVENPAG" : novaData,
                        "VRATUAPAG"    : valorAmortizado,
                        "DSCONTAPAG"   : tipoConta + "  " + parcela + "/" + n.toString().padStart(2, "0"),
                        "VRFINALBPAG"  : valorBoleto,
                        "VRAJUSTE"     : valorJuros,
                        "VRDIVIDA"     : valorDivida,
                        "IDSTATPAG"    : 'A'
                    }
                );
            }else{
                valorBoleto = parseFloat(ultimaParcelaAmortizada) + parseFloat(valorJuros);
                listaParcelas.push(
                    {
                        "NRSEQVENPAG"  : parcela,
                        "NRPARCELAPAG" : "00",
                        "DTATUAVENPAG" : novaData,
                        "VRATUAPAG"    : ultimaParcelaAmortizada,
                        "DSCONTAPAG"   : tipoConta + "  " + parcela + "/" + n.toString().padStart(2, "0"),
                        "VRFINALBPAG"  : valorBoleto,
                        "VRAJUSTE"     : valorJuros,
                        "VRDIVIDA"     : valorDivida,
                        "IDSTATPAG"    : 'A'
                    }
                );
            }
        }
        return listaParcelas;
    };

    /**
     * [amortizacaoPrice description]
     * @param  {[type]} i [Taxa de Juros]
     * @param  {[type]} n [Número de períodos]
     * @param  {[type]} valorEmprestimo [Valor do Empréstimo]
     * @return {[type]}           [description]
     */
    this.amortizacaoPrice = function(i, n, totalEmprestimo, tipoConta, valorAjusteEmp, dataIncrement){
        i = parseFloat(i) / 100;
        n = parseFloat(n);
        var listaParcelas = [];
        //Fórmula sistema Price
        var valorBoleto     = ((totalEmprestimo * i)/(1-(1/Math.pow((1+i), n)))).toFixed(2);
        var saldoDevedor    = totalEmprestimo;
        var valorJuros      = 0;
        var valorAmortizado = 0;
        var valorDivida     = totalEmprestimo;
        for(var j = 1; j <= n; j++){
            valorJuros      = (parseFloat(saldoDevedor) * parseFloat(i)).toFixed(2);
            valorAmortizado = valorBoleto - valorJuros;
            valorDivida    -= valorAmortizado;
            saldoDevedor    = saldoDevedor - valorAmortizado;
            var parcela     = j.toString().padStart(2, "0");
            var novaData    = (dataIncrement && listaParcelas.length == 0) ? dataIncrement.format('DD/MM/YYYY') : dataIncrement.add(1, 'month').format('DD/MM/YYYY');
            if (j < n){
                listaParcelas.push(
                    {
                        "NRSEQVENPAG" : parcela,
                        "NRPARCELAPAG": "00",
                        "DTATUAVENPAG": novaData,
                        "VRATUAPAG"   : valorAmortizado,
                        "DSCONTAPAG"  : tipoConta + "  " + parcela + "/" + n.toString().padStart(2, "0"),
                        "VRFINALBPAG" : valorBoleto,
                        "VRAJUSTE"    : valorJuros,
                        "VRDIVIDA"    : valorDivida,
                        "IDSTATPAG"   : 'A'
                    }
                );
            }else{
                listaParcelas.push(
                    {
                        "NRSEQVENPAG" : parcela,
                        "NRPARCELAPAG": "00",
                        "DTATUAVENPAG": novaData,
                        "VRATUAPAG"   : (valorAmortizado + saldoDevedor).toFixed(2),
                        "DSCONTAPAG"  : tipoConta + "  " + parcela + "/" + n.toString().padStart(2, "0"),
                        "VRFINALBPAG" : valorBoleto,
                        "VRAJUSTE"    : (valorJuros - saldoDevedor).toFixed(2),
                        "VRDIVIDA"    : valorDivida,
                        "IDSTATPAG"   : 'A'
                    }
                );
            }
        }
        return listaParcelas;
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag16000_EmprestimoLeasing_Service', Pag16000_EmprestimoLeasing_Service);
});

// FILE: ./assets/modules/contaPagar/assets/js/services/Pag41200_TipoAjusteContaPagarService.js
var Pag41200_TipoAjusteContaPagarService = function(Pag41200_TipoAjusteContaPagarRepository) {

  this.getContctblMask = function() {
    return Pag41200_TipoAjusteContaPagarRepository.getContctblMask();
  };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag41200_TipoAjusteContaPagarService', Pag41200_TipoAjusteContaPagarService);
});

// FILE: ./assets/modules/contaReceber/assets/js/services/Rec00001_TitulosAReceber_Service.js
var rec00001_TitulosAReceber_Service = function (Rec00001_TitulosAReceber_Repository) {

    var BASE_URL = 'http://midiatest.teknisa.com/files/';

    this.getFileUrl = function(row) {
        return BASE_URL + row.DSARQCNABREC;
    };

    this.buscaFilial = function(){
        return Rec00001_TitulosAReceber_Repository.retornaUnidadeLogada();
    };

    this.buscaParamEstoque = function(){
        return Rec00001_TitulosAReceber_Repository.getDtprocessa();
    };

    this.buscaGeralRec = function(){
        return Rec00001_TitulosAReceber_Repository.buscaGeralRec();
    };

    this.buscaParExpedicao = function(cdfilial){
        return Rec00001_TitulosAReceber_Repository.buscaParExpedicao(cdfilial);
    };
    
    this.buscaPadraoSerieNF = function(cdfilial, cdfiliemit){
        return Rec00001_TitulosAReceber_Repository.buscaPadraoSerieNF(cdfilial, cdfiliemit);
    };

    this.buscaCliente = function(cdcliente){
        return Rec00001_TitulosAReceber_Repository.buscaCliente(cdcliente);
    };

    this.buscaEndefili = function(cdfilial, idtpendefili){
        return Rec00001_TitulosAReceber_Repository.buscaEndefili(cdfilial, idtpendefili);
    };

    this.buscaContOpeCtbl = function(cdempresa){
        return Rec00001_TitulosAReceber_Repository.buscaContOpeCtbl(cdempresa);
    };

    this.buscaParamfilialFilialEmprcntrfil = function(cdempresa, cdfilial){
        return Rec00001_TitulosAReceber_Repository.buscaParamfilialFilialEmprcntrfil(cdempresa, cdfilial);
    };

    this.buscaTipooperac = function(cdtipooper){
        return Rec00001_TitulosAReceber_Repository.buscaTipooperac(cdtipooper);
    };

    this.buscaSerienfFilial = function(cdempresa, cdinscmini, cdincesta){
        return Rec00001_TitulosAReceber_Repository.buscaSerienfFilial(cdempresa, cdinscmini, cdincesta);
    };

    this.buscaAjusteTitulo = function(cdfilial, nrlanbolrec){
        return Rec00001_TitulosAReceber_Repository.buscaAjusteTitulo(cdfilial, nrlanbolrec);
    };

    this.buscaContaReceber = function(cdfilial, nrlanctonf){
        return Rec00001_TitulosAReceber_Repository.buscaContaReceber(cdfilial, nrlanctonf);
    };

    this.geraNovoCodigo = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.geraNovoCodigo(currentRow);
    };

    this.buscaCliente = function(cdcliente){
        return Rec00001_TitulosAReceber_Repository.buscaCliente(cdcliente);
    };

    this.buscaTipoctreceber = function(cdtpctreceb){
        return Rec00001_TitulosAReceber_Repository.buscaTipoctreceber(cdtpctreceb);
    };

    this.deletaVencTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.deletaVencTituloReceber(currentRow);
    };

    this.atualizaTituloAReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.atualizaTituloAReceber(currentRow);
    };

    this.deletaDocTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.deletaDocTituloReceber(currentRow);
    };

    this.geraCadastramentoAutomatico = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.geraCadastramentoAutomatico(currentRow);
    };

    this.transformaPlanejadoTitulo = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.transformaPlanejadoTitulo(currentRow);
    };

    this.transformaBoletoPlanejadoTitulo = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.transformaBoletoPlanejadoTitulo(currentRow);
    };

    this.buscaEmpresa = function(cdempresa){
        return Rec00001_TitulosAReceber_Repository.buscaEmpresa(cdempresa);
    };

    this.getEmpresaLogada = function(cdfilial){
        return Rec00001_TitulosAReceber_Repository.getEmpresaLogada(cdfilial);
    };

    this.geraTituloAReceberDoc = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.geraTituloAReceberDoc(dataSet);
    };
       
    this.saveAnexoDocGrid = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.saveAnexoDocGrid(currentRow);
    };
    
    this.salvaAnexoGrid = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.salvaAnexoGrid(currentRow);
    };
    
    this.deleteAnexoDocGrid = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.deleteAnexoDocGrid(dataSet);
    };
    
    this.deleteAnexoGrid = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.deleteAnexoGrid(dataSet);
    };
    
    this.deleteAnexoIncompletos = function(){
        return Rec00001_TitulosAReceber_Repository.deleteAnexoIncompletos();
    };

    this.getFileServerUrl = function(){
        return Rec00001_TitulosAReceber_Repository.getFileServerUrl();
    };
    
    this.salvaDocumentoBoleto = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.salvaDocumentoBoleto(currentRow);
    };

    this.editarTituloAReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.editarTituloAReceber(currentRow);
    };

    this.deletaAjusteConta = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.deletaAjusteConta(currentRow);
    };

    this.deletaAjusteConta = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.deletaAjusteConta(currentRow);
    };

    this.deletaRateioConta = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.deletaRateioConta(currentRow);
    };

    this.salvarAjusteBoleto = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.salvarAjusteBoleto(dataSet);
    };

    this.finalizaParcelamentoTitulos = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.finalizaParcelamentoTitulos(dataSet);
    };

    this.finalizaTipoRecebimento = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.finalizaTipoRecebimento(dataSet);
    };

    this.baixaTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.baixaTituloReceber(currentRow);
    };

    this.remessaTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.remessaTituloReceber(currentRow);
    };

    this.excluiBaixaTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.excluiBaixaTituloReceber(currentRow);
    };

    this.excluiBoletoBaixaTituloReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.excluiBoletoBaixaTituloReceber(currentRow);
    };

    this.excluiTituloRecRemessa = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.excluiTituloRecRemessa(currentRow);
    };

    this.excluiRemessaRec = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.excluiRemessaRec(currentRow);
    };

    this.deletaParcelamentoTitulos = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.deletaParcelamentoTitulos(dataSet);
    };

    this.deletaTipoRecebimento = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.deletaTipoRecebimento(dataSet);
    };

    this.confirmarAssociacaoTitulo = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.confirmarAssociacaoTitulo(dataSet);
    };

    this.validaAssociarTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.validaAssociarTituloAReceber(dataSet);
    };

    this.validaProrrogarTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.validaProrrogarTituloAReceber(dataSet);
    };

    this.confirmarProrrogacaoTitulo = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.confirmarProrrogacaoTitulo(dataSet);
    };

    this.emitirTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.emitirTituloAReceber(dataSet);
    };

    this.buscaEmprBanc = function(cdempresa, cdbanco){
        return Rec00001_TitulosAReceber_Repository.buscaEmprBanc(cdempresa, cdbanco);
    };

    this.buscaInstCobranca = function(cdbanco, cdinstcobr){
        return Rec00001_TitulosAReceber_Repository.buscaInstCobranca(cdbanco, cdinstcobr);
    };

    this.cancelaAssociarTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.cancelaAssociarTituloAReceber(dataSet);
    };

    this.cancelaEmissaoTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.cancelaEmissaoTituloAReceber(dataSet);
    };

    this.verificaCnabTituloAReceber = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.verificaCnabTituloAReceber(dataSet);
    };

    this.finalizaRateioConta = function(dataSet){
        return Rec00001_TitulosAReceber_Repository.finalizaRateioConta(dataSet);
    };

    this.downloadRemessaReceber = function(currentRow){
        return Rec00001_TitulosAReceber_Repository.downloadRemessaReceber(currentRow);
    };
    
    this.existeParametros = function(){
        return Rec00001_TitulosAReceber_Repository.existeParametros();
    };
    
    this.buscaContaCorrenteVinculada = function(contaCorrente){
        return Rec00001_TitulosAReceber_Repository.buscaContaCorrenteVinculada(contaCorrente);
    };
};
Configuration(function(ContextRegister){
    ContextRegister.register('Rec00001_TitulosAReceber_Service', rec00001_TitulosAReceber_Service);
});


// FILE: ./assets/modules/contaReceber/assets/js/services/Rec00200_RetornoCobranca_Service.js
var rec00200_RetornoCobranca_Service = function (Rec00200_RetornoCobranca_Repository) {

    this.buscaFilial = function(){
        return Rec00200_RetornoCobranca_Repository.retornaUnidadeLogada();
    };

    this.buscaEmpresa = function(cdempresa){
        return Rec00200_RetornoCobranca_Repository.buscaEmpresa(cdempresa);
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('Rec00200_RetornoCobranca_Service', rec00200_RetornoCobranca_Service);
});


// FILE: ./assets/modules/contaReceber/assets/js/services/Rec15000_ChequeRecebidos_Service.js
var Rec15000_ChequeRecebidos_Service = function(ZHPromise, Rec15000_ChequeRecebidos_Repository, General){
    // Valida se as linhas do parametro rows possuem CDREDUCONT
    this.verificaContaFavorecido = function(rows) {
		// 0 - Todas as rows validas
		// 1 - Existem rows validas e invalidas
		// 2 - Somente rows invalidas
		var promise = ZHPromise.defer();
		var erro = 3;
		var rowsReturn = [];
		if(rows) {
			rows.forEach(function(row){
				if(!row.CDREDUCONT) {
					if(erro === 3) {
						erro = 2;
					}
					else {
						erro = 1;
					}
				}
				else{ 
					if(erro === 3) erro = 0;
					else if(erro === 2) erro = 1;
					rowsReturn.splice(rowsReturn.length, 1, row);
				} 
			}.bind(this));
		}
		else erro = 2;
		promise.resolve([rowsReturn,erro]);
		return promise;
	};
	
	// Retorna a data atual no formato DD/MM/YYYY
	this.getDataAtual = function() {
		return General.today();
	};
	
	this.getCodLeitor = function(codCheque) {
		var codlength = codCheque.length;
		var arrayCode = codCheque.split('');
		var nrcheque = false;
		var compensacao = false;
		var banco = false;
		var agencia = false;
		var conta = false;
		nrcheque  = this.getPartCod(0, 5, arrayCode, codlength);
		if(codlength >= 7)  compensacao = this.getPartCod(6, 8, arrayCode, codlength);
		if(codlength >= 11) banco = this.getPartCod(9, 11, arrayCode, codlength);
		if(codlength >= 13) agencia = this.getPartCod(12, 15, arrayCode, codlength);
		if(codlength >= 17) conta = this.getPartCod(16, 31, arrayCode, codlength);
		return [nrcheque, compensacao, banco, agencia, conta];
	};
	
	this.getPartCod = function(tMin, tMax, arrayCode, codLength) {
		var resul = arrayCode[tMin].toString();
		for(var t = (tMin+1); (t <= tMax) && (t<=codLength-1); t++) {
			resul+= arrayCode[t];
		}
		return resul;
	};
};  
Configuration(function (ContextRegister){
    ContextRegister.register('Rec15000_ChequeRecebidos_Service', Rec15000_ChequeRecebidos_Service);
});


// FILE: ./assets/modules/contaReceber/assets/js/services/Rec18000_CompensacaoContaReceber_Service.js
var Rec18000_CompensacaoContaReceber_Service = function(localStorageEngine, Rec18000_CompensacaoContaReceber_Repositories) {

    this.getFilialLogada = function(){
        return Rec18000_CompensacaoContaReceber_Repositories.retornaUnidadeLogada();
    };

};

Configuration(function(ContextRegister) {
    ContextRegister.register("Rec18000_CompensacaoContaReceber_Service", Rec18000_CompensacaoContaReceber_Service);
});

// FILE: ./assets/modules/contaReceber/assets/js/services/Rec43000_ParametroContasReceber_Service.js
var Rec43000_ParametroContasReceber_Service = function(Rec43000_ParametroContasReceber_Repository){

    //Indice 0 e 1 corresponde a row que vai ser salva na tabela Geral_Rec
    this.saveWidgets = function(rows, isNew){
        var dataSet = this.preparaRowGeralRec(rows, isNew);

        return Rec43000_ParametroContasReceber_Repository.saveWidgets(dataSet);
    };
    
    this.saveTabedsOperadores = function(row){
        return Rec43000_ParametroContasReceber_Repository.saveTabedsOperadores(row);
    };

    this.preparaRowGeralRec = function(rows, isNew){
        var dataSet = [];
        var cdgeralrec =
        dataSet.push({
            __is_new : isNew,
            CDGERALREC : rows[0].CDGERALREC,
            NRORG : rows[0].NRORG,
            CDAJMULTAREC : rows[0].CDAJMULTAREC,
            CDAJMORAREC : rows[0].CDAJMORAREC,
            CDAJACREREC : rows[0].CDAJACREREC,
            CDAJDESCREC : rows[0].CDAJDESCREC,
            IDBLOQINCNTR : rows[0].IDBLOQINCNTR || 'N',
            CDGROPERCNTR : rows[0].CDGROPERCNTR,
            IDTPCNTRLIB : rows[0].IDTPCNTRLIB || 'N',
            IDUTCTRLCANREMR : rows[0].IDUTCTRLCANREMR || 'S',
            IDBLOQALTBOLREC : rows[0].IDBLOQALTBOLREC || 'N',
            IDUTLNFDEVOLREC : rows[0].IDUTLNFDEVOLREC || 'N',
            IDUTBCOPREFCLIE : rows[0].IDUTBCOPREFCLIE || 'N',
            IDCOMPCHQBAIXA : rows[0].IDCOMPCHQBAIXA || 'S',
            IDEXTELACALAJUBX : rows[0].IDEXTELACALAJUBX || 'B',
            CDAJACRERENEGO : rows[0].CDAJACRERENEGO,
            CDAJDESCRENEGO : rows[0].CDAJDESCRENEGO,
            CDTPBXRENEGO : rows[0].CDTPBXRENEGO,
            IDUTLPRORVCREC : rows[0].IDUTLPRORVCREC || 'N',
            CDAJTXCARTCNAB : rows[0].CDAJTXCARTCNAB,
            CDMOTIPROR : rows[0].CDMOTIPROR,
            QTDIAANTLANR : rows[1].QTDIAANTLANR,
            QTDIAAPOLANR : rows[1].QTDIAAPOLANR,
            QTDIAANTBAIR : rows[1].QTDIAANTBAIR,
            QTDIAAPOBAIR : rows[1].QTDIAAPOBAIR,
            QTDIAANTBOLR : rows[1].QTDIAANTBOLR,
            QTDIAAPOBOLR : rows[1].QTDIAAPOBOLR,
            IDEXCLAJBBXR : rows[0].IDEXCLAJBBXR || 'N',
            IDUTENVBOLAUTO : rows[0].IDUTENVBOLAUTO || 'N',
            QTDIAANTENVBOL : rows[0].QTDIAANTENVBOL,
            IDUTENVCARTAUTO : rows[0].IDUTENVCARTAUTO || 'N',
            QTDIAPOSENVCAR : rows[0].QTDIAPOSENVCAR,
            CDTPBAIRENVBOL : rows[0].CDTPBAIRENVBOL,
            IDUTVISUBOLOPEL : rows[0].IDUTVISUBOLOPEL || 'N',
            CDOPEPERVISUBOL : rows[0].CDOPEPERVISUBOL
        });
        return dataSet;
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Rec43000_ParametroContasReceber_Service', Rec43000_ParametroContasReceber_Service);
});

// FILE: ./assets/modules/contaReceber/assets/js/services/rec41200_tipoAjusteReceberService.js
/**
 * Created by leonardooliveira on 08/03/2016.
 */
var tipoAjusteReceberService = function (TipoAjusteReceberRepository) {

    this.salvarTipoAjuste = function(currentRow){
        return TipoAjusteReceberRepository.salvarTipoAjuste(currentRow);
    };

    this.deletarTipoAjuste = function(currentRow){
        return TipoAjusteReceberRepository.deletarTipoAjuste(currentRow);
    };
    
    this.getContctblMask = function() {
      return TipoAjusteReceberRepository.getContctblMask();
    };
  
    this.getNvMaxContctbl = function() {
        return TipoAjusteReceberRepository.getNvMaxContctbl();
    };

};
Configuration(function(ContextRegister){
    ContextRegister.register('TipoAjusteReceberService', tipoAjusteReceberService);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco01400_RecalculoLancamentoContaCorrente_Service.js
var Bco01400_RecalculoLancamentoContaCorrente_Service = function(Bco01400_RecalculoLancamentoContaCorrente_Repository) {
  
  this.buildDataSourceFromParams = function (params) {
    
    var dataSourceFilter = [];
    
    for(var param in params) {
      dataSourceFilter.push(
        {
          "name": param,
          "operator": "=",
          "value": params[param]
        }
      );
    }
    
    return dataSourceFilter;
    
  };
  
  this.buildParamsAccordingToFilter = function(currentFilter) {
    
    var params = {};
    
    //Lançamentos
    params.P_DEBCRED = currentFilter.P_DEBCRED;
    params.P_TIPO = "";
    if (typeof currentFilter.P_TIPOCRED instanceof Array) {
      if (currentFilter.P_TIPOCRED.length > 0)
        params.P_TIPO += currentFilter.P_TIPOCRED.join(" ");
    } 
    if (currentFilter.P_TIPODEB instanceof Array) {
      if(currentFilter.P_TIPODEB.length > 0)
        params.P_TIPO += currentFilter.P_TIPODEB.join(" ");
    }
    if (params.P_TIPO === ""){
      params.P_TTIPO = 'T';
    }
    
    //Valores
    if (valueRangeIsNotDefined(currentFilter)) {
      params.P_TVRMOVI = 'T';
    } else {
      params.P_TVRMOVI = ' ';
      params.P_VRINI = currentFilter.P_VRINI;
      params.P_VRFIN = currentFilter.P_VRFIN;
    }
    
    //Data
    if (datePeriodIsNotDefined(currentFilter)) {
      params.DTTODOS = 'T';
    } else {
      params.DTTODOS = ' ';
      params.P_DATAINI = currentFilter.DTBTMOVICRED.start+' 00:00:00';
      params.P_DATAFIN = currentFilter.DTBTMOVICRED.end+' 00:00:00';
    }
    
    params.CODIGO = currentFilter.CDREDUCONT;
    
    return params;
    
  };
  
  this.validateCheckedRow = function(row) {
    return Bco01400_RecalculoLancamentoContaCorrente_Repository.validateCheckedRow(row);
  };
  
  this.validateCheckedRows = function(checkedRows) {
    return Bco01400_RecalculoLancamentoContaCorrente_Repository.validateCheckedRows(checkedRows);
  };
  
  this.recontabilizarLancamentos = function(rows) {
    return Bco01400_RecalculoLancamentoContaCorrente_Repository.recontabilizarLancamentos(rows);
  };
  
  var valueRangeIsNotDefined = function(filter) {
    return !(filter.P_VRINI && filter.P_VRFIN);
  };
  
  var datePeriodIsNotDefined = function(filter) {
    if (filter.DTBTMOVICRED && filter.DTBTMOVICRED) {
      return !(filter.DTBTMOVICRED.end && filter.DTBTMOVICRED.start);
    } else {
      return true;
    }
  };
  
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco01400_RecalculoLancamentoContaCorrente_Service', Bco01400_RecalculoLancamentoContaCorrente_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco01500_RecontabilizacaoDeTransferencia_Service.js
var Bco01500_RecontabilizacaoDeTransferencia_Service = function(Bco01500_RecontabilizacaoDeTransferencia_Repository) {

    this.recontabilizarTransferencias = function(rows) {
        return Bco01500_RecontabilizacaoDeTransferencia_Repository.recontabilizarTransferencias(rows);
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Bco01500_RecontabilizacaoDeTransferencia_Service', Bco01500_RecontabilizacaoDeTransferencia_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco11100_movimentacaoContaCorrente_Service.js
var Bco11100_movimentacaoContaCorrente_Service = function(Bco11100_movimentacaoContaCorrente_Repository){
    this.buscaMunicipio = function(cdfilial, nrorg){
        return Bco11100_movimentacaoContaCorrente_Repository.buscaMunicipio(cdfilial, nrorg);
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_movimentacaoContaCorrente_Service', Bco11100_movimentacaoContaCorrente_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco21200_RelacaoSaldosContaCorrente_Service.js
var Bco21200_RelacaoSaldosContaCorrente_Service = function() {
  
  var self = this;
  
  this.buildDataSourceFilterFromFilterRow = function(filterRow) {
    var params = Util.clone(filterRow);
    return self.buildDataSourceFilterFromParams(self.buildParamsAccordingToFilter(params)); 
  };
  
  this.buildParamsAccordingToFilter = function(filterRow) {
    
    var params = {};
    
    //Empresa
    setParamsEmpresa(filterRow.CDEMPRESA, params);
    
    //Contcorr
    setParamsContcorr(filterRow.CDREDUCONT, params);
    
    //Data
    setParamsData(filterRow.DATA, params);
    
    return params;
    
  };
  
  this.buildDataSourceFilterFromParams = function (params) {
    
    var dataSourceFilter = [];
    
    for(var param in params) {
      dataSourceFilter.push(
        {
          "name": param,
          "operator": "=",
          "value": params[param]
        }
      );
    }
    
    return dataSourceFilter;
    
  };
  
  var setParamsEmpresa = function(CDEMPRESA, params) {
    if (CDEMPRESA instanceof Array) {
      if (CDEMPRESA.length > 0)
        params.CDEMPRESA = CDEMPRESA.join("_");
      else
        params.CDEMPRESA = 'T';
    } else {
      params.CDEMPRESA = 'T';
    }
  };
  
  var setParamsContcorr = function(CDREDUCONT, params) {
    if (CDREDUCONT instanceof Array) {
      if (CDREDUCONT.length > 0)
        params.CDREDUCONT = CDREDUCONT.join("_");
      else
        params.CDREDUCONT = 'T';
    } else {
      params.CDREDUCONT = 'T';
    } 
  };
  
  var setParamsData = function(DATA, params) {
    params.DATA = DATA;
  };
  
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco21200_RelacaoSaldosContaCorrente_Service', Bco21200_RelacaoSaldosContaCorrente_Service);
});



// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco51100_TipodeDebitoBancario_Service.js
function Bco51100_TipodeDebitoBancario_Service(Bco51100_TipodeDebitoBancario_Repository) {

	this.validarCodigo = function(row){
	    return Bco51100_TipodeDebitoBancario_Repository.validarCodigo(row);
	}; 

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Bco51100_TipodeDebitoBancario_Service', Bco51100_TipodeDebitoBancario_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco51200_TipoCreditoBancario_Service.js
function Bco51200_TipoDeCreditoBancario_Service(Bco51200_TipoDeCreditoBancario_Repository) {

	this.validarCodigo = function(row){
	    return Bco51200_TipoDeCreditoBancario_Repository.validarCodigo(row);
	}; 

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Bco51200_TipoDeCreditoBancario_Service', Bco51200_TipoDeCreditoBancario_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco54200_ExtratoBancario_Service.js
var Bco54200_ExtratoBancario_Service = function(Bco54200_ExtratoBancario_Repository){
    /*********************************Extrato Bancário***************************************/

    /**
     * Essa função tem como finalidade montar o filtro que sera passado para a função
     * que ira gerar o report.
     *
     * @param  {Object} row current row da widget
     */
    this.geraReport = function(row) {
        var filtro = {
            TDATA : row.DTINICIAL ? null : 'T',
            DTINICIAL : row.DTINICIAL || null,
            DTFINAL : row.DTFINAL || null,
            TCDREDUCONT : row.CDREDUCONT ? null : 'T',
            CDREDUCONT : row.CDREDUCONT || null,
            __report_name : row.REPORT_NAME || 'BCO54200'
        };
        return Bco54200_ExtratoBancario_Repository.geraReport(filtro);
    };

    /***************************Importação de Extrato Bancário*******************************/

    /**
     * Essa função tem como finalidade receber o conteudo do arquivo lido e dividilo onde ele de fato começa e do que
     * host local. Montar uma row que sera passado ao backend com pela classe Bco54200_ExtratoBancario_Repository com
     * a função uploadFile.
     *
     * @param  {String} contentFile conteudo do arquivo, em base 64
     * @param  {String} banco       CDBANCCNAB do banco do aquivo, caso o arquvio seja TXT
     * @param  {String} tipo        tipo do arquivo (TXT ou OFX)
     */
    this.uploadContentFile = function(contentFile, banco, tipo, empresa) {
        var contentFilSplit = contentFile.split(',');
        var contentFileFormat = contentFilSplit[1];
        var row = { CONTEUDO : contentFileFormat,
                    TIPO     : tipo,
                    BANCO    : banco,
                    EMPRESA : empresa};
        return Bco54200_ExtratoBancario_Repository.uploadFile(row);
    };

    /**
     * Essa função tem como finalidade retirar do arrauy data as linhas que tem a propriedade
     * SAVE igual a N, pois essa linhas não pode ser salvar. Depois chamar o repository que
     * ira salvar os dados que foram lidos do arquivo de extrato bancario a ser importado.
     *
     * @param  {Array} data registros lidos do arquivo
     * @return {Object}      promisse do restEngine
     */
    this.saveImport = function(data) {
        for (var i = data.length - 1; i >= 0; i--) {
            if (data[i].SAVE === 'N') {
                data.splice(i, 1);
            }
        }
        return Bco54200_ExtratoBancario_Repository.saveImport(data);
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54200_ExtratoBancario_Service', Bco54200_ExtratoBancario_Service);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/services/Bco54500_Conciliacao_Service.js
var Bco54500_Conciliacao_Service = function(Bco54500_Conciliacao_Repository){
    
    this.getSaldoContaCorr = function(currentRow){
        return Bco54500_Conciliacao_Repository.getSaldoContaCorr(currentRow);
    };
    
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54500_Conciliacao_Service', Bco54500_Conciliacao_Service);
});

// FILE: ./assets/js/controllers/BCO00700_Acompanhamento_Orcamentario.js
var BCO00700_Acompanhamento_Orcamentario = function(ScreenService, General, templateManager, ZhFilterPreferences){
    
    let paramIdtpcarfina = "";
    let auxIdRelatorio = "";
    
    this.openFilterRelatorio = function(widget){
        ZhFilterPreferences.openFilterWidget(widget);
    };

    this.tipodeacompanhamento = function(widget){
       var tipo = widget.currentRow.IDRELATORIO ;
       var grid =  templateManager.container.getWidget('BCO00700_Acompanhamento_Orcamentario');
        if(tipo == 'S'){
            grid.getField('DTPERIODO').isVisible = false;
            grid.getField('DTPERIODO').showOnForm = false;
            grid.getField('DTPERIODO').showOnList = false;
        }else{
            grid.getField('DTMES').isVisible = false;
            grid.getField('DTMES').showOnForm = false;
            grid.getField('DTMES').showOnList = false;
        }
    };
    
     this.tipodeempresa = function(widget){
       var tipo = widget.currentRow.IDTPORCAFINA;
        if(tipo == 'E'){
            widget.getField('IDTPORCAFINA').readOnly = true;
        }else{
            widget.getField('IDTPORCAFINA').readOnly = false;
        }
    };
    
     this.dataSource = function(widget,preference){
        var grid =  templateManager.container.getWidget('BCO00700_Acompanhamento_Orcamentario');
        var currentRowFiltro = preference.FILTER_ROW;
        if(grid.getField('IDRELATORIO').value() == 'S'){
            grid.dataSource.name = 'fin#/crud/acompSemestral';
        }else{
            grid.dataSource.name = 'fin#/crud/acompMensal';
        }
     };
     
   this.checkFieldsParamGeral = function(valueReport, widget) {
       General.callRestEngine("fin#/crud/paramgeral_geralrec", 'FilterData', []).then(function(success) {
           paramIdtpcarfina = success.dataset.paramgeral_geralrec[0].IDTPORCAFINA;
           var report = valueReport.owner.field.value();
           var dateMovimentation = widget.getField('DTMES');
           var dateMovimentationPeriod = widget.getField('DTPERIODO');
           var empresa = widget.getField('NMEMPRESA');
           var classFinanceira = widget.getField('CDCLASSFINA');
           if (report == 'M') {
               dateMovimentationPeriod.isVisible = true;
               dateMovimentation.isVisible =  false;
               empresa.setValue("");
               classFinanceira.setValue("");
               dateMovimentationPeriod.setValue("");
           } else {
             dateMovimentationPeriod.isVisible = false;
             dateMovimentation.isVisible =  true;
             empresa.setValue("");
             classFinanceira.setValue("");
             dateMovimentation.setValue("");
           }
            if(paramIdtpcarfina == 'E'){
                widget.getField('CDFILIAL').validations = {};
                widget.getField('CDFILIAL').isVisible = false;
                widget.getField('NMEMPRESA').validations = {};
            }else{
                widget.getField('CDFILIAL').validations = {"required": true};
                widget.getField('CDFILIAL').isVisible = true;
            }
            widget.reload();
        });
   };

   this.datasFiltros = function(date) {
        var dataCortada = date.split('/');
        var arrayData = [];
        var y = dataCortada[2];
        var m;
        //laço para gerar as datas de primeiro dia e ultimo dia e adiciona-las ao array de datas;
        for (var i = 1; i <= 6; i++) {
            m = parseInt(dataCortada[1]);
            if (m > 12) {
               m = 1;
               dataCortada[1] = `${01}`;
               dataCortada[2] ++;
               var ano = parseInt(dataCortada[2]);
               var auxRetorno = m === 2 ? y & 3 || (y % 25) && y & 15 ? 28 : 29 : 30 + (m + (m >> 3) & 1);
                if (m < 10) {
                    m = `0${m}`;
                }
                arrayData.push(`${dataCortada[0]}/${m}/${ano}`);
                arrayData.push(`${auxRetorno}/${m}/${ano}`);
            } else {
               var retorno = m === 2 ? y & 3 || (y % 25) && y & 15 ? 28 : 29 : 30 + (m + (m >> 3) & 1);
                if (m < 10) {
                    m = `0${m}`;
                }
                arrayData.push(`${dataCortada[0]}/${m}/${dataCortada[2]}`);
                arrayData.push(`${retorno}/${m}/${dataCortada[2]}`);
            }
            
            dataCortada[1]++; 
        }
        
        var aux = arrayData[0].split('/');
        
        if (aux[1] === '01') {
            arrayData.push(`0${1}/${12}/${aux[2] - 1}`);
            arrayData.push(`${31}/${12}/${aux[2] - 1 }`);
        } else {
            m = parseInt(aux[1] - 1);
            var ultimoDia = m === 2 ? y & 3 || (y % 25) && y & 15 ? 28 : 29 : 30 + (m + (m >> 3) & 1);
            if (m < 10) {
                m = `0${m}`;
            }
            arrayData.push(`${aux[0]}/${m}/${aux[2]}`);
            arrayData.push(`${ultimoDia}/${m}/${aux[2]}`);
        }
        return arrayData;
         
    };
   
   this.toApplyFilter = function(widget, preference) {
        var grid =  templateManager.container.getWidget('BCO00700_Acompanhamento_Orcamentario');
        var currentRowFiltro = preference.FILTER_ROW;
        var dtMes = currentRowFiltro.DTMES;
        var filialLogado = localStorage.getItem('CDFILIAL');
        var idRelatorio = currentRowFiltro.IDRELATORIO;
        auxIdRelatorio = idRelatorio;
        var cdtporcafina = currentRowFiltro.CDCLASSFINA;
        var cdfilial = currentRowFiltro.CDFILIAL;
        var cdempresa = currentRowFiltro.CDEMPRESA;
        var cdCodclass = currentRowFiltro.CDCLASSFINA;
        var dataBase = currentRowFiltro.DTPERIODO;
        var auxCdEmpresa = '';
        var auxCdClassificacao = '';
        // verifica se o campo empresa esta vazio;
        if (cdempresa) {
            auxCdEmpresa = cdempresa.join('_');
        } else {
            auxCdEmpresa = cdempresa;
        }
        
        // verifica se o campo classificacao esta vazio;
        if (cdCodclass) {
            auxCdClassificacao = cdCodclass.join('_');
        } else {
            auxCdClassificacao = cdCodclass;
        }
        
        // Verifica se o codigo Filial foi informado no Filtro, se foi é passado filial filtrada, se não filial logada
        if (!cdfilial){
            cdfilial = filialLogado.replace(/[^0-9]/g, "");
        }
        
        if(idRelatorio == 'S'){
            var datas = this.datasFiltros(dtMes);
            grid.getField('VRREALMESANTER').label = datas[12].substr(3, 10);
            grid.getField('VRREALMESATUAL').label = datas[0].substr(3, 10);
            grid.getField('VRREALMESPOST1').label = datas[2].substr(3, 10);
            grid.getField('VRREALMESPOST2').label = datas[4].substr(3, 10);
            grid.getField('VRREALMESPOST3').label = datas[6].substr(3, 10);
            grid.getField('VRREALMESPOST4').label = datas[8].substr(3, 10);
            grid.getField('VRREALMESPOST5').label = datas[10].substr(3, 10);
            grid.dataSource.name = 'fin#/crud/acompSemestral';
            widget.dataSourceFilter.splice(0,1);
            
            let filtroSemestral = [
                {
                    "name": "P_TPORCAFINA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDCLASSFINA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDEMPRESA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA1",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA8",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA2",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA9",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA3",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA10",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA4",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA11",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA5",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA12",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA6",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA13",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA7",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DATA14",
                    "operator": "=",
                    "value": ""
                }
            ];
            
            widget.dataSourceFilter = filtroSemestral;
            
            widget.dataSourceFilter.forEach(function(filtro) {
                switch (filtro.name) {
                    case 'P_TPORCAFINA':
                        filtro.value = paramIdtpcarfina;
                        break;
                    case 'P_CDFILIAL':
                            filtro.value = cdfilial;
                        break;
                    case 'P_CDCLASSFINA':
                        filtro.value = auxCdClassificacao? auxCdClassificacao: 'T';
                        break;
                    case 'P_CDEMPRESA':
                        filtro.value = auxCdEmpresa? auxCdEmpresa: 'T';
                        break;
                    // PRIMEIRO DIA DO MÊS ANTERIOR A DATA BASE
                    case 'P_DATA1':
                        filtro.value = datas[12];
                        break;
                    // ULTIMO DIA DO MÊS ANTERIOR A DATA BASE
                    case 'P_DATA8':
                        filtro.value = datas[13];
                        break;
                    // PRIMEIRO DIA DO MÊS DATA BASE
                    case 'P_DATA2':
                        filtro.value = datas[0];
                        break;
                    // ULTIMO DIA DO MÊS DATA BASE
                    case 'P_DATA9':
                        filtro.value = datas[1];
                        break;
                    // PRIMEIRO DIA DO MÊS 1º POSTERIOR A DATA BASE
                    case 'P_DATA3':
                        filtro.value = datas[2];
                        break;
                    // ULTIMO DIA DO MÊS 1º POSTERIOR A DATA BASE
                    case 'P_DATA10':
                        filtro.value = datas[3];
                        break;
                    // PRIMEIRO DIA DO MÊS 2º POSTERIOR A DATA BASE
                    case 'P_DATA4':
                        filtro.value = datas[4];
                        break;
                    // ULTIMO DIA DO MÊS 2º POSTERIOR A DATA BASE
                    case 'P_DATA11':
                        filtro.value = datas[5];
                        break;
                    // PRIMEIRO DIA DO MÊS 3º POSTERIOR A DATA BASE   
                    case 'P_DATA5':
                        filtro.value = datas[6];
                        break;
                    // ULTIMO DIA DO MÊS 3º POSTERIOR A DATA BASE
                    case 'P_DATA12':
                        filtro.value = datas[7];
                        break;
                    // PRIMEIRO DIA DO MÊS 4º POSTERIOR A DATA BASE
                    case 'P_DATA6':
                        filtro.value = datas[8];
                        break;
                    // ULTIMO DIA DO MÊS 4º POSTERIOR A DATA BASE
                    case 'P_DATA13':
                        filtro.value = datas[9];
                        break;
                    // PRIMEIRO DIA DO MÊS 5º POSTERIOR A DATA BASE
                    case 'P_DATA7':
                        filtro.value = datas[10];
                        break;
                    // ULTIMO DIA DO MÊS 5º POSTERIOR A DATA BASE
                    case 'P_DATA14':
                        filtro.value = datas[11];
                        break;
                }
            });
            
            delete currentRowFiltro.CDEMPRESA;
            delete currentRowFiltro.CDCLASSFINA;
            delete currentRowFiltro.IDRELATORIO;
            delete currentRowFiltro.DTMES;
            delete currentRowFiltro.CDFILIAL;
            
        }else{
            grid.dataSource.name = 'fin#/crud/acompMensal';
            widget.dataSourceFilter.splice(0,1);
            let filtroMensal= [
                {
                    "name": "P_TPORCAFINA",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDFILIAL",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DTINI",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDCLASS",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_DTFIM",
                    "operator": "=",
                    "value": ""
                },
                {
                    "name": "P_CDEMPRESA",
                    "operator": "=",
                    "value": ""
                }
                ];
            widget.dataSourceFilter = filtroMensal;
            widget.dataSourceFilter.forEach(function(filtro) {
                switch (filtro.name) {
                    case 'P_TPORCAFINA':
                        filtro.value = paramIdtpcarfina;
                        break;
                    case 'P_CDFILIAL':
                            filtro.value = cdfilial;
                        break;
                    case 'P_DTINI':
                        filtro.value = dataBase.start;
                        break;
                    case 'P_CDCLASS':
                        filtro.value = auxCdClassificacao? auxCdClassificacao: 'T';
                        break;
                    case 'P_DTFIM':
                        filtro.value = dataBase.end;
                        break;
                    case 'P_CDEMPRESA':
                        filtro.value = auxCdEmpresa? auxCdEmpresa: 'T';
                        break;
                }
            });
            delete currentRowFiltro.CDEMPRESA;
            delete currentRowFiltro.CDCLASSFINA;
            delete currentRowFiltro.IDRELATORIO;
            delete currentRowFiltro.CDFILIAL;
            delete currentRowFiltro.DTMES;
        }
    };
    
    this.filtraFilialPorEmpresa = function(widget) {
        var filial = widget.getField("CDFILIAL");
        filial.setValue("");
        var filter = [
            {
                "name": "P_CDEMPRESA",
                "operator": "=",
                "value": widget.getField('CDEMPRESA').value()
            },
            {
                "name": "P_CDFILIAL",
                "operator": "=",
                "value": 'T'
            }
        ];
        General.callRestEngine("fin#/crud/filial_acomp", 'FilterData', filter).then(function(success) {
            widget.getField("CDFILIAL").dataSource.data = success.dataset.filial_acomp;
        });
    };
    
    
    this.controlaVisualizacaoGrid = function() {
        var grid =  templateManager.container.getWidget('BCO00700_Acompanhamento_Orcamentario');
        var vrRealizado = grid.getField('VRREALIZADO');
        var vrVencido = grid.getField('VRVENCIDO');
        var vrAvencer = grid.getField('VRAVENCER');
        var vrPrevPag = grid.getField('VRPREVPAG');
        var vrOrcFinal = grid.getField('VRORCAFINA');
        var desvio = grid.getField('DESVIO');
        var vrRealMesPos5 = grid.getField('VRREALMESPOST5');
        var vrRealMesPos4 = grid.getField('VRREALMESPOST4');
        var vrRealMesPos3 = grid.getField('VRREALMESPOST3');
        var vrRealMesPos2 = grid.getField('VRREALMESPOST2');
        var vrRealMesPos1 = grid.getField('VRREALMESPOST1');
        var vrRealMesAtual = grid.getField('VRREALMESATUAL');
        var vrRealMesAnter = grid.getField('VRREALMESANTER');
        if (auxIdRelatorio == 'S'){
            vrRealizado.isVisible = false;
            vrRealizado.isVisibleOnReport = false;
            vrVencido.isVisible = false;
            vrVencido.isVisibleOnReport = false;
            vrAvencer.isVisible = false;
            vrAvencer.isVisibleOnReport = false;
            vrPrevPag.isVisible = false;
            vrPrevPag.isVisibleOnReport = false;
            vrOrcFinal.isVisible = false;
            vrOrcFinal.isVisibleOnReport = false;
            desvio.isVisible = false;
            desvio.isVisibleOnReport = false;
            vrRealMesPos4.isVisible = true;
            vrRealMesPos4.isVisibleOnReport = true;
            vrRealMesPos3.isVisible = true;
            vrRealMesPos3.isVisibleOnReport = true;
            vrRealMesPos2.isVisible = true;
            vrRealMesPos2.isVisibleOnReport = true;
            vrRealMesPos1.isVisible = true;
            vrRealMesPos1.isVisibleOnReport = true;
            vrRealMesAtual.isVisible = true;
            vrRealMesAtual.isVisibleOnReport = true;
            vrRealMesPos5.isVisible = true;
            vrRealMesPos5.isVisibleOnReport = true;
            vrRealMesAnter.isVisible = true;
            vrRealMesAnter.isVisibleOnReport = true;
        } else {
            vrRealMesPos5.isVisible = false;
            vrRealMesPos5.isVisibleOnReport = false;
            vrRealMesPos1.isVisible = false;
            vrRealMesPos1.isVisibleOnReport = false;
            vrRealMesPos4.isVisible = false;
            vrRealMesPos4.isVisibleOnReport = false;
            vrRealMesPos3.isVisible = false;
            vrRealMesPos3.isVisibleOnReport = false;
            vrRealMesPos2.isVisible = false;
            vrRealMesPos2.isVisibleOnReport = false;
            vrRealMesAtual.isVisible = false;
            vrRealMesAtual.isVisibleOnReport = false;
            vrRealMesAnter.isVisible = false;
            vrRealMesAnter.isVisibleOnReport = false;
            vrRealizado.isVisible = true;
            vrRealizado.isVisibleOnReport = true;
            vrVencido.isVisible = true;
            vrVencido.isVisibleOnReport = true;
            vrAvencer.isVisible = true;
            vrAvencer.isVisibleOnReport = true;
            vrPrevPag.isVisible = true;
            vrPrevPag.isVisibleOnReport = true;
            vrOrcFinal.isVisible = true;
            vrOrcFinal.isVisibleOnReport = true;
            desvio.isVisible = true;
            desvio.isVisibleOnReport = true;
        }
        grid.reload();
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('BCO00700_Acompanhamento_Orcamentario', BCO00700_Acompanhamento_Orcamentario);
});

// FILE: ./assets/js/controllers/Bco55200_TalaoCheque_Controller.js
var Bco55200_TalaoCheque_Controller = function(ScreenService, General, templateManager){
    
    var data;
    var valida;
    var numeroCheque;
    var valida1;

    this.ativarWidget = function(args){
        var gridTalao =  templateManager.container.getWidget('grid_talao');
        gridTalao.activate();
    };
    
    this.adicionarTalao = function(widget) {
        var filter_row = templateManager.container.getWidget('grid_talao').appliedFilter.FILTER_ROW;
        
        if(filter_row.CDREDUCONT)
            General.widgetNewRow(widget);
        else ScreenService.showMessage('Operação bloqueada. Banco não selecionado no filtro.');
    };
    
    
    this.salvarTalao = function(widget){
        var gridTalao =  templateManager.container.getWidget('grid_talao');
        var filterPopup = gridTalao.appliedFilter;
        if(widget.currentRow.NRCHEQINIC<=widget.currentRow.NRCHEQFINA){
            widget.currentRow.CDRESPRECEB = getLocalVar("CDOPERADOR");
            widget.currentRow.CDREDUCONT  = filterPopup.FILTER_ROW.CDREDUCONT;
            widget.currentRow.QTCHEQUE    = (widget.currentRow.NRCHEQFINA - widget.currentRow.NRCHEQINIC) + 1;
            var filter = [
                {
                    "name": "CDOPERADOR",
                    "operator": "=",
                    "value": getLocalVar("CDOPERADOR")
                }  
            ];
            General.callRestEngine('fin#/crud/operador', 'FilterData', filter).then(function(success){
                widget.currentRow.NMOPERADOR  = success.dataset.operador[0].NMOPERADOR;
                General.widgetSave(widget);
            });
        }else{
            ScreenService.showMessage("Cheque inicial deve ser menor que cheque final.");
        }
    };
    
    this.verificaActionCheque = function(args){
        var gridCheque =  templateManager.container.getWidget('grid_cheque');
        var situacao = gridCheque.currentRow.IDSTCHEQUE;

        switch (situacao) {
            case "0":
                gridCheque.getAction('cancelarCheque').isVisible = true;
                gridCheque.getAction('disponibilizarCheque').isVisible = false;
                break;
            case "1":
                gridCheque.getAction('cancelarCheque').isVisible = false;
                gridCheque.getAction('disponibilizarCheque').isVisible = false;
                break;
            case "2":
                gridCheque.getAction('cancelarCheque').isVisible = false;
                gridCheque.getAction('disponibilizarCheque').isVisible = true;
                break;
            default:
                gridCheque.getAction('cancelarCheque').isVisible = false;
                gridCheque.getAction('disponibilizarCheque').isVisible = false;
        }

    };
    this.limpaAction = function(){
        var gridCheque =  templateManager.container.getWidget('grid_cheque');
        gridCheque.getAction('cancelarCheque').isVisible = false;
        gridCheque.getAction('disponibilizarCheque').isVisible = false;
    };

    this.desabilitarAction = function(){
        var gridCheque =  templateManager.container.getWidget('grid_cheque');
        gridCheque.getAction('voltarPagina').isVisible = false;
        gridCheque.getAction('cancelarCheque').isVisible = false;
    };
    this.habilitarAction =  function(){
        var gridCheque =  templateManager.container.getWidget('grid_cheque');
        gridCheque.getAction('voltarPagina').isVisible = true;
        gridCheque.getAction('cancelarCheque').isVisible = true;
    };
    this.disponibilizarCheque = function(args){
            ScreenService.confirmMessage("Deseja disponibilizar o cheque ?", "question", function(success){
                var Widget = args.owner.widget;
                Widget.currentRow.IDSTCHEQUE="0";

                Widget.currentRow.CDMOTICANC="";

                Widget.currentRow.DSMOTICANC="";
                delete(Widget.getField('DSMOTICANC').validations.required);

                Widget.currentRow.DTEMISCHEQ="";
                delete(Widget.getField('DTEMISCHEQ').validations.required );

                Widget.currentRow.VRCHEQUE="";
                delete(Widget.getField('VRCHEQUE').validations.required);

                Widget.dataSource.name = "fin#/crud/cheque";
                General.saveCrud(Widget).then(function(Success) {
                    Widget.dataSource.name = "fin#/crud/cheque_motivocanc";
                    var gridCheque =  templateManager.container.getWidget('grid_cheque');
                    gridCheque.getAction('cancelarCheque').isVisible = true;
                    gridCheque.getAction('disponibilizarCheque').isVisible = false;
                    ScreenService.showMessage('Cheque Disponibilizado.');
                    gridCheque.reload();
                },function(callback){
                    ScreenService.showMessage(callback);
                    Widget.dataSource.name = "fin#/crud/cheque_motivocanc";

                });
            });

    };
    this.cancelarCheque = function(args){
        var Widget = args.owner.widget;
        Widget.currentRow.IDSTCHEQUE="2";
        var data = new Date();
        data = data.getDateBr();
        Widget.currentRow.DTEMISCHEQ=data;
        Widget.dataSource.name = "fin#/crud/cheque";
        if(Widget.isValid()){
        General.saveCrud(Widget).then(function(Success) {
            Widget.dataSource.name = "fin#/crud/cheque_motivocanc";
            ScreenService.showMessage('Cheque Cancelado.');
            var gridCheque =  templateManager.container.getWidget('grid_cheque');
            gridCheque.getAction('voltarPagina').isVisible = true;
            gridCheque.getAction('disponibilizarCheque').isVisible = true;
        },function(callback){
            Widget.dataSource.name = "fin#/crud/cheque_motivocanc";
            ScreenService.showMessage(callback.data.error);
        });
        }      
    };

    this.validacao = function(){
        var gridCheque =  templateManager.container.getWidget('grid_cheque');
        gridCheque.getField('DSMOTICANC').validations.required = true;
        gridCheque.getField('DTEMISCHEQ').validations.required = true;
        gridCheque.getField('VRCHEQUE').validations.required = true;
    };
    
    this.setFilter = function (widget){
        widget.dataSourceFilter = [
            {
                "name": "CDREDUCONT",
                "operator": "=",
                "value": General.getFieldValue('NRCHEQINIC','grid_talao', true )||null
            },
            {
                "name": "DTRECTALCHE",
                "operator": "=",
                "value": General.getFieldValue('DTRECTALCHE','grid_talao', true )||null
            },
            {
                "name": "CDTALCHEQUE",
                "operator": "=",
                "value": General.getFieldValue('CDTALCHEQUE','grid_talao', true )||null
            },
            {
                "name": "P_NRCHEQUE",
                "operator": "=",
                "value": General.getFieldValue('NRCHEQUE','grid_talao', true )||null
            }
        ];
    };
    
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco55200_TalaoCheque_Controller', Bco55200_TalaoCheque_Controller);
});

// FILE: ./assets/js/controllers/Bco_Evolucao_Fluxo_Caixa_Controller.js
var Bco_Evolucao_Fluxo_Caixa_Controller = function(ScreenService, General, templateManager){
    
    this.executaEvolucaoFluxoCaixa = function(widget) {
        if(widget.isValid()){
            General.callRestEngine('fin#/evolucaoFluxoCaixa', 'Row', widget.currentRow).then(function (result) {
                ScreenService.closePopup();
                ScreenService.showMessage('A Evolução do fluxo de caixa foi efetuada com sucesso');
            }).catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
            
        }
    };
    
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco_Evolucao_Fluxo_Caixa_Controller', Bco_Evolucao_Fluxo_Caixa_Controller);
});

// FILE: ./assets/js/controllers/Ger01600_IndiceFinanceiro_Controller.js
var Ger01600_IndiceFinanceiro_Controller = function(ZHPromise, ScreenService, General, Ger01600_IndiceFinanceiro_Service) {

	var fieldDateForm = {
		"1" : {
			
			"template": "field/calendar.html",
			"separator" : "/",
            "displayFormat" : moment()._locale.longDateFormat('L'),
            "dateFormat" : "L",
            "datetime-preset": "date"
		},
		"2" : {
			
			"template": "field/select-month.html",
            "separator": "/",
            "displayFormat": "MM/YYYY",
            "dateFormat": "MM/YYYY",
            "datetime-preset": "date"
		},
		"3" : {			
			"template": "field/datetime.html",
            "separator": "/",
            "datetime-dateFormat": "yyyy",
            "datetime-dateOrder" : "yyyy",
            "datetime-preset": "date"
		}
	};

	var fieldDateView = {
		"1" : {
			"mask": {
            	"type": "dateFormat",
            	"params": {
            		"format": "DD/MM/YYYY HH:mm:ss",
            		"toFormat": "L"
            	}
            }
		},
		"2": {
			"mask": {
            	"type": "dateFormat",
            	"params": {
            		"format": "DD/MM/YYYY HH:mm:ss",
            		"toFormat": "MM/YYYY"
            	}
            }
		},
		"3" : {
			"mask": {
            	"type": "dateFormat",
            	"params": {
            		"format": "DD/MM/YYYY HH:mm:ss",
            		"toFormat": "YYYY"
            	}
            }
		}
	};

	var valueFormat = {
		"1" : {
			"label" : "Valor do Índice (%)"
		},
		"2" : {
			"label" : "Valor do Índice"
		}
	};
	var tipoData = { 
		"1" : {
			type: "days",
			format: "L"
		}, 
		"2" : {
			type: "months",
			format: "MM/YYYY"
		},
		"3" : {
			type: "years",
			format: "YYYY"
		}
	};
	this.back = function (widget) {
		var mode = widget.mode;

		ScreenService.closeSwipe().then(function () {
			if (mode == 'view') {
				widget.activate();
			} else if (mode == 'list') {
				widget.container.widgets[0].activate();
			}
		});
	};
	this.saveIndice = function(widget){
		widget.paramsNewCode.__new_code_pk = widget.paramsNewCode.__new_code_pk+widget.currentRow.CDTPINDICE;
		General.saveCrud(widget);
	};
	this.deleteIndice = function(widget){
		ScreenService.confirmMessage(ScreenService.i18n('Deseja deletar registros selecionados?'), 'question', function (success) {
			Ger01600_IndiceFinanceiro_Service.deleteIndice(widget.currentRow).then(function(success){
				ScreenService.closeSwipe().then(function () {
					widget.activate();
					widget.reload();
				});
			}, function (error){
				ScreenService.showMessage(error.data.error);
			});
		});
	};
	this.habilitaWidgetFilha = function (widget, habilitar) {
		widget.isVisible = habilitar;
	};
	this.onEnterValoresTab = function (widgetFilha) {
		if (widgetFilha.mode == "list") {
			var widget = widgetFilha.getParent();

			var data = {
				CDTPINDICE : widget.currentRow.CDTPINDICE,
				IDPERVALOR : widget.currentRow.IDPERVALOR,
				CDINDICE : widget.currentRow.CDINDICE,
				NRORG : getLocalVar('NRORG')
			};

	 		General.buildDataSourceFilter(widgetFilha.dataSourceFilter, data);
	 		this.setDateViewMask(widgetFilha.getField('DTREFIND'), data.CDTPINDICE);
	 		this.setValueTemplate(widgetFilha.getField('VRINDICE'), data.IDPERVALOR);
	 		widgetFilha.reload();
		}
	};
	this.setDateViewMask = function(field, cdtpindice){
		for(var attr in fieldDateView[cdtpindice]) {
			delete field[attr];
			field[attr] = fieldDateView[cdtpindice][attr];
		}
		field.reload();
	};
	this.setDateFormTemplate = function (field, cdtpindice) {
		for(var attr in fieldDateForm[cdtpindice]) {
			delete field[attr];
			field[attr] = fieldDateForm[cdtpindice][attr];
		}
		field.reload();
	};
	this.setValueTemplate = function(field, idpervalor) {
		for(var attr in valueFormat[idpervalor]) {
			delete field[attr];
			field[attr] = valueFormat[idpervalor][attr];
		}
		field.reload();
	};
	this.addIndice = function(widget){
		widget.getField('CDTPINDICE').readOnly = false;
		widget.getField('IDPERVALOR').readOnly = false;
		widget.getField('SGINDICE').readOnly = false;
		this.habilitaWidgetFilha(widget.widgets[0], false);
		widget.newRow();
	};
	this.editIndice = function(widget){
		widget.getField('CDTPINDICE').readOnly = true;
		widget.getField('IDPERVALOR').readOnly = true;
		widget.getField('SGINDICE').readOnly = true;
		this.habilitaWidgetFilha(widget.widgets[0], false);
		General.edit(widget);
	};
	this.preencheValores = function(widget) {
		widget.currentRow.CDTPINDICE  = widget.dataSourceFilter[0].value;
		widget.currentRow.IDPERVALOR  = widget.dataSourceFilter[1].value;
		widget.currentRow.CDINDICE  = widget.dataSourceFilter[2].value;
	};
	this.addValue = function(widget) {
		widget.newRow();
		Ger01600_IndiceFinanceiro_Service.maxDtrefind(widget.dataSourceFilter).then(function(success){
			var maxDate = success.dataset.max_dtrefind[0].DTREFIND;
			var fieldDtrefind = widget.getField('formDTREFIND');
			fieldDtrefind.showOnForm = true;
			var cdtpindice = widget.dataSourceFilter[0].value;
			widget.getField('DTREFIND').showOnForm = false; 
			widget.currentRow.CDTPINDICE = cdtpindice;
			widget.currentRow.IDPERVALOR = widget.dataSourceFilter[1].value;
			widget.currentRow.CDINDICE = widget.dataSourceFilter[2].value;
			if(success.dataset.max_dtrefind[0].DTREFIND){
				maxDate = moment(maxDate, 'DD/MM/YYYY HH:mm:ss');
	 			widget.currentRow.formDTREFIND = maxDate.add(1, tipoData[cdtpindice].type).format(tipoData[cdtpindice].format);
				fieldDtrefind.readOnly = true;
			} else {
				fieldDtrefind.readOnly = false;
				maxDate = false;
	 			this.setDateFormTemplate(fieldDtrefind, cdtpindice);
	 			fieldDtrefind.validations.required = true;
			}
			fieldDtrefind.reload();
		}.bind(this));
	};
	this.editValue =function(widget) {
		var cdtpindice = widget.dataSourceFilter[0].value;
		var fieldDtrefind = widget.getField('formDTREFIND');
		fieldDtrefind.showOnForm = false;
		fieldDtrefind.validations = {};
		widget.getField('DTREFIND').showOnForm = true;
		this.setDateFormTemplate(fieldDtrefind, cdtpindice);
		widget.edit();
		fieldDtrefind.reload();
	};
	this.saveValue = function(widget) {
		var cdtpindice = widget.dataSourceFilter[0].value;
		if(widget.currentRow.__is_new) {
			var dataReferencia = widget.currentRow.formDTREFIND;
			dataReferencia = moment(dataReferencia,tipoData[cdtpindice].format).format('DD/MM/YYYY');
			widget.currentRow.DTREFIND = dataReferencia;
		}
		widget.currentRow.DTULTATUMVIN = moment().format('DD/MM/YYYY');
		this.preencheValores(widget);
		General.setFloatAttrOnCurrentRow(widget.currentRow, 'VRINDICE');
		General.saveCrud(widget);
	};
	this.deleteValue = function(widget){
		Ger01600_IndiceFinanceiro_Service.maxDtrefind(widget.dataSourceFilter).then(function(success){
			var maxDate = success.dataset.max_dtrefind[0].DTREFIND;
			var cdtpindice = widget.dataSourceFilter[0].value;
			maxDate = moment(maxDate, 'DD/MM/YYYY HH:mm:ss');
			var dtrefind = moment(widget.currentRow.DTREFIND, 'DD/MM/YYYY HH:mm:ss');
			if(maxDate.unix() == dtrefind.unix()) {
				widget.currentRow.DTREFIND = dtrefind.format('DD/MM/YYYY HH:mm:ss');
				General.deleteCrud(widget);
			} else {
				ScreenService.showMessage("Somente a ultima data pode ser excluída!");
			}
		}.bind(this));
	};

	this.changeOperation = function(widget) {
		var oper = widget.currentRow.OPERACAO;
		var fields = [];
		fields.push(widget.getField('DTINICIAL'));
		fields.push(widget.getField('DTFINAL'));
		fields.push(widget.getField('VRINDICE'));
		this.cleanPopup(widget);

		switch (oper) {
			case "incluir" : this.includeAuto(widget, fields); break;
			case "editar" : this.editAuto(widget, fields); break;
			case "excluir" : this.excludeAuto(widget, fields); break;
		}
	};
	this.openAutoOperation = function(widget) {
		var promises = [];
		promises.push(Ger01600_IndiceFinanceiro_Service.maxDtrefind(widget.dataSourceFilter));
		promises.push(Ger01600_IndiceFinanceiro_Service.minDtrefind(widget.dataSourceFilter));
		var widgetCadastroAuto = widget.widgets[0];
		var cdtpindice = widget.dataSourceFilter[0].value;
		var idpervalor = widget.dataSourceFilter[1].value;
		var cdindice = widget.dataSourceFilter[2].value;
		widgetCadastroAuto.isVisible = true;
		widgetCadastroAuto.currentRow.cdtpindice = cdtpindice;
		widgetCadastroAuto.currentRow.idpervalor = idpervalor;
		widgetCadastroAuto.currentRow.cdindice = cdindice;

		this.setDateFormTemplate(widgetCadastroAuto.getField('DTINICIAL'), cdtpindice);
		this.setDateFormTemplate(widgetCadastroAuto.getField('DTFINAL'), cdtpindice);
		this.setValueTemplate(widgetCadastroAuto.getField('VRINDICE'), idpervalor);

		ZHPromise.all(promises).then(function(success) {
			var maxDate = success[0].dataset.max_dtrefind[0].DTREFIND;
			var minDate = success[1].dataset.min_dtrefind[0].DTREFIND;
			widgetCadastroAuto.currentRow.maxDate = maxDate;
			widgetCadastroAuto.currentRow.minDate = minDate;
			if(!maxDate) {
				widgetCadastroAuto.currentRow.OPERACAO = "incluir";
				widgetCadastroAuto.getField('OPERACAO').readOnly = true;
				widgetCadastroAuto.getField('OPERACAO').reload();
				this.changeOperation(widgetCadastroAuto);
			}
			ScreenService.openPopup(widgetCadastroAuto);

		}.bind(this));
	};

	this.editAuto = function(widget,fields){
		widget.getField('VRINDICE').validations = {"required": true};
		widget.currentRow.OPERACAO = "editar";
		widget.currentRow.__is_new = false;
		fields[0].readOnly = false;
		fields[1].readOnly = false;
		fields[2].readOnly = false; 

		var maxDate = moment(widget.currentRow.maxDate, "DD/MM/YYYY HH:mm:ss");
		var minDate = moment(widget.currentRow.minDate, "DD/MM/YYYY HH:mm:ss");
		
		widget.currentRow.DTINICIAL = minDate.format(tipoData[widget.currentRow.cdtpindice].format);
		widget.currentRow.DTFINAL = maxDate.format(tipoData[widget.currentRow.cdtpindice].format);

		if(fields[0].template == "field/datetime.html"){
			fields[0]["datetime-maxDate"] = maxDate.format("YYYY-MM-DD");
			fields[0]["datetime-minDate"] = minDate.format("YYYY-MM-DD");
			fields[1]["datetime-maxDate"] = maxDate.format("YYYY-MM-DD");
			fields[1]["datetime-minDate"] = minDate.format("YYYY-MM-DD");
		} else {
			fields[0].maxDate = widget.currentRow.DTFINAL;
			fields[0].minDate = widget.currentRow.DTINICIAL;
			fields[1].minDate = widget.currentRow.DTINICIAL;
			fields[1].maxDate = widget.currentRow.DTFINAL;
		}
		fields[0].reload();
		fields[1].reload();
		fields[2].reload();
	};
	this.includeAuto = function(widget,fields){
		widget.getField('VRINDICE').validations = {"required": true};
		widget.currentRow.OPERACAO = "incluir";
		widget.currentRow.__is_new = true;
		if(!widget.currentRow.maxDate) {
			fields[0].readOnly = false;
		} else {
			fields[0].readOnly = true;
		}
		fields[1].readOnly = false;
		fields[2].readOnly = false;

		var maxDate = !widget.currentRow.maxDate ? false : moment(widget.currentRow.maxDate, "DD/MM/YYYY HH:mm:ss");
		widget.currentRow.DTINICIAL = maxDate ? maxDate.add(1, tipoData[widget.currentRow.cdtpindice].type).format(tipoData[widget.currentRow.cdtpindice].format): "";
		widget.currentRow.DTFINAL = widget.currentRow.DTINICIAL;	
		if(fields[0].template == "field/datetime.html"){
			fields[0]["datetime-maxDate"] = maxDate.format("YYYY-MM-DD");
			fields[0]["datetime-minDate"] = "";
			fields[1]["datetime-minDate"] = maxDate.format("YYYY-MM-DD");
			fields[1]["datetime-maxDate"] = "";
		} else {
			fields[0].maxDate = widget.currentRow.DTFINAL;
			fields[0].minDate = "";
			fields[1].maxDate = "";
			fields[1].minDate = widget.currentRow.DTFINAL;
		}
		fields[0].reload();
		fields[1].reload();
		fields[2].reload();
	};
	this.excludeAuto = function(widget,fields){
		widget.getField('VRINDICE').validations = {};
		widget.currentRow.OPERACAO = "excluir";
		widget.currentRow.__is_new = false;
		fields[0].readOnly = false;
		fields[1].readOnly = true;
		fields[2].readOnly = true;

		var maxDate = moment(widget.currentRow.maxDate, "DD/MM/YYYY HH:mm:ss");
		widget.currentRow.DTFINAL = maxDate.format(tipoData[widget.currentRow.cdtpindice].format);
		var minDate = moment(widget.currentRow.minDate, "DD/MM/YYYY HH:mm:ss");
		widget.currentRow.DTINICIAL = minDate.format(tipoData[widget.currentRow.cdtpindice].format);
		
		if(fields[0].template == "field/datetime.html"){
			fields[0]["datetime-maxDate"] = maxDate.format("YYYY-MM-DD");
			fields[0]["datetime-minDate"] = minDate.format("YYYY-MM-DD");
			fields[1]["datetime-maxDate"] = "";
			fields[1]["datetime-minDate"] = "";
		} else {
			fields[0].maxDate = widget.currentRow.DTFINAL;
			fields[0].minDate = widget.currentRow.DTINICIAL;
			fields[1].maxDate = "";
			fields[1].minDate = "";
		}
		fields[0].reload();
		fields[1].reload();
		fields[2].reload();
	};
	this.cancelOperation = function(widget){
		ScreenService.closePopup();
		delete widget.getParent().currentWidget;
		widget.getParent().activate();
		this.cleanPopup(widget);
		widget.getField('OPERACAO').readOnly=false;
		widget.isVisible = false;
	};
	this.cleanPopup = function(widget){
		widget.getField('DTINICIAL').readOnly=true;
		widget.getField('DTFINAL').readOnly=true;
		widget.getField('VRINDICE').readOnly=true;
		widget.currentRow.DTFINAL = "";
		widget.currentRow.DTINICIAL = "";
		widget.currentRow.OPERACAO = "";
		widget.currentRow.VRINDICE = "";
		widget.currentRow.__is_new = false;
	};
	this.confirmOperation = function(widget){
		var dtIni = moment(widget.currentRow.DTINICIAL, tipoData[widget.currentRow.cdtpindice].format);
		var dtFim = moment(widget.currentRow.DTFINAL, tipoData[widget.currentRow.cdtpindice].format);
		
		if(dtFim.diff(dtIni,tipoData[widget.currentRow.cdtpindice].type) < 0){
			ScreenService.showMessage("A data inicial não pode ser maior que a data final");
			return 0;
		} else if(widget.isValid()) {
			var dataSave=[];
			var differenceDate = dtFim.diff(dtIni, tipoData[widget.currentRow.cdtpindice].type);
			var oper = widget.currentRow.OPERACAO;

			if (oper !== 'editar'){
				for(;differenceDate>=0; differenceDate--) {
					General.setFloatAttrOnCurrentRow(widget.currentRow, 'VRINDICE');
					var data = {
						__is_new : widget.currentRow.__is_new,
						CDTPINDICE : widget.currentRow.cdtpindice,
						CDINDICE : widget.currentRow.cdindice,
						IDPERVALOR : widget.currentRow.idpervalor,
						DTREFIND : dtIni.format("DD/MM/YYYY HH:mm:ss"),
						VRINDICE : widget.currentRow.VRINDICE
					};
					dataSave.push(data);
					dtIni.add(1, tipoData[widget.currentRow.cdtpindice].type);
				}
			}
			
			switch (oper) {
				case "editar"  : prepareEditAndDeleteOperations(widget, dtIni, differenceDate, dataSave);this.saveOperation(widget, dataSave); break;
				case "incluir" : this.saveOperation(widget, dataSave); break;
				case "excluir" : this.excludeOperation(widget, dataSave); break;
			}
		} else {
			ScreenService.showMessage("Preencha os campos obrigatórios!</br>(*)");
		}

	};
	
	function prepareEditAndDeleteOperations (widget, dtIni, differenceDate, dataSave){
		for(;differenceDate>=0; differenceDate--) {
				var exists = filterDatesToSave(widget.getParent().dataSource.data, dtIni);
				if (!Util.isEmptyOrBlank(exists)){
					General.setFloatAttrOnCurrentRow(widget.currentRow, 'VRINDICE');
					var data = {
						__is_new : widget.currentRow.__is_new,
						CDTPINDICE : widget.currentRow.cdtpindice,
						CDINDICE : widget.currentRow.cdindice,
						IDPERVALOR : widget.currentRow.idpervalor,
						DTREFIND : dtIni.format("DD/MM/YYYY")+ "00:00:00",
						VRINDICE : widget.currentRow.VRINDICE
					};
					dataSave.push(data);
				}
				dtIni.add(1, tipoData[widget.currentRow.cdtpindice].type);
			}
	}
	
	function filterDatesToSave (data, dtIni){
		return data.filter(function(row){
			return row.DTREFIND.substr(0,10) === dtIni.format("DD/MM/YYYY");
		});
	}
	
	this.saveOperation = function(widget, dataSave){
		Ger01600_IndiceFinanceiro_Service.saveMovindice(dataSave).then(function(success){
			widget.getParent().reload().then( function(){
				this.cancelOperation(widget);
			}.bind(this));
		}.bind(this), function(error){
			ScreenService.showMessage(error.data.error);
		});
	};
	this.excludeOperation = function(widget, dataSave){
		ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question', function (success) {
			Ger01600_IndiceFinanceiro_Service.deleteMovindice(dataSave).then(function(){
				widget.getParent().reload().then( function(){
					this.cancelOperation(widget);
				}.bind(this));
			}.bind(this));
		}.bind(this));
	};
	
};
Configuration(function(ContextRegister) {
	ContextRegister.register("Ger01600_IndiceFinanceiro_Controller", Ger01600_IndiceFinanceiro_Controller);
});

// FILE: ./assets/js/controllers/Ger02700_MotivoParcelamento_Controller.js
var Ger02700_MotivoParcelamento_Controller = function(ScreenService, templateManager,ZhFilterPreferences){
	
	var self = this;
	
	/**
     * Confirmar Edição
     */
    self.confirmaEditar = function(){
        var widget = templateManager.container.getWidget('ger02700_motivo_parcelamento_grid');

        widget.getField('DSMOTIPARC').focused = true;
        delete widget.getField('CDMOTIPARC').focused;

    };
    
    /**
     * Confirmar Adição
     */
    self.confirmaAdicionar = function(){
        var widget = templateManager.container.getWidget('ger02700_motivo_parcelamento_grid');

        widget.getField('CDMOTIPARC').focused = true;
        delete widget.getField('DSMOTIPARC').focused;

    };
	
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger02700_MotivoParcelamento_Controller', Ger02700_MotivoParcelamento_Controller);
});

// FILE: ./assets/js/controllers/Ger03000_historicoIntegracao_Controller.js
var Ger03000_historicoIntegracao_Controller = function(ScreenService, Ger03000_historicoIntegracao_Service){
	this.insertInit = function(){
		Ger03000_historicoIntegracao_Service.insertInit().then(undefined, function(error){
			ScreenService.showMessage(error.data.error);
		});
	};
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03000_historicoIntegracao_Controller', Ger03000_historicoIntegracao_Controller);
});

// FILE: ./assets/js/controllers/Ger03200_ParametroFinanceiro_Controller.js
var Ger03200_ParametroFinanceiro_Controller = function(ScreenService, General, templateManager, Ger03200_ParametroFinanceiro_Service){
    
    this.consisteOrcamentoFinanceiroOnChange = function(widget) {
        if(widget.getField('IDCONSORCFIN').value() == 'S'){
            widget.getField('IDTPORCAFINA').readOnly = false;
        }else{
            widget.getField('IDTPORCAFINA').readOnly = true;
        }
    };

    this.widgetOnEnter = function(args) {
        args.owner.currentRow = args.owner.dataSource.data[0];
    };

    
    this.editWidget = function(widget) {
       General.callRestEngine('fin#/crud/geral_rec', 'FilterData', []).then(function(result){
            if(result.dataset.geral_rec.length == 0 ){
                ScreenService.showMessage('Operação bloqueada!!! Favor reveja as parametrizações na tela de Parâmetros de Títulos a Receber.');
            }else{    
                var tabbedWidgets = templateManager.container.getWidget('tabbedParametrosParaFinanceiro').widgets;

                for (var i = 0; i < tabbedWidgets.length; i++) {
                    if(tabbedWidgets[i].name !== widget.name) {
                        tabbedWidgets[i].enabled = false;
                    }
                }

                widget.template = widget.templateToForm;
                General.edit(widget);
            }
			
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            return;
        });  
    };

    this.cancelEdit = function(widget) {
        var tabbedWidgets = templateManager.container.getWidget('tabbedParametrosParaFinanceiro').widgets;

        for (var i = 0; i < tabbedWidgets.length; i++) {
            tabbedWidgets[i].enabled = true;
        }

        widget.currentRow = widget.backupRow;
        widget.view();
        widget.template = widget.templateToView;
    };

    this.filterProduto = function(widget, nameField) {
        var gridWidget = widget.widgets[0];
        var prodini = widget.currentRow.CDPRODINI;
        var prodfin = widget.currentRow.CDPRODFIN;

        if(prodini && prodfin){
            if(General.comparaPratoProduto(prodini, prodfin, getLocalVar('GER_CDPICTPROD'))){
                gridWidget.dataSourceFilter[0].value = prodini;
                gridWidget.dataSourceFilter[1].value = prodfin;
                
/*                //coloca o nivel da query no menor possivel (nulo incluso)
                var anteprodini = widget.currentRow.CDPRODNIVANTEINI;
                var anteprodfin = widget.currentRow.CDPRODNIVANTEFIN;
                if(anteprodini == null || anteprodfin == null){
                    gridWidget.dataSourceFilter[2].operator = 'IS_NULL';
                }else{
                    gridWidget.dataSourceFilter[2].operator = '=>';
                    gridWidget.dataSourceFilter[2].value = anteprodini < anteprodfin ? anteprodini : anteprodfin;
                }*/
                
                gridWidget.reload();
            } else {
                ScreenService.showMessage("O Produto Final não pode ser maior que o Produto Inicial.");
                widget.getField(nameField).setValue(null);
            }
        }
    };

    this.geraParcelaOnChange = function(widget) {
        var fieldGroup;
        for (var i = 0; i < widget.fieldGroups.length; i++) {
            if(widget.fieldGroups[i].name === 'geraParcelaSeparada'){
                fieldGroup = widget.fieldGroups[i];
                i = widget.fieldGroups.length;
            }
        }
        if(widget.currentRow.IDGERAPARCFIN === "S") {
        	widget.getField('IDGERAPARCFIN').readOnly = false;
        	widget.getField('IDGERAPARCFIN').setValue(widget.currentRow.IDGERAPARCFIN);
        	widget.getField('IDGERAPARCFIN').hideEmptyValue = true;
        	widget.getField('NRDIASEMISDOC').setValue(widget.currentRow.NRDIASEMISDOC || null);
        	widget.getField('DSTPCTRECEB').setValue(widget.currentRow.DSTPCTRECEB || null);

            widget.getField('NRDIASEMISDOC').readOnly = false;
            widget.getField('NRDIASEMISDOC').validations.required = true;

            widget.getField('DSTPCTRECEB').readOnly = false;
            widget.getField('DSTPCTRECEB').validations.required = true;

            fieldGroup.isVisible = true;
        } else if(widget.currentRow.CDIMPOSTO === "") {
        	widget.getField('IDGERAPARCFIN').readOnly = true;
        	widget.getField('IDGERAPARCFIN').hideEmptyValue = false;
        	widget.getField('IDGERAPARCFIN').setValue(null);

            fieldGroup.isVisible = false;
    	} else {
            widget.getField('IDGERAPARCFIN').readOnly = false;
        	widget.getField('IDGERAPARCFIN').setValue(widget.currentRow.IDGERAPARCFIN);
        	widget.getField('IDGERAPARCFIN').hideEmptyValue = true;
        	widget.getField('NRDIASEMISDOC').setValue(widget.currentRow.NRDIASEMISDOC || null);
        	widget.getField('DSTPCTRECEB').setValue(widget.currentRow.DSTPCTRECEB || null);

            delete widget.getField('NRDIASEMISDOC').validations.required;
            widget.getField('NRDIASEMISDOC').readOnly = true;
            widget.getField('NRDIASEMISDOC').value(null);

            delete widget.getField('DSTPCTRECEB').validations.required;
            widget.getField('DSTPCTRECEB').readOnly = true;
            widget.getField('DSTPCTRECEB').value(null);

            fieldGroup.isVisible = false;
        }
    };

    this.saveGeral = function(widget){
        Ger03200_ParametroFinanceiro_Service.save(widget.currentRow).then(function(){
        	widget.template = widget.templateToView;
        	widget.view();
            var tabbedWidgets = templateManager.container.getWidget('tabbedParametrosParaFinanceiro').widgets;
            for (var i = 0; i < tabbedWidgets.length; i++) {
                if(tabbedWidgets[i].name !== widget.name) {
                    tabbedWidgets[i].enabled = true;
                }
            }
        }, function(error){
        	ScreenService.showMessage(error);
        });
    };

    this.saveImposto = function(widget){
        if (widget.isValid()){
        	widget.currentRow.__is_new = false;
        	widget.saveRow().then((function(success){
            	widget.template = widget.templateToView;
            	widget.view();
            	var tabbedWidgets = templateManager.container.getWidget('tabbedParametrosParaFinanceiro').widgets;
            	for (var i = 0; i < tabbedWidgets.length; i++) {
                	if(tabbedWidgets[i].name !== widget.name) {
                	    tabbedWidgets[i].enabled = true;
                	}
            	}
            	widget.getField('NMIMPOSTO').reload();
        	}).bind(this), function(error){
            	var errorMessage = 'Timeout';
    			if (error.data) {
    				errorMessage = error.data.error;
    			}
    			ScreenService.showMessage(errorMessage);
    			deferred.reject(errorMessage);
        	});
        }
    };

    this.hideGeraParcelas = function(widget){
    	var tabbedWidgets = templateManager.container.getWidget('tabbedParametrosParaFinanceiro').widgets;
        for (var i = 0; i < tabbedWidgets.length; i++) {
            tabbedWidgets[i].enabled = true;
        }
        widget.view();
        widget.template = widget.templateToView;

        if(!widget.getField('NMIMPOSTO').value()){
    		widget.getField('IDGERAPARCFIN').readOnly = true;
       		widget.getField('IDGERAPARCFIN').hideEmptyValue = false;
       		widget.getField('IDGERAPARCFIN').setValue(null);
       		widget.getField('NRDIASEMISDOC').setValue(null);
       		widget.getField('DSTPCTRECEB').setValue(null);
		}
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger03200_ParametroFinanceiro_Controller', Ger03200_ParametroFinanceiro_Controller);
});

// FILE: ./assets/js/controllers/Ger06700_FormaPagamento_Controller.js
var Ger06700_FormaPagamento_Controller = function(ScreenService, templateManager, General, Ger06700_FormaPagamento_Service){

    this.getFieldFromWidget  = function(widgetName, fieldName) {
        return General.getFieldValue(fieldName,widgetName);
    };
    
    this.flagToString = function(args){
        var arrayLength = args.owner.dataSource.data.length;
        if(arrayLength > 0){
            var campo = args.owner.dataSource.data[0].NRMESFORMPGT;
            if(campo < 11){
                args.owner.dataSource.data[0].NRMESFORMPGT = campo.toString();
                //args.owner.currentRow.NRMESFORMPGT = campo.toString();
            }
        }
    };
    
    this.flagToString2 = function(args){
        var campo = args.owner.widget.dataSource.data[0].NRMESFORMPGT;
        if(campo < 11 && campo){
            args.owner.widget.dataSource.data[0].NRMESFORMPGT = campo.toString();
            //args.owner.currentRow.NRMESFORMPGT = campo.toString();
        }
    };
    
    this.validaMinMaxDia = function(args, condition){
        var valueIni = args.row.NRDIAINIPGTO;
        var valueFin = args.row.NRDIAFINPGTO;
        var valuePag = args.row.NRDIAPGTOVAR;
        var mesPgt = args.row.NRMESFORMPGT;
        var diaPgt = args.row.NRDIAPGTOVAR;
        
        if(condition == "DI"){
            if(valueIni > '31'){
                ScreenService.showMessage('Dia Início do Período deve estar entre 01 e 31.');
                args.row.NRDIAINIPGTO = '';
            }
        }else if(condition == "DF"){
            if(valueFin > '31'){
                ScreenService.showMessage('Dia Final do Período deve estar entre 01 e 31.');
                args.row.NRDIAFINPGTO = '';
            }else if((mesPgt == '0') && (valueFin > diaPgt)){
                ScreenService.showMessage('Dia de Pagamento deve ser MAIOR ou IGUAL ao Dia Final do Período.');
                args.row.NRDIAPGTOVAR = '';
            }
        }else if(condition == "DP"){
            if(valuePag > '31'){
                ScreenService.showMessage('Dia de Pagamento deve estar entre os dias 01 e 31.');
                args.row.NRDIAPGTOVAR = null;
            }else if((mesPgt == '0') && (valuePag < valueFin)){
                ScreenService.showMessage('Dia de Pagamento deve ser MAIOR ou IGUAL ao Dia Final do Período.');
                args.row.NRDIAPGTOVAR = null;
            }
        }
    };
    
    this.validaMaxSemanaMes = function(args){
        
        var tipoGrid = General.getFieldValue('IDTPPERIODO','FormaPagamento');
        
        if(tipoGrid == "A"){
            if(args.row.NRNUMMESPGTO > '04'){
                ScreenService.showMessage('Semana de pagamento deve estar entre 01 e 04.');
                args.row.NRNUMMESPGTO = '';
            }
        }else if(tipoGrid == "S"){
            if(args.row.NRMESFORMPGT > '04'){
                ScreenService.showMessage('Semana de pagamento deve estar entre 01 e 04.');
                args.row.NRMESFORMPGT = '';
            }
        }
    };

    this.validaGridAtivo = function (args){
        
        var widget = args.owner;
        
        if(widget.name == "PeriodoPagamento"){
            if(widget.dataSource.data[0]){
                this.disableEnableNewRow(widget, false);
            }else{
                this.disableEnableNewRow(widget, true);
            }
        }

        var tipoGrid = General.getFieldValue('IDTPPERIODO','FormaPagamento');
  
        switch(tipoGrid) {
            case 'M':
                this.validaFieldsMes(widget);
                widget.getField('NRNUMMESPGTO').isVisible = true;
                widget.getField('NRDIAPGTOSEMANA').isVisible = true;
                widget.getField('NRMESFORMPGT').label = 'Mês de Pagamento';
                widget.getField('NRNUMMESPGTO').label = 'Número de Meses';
                break;
            case 'S':
                this.validaFieldsSemana(widget);
                widget.getField('NRNUMMESPGTO').isVisible = false;
                widget.getField('NRDIAPGTOSEMANA').isVisible = false;
                widget.getField('NRMESFORMPGT').label = 'Semana de Pagamento';
                break;
            case 'A':
                this.validaFieldsSemanaMes(widget);
                widget.getField('NRNUMMESPGTO').isVisible = true;
                widget.getField('NRDIAPGTOSEMANA').isVisible = false;
                widget.getField('NRMESFORMPGT').label = 'Mês de Pagamento';
                widget.getField('NRNUMMESPGTO').label = 'Semana de Pagamento';
                break;
            default:
        }
    };
    
    this.disableEnableNewRow = function(widget, flag){
        var newWidget = widget.container.getWidget('PeriodoPagamento');
        newWidget.getAction('add').isVisible = flag;
    };
    
    this.getSemana = function(){
        var diaSemana = [
            {
                "value": "01",
                "description": "Domingo"
            },
            {
                "value": "02",
                "description": "Segunda"
            },
            {
                "value": "03",
                "description": "Terça"
            },
            {
                "value": "04",
                "description": "Quarta"
            },
            {
                "value": "05",
                "description": "Quinta"
            },
            {
                "value": "06",
                "description": "Sexta"
            },
            {
                "value": "07",
                "description": "Sábado"
            }
        ];
        
        return diaSemana;
    };
    
    this.getMesPosterior = function(){
        var mesPosterior = [
            {
                "value": "1",
                "description": "1º Mês Posterior"
            },
            {
                "value": "2",
                "description": "2º Mês Posterior"
            },
            {
                "value": "3",
                "description": "3º Mês Posterior"
            },
            {
                "value": "4",
                "description": "4º Mês Posterior"
            },
            {
                "value": "5",
                "description": "5º Mês Posterior"
            },
            {
                "value": "6",
                "description": "6º Mês Posterior"
            },
            {
                "value": "7",
                "description": "7º Mês Posterior"
            },
            {
                "value": "8",
                "description": "8º Mês Posterior"
            },
            {
                "value": "9",
                "description": "9º Mês Posterior"
            },
            {
                "value": "10",
                "description": "10º Mês Posterior"
            }
        ];  
        
        return mesPosterior;
    };
    
    this.validaFieldsSemana = function(widget){
        widget.getField('NRDIAINIPGTO').template = 'field/select-native.html';
        widget.getField('NRDIAINIPGTO').dataSource.data = this.getSemana();
        widget.getField('NRDIAFINPGTO').template = 'field/select-native.html';
        widget.getField('NRDIAFINPGTO').dataSource.data = this.getSemana();
        widget.getField('NRMESFORMPGT').dataSource.data = [
            {
                "value": "0",
                "description": "Atual"
            },
            {
                "value": "1",
                "description": "Seguinte"
            }
        ];
        widget.getField('NRDIAPGTOVAR').template = 'field/select-native.html';
        widget.getField('NRDIAPGTOVAR').dataSource.data = this.getSemana();
    };
    
    this.validaFieldsSemanaMes = function(widget){
        widget.getField('NRDIAINIPGTO').template = 'field/text-edit.html';
        widget.getField('NRDIAINIPGTO').dataSource.data = [];
        widget.getField('NRDIAFINPGTO').template = 'field/text-edit.html';
        widget.getField('NRDIAFINPGTO').dataSource.data = [];
        widget.getField('NRMESFORMPGT').dataSource.data = this.getMesPosterior();
        widget.getField('NRDIAPGTOVAR').dataSource.data = this.getSemana();
    };

    this.validaFieldsMes = function(widget){

        widget.getField('NRDIAINIPGTO').template = 'field/text-edit.html';
        widget.getField('NRDIAINIPGTO').dataSource.data = [];
        widget.getField('NRDIAFINPGTO').template = 'field/text-edit.html';
        widget.getField('NRDIAFINPGTO').dataSource.data = [];
        widget.getField('NRMESFORMPGT').dataSource.data = [
            {
                "value": "0",
                "description": "Atual"
            },
            {
                "value": "1",
                "description": "Seguinte"
            }
        ];
        widget.getField('NRDIAPGTOVAR').template = 'field/text-edit.html';
        widget.getField('NRDIAPGTOVAR').dataSource.data = [];
        widget.getField('NRDIAPGTOSEMANA').template = 'field/select-native.html';
        widget.getField('NRDIAPGTOSEMANA').dataSource.data = this.getSemana();
        
    };
    
    this.ativaTab = function(widget){
        //this.disableEnableNewRow(widget, true);
        //if(widget.name == 'FormaPagamento'){
          //  widget.container.getWidget('PeriodoPagamento').activate();
        //}
    };
    
    this.salvarPeriodoPgto = function(args){
        var widget = args.owner.widget;
        if(!(widget.currentRow.__is_new)){
            var campo = widget.dataSource.data[0].NRMESFORMPGT;
            widget.currentRow.NRMESFORMPGT = parseInt(campo, 10);
        }
        General.widgetSave(widget).then(function(){
            widget = args.owner.widget;
            if (widget.currentRow.__is_new) {
                widget.currentRow.__is_new = false;
            }        
            var widget = templateManager.container.getWidget('FormaPagamento');
            args.owner.widget.reload().then(function(){
                this.disableEnableNewRow(widget, false);
                this.flagToString2(args);
            }.bind(this));
        }.bind(this), function (error) {
            ScreenService.showMessage(error);
        });
    };
    
    this.confirmaExclusaoPeriodo = function(args){
        
        var cdformpgto = args.row.CDFORMPGTO;
        var nrorg = getLocalVar('NRORG');
        
        this.getPeriodo(cdformpgto, nrorg).then(function(diaPgto){
            if(diaPgto.dataset.diapgto[0]){
                ScreenService.confirmMessage('Período de Pagamento será apagado. Deseja continuar?', 'question', function() {
        
                    var row = {
                        "CDFORMPGTO": diaPgto.dataset.diapgto[0].CDFORMPGTO,
                        "NRDIAINIPGTO": diaPgto.dataset.diapgto[0].NRDIAINIPGTO,
                        "NRORG": nrorg
                    };
                    
                    this.excluiPeriodo(row).then(function(deletePeriodo){
                        //console.log('deleta lançamento Contabil');
                    });
        
                }.bind(this), function (error) {
                    ScreenService.showMessage(error);
                });
            }
        }.bind(this), function (error) {
            ScreenService.showMessage(error);
        });
    };
    
    this.deletaRegistro = function(args){
        General.deleteCrud(args.owner.widget).then(function (success){
            args.owner.widget.reload();
        });
    };
    
    this.getPeriodo = function(cdformpgto, nrdiainipgto, nrorg){
        return Ger06700_FormaPagamento_Service.getPeriodo(cdformpgto, nrdiainipgto, nrorg);
    };
    
    this.excluiPeriodo = function(row){
        return Ger06700_FormaPagamento_Service.excluiPeriodo(row);
    };
    
    this.setPeriodoPagamentoDataSourceFilter = function(){
        var widget = templateManager.container.getWidget('PeriodoPagamento');
        widget.dataSourceFilter = [
            {
                "name": "CDFORMPGTO",
                "operator": "=",
                "value": this.getFieldFromWidget('FormaPagamento', 'CDFORMPGTO')
            }
        ];
    };
    this.salvarFormaPgto = function(args){
        var widget = args.owner.widget;
        General.widgetSave(widget).then(function(){
            args.owner.widget.reload();
        }.bind(this), function (error) {
            ScreenService.showMessage(error);
        });
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger06700_FormaPagamento_Controller', Ger06700_FormaPagamento_Controller);
});

// FILE: ./assets/js/controllers/Ger08200_ClassificacaoFinanceira_Controller.js
var Ger08200_ClassificacaoFinanceira_Controller = function(General ,ScreenService, templateManager,ZhFilterPreferences){
    
    var self = this;
    
    /**
     * Confirmar Edição
     */
    self.confirmaEditar = function(){
        var widget = templateManager.container.getWidget('ger08200_classificacao_financeira_grid');

        widget.getField('DSCLASSFINA').focused = true;
        delete widget.getField('CDCLASSFINA').focused;

    };
    
    /**
     * Confirmar Adição
     */
    self.confirmaAdicionar = function(){
        var widget = templateManager.container.getWidget('ger08200_classificacao_financeira_grid');

        widget.getField('CDCLASSFINA').focused = true;
        delete widget.getField('DSCLASSFINA').focused;

    };

};


Configuration(function (ContextRegister){
    ContextRegister.register('Ger08200_ClassificacaoFinanceira_Controller', Ger08200_ClassificacaoFinanceira_Controller);
});

// FILE: ./assets/js/controllers/Ger08300_banco_Controller.js
var Ger08300_banco_Controller = function(Ger08300_banco_Service,General ,ScreenService, templateManager,ZhFilterPreferences){

    this.verificaCaracterIvalidoConta = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTCONT').value();
        var contNumero=0;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)==9||mascara.charAt(i)=="-")
                contNumero++;
        }

        return contNumero<mascara.length;
    };
    
    this.verificaMascaraContaTraco = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTCONT').value();
        var cont_traco=0;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)=="-")
                cont_traco++;
        }

        return cont_traco>1;
    };
    
    this.verificaMascaraAgenciaTraco = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTAGEN').value();
        var cont_traco=0;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)=="-")
                cont_traco++;
        }

        return cont_traco>1;
    };
    
    this.verificaMascaraAgenciaNumero9 = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTAGEN').value();
        var contNumero=0;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)==9)
                contNumero++;
        }
        return contNumero>5;
    };
    
    this.verificaCaracterIvalido = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTAGEN').value();
        var contNumero=0;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)==9||mascara.charAt(i)=="-")
                contNumero++;
        }
        return contNumero<mascara.length;
    };
    
    this.verificaPosicaoTraco = function(){
        var mascara = templateManager.container.getWidget('grid_banco').getField('CDPICTAGEN').value();
        var traco;
        var length = mascara.length;
        for(var i = 0;i<mascara.length;i++){
            if(mascara.charAt(i)=="-")
                traco=true;
        }
        if(traco===true){
            if(mascara.charAt(length-2)=="-"){
                return false;
            }else  if(mascara.charAt(length-3)=="-"){
                return false;
            }else {
                return true;
            }
        }
        else{
            return false;
        }
    };
    
    this.habilitarWidget = function(){
        templateManager.container.getWidget('widget_cnab').enabled = true;
        templateManager.container.getWidget('grid_cobranca').enabled = true;
        templateManager.container.getWidget('FORMDDA').enabled = true;
    };
    
    this.desabilitarWidget = function(){
        templateManager.container.getWidget('widget_cnab').enabled = false;
        templateManager.container.getWidget('grid_cobranca').enabled = false;
        templateManager.container.getWidget('FORMDDA').enabled = false;
    };
    
    this.setFiltroCnab = function(widget){
        if (!widget.isNewRow()){
            var current = widget.currentRow;
            var dadosEmpresa = templateManager.container.getWidget('dadosEmpresa');
            var ocorrenciaRetornoPagar = templateManager.container.getWidget('ocorrenciaRetornoPagar');
            var ocorrenciaRetornoReceber = templateManager.container.getWidget('ocorrenciaRetornoReceber');
            var instrucao = templateManager.container.getWidget('grid_cobranca');
            var filtro = current.CDBANCO;
            //dadosEmpresa.dataSourceFilter[0].value = filtro;
            //ocorrenciaRetornoPagar.dataSourceFilter[0].value = filtro;
            //ocorrenciaRetornoReceber.dataSourceFilter[0].value = filtro;
            templateManager.container.getWidget('dadosEmpresa').getField('CDINS1COBR').dataSourceFilter[0].value=filtro;
            templateManager.container.getWidget('dadosEmpresa').getField('CDINS2COBR').dataSourceFilter[0].value=filtro;
            templateManager.container.getWidget('dadosEmpresa').getField('CDINS1COBR').reload();
            templateManager.container.getWidget('dadosEmpresa').getField('CDINS2COBR').reload();
            var gridcontacorrente =templateManager.container.getWidget('grid_banco');
            cnab =current.CDBANCCNAB;
            if(cnab==341){
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').isVisible=true;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showOnForm=true;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showOnView=true;

                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnView=false;

                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnView=false;

            }else{
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showOnView=false;


                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnView=false;

                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnView=false;


            }if(cnab==399){
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').isVisible=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showOnForm=false;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYEXTBANCO').showView=false;

                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').isVisible=true;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnForm=true;
                templateManager.container.getWidget('dadosEmpresa').getField('IDLAYOUTCNAB').showOnView=true;

                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnForm=true;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').isVisible=true;
                templateManager.container.getWidget('dadosEmpresa').getField('NMRAZSOCFORN').showOnView=true;
            }
        }
    };
    
    this.excluir = function(widget){
        General.deleteCrud(widget);
    };
    
    this.salvar = function(args){
        var Widget = args.owner.widget;
        if(this.verificaMascaraAgenciaNumero9()){
            ScreenService.showMessage('Máscara Agência inválida. Nº de caracteres válidos (9) deve ser menor ou igual a 5.');
        }
        else if(this.verificaMascaraAgenciaTraco()){
            ScreenService.showMessage('Máscara Agência inválida. Nº de caracteres válidos (-) deve ser menor ou igual a 1.');
        }
        else if(this.verificaCaracterIvalido()){
            ScreenService.showMessage('Máscara Agência inválida.Caracteres Inválidos');
        }
        else if(this.verificaPosicaoTraco()){
            ScreenService.showMessage('Máscara Agência inválida.Quantidade de dígitos de controle deve estar entre 1 e 2');
        }
        else if(this.verificaMascaraContaTraco()){
            ScreenService.showMessage('Máscara C/C inválida. Nº de caracteres válidos (-) deve ser menor ou igual a 1.');
        }
        else if(this.verificaCaracterIvalidoConta()){
            ScreenService.showMessage('Máscara  C/C inválida.Caracteres Inválidos');
        }

        else if(Widget.isValid()){
            General.saveCrud(Widget).then(function(Success) {
                ScreenService.showMessage('Salvo com sucesso!');
                var currentRow = Widget.currentRow;
                templateManager.container.getWidget('widget_cnab').enabled = true;
                templateManager.container.getWidget('grid_cobranca').enabled = true;
                templateManager.container.getWidget('FORMDDA').enabled = true;
            }, function(callback){
                if(callback.data){
                    ScreenService.showMessage(callback.data.error);
                }
            });
        }
    };
    
    this.salvarCNAB = function(args){
        var widget = args.owner.widget;
        templateManager.container.getWidget('dadosEmpresa').currentRow.CDBANCO=templateManager.container.getWidget('grid_banco').currentRow.CDBANCO;
        if(!widget.currentRow.IDLAYOUTCNAB){
            widget.currentRow.IDLAYOUTCNAB='240';
        }
        if(!widget.currentRow.IDLAYEXTBANCO){
            widget.currentRow.IDLAYEXTBANCO='200';
        }
        if(widget.isValid()){
            if(this.checkInstrucao(widget) == true){
                Ger08300_banco_Service.salvar(widget.currentRow).then(function() {
                    if (widget.currentRow.__is_new) {
                        widget.list(true);
                        ScreenService.goBack();
                    } else {
                        widget.view();
                    }
                    widget.activate();
                }, function(callback){
                        if(callback.data){
                            ScreenService.showMessage(callback.data.error);
                        }
                });
            }
        }
    };
    
    this.checkInstrucao =  function(widget){
        var widgetInst1 = widget.currentRow.CDINS1COBR;
        var widgetInst2 = widget.currentRow.CDINS2COBR;
        
        if(widgetInst2 == widgetInst1){
            ScreenService.showMessage('Erro. Instrução 1 igual à Instrução 2');
            return false;
        }
        return true;
    };
    
    this.salvarInstrucao = function(args){
        var Widget = args.owner.widget;
        templateManager.container.getWidget('grid_cobranca').currentRow.CDBANCO=templateManager.container.getWidget('grid_banco').currentRow.CDBANCO;
        General.saveCrud(Widget).then(function() {
            ScreenService.showMessage('Salvo com sucesso!');
        }, function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };
    
    this.cancelar = function(args){
        General.backOrCancel(args);
        this.habilitarWidget();
    };
    
    this.openFilterActionEvent = function() {
        var gridBanco = templateManager.container.getWidget('grid_banco');
        ZhFilterPreferences.openFilterWidget(gridBanco);
    };
    
    this.openFilterPopupWidgetOnEnter = function() {
        this.openFilterActionEvent();
    };
    
    this.setFieldsVisibleOnCNAB = function(widget){
        switch (widget.getParent().getParent().currentRow.CDBANCO) {
            case "341":
                widget.getField('IDPADRCNABPAG').isVisible = true;
                widget.getField('IDPADRCNABPAG').dataSource.data[0].name = 'SISPAG';
                widget.getField('IDPADRCNABPAG').dataSource.data[1].name = 'RISCO SACADO';
                widget.getField('IDPADRAGENCNABPAG').isVisible = false;
                widget.getField('IDLAYEXTBANCO').isVisible = true;
                widget.getField('IDLAYOUTCNAB').isVisible = false;
                widget.getField('IDPADRCNABREC').isVisible = false;
                break;
            case "237":
                widget.getField('IDPADRCNABPAG').isVisible = true;
                widget.getField('IDPADRCNABPAG').dataSource.data[0].name = 'PAGFORN';
                widget.getField('IDPADRCNABPAG').dataSource.data[1].name = 'MULTIPAG';
                widget.getField('IDPADRAGENCNABPAG').isVisible = true;
                widget.getField('IDLAYEXTBANCO').isVisible = false;
                widget.getField('IDLAYOUTCNAB').isVisible = false;
                widget.getField('IDPADRCNABREC').isVisible = false;
                break;
            case "399":
                widget.getField('IDPADRCNABPAG').isVisible = false;
                widget.getField('IDPADRAGENCNABPAG').isVisible = false;
                widget.getField('IDLAYEXTBANCO').isVisible = false;
                widget.getField('IDLAYOUTCNAB').isVisible = true;
                widget.getField('IDPADRCNABREC').isVisible = false;
                break;
            case "104":
                widget.getField('IDPADRCNABPAG').isVisible = false;
                widget.getField('IDPADRAGENCNABPAG').isVisible = false;
                widget.getField('IDLAYEXTBANCO').isVisible = false;
                widget.getField('IDLAYOUTCNAB').isVisible = false;
                widget.getField('IDPADRCNABREC').isVisible = true;
                break;
            default:
                widget.getField('IDPADRCNABPAG').isVisible = false;
                widget.getField('IDPADRAGENCNABPAG').isVisible = false;
                widget.getField('IDLAYEXTBANCO').isVisible = false;
                widget.getField('IDLAYOUTCNAB').isVisible = false;
                widget.getField('IDPADRCNABREC').isVisible = false;
                break;
            
        }
    };
    
    this.cancelaCadastroAutomaticoDDA = function (args){
        General.backOrCancel(args).then(function (){
            var widget = args.owner.widget;
            
            General.formWidgetSwitchTemplate(widget);
			General.changeWidgetAcessibility(widget.getParent().widgets.concat(widget.getParent()), true);
        });
    };
    
    this.cadastraDDAAutomatico = function (widget){
        General.saveCrud(widget).then(function (){
            General.formWidgetSwitchTemplate(widget);
			General.changeWidgetAcessibility(widget.getParent().widgets.concat(widget.getParent()), true);
        });
    };
    
    this.checkInstrucao = function(widget){
        var widgetInst1 = widget.currentRow.CDINS1COBR;
        var widgetInst2 = widget.currentRow.CDINS2COBR;
        if(widgetInst2 == widgetInst1){
            ScreenService.showMessage('Campo inválido. Instrução 1 = instrução 2');
            return false;
        }else{
            return true;
        }
    }; 
};


Configuration(function (ContextRegister){
    ContextRegister.register('Ger08300_banco_Controller', Ger08300_banco_Controller);
});

// FILE: ./assets/js/controllers/Ger08400_Agencia_Controller.js
var Ger08400_Agencia_Controller = function(ScreenService, templateManager,General,Endereco){

    this.fieldEdicao = function(){
        templateManager.container.getWidget('contato').enabled = false;
        var grid_agencia =  templateManager.container.getWidget('grid_agencia');
        var pais = templateManager.container.getWidget('grid_agencia').getField('NMPAIS').value();
        var estado = templateManager.container.getWidget('grid_agencia').getField('NMESTADO').value();
        var municipio = templateManager.container.getWidget('grid_agencia').getField('NMMUNICIPIO').value();
        if(!pais){
            grid_agencia.getField('NMESTADO').readOnly = true;
            grid_agencia.getField('NMMUNICIPIO').readOnly = true;
            grid_agencia.getField('NMBAIRRO').readOnly = true;
        }
        else{
            if(!estado){
                grid_agencia.getField('NMESTADO').readOnly = false;
                grid_agencia.getField('NMMUNICIPIO').readOnly = true;
                grid_agencia.getField('NMBAIRRO').readOnly = true;
            }
            else{
                grid_agencia.getField('NMESTADO').readOnly = false;
                if(!municipio){
                    grid_agencia.getField('NMMUNICIPIO').readOnly = false;
                    grid_agencia.getField('NMBAIRRO').readOnly = true;

                }
                else{
                grid_agencia.getField('NMMUNICIPIO').readOnly = false;
                grid_agencia.getField('NMBAIRRO').readOnly = false;
                }
            }
        }
    };
    this.setAgencia = function(widget,args){
        var current= widget.currentRow;
        var gridWidget =  templateManager.container.getWidget('grid_agencia');
        gridWidget.getField('CDAGENCIA').mask.params = {'mask': current.CDPICTAGEN};
        //widget.currentRow.CDPICTAGEN ="";
    };

    this.setEstado = function(){
        var widget = templateManager.container.getActiveWidget();
        var cdpais = widget.getField('CDPAIS').value();
        widget.getField('NMESTADO').dataSourceFilter[0].value = cdpais;
        widget.getField('NMESTADO').readOnly= false;
        widget.getField('NMESTADO').reload();
    };

    this.setMunicipio = function(){
        var widget = templateManager.container.getActiveWidget();
        var cdpais = widget.getField('CDPAIS').value();
        var cdestado = widget.getField('SGESTADO').value();
        widget.getField('NMMUNICIPIO').dataSourceFilter[0].value = cdpais;
        widget.getField('NMMUNICIPIO').dataSourceFilter[1].value = cdestado;
        widget.getField('NMMUNICIPIO').readOnly= false;
        widget.getField('NMMUNICIPIO').reload();
    };

    this.setBairro = function(){
        var widget = templateManager.container.getActiveWidget();
        var cdestado = widget.getField('SGESTADO').value();
        var cdmunicipio = widget.getField('CDMUNICIPIO').value();
        var cdpais = widget.getField('CDPAIS').value();
        widget.getField('NMBAIRRO').dataSourceFilter[0].value = cdpais;
        widget.getField('NMBAIRRO').dataSourceFilter[1].value = cdestado;
        widget.getField('NMBAIRRO').dataSourceFilter[2].value = cdmunicipio;
        widget.getField('NMBAIRRO').readOnly= false;
        widget.getField('NMBAIRRO').reload();

    };

    this.limparValorPais = function(){
        var widget = templateManager.container.getActiveWidget();
        var pais = widget.getField('NMPAIS');
        if(pais.value()){
            widget.getField('NMESTADO').value("");
            widget.getField('NMMUNICIPIO').value("");
            widget.getField('NMBAIRRO').value("");
            widget.getField('SGESTADO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('CDMUNICIPIO').value("");
            widget.getField('NMESTADO').readOnly= false;
            widget.getField('NMMUNICIPIO').readOnly= true;
            widget.getField('NMBAIRRO').readOnly= true;
            widget.getField('NMESTADO').reload();
        }else{
            widget.getField('NMESTADO').value("");
            widget.getField('NMMUNICIPIO').value("");
            widget.getField('NMBAIRRO').value("");
            widget.getField('SGESTADO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('CDMUNICIPIO').value("");
            widget.getField('NMESTADO').readOnly= true;
            widget.getField('NMMUNICIPIO').readOnly= true;
            widget.getField('NMBAIRRO').readOnly= true;
            widget.getField('NMESTADO').reload();
        }
    };

    this.limparValorEstado= function(){
        var widget = templateManager.container.getActiveWidget();
        var estado = widget.getField('NMESTADO');
        if(estado.value()){
            widget.getField('NMMUNICIPIO').value("");
            widget.getField('NMBAIRRO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('CDMUNICIPIO').value("");
            widget.getField('NMMUNICIPIO').readOnly= false;
            widget.getField('NMBAIRRO').readOnly= true;
            widget.getField('NMMUNICIPIO').reload();
        }else{
            widget.getField('NMMUNICIPIO').value("");
            widget.getField('NMBAIRRO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('CDMUNICIPIO').value("");
            widget.getField('NMMUNICIPIO').readOnly= true;
            widget.getField('NMBAIRRO').readOnly= true;
            widget.getField('NMMUNICIPIO').reload();
        }
    };

    this.limparValorMunicipio = function(){
        var widget = templateManager.container.getActiveWidget();
        var municipio = widget.getField('NMMUNICIPIO');
        if(municipio.value()){
            widget.getField('NMBAIRRO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('NMBAIRRO').readOnly= false;
            widget.getField('NMBAIRRO').reload();
        }else{
            widget.getField('NMBAIRRO').value("");
            widget.getField('CDBAIRRO').value("");
            widget.getField('NMBAIRRO').readOnly= true;
            widget.getField('NMBAIRRO').reload();
        }
    };

    this.Adicionar = function(args){
        var widget = args.owner.widget;
        var filtro = widget.appliedFilter;
        templateManager.container.getWidget('contato').enabled = false;
        widget.newRow();
        widget.getField('CDAGENCIA').readOnly = false;
        widget.getField('NMESTADO').readOnly= true;
        widget.getField('NMMUNICIPIO').readOnly= true;
        widget.getField('NMBAIRRO').readOnly= true;
        if(!filtro.currentRow.NMBANCO){
            //args.owner.widget.getField('CDCONTCORR').mask.params = {'mask':''};
            args.owner.widget.getField('CDAGENCIA').readOnly = true;
            if(widget.dataSource.data){
                args.owner.widget.getField('CDAGENCIA').mask.params = {'mask':  widget.dataSource.data[0].CDPICTAGEN};
            }
        }
        else{
            //args.owner.widget.currentRow.CDBANCO= filtro.currentRow.CDBANCO;
            args.owner.widget.getField('CDAGENCIA').mask.params = {'mask':filtro.currentRow.CDPICTAGEN};
        }
    };

    this.excluirAgencia = function(widget){
        General.deleteCrud(widget);
    };

    this.salvarAgencia = function(args){
        var grid_agencia =  templateManager.container.getWidget('grid_agencia');
        var filterPopup = grid_agencia.appliedFilter;
        var widget = args.owner.widget;
        var cnpj = grid_agencia.currentRow.NRCGCAGEN;
        var agencia = grid_agencia.getField('CDAGENCIA').value();
        var telefone = grid_agencia.getField('NRFONEAGEN').value();
        var fax = grid_agencia.getField('NRFAXAGEN').value();

        grid_agencia.getField('CDAGENCIA').value(agencia.replace("-",""));
        if(telefone){
            grid_agencia.currentRow.NRFONEAGEN =grid_agencia.currentRow.NRFONEAGEN.replace(/[^0-9]/g,'');
        }
        if(fax){
            grid_agencia.currentRow.NRFAXAGEN =grid_agencia.currentRow.NRFAXAGEN.replace(/[^0-9]/g,'');
        }
        if(widget.currentRow.NRCEPAGEN){
            grid_agencia.currentRow.NRCEPAGEN =grid_agencia.currentRow.NRCEPAGEN.replace(/[^0-9]/g,'');
        }
        if(cnpj){
            grid_agencia.currentRow.NRCGCAGEN =grid_agencia.currentRow.NRCGCAGEN.replace(/[^0-9]/g,'');
        }
        General.saveCrud(widget).then(function(Success) {
            if(widget.currentRow.__is_new ===true && widget.currentRow.CDBANCO!==filterPopup.currentRow.CDBANCO){
                widget.reload();
            }
            templateManager.container.getWidget('contato').enabled = true;
        }, function(callback){
            if(callback.data){
                ScreenService.showMessage(callback.data.error);
            }
        });
    };

    this.salvarContato = function(args){
        var grid_agencia =  templateManager.container.getWidget('grid_agencia');
        var widget =args.owner.widget;
        if(widget.currentRow.__is_new){
            widget.currentRow.CDBANCO = grid_agencia.currentRow.CDBANCO;
            widget.currentRow.CDAGENCIA = grid_agencia.currentRow.CDAGENCIA;
        }
        if(widget.currentRow.NRFONECOAGEN){
            widget.currentRow.NRFONECOAGEN =widget.currentRow.NRFONECOAGEN.replace(/[^0-9]/g,'');
        }
        General.saveCrud(widget);
    };

    this.testGridFilterBeforeApply = function(args) {
        var grid_agencia =  templateManager.container.getWidget('grid_agencia');
        var filterPopup = grid_agencia.appliedFilter;
        if(filterPopup.isValid()){
            this.setAgencia(filterPopup,args);
        }
    };

    this.validarCepEnd = function(args) {

        var widget = args.owner.getParent().widget;
        var CEP = widget.currentRow.NRCEPAGEN;
        if(CEP){
            Endereco.buscarCep(CEP).then(
                function(response) {
                    var endereco = response.dataset.ENDERECO;
                    if(endereco) {
                        widget.currentRow.NRCEPAGEN = CEP;
                        widget.currentRow.CDPAIS = (endereco.CDPAIS)? endereco.CDPAIS : '';
                        widget.currentRow.NMPAIS = (endereco.NMPAIS)? endereco.NMPAIS : '';
                        widget.currentRow.SGESTADO = (endereco.SGESTADO)? endereco.SGESTADO : '';
                        widget.currentRow.NMESTADO = (endereco.NMESTADO)? endereco.NMESTADO : '';
                        widget.currentRow.CDMUNICIPIO = (endereco.CDMUNICIPIO)? endereco.CDMUNICIPIO : '';
                        widget.currentRow.NMMUNICIPIO = (endereco.NMMUNICIPIO)? endereco.NMMUNICIPIO : '';
                        widget.currentRow.CDBAIRRO = (endereco.CDBAIRRO)? endereco.CDBAIRRO : '';
                        widget.currentRow.NMBAIRRO = (endereco.NMBAIRRO)? endereco.NMBAIRRO : '';
                        widget.currentRow.DSENDEAGEN = (endereco.DSENDERECO)? endereco.DSENDERECO : '';
                        this.verificaFieldOnChangePais(endereco);
                    }
                    if(response.dataset.ERRO) ScreenService.showMessage(response.dataset.ERRO.MESSAGE, "warning");
                }.bind(this)
            );
        }
    };

    this.verificaFieldOnChangePais = function(endereco){
        var widget =templateManager.container.getWidget('grid_agencia');
        if(!endereco){
            widget.getField('NMPAIS').value('');
            widget.getField('CDPAIS').value('');


            widget.getField('NMESTADO').readOnly = true;
            widget.getField('NMESTADO').value('');
            widget.getField('SGESTADO').value('');


            widget.getField('NMMUNICIPIO').readOnly = true;
            widget.getField('NMMUNICIPIO').value('');
            widget.getField('CDMUNICIPIO').value('');

            widget.getField('NMBAIRRO').readOnly = true;
            widget.getField('NMBAIRRO').value('');
            widget.getField('CDBAIRRO').value('');
        }
        else{
            var pais = endereco.NMPAIS;
            var estado = endereco.NMESTADO;
            var municipio = endereco.NMMUNICIPIO;
            if(!pais){
            widget.getField('NMESTADO').readOnly = true;
            widget.getField('NMMUNICIPIO').readOnly = true;
            widget.getField('NMBAIRRO').readOnly = true;
            }
            else{
                if(!estado){
                    widget.getField('NMESTADO').readOnly = false;
                    widget.getField('NMMUNICIPIO').readOnly = true;
                    widget.getField('NMBAIRRO').readOnly = true;
                }
                else{
                    widget.getField('NMESTADO').readOnly = false;
                    if(!municipio){
                        widget.getField('NMMUNICIPIO').readOnly = false;
                        widget.getField('NMBAIRRO').readOnly = true;

                    }
                    else{
                    widget.getField('NMMUNICIPIO').readOnly = false;
                    widget.getField('NMBAIRRO').readOnly = false;
                    }
                }
            }
        }
    };

    this.verificaMacaraAgencia = function(args){
        var widget = args.owner.owner.widget;
        var field;
        if(widget.name=='filterPopup'){
            field = widget.getField('NMAGENCIA');
        }
        else{
            field = widget.getField('CDAGENCIA');
        }
        widget.currentRow.CDAGENCIA ='';
        if(!widget.currentRow.NMBANCO){
            field.readOnly=true;
            field.value("");
        }else{
            field.readOnly=false;
            field.value("");
            if(widget.name =='grid_agencia'){
                field.element.mask(widget.currentRow.CDPICTAGEN);
            }



        }
    };

    this.aplicarMascaraAgenciaFiltro = function(args){
        var widget = args.owner.owner.widget;
        var agencia = widget.getField('NMAGENCIA');
        agencia.fields[0].mask.params.mask = widget.currentRow.CDPICTAGEN;
        agencia.reload();
    };

    this.checkCPNJ = function(field) {
        var valid = General.validateCNPJ(field.value());
        if(!valid){
            ScreenService.showMessage("CPNJ : '"+field.value()+"' inválido.");
            field.value('');
        }
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Ger08400_Agencia_Controller', Ger08400_Agencia_Controller);
});




// FILE: ./assets/js/controllers/Ger08500_ContaCorrente_Controller.js
var Ger08500_ContaCorrente_Controller = function(Ger08500_ContaCorrente_Service, ScreenService, templateManager, General, ZHPromise){
	this.CAIXA_ECONOMICA_FEDERAL = '104';

	this.validacoesBanco = function(args){
		var widget = templateManager.container.getActiveWidget();
		this.setAgencia(args);
		this.verificaMacaraCC(args);
		var contaCorrente = templateManager.container.getWidget('grid_contacorrente');
		this.verificaValidadorCnab(contaCorrente);
		contaCorrente.getField('CDOPERCONTCORR').isVisible = (widget.currentRow.CDBANCO == this.CAIXA_ECONOMICA_FEDERAL) ? true : false;
		//this.filterAgencia(args);
		this.setDataSourceAgencia(widget);
	};

	this.verificaValidadorCnab = function(widget){
		var validadoresCnabImplementados = [this.CAIXA_ECONOMICA_FEDERAL];
		if (validadoresCnabImplementados.indexOf(widget.currentRow.CDBANCO) >= 0){
			widget.getField('IDFASCNABREC').isVisible = true;
			widget.getField('IDFASCNABPAG').isVisible = true;
		}else{
			widget.getField('IDFASCNABREC').isVisible = false;
			widget.getField('IDFASCNABPAG').isVisible = false;
		}
	};

	this.adicionar = function(args){
		var widget = args.owner.widget;
		widget.newRow();
		templateManager.container.getWidget('grid_contas').isVisible = false;
		templateManager.container.getWidget('grid_limitecredito').isVisible = false;
		templateManager.container.getWidget('implantacaoSaldo').isVisible = false;
		templateManager.container.getWidget('implantacaoSaldoForm').isVisible = true;
		templateManager.container.getWidget('implantacaoSaldoForm').enabled = false;
		var filtro = widget.appliedFilter;
	 	widget.getField('NMEMPRESA').readOnly=false;
		widget.getField('CDCONTCORR').readOnly=false;
		widget.getField('CDREDUCONT').readOnly=false;
		widget.getField('NMBANCO').readOnly=false;
		if(!filtro.FILTER_ROW.NMBANCO){
			widget.getField('NMAGENCIA').readOnly=true;
			widget.getField('CDCONTCORR').readOnly = true;
			if(widget.dataSource.data){
				widget.getField('CDCONTCORR').mask.params = {'mask':  widget.dataSource.data[0].CDPICTCONT};
			}

		}
		else{
			widget.getField('NMAGENCIA').readOnly=false;
			widget.getField('NMAGENCIA').setValue(filtro.FILTER_ROW.NMAGENCIA);
			widget.getField('NMBANCO').setValue(filtro.FILTER_ROW.NMBANCO);
			widget.currentRow.CDAGENCIA= filtro.FILTER_ROW.CDAGENCIA;
			widget.currentRow.CDBANCO= filtro.FILTER_ROW.CDBANCO;
			widget.getField('CDCONTCORR').mask.params = {'mask':filtro.FILTER_ROW.CDPICTCONT};

		}
 	};

 	this.codigoReduzidoOnchage= function(widget){
 		Ger08500_ContaCorrente_Service.validarContaCorrente(widget.currentRow).then(
 			null,
 			function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				widget.currentRow.CDREDUCONT= "";
			}
 		);
 	};

  	this.verificaMacaraCC = function(args){
		var widget = templateManager.container.getActiveWidget();
		if(widget.name=="grid_contacorrente"){
			if(!widget.currentRow.NMBANCO){
				widget.getField('CDCONTCORR').readOnly=true;
				widget.getField('CDCONTCORR').setValue("");
 			}else{
				widget.getField('CDCONTCORR').readOnly=false;
		  	widget.getField('CDCONTCORR').setValue("");
				$('#CDCONTCORR').mask(widget.currentRow.CDPICTCONT);
			}
		}
  	};

  	this.editar = function(widget){
		widget.template = widget.templateToForm;
		General.edit(widget);
	};
	this.cancelar = function(widget){
		for (var i in widget.backupRow) {
			widget.currentRow[i] = widget.backupRow[i];
		}
		widget.template = widget.templateToView;
		widget.view();
	};
	this.aplicarFiltro = function(widget) {
      	this.setFiltro(widget);
  	};
	this.setFiltro = function(widget){
    	var current= widget.FILTER_ROW;
  		var gridWidget =  templateManager.container.getWidget('grid_contacorrente');
	  	gridWidget.getField('CDCONTCORR').mask.params = {'mask':  current.CDPICTCONT};
  	};
	this.setAgencia = function(args){
    	var widget = args.owner.owner.widget;
	 	var current = widget.currentRow;
 		widget.getField("NMAGENCIA").readOnly =false;
	 	widget.getField("NMAGENCIA").setValue("");
	 	//widget.getField("CDAGENCIA").setValue("");
	 	if(!current.NMBANCO){
	 		widget.getField("NMAGENCIA").readOnly =true;
	 		widget.getField("NMAGENCIA").setValue("");
	 		//widget.getField("CDAGENCIA").setValue("");
	 		widget.getField("NMAGENCIA").reload();
	 	}
	 	if(widget.name=='zhFilterPreferencesSelectConditions'){
	 			widget.getField("CDREDUCONT").readOnly =true;
		 		widget.getField("CDREDUCONT").setValue("");
	 	}
  	};
  	this.setMascara = function(args){
  		var widget = templateManager.container.getActiveWidget();
	 	var current = widget.currentRow;
 		if(widget.name=='grid_contacorrente'){
 			widget.getField('CDCONTCORR').isVisible = false;
			widget.reload();
			widget.activate();
 			widget.getField('CDCONTCORR').setValue("");
 		 			widget.getField('CDCONTCORR').mask.params = {'mask':  current.CDPICTCONT};
 			widget.getField('CDCONTCORR').isVisible = true;
 			widget.reload();
			widget.activate();
		}
  	};
  	this.setDataSourceAgencia = function(widget){
   		widget.getField('NMAGENCIA').dataSourceFilter[0].value = widget.currentRow.CDBANCO;
    };
    
  	this.desabilitarWidgetCC = function(){
		templateManager.container.getWidget('grid_contas').enabled = false;
		templateManager.container.getWidget('grid_limitecredito').enabled = false;
		templateManager.container.getWidget('implantacaoSaldo').enabled = false;
	};
	this.habilitarWidgetCC = function(){
		templateManager.container.getWidget('grid_contas').enabled = true;
		templateManager.container.getWidget('grid_limitecredito').enabled = true;
		templateManager.container.getWidget('implantacaoSaldo').enabled = true;
 	};
 	this.desabilitarWidgetDados = function(){
		templateManager.container.getWidget('grid_contacorrente').enabled = false;
		templateManager.container.getWidget('grid_limitecredito').enabled = false;
		templateManager.container.getWidget('implantacaoSaldo').enabled = false;
	};
	this.habilitarWidgetDados = function(){
		templateManager.container.getWidget('grid_contacorrente').enabled = true;
		templateManager.container.getWidget('grid_limitecredito').enabled = true;
		templateManager.container.getWidget('implantacaoSaldo').enabled = true;
 	};
 	this.desabilitarWidgetLimite = function(){
		templateManager.container.getWidget('grid_contacorrente').enabled = false;
		templateManager.container.getWidget('grid_contas').enabled = false;
		templateManager.container.getWidget('implantacaoSaldo').enabled = false;
	};
	this.habilitarWidgetLimite = function(){
		templateManager.container.getWidget('grid_contacorrente').enabled = true;
		templateManager.container.getWidget('grid_contas').enabled = true;
		templateManager.container.getWidget('implantacaoSaldo').enabled = true;
 	};
 	
 	this.excluirContaCorrente = function(widget){
		General.deleteCrud(widget);
	};
	
	this.cancelarContaCorrente = function(args){
		General.backOrCancel(args);
		this.habilitarWidgetCC();
		templateManager.container.getWidget('grid_contas').isVisible = true;
		templateManager.container.getWidget('grid_limitecredito').isVisible = true;
		templateManager.container.getWidget('implantacaoSaldo').isVisible = true;
		templateManager.container.getWidget('implantacaoSaldoForm').isVisible = false;
	};
	
	this.salvarContaCorrente = function(contaCorrente){
    	var implantacaoSaldoForm = templateManager.container.getWidget('implantacaoSaldoForm');
    	this.changeToUpperCaseCodReduzido(contaCorrente);
    	if(contaCorrente.isValid()){
    		if (contaCorrente.currentRow.__is_new){
    			var msg = 'Deseja realizar implantação de Saldo?';
    			ScreenService.confirmMessage(msg, 'question',
        	        function() {
        	        	implantacaoSaldoForm.isVisible = true;
        	        	implantacaoSaldoForm.enabled = true;
        	        	implantacaoSaldoForm.edit();
        	            implantacaoSaldoForm.activate();
        	        }.bind(this),
        	        function(){
        	            this.confirmSaveContaCorrente(contaCorrente);
        	        }.bind(this)
        	    );
    		}else{
    			this.confirmSaveContaCorrente(contaCorrente);
    		}

    	}
  	};

  	this.confirmSaveContaCorrente = function(widget){
  		if(widget.isValid()){
	  		var deferred = ZHPromise.defer();
			Ger08500_ContaCorrente_Service.retornaFilial().then(
	    			function(filial) {
			 			if(widget.currentRow.__is_new){
			 				var cdfilial = filial[0].CDFILIAL;
							widget.currentRow.CDFILIAL=cdfilial;
			 			}
						widget.currentRow.CDCONTCORR =widget.currentRow.CDCONTCORR.replace(/[^0-9]/g,'');
						widget.currentRow.VRSALDATUA= 0;
						General.saveCrud(widget).then(
							function(){
								templateManager.container.getWidget('grid_contas').enabled = true;
								templateManager.container.getWidget('grid_limitecredito').enabled = true;
								templateManager.container.getWidget('implantacaoSaldo').enabled = true;
								templateManager.container.getWidget('grid_contas').isVisible = true;
								templateManager.container.getWidget('grid_limitecredito').isVisible = true;
								templateManager.container.getWidget('implantacaoSaldo').isVisible = true;
								templateManager.container.getWidget('implantacaoSaldoForm').isVisible = false;
								var filtro = widget.appliedFilter;
								var bancoFiltro = filtro.FILTER_ROW.CDBANCO;
								var agenciaFiltro = filtro.FILTER_ROW.CDAGENCIA;
								if( (bancoFiltro!==widget.currentRow.CDBANCO) || (agenciaFiltro!==widget.currentRow.CDAGENCIA)  ){
									widget.reload();
								}
							},
							function(callback){
								if(callback.data){
									ScreenService.showMessage(callback.data.error);
									}
						}
					);
				}.bind(this)
			)
			.catch(
				function(callback){
					deferred.reject();
					if(callback.data){
						ScreenService.showMessage(callback.data.error);
					}
				}
			);
			if(templateManager.container.getActiveWidget().name == 'implantacaoSaldoForm'){
				this.confirmSaveImplantacaoSaldo(templateManager.container.getActiveWidget());
			}
			
			return deferred.promise;
  		}
  	};

	this.setMascaraConta = function(widget){
		var gridWidget =  templateManager.container.getWidget('grid_contacorrente');
		gridWidget.getField('CDCONTCORR').mask.params = {'mask':  gridWidget.currentRow.CDPICTCONT};
	};
	
	this.salvarContaCC = function(args){
   		var Widget = args.owner.widget;
		if(Widget.currentRow.NRININOSNUMC>Widget.currentRow.NRFIMNOSNUMC){
			ScreenService.showMessage('Número inicial deve ser menor ou igual ao final');
		}
		else{
			General.saveCrud(Widget).then(function() {
				this.habilitarWidgetDados();
				Widget.list();
				Widget.template = Widget.templateToView;
				Widget.view();
			}.bind(this),
			function(callback){
				if(callback.data){
					ScreenService.showMessage(callback.data.error);
				}
			});
    	}
  	};
  	this.CheckBoxConta = function(){
		var grid_contas = templateManager.container.getWidget('grid_contas');
		if(grid_contas.getField('IDVERFXNUMP').value()=="S"){
			grid_contas.getField('NRININOSNUMC').readOnly=false;
			grid_contas.getField('NRFIMNOSNUMC').readOnly=false;
			grid_contas.getField('NRININOSNUMC').validations.required = true;
			grid_contas.getField('NRFIMNOSNUMC').validations.required = true;
		}
		else{
			grid_contas.getField('NRININOSNUMC').readOnly=true;
			grid_contas.getField('NRFIMNOSNUMC').readOnly=true;
			grid_contas.getField('NRININOSNUMC').setValue("");
			grid_contas.getField('NRFIMNOSNUMC').setValue("");
			delete(grid_contas.getField('NRININOSNUMC').validations.required );
			delete(grid_contas.getField('NRFIMNOSNUMC').validations.required );


		}
  	};
    this.CheckBoxControleCredito = function(){
 		var grid_limitecredito = templateManager.container.getWidget('grid_limitecredito');
 		if(grid_limitecredito.getField('IDCTRCREDCHQ').value()=="S"){
 			grid_limitecredito.getField('VRCTRCREDCHQ').readOnly=false;
			grid_limitecredito.getField('VRCTRCREDCHQ').validations.required = true;

		}
 		else{
 			grid_limitecredito.getField('VRCTRCREDCHQ').readOnly=true;
 			grid_limitecredito.getField('VRCTRCREDCHQ').setValue("");
 			delete(grid_limitecredito.getField('VRCTRCREDCHQ').validations.required );
 		}
    };
 	this.CheckBoxControleCobranca = function(){
 		var grid_limitecredito = templateManager.container.getWidget('grid_limitecredito');
 		if(grid_limitecredito.getField('IDCTRCOBDESC').value()=="S"){
 			grid_limitecredito.getField('VRCTRCOBDESC').readOnly=false;
 			grid_limitecredito.getField('VRPERCOBDESC').readOnly=false;
 			grid_limitecredito.getField('VRCTRCOBDESC').validations.required = true;
			grid_limitecredito.getField('VRPERCOBDESC').validations.required = true;

		}
 		else{
 			grid_limitecredito.getField('VRCTRCOBDESC').readOnly=true;
 			grid_limitecredito.getField('VRPERCOBDESC').readOnly=true;
 			grid_limitecredito.getField('VRCTRCOBDESC').setValue("");
 			grid_limitecredito.getField('VRPERCOBDESC').setValue("");
 			delete(grid_limitecredito.getField('VRCTRCOBDESC').validations.required );
 			delete(grid_limitecredito.getField('VRPERCOBDESC').validations.required );

 		}
    };
	this.salvarLimite = function(args){
    	var Widget = args.owner.widget;
		if(Widget.currentRow.VRPERCOBDESC<=100){
		    General.saveCrud(Widget).then(function() {
				this.habilitarWidgetLimite();
				Widget.list();
				Widget.template = Widget.templateToView;
				Widget.view();
				this.verficaVisibilidadeActionRecalcularValor(Widget);
			}.bind(this),
			function(callback){
				if(callback.data){
					ScreenService.showMessage(callback.data.error);
				}
			});
 	 	}else{
 	 		ScreenService.showMessage('Percentual de alerta não pode ser maior que 100%');
 	 	}
 	 };
  	this.WidgetOnEnter = function(args){
    	args.owner.currentRow = args.owner.dataSource.data[0];
	};
	this.fieldCCReduzido = function(args){
		var widget = args.owner.owner.widget;
	 	if(widget.name=='zhFilterPreferencesSelectConditions'){
		 	var current = widget.currentRow;
	 		if(!current.NMAGENCIA){
		 		widget.getField("CDREDUCONT").readOnly =true;
		 		widget.getField("CDREDUCONT").setValue("");
		 	}else{
		 		widget.getField("CDREDUCONT").readOnly =false;
		 		widget.getField("CDREDUCONT").setValue("");
		 	}
		}
	};
	
	this.fieldCCReduzidoDataSource = function(args){
		var widget = args.owner.owner.widget;
	 	var row = widget.currentRow;                                        
		widget.getField("CDREDUCONT").dataSourceFilter[0].value = row.CDBANCO;
		widget.getField("CDREDUCONT").dataSourceFilter[1].value = row.CDAGENCIA;
	 	widget.getField("CDREDUCONT").reload();
	};
	
	this.recalcularValorUlizado = function(args){
		ScreenService.confirmMessage(ScreenService.i18n('Deseja recalcular o valor utilizado em cobrança descontada?'), 'question', function (success) {
			var widget = args.owner.widget;
			Ger08500_ContaCorrente_Service.recalcularValorUtilizado(widget.currentRow).then(function(Success) {
				if(Success.dataset.recalcular_valor_utiliado[0].VRCOBDESCONTO){
					widget.getField('VRUTLCOBDESC').setValue(Success.dataset.recalcular_valor_utiliado[0].VRCOBDESCONTO);
				}
				else{
					widget.getField('VRUTLCOBDESC').setValue('0');
				}
				widget.saveRow().then(function() {
					ScreenService.closePopup();
				},
				function(callback){
					if(callback.data){
						ScreenService.showMessage(callback.data.error);
					}
				});
			},function(callback){
				if(callback.data){
					ScreenService.showMessage(callback.data.error);
				}
			});
		}, function (error) {
		});
	};

	this.verficaVisibilidadeActionRecalcularValor = function (widget){
		var data = widget.dataSource.data[0];
		var recalcularValor = widget.getAction('recalcularValor');
		if(data){
			if(data.IDCTRCOBDESC=='S'){
				recalcularValor.readOnly = false;
			}else{
				recalcularValor.readOnly = true;
			}
		}

	};

	this.atualizaImplantacao = function(widget){
		var implantacao = templateManager.container.getWidget('implantacaoSaldo');
		this.setFiltroImplantacaoSaldo(widget, implantacao).then(function(){
			implantacao.reload();
		});
	};

	this.setFiltroImplantacaoSaldo = function(widget, implantacao){
		var deferred = ZHPromise.defer();
		implantacao.dataSourceFilter.forEach(function(filter){
			if (filter.name == 'CDREDUCONT'){
				filter.value = widget.currentRow.CDREDUCONT;
			}
		});
		deferred.resolve();
		return deferred.promise;
	};

	this.saveImplantacaoSaldo = function(implantacaoSaldo){
		var contaCorrente = templateManager.container.getWidget('grid_contacorrente');
		if (contaCorrente.currentRow.__is_new){
			this.confirmSaveContaCorrente(contaCorrente);
		}else{
			this.confirmSaveImplantacaoSaldo(implantacaoSaldo);
		}
	};

	this.confirmSaveImplantacaoSaldo = function(implantacaoSaldo){
		implantacaoSaldo.currentRow.CDREDUCONT = implantacaoSaldo.getParent().currentRow.CDREDUCONT;
		implantacaoSaldo.currentRow.VRSDOCCORR = 0;
		var route = 'fin#/saveImplantacaoSaldo';
		General.callRestEngine(route, 'Row', implantacaoSaldo.currentRow)
			.then(function(){
				ScreenService.closeSwipe();
				if (implantacaoSaldo.name == 'implantacaoSaldoForm'){
					implantacaoSaldo.isVisible = false;
					implantacaoSaldo.enabled = false;
				}
				templateManager.container.getWidget('grid_contas').enabled = true;
				templateManager.container.getWidget('grid_limitecredito').enabled = true;
				templateManager.container.getWidget('implantacaoSaldo').enabled = true;
				templateManager.container.getWidget('grid_contas').isVisible = true;
				templateManager.container.getWidget('grid_limitecredito').isVisible = true;
				templateManager.container.getWidget('implantacaoSaldo').isVisible = true;
				templateManager.container.getWidget('implantacaoSaldoForm').isVisible = false;
				implantacaoSaldo.reload();
				ScreenService.showMessage('Registro salvo com sucesso.');
			})
			.catch(function(erro){
				ScreenService.showMessage(erro.data.error);
			});
	};

	this.excluiImplantacao = function(widget){
		ScreenService.confirmMessage('Deseja excluir o registro?', 'question', function() {
			var route = 'fin#/excluiImplantacaoSaldo';
			General.callRestEngine(route, 'Row', widget.currentRow)
			.then(function(){
				ScreenService.closeSwipe();
				widget.reload();
				ScreenService.showMessage('Registro excluído com sucesso.');
			})
			.catch(function(erro){
				ScreenService.showMessage(erro.data.error);
			});
		});
	};

	this.openEvolucaoSaldo = function(){
		var evolucaoSaldo = templateManager.container.getWidget('evolucaoSaldo');
		if (evolucaoSaldo.getParent()._hasCheckedRows()){
			ScreenService.openPopup(evolucaoSaldo).then(function(){
				evolucaoSaldo.isVisible = true;
			});
		}else{
			ScreenService.showMessage('Nenhuma conta selecionada para evolução.');
		}
	};

	this.evoluirSaldo = function(widget){
		if(widget.isValid()){
			var dataSet = [];
			var route = 'fin#/evolucaoSaldo';
			var dataInicial = widget.currentRow.DATAINICIAL;
			var contas = widget.getParent().getCheckedRows();
			contas.forEach(function(conta){
					var indice = dataSet.length;
	    	        dataSet[indice]={};
	    	        dataSet[indice].CDREDUCONT = conta.CDREDUCONT;
	    	        dataSet[indice].DATAINICIAL = dataInicial;
	    	    }
	        );
			General.callRestEngine(route, 'DataSet', dataSet)
			.then(function(){
				widget.isVisible = false;
				widget.getParent().unSelectAll();
				ScreenService.closePopup();
				ScreenService.showMessage('Saldo evoluído com sucesso.');
			})
			.catch(function(){
			});
		}

	};

	this.cancelarEvolucao = function(widget){
		ScreenService.closePopup();
		widget.isVisible = false;

	};
	
	this.filterAgencia = function (args){
		var widget = args.owner.field.widget;
		var row = args.row;
		widget.getField('NMAGENCIA').dataSourceFilter[0].value = row.CDBANCO;
	};
	
	this.changeToUpperCaseCodReduzido = function(widget) {
		widget.currentRow.CDREDUCONT = widget.currentRow.CDREDUCONT.toUpperCase(); 
	};

};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger08500_ContaCorrente_Controller', Ger08500_ContaCorrente_Controller);
});

// FILE: ./assets/js/controllers/Ger08900_Historico_Conciliacao_Automatica_Controller.js
var ger08900_Historico_Conciliacao_Automatica_Controller = function(templateManager, ScreenService, ZHPromise, General){


var self = this;

    self.exportDocument = function(row) {
        var widget = templateManager.container.getWidget('ger08900_historico_conciliacao_automatica_grid');
        var route = 'fin#/crud/conttrckidacs';
        
        var filter = [
            {
                "name": "DSTRACKINGID",
                "operator": "=",
                "value": row.DSTRACKINGID
            }
        ];
        
        General.callRestEngine(route, 'FilterData', filter).then(function(documents) {
            var doc = documents.dataset.conttrckidacs;
            if(doc.length > 0){
                var data = doc[0].DSCONTTRCKIDACS;
                var filename = doc[0].DSTRACKINGID;
                var type = 'data:text/ofx,';
                var ext = '';
                if(row.IDTPARQUIVO == 'O'){
                    ext = '.ofx';
                }else if(row.IDTPARQUIVO == 'C' || row.IDTPARQUIVO == 'P'){
                    ext = '.ret';
                }else if(row.IDTPARQUIVO == 'E'){
                    type = "data:text/plain;charset=utf-8,";
                }
                
                self.download(filename, data, type, ext);
            }else{
               ScreenService.showMessage('Arquivo não localizado.');
            }
        });
    };
    
    self.download = function(filename, text, type, ext) {
          var element = document.createElement('a');
          element.setAttribute('href', type + encodeURIComponent(text));
          element.setAttribute('download', filename + ext);
        
          //element.style.display = 'none';
          document.body.appendChild(element);
        
          element.click();
        
          document.body.removeChild(element);
        
        // var blob = new Blob([text], {type: 'text/csv'});
        
        // if(window.navigator.msSaveOrOpenBlob) {
        //     window.navigator.msSaveBlob(blob, filename);
        // }
        // else{
        //     var elem = window.document.createElement('a');
        //     elem.href = window.URL.createObjectURL(blob);
        //     elem.download = filename;        
        //     document.body.appendChild(elem);
        //     elem.click();        
        //     document.body.removeChild(elem);
        // }
        
        // var link = document.createElement("a");
        // link.setAttribute("target","_blank");
        // if(Blob !== undefined) {
        //     var blob = new Blob([text], {type: "text/xml"});
        //     link.setAttribute("href", URL.createObjectURL(blob));
        // } else {
        //     link.setAttribute("href","data:text/xml," + encodeURIComponent(text));
        // }
        // link.setAttribute("download",filename);
        // document.body.appendChild(link);
        // link.click();
        // document.body.removeChild(link);
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Ger08900_Historico_Conciliacao_Automatica_Controller', ger08900_Historico_Conciliacao_Automatica_Controller);
});

// FILE: ./assets/js/controllers/Ger25940_TipoRecebimento_Controller.js
var Ger25940_TipoRecebimento_Controller = function(ScreenService, General, Ger25940_TipoRecebimento_Service, templateManager){
    
    var self = this;

    var fieldSGESTADO = 'NMESTADO';

    this.validacoesSelect = function(widget){
        if (widget.getField('IDTIPORECE').value() !== "6") {
            widget.getField('NMINDICE').readOnly = true;
            widget.getField('NMINDICE').setValue(null);
        } else {
            widget.getField('NMINDICE').readOnly = false;
        }
    };
    
    this.save = function(widget){
        if(widget.isValid()){
            var ordensEmUso = widget.dataSource.data;
            var usada = false;
            for (var i = 0 ; i < ordensEmUso.length ; i++){
                if(ordensEmUso[i].NRORDTROCO == widget.currentRow.NRORDTROCO && ordensEmUso[i].CDTIPORECE !== widget.currentRow.CDTIPORECE) usada = true;
            }
            if (!usada){
                widget.saveRow().then(function(){
                    var currentRow = widget.currentRow;
                    
                    if (currentRow.__is_new) {
                        widget.list();
                        ScreenService.goBack();
                    }
                    else
                        widget.view();
                        
                    currentRow.__is_new = false;
                    currentRow.changed = false;
                    self.verificarValorFieldTipoRecebimento();
                    widget.activate();
                    widget.reload();
                }, function(calback){
                    ScreenService.showMessage(calback.data.error);
                });
            } else {
                ScreenService.showMessage("Ordem de troco já cadastrada");
            }
        }
    };
    
    this.saveEndereco = function (widget) {

        if (widget.isValid()){
            var currentRow = widget.currentRow;
            var wid = templateManager.container.getWidget('ger25940_tipo_recebimento');
            widget.currentRow.__is_new = wid.currentRow.__is_new;
            
            if(currentRow.IDTIPORECE && currentRow.NMTIPORECE && currentRow.CDTIPORECE) {
                
                widget.saveRow().then(function(){
                    
                    if (widget.currentRow.__is_new) {
                        widget.template = 'widget/form-view.html';
    	                widget.view();
    	                
                        wid.currentRow.__is_new = false;
                        self.save(wid);
                        
                        ScreenService.closeSwipe().then(function() {
                            wid.activate().then(function(){
                                wid.reload().then(function(){
                                    wid.list();
                                });
                            });
                        });
                    } else {
                        self.viewAdministradoraCartao(widget);
                        currentRow.__is_new = false;
                        currentRow.changed = false;
                        widget.activate();
                        widget.reload();
                    }
                    
                }, function(calback){
                    ScreenService.showMessage(calback.data.error);
                });
            
            } else {
                var widTpRece = templateManager.container.getWidget('ger25940_tipo_recebimento');
                widTpRece.activate();
                widTpRece.getField('CDTIPORECE').readOnly = false;
                ScreenService.showMessage("Preencha os campos: Código, Nome, Tipo de Recebimento e Ordem p/ Troco.");
            }
        } 
    };
    
    var jaPossuiTipoDeRecebimentoTrocaDeMercadoria = function () {
        var jaPossui = false;
        var data = templateManager.container.getWidget('ger25940_tipo_recebimento').dataSource.data;
      
        data.forEach(function(campo){
            if(campo.IDTPRECTROCAMER == 'S')
                jaPossui = true;
        });
        
        return jaPossui;
      
    };
    
    var jaPossuiOrdemTroco = function (widget) {
        var jaPossui = false;
        var data = templateManager.container.getWidget('ger25940_tipo_recebimento').dataSource.data;
        var CDTIPORECE = widget.currentRow.CDTIPORECE;
        var NRORDTROCO = widget.currentRow.NRORDTROCO;
        var NMTIPORECE = widget.currentRow.NMTIPORECE;
      
        data.forEach(function(campo){
            if(campo.NRORDTROCO == NRORDTROCO && campo.CDTIPORECE != CDTIPORECE && campo.NMTIPORECE != NMTIPORECE)
                jaPossui = true;
        });
        
        return jaPossui;
    };
    
    
    this.changeFilter = function (field) {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
        
        widget.getField(fieldSGESTADO).dataSourceFilter[1].value = widget.getField('CDPAIS').value();
        widget.getField('NMMUNICIPIO').dataSourceFilter[1].value = widget.currentRow.SGESTADO;
        widget.getField('NMBAIRRO').dataSourceFilter[2].value = widget.getField('CDMUNICIPIO').value();
        
        if(field) {
            if (field.name == 'NMPAIS') {
                widget.getField(fieldSGESTADO).value('');
                widget.getField('NMMUNICIPIO').value('');
                widget.getField('NMBAIRRO').value('');
            } else if (field.name == fieldSGESTADO) {
                widget.getField('NMMUNICIPIO').value('');
                widget.getField('NMBAIRRO').value('');
            }
        }
    };
    
    var codigoValido = function(widget) {
        var cdValido = true;
        var CDTIPORECE = widget.currentRow.CDTIPORECE;
        var NMTIPORECE = widget.currentRow.NMTIPORECE;
        var IDTIPORECE = widget.currentRow.IDTIPORECE;
        var NRORDTROCO = widget.currentRow.NRORDTROCO;
        
        var data = templateManager.container.getWidget('ger25940_tipo_recebimento').dataSource.data;
      
        data.forEach(function(campo){
            if(campo.CDTIPORECE == CDTIPORECE && (campo.NMTIPORECE != NMTIPORECE || campo.NRORDTROCO != NRORDTROCO))
                cdValido = false;
        });
        
        return cdValido;
    };
    
    this.realizaValidacoesSalva = function (widget) {
        
        var prontoParaSalvar = true;
        
        if(widget.isValid()) {
            
            if (widget.getField('IDSOLIQTPARC').value() == 'S' && prontoParaSalvar) {
                var qtMaxParcelas = widget.getField('QTMAXPARC').value();
                var vrMinParcelas = widget.getField('VRMINPARC').value();
                
                if (qtMaxParcelas == '0'){
                    prontoParaSalvar = false;
                    ScreenService.showMessage('A quantidade máxima de parcelas não pode ser zero.');
                }
                else if (vrMinParcelas == '0') {
                    prontoParaSalvar = false;
                    ScreenService.showMessage('O valor mínimo da parcela não pode ser zero.');
                }
                else if (!qtMaxParcelas) {
                    prontoParaSalvar = false;
                    ScreenService.showMessage('A quantidade máxima de parcelas deve ser informada.');
                }
                else if (!vrMinParcelas) {
                    prontoParaSalvar = false;
                    ScreenService.showMessage('O valor mínimo de parcelas deve ser informado.');
                }
            }
            
            if (widget.getField('IDTPRECTROCAMER').value() == 'S' && prontoParaSalvar) {
                
                if (jaPossuiTipoDeRecebimentoTrocaDeMercadoria()){
                    ScreenService.showMessage('Operação bloqueada! Já existe um tipo de recebimento parametrizado como troca de mercadoria.');
                    prontoParaSalvar = false;
                }
            }
           
            if (widget.getField('NRORDTROCO').value() && prontoParaSalvar) {
                if(jaPossuiOrdemTroco(widget)){
                    ScreenService.showMessage('Ordem do troco já cadastrada.');
                    prontoParaSalvar = false;
                }
            }   
           
            if (!codigoValido(widget)) {
                prontoParaSalvar = false;
                ScreenService.showMessage('Código Inválido.');
            }
            
            
            if(widget.getField('DSIMPFISCAL').value()){
                var valor = widget.getField('DSIMPFISCAL').value();
                // Retira espaços
                valor = valor.split('').reduce(function (amount, ch) { amount += ch != ' ' ? ch : ''; return amount; });
                widget.getField('DSIMPFISCAL').value(valor);
            }
            
            if ((widget.getField('IDTIPORECE').value() == '1' || widget.getField('IDTIPORECE').value() == '2') && prontoParaSalvar) {
                if(!widget.getField('CDBANCARTSEFAZ').value()) {
                    prontoParaSalvar = false;
                    ScreenService.showMessage('Informe o código da bandeira do cartão de crédito/débito.');
                } else {
                    var widgetEndereco = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
                    self.changeFilter();
                    var resultado = false;
                    
                    widgetEndereco.reload().then(function(){ 
                        var wid = widgetEndereco;
                        var data = wid.dataSource.data[0];
                        
                        if (widget.currentRow.__is_new) {
                            if(wid.currentRow.CDPAIS && wid.currentRow.CDMUNICIPIO && wid.currentRow.SGESTADO)
                                resultado = true;
                        } else if(data) {
                            if(data.CDPAIS && data.CDMUNICIPIO && data.SGESTADO)
                                resultado = true;
                        }
                        
                        if (!resultado) {
                            ScreenService.showMessage('País, estado e município devem ser informados.');
                            wid.reload().then(function(){ 
                                self.editAdministradoraCartao(wid);
                                wid.currentRow.IDTIPORECE = widget.currentRow.IDTIPORECE;
                                wid.currentRow.NMTIPORECE = widget.currentRow.NMTIPORECE;
                                wid.currentRow.CDTIPORECE = widget.currentRow.CDTIPORECE;
                                wid.currentRow.NRORG = getLocalVar('NRORG');
                                wid.activate();
                                wid.currentRow.__is_new = true;
                            });
                            prontoParaSalvar = false;
                            
                        } else {
                            if(prontoParaSalvar) self.save(widget);  
                        }
                    });
            
                }
            } else {
                if(prontoParaSalvar)
                    this.save(widget);
            }
        }
        
    };
    
    this.delete = function(widget){
        Ger25940_TipoRecebimento_Service.validateDelete(widget).then(function(callback){
            if (callback) {
                ScreenService.showMessage('Não é Possivel excluir esse Registro, pois este possui Registro(s) Filho(s)');
            } else {
                General.deleteCrud(widget);
            }
        });
    };
    
    /*
        A função abaixo verifica o valor do field e marca seus fields associados como readOnly (ou não).
        Parâmetro 1: field
    */
    this.verificarValorFieldTipoRecebimento = function () {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento');
        var field = widget.getField('IDTIPORECE');
        var valorCampo = field.value();
        
        if (valorCampo == 'A')
            valorCampo = 10;
        if (valorCampo == 'B')
            valorCampo = 11;
        if (valorCampo == 'C')
            valorCampo = 12;
        
        var valoresAssociados = [
            ['NMINDICE', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDADMINCART', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDADMINCART', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER'],
	    	['CDCREDENCCAR', 'CDBANCARTCR', 'CDADMINCART', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN'],
	    	['NMINDICE', 'CDCREDENCCAR', 'CDBANCARTCR', 'CDBANCARTSEFAZ', 'IDMOSTRARECE', 'IDDESABTEF',
	    	 'IDSOLNSUCREDDEB', 'IDUTILVOUCHER', 'IDHABVENDAMAN']
	    ];
        var camposParaTroca = [];
        
        if (valorCampo == '1' || valorCampo == '2') {
            templateManager.container.getWidget('ger25940_tipo_recebimento_endereco').enabled = true;
        } else {
            templateManager.container.getWidget('ger25940_tipo_recebimento_endereco').enabled = false;
        }
        
        templateManager.updateTemplate();
        
        if(valorCampo)
            camposParaTroca = valoresAssociados[valorCampo-1];
        if(camposParaTroca)
            camposParaTroca.forEach(function(campo){
                widget.getField(campo).readOnly = true; 
            });
            
        
    };
    
    this.verificarValorField = function (field) {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento');
        var widgetEndereco = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
        
        if (widget.currentRow.__is_new) {
            widget.getField('CDTIPORECE').readOnly = false;
        } else
            widget.getField('CDTIPORECE').readOnly = true;
        
        if (!field) {
            
            var fieldIDSOLIQTPARC = widget.getField('IDSOLIQTPARC');
            
            self.verificarValorField(fieldIDSOLIQTPARC);
            
            if(widget.__is_new)
                widgetEndereco.currentRow.__is_new = true;
            else
                widgetEndereco.currentRow.__is_new = false;
            
        } else {
            var nomeCampo = field.name;
            var valorCampo = field.value();
            widget = field.getParent();
            
            var camposAssociados = {
                "IDSOLIQTPARC": ['QTMAXPARC', 'VRMINPARC'],
                "SGESTADO": ['NMMUNICIPIO', "NMBAIRRO"],
                "NMMUNICIPIO": ['NMBAIRRO'],
                "NMPAIS": ['SGESTADO', 'NMMUNICIPIO', "NMBAIRRO"]
    	    };
    	    
    	     var camposParaTroca = [];
                
            switch (nomeCampo) {
                    case "IDSOLIQTPARC":
                        camposParaTroca = camposAssociados.IDSOLIQTPARC;
                        if(valorCampo == 'N') {
                            widget.getField('QTMAXPARC').value('');
                            widget.getField('VRMINPARC').value('');
                        }
                    break;
                    case fieldSGESTADO:
                        camposParaTroca = camposAssociados.SGESTADO;
                        if (!valorCampo) {
                            widget.getField('CDMUNICIPIO').value('');
                            widget.getField('NMMUNICIPIO').value('');
                        }
                    break;
                    case "NMMUNICIPIO":
                        camposParaTroca = camposAssociados.NMMUNICIPIO;
                        if (!valorCampo) {
                            widget.getField('NMBAIRRO').value('');
                            widget.getField('CDBAIRRO').value('');
                        }
                    break;
                     case "NMPAIS":
                        camposParaTroca = camposAssociados.NMMUNICIPIO;
                        if (!valorCampo) {
                            widget.getField('NMESTADO').value('');
                            widget.getField('NMESTADO').value('');
                            widget.getField('NMBAIRRO').value('');
                            widget.getField('CDBAIRRO').value('');
                            widget.getField('CDMUNICIPIO').value('');
                            widget.getField('NMMUNICIPIO').value('');
                        }
                    break;
            }
            
            if (field.template == 'field/select-native.html') {
                if (valorCampo == 'S') {
                    camposParaTroca.forEach(function(campo){
                        widget.getField(campo).readOnly = false;
                    });
                } else {
                    camposParaTroca.forEach(function(campo){
                        widget.getField(campo).readOnly = true;
                    });
                }
            } else {
                camposParaTroca.forEach(function(campo){
                    widget.getField(campo).readOnly = !field.value();
                });
                
            }
        }
    };
    
    this.prepararCamposReadOnly = function () {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento');
        
        widget.fields.forEach(function(field){
            field.readOnly = false; 
        });
        
        widget.currentRow.NRORG = getLocalVar('NRORG');
        
        this.verificarValorFieldTipoRecebimento();
        
        this.verificarValorField(widget.getField('IDSOLIQTPARC'));
        
        this.editAdministradoraCartao();
        
        
        var wid = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
        wid.template = 'widget/form-view.html';
    	wid.view();
    	
        widget.activate();
    };
    
    this.setDataIntoForm = function() {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');

        var data = widget.dataSource.data[0];
        
        if(data) {
            widget.getField('DSENDERECE').value(data.DSENDERECE);
            widget.getField('NRENDERECE').value(data.NRENDERECE);
            widget.getField('DSCOMPLERECE').value(data.DSCOMPLERECE);
            widget.getField('DSLOGRECE').value(data.DSLOGRECE);
            widget.getField('CDPAIS').value(data.CDPAIS);
            widget.getField('NMPAIS').value(data.NMPAIS);
            widget.getField(fieldSGESTADO).value(data.SGESTADO);
            widget.getField('CDMUNICIPIO').value(data.CDMUNICIPIO);
            widget.getField('NMMUNICIPIO').value(data.NMMUNICIPIO);
            widget.getField('NRCEPRECE').value(data.NRCEPRECE);
            widget.getField('CDBAIRRO').value(data.CDBAIRRO);
            widget.getField('NMBAIRRO').value(data.NMBAIRRO);
        }
    };
    
    this.editAdministradoraCartao = function(widget){
        widget = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
        var wid = templateManager.container.getWidget('ger25940_tipo_recebimento');
        widget.edit();
        widget.formTemplate = "widget/form.html";
        widget.template = "widget/form.html";
        
        this.changeFilter();
        
        widget.currentRow.IDTIPORECE = wid.currentRow.IDTIPORECE;
        widget.currentRow.NMTIPORECE = wid.currentRow.NMTIPORECE;
        widget.currentRow.CDTIPORECE = wid.currentRow.CDTIPORECE;
        
        widget.getField('NMMUNICIPIO').dataSourceFilter[1].value = widget.getField(fieldSGESTADO).value();
    };
    
    this.viewAdministradoraCartao = function(widget) {
        widget.template = 'widget/form-view.html';
    	widget.view();
        widget.activate();
    };
    
    this.limparCampos = function () {
        var widget = templateManager.container.getWidget('ger25940_tipo_recebimento_endereco');
        
        widget.getField('DSENDERECE').value('');
        widget.getField('NRENDERECE').value('');
        widget.getField('DSCOMPLERECE').value('');
        widget.getField('DSLOGRECE').value('');
        widget.getField('CDPAIS').value('');
        widget.getField('NMPAIS').value('');
        widget.getField(fieldSGESTADO).value('');
        widget.getField('CDMUNICIPIO').value('');
        widget.getField('NMMUNICIPIO').value('');
        widget.getField('NRCEPRECE').value('');
        widget.getField('CDBAIRRO').value('');
        widget.getField('NMBAIRRO').value('');
    };
};
Configuration(function (ContextRegister){
 ContextRegister.register('Ger25940_TipoRecebimento_Controller', Ger25940_TipoRecebimento_Controller);
});

/*global getLocalVar*/

// FILE: ./assets/js/controllers/Ger38000_TipoBaixaReceber_Controller.js
var Ger38000_TipoBaixaReceber_Controller = function(General, ScreenService, Ger38000_TipoBaixaReceber_Service){

	this.insertInit = function(widget){
		Ger38000_TipoBaixaReceber_Service.insertInit().then(undefined, function(error){
			ScreenService.showMessage(error.data.error);
		});
	};

	this.remover = function(widget){
		var codigo = widget.currentRow.CDTPBAIXAREC;
		if(codigo == '01'||codigo == '02'||codigo == '03'){
			ScreenService.showMessage('Esse registro não pode ser excluído.');
		} else {
			General.deleteCrud(widget);
		}
	};
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger38000_TipoBaixaReceber_Controller', Ger38000_TipoBaixaReceber_Controller);
});

// FILE: ./assets/js/controllers/Ger38100_ImportacaoTitulosAberto_Controller.js
var Ger38100_ImportacaoTitulosAberto_Controller = function(ScreenService, Ger38100_ImportacaoTitulosAberto_Service){
    
    var CONTA_PAGAR = 'P';
    var CONTA_RECEBER = 'R';
    var IMPORT_TIME_OUT = 'Falha ao importar. Se o erro persistir, contate a Teknisa.';
    var IMPORT_UNEXPECTED_ERROR = 'Erro inseperado. Favor entrar em contato com a Teknisa.';
    var IMPORT_EMPTY_CELL_VALUE_ERROR = 'Célula vazia.';
    var IMPORT_FILE_FORMAT_ERROR = 'Arquivo insconsistente.';
    var IMPORT_LOAD_FILE_ERROR = 'Falha ao carregar arquivo. Favor tentar novamente';
    var IMPORT_NOT_FOUND_CELL_ERROR = 'Valor do campo V1 não se encontra nos registros.';
    var IMPORT_NOT_RELATED_CELL_ERROR = 'Valor do campo V1 não relacionado com o campo V2 nos registros.';
    var IMPORT_SUCCESS = 'Importação concluida com successo';
    var IMPORT_ROW_EXISTS_ERROR =  "Nota Fiscal já cadastrada no sistema.";
    var XLS_READER_EMPTY_COLUMN_VALUE_ERROR = "Título da coluna vazio.";
    var XLS_READER_FILE_FORMAT_ERROR = 'Formato do arquivo inválido';
    var XLS_READER_HEADER_SIZE_ERROR  = 'Cabeçalho com um número diferente de colunas do esperado.';
    var XLS_READER_INVALID_COLUMN_VALUE_ERROR = 'Título da coluna com nome inválido.';
    var IMPORT_ROW_CENTCUST_ERROR = 'Operação bloqueada. Centro de custo não cadastrado para a filial.';
    
    var errorMessages = {
        'IMPORT_UNEXPECTED_ERROR': IMPORT_UNEXPECTED_ERROR,
        'IMPORT_EMPTY_CELL_VALUE_ERROR': IMPORT_EMPTY_CELL_VALUE_ERROR,
        'XLS_READER_EMPTY_COLUMN_VALUE_ERROR': XLS_READER_EMPTY_COLUMN_VALUE_ERROR,
        'IMPORT_FILE_FORMAT_ERROR': IMPORT_FILE_FORMAT_ERROR,
        'XLS_READER_HEADER_SIZE_ERROR': XLS_READER_HEADER_SIZE_ERROR,
        'XLS_READER_INVALID_COLUMN_VALUE_ERROR': XLS_READER_INVALID_COLUMN_VALUE_ERROR,
        'IMPORT_LOAD_FILE_ERROR': IMPORT_LOAD_FILE_ERROR,
        'IMPORT_NOT_FOUND_CELL_ERROR': IMPORT_NOT_FOUND_CELL_ERROR,
        'IMPORT_NOT_RELATED_CELL_ERROR': IMPORT_NOT_RELATED_CELL_ERROR,
        'IMPORT_ROW_EXISTS_ERROR': IMPORT_ROW_EXISTS_ERROR,
        'XLS_READER_FILE_FORMAT_ERROR': XLS_READER_FILE_FORMAT_ERROR,
        'IMPORT_ROW_CENTCUST_ERROR': IMPORT_ROW_CENTCUST_ERROR
    };
    
    function addError(widget, error){
        var row = createErrorRow(error);
        widget.dataSource.data.push(row);
    }
    
    function createErrorRow(error){
        var columnName = error.columnName ? error.columnName : undefined;
        var dbColumnRelated = error.dbColumnRelated ? error.dbColumnRelated : undefined;
        var errorMessage = getErrorMessage(error.message, columnName, dbColumnRelated);
        var row = {
            'NRLINHA': error.row,
            'NRCOLUNA': error.column ? error.column : null,
            'DSERRO': errorMessage
        };
        return row;
    }
    
    function getErrorMessage(messageCode, columnName, dbColumnRelated){
        var errorMessage;
        switch(messageCode){
            case 'IMPORT_NOT_FOUND_CELL_ERROR':
                errorMessage = errorMessages[messageCode].replace('V1', columnName);
                break;
            case 'IMPORT_NOT_RELATED_CELL_ERROR':
                errorMessage = errorMessages[messageCode].replace('V1', columnName);
                errorMessage = errorMessage.replace('V2', dbColumnRelated);
                break;
            default:
                errorMessage = errorMessages[messageCode];
        }
        return errorMessage;
    }
    
    function importSuccess(widget){
        return function(success){
            if(success.dataset && success.dataset.importTitulos &&
               success.dataset.importTitulos.success === true){
        		ScreenService.showMessage(IMPORT_SUCCESS, 'SUCCESS');
                if(widget.widgets[0].dataSource.data.length > 0 ){
                    widget.widgets[0].dataSource.data = [];
                }
            }
            else{
                ScreenService.showMessage(IMPORT_TIME_OUT);
            }
        };
    }
    
    function importError(widget){
        return function(error){
            var errorMessage = 'Timeout';
    		if (error.data) {
    		    if(error.data.error in errorMessages){
    		        errorMessage = errorMessages[error.data.error];
    		    }
    			else{
    			    errorMessage = error.data.error;
    			}
    			widget.widgets[0].dataSource.data = [];
    			if(error.data.dataset){
    			    if( error.data.dataset.errors){
            		    var errorsInFile = error.data.dataset.errors;
            		    errorsInFile.forEach(
            		        function(errorInFile){
            		            addError(widget.widgets[0], errorInFile);
            		        }
            		    );
    			    }
        		}
    		}
    		ScreenService.showMessage(errorMessage);
        };
    }
    
    this.importTitulo = function(widget){ //tabbedWidget
        if(widget.isValid()){
            var file = widget.currentRow.FILEXLS;
            var tipoImportacao = widget.currentRow.IMPORTTYPE;
            var pos = file[0].b64File.indexOf(',') + 1;
            file = {
                'name': file[0].name,
                'content': file[0].b64File.substr(pos)
            };
            var row = {
                'CDEMPRESA': widget.currentRow.CDEMPRESA,
                'FILE': file
            };
            if(tipoImportacao == CONTA_PAGAR){
                Ger38100_ImportacaoTitulosAberto_Service.importTituloPagar(row).then(
                    importSuccess(widget), 
                    importError(widget)
        		);
            }
            else{
                if(tipoImportacao == CONTA_RECEBER){
                    Ger38100_ImportacaoTitulosAberto_Service.importTituloReceber(row).then(
                        importSuccess(widget), 
                        importError(widget)
            		);
                }
            }
        }
    };
    
    this.isFieldEmpty = function(field){
        return (!field || field.length === 0);
    };
    
    this.onChangeSelectedFile = function(widget){
        widget.widgets[0].dataSource.data = [];
    };
    
};

Configuration(function (ContextRegister){
 ContextRegister.register('Ger38100_ImportacaoTitulosAberto_Controller', Ger38100_ImportacaoTitulosAberto_Controller);
});

// FILE: ./assets/js/controllers/Ger40000_Relatorio_Dre_Gerencial_Controller.js
var ger40000_Relatorio_Dre_Gerencial_Controller = function(ScreenService, templateManager, General, ZHPromise){
	
	var self = this;
	
	self.configFilter = function(widget, preference){
	    
	    var currentRowFilter = preference.FILTER_ROW;

        var empresa = currentRowFilter.CDEMPRESA;
        var ano     = currentRowFilter.ANO;

        widget.dataSourceFilter.forEach(function (filtro) {
            switch(filtro.name){
                case 'CDEMPRESA': filtro.value = empresa;
                    break;
                case 'DTINICIAL': filtro.value = '01/01/' + ano;
                    break;
                case 'DTFINAL': filtro.value = '31/12/' + ano;
                    break;
            }
        });
        
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        delete preference.CUSTOM_FILTER_ROW.ANO;
    };
    
};

/* global Configuration */
Configuration(function (ContextRegister){
    ContextRegister.register('Ger40000_Relatorio_Dre_Gerencial_Controller', ger40000_Relatorio_Dre_Gerencial_Controller);
});

// FILE: ./assets/js/controllers/Ger40100_Categorias_Dre_Gerencial_Controller.js
var ger40100_Categorias_Dre_Gerencial_Controller = function(ScreenService, templateManager, General, ZHPromise){
	
	var self = this;
	
	self.abreJanelaDRE = function(){
        var categorias_dre_tabbed = templateManager.container.getWidget('categorias_dre_tabbed');
        var categorias_dre_form   = templateManager.container.getWidget('categorias_dre_form');
        
        ScreenService.openSwipe(categorias_dre_tabbed).then(function(){
            var categorias_dre_grid   = templateManager.container.getWidget('categorias_dre_grid');
        
            categorias_dre_form.dataSourceFilter.forEach(function(filtro){
                switch(filtro.name){
                    case 'NRORG': filtro.value = categorias_dre_grid.currentRow.NRORG;
                        break;
                    case 'CDITEMESTRU': filtro.value = categorias_dre_grid.currentRow.CDITEMESTRU;
                        break;
                }
            });
            
            categorias_dre_form.reload().then(function(){
                self.atualizaFormCategoria(categorias_dre_form).then(function(){
                    categorias_dre_form.mode  = "view";
                    categorias_dre_form.redraw();
                    categorias_dre_form.reload();
                });
            });
            
           // self.aplicaFiltroCategoria().then(function(){
                //categorias_dre_form.reload();
            //});
        
            
        });
    };
    
    /*self.aplicaFiltroCategoria = function(){
        var deferred = ZHPromise.defer();
        
        var categorias_dre_form   = templateManager.container.getWidget('categorias_dre_form');
        var categorias_dre_grid   = templateManager.container.getWidget('categorias_dre_grid');
        
        categorias_dre_form.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'NRORG': filtro.value = categorias_dre_grid.currentRow.NRORG;
                    break;
                case 'CDITEMESTRU': filtro.value = categorias_dre_grid.currentRow.CDITEMESTRU;
                    break;
            }
        });
        
        deferred.resolve();
        return deferred.promise;
        
    };*/
	
	self.novaCategoriaDRE = function(){
        var categorias_dre_tabbed = templateManager.container.getWidget('categorias_dre_tabbed');
        ScreenService.openSwipe(categorias_dre_tabbed);
        
        var categorias_dre_form   = templateManager.container.getWidget('categorias_dre_form');
        General.widgetNewRow(categorias_dre_form);
    };
    
    self.salvaCategoriaDRE = function(){
        var categorias_dre_form = templateManager.container.getWidget('categorias_dre_form');
        
        General.saveCrud(categorias_dre_form).then(function(success) {
            var categorias_dre_grid = templateManager.container.getWidget('categorias_dre_grid');
            
            categorias_dre_grid.reload();
            categorias_dre_grid.activate();
        });
    };
    
    self.atualizaFormCategoria = function(widget) {
        var deferred = ZHPromise.defer();
        widget.currentRow = widget.dataSource.data[0]?widget.dataSource.data[0]:{};
        templateManager.updateTemplate();
        deferred.resolve();
        return deferred.promise;
    };
    
};

/* global Configuration */
Configuration(function (ContextRegister){
    ContextRegister.register('Ger40100_Categorias_Dre_Gerencial_Controller', ger40100_Categorias_Dre_Gerencial_Controller);
});

// FILE: ./assets/js/controllers/Ger56000_relatorio_customizavel_rateio_de_despesas_receitas_x_centro_de_custo.js
var Ger56000_relatorio_customizavel_rateio_de_despesas_receitas_x_centro_de_custo = function(templateManager, ScreenService, General, ZhFilterPreferences){
    /**
     * Variáveis de Inicialização
     **/
    var self = this;
    
    /**
     * Abertura do Filtro de Relatório
     * @param widget 
     */
     
    self.openFilterRelatorio = function(widget){
      ZhFilterPreferences.openFilterWidget(widget);
    };

    /**
     * Valida os campos valor inicial, valor final, caso algum deles for preenchido
     * insere o requerido no outro para não dar problema no campo.
     * @param widget 
     */
    
    self.validaCampoVrIni = function(widget) {
      var valorInicial = widget.getField("VALORINI");
      var valorFinal = widget.getField("VALORFIM");
      if ((valorInicial.value() == "") && (valorFinal.value() == "")) {
          valorInicial.validations = {};
          valorInicial.setValue("");
          valorFinal.validations = {};
          valorFinal.setValue("");
      } else if ((valorInicial.value() != "") || (valorFinal.value() == "")) {
          valorInicial.validations = {};
          valorFinal.validations = {
              "required": {
                  "message": "Campo obrigatório"
              }
          };
      } else if ((valorInicial.value() == "") || (valorFinal.value() != "")) {
          valorFinal.validations = {};
          valorInicial.validations = {
              "required": {
                  "message": "Campo obrigatório"
              }
          };
      } else {
          valorInicial.validations = {};
          valorFinal.validations = {};
      }
      widget.reload();
    };
    
    /**
     * Apliaca o filtro para montagem do relatório
     * @param widget 
    */
    self.filterRelatorio = function(widget, preference) {

      // this.validaCampoVrIni(widget);

      var currentRowFiltro = preference.FILTER_ROW;
      var cdempresa = currentRowFiltro.CDEMPRESA;
      var cdcentcust = currentRowFiltro.CDCENTCUST;
      var dtperiodo = currentRowFiltro.DTATUAVEN;
      var cdfilial = currentRowFiltro.CDFILILANC;
      var valorinicial = currentRowFiltro.VALORINI;
      var valorfinal = currentRowFiltro.VALORFIM;
      var status = currentRowFiltro.IDSTATUS;

      widget.dataSourceFilter.forEach(function(filtro) {
          switch (filtro.name) {
              case 'CDEMPRESA':
                  filtro.value = cdempresa ? cdempresa : 'T';
                  break;
              case 'CENTCUST':
                  filtro.value = cdcentcust ? cdcentcust : 'T';
                  break;
              case 'DATVENIN':
                  filtro.value = dtperiodo.start;
                  break;
              case 'DATVENFIN':
                  filtro.value = dtperiodo.end;
                  break;
              case 'CDFILILANC':
                  filtro.value = cdfilial ? cdfilial : 'T';
                  break;
              case 'STATUS':
                  filtro.value = status ? status : 'T';
                  break;
              case 'VALORINI':
                  filtro.value = valorinicial ? valorinicial : '';
                  if (filtro.value !== '') {
                      widget.dataSourceFilter[0].value = '';
                  } else {
                      widget.dataSourceFilter[0].value = 'T';
                  }
                  break;
              case 'VALORFIM':
                  filtro.value = valorfinal ? valorfinal : '';
                  break;
          }
      });
      preference.CUSTOM_FILTER_ROW = {};
  };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger56000_relatorio_customizavel_rateio_de_despesas_receitas_x_centro_de_custo', Ger56000_relatorio_customizavel_rateio_de_despesas_receitas_x_centro_de_custo);
});

// FILE: ./assets/js/controllers/Ger99000_HistoricoImportacaoBancariaAutomatica_Controller.js
var Ger99000_HistoricoImportacaoBancariaAutomatica_Controller = function(ScreenService, General, templateManager){

    var self = this;

    self.exportDocument = function() {
        var widget = templateManager.container.getWidget('historico_importacao_bancaria_automatica');
      
        var tipo     = widget.currentRow.TIPO;
        var data     = widget.currentRow.CONTEUDO;
        var filename = widget.currentRow.ARQUIVO?widget.currentRow.ARQUIVO:'arquivo';
        
        var type      = 'data:text/ofx,';
        var extension = '';
        
        if(tipo == 'O'){
            extension = '.ofx';
        }else if(tipo == 'P' || tipo == 'R'){
            extension = '.RET';
        }else{
            type = "data:text/plain;charset=utf-8,";
        }
        
        self.download(filename, data, type, extension);
    };
    
    self.download = function(filename, text, type, extension) {
          var element = document.createElement('a');
          element.setAttribute('href', type + encodeURIComponent(text));
          element.setAttribute('download', filename + extension);
          
          document.body.appendChild(element);
        
          element.click();
        
          document.body.removeChild(element);
    };
    
};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger99000_HistoricoImportacaoBancariaAutomatica_Controller', Ger99000_HistoricoImportacaoBancariaAutomatica_Controller);
});

// FILE: ./assets/js/controllers/Lib00101_Contabilizacao_Controller.js
var Lib00101_Contabilizacao_Controller = function(contabilizacaoService, ZHPromise, ScreenService, templateManager, maskEngine, ZhFilterPreferences, engine, General, Pag00001_TitulosAPagar_Service, Rec00001_TitulosAReceber_Service, localStorageEngine) {

    var self = this;

    /********************************************************************
     *                                                                  *
     *                          Funções gerais                          *
     *                                                                  *
     ********************************************************************/

    /*
     * Função para validação de erros de contabilização, deve ser chamada
     * em caso de erro de uma requisição de lançamento que utilize contabilização.
     * Caso tenha ocorrido algum erro, os dados são salvos localmente
     * e uma mensagem é exibida para o usuário.
     * @return Boolean
     *   Retorna true se não houve problemas de contabilização, false caso contrário.
     */
    self.checkErrosContabilizacao = function(errorResponse, errorMessage) {
        // Variável que retorna se a contabilização não ocasionou o erro.
        var isValid = true;
        // Verifica se houve uma resposta da requisição e se há algum dataset associado.
        if (errorResponse && errorResponse.data.dataset) {
            var dataSets = errorResponse.data.dataset;
            // Verificar se o dataset em questão é válido.
            // Caso a requisição retorne um array vazio, não entrará no if.
            if (!Util.isEmptyOrBlank(dataSets.errosContabilizacao)) {
                // Inicializa a mensagem com o valor padrão, caso nenhuma
                // mensagem seja passada como parâmetro.
                errorMessage = errorMessage || "Não foi possível realizar o lançamento devido a erros de contabilização.";
                // A variável de controle é alterada, pois o dataSet de erros não está vazio.
                isValid = false;
                // Array com as promises que devem ser executadas antes de abrir a tela de contabilização.
                var promises = [
                    // Seta os dataSets localmente.
                    contabilizacaoService.setLocalData(dataSets),
                    // Exibe a mensagem pro usuário informando que há erros no lançamento.
                    ScreenService.showMessage(errorMessage)
                ];
                // Executa todas as promises do array.
                ZHPromise.all(promises).then(function() {
                    // Abre a tela de contabilização.
                    engine.openWindow("fin#lib00101_contabilizacao");
                });
            }
        }
        return isValid;
    };
    
    self.checkErrosContabilizacaoRetorno = function(errorResponse, errorMessage) {
        // Variável que retorna se a contabilização não ocasionou o erro.
        var isValid = true;
        // Verifica se houve uma resposta da requisição e se há algum dataset associado.
        if (errorResponse && errorResponse.data.dataset) {
            var dataSets = errorResponse.data.dataset;
            // Verificar se o dataset em questão é válido.
            // Caso a requisição retorne um array vazio, não entrará no if.
            if (!Util.isEmptyOrBlank(dataSets.errosContabilizacao)) {
                // Inicializa a mensagem com o valor padrão, caso nenhuma
                // mensagem seja passada como parâmetro.
                errorMessage = errorMessage || "Não foi possível realizar o lançamento devido a erros de contabilização.";
                // A variável de controle é alterada, pois o dataSet de erros não está vazio.
                isValid = false;
                // Array com as promises que devem ser executadas antes de abrir a tela de contabilização.
                var promises = [
                    // Seta os dataSets localmente.
                    contabilizacaoService.setLocalData(dataSets),
                    // Exibe a mensagem pro usuário informando que há erros no lançamento.
                    ScreenService.showMessage(errorMessage)
                ];
                // Executa todas as promises do array.
                ZHPromise.all(promises).then(function() {
                    // Abre a tela de contabilização.
                    templateManager.container.getActiveWidget().view();
                    ScreenService.goBack();
                    templateManager.container.getActiveWidget().reload();
                });
            }
        }
        return isValid;
    };

    /*
     * Função para exibição da contabilização de lançamentos, abrindo uma tela
     * para a exibição dos dados.
     * Recebe como parâmetro um objeto ou um array de objetos, onde cada objeto
     * tem as propriedades CDEMPRESA e NRLANCCONT, indicando o código da
     * empresa e o número do lançamento, respectivamente.
     */
    self.exibeContabilizacao = function(lancamentos) {
        var isMultiplosLancamentos = true;
        // Verifica se foi enviado um lançamento somente, colocando-o em um
        // array.
        if (Util.isObject(lancamentos)) {
            lancamentos = [lancamentos];
            isMultiplosLancamentos = false;
        }
        contabilizacaoService.getContabilizacaoLancamentos(lancamentos)
        .then(function(response) {
            if (response && response.dataset) {
                var dataSets = response.dataset;
                // Verificar se o dataset em questão é válido.
                // Verifica se a requisição retorna partidas da contabilização.
                if (!Util.isEmptyOrBlank(dataSets.partidasContabilizacao)) {
                    // Salva os dados retornados e abre a tela de contabilização.
                    contabilizacaoService.setLocalData(dataSets).then(function() {
                        engine.openWindow("fin#lib00101_contabilizacao");
                    });
                }
                else {
                    // Exibe mensagem de que não há contabilização para o(s)
                    // lançamento(s) selecionado(s).
                    var noDataMessage = "Não há contabilização para " + (isMultiplosLancamentos ? "os lançamentos selecionados." : "o lançamento selecionado.");
                    ScreenService.showMessage(noDataMessage);
                }
            }
        });
    };


    /********************************************************************
     *                                                                  *
     *                          Funções da tela                         *
     *                                                                  *
     ********************************************************************/
    // Variável para identificação se a tela está aberta para exibição de erros.
    var isErrorRoutine = true;
    // Variáveis de armazenamento de empresa e número de lançamento para o filtro.
    var cdempresa = "";
    var nrlancctbtemp = "";
    // Variável contendo os códigos sequenciais das partidas inválidas.
    // Utilizada para controlar a exibição de partidas, exibindo as inválidas com destaque.
    var partidasInvalidas = [];
    // Variável contendo os códigos sequenciais das partidas que possuem erros relacionados
    // aos seus rateios. Utilizado para o filtro na tab de rateios.
    var partidasRateiosInvalidos = [];

    self.beforeGridFilterApply = function(partidasWidget) {
        var filterWidget = partidasWidget.appliedFilter;
        cdempresa = filterWidget.currentRow.CDEMPRESA;
        nrlancctbtemp = filterWidget.currentRow.NRLANCCTBTEMP;
        checkContopectblParams(cdempresa);
    };

    /*
     * Função auxiliar para montar os dados a ser exibidos nos fields do filtro.
     */
    var buildFilterData = function(partidasWidget, partidas) {
        var empresaField = partidasWidget.getField("NMEMPRESA");
        var lancamentoField = partidasWidget.getField("NRLANCCTBTEMP");
        var empresaAux = {};
        var empresaData = [];
        var lancamentoData = [];
        // Varre todos as partidas, para buscar somente empresas distintas e
        // lançamentos distintos.
        partidas.forEach(function(partida) {
            // Verifica se a empresa da partida já foi adicionada ao objeto.
            if (!empresaAux[partida.CDEMPRESA]) {
                // Se não, adiciona, acrescentando o nome da empresa.
                empresaAux[partida.CDEMPRESA] = {NMEMPRESA: partida.NMEMPRESA};
            }
            // Verifica se o número de lançamento da partida já foi adicionado.
            if (!empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP]) {
                // Se não, adiciona, acrescentando informações complementares
                // e inicializando a contabilização total de créditos/débitos.
                empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP] = {
                    CDDOCUCONT: partida.CDDOCUCONT,
                    DTLANCCTBTEMP: partida.DTLANCCTBTEMP,
                    VRCREDLANC: partida.IDDEBCRE == 'CR' ? partida.VRLANCCTBTEMP : 0,
                    VRDEBLANC: partida.IDDEBCRE == 'CR' ? 0 : partida.VRLANCCTBTEMP
                };
            }
            else {
                // Se o lançamento já foi adicionado, adiciona o valor da
                // partida à contabilização dos valores.

                // Se a partida tiver um valor de crédito, acrescenta o valor aos créditos.
                if (partida.IDDEBCRE == 'CR') {
                    empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP].VRCREDLANC += partida.VRLANCCTBTEMP;
                }
                // Senão, acrescenta à contabilização de débitos.
                else {
                    empresaAux[partida.CDEMPRESA][partida.NRLANCCTBTEMP].VRDEBLANC += partida.VRLANCCTBTEMP;
                }
            }
        });

        // Varre todas as empresas distintas, adicionando ao array principal.
        Object.keys(empresaAux).forEach(function(cdempresa) {
            empresaData.push({
                CDEMPRESA: cdempresa,
                NMEMPRESA: empresaAux[cdempresa].NMEMPRESA
            });
            // Deleta o nome da empresa para restar somente os lançamentos
            // como chaves do objeto.
            delete empresaAux[cdempresa].NMEMPRESA;
            Object.keys(empresaAux[cdempresa]).forEach(function(nrlancctbtemp) {
                // Adiciona todos os lançamentos ao array principal.
                lancamentoData.push({
                    CDEMPRESA: cdempresa,
                    NRLANCCTBTEMP: nrlancctbtemp,
                    CDDOCUCONT: empresaAux[cdempresa][nrlancctbtemp].CDDOCUCONT,
                    DTLANCCTBTEMP: empresaAux[cdempresa][nrlancctbtemp].DTLANCCTBTEMP,
                    VRCREDLANC: empresaAux[cdempresa][nrlancctbtemp].VRCREDLANC,
                    VRDEBLANC: empresaAux[cdempresa][nrlancctbtemp].VRDEBLANC
               });
            });
        });

        // Seta os arrays aos dataSources dos respectivos fields.
        empresaField.dataSource.data = empresaData;
        lancamentoField.dataSource.data = lancamentoData;
    };

    /*
     * Função para alterar o filtro do field select de partidas, ao selecionar
     * os rateios que serão exibidos.
     */
    self.changeFilterPartidas = function(field) {
        var widget = field.widget;
        var partidasField = widget.getField("NRSEQCTBTEMP");
        // Seta um filtro vazio como default, caso seja selecionado todas as partidas.
        var filter = [
            {
                name: "CDEMPRESA",
                operator: "=",
                value: "@contabilizacaoController.getCdempresa()@"
            },
            {
                name: "NRLANCCTBTEMP",
                operator: "=",
                value: "@contabilizacaoController.getNrlancctbtemp()@"
            }
        ];
        // Se for selecionado o filtro das partidas com problemas de rateios, muda o filtro.
        if (field.value() === 'R') {
            filter.push({
                name: "NRSEQCTBTEMP",
                operator: "IN",
                value: partidasRateiosInvalidos
            });
        }
        // Troca o filtro e recarrega os dados do field.
        partidasField.dataSourceFilter = filter;
        partidasField.reload();
        partidasField.setValue([]);
    };

    /*
     * Função para verificar os parâmetros na tabela CONTOPECTBL de acordo com a empresa do lançamento.
     * Caso o campo IDCONTGERE não esteja setado como 'S', a tab "Rateios" não é exibida.
     * Caso o campo IDCONTCOME não esteja setado como 'S', os fields "Tipo Conta" e
     * "Código Comercial" não são exibidos.
     */
    var checkContopectblParams = function(cdempresa) {
        // Busca o código da empresa da primeira partida retornada.
        var filter = [{
            "name": "CDEMPRESA",
            "operator": "=",
            "value": cdempresa
        }];
        contabilizacaoService.getContopectblParams(filter)
        .then(function(response) {
            // Verifica se a resposta da requisição trouxe o dataSource "CONTOPECTBL".
            if (response && response.dataset && response.dataset.contopectbl) {
                // Busca a primeira linha retornada, que seria o
                // objeto da parametrização em si.
                var contopectbl = response.dataset.contopectbl[0];
                if (contopectbl) {
                    // Se o parâmetro não estiver setado, a tab de rateios é escondida.
                    var rateiosIsVisible = contopectbl.IDCONTGERE == 'S';
                    templateManager.container.getWidget("rateiosTabbed").isVisible = rateiosIsVisible;

                    // Se o parâmetro não estiver setado, os campos
                    // "Tipo Conta" e "Código Comercial" são escondidos.
                    var fieldIsVisible = contopectbl.IDCONTCOME == 'S';
                    var partidasWidget = templateManager.container.getWidget("partidas");
                    partidasWidget.getField("IDLANCCTBCOME").showOnList = fieldIsVisible;
                    //partidasWidget.getField("CDLANCCTBCOME").showOnList = fieldIsVisible;
                    templateManager.updateTemplate();
                }
            }
        });
    };

    /*
     * Verifica se a rotina para a qual a tela foi aberta é a de exibição de
     * erros ou exibição de contabilidade de um lançamento já contabilizado.
     */
    self.checkExibicaoErros = function() {
        contabilizacaoService.getErros().then(function(erros) {
            // Verifica se a rotina é de erros com base nos erros retornados.
            isErrorRoutine = erros.length > 0;

            // Exibe ou esconde a action de erros conforme rotina.
            var partidasWidget = templateManager.container.getWidget("partidas");
            var errosAction = partidasWidget.actions.filter(function(action) {
                return action.name == 'erros';
            })[0];
            errosAction.isVisible = isErrorRoutine;

            var rateiosTabbedWidget = templateManager.container.getWidget("rateiosTabbed");
            var filtroRateiosField = rateiosTabbedWidget.getField("TODASPARTIDAS");
            filtroRateiosField.isVisible = isErrorRoutine;
        });
    };

    /*
     * Função para verificação de quantos lançamentos serão exibidos, para
     * verificar a necessidade do filtro.
     */
    self.checkMultiplosLancamentos = function() {
        // Busca as partidas salvas localmente.
        contabilizacaoService.getPartidas()
        .then(function(partidas) {
            cdempresa = partidas[0].CDEMPRESA;
            nrlancctbtemp = partidas[0].NRLANCCTBTEMP;
            // Verifica se há alguma partida cujo número de lançamento e empresa
            // sejam diferentes da primeira partida.
            var multiplosLancamentos = partidas.some(function(partida) {
                return partida.NRLANCCTBTEMP != nrlancctbtemp || partida.CDEMPRESA != cdempresa;
            });
            var partidasWidget = templateManager.container.getWidget("partidas");
            // Se houver mais de um lançamento, remove os valores das variáveis
            // de armazenamento de empresa e lançamento, abre o filtro e exibe
            // a action de filtrar do stripe.
            if (multiplosLancamentos) {
                cdempresa = "";
                nrlancctbtemp = "";
                buildFilterData(partidasWidget, partidas);
                templateManager.container.getWidget("stripe").actions[0].isVisible = true;
                ZhFilterPreferences.openFilterWidget(partidasWidget);
            }
            // Se não, carrega o grid principal e verifica os parâmetros
            // de contabilização da empresa do lançamento.
            else {
                partidasWidget.reload();
                checkContopectblParams(cdempresa);
            }
        });
    };

    /*
     * Função para recarregar widgets que dependem de outros valores.
     * Um caso em que é utilizada é quando há uma tabbed com fields, onde
     * sua widget só é carregada quando seus fields estão preenchidos.
     */
    self.checkReloadDependentWidget = function(currentWidget) {
        // Busca a widget filha.
        var childWidget = templateManager.container.getWidget(currentWidget.dependentWidgetName);
        // Verifica se todos os fields de que a widget filha depende estão preenchidos.
        var toReload = currentWidget.widgetDependentFields.every(function(fieldValue) {
            return !!currentWidget.currentRow[fieldValue];
        });
        // Se todos os fields estiverem preenchidos, recarrega a widget filha.
        if (toReload) {
            childWidget.reload();
        }
        // Se houver algum field não preenchido, a widget filha é resetada e fica em estado readonly.
        else {
            childWidget.dataSource.data = [];
        }
    };

    /*
     * Função chamada quando o usuário abre outra tela, ao sair desta.
     * Limpa os dataSets locais que foram setados para esta tela.
     */
    self.clearLocalData = function() {
        contabilizacaoService.clearLocalData();
    };

    /*
     * Função chamada ao alterar a empresa selecionada no filtro, caso haja
     * mais de um lançamento.
     */
    self.empresaFilterOnChange = function(filterWidget) {
        var lancamentoField = filterWidget.getField("NRLANCCTBTEMP");
        lancamentoField.setValue(null);
        if (filterWidget.currentRow.CDEMPRESA) {
            lancamentoField.readOnly = false;
            lancamentoField.reload();
        }
        else {
            lancamentoField.readOnly = true;
        }
    };

    /*
     * Função auxiliar utilizada pelos dataSourceFilters para filtrar lançamento.
     */
    self.getCdempresa = function() {
        return cdempresa;
    };

    /*
     * Função auxiliar utilizada pelos dataSourceFilters para filtrar lançamento.
     */
    self.getNrlancctbtemp = function() {
        return nrlancctbtemp;
    };

    /*
     * Função para abrir a widget contendo a lista de erros de contabilização.
     */
    self.openErrosWidget = function(widget) {
        widget.isVisible = true;
        ScreenService.openSwipe(widget.widgets[0]);
    };

    /*
     * Função para verificar se a partida em questão faz parte da lista de
     * partidas com erros. É utilizada pelas widgets de exibição de partidas
     * para destacar as partidas com erros.
     */
    self.partidaIsValid = function(nrseqctbtemp) {
        // Retorna true caso o sequencial não seja encontrado no array.
        return partidasInvalidas.indexOf(nrseqctbtemp) === -1;
    };

    /*
     * Função chamada quando o usuário abre outra tela para setar a máscara
     * de conta contábil no respectivo field.
     */
    self.setContaContabilMask = function(widget) {
        contabilizacaoService.getParamGeral().then(function(response) {
            // Verifica se a requisição foi bem-sucedida.
            if (response && response.dataset && response.dataset.paramgeral) {
                // Variável do primeiro dado da PARAMGERAL.
                var paramGeral = response.dataset.paramgeral[0];
                // Cria o objeto de máscara dos fields.
                var mask = {
                    type: "fix",
                    params: {
                    // Adicionado '?' para permitir mais caracteres em códigos inválidos.
                        mask: '?' + paramGeral.CDPICTCCTB + 'AAAAAA'
                    }
                };
                // Aplica a máscara no campo "Conta Contábil" do grid principal.
                widget.getField("CDCONTCTBL").mask = mask;

                // Busca o field de conta contábil no filtro da tab "Rateios".
                var rateiosTabbedWidget = templateManager.container.getWidget("rateiosTabbed");
                rateiosTabbedWidget.getField("NRSEQCTBTEMP").fields[1].mask = mask;
            }
        });
    };


    /*
     * Função executada ao abrir a tela para diferenciar as partidas que têm
     * erros e aquelas que têm erros de rateios.
     */
    self.setPartidasInvalidas = function() {
        // Verifica se a tela está aberta para exibição de erros de contabilização.
        if (isErrorRoutine) {
            // Recarrega os dados dos erros, conforme o lançamento filtrado.
            var errosWidget = templateManager.container.getWidget("erros");
            errosWidget.reload().then(function() {
                var data = errosWidget.dataSource.data;
                // Filtra somente os erros que estejam relacionados a uma partida.
                var errosComPartidas = data.filter(function(erro) {
                    // Retorna true caso o erro tenha um código de partida.
                    return !!erro.NRSEQCTBTEMP;
                });
                // A variável de partidas recebe somente um array com os códigos,
                // sem os objetos dos erros em si.
                partidasInvalidas = errosComPartidas.map(function(erro) {
                    // Retorna o código de cada erro.
                    return erro.NRSEQCTBTEMP;
                });

                // Esta variável recebe os códigos das partidas que tem erros relacionados
                // aos rateios. Mais especificamente as partidas dos erros de código
                // "007", "008", "009", "010" e "013".
                var codigosErrosRateios = [
                    "007",
                    "008",
                    "009",
                    "010",
                    "013"
                ];
                var errosRateios = errosComPartidas.filter(function(erro) {
                    // Retorna true caso o código do erro esteja no array.
                    return codigosErrosRateios.indexOf(erro.CDERROTEMP) !== -1;
                });
                partidasRateiosInvalidos = errosRateios.map(function(erro) {
                    // Retorna o código de cada erro.
                    return erro.NRSEQCTBTEMP;
                });
            });
        }
    };

    /*
     * Função chamada ao abrir a tela e carregar o dataSource pela primeira vez.
     * Seta os valores do cabeçalho da tela.
     */
    self.setStripeData = function(gridWidget) {
        // Como os dados do cabeçalho são os mesmos para todas as partidas,
        // a primeira partida do dataSet será utilizada como referência.
        var currentRow = gridWidget.dataSource.data[0];
        var stripeWidget = templateManager.container.getWidget("stripe");

        // Contabiliza o valor total de débitos e créditos no lançamento.
        var creditos = 0;
        var debitos = 0;
        gridWidget.dataSource.data.forEach(function(partida) {
            // Se a partida tiver um valor de crédito, acrescenta o valor aos créditos.
            if (partida.IDDEBCRE == 'CR') {
                creditos += partida.VRLANCCTBTEMP;
            }
            // Senão, acrescenta à contabilização de débitos.
            else {
                debitos += partida.VRLANCCTBTEMP;
            }
        });

        // Seta os dados na stripe.
        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMEMPRESA = currentRow.CDEMPRESA + ' - ' + currentRow.NMEMPRESA;
        stripeWidget.currentRow.CDDOCUCONT = currentRow.CDDOCUCONT;
        stripeWidget.currentRow.DTLANCCTBTEMP = currentRow.DTLANCCTBTEMP;
        stripeWidget.currentRow.VRCREDLANC = currencyMask(creditos);
        stripeWidget.currentRow.VRDEBLANC = currencyMask(debitos);
    };

    /*
     * Função temporária para colocar máscara nos valores que estão na Stripe.
     * Utilizada devido a problemas no Framework em relação às máscaras.
     * @return String
     *   Retorna o valor com a máscara do tipo currency.
     */
    var currencyMask = function(value) {
        // Verifica se é um valor válido.
        value = value || 0;
        // Cria a parametrização da máscara.
        var params = {
            type: "currency",
            params: {
                showSymbol: false,
                precision: 2,
                thousands: ".",
                decimal: ","
            }
        };
        // Aplica a máscara no valor.
        return Util.applyMask(value, params, maskEngine);
    };
    
    /*
     * Função para editar a tela de Rateio.
     * Seta modo de edição.
     */
    
    self.editWidget = function(widget) {
        General.edit(widget);
        if(widget.name === 'partidas'){
            self.verificaTipoContaChange(widget);
        }
    };
    
    /*
     * Função para salvar dados da partida.
     * Seta modo de edição.
     */
    self.savePartidaCtbl = function(widget) {
        var TIPOBAIXA = localStorage.getItem('TIPOBAIXA');
        if(typeof(widget.currentRow.VRLANCCTBTEMP) === 'string'){
            widget.currentRow.VRLANCCTBTEMP = parseFloat(widget.currentRow.VRLANCCTBTEMP.replace(/\./g,'').replace(',','.'));
        }
        
        contabilizacaoService.setPartidas(widget.dataSource.data).then(function(){
            var promises = [ 
                contabilizacaoService.getPartidas(),
                contabilizacaoService.getRateios(),
                contabilizacaoService.getErros()
            ];
            ZHPromise.all(promises).then(function(dataSets){
                General.callRestEngine('fin#/verificaErroContab', 'DataSet', [
                    {'CDEMPRESA': widget.currentRow.CDEMPRESA},
                    {'NRLANCCTBTEMP': widget.currentRow.NRLANCCTBTEMP},
                    {'DATASET': dataSets},
                    {'TIPOBAIXA': TIPOBAIXA}
                    ]).then( function(sucess){
                        if(sucess.dataset.response[0] === true){
                            ScreenService.showMessage('Lançamento realizado');
                            ScreenService.goBack().then(function (){
                                if(TIPOBAIXA != 'LANCTRANS'){
                                    ScreenService.goBack();
                                }
                                self.confirmaBaixaTituloSave();
                            });
                        } else {
                            widget.view();
                        }
                }, function(error){
                    if (self.checkErrosContabilizacaoRetorno(error)) {
                            ScreenService.showMessage(error.data.error);
                        }
                });
            });
        });
        
    };
    
    /*
     * Função para salvar dados do rateio.
     * Seta modo de edição.
     */
    self.saveRateioCtbl = function(widget) {
        var TIPOBAIXA = localStorage.getItem('TIPOBAIXA');
        /*if(typeof(widget.currentRow.VRRATECTBTEMP) === 'string'){
            widget.currentRow.VRRATECTBTEMP = parseFloat(widget.currentRow.VRRATECTBTEMP.replace(/\./g,'').replace(',','.'));
        }*/
        
        contabilizacaoService.setRateio(widget.dataSource.data).then(function(){
            var promises = [ 
                contabilizacaoService.getPartidas(),
                contabilizacaoService.getRateios(),
                contabilizacaoService.getErros()
            ];
            ZHPromise.all(promises).then(function(dataSets){
                General.callRestEngine('fin#/verificaErroContab', 'DataSet', [
                    {'CDEMPRESA': widget.currentRow.CDEMPRESA},
                    {'NRLANCCTBTEMP': widget.currentRow.NRLANCCTBTEMP},
                    {'DATASET': dataSets},
                    {'TIPOBAIXA': TIPOBAIXA}
                    ]).then( function(sucess){
                        if(sucess.dataset.response[0] === true){
                            ScreenService.showMessage('Lançamento realizado');
                            ScreenService.goBack().then(function (){
                                ScreenService.goBack();
                                self.confirmaBaixaTituloSave();
                            });
                        } else {
                            widget.view();
                        }
                }, function(error){
                    if (self.checkErrosContabilizacaoRetorno(error)) {
                            ScreenService.showMessage(error.data.error);
                        }
                });
            });
        });
        
    };
    
    /*self.getContabilizacaoLancamentos = function(lancamentos) {
        return restEngine.post('', {
            "requestType": "DataSet",
            "dataset": lancamentos
        });
        
    };*/
    
    self.tipoContaChange = function(widget){
        widget.getField('DSCONTCORR').clearValue();
        widget.getField('NMRAZSOCCLIE').clearValue();
        widget.getField('NMRAZSOCFORN').clearValue();
    
        self.verificaTipoContaChange(widget);
        
        templateManager.updateTemplate();      
    };
    
    self.verificaTipoContaChange = function(widget){
        if(widget.currentRow.IDLANCCTBCOME === 'N' ){
            widget.getField('DSCONTCORR').readOnly = true;
            widget.getField('NMRAZSOCCLIE').readOnly = true;
            widget.getField('NMRAZSOCFORN').readOnly = true;
            widget.getField('DSCONTCORR').validations = {};
            widget.getField('NMRAZSOCCLIE').validations = {};
            widget.getField('NMRAZSOCFORN').validations = {};
            
        } 
        else if(widget.currentRow.IDLANCCTBCOME === "B"){
            widget.getField('DSCONTCORR').readOnly = false;
            widget.getField('NMRAZSOCCLIE').readOnly = true;
            widget.getField('NMRAZSOCFORN').readOnly = true;
            widget.getField('DSCONTCORR').validations = { 'required': true };
            widget.getField('NMRAZSOCCLIE').validations = {};
            widget.getField('NMRAZSOCFORN').validations = {};
            
        }
        else if(widget.currentRow.IDLANCCTBCOME === "F"){
            widget.getField('DSCONTCORR').readOnly = true;
            widget.getField('NMRAZSOCCLIE').readOnly = true;
            widget.getField('NMRAZSOCFORN').readOnly = false;
            widget.getField('DSCONTCORR').validations = {};
            widget.getField('NMRAZSOCCLIE').validations = {};
            widget.getField('NMRAZSOCFORN').validations = { 'required': true };
            
        }
        else if(widget.currentRow.IDLANCCTBCOME === "C"){
            widget.getField('DSCONTCORR').readOnly = true;
            widget.getField('NMRAZSOCCLIE').readOnly = false;
            widget.getField('NMRAZSOCFORN').readOnly = true;
            widget.getField('DSCONTCORR').validations = {};
            widget.getField('NMRAZSOCCLIE').validations = { 'required': true };
            widget.getField('NMRAZSOCFORN').validations = {};
            
        }
        
    };
    
    self.confirmaBaixaTituloSave = function(){
        var TIPOBAIXA = localStorage.getItem('TIPOBAIXA');
        var currentRow;
        if(TIPOBAIXA == 'PAG'){    
            currentRow = JSON.parse(localStorage.getItem('baixaPagRow'));
            currentRow.contabiliza = false;
        
            Pag00001_TitulosAPagar_Service.baixaTituloPagar(currentRow).then(function () {
                if( currentRow.baixaboleto.IDBAIXAUTILP == 'A') {
                    ScreenService.confirmMessage('Cheque inicial '+currentRow.baixaboleto.NRCHEQUE+'. Confirmar emissão do cheque?', 'question', function () {
                
                    Pag00001_TitulosAPagar_Service.buscaMunicipio(getLocalVar('CDFILIAL')).then(
                        function (response) {
        
                                var report = 'ModeloCheque';
                                var routes = 'fin_contaPagar#/' + report;
                                var row = {
                                    P_TESTE   : 'N',
                                    P_STLOCAL : response.dataset.municipio_endefili[0].NMMUNICIPIO,
                                    P_CODREDU : currentRow.baixaboleto.CDREDUCONT,
                                    P_NUMERO  : currentRow.baixaboleto.NRCHEQUE
                                };
        
                                General.openReport(routes, report, row);
        
                                ScreenService.showMessage('Título(s) baixado(s) com sucesso.');
                                templateManager.container.getWidget('TitulosAPagarGrid').unSelectAll();
                                
                                templateManager.container.getWidget('BaixaForm').isVisible = false;
                                templateManager.container.getWidget('TitulosAPagarGrid').reload();
                                
                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                }
                
                ScreenService.showMessage('Título(s) baixado(s) com sucesso.');
                templateManager.container.getWidget('BaixaForm').isVisible = false;
                templateManager.container.getWidget('TitulosAPagarGrid').unSelectAll();
                templateManager.container.getWidget('TitulosAPagarGrid').reload();
                
            }, function (error) {
                if (self.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        } else if(TIPOBAIXA == 'REC'){
            currentRow = JSON.parse(localStorage.getItem('baixaRecRow'));
            currentRow.contabiliza = false;
            
            Rec00001_TitulosAReceber_Service.baixaTituloReceber(currentRow).then(function () {
                ScreenService.showMessage('Título(s) baixado(s) com sucesso.');
                templateManager.container.getWidget('NewBaixaTabbed').isVisible = false;
                templateManager.container.getWidget('TitulosAReceberGrid').unSelectAll();
                templateManager.container.getWidget('TitulosAReceberGrid').reload();
                
            }, function (error) {
                if (self.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        } else if(TIPOBAIXA == 'LANCBANC'){
            
            //PAREEEI AQUI
            currentRow = JSON.parse(localStorage.getItem('lancBancRow'));
            currentRow.push({'contabiliza': false});
            
            General.callRestEngine('fin_movimentacaoBancaria#/saveLancamentoContaCorrente', 'DataSet', currentRow).then(function () {
                ScreenService.showMessage('Registro salvo com sucesso');
                templateManager.container.getWidget('lancamentoContaCorrente').activate();
                templateManager.container.getWidget('lancamentoContaCorrente').reload();
            })
            .catch(function(erro){
                if (self.checkErrosContabilizacao(erro)){
                    ScreenService.showMessage(erro.data.error);
                }
            });
        } else if(TIPOBAIXA == 'LANCTRANS'){
            
            //PAREEEI AQUI
            currentRow = JSON.parse(localStorage.getItem('lancTransRow'));
            currentRow.push({'contabiliza': false});
            
            General.callRestEngine('fin_movimentacaoBancaria#/saveTransferenciaSaldo', 'DataSet', currentRow).then(function () {
                ScreenService.showMessage('Registro salvo com sucesso');
                templateManager.container.getWidget('transferenciaContaCorrente').activate();
                templateManager.container.getWidget('transferenciaContaCorrente').reload();
            })
            .catch(function(erro){
                if (self.checkErrosContabilizacao(erro)){
                    ScreenService.showMessage(erro.data.error);
                }
            });
        }
    };
    
    self.completaTipoConta = function(widget){
        if(widget.currentRow.IDLANCCTBCOME === "B"){
            General.callRestEngine('fin#/crud/contcorr', 'FilterData', [
                {
                    name: 'CDREDUCONT',
                    operator: '=' ,
                    value: widget.currentRow.CDLANCCTBCOME
                }
                ]).then( 
                function(sucess){
                widget.currentRow.CDREDUCONT = sucess.dataset.contcorr[0].CDREDUCONT;
                widget.currentRow.CDBANCO = sucess.dataset.contcorr[0].CDBANCO;
                widget.currentRow.CDAGENCIA = sucess.dataset.contcorr[0].CDAGENCIA;
                widget.currentRow.CDCONTCORR = sucess.dataset.contcorr[0].CDCONTCORR;
                widget.currentRow.DSCONTCORR = sucess.dataset.contcorr[0].DSCONTCORR;
            });
        }
        else if(widget.currentRow.IDLANCCTBCOME === "F"){
            General.callRestEngine('fin#/crud/fornecedor', 'FilterData', [
                {
                    name: 'CDFORNECED',
                    operator: '=' ,
                    value: widget.currentRow.CDLANCCTBCOME
                }
                ]).then( 
                function(sucess){
                widget.currentRow.CDFORNECED = sucess.dataset.fornecedor[0].CDFORNECED;
                widget.currentRow.NRINSJURFORN = sucess.dataset.fornecedor[0].NRINSJURFORN;
                widget.currentRow.NMRAZSOCFORN = sucess.dataset.fornecedor[0].NMRAZSOCFORN;
                widget.currentRow.NMFANTFORN = sucess.dataset.fornecedor[0].NMFANTFORN;
            });
        }
        else if(widget.currentRow.IDLANCCTBCOME === "C"){
            General.callRestEngine('fin#/crud/cliente', 'FilterData', [
                {
                    name: 'CDCLIENTE',
                    operator: '=' ,
                    value: widget.currentRow.CDLANCCTBCOME
                }
                ]).then( 
                function(sucess){
                widget.currentRow.CDCLIENTE = sucess.dataset.cliente[0].CDCLIENTE;
                widget.currentRow.NRINSJURCLIE = sucess.dataset.cliente[0].NRINSJURCLIE;
                widget.currentRow.NMRAZSOCCLIE = sucess.dataset.cliente[0].NMRAZSOCCLIE;
                widget.currentRow.NMFANTCLIE = sucess.dataset.cliente[0].NMFANTCLIE;
            });
        }
        
        widget.currentRow.DSHISTPADR = widget.currentRow.DSHISTCTB;
        
    };
    
};

Configuration(function (ContextRegister){
 ContextRegister.register('contabilizacaoController', Lib00101_Contabilizacao_Controller);
});

// FILE: ./assets/js/controllers/Pag00600_Posicao_Contas_Pagar_Controller.js
var Pag00600_Posicao_Contas_Pagar_Controller = function(templateManager) {
    var self = this;

//Monta um datasource filter que o backend trabalha sem sofrer
    this.preparaFiltro = function(args) {
        var widget    = templateManager.container.getWidget('posicao_contas_pagar');
    	var filterRow = args.e.preference.FILTER_ROW;
        var dataSourceFilterClone = Util.clone(widget.dataSourceFilter);

    	for(var index = 0, filters = widget.dataSourceFilter; index< filters.length ; index ++){
    	   var item = filters[index];
    	   var cloneIndex = dataSourceFilterClone.findIndex(filter=> filter.name == item.name);
            if(item.name == 'DTENTRSAID' || item.name == 'NMEMPRESA'){
    	        dataSourceFilterClone.splice(cloneIndex, 1);
    	    }else{
    	        switch (item.name) {
    	            case 'DTINICIAL':
    	                item.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.start : null;
    	                break;
        	        case 'DTFINAL':
    	                item.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.end : null;
    	                break;
    	            case 'CDEMPRESA':
    	            case 'CDAREAGERE':
	                case 'CDCENTCUST':
    	            case 'CDFILIAL':
    	                item.operator = 'LIKE_ALL';
        	            item.value = item.value ? ('%'+item.value+'%'):'%%';
    	                break;
    	           case 'DTBAIXAPAG':
    	               item.value = item.value ? item.value: null;
    	               break;

    	            default:
    	                if(!filterRow[item.name]){
                	        item.value = '%%';
                	    }else if(filterRow[item.name] && (!item.value|| !Array.isArray(item.value))){
                	        item.value = filterRow[item.name];
                	    }
    	        }
    	        dataSourceFilterClone[cloneIndex] = item;
    	    }
    	}

    	widget.dataSourceFilter  = dataSourceFilterClone;
        widget.reload();

    };


};

Configuration(function(ContextRegister) {
    ContextRegister.register('Pag00600_Posicao_Contas_Pagar_Controller', Pag00600_Posicao_Contas_Pagar_Controller);
});


// FILE: ./assets/js/controllers/Pag00666_RelatorioDespesasXReceitas_Controller.js
var Pag00666_RelatorioDespesasXReceitas_Controller = function(templateManager, ScreenService, General, ZhFilterPreferences)
{
    /**
     * Variáveis de Inicialização
     **/
    var self = this;
    
    /**
     * Abertura do Filtro de Relatório
     * @param widget 
     */
    self.openFilterRelatorio = function(widget){
        var rowSetFilter = {
            'IDPERIODESP': "CO",
            'IDPERIORECE': "CO"
        };
        
        ZhFilterPreferences.setOriginalFilter(widget.id, rowSetFilter);
        ZhFilterPreferences.openFilterWidget(widget, rowSetFilter);
    };
    
    /**
     * Filtro Relatório
     * @param widget
     */
    self.filterRelatorio = function(widget, preference){
        var currentRowFiltro = preference.FILTER_ROW;

        var cdempresa   = currentRowFiltro.CDEMPRESA;
        var dtperiodo   = currentRowFiltro.DTPERIODO;
        var idperiodesp = currentRowFiltro.IDPERIODESP;
        var idperiorece = currentRowFiltro.IDPERIORECE;
        var cdclassfina = currentRowFiltro.CDCLASSFINA;
        var cdtpctpagar = currentRowFiltro.CDTPCTPAGAR;
        var cdtpctreceb = currentRowFiltro.CDTPCTRECEB;
        
        var dtemissaod, dtvencid, dtbaixad, dtemissaor, dtvencir, dtbaixar;
        
        if(idperiodesp == 'CO'){
              dtemissaod = dtperiodo;
        }else if(idperiodesp == 'VE'){
              dtvencid = dtperiodo;
        }else dtbaixad = dtperiodo;
        
        if(idperiorece == 'CO'){
              dtemissaor = dtperiodo;
        }else if(idperiodesp == 'VE'){
              dtvencir = dtperiodo;
        }else dtbaixar = dtperiodo;
        
        widget.dataSourceFilter.forEach(function (filtro) {
            switch(filtro.name){
                case 'P_CDEMPRESA'   : filtro.value = cdempresa;
                    break;
                case 'P_DTENTRSAIDINID': filtro.value = dtemissaod?dtemissaod.start?dtemissaod.start:'':'';
                    break;
                case 'P_DTENTRSAIDFIND' : filtro.value = dtemissaod?dtemissaod.end?dtemissaod.end:'':'';
                    break;
                case 'P_TDTENTRSAIDD'  : filtro.value = dtemissaod?(dtemissaod.start && dtemissaod.end)?'':'T':'T';
                    break;
                case 'P_DTATUAVENINID' : filtro.value = dtvencid?dtvencid.start?dtvencid.start:'':'';
                    break;
                case 'P_DTATUAVENFIND' : filtro.value = dtvencid?dtvencid.end?dtvencid.end:'':'';
                    break;
                case 'P_TDTATUAVEND'   : filtro.value = dtvencid?(dtvencid.start && dtvencid.end)?'':'T':'T';
                    break;
                case 'P_DTBAIXAINID'   : filtro.value = dtbaixad?dtbaixad.start?dtbaixad.start:'':'';
                    break;
                case 'P_DTBAIXAFIND'   : filtro.value = dtbaixad?dtbaixad.end?dtbaixad.end:'':'';
                    break;
                case 'P_TDTBAIXAD'     : filtro.value = dtbaixad?(dtbaixad.start && dtbaixad.end)?'':'T':'T';
                    break;
                case 'P_DTEMISSAOINIR' : filtro.value = dtemissaor?dtemissaor.start?dtemissaor.start:'':'';
                    break;
                case 'P_DTEMISSAOFINR' : filtro.value = dtemissaor?dtemissaor.end?dtemissaor.end:'':'';
                    break;
                case 'P_TDTEMISSAOR'   : filtro.value = dtemissaor?(dtemissaor.start && dtemissaor.end)?'':'T':'T';
                    break;
                case 'P_DTATUAVENINIR' : filtro.value = dtvencir?dtvencir.start?dtvencir.start:'':'';
                    break;
                case 'P_DTATUAVENFINR' : filtro.value = dtvencir?dtvencir.end?dtvencir.end:'':'';
                    break;
                case 'P_TDTATUAVENR'   : filtro.value = dtvencir?(dtvencir.start && dtvencir.end)?'':'T':'T';
                    break;
                case 'P_DTBAIXAINIR'   : filtro.value = dtbaixar?dtbaixar.start?dtbaixar.start:'':'';
                    break;
                case 'P_DTBAIXAFINR'   : filtro.value = dtbaixar?dtbaixar.end?dtbaixar.end:'':'';
                    break;
                case 'P_TDTBAIXAR'     : filtro.value = dtbaixar?(dtbaixar.start && dtbaixar.end)?'':'T':'T';
                    break;
                case 'P_CDCLASSFINA' : filtro.value = cdclassfina?(cdclassfina.length > 0?cdclassfina.join('_'):'T'):'T';
                    break;
                case 'P_CDTPCTPAGAR' : filtro.value = cdtpctpagar?(cdtpctpagar.length > 0?cdtpctpagar.join('_'):'T'):'T';
                    break;
                case 'P_CDTPCTRECEB' : filtro.value = cdtpctreceb?(cdtpctreceb.length > 0?cdtpctreceb.join('_'):'T'):'T';
                    break;
            }
        });
        
        preference.CUSTOM_FILTER_ROW = {};
    };

};

/* global Configuration */
Configuration(function (ContextRegister){
    ContextRegister.register('Pag00666_RelatorioDespesasXReceitas_Controller', Pag00666_RelatorioDespesasXReceitas_Controller);
});

// FILE: ./assets/js/controllers/Pag00800_RecalculoContabilizacaoTitulosBaixas_Controller.js
var Pag00800_RecalculoContabilizacaoTitulosBaixas_Controller = function(ZhFilterPreferences, ScreenService, contabilizacaoController, General, templateManager, Pag00800_RecalculoContabilizacaoTitulosBaixas_Service, ZHPromise) {

    var INVALIDO_CODE = 1;
    var FORA_DO_PERIODO_CODE = 2;

    //Armazena máscara de conta contábil
    var contctblmask = "";

    var self = this;
    
    this.openFilterRecalculoTitulosAPagar = function(recalculoTituloAPagar){
        this.buildFilterFormTitulosPagar().then(function(filterRow){
            ZhFilterPreferences.setOriginalFilter(recalculoTituloAPagar.id, filterRow);
            ZhFilterPreferences.openFilterWidget(recalculoTituloAPagar);    
        });
    };
    
    this.openFilterRecalculoTitulosAReceber = function(recalculoTituloAReceber){
        this.buildFilterFormTitulosReceber().then(function(filterRow){
            ZhFilterPreferences.setOriginalFilter(recalculoTituloAReceber.id, filterRow);
            ZhFilterPreferences.openFilterWidget(recalculoTituloAReceber);    
        });
    };
    
    this.openFilterRecalculoBaixasAPagar = function(recalculoBaixasAPagar){
        this.buildFilterFormBaixasPagar().then(function(filterRow){
            ZhFilterPreferences.setOriginalFilter(recalculoBaixasAPagar.id, filterRow);
            ZhFilterPreferences.openFilterWidget(recalculoBaixasAPagar);    
        });
    };
    
    this.openFilterRecalculoBaixasAReceber = function(recalculoBaixasAReceber){
        this.buildFilterFormBaixasReceber().then(function(filterRow){
            ZhFilterPreferences.setOriginalFilter(recalculoBaixasAReceber.id, filterRow);
            ZhFilterPreferences.openFilterWidget(recalculoBaixasAReceber);    
        });
    };

    this.validaIntervaloValor = function (Inicial, Final, fieldAtivo){
        if ((fieldAtivo.name == Inicial.name) && Final.value() === ''){
            var id = '#' + Final.name;
            $(id).focus();
        }else{
            Inicial.setValue(((Inicial.value() === '') || (Inicial.value() === undefined))? undefined : General.valueToFloat(Inicial.value()));
            Final.setValue(((Final.value() === '') || (Final.value() === undefined))? undefined : General.valueToFloat(Final.value()));
            if(Inicial.value() && Final.value()) {
                if(General.valueToFloat(Inicial.value()) > General.valueToFloat(Final.value())) {
                    Inicial.setValue(undefined);
                    Final.setValue(undefined);
                    ScreenService.showMessage('Valor inicial deve ser menor que o valor final.');
                }
            }
        }
    };

    this.validateRangeFields = function(field) {

        var widget = field.widget;

    };

    /**
     * Consulta Contabilização
     */
    this.consultaContabilizacaoTitulosPagar = function(widget){
        var checkedRows = widget.getCheckedRows();

        if(checkedRows.length) {
            var contabil = checkedRows.filter(function(checkedRow) {
                return checkedRow.NRLANCTBDOC;
            }).map(function(filteredRow) {
                return {
                    CDEMPRESA: filteredRow.CDEMPRESA,
                    NRLANCCONT: filteredRow.NRLANCTBDOC
                };
            });

            if(contabil.length) {
                contabilizacaoController.exibeContabilizacao(contabil);
            } else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };

    this.consultaContabilizacaoTitulosReceber = function(widget){
        var checkedRows = widget.getCheckedRows();

        if(checkedRows.length) {
            var contabil = checkedRows.filter(function(checkedRow) {
                return checkedRow.NRLANCTBDOC;
            }).map(function(filteredRow) {
                return {
                    CDEMPRESA: filteredRow.CDEMPRESA,
                    NRLANCCONT: filteredRow.NRLANCTBDOC
                };
            });

            if(contabil.length) {
                contabilizacaoController.exibeContabilizacao(contabil);
            } else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };

    this.consultaContabilizacaoBaixasPagar = function(widget){
        var checkedRows = widget.getCheckedRows();

        if(checkedRows.length) {
            var contabil = checkedRows.filter(function(checkedRow) {
                return checkedRow.NRLCTBBPAG;
            }).map(function(filteredRow) {
                return {
                    CDEMPRESA: filteredRow.CDEMPRESA,
                    NRLANCCONT: filteredRow.NRLCTBBPAG
                };
            });

            if(contabil.length) {
                contabilizacaoController.exibeContabilizacao(contabil);
            } else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };

    this.consultaContabilizacaoBaixasReceber = function(widget){
        var checkedRows = widget.getCheckedRows();

        if(checkedRows.length) {
            var contabil = checkedRows.filter(function(checkedRow) {
                return checkedRow.NRLCTBBREC;
            }).map(function(filteredRow) {
                return {
                    CDEMPRESA: filteredRow.CDEMPRESA,
                    NRLANCCONT: filteredRow.NRLCTBBREC
                };
            });

            if(contabil.length) {
                contabilizacaoController.exibeContabilizacao(contabil);
            } else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };

    this.setContctblMask = function() {
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getCDPICTCCTB().then(function(CDPICTCCTB) {
            contctblmask = CDPICTCCTB;
        });
    };

    this.getContctblMask = function() {
        return contctblmask;
    };

    this.setNRNOTAFISCMaskLength = function() {

        getQtyZeroes().then(function(qtyZeroes) {

           var maxlength = qtyZeroes;

           templateManager.container.widgets[0].widgets.forEach(function(tabbed) {
               tabbed.widgets.forEach(function(grid) {
                   grid.fields.filter(function(field) {
                       return /^NRNOTAFISC.*$/.test(field.name);
                   }).forEach(function(nrnotafiscField) {
                        nrnotafiscField.maxlength = maxlength;
                        nrnotafiscField.mask = {
                            type: "zerofill",
                            params: {
                                right: false,
                                left: true,
                                maxlength: maxlength,
                                acceptOnlyNumbers: true
                            }
                        };
                   });
               });
           });

        });

    };

    var getQtyZeroes = function() {
        return Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getNRTAMNUMNFUT().then(function(NRTAMNUMNFUT) {
            return NRTAMNUMNFUT;
        });
    }.bind(this);

    this.buildFilterFormTitulosPagar = function() {
        var deferred = ZHPromise.defer();
        var filterRow = {'IDCONTABILIZA': 'A', 'IDVENCIDAS': 'A'};
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getDiasTituloPagar()
        .then(function(response) {
            filterRow.DTVENC = {
                start: moment().subtract(response[0] ? response[0] : 0, 'days').format('DD/MM/YYYY'),
                period: undefined,
                end: moment().add(response[1] ? response[1] : 0, 'days').format('DD/MM/YYYY')
            };
            deferred.resolve(filterRow);
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    this.buildFilterFormTitulosReceber = function() {
        var deferred = ZHPromise.defer();
        var filterRow = {'IDCONTABILIZA': 'A'};
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getDiasTituloReceber()
        .then(function(response) {
            filterRow.DTVENC = {
                start: moment().subtract(response[0] ? response[0] : 0, 'days').format('DD/MM/YYYY'),
                period: undefined,
                end: moment().add(response[1] ? response[1] : 0, 'days').format('DD/MM/YYYY')
            };
            deferred.resolve(filterRow);
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    this.buildFilterFormBaixasPagar = function() {
        var deferred = ZHPromise.defer();
        var filterRow = {'IDCONTABILIZA': 'A'};
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getDiasBaixaPagar()
        .then(function(response) {
            filterRow.DTBAIXAPAG = {
                start: moment().subtract(response[0] ? response[0] : 0, 'days').format('DD/MM/YYYY'),
                period: undefined,
                end: moment().add(response[1] ? response[1] : 0, 'days').format('DD/MM/YYYY')
            };
            deferred.resolve(filterRow);
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    this.buildFilterFormBaixasReceber = function() {
        var deferred = ZHPromise.defer();
        var filterRow = {'IDCONTABILIZA': 'A'};
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.getDiasBaixaReceber().then(function(response) {
            filterRow.DTBAIXAPAG = {
                start: moment().subtract(response[0] ? response[0] : 0, 'days').format('DD/MM/YYYY'),
                period: undefined,
                end: moment().add(response[1] ? response[1] : 0, 'days').format('DD/MM/YYYY')
            };
            deferred.resolve(filterRow);
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    this.updateAvailableContcorrToFilter = function(widget) {

        var fieldContcorr = widget.getField('CDREDUCONT');

        fieldContcorr.readOnly = !widget.currentRow.CDEMPRESA;

        fieldContcorr.dataSourceFilter[1].value = widget.currentRow.CDEMPRESA;
        widget.currentRow.CDREDUCONT = [];
    };

    this.nossoNumeroUpdate = function(widget) {
        var nossoNumeroSelect = widget.currentRow.SLNOSNUMREC;
        var fieldsToBeChanged = widget.fields.filter(getFieldsToEnableOrDisable);
        widget.getField('CDNOSNUMREC').isVisible = !!nossoNumeroSelect;

        fieldsToBeChanged.forEach(function(field) {
            field.readOnly = !!nossoNumeroSelect;
            widget.currentRow[field.name] = null;
            if (!nossoNumeroSelect) {
                if (field.defaultValue != undefined)
                    widget.currentRow[field.name] = field.defaultValue;
            }
        });

        if (!nossoNumeroSelect)
            widget.currentRow.CDNOSNUMREC = null;
        self.updateAvailableContcorrToFilter(widget);

    };

    var getFieldsToEnableOrDisable = function(field) {
        return field.name != "SLNOSNUMREC" && field.name != "NMRAZSOCEMPR" && field.name != "CDNOSNUMREC" && field.name != "IDCONTABILIZA";
    };

    this.formatFilter = function(widget) {

        widget.dataSourceFilter = Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.buildMainGridDataSourceFilter(widget);
        widget.activate();
    };

    this.loadAjustes = function(widget) {
        var currentConta = widget.currentRow;
        var widgetAjustes = widget.widgets[0];
        widgetAjustes.dataSourceFilter = [];
        widgetAjustes.dataSourceFilter = Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.buildAjustesDataSourceFilterFromRow(currentConta, widgetAjustes.dataSourceFilter);
        widgetAjustes.reload();
    };


    this.validateCheckedRow = function(widget) {
        var checkedRow = widget.selectedRow;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRow(checkedRow).then().catch(function(error) {

            if (error.data.messages[0].variables.CODE == FORA_DO_PERIODO_CODE) {
                ScreenService.showMessage("Lançamento fora do período contábil: __DTINICCTBL__ a __DTFINACCTBL__", null, null, error.data.messages[0].variables);
            }
            else {
                ScreenService.showMessage("Período contábil inexistente");
            }

            unselectRow(widget, checkedRow);

        });
    };

    this.validateCheckedRowBaixaPagar = function(widget) {
        var checkedRow = widget.selectedRow;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRowBaixaPagar(checkedRow).then().catch(function(error) {

            if (error.data.messages[0].variables.CODE == FORA_DO_PERIODO_CODE) {
                ScreenService.showMessage("Lançamento fora do período contábil: __DTINICCTBL__ a __DTFINACCTBL__", null, null, error.data.messages[0].variables);
            }
            else {
                ScreenService.showMessage("Período contábil inexistente");
            }

            unselectRow(widget, checkedRow);

        });
    };

    this.validateCheckedRowBaixaReceber = function(widget) {
        var checkedRow = widget.selectedRow;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRowBaixaReceber(checkedRow).then().catch(function(error) {

            if (error.data.messages[0].variables.CODE == FORA_DO_PERIODO_CODE) {
                ScreenService.showMessage("Lançamento fora do período contábil: __DTINICCTBL__ a __DTFINACCTBL__", null, null, error.data.messages[0].variables);
            }
            else {
                ScreenService.showMessage("Período contábil inexistente");
            }

            unselectRow(widget, checkedRow);

        });
    };

    this.validateCheckedRows = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRows(checkedRows).then(function(response) {
            unselectGivenRows(widget, response.dataset.rows_to_be_unchecked);
        });

    };

    this.validateCheckedRowsBaixasPagar = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRowsBaixasPagar(checkedRows).then(function(response) {
            unselectGivenRows(widget, response.dataset.rows_to_be_unchecked);
        });

    };

    this.validateCheckedRowsBaixasReceber = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;
        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.validateCheckedRowsBaixasReceber(checkedRows).then(function(response) {
            unselectGivenRows(widget, response.dataset.rows_to_be_unchecked);
        });

    };

    this.recontabilizarLancamentos = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.recontabilizarLancamentos(checkedRows).then(function(success) {
            ScreenService.showMessage("Recontabilizacao feita com sucesso.");
        }).catch(function(error) {
            if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os lançamentos devido a erros na contablização.")) {
                var message = "Timeout";
                if (error.data.error) {
                    message = error.data.error;
                }
                ScreenService.showMessage(message);
            }

        });

    };

    this.recontabilizarRecebimentos = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.recontabilizarRecebimentos(checkedRows).then(function(success) {
            ScreenService.showMessage("Recontabilizacao feita com sucesso.");
        }).catch(function(error) {
            if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os recebimentos devido a erros na contablização.")) {
                var message = "Timeout";
                if (error.data.error) {
                    message = error.data.error;
                }
                ScreenService.showMessage(message);
            }

        });

    };

    this.recontabilizarBaixasPagar = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.recontabilizarBaixasPagar(checkedRows).then(function(success) {
            ScreenService.showMessage("Recontabilizacao feita com sucesso.");
        }).catch(function(error) {
            if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os recebimentos devido a erros na contablização.")) {
                var message = "Timeout";
                if (error.data.error) {
                    message = error.data.error;
                }
                ScreenService.showMessage(message);
            }

        });

    };

    this.recontabilizarBaixasReceber = function(widget) {

        var checkedRows = widget.dataSource.checkedRows;

        Pag00800_RecalculoContabilizacaoTitulosBaixas_Service.recontabilizarBaixasReceber(checkedRows).then(function(success) {
            ScreenService.showMessage("Recontabilizacao feita com sucesso.");
        }).catch(function(error) {
            if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os recebimentos devido a erros na contablização.")) {
                var message = "Timeout";
                if (error.data.error) {
                    message = error.data.error;
                }
                ScreenService.showMessage(message);
            }

        });

    };

    var unselectRow = function(widget, row) {
        var index = widget.getCheckedRows().indexOf(row);
        widget.dataSource.checkedRows.splice(index, 1);
        row.__isSelected = false;
    };

    var unselectGivenRows = function(widget, rows) {

        rows.forEach(function(row) {

            var removedRow;

            widget.dataSource.checkedRows.forEach(function(checkedRow, index, checkedRows) {
                if (checkedRow.CDFILIAL === row.CDFILIAL && checkedRow.NRLANCTONF === row.NRLANCTONF) {
                    removedRow = checkedRows.splice(index, 1)[0];
                    removedRow.__isSelected = false;
                }
            });


        });


    };



};

Configuration(function(ContextRegister) {
    ContextRegister.register('Pag00800_RecalculoContabilizacaoTitulosBaixas_Controller', Pag00800_RecalculoContabilizacaoTitulosBaixas_Controller);
});

// FILE: ./assets/js/controllers/dashboard.js
var dashboard = function(templateManager, ScreenService, ZHPromise, General)
{

     /**
     * Variaveis de Inicializacao
     */
    var self = this;
    
    var dataSourceA = [];
    var dataSourceB = [];
    var dataSourceC = [];
    
    self.setMaskContaCorr = function(widget) {
	    var data = widget.dataSource.data;
	    var contFormatada;
	    data.forEach(function(value){
	        var tamanho = value.CDPICTCONT.split("-")[0].length;
            value.CDCONTCORR = value.CDCONTCORR.substring(0, tamanho)+"-"+value.CDCONTCORR.substring(tamanho,value.CDCONTCORR.length+1);
	    });
	};

    /**
     * Define Mascara de CNPJ/CPF de Cliente
     */ 
    self.setMaskCnpjCpfCliente = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURCLIE.length === 14) inscFormatada = value.NRINSJURCLIE.substr(0, 2) + '.' + value.NRINSJURCLIE.substr(2, 3) + '.' + value.NRINSJURCLIE.substr(5, 3) + '/' + value.NRINSJURCLIE.substr(8, 4) + '-' + value.NRINSJURCLIE.substr(12, 2);
            else if(value.NRINSJURCLIE.length === 11) inscFormatada = value.NRINSJURCLIE.substr(0, 3) + '.' + value.NRINSJURCLIE.substr(3, 3) + '.' + value.NRINSJURCLIE.substr(6, 3) + '-' + value.NRINSJURCLIE.substr(9, 2);
            value.NRINSJURCLIE = inscFormatada?inscFormatada:value.NRINSJURCLIE;
	    });
	};
	
	/**
     * Define Mascara de CNPJ/CPF de Fornecedor
     */
    self.setMaskCnpjCpfForneced = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURFORN.length === 14) inscFormatada = value.NRINSJURFORN.substr(0, 2) + '.' + value.NRINSJURFORN.substr(2, 3) + '.' + value.NRINSJURFORN.substr(5, 3) + '/' + value.NRINSJURFORN.substr(8, 4) + '-' + value.NRINSJURFORN.substr(12, 2);
            else if(value.NRINSJURFORN.length === 11) inscFormatada = value.NRINSJURFORN.substr(0, 3) + '.' + value.NRINSJURFORN.substr(3, 3) + '.' + value.NRINSJURFORN.substr(6, 3) + '-' + value.NRINSJURFORN.substr(9, 2);
            value.NRINSJURFORN = inscFormatada?inscFormatada:value.NRINSJURFORN;
	    });
	};
	
	self.setFilter = function(cdempresa, data){
	    templateManager.container.getWidget('widgetTitulosReceber').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	        // Passar para os graficos de Divisão de Receitas e Divisão de Despesas
	        // a data do stripe como pameratro 
	        if(filter.name =='P_DTVENBOLRECINI'){
	            filter.value = data.start;
	        }
            if(filter.name =='P_DTVENBOLRECFIN'){
                filter.value = data.end;
            }
            if(filter.name =='P_TDTVENBOLREC'){
                filter.value = "";
            }
	    });
        templateManager.container.getWidget('widgetTitulosPagar').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	        // Passar para os graficos de Divisão de Receitas e Divisão de Despesas
    	    // a data do stripe como pameratro 
    	    if(filter.name =='P_DTVENBOLPAGINI'){
	            filter.value = data.start;
	        }
            if(filter.name =='P_DTVENBOLPAGFIN'){
                filter.value = data.end;
            }
            if(filter.name =='P_TDTVENBOLPAG'){
                filter.value = "";
            }
	    });
	    
	    // Passar para os graficos de Divisão de Receitas e Divisão de Despesas
	    // a data do stripe como pameratro 
	    templateManager.container.getWidget('DONUTRECEITA').dataSourceFilter.forEach(function(filter){
	        if(filter.name =='P_DTVENBOLRECINI'){
	            filter.value = data.start;
	        }
            if(filter.name =='P_DTVENBOLRECFIN'){
                filter.value = data.end;
            }
            if(filter.name =='P_TDTVENBOLREC'){
                filter.value = "";
            }
            if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	        
	    });
        templateManager.container.getWidget('DONUTDESPESAS').dataSourceFilter.forEach(function(filter){
	        if(filter.name =='P_DTVENBOLPAGINI'){
	            filter.value = data.start;
	        }
            if(filter.name =='P_DTVENBOLPAGFIN'){
                filter.value = data.end;
            }
            if(filter.name =='P_TDTVENBOLPAG'){
                filter.value = "";
            }
            if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	    });
        
        templateManager.container.getWidget('clientInadimplenteWidget').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	    });
        templateManager.container.getWidget('clientInadimplenteChart').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	    });
        templateManager.container.getWidget('donutClientInadimplente').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'P_CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	    });
        templateManager.container.getWidget('curvaABCWidget').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'CODEMPRESA'){
	            filter.value = cdempresa;
	        }
	        if(filter.name == 'DATVENIN'){
	            filter.value = data.start;
	        }
	        if(filter.name == 'DATVENFIN'){
	            filter.value = data.end;
	        }
	    });
        templateManager.container.getWidget('curvaABCChart').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'CODEMPRESA'){
	            filter.value = cdempresa;
	        }
	        // Passar para os graficos a data do stripe como pameratro 
	        /*if(filter.name == 'DATVENIN'){
	            filter.value = data.start;
	        }
	        if(filter.name == 'DATVENFIN'){
	            filter.value = data.end;
	        }*/
	    });
        templateManager.container.getWidget('saldoWidget').dataSourceFilter.forEach(function(filter){
	        if(filter.name == 'CDEMPRESA'){
	            filter.value = cdempresa;
	        }
	        if(filter.name == 'DATA'){
	            var stripeWidget = templateManager.container.getWidget('stripe');
	            var data = stripeWidget.currentRow.DTVENBOLREC.split(' até ');
	            if(data[1]){
	                filter.value = data[1];
	            } else {
	                filter.value = moment().format('DD/MM/YYYY');
	            }
	        }
	    });
	    
	    var filterEmpresa = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": cdempresa
            }
        ];
        
	    General.callRestEngine('fin#/crud/empresa', 'FilterData', filterEmpresa).then(function (empresa) {
    	    templateManager.container.getWidget('donutClientInadimplente').labelInfo =
    	    "Clientes Inadimplentes Empresa: "+ empresa.dataset.empresa[0].NMEMPRESA;
            //templateManager.container.getWidget('clientInadimplenteChart').labelInfo =
    	    //"Clientes Inadimplentes Empresa: "+ empresa.dataset.empresa[0].NMEMPRESA;
    	    //templateManager.container.getWidget('curvaABCChart').labelInfo =
            //"Curva ABC Empresa: "+ empresa.dataset.empresa[0].NMEMPRESA;
    	    templateManager.container.getWidget('donutCurvaABC').labelInfo =
            "Curva A Empresa: "+ empresa.dataset.empresa[0].NMEMPRESA;
        });
        
        templateManager.container.getWidget('widgetTitulosReceber').reload();
        templateManager.container.getWidget('widgetTitulosPagar').reload();
        templateManager.container.getWidget('DONUTRECEITA').reload();
        templateManager.container.getWidget('DONUTDESPESAS').reload();
	    templateManager.container.getWidget('clientInadimplenteWidget').reload();
        templateManager.container.getWidget('clientInadimplenteChart').reload();
        templateManager.container.getWidget('donutClientInadimplente').reload();
        templateManager.container.getWidget('curvaABCWidget').reload();
        templateManager.container.getWidget('curvaABCChart').reload();
        templateManager.container.getWidget('saldoWidget').reload();
	};
	
	self.setABCcurve = function(widget){
        // Função comentada foi feita para alimentar o ultimo grafico do dashboard
        // foi mantida apenas como referencia para futuras necessidades
        //var curvaABCGraph = templateManager.container.getWidget('curvaABCGraph');
        var widgetDonutCurv = templateManager.container.getWidget('donutCurvaABC');
	    dataSourceA = [];
	    dataSourceB = [];
	    dataSourceC = [];
	    if(widget.dataSource.data.length > 0){
    	    self.setMaskCnpjCpfCliente(widget);
    	    var percentual = 0;
    	    //curvaABCGraph.dataSource.data = [];
    	    widget.dataSource.data.forEach(function(data){
    	        if(percentual < data.CURVAA){
    	            percentual = percentual + data.PERC;
    	            data.CURVA = 'A';
    	            dataSourceA.push(data);
    	            //curvaABCGraph.dataSource.data.push(data);
    	        } else if((percentual > data.CURVAA) || (percentual < data.CURVAB)){
    	            percentual = percentual + data.PERC;
    	            data.CURVA = 'B';
    	            dataSourceB.push(data);
    	            //curvaABCGraph.dataSource.data.push(data);
    	        } else if((percentual > data.CURVAA) || (percentual > data.CURVAB)){
    	            data.CURVA = 'C';
    	            dataSourceC.push(data);
    	            //curvaABCGraph.dataSource.data.push(data);
    	        }
    	    });
            /*var vrtotal = [];
            var nmRazao = [];
            var dataSourceFin = [];
            var name;
            var value;
            var element;
            var objetoA = {"CURVA" : "A"};
            var objetoB = {"CURVA" : "B"};
            var objetoC = {"CURVA" : "C"};
            curvaABCGraph.dataSource.data.forEach(function(data){
    	        if(data.CURVA == "A"){
                    name = data.NMRAZSOCCLIE.replace(/ /g, "_");
                    value = data.VRTOTCLIE;
	                element = {};
    	            element[name] = value; 
    	            vrtotal.push(name);
    	            nmRazao.push(data.NMRAZSOCCLIE);
    	            Object.assign(objetoA, element);
    	        }
    	        else if(data.CURVA == "B"){
    	            name = data.NMRAZSOCCLIE.replace(/ /g, "_");
                    value = data.VRTOTCLIE;
	                element = {};
    	            element[name] = value; 
    	            vrtotal.push(name);
    	            nmRazao.push(data.NMRAZSOCCLIE);
    	            Object.assign(objetoB, element);
    	        }
    	        else if(data.CURVA == "C"){
    	            name = data.NMRAZSOCCLIE.replace(/ /g, "_");
                    value = data.VRTOTCLIE;
	                element = {};
    	            element[name] = value; 
    	            vrtotal.push(name);
    	            nmRazao.push(data.NMRAZSOCCLIE);
    	            Object.assign(objetoC, element);
    	        }
    	    });
    	    dataSourceFin.push(objetoA);
    	    dataSourceFin.push(objetoB);
    	    dataSourceFin.push(objetoC);
    	    curvaABCGraph.dataSource.data = dataSourceFin;
    	    curvaABCGraph.chartSeries[0].serieFields = vrtotal;
    	    curvaABCGraph.chartSeries[0].serieLabels = nmRazao;
            curvaABCGraph.reload();
            curvaABCGraph.updateChart();*/
    	    widgetDonutCurv.dataSource.data = dataSourceA;
    	    widgetDonutCurv.reload();
    	}
	};
	
	self.setStripeData = function(widget) {
        var stripeWidget = templateManager.container.getWidget("stripe");
        
        var filterFilial = [
            {
                "name": "CDFILIAL",
                "operator": "=",
                "value": "FILIAL_LOGADA"
            }
        ];
        
        General.callRestEngine('fin#/crud/filial', 'FilterData', filterFilial).then(function (filial) {
            var filterEmpresa = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": filial.dataset.filial[0].CDEMPRESA
                }
            ];
            General.callRestEngine('fin#/crud/empresa', 'FilterData', filterEmpresa).then(function (empresa) {
                stripeWidget.currentRow = {};
                stripeWidget.currentRow.NMEMPRESA   = empresa.dataset.empresa[0].CDEMPRESA + ' - ' + empresa.dataset.empresa[0].NMEMPRESA;
                stripeWidget.currentRow.DTVENBOLREC = 'De ' + moment().subtract(1, 'months').format('DD/MM/YYYY') + ' até ' + moment().format('DD/MM/YYYY');
                var data = {
                    "start": moment().subtract(1, 'months').format('DD/MM/YYYY'),
                    "end": moment().format('DD/MM/YYYY')
                };
                templateManager.container.getWidget('widDashboard').active = true;
                templateManager.container.getWidget('widDashboard').activate();
                self.setFilter(empresa.dataset.empresa[0].CDEMPRESA, data);
            });
        });
        
    };

	self.openTilutoRecClientInadim = function(){
	    var widget = templateManager.container.getWidget('clientInadimplenteWidget');
        
        var dataMax = widget.dataSource.data[0].DTVENBOLRECMAX, 
        dataMin = widget.dataSource.data[0].DTVENBOLRECMIN,
        date1, date2;
        widget.dataSource.data.forEach(function(element){
            if(element.DTVENBOLRECMAX){
                date1 = new Date(moment(element.DTVENBOLRECMAX, "DD/MM/YYYY").format("MM/DD/YYYY"));
                date2 = new Date(moment(dataMax, "DD/MM/YYYY").format("MM/DD/YYYY"));
                if(date1.getTime() > date2.getTime()){
                    dataMax = element.DTVENBOLRECMAX;
                }
            }
            if(element.DTVENBOLRECMIN){
                date1 = new Date(moment(element.DTVENBOLRECMIN, "DD/MM/YYYY").format("MM/DD/YYYY"));
                date2 = new Date(moment(dataMin, "DD/MM/YYYY").format("MM/DD/YYYY"));
                if(date1.getTime() < date2.getTime()){
                    dataMin = element.DTVENBOLRECMIN;
                }
            }
        });
        
        sessionStorage.setItem('DTVENBOLREC.start', dataMin);    
        sessionStorage.setItem('DTVENBOLREC.end', dataMax);    
        sessionStorage.setItem('DASHBOARD_NAME', widget.name);
        sessionStorage.setItem('CDEMPRESA', widget.dataSource.data[0].CDEMPRESA);
        sessionStorage.setItem('NMEMPRESA', widget.dataSource.data[0].NMEMPRESA);
        ScreenService.openWindow('fin_contaReceber#rec00001_titulos_a_receber');
	};
	
	self.openTilutoRecClient = function(){
	    var widget = templateManager.container.getWidget('widgetTitulosReceber');
        
        var tamanho = widget.dataSource.data.length;
        
        sessionStorage.setItem('DTVENBOLREC.start', widget.dataSource.data[0].DTVENBOLREC);     
        sessionStorage.setItem('DTVENBOLREC.end', widget.dataSource.data[(tamanho - 1)].DTVENBOLREC);    
        sessionStorage.setItem('DASHBOARD_NAME', widget.name);
        sessionStorage.setItem('CDEMPRESA', widget.dataSource.data[0].CDEMPRESA);
        sessionStorage.setItem('NMEMPRESA', widget.dataSource.data[0].NMEMPRESA);
        ScreenService.openWindow('fin_contaReceber#rec00001_titulos_a_receber');
	};
	
	self.openTilutoPag = function(){
	    var widget = templateManager.container.getWidget('widgetTitulosPagar');
        
        var tamanho = widget.dataSource.data.length;
        
        sessionStorage.setItem('DTVENBOLPAG.start', widget.dataSource.data[0].DTVENBOLPAG); 
        sessionStorage.setItem('DTVENBOLPAG.end', widget.dataSource.data[(tamanho - 1)].DTVENBOLPAG);
        sessionStorage.setItem('DASHBOARD_NAME', widget.name);
        sessionStorage.setItem('CDEMPRESA', widget.dataSource.data[0].CDEMPRESA);
        sessionStorage.setItem('NMEMPRESA', widget.dataSource.data[0].NMEMPRESA);
        ScreenService.openWindow('fin_contaPagar#pag00001_titulos_a_pagar');
	};
	
	self.changeStripe = function(){
	    var widget = templateManager.container.getWidget('popUpStripe');
	    ScreenService.openPopup(widget);
	    var arrayEmpresa = templateManager.container.getWidget('stripe').currentRow.NMEMPRESA.split(" - ");
	    var arrayVencimento = templateManager.container.getWidget('stripe').currentRow.DTVENBOLREC.split(" ");
	    widget.currentRow.CDEMPRESA = arrayEmpresa[0];
	    widget.currentRow.NMEMPRESA = arrayEmpresa[1];
	    widget.currentRow.DTVENBOLREC = {
	        "start": arrayVencimento[1],
	        "end": arrayVencimento[3]
	    };
	    widget.reload();
	};
	
	self.comfirmChangeStripe = function(widget){
	    var widgetStripe = templateManager.container.getWidget('stripe');
        widgetStripe.currentRow = {};
        widgetStripe.currentRow.NMEMPRESA   = widget.currentRow.CDEMPRESA + ' - ' + widget.currentRow.NMEMPRESA;
        widgetStripe.currentRow.DTVENBOLREC = 'De ' + widget.currentRow.DTVENBOLREC.start + ' até ' + widget.currentRow.DTVENBOLREC.end;
        ScreenService.closePopup();
        self.setFilter(widget.currentRow.CDEMPRESA, widget.currentRow.DTVENBOLREC);
        templateManager.container.getWidget('widDashboard').active = true;
        templateManager.container.getWidget('widDashboard').activate();
	};
	
	this.changeCurv = function(widget){
	    if(widget.tipoVisualizacao == "A"){
	        if(dataSourceB.length > 0){
    	        widget.dataSource.data = dataSourceB;
                widget.label = "Curva B";
                widget.tipoVisualizacao = "B";
                widget.labelInfo = widget.labelInfo.replace("Curva A","Curva B");
                widget.reload();
                widget.updateChart();
	        }
	    }
	    else if(widget.tipoVisualizacao == "B"){
	        if(dataSourceC.length > 0){
    	        widget.dataSource.data = dataSourceC;
    	        widget.label = "Curva C";
    	        widget.tipoVisualizacao = "C";
                widget.labelInfo.replace("Curva B","Curva C");
                widget.reload();
    	        widget.updateChart();
	        } else {
	            widget.tipoVisualizacao = "C";
                widget.labelInfo = widget.labelInfo.replace("Curva B","Curva C");
	            this.changeCurv(widget);
	        }
	    }
	    else if(widget.tipoVisualizacao == "C"){
	            widget.dataSource.data = dataSourceA;
    	        widget.label = "Curva A";
    	        widget.tipoVisualizacao = "A";
                widget.labelInfo = widget.labelInfo.replace("Curva C","Curva A");
                widget.reload();
    	        widget.updateChart();
	    }
	};
};

/* global Configuration */
Configuration(function (ContextRegister){
    ContextRegister.register('dashboard', dashboard);
});

// FILE: ./assets/js/controllers/zhFavorites.js
var ZeedhiFields = angular.module("ZeedhiFields");

ZeedhiFields.directive('zhNavbarDefaultHeader', function(templateManager, ScreenService, eventAggregator, $compile, ApplicationContext) {
	return {
		restrict: 'C',
		link: function($scope, element) {
		    $scope.favoritesList = [];
		    $scope.lastDeletedItem = undefined;
		    
		    var favoritesText = 'Favoritos';
		    var favoritesAddText = 'Adicionar aos Favoritos';
		    var favoritesRemoveText = 'Remover dos Favoritos';
		    var favoritesUndoText = 'DESFAZER';

			// TRANSLATE - Adicionar essas variaveis a lista da aplicação (words.jon)
		    // var favoritesText = ScreenService.i18n('FAVORITES');
		    // var favoritesAddText = ScreenService.i18n('ADD_TO_FAVORITES');
		    // var favoritesRemoveText = ScreenService.i18n('REMOVE_FROM_FAVORITES');
		    // var favoritesUndoText = ScreenService.i18n('UNDO');
		    
		    var favoritesContainer;
		    
		    var startHollowCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M15,4.21l3.51,7.1l7.84,1.14l-5.67,5.53l1.34,7.81L15,22.1l-7.01,3.69l1.34-7.81l-5.67-5.53l7.84-1.14L15,4.21'+
				'			 M15,2.21c-0.76,0-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05'+
				'			l4.92,4.8l-1.16,6.77c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			c0.55-0.53,0.74-1.33,0.51-2.05c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21L15,2.21z"/>'+
				'	</g>'+
				'</svg>';
		    var startFilledCode = '<svg version="1.1" id="Layer_1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 30 30" style="enable-background:new 0 0 30 30;" xml:space="preserve">'+
				'	<style type="text/css">'+
				'		.st0{fill:#FFFFFF;}'+
				'	</style>'+
				'	<g>'+
				'		<path class="st0" d="M28.25,11.83c-0.24-0.72-0.86-1.25-1.61-1.36l-6.8-0.99l-3.04-6.16C16.46,2.64,15.76,2.21,15,2.21'+
				'			s-1.46,0.43-1.79,1.11l-3.04,6.16l-6.8,0.99c-0.75,0.11-1.38,0.64-1.61,1.36c-0.24,0.72-0.04,1.52,0.51,2.05l4.92,4.8l-1.16,6.77'+
				'			c-0.13,0.75,0.18,1.51,0.8,1.96c0.35,0.25,0.76,0.38,1.18,0.38c0.32,0,0.64-0.08,0.93-0.23l6.08-3.2l6.08,3.2'+
				'			c0.29,0.15,0.61,0.23,0.93,0.23c0.41,0,0.83-0.13,1.18-0.38c0.62-0.45,0.92-1.21,0.8-1.96l-1.16-6.77l4.92-4.8'+
				'			C28.29,13.35,28.48,12.56,28.25,11.83z"/>'+
				'	</g>'+
				'</svg>';
		    
		    function createVisualElements() {
    			var headerContainer = $(element).children()[0];
    			headerContainer.style.width = 'calc(100% - 50px)';
    			
    			favoritesContainer = $('<div class="zh-favorites"></div>').appendTo(element);
    			
    			var favoritesButtonElement = $compile(
    				'<div class="zh-favorites-button" ng-click="toggleList()">'+
    				'  <div class="zh-favorites-button-star" ng-if="!isFavorited">'+
    				'    '+startHollowCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-star" ng-if="isFavorited">'+
    				'    '+startFilledCode+
    				'  </div>'+
    				'  <div class="zh-favorites-button-text">'+favoritesText+'</div>'+
    				'</div>'
    			)($scope);
    			var favoritesButton = $(favoritesButtonElement).appendTo(favoritesContainer);
    			
    			var favoritesListElement = $compile(
    				'<div class="zh-favorites-list" ng-class="{\'opened\': opened}">'+
    				'  <div class="zh-favorites-list-button" ng-if="!isFavorited" ng-click="addToFavorites()">'+favoritesAddText+'</div>'+
    				'  <div class="zh-favorites-list-button" ng-if="isFavorited" ng-click="removeFromFavorites()">'+favoritesRemoveText+'</div>'+
    				'  <div class="zh-favorites-list-undo" ng-show="lastDeletedItem" ng-click="undo()">'+favoritesUndoText+'</div>'+
    				'  <div class="zh-favorites-list-items" ng-show="favoritesList.length > 0">'+
    				'	 <div class="zh-favorites-list-item" ng-repeat="item in favoritesList track by item.id" ng-click="openItem(item)">'+
    				'      {{ item.label }}'+
    				'      <span class="zh-favorites-list-item-delete" data-zh-icon="trash icon-white no-border" ng-click="deleteItem(item, $index, $event)"></span>'+
    				'    </div>'+
    				'  </div>'+
    				'</div>'
    			)($scope);
    			var favoritesList = $(favoritesListElement).appendTo(favoritesContainer);
		    }
		    
		    function checkIsFavorited(containerId) {
		    	var oldIsFavorited = $scope.isFavorited;
			    $scope.isFavorited = $scope.favoritesList.findIndex((item) => item.id === containerId) !== -1;
			    if (oldIsFavorited !== $scope.isFavorited) {
			    	templateManager.updateTemplate();
			    }
		    }
		    
		    $scope.addToFavorites = function() {
		    	var item = {
		    		id: templateManager.container.id,
		    		name: templateManager.container.fileName.split('.json')[0],
		    		label: templateManager.container.label,
		    	};
		    	$scope.favoritesList.push(item);
		    	$scope.isFavorited = true;
		    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'add' }});
		    };
		    
		    $scope.removeFromFavorites = function() {
		    	var index = $scope.favoritesList.findIndex((item) => item.id === templateManager.container.id);
		    	if (index > -1) {
			    	var item = $scope.favoritesList.splice(index, 1);
			    	$scope.isFavorited = false;
			    	eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});
		    	}
		    };
		    
		    $scope.openItem = function(item) {
		    	ApplicationContext.LibUtilities.openWindow(item.name);
	    		$scope.opened = false;
		    };
		    
		    var undoTimeout;
		    
		    $scope.deleteItem = function(item, index, event) {
		    	$scope.favoritesList.splice(index, 1);
		    	$scope.lastDeletedItem = { item, index };

		    	clearTimeout(undoTimeout);
		    	undoTimeout = setTimeout(() => {
		    		$scope.lastDeletedItem = undefined;
		    		templateManager.updateTemplate();
		    	}, 10000);

			    eventAggregator.publish('ZhFavoritesOnChange', { fullList: $scope.favoritesList, changed: { item, status: 'delete' }});

		    	if (item.id === templateManager.container.id) {
		    		$scope.isFavorited = false;
		    	}

		    	event.stopImmediatePropagation();
		    };
		    
		    $scope.undo = function() {
		    	if ($scope.lastDeletedItem) {
		    		$scope.favoritesList.splice($scope.lastDeletedItem.index, 0, $scope.lastDeletedItem.item);
		    	}
		    	$scope.lastDeletedItem = undefined;
		    };

		    $scope.toggleList = function() {
		    	$scope.opened = !$scope.opened;
		    };
		    
		    document.addEventListener('mousedown', (evt) => {
		    	if ($(favoritesContainer).find(evt.target).length === 0 && $scope.opened) {
		    		$scope.opened = false;
			    	templateManager.updateTemplate();
		    	}	
		    });
		    
		    eventAggregator.subscribe(eventAggregator.ON_CONTAINER_OPEN, (args) => {
		    	checkIsFavorited(args.container.id);
		    });
		    
		    eventAggregator.subscribe('ZhFavoritesOnLoad', (favorites) => {
		    	
		    	$scope.favoritesList = favorites;
			    if (templateManager.container) {
			    	checkIsFavorited(templateManager.container.id);
			    }
		    });

		    createVisualElements();
		}
	};
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Ger37000_TipoBaixaPagar_Controller.js
var Ger37000_TipoBaixaPagar_Controller = function(ScreenService, General){
    var i = 0;

    this.novoTipoBaixaPagar = function(widget) {
        widget.newRow();
        widget.getField('DSTPCREDBANC').readOnly = true;
    };

    this.validacoes = function(widget) {
        var geraEstorno = widget.getField('IDESTOREXPAG');
        var tipoCredito = widget.getField('DSTPCREDBANC');
        if (geraEstorno.value() == "N") {
            this.modificaReadOnly(tipoCredito);
        } else {
            tipoCredito.readOnly = false;
        }
    };
    
    this.validacoesSelect = function(widget){
        var operacao = widget.getField('IDBAIXAUTILP');
        var habilitaValorMax = ['B', 'C', 'D', 'T', 'S', 'O'];
        var valorMax = widget.getField('VRMAXDOCCNAB');
        if (habilitaValorMax.indexOf(operacao.value()) !== -1){
            this.modificaReadOnly(valorMax);
        } else {
            valorMax.readOnly = true;
            valorMax.setValue(0);
            
        }
    };

    this.modificaReadOnly = function(field){
    	if (field.name == "DSTPCREDBANC") {
        	field.readOnly = true;
         	field.setValue(null);
      	} else {
        	field.readOnly = false;
      	}
    };

};
Configuration(function (ContextRegister){
    ContextRegister.register('Ger37000_TipoBaixaPagar_Controller', Ger37000_TipoBaixaPagar_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag00001_TitulosAPagar_Controller.js
var Pag00001_TitulosAPagar_Controller = function(templateManager, ScreenService, ZHPromise, General, contabilizacaoController, Pag00001_TitulosAPagar_Service, ZhFilterPreferences, localStorageEngine, metaDataFactory, ValidationEngine)
{
    /**
     * Variáveis de Inicialização
     **/
    var self = this;
    var idhabibole_geral       = null;
    var idutilpadrbolhab_geral = null;
    
    var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
    /** [Início]Título a Pagar/Pagos[Filtro] */

    /**
     * Force Activate
     */
    self.forceActivate = function(){
        templateManager.container.getWidget('TitulosAPagarGrid').activate();
    };

    /**
     * Preenche Valor Final
     * @param widget
     * @param inicial
     * @param final
     */
    self.preencheValorFinal = function(widget, inicial, final){
        var fieldInicial = widget.getField(inicial);
        var fieldFinal = widget.getField(final);
        if ((!fieldFinal.value()) || (parseInt(fieldInicial.value()) > parseInt(fieldFinal.value()))){
            fieldFinal.setValue(fieldInicial.value());
        }
    };

    /**
     * Filtro
     * @param widget
     */
    self.filtroCnabRetorno = function(widget, preference){
        var currentRowFiltro = preference.FILTER_ROW;

        var vrbaixapagini     = currentRowFiltro.VRBAIXAPAGINI;
        var vrbaixapagfin     = currentRowFiltro.VRBAIXAPAGFIN;
        var vrtotalcnabpagini = currentRowFiltro.VRTOTALCNABPAGINI;
        var vrtotalcnabpagfin = currentRowFiltro.VRTOTALCNABPAGFIN;
        var nrnotafiscini     = currentRowFiltro.NRNOTAFISCINI;
        var nrnotafiscfin     = currentRowFiltro.NRNOTAFISCFIN;
        var vrfinalbpagini    = currentRowFiltro.VRFINALBPAGINI;
        var vrfinalbpagfin    = currentRowFiltro.VRFINALBPAGFIN;
        var vrbolpagini       = currentRowFiltro.VRBOLPAGINI;
        var vrbolpagfin       = currentRowFiltro.VRBOLPAGFIN;
        var nrcnabpagremini   = currentRowFiltro.NRCNABPAGREMINI;
        var nrcnabpagremfin   = currentRowFiltro.NRCNABPAGREMFIN;
        var nrcnabpagretini   = currentRowFiltro.NRCNABPAGRETINI;
        var nrcnabpagretfin   = currentRowFiltro.NRCNABPAGRETFIN;
        var cdreducont        = currentRowFiltro.CDREDUCONT;

        widget.dataSourceFilter.forEach(function (filtro) {
            switch(filtro.name){
                case 'VRBAIXAPAGINI': filtro.value = vrbaixapagini <= 0 ? null : vrbaixapagini;
                    break;
                case 'VRBAIXAPAGFIN': filtro.value = vrbaixapagfin <= 0 ? null : vrbaixapagfin;
                    break;
                case 'VRBAIXAPAGTOT': filtro.value = ((vrbaixapagini == 0) && (vrbaixapagfin == 0)) || (!vrbaixapagini && !vrbaixapagfin) ? 'T' : null;
                    break;
                case 'VRTOTALCNABPAGINI': filtro.value = vrtotalcnabpagini <= 0 ? null : vrtotalcnabpagini;
                    break;
                case 'VRTOTALCNABPAGFIN': filtro.value = vrtotalcnabpagfin <= 0 ? null : vrtotalcnabpagfin;
                    break;
                case 'VRTOTALCNABPAGTOT': filtro.value = ((vrtotalcnabpagini == 0) && (vrtotalcnabpagfin == 0)) || (!vrtotalcnabpagini && !vrtotalcnabpagfin) ? 'T' : null;
                    break;
                case 'NRNOTAFISCINI': filtro.value = nrnotafiscini;
                    break;
                case 'NRNOTAFISCFIN': filtro.value = nrnotafiscfin;
                    break;
                case 'NRNOTAFISCTOT': filtro.value = (nrnotafiscini && nrnotafiscfin) ? null : 'T';
                    break;
                case 'VRFINALBPAGINI': filtro.value = vrfinalbpagini <= 0 ? null : vrfinalbpagini;
                    break;
                case 'VRFINALBPAGFIN': filtro.value = vrfinalbpagfin <= 0 ? null : vrfinalbpagfin;
                    break;
                case 'VRFINALBPAGTOT': filtro.value = ((vrfinalbpagini == 0) && (vrfinalbpagfin == 0)) || (!vrfinalbpagini && !vrfinalbpagfin) ? 'T' : null;
                    break;
                case 'VRBOLPAGINI': filtro.value = vrbolpagini <= 0 ? null : vrbolpagini;
                    break;
                case 'VRBOLPAGFIN': filtro.value = vrbolpagfin <= 0 ? null : vrbolpagfin;
                    break;
                case 'VRBOLPAGTOT': filtro.value = ((vrbolpagini == 0) && (vrbolpagfin == 0)) || (!vrbolpagini && !vrbolpagfin) ? 'T' : null;
                    break;
                case 'NRCNABPAGREMINI': filtro.value = nrcnabpagremini;
                    break;
                case 'NRCNABPAGREMFIN': filtro.value = nrcnabpagremfin;
                    break;
                case 'NRCNABPAGREMTOT': filtro.value = (nrcnabpagremini && nrcnabpagremfin) ? null : 'T';
                    break;
                case 'NRCNABPAGRETINI': filtro.value = nrcnabpagretini;
                    break;
                case 'NRCNABPAGRETFIN': filtro.value = nrcnabpagretfin;
                    break;
                case 'NRCNABPAGRETTOT': filtro.value = (nrcnabpagretini && nrcnabpagretfin) ? null : 'T';
                    break;
                case 'CDREDUCONT': filtro.value = cdreducont;
                    break;
                case 'TCDREDUCONT': filtro.value = (cdreducont) ? null : 'T';
                    break;
            }
        });
        
    };
    
    self.cloneFilter = function (widgetOrigem_name, widgetDestino_name){
        var widgetOrigem  = templateManager.container.getWidget(widgetOrigem_name);
        var widgetDestino = templateManager.container.getWidget(widgetDestino_name);
        
        widgetDestino.appliedFilter = widgetDestino.appliedFilter == undefined ? [] : widgetDestino.appliedFilter;
        widgetDestino.appliedFilter.FILTER_ROW = widgetOrigem.appliedFilter.FILTER_ROW;
        
        widgetDestino._filterInfo = widgetOrigem._filterInfo;
    };
    
    self.cloneData = function (widgetOrigem_name, widgetDestino_name){
        var widgetOrigem  = templateManager.container.getWidget(widgetOrigem_name);
        var widgetDestino = templateManager.container.getWidget(widgetDestino_name);
        
        widgetDestino.dataSourceFilter         = widgetOrigem.dataSourceFilter;
        widgetDestino.dataSource.data          = widgetOrigem.dataSource.data;
        
        self.buildDataSourceKanban();
    };

	/**
     * Busca Empresa
     * @return empresa
     */
    self.buscaEmpresa = function (cdempresa){
        return Pag00001_TitulosAPagar_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Parametro por Filial - Filial - Empresa Controla Filial
     * @return paramfilial_filial_emprcntrfil
     */
    self.buscaParamfilialFilialEmprcntrfil = function (cdempresa, cdfilial){
        return Pag00001_TitulosAPagar_Service.buscaParamfilialFilialEmprcntrfil(cdempresa, cdfilial).then(function(paramfilial_filial_emprcntrfil) {
            return paramfilial_filial_emprcntrfil;
        });
    };

    /**
     * Tipo de Operação
     * @return tipooperac
     */
    self.buscaTipooperac = function (cdtipooper){
        return Pag00001_TitulosAPagar_Service.buscaTipooperac(cdtipooper).then(function(tipooperac) {
            return tipooperac;
        });
    };

    /**
     * Série de Entrada
     * @return serieentr
     */
    self.buscaSerieentr = function (cdserieentr){
        return Pag00001_TitulosAPagar_Service.buscaSerieentr(cdserieentr).then(function(serieentr) {
            return serieentr;
        });
    };

    /**
     * Tipo de Conta a Pagar
     * @return cdtpctpagar
     */
    self.buscaTipoctpagar = function (cdtpctpagar){
        return Pag00001_TitulosAPagar_Service.buscaTipoctpagar(cdtpctpagar).then(function(tipoctpagar) {
            return tipoctpagar;
        });
    };

    /**
     * Busca Filial
     * @return filial
     */
    self.buscaFilial = function (){
        return Pag00001_TitulosAPagar_Service.buscaFilial().then(function(filial) {
            return filial;
        });
    };

    /**
     * Busca Parametrização Geral Pagar
     * @return paramGeral
     */
    self.buscaGeralPag = function (){
        return Pag00001_TitulosAPagar_Service.buscaGeralPag().then(function(geralpag) {
            return geralpag;
        });
    };

    /**
     * Busca Parâmetros de Estoque
     * @return dataProcessamento
     */
    self.buscaParamEstoque = function (){
        return Pag00001_TitulosAPagar_Service.buscaParamEstoque().then(function(paramestoque) {
            return paramestoque;
        });
    };

    /**
     * Busca Parâmetros de Contabilização
     * @return contOpeCtbl
     */
    self.buscaContOpeCtbl = function (){
        return Pag00001_TitulosAPagar_Service.buscaContOpeCtbl().then(function(paramestoque) {
            return paramestoque;
        });
    };
    
    self.openFilterTitulosAPagar = function(widgetTitulosAPagar){
        self.buscaDadosIniciaisFiltro(widgetTitulosAPagar).then(function(){
            var rowSetFilter = {};
            if(sessionStorage.getItem('DASHBOARD_NAME') == 'widgetTitulosPagar'){
                rowSetFilter = {
                    'DTVENBOLPAG': {
                        "start": sessionStorage.getItem('DTVENBOLPAG.start'),
                        "end": sessionStorage.getItem('DTVENBOLPAG.end'),
                    },    
                    'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                    'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                    'IDSTATBPAG': ["A"]
                };
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
            } else if((sessionStorage.getItem('DASHBOARD_NAME') == 'boletPagarVenc') || (sessionStorage.getItem('DASHBOARD_NAME') == 'boletPagarAberto') || (sessionStorage.getItem('DASHBOARD_NAME') == 'boletPagarBaixa')){
                rowSetFilter = {
                    'DTVENBOLPAG': {
                        "start": sessionStorage.getItem('DTVENBOLPAG.start'),
                        "end": sessionStorage.getItem('DTVENBOLPAG.end'),
                    },    
                    'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                    'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                    'IDSTATBPAG': getItem('IDSTATBPAG').split(",")
                };
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
            } else {
                rowSetFilter = {
                    'DTVENBOLPAG': widgetTitulosAPagar.DTVENBOLPAG,
                    'CDEMPRESA': widgetTitulosAPagar.empresa.CDEMPRESA,
                    'NMEMPRESA': widgetTitulosAPagar.empresa.NMEMPRESA
                };
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
            }
            
            if((sessionStorage.getItem('DASHBOARD_NAME') == 'rastreioContabil')){
                var fornecedor = [];
                fornecedor.push(sessionStorage.getItem('CDFORNECED'));
                if(sessionStorage.getItem('ORIGEM') == 'BAIXA_PAG'){
                    rowSetFilter = {   
                        'DTVENBOLPAG': {start : '', end : ''},
                        'DTBAIXAPAG': {start : sessionStorage.getItem('DTBAIXAPAG'), 
                                         end : sessionStorage.getItem('DTBAIXAPAG')},
                        'VRBAIXAPAGINI': sessionStorage.getItem('VRBAIXAPAGINI'),
                        'VRBAIXAPAGFIN': sessionStorage.getItem('VRBAIXAPAGFIN'),
                        'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                        'NMEMPRESA': sessionStorage.getItem('NMEMPRESA')
                    };
                }else{
                    rowSetFilter = {   
                        'DTVENBOLPAG': {start : '', end : ''},
                        'NRNOTAFISCINI': sessionStorage.getItem('NRNOTAFISCINI'),
                        'NRNOTAFISCFIN': sessionStorage.getItem('NRNOTAFISCFIN'),
                        'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                        'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                        'CDFORNECED': fornecedor
                    };
                }
                
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
            }
            
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAPagar) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAPagar).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAPagar).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };

    /**
     * Busca Padrão Empresa
     * @param widget
     */
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        self.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            self.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };

    /**
     * Busca Padrão Intervalo Vencimento
     * @returns {{start: *, end: *}}
     */
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralpag     = self.buscaGeralPag();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralpag, paramestoque]).then(function(response){
            geralpag     = response[0].dataset.geral_pag[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralpag?geralpag.QTDIAANTBOLP?geralpag.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralpag?geralpag.QTDIAAPOBOLP?geralpag.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLPAG = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };

    /** [Fim]Título a Pagar/Pagos[Filtro] */

    /** [Início]Título a Pagar/Pagos */

    /**
     * Novo Título a Pagar
     */
    self.novoTituloAPagar = function(){
        if(!templateManager.container.getWidget('TitulosAPagarGrid').appliedFilter.FILTER_ROW.CDEMPRESA) {
            ScreenService.showMessage('Operação bloqueada. Empresa não selecionada no filtro.');
            return;
        }
        
        General.callRestEngine('fin#/crud/geral_pag','FilterData', []).then(function(result){
            if(result.dataset.geral_pag.length == 0){
                ScreenService.showMessage('Operação bloqueada. Favor reveja as parametrizações na tela de Parâmetros de Títulos a Pagar.');
                return;
            }
            
            self.validaTipoContaEmpresa().then(function(){
                var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
                widgetDocumentosForm.isVisible = true;
                widgetDocumentosForm.editing = true;
                ScreenService.openSwipe(widgetDocumentosForm).then(function(){
                    templateManager.updateTemplate();
                    self.buscaPadraoTitulo();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                    return;
                });
            }, function (error) {
                ScreenService.showMessage('Operação bloqueada. Empresa selecionada no filtro não possui tipo de conta relacionada na tela Tipo de Título a Pagar.');
                return;
            });
            
            
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            return;
        });
        
    };
    
    self.validaTipoContaEmpresa = function(){
        var deferred = ZHPromise.defer();
        
        var filter = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": "T"
            }
        ];
        
        General.callRestEngine('fin#/crud/emprtpctpag_empresa','FilterData',filter).then(function(empresas){
            empresas = empresas.dataset.emprtpctpag_empresa;
            
            if(empresas.length > 0){
                var cdempresa = templateManager.container.getWidget('TitulosAPagarGrid').appliedFilter.FILTER_ROW.CDEMPRESA;
                
                filter = [
                    {
                        "name": "CDEMPRESA",
                        "operator": "=",
                        "value": cdempresa
                    }
                ];
        
                General.callRestEngine('fin#/crud/emprtpctpag_empresa','FilterData',filter).then(function(empresas){
                    empresas = empresas.dataset.emprtpctpag_empresa;
                    
                    if(empresas.length > 0)
                       deferred.resolve();
                    else deferred.reject();
                });
            }else{
                deferred.resolve();
            }
        });
        
        return deferred.promise;
    };
    
    /**
     * Busca Padrão Título
     */
    self.buscaPadraoTitulo = function (){
        // Documento
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
        
        self.buscaPadraoFilial(widgetDocumentosForm);
        widgetDocumentosForm.getField('NRORG').setValue(getLocalVar('NRORG'));
        widgetDocumentosForm.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widgetDocumentosForm.getField('CDOPERACTPG').setValue(getLocalVar('CDOPERADOR'));
        widgetDocumentosForm.getField('IDSTATUS').setValue('A');
        widgetDocumentosForm.getField('DTEMISSAO').setValue(moment().format('DD/MM/YYYY'));
        widgetDocumentosForm.getField('DTENTRSAID').setValue(moment().format('DD/MM/YYYY')); 
        widgetDocumentosForm.getField('DTATUAVENPAG').setValue(moment().format('DD/MM/YYYY'));
        widgetDocumentosForm.getField('CDFORNECED').setValue(null);
        widgetDocumentosForm.getField('NMRAZSOCFORN').setValue(null);
        widgetDocumentosForm.getField('CDTIPOOPER').setValue(null);
        widgetDocumentosForm.getField('NMTIPOOPER').setValue(null);
        widgetDocumentosForm.getField('CDTPCTPAGAR').setValue(null);
        widgetDocumentosForm.getField('DSTPCTPAGAR').setValue(null);
        widgetDocumentosForm.getField('DSDEFACTPAG').setValue(null);
        widgetDocumentosForm.getField('VRNOTAFISC').setValue(null);
        
        // Parcelamento
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
        
        widgetParcelamentoDocGrid.currentRow = {};
        widgetParcelamentoDocGrid.dataSource.data = [];
        
        
        
    };
    
    /**
     * Busca Padrão Título
     */
    self.changeValorDoc = function (){
        // Documento
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
        
        // Parcelamento
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
        
        if(self.formataValor(widgetDocumentosForm.currentRow.VRNOTAFISC) > 0){
            widgetParcelamentoDocGrid.currentRow.__index = 0;
            widgetParcelamentoDocGrid.currentRow.NRPARCBOLPAG = '00';
            widgetParcelamentoDocGrid.currentRow.NRPARCBOLPAGVISU = '000';
            widgetParcelamentoDocGrid.currentRow.IDEXCLUIR = null;
            widgetParcelamentoDocGrid.currentRow.DSCONTAPAG = widgetDocumentosForm.currentRow.DSDEFACTPAG?widgetDocumentosForm.currentRow.DSDEFACTPAG:widgetDocumentosForm.currentRow.DSTPCTPAGAR;
            widgetParcelamentoDocGrid.currentRow.VRFINALBPAG = widgetDocumentosForm.currentRow.VRNOTAFISC;
            widgetParcelamentoDocGrid.dataSource.data = [widgetParcelamentoDocGrid.currentRow];
        }else{
            widgetParcelamentoDocGrid.currentRow = {};
            widgetParcelamentoDocGrid.dataSource.data = [];
        }
    };

    /**
     * Edita Titulo A Pagar
     * @param args
     */
    self.editTituloAPagar = function(args){
        var widget = args.owner.widget;

        if((widget.currentRow.IDSTATBPAG == 'B') || (widget.currentRow.IDSTATBPAG == 'P')) {
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widget.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }
        if(widget.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        General.widgetEdit(args.owner.widget);
        self.changeTipoBaixa(widget, true);
        self.changeDsboletpag(false);
    };

    /**
     * Edita Titulo A Pagar
     * @param args
     */
    self.editaTituloAPagar = function(args){
        var widget = args.owner.widget;
        widget.currentRow.CODEQRPIX = widget.currentRow.CODEQRPIX ? widget.currentRow.CODEQRPIX : null;//por algum motivo o default value n quer preencher
        if(widget.isValid()){
            //pix santanter 
            // if(widget.currentRow.CDBANCO == '033' && (widget.currentRow.CHAVEPIX ||widget.currentRow.CODEQRPIX)  && !widget.currentRow.CDCCORBOLP){
            //     ScreenService.showMessage('Operação bloqueada. Santander requer conta do favorecido para pagamento pix.', 'checked');
            //     return;
            // }
            if(widget.currentRow.DSLINHBARPAG && !widget.currentRow.CDBARRAPAG){
                ScreenService.showMessage('Operação bloqueada. Representação numérica inválida.', 'checked');
                return;
            }

            if(widget.currentRow.CDBARRAPAG && !widget.currentRow.DSLINHBARPAG) {
                self.validaCodigoBarras('TitulosAPagarGrid', 'CDBARRAPAG', 'DSLINHBARPAG', 'VRBOLPAG', 'VRFINALBPAG').then(function () {
                    self.validaPix(widget).then(function(){
                        self.confirmaEditaTituloAPagar(widget);    
                    });
                });
            }else {
                self.validaPix(widget).then(function(){
                    self.confirmaEditaTituloAPagar(widget);    
                });
            }
        }
    };
    
   /**
    *Valida Pix antes de salvar
    * se resolvida a promisse salva
    */

    self.validaPix = function(widget){
        var deferred = ZHPromise.defer();
        widget.naoRelacionaPix = false;
        if(widget.getField('CHAVEPIX')._lastItemSearch && !widget.naoRelacionaPix){
            if(!widget.currentRow.IDCHAVEPIX){
                ScreenService.showMessage('Operação bloqueada. Informe o tipo de chave pix.', 'checked');
                deferred.reject();
                return deferred;
            }
            ScreenService.confirmMessage('Deseja salvar essa chave pix para este fornecedor?', 'question', function () {
                var pixWidget = widget.getField('CHAVEPIX').widgets[0];
                pixWidget.relacionandoPix = true;
                self.abrePopUpAddPIX(pixWidget);
                return deferred.reject();
            }, function(){
                widget.naoRelacionaPix = true;
                widget.currentRow.CHAVEPIX = widget.getField('CHAVEPIX')._lastItemSearch;
                return deferred.resolve();
            });
        }
        else{
            deferred.resolve();
        }
        return deferred;
    };


    /**
     * Valida Is Reload
     * @param widget
     * @param field_name
     */
    self.validaIsReload = function(widget, field_name){
        var contaFavorecido =  widget.getField(field_name);
        if (contaFavorecido.isReload){
            contaFavorecido.reload().then(function(){
                self.validaReloadContaFavorecido(widget, field_name, false);
            });
        }
    };

    /**
     * Valida Reload Conta Favorecido
     * @param widget
     * @param field_name
     * @param status
     */
    self.validaReloadContaFavorecido = function(widget, field_name, status){
        widget.getField(field_name).isReload = status;
    };

    /**
     * Confirma Titulo A Pagar
     * @param widget
     * @param args
     */
    self.confirmaEditaTituloAPagar = function(widget){

        Pag00001_TitulosAPagar_Service.editaTituloAPagar(widget.currentRow).then(function () {
            
            if(widget.backupRow.DTVENBOLPAG.substr(0, 10) != widget.currentRow.DTVENBOLPAG.substr(0, 10)){
                
                var params = {
                    nrorg       : getLocalVar('NRORG'),
                    dtvenbolpag : widget.currentRow.DTVENBOLPAG,
                    cdmotipror  : widget.currentRow.CDMOTIPROR,
                    idtppror    : 'F',
                    nrdiasvenc  : 0
                };
    
                var checkedrows = [
                    params,
                    [widget.currentRow]
                ];
    
                Pag00001_TitulosAPagar_Service.prorrogaTituloAPagar(checkedrows).then(function () {
                    General.changeWidgetAcessibility(widget.widgets, true);
                    ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                    widget.view();
                    widget.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }else{
                General.changeWidgetAcessibility(widget.widgets, true);
                ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                widget.view();
                widget.reload();
            }
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });

    };

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     * @param insertOrEdit
     */
    self.changeTipoBaixa = function(widget, insertOrEdit) {
        var idbaixautilp = widget.getField('IDBAIXAUTILP').value();
        var cdBarraPag = widget.getField('CDBARRAPAG');
        var cdContaFavorecido = widget.getField('CDCCORBOLP');

        if(['B', 'R', 'P', 'G', 'I', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X'].includes(idbaixautilp)){
            widget.getField('CDCCORBOLP').readOnly = true;
        }else{
            widget.getField('CDCCORBOLP').readOnly = false;
        }

        if(['B', 'R', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X', 'I', 'G'].includes(idbaixautilp)){
            widget.getField('DSLINHBARPAG').readOnly = false;
            widget.getField('CDBARRAPAG').readOnly   = false;
            widget.getField('CDCCORBOLP').readOnly   = true;
            if (!insertOrEdit){
                widget.getField('CDBANCBOLP').setValue('');
                widget.getField('CDAGENBOLP').setValue('');
                widget.getField('CDCCORBOLP').setValue('');
            }
        }else{
            if (idbaixautilp != 'P'){
                widget.getField('DSLINHBARPAG').readOnly = true;
                widget.getField('CDBARRAPAG').readOnly   = true;
                widget.getField('CDCCORBOLP').readOnly   = false;
                if (!insertOrEdit){
                    widget.getField('DSLINHBARPAG').setValue('');
                    widget.getField('CDBARRAPAG').setValue('');
                }
            }else{
                widget.getField('DSLINHBARPAG').readOnly = true;
                widget.getField('CDBARRAPAG').readOnly   = true;
                widget.getField('CDCCORBOLP').readOnly   = true;
                if (!insertOrEdit){
                    widget.getField('DSLINHBARPAG').setValue('');
                    widget.getField('CDBARRAPAG').setValue('');
                }
            }
        }
        
        // PIX
        if(['F', 'Z'].includes(idbaixautilp)){
            widget.getField('IDCHAVEPIX').readOnly = false;
            widget.getField('CHAVEPIX').readOnly = false;
            widget.getField('CODEQRPIX').readOnly = false;
            
            widget.fieldGroups.forEach(function(item){ 
                if(item.name == 'dadosPix_pag')
                    item.opened = true;
            });
            
            /** Busca Chave Pix Padrão do Fornecedor */
            var filter = [
                {
                    "name": "CDFORNECED",
                    "operator": "=",
                    "value": widget.currentRow.CDFORNECED
                },
                {
                    "name": "IDCHAVEPIXPREF",
                    "operator": "=",
                    "value": 'S'
                }
            ];
            
            General.callRestEngine('fin#/crud/chavepix', "FilterData", filter).then(function(pixPrincipal){
                if(pixPrincipal && pixPrincipal.dataset && pixPrincipal.dataset.chavepix[0]){
                    widget.currentRow.IDCHAVEPIX = pixPrincipal.dataset.chavepix[0].IDCHAVEPIX; 
                    widget.currentRow.CHAVEPIX   = pixPrincipal.dataset.chavepix[0].CDCHAVEPIX;
                    widget.currentRow.CODEQRPIX  = pixPrincipal.dataset.chavepix[0].CODEQRPIX;
                    widget.currentRow.DSCHAVEPIX = pixPrincipal.dataset.chavepix[0].DSCHAVEPIX;
                    templateManager.updateTemplate();
                }
            });
            
        }else{
            widget.getField('IDCHAVEPIX').readOnly = true;
            widget.getField('CHAVEPIX').readOnly   = true;
            widget.getField('CODEQRPIX').readOnly  = true;
            
            widget.fieldGroups.forEach(function(item){ 
                if(item.name == 'dadosPix_pag')
                    item.opened = false;
            });
            
            if (!insertOrEdit){
                widget.getField('IDCHAVEPIX').setValue('');
                widget.getField('CHAVEPIX').setValue('');
                widget.getField('CODEQRPIX').setValue('');
                widget.getField('DSCHAVEPIX').setValue('');
            }
        }
        
        //Adicionado validação caso o Tipo de baixa seja Boleto Bancário o código de barras seja obrigatório
        if(['B'].includes(idbaixautilp)){
            cdBarraPag.validations = {
              "required": {
                  "message": "Campo obrigatório"
              }
          };
        } else {
          cdBarraPag.validations = {};
        }
        
        /* Adicionado validação caso o Tipo de baixa seja:
            * Credito em Conta - C
            * Documento de Crédito(Doc) - D
            * Transferencia via CIP(Ted-CIP) - T
            * Transferencia via STR(Ted-STR) - S
            O campo do favorecido é colocado como obrigatório
        */
        if(['D', 'C', 'T', 'S'].includes(idbaixautilp)){
            cdContaFavorecido.validations = {
                "required": {
                    "message": "Campo obrigatório"
                }
            };
            
            if (!insertOrEdit){
                /** Busca Conta Favorecido Padrão */
                var filterCcorforn = [
                    {
                        "name": "CDFORNECED",
                        "operator": "=",
                        "value": widget.currentRow.CDFORNECED
                    },
                    {
                        "name": "IDCCORFORNPREF",
                        "operator": "=",
                        "value": 'S'
                    }
                ];
                
                General.callRestEngine('fin#/crud/ccorforn', "FilterData", filterCcorforn).then(function(ccorforn){
                    ccorforn = ccorforn.dataset.ccorforn;
                    
                    if(ccorforn.length > 0){
                        widget.currentRow.CDCCORBOLP = ccorforn[0].CDCCORFORN;
                        widget.currentRow.CDBANCBOLP = ccorforn[0].CDBANCFORN;
                        widget.currentRow.CDAGENBOLP = ccorforn[0].CDAGENFORN;
                    }
                });
            }
        } else {
            cdContaFavorecido.validations = {};
            
            if (!insertOrEdit){
                widget.getField('CDBANCBOLP').setValue('');
                widget.getField('CDAGENBOLP').setValue('');
                widget.getField('CDCCORBOLP').setValue('');
            }
        }
    };

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     * @param insertOrEdit
     */
    self.changeTipoBaixaAntecipacao = function(widget, insertOrEdit) {
        var idbaixautilp = widget.getField('IDBAIXAUTILP').value();

        if(['B', 'R', 'P', 'G', 'I', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X'].includes(idbaixautilp)){
            widget.getField('CDCCORANTECIPA').readOnly = true;
        }else{
            widget.getField('CDCCORANTECIPA').readOnly = false;
        }

        if(['B', 'R', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X', 'I', 'G'].includes(idbaixautilp)){
            widget.getField('DSLINHBARANTECI').readOnly = false;
            widget.getField('CDBARRAANTECIPA').readOnly   = false;
            widget.getField('CDCCORANTECIPA').readOnly   = true;
            if (!insertOrEdit){
                widget.getField('CDBANCANTECIPA').setValue('');
                widget.getField('CDAGENANTECIPA').setValue('');
                widget.getField('CDCCORANTECIPA').setValue('');
            }
        }else{
            if (idbaixautilp != 'P'){
                widget.getField('DSLINHBARANTECI').readOnly = true;
                widget.getField('CDBARRAANTECIPA').readOnly   = true;
                widget.getField('CDCCORANTECIPA').readOnly   = false;
                if (!insertOrEdit){
                    widget.getField('DSLINHBARANTECI').setValue('');
                    widget.getField('CDBARRAANTECIPA').setValue('');
                }
            }else{
                widget.getField('DSLINHBARANTECI').readOnly = true;
                widget.getField('CDBARRAANTECIPA').readOnly   = true;
                widget.getField('CDCCORANTECIPA').readOnly   = true;
                if (!insertOrEdit){
                    widget.getField('DSLINHBARANTECI').setValue('');
                    widget.getField('CDBARRAANTECIPA').setValue('');
                }
            }
        }
    };

    /**
     * Cálcula Valor para Percentual
     * @param row
     */
    self.calcValueToPercent = function(valor, total, percentual){
        valor = parseFloat(valor.replace(',','.'));
        total  = parseFloat(total);

        percentual = (valor * 100)/total;
    };

    /**
     * Cálcula Percentual para Valor
     * @param row
     */
    self.calcPercentToValue = function(percentual, total, valor){
        percentual  = parseFloat(percentual.replace(',','.'));
        total  = parseFloat(total);

        valor = (percentual * total)/100;
    };

    /**
     * Deleta Titulos a Pagar
     */
    self.deletaTituloPagar = function(args){
        var widget = args.owner.widget;

        if((widget.currentRow.IDSTATBPAG == 'B') || (widget.currentRow.IDSTATBPAG == 'P')) {
            ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
            return;
        }
        if(widget.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }
        if(widget.currentRow.IDASSOCBOLPAG == 'S') {
            ScreenService.showMessage('Operação bloqueada. Título associado não pode ser excluído.');
            return;
        }

        ScreenService.showCustomDialog('O que deseja excluir?', 'question', 'question', [{
            label: 'Vencimento',
            code: self.deletaVencTituloPagar
        }, {
            label: 'Documento',
            code: self.deletaDocTituloPagar
        }, {
            label: 'Cancelar',
            code: ''
        }]);
    };

    /**
     * Deleta Vencimento Titulos a Pagar
     */
    self.deletaVencTituloPagar = function(){

        ScreenService.confirmMessage('Deseja realmente excluir o Vencimento?', 'question', function() {

            var widget = templateManager.container.getWidget('TitulosAPagarGrid');

            if(widget.currentRow.NRPARCBOLPAG != '00'){
                ScreenService.showMessage('Operação bloqueada. Título parcelado deve ser excluído na aba Parcelamento.');
                return;
            }

            Pag00001_TitulosAPagar_Service.deletaVencTituloPagar(widget.currentRow).then(function () {
                ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                ScreenService.goBack();
                widget.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });

        });

    };

    /**
     * Deleta Documento Titulos a Pagar
     */
    self.deletaDocTituloPagar = function(){

        ScreenService.confirmMessage('Deseja realmente excluir o Documento?', 'question', function() {

            var widget = templateManager.container.getWidget('TitulosAPagarGrid');

            Pag00001_TitulosAPagar_Service.deletaDocTituloPagar(widget.currentRow).then(function () {
                ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                ScreenService.goBack();
                widget.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

        });

    };

    /**
     * Prorrogar Títulos a Pagar
     * @param widget
     */
    self.prorrogarTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {

            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBPAG == 'B') || (checkedrows[i].IDSTATBPAG == 'P')){
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSITUCNABP != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSTATUS == 'V') {
                    ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
                    return;
                }
            }

            var widgetProrrogar = templateManager.container.getWidget('ProrrogarForm');
            widgetProrrogar.isVisible = true;
            ScreenService.openPopup(widgetProrrogar);
            widgetProrrogar.getField('IDTPPROR').setValue('F');
            self.alteraTipoProrrogacao();

        }else ScreenService.showMessage('Marque o(s) título(s) para prorrogação.');
    };

    /**
     * Confirmar Prorrogacao Títulos a Pagar
     * @param args
     */
    self.confirmaProrrogacaoTitulo = function(args){
        var widget = args.owner.widget;

        if(widget.isValid()) {

            var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');

            var params = {
                nrorg       : getLocalVar('NRORG'),
                dtvenbolpag : widget.currentRow.DTVENBOLPAG,
                cdmotipror  : widget.currentRow.CDMOTIPROR,
                idtppror    : widget.currentRow.IDTPPROR,
                nrdiasvenc  : widget.currentRow.NRDIASVENC
            };

            var checkedrows = [
                params,
                widgetTitulos.getCheckedRows()
            ];

            Pag00001_TitulosAPagar_Service.prorrogaTituloAPagar(checkedrows).then(function () {
                ScreenService.showMessage('Título(s) prorrogado(s) com sucesso.', 'checked');
                widgetTitulos.unSelectAll();
                self.unsetStripeDataFooter();

                widget.currentRow.DTVENBOLPAG  = '';
                widget.currentRow.DSMOTIPROR   = '';

                ScreenService.closePopup(widget);
                ScreenService.closePopup();

                widget.isVisible = false;

                widgetTitulos.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Cancela Prorrogação Titulo a Pagar
     * @param args
     */
    self.cancelaProrrogacaoTitulo = function(args){
        var widget = args.owner.widget;

        widget.currentRow.DTVENBOLPAG  = '';
        widget.currentRow.DSMOTIPROR   = '';

        widget.isVisible = false;

        ScreenService.closePopup(widget);
    };

    /**
     * Remessa Titulo a Pagar
     * @param widget
     */
    self.remessaTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {
            var tpbaixaerror = false;

            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBPAG == 'B') || (checkedrows[i].IDSTATBPAG == 'P')){
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSITUCNABP != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSTATUS == 'V') {
                    ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
                    return;
                }
                if(!checkedrows[i].CDTPBAIXAPAG) {
                    ScreenService.showMessage('Operação bloqueada. Título(s) não possui(em) tipo de baixa.');
                    return;
                }
                if(!checkedrows[i].CDREDUCONT) {
                    ScreenService.showMessage('Operação bloqueada. Título(s) não possui(em) conta de pagamento.');
                    return;
                }
                if(!(['D','C','B','T','S','O', 'E', 'F', 'G', 'P', 'R', 'I', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X'].includes(checkedrows[i].IDBAIXAUTILP)))
                    tpbaixaerror = true;
            }

            if(tpbaixaerror) {
                ScreenService.confirmMessage('O(s) título(s) marcado(s), que não possui(em) tipo(s) de baixa(s) para remessa, será(ão) desconsiderado(s). Deseja prosseguir com o processo?', 'question', function () {
                    var widgetRemessa = templateManager.container.getWidget('RemessaForm');
                    widgetRemessa.isVisible = true;
                    ScreenService.openPopup(widgetRemessa);
                    self.buscaPadraoRemessaTitulo(widgetRemessa);
                });
            }else{
                var widgetRemessa = templateManager.container.getWidget('RemessaForm');
                widgetRemessa.isVisible = true;
                ScreenService.openPopup(widgetRemessa);
                self.buscaPadraoRemessaTitulo(widgetRemessa);
            }


        }else ScreenService.showMessage('Marque o(s) título(s) para geração da remessa.');
    };

    /**
     * Busca Padrão Remessa Titulo a Pagar
     */
    self.buscaPadraoRemessaTitulo = function (widget){
        var nrremessapag  = 0;
        var vrremessapag  = 0;
        var widgetTitulos       = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetTitulosFilter = widgetTitulos.appliedFilter;

        var cdempresa     = widgetTitulosFilter.FILTER_ROW.CDEMPRESA||null;
        var checkedrows   = widgetTitulos.getCheckedRows();

        for(var i in checkedrows) {
            vrremessapag += self.formataValor(checkedrows[i].VRFINALBPAG);
            nrremessapag += 1;
        }

        widget.currentRow = {};

        widget.getField('CDEMPRESA').setValue(cdempresa);
        widget.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widget.getField('DTCNABPAG').setValue(moment().format('DD/MM/YYYY'));
        widget.getField('IDPRENEGOCIADO').setValue('');
        widget.getField('IDOPECNABPAG').setValue('I');
        widget.getField('VRTOTALCNABPAG').setValue(vrremessapag);
        widget.getField('QTBOLCNABPAG').setValue(nrremessapag);
        widget.getField('DSARQCNABPAG').setValue('/tmp/');
        widget.getField('DSCONTCORR').setValue('');
        widget.getField('IDTPREMCNABPAG').setValue('');
        widget.getField('IDAGENREMCNABPAG').setValue('');
        widget.getField('NMFILIAL').setValue('');
        widget.getField('IDTPREMCNABPAG').readOnly = true;
        widget.getField('IDAGENREMCNABPAG').readOnly = true;
        widget.getField('IDPRENEGOCIADO').readOnly = true;
        widget.getField('IDTPREMCNABPAG').validations.required = true;
        delete widget.getField('IDTPREMCNABPAG').validations.required;

        templateManager.updateTemplate();
    };

    /**
     * Alteração Conta Corrente Remessa
     */
    self.alteraContCorrRem = function (widget){
        widget = widget.owner.field.widget;
        var cdbanccnab = widget.getField('CDBANCCNAB').value();
        
        // Bradesco
        if(cdbanccnab == '237') {
            widget.getField('IDTPREMCNABPAG').readOnly = false;
            widget.getField('IDTPREMCNABPAG').validations.required = true;
            widget.getField('IDTPREMCNABPAG').setValue('1');
            widget.getField('IDAGENREMCNABPAG').readOnly = false;
            widget.getField('IDAGENREMCNABPAG').validations.required = true;
            widget.getField('IDAGENREMCNABPAG').setValue('');
        }else{
            widget.getField('IDTPREMCNABPAG').readOnly = true;
            delete widget.getField('IDTPREMCNABPAG').validations.required;
            widget.getField('IDTPREMCNABPAG').setValue('');
            widget.getField('IDAGENREMCNABPAG').readOnly = true;
            delete widget.getField('IDAGENREMCNABPAG').validations.required;
            widget.getField('IDAGENREMCNABPAG').setValue('');
        }
        
        // Santander
        if(cdbanccnab == '033') {
            widget.getField('IDPRENEGOCIADO').readOnly = false;
            widget.getField('IDPRENEGOCIADO').validations.required = true;
            widget.getField('IDPRENEGOCIADO').setValue('N');
        }else{
            widget.getField('IDPRENEGOCIADO').readOnly = true;
            delete widget.getField('IDPRENEGOCIADO').validations.required;
            widget.getField('IDPRENEGOCIADO').setValue('');
        }
    };

    /**
     * Confirmar Remessa Titulo a Pagar
     * @param args
     */
    self.confirmaRemessaTitulo = function(args){
        var widget = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
        widget.currentRow.CDREDUCONTCOBR = widget.currentRow.CDREDUCONTCOBR||null;

        var currentRow = {
            nrorg           : getLocalVar('NRORG')   ,
            cdfilial        : getLocalVar('CDFILIAL'),
            remessaboleto   : widget.currentRow      ,
            boletosmarcados : widgetTitulos.getCheckedRows()
        };

        if(widget.isValid()) {
            Pag00001_TitulosAPagar_Service.remessaTituloPagar(currentRow).then(function (dsarqcnabpag) {
                var arquivo = dsarqcnabpag.dataset.remessaTituloPagar.dsarqcnabpag;

                ScreenService.showMessage('Remessa gerada com sucesso.');

                widget.currentRow.CDEMPRESA        = '';
                widget.currentRow.NRCNABPAG        = '';
                widget.currentRow.CDOPERADOR       = '';
                widget.currentRow.DTCNABPAG        = '';
                widget.currentRow.IDOPECNABPAG     = '';
                widget.currentRow.NMOPECNABPAG     = '';
                widget.currentRow.CDBANCCNAB       = '';
                widget.currentRow.CDREDUCONT       = '';
                widget.currentRow.DSCONTCORR       = '';
                widget.currentRow.CDREDUCONTCOBR   = '';
                widget.currentRow.DSREDUCONTCOBR   = '';
                widget.currentRow.IDTPREMCNABPAG   = '';
                widget.currentRow.NMTIPCNABPAG     = '';
                widget.currentRow.QTBOLCNABPAG     = '';
                widget.currentRow.VRTOTALCNABPAG   = '';
                widget.currentRow.IDAGENREMCNABPAG = '';
                widget.currentRow.CDFILIAL         = '';
                widget.currentRow.NMFILIAL         = '';

                ScreenService.closePopup(widget);
                ScreenService.closePopup();
                widget.isVisible = false;
                
                widgetTitulos.unSelectAll();
                self.unsetStripeDataFooter();

                self.downloadRemessaTitulo(arquivo);

                widgetTitulos.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Download Remessa Titulo a Pagar
     */
    self.downloadRemessaTitulo = function(arquivo) {
        window.open(window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl+"/donwloadArquivo?requestType=DataSet&dataset[0][DSARQUIVO]="+arquivo);
    };

    /**
     * Exclusão Remessa Titulo a Pagar
     */
    self.exclusaoRemessaTitulo = function(){
        var checkedrows = templateManager.container.getWidget('TitulosAPagarGrid').getCheckedRows();

        if(checkedrows.length > 0) {
            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBPAG == 'B') || (checkedrows[i].IDSTATBPAG == 'P')) {
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSITUCNABP == '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) não enviado(s), via CNAB, não possui(em) remessa(s).');
                    return;
                }
            }

            ScreenService.showCustomDialog('O que deseja cancelar?', 'question', 'question', [{
                label: 'Remessa',
                code: self.excluiRemessaPag
            }, {
                label: 'Título',
                code: self.excluiTituloRemessaPag
            }, {
                label: 'Cancelar',
                code: ''
            }]);

        }else ScreenService.showMessage('Marque o(s) título(s) para exclusão da remessa.');
    };

    /**
     * Exclui Titulo da Remessa Pagar
     */
    self.excluiTituloRemessaPag = function(){
        var widget      = templateManager.container.getWidget('TitulosAPagarGrid');

        var paramsRem = {
            nrorg     : getLocalVar('NRORG'),
            cdfilial  : getLocalVar('CDFILIAL'),
            cdoperador: getLocalVar('CDOPERADOR')
        };

        var checkedrows = [paramsRem, widget.getCheckedRows()];

        ScreenService.confirmMessage('Deseja realmente cancelar o(s) título(s) da(s) remessa(s)?', 'question', function () {
            Pag00001_TitulosAPagar_Service.excluiTituloPagRemessa(checkedrows).then(function () {
                ScreenService.showMessage('Título(s) excluído(s) da(s) remessa(s) com sucesso.', 'checked');
                ScreenService.closePopup();
                
                widget.unSelectAll();
                self.unsetStripeDataFooter();
                
                widget.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });

    };

    /**
     * Exclui Remessa Pagar
     */
    self.excluiRemessaPag = function(){
        var widget      = templateManager.container.getWidget('TitulosAPagarGrid');

        var paramsRem = {
            nrorg     : getLocalVar('NRORG'),
            cdfilial  : getLocalVar('CDFILIAL'),
            cdoperador: getLocalVar('CDOPERADOR')
        };
        var checkedrows = [paramsRem, widget.getCheckedRows()];

        ScreenService.confirmMessage('Deseja realmente cancelar a(s) remessa(s)?', 'question', function () {
            Pag00001_TitulosAPagar_Service.excluiRemessaPag(checkedrows).then(function () {
                ScreenService.showMessage('Remessa(s) excluída(s) com sucesso.');
                ScreenService.closePopup();
                
                widget.unSelectAll();
                self.unsetStripeDataFooter();
                
                widget.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    /**
     * Baixa Titulo a Pagar
     * @param widget
     */
    self.baixaTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        if(checkedrows.length > 0) {
            if (self.statusTituloPermiteRealizarBaixa(checkedrows)){
                var boletosBaixaPlanejada = checkedrows.filter(function(boleto){
                    return boleto.IDSTATBPAG == 'P';
                });
                var boletosEmAberto = checkedrows.filter(function(boleto){
                    return (boleto.IDSTATBPAG == 'A' && idhabibole_geral == 'N') || (boleto.IDSTATBPAG == 'A' && idhabibole_geral == 'S' && boleto.IDHABIBPAG == 'S');
                });
                if(boletosBaixaPlanejada.length > 0) {
                    ScreenService.confirmMessage('Existe(m) título(s) marcado(s) com baixa(s) planejada(s).Deseja transfomá-la(s) em baixa(s) real(is)?', 'question', function () {
                        self.baixarTitulosMarcadosOuTitulosBaixa();
                    }, function (){
                        if(boletosEmAberto.length > 0) {
                            self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                        }else{
                            ScreenService.closePopup();
                        }
                    });
                }else{
                    self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                }
            }else{
                ScreenService.closePopup();
            }
        }else{
            ScreenService.showMessage('Marque o(s) título(s) para baixá-lo(s).');
        }
    };

    /**
     * Efetiva Baixa
     * @param baixa
     * @returns {*}
     */
    self.efetivaBaixa = function(baixa){
        var deferred = ZHPromise.defer();

        var currentRow = {
            nrorg: getLocalVar('NRORG'),
            cdfilial: getLocalVar('CDFILIAL'),
            baixaboleto: baixa,
            boletosmarcados: baixa.BOLETOS,
            cheque: null
        };

        Pag00001_TitulosAPagar_Service.transformaBaixaPlanejado(currentRow).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });

        return deferred.promise;
    };

    /**
     * Efetiva Baixa
     * @param baixa
     * @returns {*}
     */
    self.efetivaBaixaTitulo = function(baixa){
        var deferred = ZHPromise.defer();

        var currentRow = {
            nrorg: getLocalVar('NRORG'),
            cdfilial: getLocalVar('CDFILIAL'),
            baixaboleto: baixa,
            boletosmarcados: baixa.BOLETOS
        };

        Pag00001_TitulosAPagar_Service.transformaTituloPlanejado(currentRow).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });

        return deferred.promise;
    };

    /**
     * Efetiva Titulos Marcados
     */
    self.efetivaTitulosMarcados = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        var boletosBaixaPlanejada = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBPAG == 'P';
        });
        var boletosEmAberto = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBPAG == 'A';
        });
        var baixatitulo = self.preparaDataSourceBoletosPlanejados(boletosBaixaPlanejada, 'E');
        baixatitulo.forEach(function(baixa){
            self.efetivaBaixaTitulo(baixa).then(function(){
                if(boletosEmAberto.length > 0) {
                    self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                }else{
                    ScreenService.closePopup();
                    widget.reload().then(function(){
                        ScreenService.showMessage('Título(s) transformado(s) em baixa(s) real(is) com sucesso.');
                    });
                    widget.unSelectAll();
                    self.unsetStripeDataFooter();
                }
            });
        });
    };

    /**
     * Efetiva Titulos da Baixa
     */
    self.efetivaTitulosDaBaixa = function(){
        var route = 'fin#/crud/boletpagar';
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');

        var boletosBaixaPlanejada = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBPAG == 'P';
        });
        var boletosEmAberto = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBPAG == 'A';
        });

        var baixatitulo = self.preparaDataSourceBoletosPlanejados(boletosBaixaPlanejada, 'E');
        baixatitulo.forEach(function(baixa){
            var filter = [
                {
                    "name": "CDFILIBAIPAG",
                    "operator": "=",
                    "value": baixa.BOLETOS[0].CDFILIBAIPAG
                },
                {
                    "name": "NRBAIXAPAG",
                    "operator": "=",
                    "value": baixa.NRBAIXAPAG
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": baixa.BOLETOS[0].NRORG
                }
            ];

            General.callRestEngine(route, 'FilterData', filter).then(function(boletos) {
                baixa.BOLETOS = boletos.dataset.boletpagar;
                self.efetivaBaixa(baixa).then(function(){
                    if(boletosEmAberto.length > 0) {
                        self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                    }else{
                        ScreenService.closePopup();
                        widget.reload().then(function(){
                            ScreenService.showMessage('Título(s) transformado(s) em baixa(s) real(is) com sucesso.');
                        });
                        widget.unSelectAll();
                        self.unsetStripeDataFooter();
                    }
                });
            });
        });
    };

    /**
     * Baixa Titulos Marcados ou Titulos Baixados
     */
    self.baixarTitulosMarcadosOuTitulosBaixa = function(){
        ScreenService.showCustomDialog('Transformar em Real?', 'question', 'question', [{
            label: 'Baixa',
            code: self.efetivaTitulosDaBaixa
        }, {
            label: 'Títulos',
            code: self.efetivaTitulosMarcados
        }, {
            label: 'Cancelar',
            code: self.verificaTituloEmAberto
        }]);
    };

    /**
     * Baixa Titulo a Pagar
     */
    self.verificaTituloEmAberto = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        var checkedrows = widget.getCheckedRows();

        var boletosEmAberto = checkedrows.filter(function(boleto){
            return boleto.IDSTATBPAG == 'A';
        });

        if(boletosEmAberto.length > 0)
            self.efetivarBaixaTitulosAbertos(boletosEmAberto);
        else ScreenService.closePopup();
    };

    /**
     * Titulos Baixados
     * @param boletoMarcado
     * @param indice
     * @param array
     * @returns {boolean}
     */
    function titulosBaixados(boletoMarcado, indice, array){
        return boletoMarcado.IDSTATBPAG != "B";
    }

    /**
     * Titulos Previstos
     * @param boletoMarcado
     * @param indice
     * @param array
     * @returns {boolean}
     */
    function titulosPrevistos(boletoMarcado, indice, array){
        return boletoMarcado.IDSTATUS != "V";
    }

    /**
     * Titulos Aberto e Planejados
     * @param boletoMarcado
     * @param indice
     * @param array
     * @returns {boolean}
     */
    function titulosAbertosEPlanejados(boletoMarcado, indice, array){
        return ((boletoMarcado.IDSTATBPAG == "A") || (boletoMarcado.IDSTATBPAG == "P"));
    }
    
    self.efetivarBaixaTitulosAbertos = function(boletosEmAberto){
        ScreenService.confirmMessage('Deseja realmente realizar baixa do(s) título(s) "Em Aberto" marcados?', 'question', function () {
            var widgetBaixa = templateManager.container.getWidget('NewBaixaTabbed');
            var widgetTitulosBaixa = templateManager.container.getWidget('NewTitulosBaixaGrid');
            
            ScreenService.closePopup();
            
            widgetBaixa.isVisible = true;
            self.widgetEmEdicao("NewTitulosBaixaGrid", true);

            ScreenService.openSwipe(widgetBaixa);
            
            self.buscaPadraoBaixaTitulo(widgetBaixa, widgetTitulosBaixa, boletosEmAberto);
            self.dataSourceAjusteTituloBaixo();
        });
    };
    
    self.widgetEmEdicao = function(widget_name, editing){
        var widget = templateManager.container.getWidget(widget_name);
        widget.editing = editing;
        templateManager.updateTemplate();
    };
    
    self.dataSourceAjusteTituloBaixo = function(){
        var widgetTitulosAPagar   = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');
        
        var route = 'fin_contaPagar#/buscaTodosAjustesBoletos';
        
        General.callRestEngine(route, 'DataSet', widgetTitulosAPagar.getCheckedRows()).then(function (ajustesboletos) {
            var dadosDeTodosAjustes = ajustesboletos.dataset.buscaTodosAjustesBoletos;
            
            localStorageEngine.setDataSource('fin_contaPagar#/ajbolpag_tipoajpagar', dadosDeTodosAjustes);
            
            widgetAjusteTituloBaixa.reload();
        });
    };
    
    self.buscaPadraoBaixaTitulo = function (widgetBaixa, widgetTitulosBaixa, titulosEmAberto){
        var deferred = ZHPromise.defer();
        
        var vrbaixapag  = 0;
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid').getCheckedRows();

        for(var i in widgetTitulos)
            vrbaixapag += parseFloat(widgetTitulos[i].VRFINALBPAG);

        widgetBaixa.getField('CDEMPRESA').setValue(templateManager.container.getWidget('TitulosAPagarGrid').appliedFilter.FILTER_ROW.CDEMPRESA||null);
        widgetBaixa.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widgetBaixa.getField('DTLANCBAIXAP').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixa.getField('DTBAIXAPAG').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixa.getField('IDUNICABAIXA').setValue('S');
        widgetBaixa.getField('IDSTATUSBPAG').setValue('E');
        widgetBaixa.getField('DSCONTCORR').setValue('');
        widgetBaixa.getField('CDBANCO').setValue('');
        widgetBaixa.getField('CDAGENCIA').setValue('');
        widgetBaixa.getField('CDCONTCORR').setValue('');
        widgetBaixa.getField('NMTPBAIXAPAG').setValue('');
        widgetBaixa.getField('DSBAIXAPAG').setValue('');
        widgetBaixa.getField('VRBAIXAPAG').setValue(vrbaixapag.toFixed(2));
        
        var nrorg   = null;
        var emissaomax = null;
        var emissaoatu = null;
        var nrtitulo  = [];
        
        titulosEmAberto.forEach(function (data) {
            nrorg = data.NRORG;
            emissaoatu = moment(data.DTEMISSAO, "DD/MM/YYYY HH:mm:ss");
            emissaomax = emissaomax ? emissaoatu > emissaomax ? emissaoatu : emissaomax : emissaoatu;
            
            nrtitulo.push(data.CDFILIAL+data.NRLANBOLPAG);
        });
        
        widgetBaixa.getField('DTEMISSAO').setValue(emissaomax.format('DD/MM/YYYY'));
        
        var filter = [
            {
                name: "NRORG", 
                operator: "=", 
                value: nrorg
            },
            {
                name: "NRTITULO", 
                operator: "IN", 
                value: nrtitulo
            }
        ];
        
        widgetTitulosBaixa.dataSourceFilter = filter;
        widgetTitulosBaixa.reload().then(function () {
              widgetTitulosBaixa.dataSource.selectAll();  
        });

        
        deferred.resolve();
        return deferred.promise;
    };

    /**
     * Prepara Data Source Boletos Planejados
     * @param boletosBaixaPlanejada
     * @param statusBaixaReal
     * @returns {Array}
     */
    self.preparaDataSourceBoletosPlanejados = function(boletosBaixaPlanejada, statusBaixaReal){
        var baixaTitulo = [];
        boletosBaixaPlanejada.forEach(function(boletoMarcado){
            var baixa = baixaTitulo.filter(function(baixa){
                return baixa.NRBAIXAPAG == boletoMarcado.NRBAIXAPAG;
            });
            if (baixa.length > 0){
                baixa[0].BOLETOS.push(boletoMarcado);
            }else{
                baixaTitulo[baixaTitulo.length] = {
                    'NRBAIXAPAG': boletoMarcado.NRBAIXAPAG,
                    'CDEMPRESA': boletoMarcado.CDEMPRESA,
                    'CDOPERADOR': boletoMarcado.CDOPERADOR,
                    'CDTPBAIXAPAG': boletoMarcado.CDTPBAIXAPAG,
                    'DTBAIXAPAG': boletoMarcado.DTBAIXAPAG,
                    'IDBAIXAUTILP': boletoMarcado.IDBAIXAUTILP,
                    'DTLANCBAIXAP': moment().format('DD/MM/YYYY'),
                    'IDSTATUSBPAG': statusBaixaReal,
                    'CDREDUCONT': boletoMarcado.CDREDUCONTBAIXA,
                    'DSBAIXAPAG': boletoMarcado.DSBAIXAPAG,
                    'NRCHEQUE': boletoMarcado.NRCHEQUE,
                    'BOLETOS': [boletoMarcado]
                };
            }
        });
        return baixaTitulo;
    };

    /**
     * Status Titulo Permite Realização da Baixa
     * @param checkedrows
     * @returns {boolean}
     */
    self.statusTituloPermiteRealizarBaixa = function(checkedrows){
        if(!checkedrows.every(titulosBaixados)) {
            ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) não pode(m) ser alterado(s).');
            return false;
        }
        if(!checkedrows.every(titulosPrevistos)) {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return false;
        }
        if(!checkedrows.every(titulosAbertosEPlanejados)) {
            ScreenService.showMessage('Operação bloqueada. Marque somente título(s) com status "Em Aberto" ou "Planejado".');
            return false;
        }
        return true;
    };

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     */
    self.changeTipoBaixaForm = function() {
        var widget = templateManager.container.getWidget('NewBaixaTabbed');

        var idbaixautilp = widget.getField('IDBAIXAUTILP').value();

        if((idbaixautilp == 'M') || (idbaixautilp == 'A')) {
            widget.getField('NRCHEQUE').readOnly  = false;
            widget.getField('NRCHEQUE').validations.required = true;
            widget.getField('DSNOMCHEQ').readOnly = false;
            widget.getField('DSCHEQUE').readOnly  = false;
        }else{
            widget.getField('NRCHEQUE').readOnly  = true;
            delete widget.getField('NRCHEQUE').validations.required;
            widget.getField('DSNOMCHEQ').readOnly = true;
            widget.getField('DSCHEQUE').readOnly  = true;

            widget.getField('NRCHEQUE').setValue('');
            widget.getField('DSNOMCHEQ').setValue('');
            widget.getField('DSCHEQUE').setValue('');
        }
        
        if(widget.getField('NMTPBAIXAPAG').value()){
            var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');
            
            if((idbaixautilp != 'M') && (idbaixautilp != 'A')) {
                ScreenService.showCustomDialog('Deseja aplicar o Tipo de Baixa selecionado para:', 'question', 'question', [{
                    label: 'Todos os Títulos',
                    code: self.aplicarTipoBaixaTitulos
                }, {
                    label: 'Somente Títulos sem Tipo de Baixa pré-cadastrado',
                    code: self.aplicarTipoBaixaTitulosSemTipoBaixa
                }, {
                    label: 'Cancelar',
                    code: self.cancelarTipoBaixaTitulo
                }]);
            }else self.aplicarTipoBaixaTitulos();
        }
    };
    
    /**
     * Aplica Tipo de Baixa
     */
    self.cancelarTipoBaixaTitulo = function() {
        var widget = templateManager.container.getWidget('NewBaixaTabbed');
        
        widget.currentRow.CDTPCTPAGAR  = '';
        widget.currentRow.NMTPBAIXAPAG = '';
    };
    
    /**
     * Aplica Tipo de Baixa
     */
    self.aplicarTipoBaixaTitulos = function() {
        var widget = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');
        
        widgetNewTitulosBaixaGrid.dataSource.data.forEach(function(titulo){
            titulo.CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
            titulo.NMTPBAIXAPAG = widget.currentRow.NMTPBAIXAPAG;
        });
    };
    
    /**
     * Aplica Tipo de Baixa somente para Títulos sem Tipo de Baixa pré-cadastrado
     */
    self.aplicarTipoBaixaTitulosSemTipoBaixa = function() {
        var widget = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');
        
        widgetNewTitulosBaixaGrid.dataSource.data.forEach(function(titulo){
            if(!titulo.CDTPBAIXAPAG){
                titulo.CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
                titulo.NMTPBAIXAPAG = widget.currentRow.NMTPBAIXAPAG;
            }
        });
    };

    /**
     * Confirmar Baixa Titulo a Pagar
     * @param args
     */
    self.confirmaBaixaTitulo = function(args){
        var widget = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulos = templateManager.container.getWidget('NewTitulosBaixaGrid');
        var checkedrows      = widgetNewTitulos.getCheckedRows();
        var boletostotais    = widgetNewTitulos.dataSource.data;

        if(widget.isValid()) {
            if(checkedrows.length > 0){
                var dtemissao  = moment(widget.currentRow.DTEMISSAO, "DD/MM/YYYY HH:mm:ss");
                var dtbaixapag = moment(widget.currentRow.DTBAIXAPAG, "DD/MM/YYYY HH:mm:ss");
                
                if(dtbaixapag < dtemissao){
                    ScreenService.showMessage('Operação bloqueada. Data da baixa inferior a data de emissão do(s) título(s).');
                    return;
                }
                
                localStorageEngine.getDataSource('fin_contaPagar#/ajbolpag_tipoajpagar').then(
                    function (datasource_aju) {
                        
                        var widgetTitulos    = templateManager.container.getWidget('TitulosAPagarGrid');
                        var cdempresa        = widgetTitulos.appliedFilter.FILTER_ROW.CDEMPRESA;
                        
                        self.buscaEmpresa(cdempresa).then(function (response_empresa){
                            var empresa = response_empresa.dataset.empresa[0];
                            
                            var currentRow = {
                                nrorg           : getLocalVar('NRORG')   ,
                                cdfilial        : getLocalVar('CDFILIAL'),
                                baixaboleto     : widget.currentRow      ,
                                boletosmarcados : checkedrows            ,
                                ajustesboletos  : datasource_aju         ,
                                boletostotais   : boletostotais
                            };
            
                            if(empresa.IDCTBBAIXAEMPR == 'S' && widget.currentRow.IDCTBLBXPAG == 'S'){
                                Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulos.appliedFilter.FILTER_ROW.CDEMPRESA).then(function (response) {
                                    var contopectbl  = response.dataset.contopectbl[0];
                
                                    if(!contopectbl){
                                        ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                                        return;
                                    }
                                    
                                    var dtinicctbl  = contopectbl.DTINICCTBL;
                                    var dtfinacctbl = contopectbl.DTFINACCTBL;
                
                                    if(dtinicctbl && dtfinacctbl) {
                                        if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widget.currentRow.DTBAIXAPAG, 'DD/MM/YYYY')) ||
                                            (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widget.currentRow.DTBAIXAPAG, 'DD/MM/YYYY'))) {
                                            ScreenService.showMessage('Operação bloqueada. Data da baixa fora do período contábil financeiro.');
                                            return;
                                        }
                                    }
                
                                    self.confirmaBaixaTituloSave(currentRow, widget, widgetTitulos);
                                }, function (error) {
                                    if (contabilizacaoController.checkErrosContabilizacao(error)) {
                                        ScreenService.showMessage(error.data.error);
                                    }
                                });
                            }else {
                                self.confirmaBaixaTituloSave(currentRow, widget, widgetTitulos);
                            }
                            
                        });
                    }
                );
            }else ScreenService.showMessage('Operação bloqueada. Selecione o(s) título(s) para realizar a baixa.');
        }
    };

    /**
     * Confirma Baixa Titulo a Pagar Salvamento
     * @param widget
     * @param widgetTitulos
     * @param checkedrows
     * @param boletosmarcados
     */
    self.confirmaBaixaTituloSave = function(currentRow, widget, widgetTitulos){

        localStorage.setItem('baixaPagRow',JSON.stringify(currentRow));
        localStorage.setItem('TIPOBAIXA','PAG');
        
        Pag00001_TitulosAPagar_Service.baixaTituloPagar(currentRow).then(function () {

            if( widget.currentRow.IDBAIXAUTILP == 'A') {
                ScreenService.confirmMessage('Cheque inicial '+widget.currentRow.NRCHEQUE+'. Confirmar emissão do cheque?', 'question', function () {
                    Pag00001_TitulosAPagar_Service.buscaMunicipio(getLocalVar('CDFILIAL')).then(function (response) {

                        var report = 'ModeloCheque';
                        var routes = 'fin_contaPagar#/' + report;
                        var row = {
                            P_TESTE   : 'N',
                            P_STLOCAL : response.dataset.municipio_endefili[0].NMMUNICIPIO,
                            P_CODREDU : widget.currentRow.CDREDUCONT,
                            P_NUMERO  : widget.currentRow.NRCHEQUE
                        };

                        General.openReport(routes, report, row);

                        ScreenService.showMessage('Título(s) baixado(s) com sucesso.');

                        widget.currentRow.DTBAIXAPAG = '';
                        widget.currentRow.IDSTATUSBPAG = '';
                        widget.currentRow.NMSTATUSBPAG = '';
                        widget.currentRow.CDREDUCONT = '';
                        widget.currentRow.DSCONTCORR = '';
                        widget.currentRow.CDTPBAIXAPAG = '';
                        widget.currentRow.NMTPBAIXAPAG = '';
                        widget.currentRow.VRBAIXAPAG = '';
                        widget.currentRow.DSBAIXAPAG = '';
                        widget.currentRow.NRCHEQUE = '';
                        widget.currentRow.DSNOMCHEQ = '';
                        widget.currentRow.DSCHEQUE = '';
        
                        ScreenService.closeSwipe(widget);
                        ScreenService.closeSwipe();
                        widget.isVisible = false;
                        widgetTitulos.reload();
                        
                        widgetTitulos.dataSource.unSelectAll();
                        self.unsetStripeDataFooter();
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }, function(){
                    ScreenService.showMessage('Título(s) baixado(s) com sucesso.');

                    widget.currentRow.DTBAIXAPAG = '';
                    widget.currentRow.IDSTATUSBPAG = '';
                    widget.currentRow.NMSTATUSBPAG = '';
                    widget.currentRow.CDREDUCONT = '';
                    widget.currentRow.DSCONTCORR = '';
                    widget.currentRow.CDTPBAIXAPAG = '';
                    widget.currentRow.NMTPBAIXAPAG = '';
                    widget.currentRow.VRBAIXAPAG = '';
                    widget.currentRow.DSBAIXAPAG = '';
                    widget.currentRow.NRCHEQUE = '';
                    widget.currentRow.DSNOMCHEQ = '';
                    widget.currentRow.DSCHEQUE = '';
    
                    ScreenService.closeSwipe(widget);
                    ScreenService.closeSwipe();
                    widget.isVisible = false;
                    widgetTitulos.reload();
                    
                    widgetTitulos.dataSource.unSelectAll();
                    self.unsetStripeDataFooter();
                });
            }else{
                ScreenService.showMessage('Título(s) baixado(s) com sucesso.');

                widget.currentRow.DTBAIXAPAG = '';
                widget.currentRow.IDSTATUSBPAG = '';
                widget.currentRow.NMSTATUSBPAG = '';
                widget.currentRow.CDREDUCONT = '';
                widget.currentRow.DSCONTCORR = '';
                widget.currentRow.CDTPBAIXAPAG = '';
                widget.currentRow.NMTPBAIXAPAG = '';
                widget.currentRow.VRBAIXAPAG = '';
                widget.currentRow.DSBAIXAPAG = '';
                widget.currentRow.NRCHEQUE = '';
                widget.currentRow.DSNOMCHEQ = '';
                widget.currentRow.DSCHEQUE = '';

                ScreenService.closeSwipe(widget);
                ScreenService.closeSwipe();
                widget.isVisible = false;
                widgetTitulos.reload();
                
                widgetTitulos.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }

        }, function (error) {
            if (contabilizacaoController.checkErrosContabilizacao(error)) {
                ScreenService.showMessage(error.data.error);
            }
        });
    };

    /**
     * Exclusão Baixa Titulo a Pagar
     */
    self.exclusaoBaixaTitulo = function(){
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
        var titulos = widgetTitulos.getCheckedRows();

        if(titulos.length > 0) {
            
            var titulosBaixados = widgetTitulos.getCheckedRows().filter(function(titulo){
                return ((titulo.IDSTATBPAG == 'B') || (titulo.IDSTATBPAG == 'P'));
            });
            
            self.existeBaixaAdiantamentoUtilizado(titulosBaixados)
            .then(function(){
                self.existeBaixaDevolucaoUtilizada(titulosBaixados)
                .then(function(){
                    Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulos.appliedFilter.FILTER_ROW.CDEMPRESA).then(function (response) {
                        var contopectbl  = response.dataset.contopectbl[0];
                        var dtinicctbl  = contopectbl.DTINICCTBL;
                        var dtfinacctbl = contopectbl.DTFINACCTBL;

                        for(var i in titulosBaixados) {
                            if (!((titulosBaixados[i].IDSTATBPAG == 'B') || (titulosBaixados[i].IDSTATBPAG == 'P'))) {
                                ScreenService.showMessage('Operação bloqueada. Título(s) não está(ão) baixado(s).');
                                return;
                            }

                            if(dtinicctbl && dtfinacctbl) {
                                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(titulosBaixados[i].DTBAIXAPAG, 'DD/MM/YYYY')) ||
                                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(titulosBaixados[i].DTBAIXAPAG, 'DD/MM/YYYY'))) {
                                    ScreenService.showMessage('Operação bloqueada. Data da baixa fora do período financeiro.');
                                    return;
                                }
                            }
                        }

                        ScreenService.showCustomDialog('O que deseja cancelar?', 'question', 'question', [{
                            label: 'Baixa',
                            code: self.confirmaExclusaoBaixa
                        }, {
                            label: 'Título',
                            code: self.confirmaExclusaoTituloBaixa
                        }, {
                            label: 'Cancelar',
                            code: ''
                        }]);
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                })
                .catch(function(error){
                    //ScreenService.showMessage(error.data.error);
                    ScreenService.closePopup();
                    ScreenService.showMessage('Operação bloqueada. Baixa referente à devolucao já utilizado, não pode ser excluída.');
                });
            })
            .catch(function(){
                ScreenService.closePopup();
                ScreenService.showMessage('Operação bloqueada. Baixa referente à adiantamento já utilizado, não pode ser excluída.');
            });
        }else ScreenService.showMessage('Operação bloaqueado. Marque o(s) título(s) para cancelamento de baixa.');
    };

    self.existeBaixaAdiantamentoUtilizado = function(checkedrows){
        
        var deferred = ZHPromise.defer();
        var route= 'fin_contaPagar#/crud/adiantapagar_contapagar';
        var nrbaixapag = checkedrows.map(function(row){return row.NRBAIXAPAG;}).join('_');
        var cdfilial = checkedrows.map(function(row){return row.CDFILIBAIPAG;}).join('_');
        var filter = [
          {
            'name':'P_NRBAIXAPAG', 
            'operator':'IN',
            'value':nrbaixapag 
          },
          {
            'name':'P_CDFILIAL', 
            'operator':'IN',
            'value':cdfilial
          }
        ];
        General.callRestEngine(route, 'FilterData', filter).then(function(adiantamentosUtilizados){
            var existeAdiantamento = adiantamentosUtilizados.dataset.adiantapagar_contapagar.length;
            if (existeAdiantamento == 0){
                deferred.resolve();
            }else {
                deferred.reject();
            }
        });
        return deferred.promise;
    };

    self.existeBaixaDevolucaoUtilizada = function(checkedrows){
        var deferred = ZHPromise.defer();
        var route= 'fin_contaPagar#/crud/devolucaopag_contapagar';
        var nrlanbolpag = checkedrows.map(function(row){return row.NRLANBOLPAG;});
        var cdfilial = checkedrows.map(function(row){return row.CDFILIAL;});
        var filter = [
            {'name':'NRLANBOLPAG', 'operator':'IN','value':nrlanbolpag},
            {'name':'CDFILIAL', 'operator':'IN','value':cdfilial}
        ];
        General.callRestEngine(route, 'FilterData', filter).then(function(devolucoesUtilizadas){
            var existeDevolucao = devolucoesUtilizadas.dataset.devolucaopag_contapagar.length;
            if (existeDevolucao == 0){
                deferred.resolve();
            }else {
                deferred.reject();
            }
        });
        return deferred.promise;
    };

    /**
     * Confirmar Exclusão Baixa Paga
     */
    self.confirmaExclusaoBaixa = function(){
        ScreenService.confirmMessage('Deseja realmente cancelar a(s) baixa(s)?', 'question', function() {
            var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
            var cheque = false;
            
            var titulosBaixados = widgetTitulos.getCheckedRows().filter(function(titulo){
                return ((titulo.IDSTATBPAG == 'B') || (titulo.IDSTATBPAG == 'P'));
            });
            
            titulosBaixados.forEach(function(titulo){
                if ((titulo.NRCHEQUE) && (titulo.VRFINALBPAG == titulo.VRBAIXAPAG))
                    cheque = true;
            });

            if(!cheque) {
                var currentRow = {
                    nrorg           : getLocalVar('NRORG'),
                    cdfilial        : getLocalVar('CDFILIAL'),
                    boletosmarcados : titulosBaixados,
                    cheque          : null
                };

                Pag00001_TitulosAPagar_Service.excluiBaixaPagar(currentRow).then(function () {
                    ScreenService.showMessage('Baixa(s) excluída(s) com sucesso.');
                    ScreenService.closePopup();
                    
                    self.reloadTitulosAPagar();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }else{
                var widgetCheque = templateManager.container.getWidget('ChequeForm');
                widgetCheque.isVisible = true;
                
                ScreenService.openPopup(widgetCheque);
            }
        });
    };

    /**
     * Recarrega Títulos a Pagar
     */
    self.reloadTitulosAPagar = function(){
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
        
        widgetTitulos.reload();
        widgetTitulos.unSelectAll();
        self.unsetStripeDataFooter();
    };

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     */
    self.changeCheque = function() {
        var widget = templateManager.container.getWidget('ChequeForm');

        var idstcheque = widget.getField('IDSTCHEQUE').value();

        if(idstcheque == '0'){
            widget.getField('DSMOTICANC').readOnly = true;
            widget.getField('DSMOTICANC').setValue('');
        }else{
            widget.getField('DSMOTICANC').readOnly = false;
        }
    };

    /**
     * Confirma Cheque Exclusão Baixa
     * @param args
     */
    self.confirmaChequeExclusaoBaixa = function(args){
        var widgetCheque = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');

        var currentRow = {
            nrorg: getLocalVar('NRORG'),
            cdfilial: getLocalVar('CDFILIAL'),
            boletosmarcados: widgetTitulos.getCheckedRows(),
            cheque: widgetCheque.currentRow
        };

        Pag00001_TitulosAPagar_Service.excluiBaixaPagar(currentRow).then(function () {
            ScreenService.showMessage('Baixa(s) excluída(s) com sucesso.');
            widgetTitulos.unSelectAll();
            self.unsetStripeDataFooter();

            widgetCheque.currentRow.IDSTCHEQUE    = '';
            widgetCheque.currentRow.CDMOTICANC     = '';
            widgetCheque.currentRow.DSMOTICANC     = '';

            ScreenService.closePopup(widgetCheque);
            ScreenService.closePopup();

            widgetCheque.isVisible = false;

            widgetTitulos.reload();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Confirmar Exclusão Boleto Baixa Titulo a Pagar
     */
    self.confirmaExclusaoTituloBaixa = function(){
        ScreenService.confirmMessage('Deseja realmente cancelar o(s) título(s) da(s) baixa(s)?', 'question', function() {
            var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
            var titulosChecked = widgetTitulos.getCheckedRows();

            titulosChecked = titulosChecked.filter(function(titulo){
                return ((titulo.IDSTATBPAG == 'B') || (titulo.IDSTATBPAG == 'P'));
            });

            var currentRow = {
                nrorg   : getLocalVar('NRORG'),
                cdfilial: getLocalVar('CDFILIAL'),
                boletosmarcados: titulosChecked
            };

            Pag00001_TitulosAPagar_Service.excluiTituloBaixaPagar(currentRow).then(function () {
                ScreenService.showMessage('Título(s) excluído(s) da(s) baixa(s) com sucesso.');
                widgetTitulos.unSelectAll();
                self.unsetStripeDataFooter();
                ScreenService.closePopup();
                widgetTitulos.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    /**
     * Associar Títulos a Pagar
     * @param widget
     */
    self.associacaoTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        var valortotal = 0;

        if(checkedrows.length > 1) {
            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBPAG == 'B') || (checkedrows[i].IDSTATBPAG == 'P')){
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSITUCNABP != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSTATUS == 'V') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) previsto(s) não pode(m) ser alterado(s).');
                    return;
                }

                valortotal += parseFloat(checkedrows[i].VRFINALBPAG);
            }

            Pag00001_TitulosAPagar_Service.validaAssociacaoTitulo(checkedrows).then(function () {
                var widgetAssociar = templateManager.container.getWidget('AssociarForm');
                widgetAssociar.isVisible = true;
                ScreenService.openPopup(widgetAssociar);
                widgetAssociar.getField('VRFINALBPAG').setValue(valortotal.toFixed(2));
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else ScreenService.showMessage('Operação bloqueada. Marque no mínimo dois títulos para associação.');
    };

    /**
     * Confirmar Associacao Títulos a Pagar
     * @param args
     */
    self.confirmaAssociacaoTitulo = function(args){
        var widget        = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');

        var params = {
            nrorg       : getLocalVar('NRORG')          ,
            cdfilial    : getLocalVar('CDFILIAL')       ,
            dtvenbolpag : widget.currentRow.DTVENBOLPAG ,
            cdreducont  : widget.currentRow.CDREDUCONT  ,
            cdtpbaixapag: widget.currentRow.CDTPBAIXAPAG
        };

        var checkedrows = [params, widgetTitulos.getCheckedRows()];

        if(widget.isValid()) {
            Pag00001_TitulosAPagar_Service.confirmaAssociacaoTitulo(checkedrows).then(function () {
                ScreenService.showMessage('Títulos associados com sucesso.');
                widgetTitulos.unSelectAll();
                self.unsetStripeDataFooter();

                widget.currentRow.DTVENBOLPAG  = '';
                widget.currentRow.DSCONTCORR   = '';
                widget.currentRow.NMTPBAIXAPAG = '';
                widget.isVisible = false;

                ScreenService.closePopup(widget);
                ScreenService.closePopup();

                widgetTitulos.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Cancela Associação Títulos a Pagar
     * @param widget
     */
    self.cancelaAssociacaoTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        if(checkedrows.length > 0) {
            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBPAG == 'B') || (checkedrows[i].IDSTATBPAG == 'P')){
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) ou com baixa(s) planejada(s) não pode(m) ser alterado(s).');
                    return;
                }
                if(checkedrows[i].IDSITUCNABP != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
            }

            ScreenService.confirmMessage('Deseja realmente desassociar o(s) título(s)?', 'question', function () {
                Pag00001_TitulosAPagar_Service.cancelaAssociacaoTitulo(checkedrows).then(function () {
                    ScreenService.showMessage('Títulos desassociados com sucesso.');
                    widget.unSelectAll();
                    self.unsetStripeDataFooter();
                    ScreenService.closePopup();
                    widget.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }else ScreenService.showMessage('Marque o(s) título(s) para desassociação.');
    };

    /**
     * Atualiza Formulário
     */
    self.atualizaForm = function(widget_name) {
        var widget = templateManager.container.getWidget(widget_name);
        widget.currentRow = widget.dataSource.data[0]?widget.dataSource.data[0]:{};
        templateManager.updateTemplate();
    };

    self.atualizaFormCnab = function() {
        var widget = templateManager.container.getWidget('CnabForm');
        var dadosRemessa = widget.dataSource.data.filter(function(data){
            return data.IDTIPCNABPAG == 0;
        });
        var dadosRetorno = widget.dataSource.data.filter(function(data){
            return data.IDTIPCNABPAG == 1;
        });
        if (dadosRemessa.length > 0){
            widget.currentRow.CDEMPRESA = dadosRemessa[0].CDEMPRESA;
            widget.currentRow.NRCNABPAG = dadosRemessa[0].NRCNABPAG;
            widget.currentRow.CDOPERADOR = dadosRemessa[0].CDOPERADOR;
            widget.currentRow.NMOPERADOR = dadosRemessa[0].NMOPERADOR;
            widget.currentRow.DTCNABPAG = dadosRemessa[0].DTCNABPAG;
            widget.currentRow.IDOPECNABPAG = dadosRemessa[0].IDOPECNABPAG;
            widget.currentRow.NMOPECNABPAG = dadosRemessa[0].NMOPECNABPAG;
            widget.currentRow.CDREDUCONT = dadosRemessa[0].CDREDUCONT;
            widget.currentRow.CDBANCO = dadosRemessa[0].CDBANCO;
            widget.currentRow.DSCONTCORR = dadosRemessa[0].DSCONTCORR;
            widget.currentRow.IDTIPCNABPAG = dadosRemessa[0].IDTIPCNABPAG;
            widget.currentRow.NMTIPCNABPAG = dadosRemessa[0].NMTIPCNABPAG;
            widget.currentRow.QTBOLCNABPAG = dadosRemessa[0].QTBOLCNABPAG;
            widget.currentRow.VRTOTALCNABPAG = dadosRemessa[0].VRTOTALCNABPAG;
        }
        if (dadosRetorno.length > 0){
            widget.currentRow.NRCNABPAGRETORNO = dadosRetorno[0].NRCNABPAG;
            widget.currentRow.NMOPERADORRETORNO = dadosRetorno[0].NMOPERADOR;
            widget.currentRow.DTCNABPAGRETORNO = dadosRetorno[0].DTCNABPAG;
        }
        templateManager.updateTemplate();
    };
    
    self.habilitarTitulo = function(widget){
        var checkedRows = widget.getCheckedRows();
        
        ScreenService.confirmMessage('Deseja realmente habilitar o(s) título(s) marcado(s)?', 'question', function () {

            checkedRows = checkedRows.filter(function(data){
                return data.IDHABIBPAG != 'S';
            });
            
            var dataSet = [checkedRows, ['S']];
            
            General.callRestEngine('fin_contaPagar#/habilitaTituloAPagar', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Registro(s) habilitado(s) com sucesso.');
                ScreenService.closePopup();
                widget.reload();
                widget.unSelectAll();
                self.unsetStripeDataFooter();
            }).catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
                
        });
    };
    
    self.desabilitarTitulo = function(widget){
        var checkedRows = widget.getCheckedRows();
        
        ScreenService.confirmMessage('Deseja realmente desabilitar o(s) título(s) marcado(s)?', 'question', function () {

            checkedRows = checkedRows.filter(function(data){
                return data.IDHABIBPAG != 'N';
            });
            
            var dataSet = [checkedRows, ['N']];
            
            General.callRestEngine('fin_contaPagar#/habilitaTituloAPagar', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Registro(s) desabilitado(s) com sucesso.');
                ScreenService.closePopup();
                widget.reload();
                widget.unSelectAll();
                self.unsetStripeDataFooter();
            }).catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
                
        });
    };
    
    /** [Fim]Título a Pagar/Pagos */

    /** [Inicio]Ajuste Grid */

    /**
     * Novo Ajuste Boleto Pagar
     * @param args
     */
    self.novoAjusteBoleto = function(args){
        var widget = args.owner.widget;
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulosAPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosAPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulosAPagar.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        if(!widgetTitulosAPagar.currentRow.NRCNABPAGREM){
            widget.newRow();
            widget.getField('VRATUABPAG').setValue(widgetTitulosAPagar.currentRow.VRFINALBPAG);
            widget.getField('VRAJBOLPAG').setValue(0);
            widget.getField('VRDESEBPAG').setValue(widgetTitulosAPagar.currentRow.VRFINALBPAG);
        }else{
            var paramsRem  = {nrorg: getLocalVar('NRORG'), cdfilial: getLocalVar('CDFILIAL'), cdoperador: getLocalVar('CDOPERADOR')};
            var currentRowRem = [paramsRem, [widgetTitulosAPagar.currentRow]];

            ScreenService.confirmMessage('O título está vinculado a uma remessa. Deseja removê-lo para prosseguir?', 'question', function () {
                Pag00001_TitulosAPagar_Service.excluiTituloPagRemessa(currentRowRem).then(function () {
                    widget.newRow();
                    widget.getField('VRATUABPAG').setValue(widgetTitulosAPagar.currentRow.VRFINALBPAG);
                    widget.getField('VRAJBOLPAG').setValue(0);
                    widget.getField('VRDESEBPAG').setValue(widgetTitulosAPagar.currentRow.VRFINALBPAG);
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });

        }
    };

    /**
     * Salva Ajuste Boleto
     * @param args
     */
    self.salvaAjusteBoleto = function(args){
        var widget        = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');

        if(widget.isValid()){
            
            var vrajbolpag = self.formataValor(widget.currentRow.VRAJBOLPAG);
            var vratuabpag = self.formataValor(widget.currentRow.VRATUABPAG);

            if(vrajbolpag <= 0){
                ScreenService.showMessage('Operação bloqueada. Valor do ajuste deve ser maior que zero.');
                return;
            }

            if((widget.currentRow.IDAJUCTPAG == 'D')&&(vrajbolpag > vratuabpag)){
                ScreenService.showMessage('Operação bloqueada. Valor do ajuste de desconto deve ser menor ou igual ao valor do título.');
                return;
            }


            Pag00001_TitulosAPagar_Service.salvaAjusteBoleto(widget.currentRow).then(function () {
                ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                ScreenService.goBack();

                // Valor de Ajuste do Título
                var idajuctpag  = widget.currentRow.IDAJUCTPAG;
                var vrajbolpag  = self.formataValor(widget.currentRow.VRAJBOLPAG);
                vrajbolpag      = (idajuctpag == 'A')?vrajbolpag:-(vrajbolpag);

                // Atualiza Valor Líquido do Título
                var vrfinalbpag = self.formataValor(widgetTitulos.currentRow.VRFINALBPAG);
                vrfinalbpag    += vrajbolpag;
                widgetTitulos.currentRow.VRFINALBPAG = vrfinalbpag;

                widget.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Deleta Ajuste Boleto
     * @param args
     */
    self.deletaAjusteBoleto = function(args){
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulosAPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosAPagar.currentRow.IDSTATBPAG == 'P')) {
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulosAPagar.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        if(!widgetTitulosAPagar.currentRow.NRCNABPAGREM){
            self.confirmaDeletaAjusteBoleto(args);
        }else{
            var paramsRem  = {nrorg: getLocalVar('NRORG'), cdfilial: getLocalVar('CDFILIAL'), cdoperador: getLocalVar('CDOPERADOR')};
            var currentRowRem = [paramsRem, [widgetTitulosAPagar.currentRow]];

            ScreenService.confirmMessage('O título está vinculado a uma remessa. Deseja removê-lo para prosseguir?', 'question', function () {
                Pag00001_TitulosAPagar_Service.excluiTituloPagRemessa(currentRowRem).then(function () {
                    self.confirmaDeletaAjusteBoleto(args);
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });

        }

    };

    /**
     * Deleta Ajuste Boleto
     * @param args
     */
    self.confirmaDeletaAjusteBoleto = function(args){
        ScreenService.confirmMessage('Deseja realmente excluir o Ajuste?', 'question', function() {
            var widget = args.owner.widget;
            var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');

            if (widget.isValid()) {
                Pag00001_TitulosAPagar_Service.deletaAjusteBoleto(widget.currentRow).then(function () {
                    ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                    ScreenService.goBack();

                    // Valor de Ajuste do Título
                    var idajuctpag  = widget.currentRow.IDAJUCTPAG;
                    idajuctpag      = (idajuctpag == 'A')?'D':'A';
                    var vrajbolpag  = parseFloat(widget.currentRow.VRAJBOLPAG);
                    vrajbolpag      = (idajuctpag == 'A')?vrajbolpag:-(vrajbolpag);

                    // Atualiza Valor Líquido do Título
                    var vrfinalbpag = parseFloat(widgetTitulos.currentRow.VRFINALBPAG);
                    vrfinalbpag    += vrajbolpag;
                    widgetTitulos.currentRow.VRFINALBPAG = vrfinalbpag;

                    widget.reload();
                    widgetTitulos.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }
        });
    };

	/**
     * Busca Ajuste Titulo a Pagar
     * @param cdfilial
     * @param nrlanbolpag
     */
    self.buscaAjusteTitulo = function(cdfilial, nrlanbolpag){
        return Pag00001_TitulosAPagar_Service.buscaAjusteTitulo(cdfilial, nrlanbolpag).then(function (ajbolpag) {
            return ajbolpag;
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Busca Conta a Pagar
     * @param cdfilial
     * @param nrlanctonf
     */
    self.buscaContaPagar = function(cdfilial, nrlanctonf){
        return Pag00001_TitulosAPagar_Service.buscaContaPagar(cdfilial, nrlanctonf).then(function (contapagar) {
            return contapagar;
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /** [Fim]Ajuste Grid */

    /** [Inicio] Parcelamento Grid */

    /**
     * Valor Total Parcelamento Títulos a Pagar
     * @param args
     */
    var valorTotalParcelamento = function(args){
        var vrtotal = 0;
        for(var i in args.dataSource.data)
            vrtotal += args.dataSource.data[i].VRFINALBPAG?parseFloat(args.dataSource.data[i].VRFINALBPAG):0;
            
        return vrtotal;
    };

    /**
     * Nova Parcela Títulos a Pagar
     * @param args
     */
    self.novaParcela = function(args){
        var widget = args.owner.widget;
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulosAPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosAPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }

        if(!widgetTitulosAPagar.currentRow.NRCNABPAGREM){
            self.confirmaNovaParcela(widget, widgetTitulosAPagar);
        }else{
            var paramsRem  = {nrorg: getLocalVar('NRORG'), cdfilial: getLocalVar('CDFILIAL'), cdoperador: getLocalVar('CDOPERADOR')};
            var currentRowRem = [paramsRem, [widgetTitulosAPagar.currentRow]];

            ScreenService.confirmMessage('O título está vinculado a uma remessa. Deseja removê-lo para prosseguir?', 'question', function () {
                Pag00001_TitulosAPagar_Service.excluiTituloPagRemessa(currentRowRem).then(function () {
                    self.confirmaNovaParcela(widget, widgetTitulosAPagar);
                    widgetTitulosAPagar.currentRow.IDSITUCNABP = '00';
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }
    };

    /**
     * Confirma Nova Parcela Títulos a Pagar
     * @param widget
     * @param widgetTitulosAPagar
     */
    self.confirmaNovaParcela = function(widget, widgetTitulosAPagar){
        self.buscaAjusteTitulo(widgetTitulosAPagar.currentRow.CDFILIAL, widgetTitulosAPagar.currentRow.NRLANBOLPAG).then(function (response_ajbolpag){
            var ajbolpag = response_ajbolpag.dataset.ajbolpag[0];
            if(!ajbolpag) {
                widget.currentRow = widget.dataSource.data[0];
                
                if (widget.dataSource.data[0].NRPARCBOLPAG == "00") {
                    widget.afterMoveRow();
                    widget.edit();
                    widget.currentRow.NRPARCBOLPAG = "01";
                    widget.currentRow.NRPARCBOLPAGVISU = "001";
                    widget.currentRow.__createdLocal = true;
                    widget.currentRow.__is_new       = false;
                    widget.getField('DTVENBOLPAG').readOnly = true;
                } else {
                    
                    var nrparcbolpag     = retornaNumeroParcela(widget.dataSource.data.length + 1);
                    var nrparcbolpagvisu = ("00" + (widget.dataSource.data.length + 1)).slice(-3);
                    var dtvenbolpag = widget.dataSource.data[widget.dataSource.data.length - 1].DTVENBOLPAG;
                    var vrtotalparc = valorTotalParcelamento(widget);
                    var vttotbolpag = widget.currentRow.VRTOTBOLPAG;

                    widget.newRow();

                    widget.currentRow.__is_new     = true;
                    widget.currentRow.NRPARCBOLPAG = nrparcbolpag;
                    widget.currentRow.NRPARCBOLPAGVISU = nrparcbolpagvisu;
                    widget.currentRow.DTVENBOLPAG  = dtvenbolpag;
                    widget.currentRow.VRFINALBPAG  = ((vttotbolpag - vrtotalparc) >= 0)?(vttotbolpag - vrtotalparc):0;
                    widget.getField('DTVENBOLPAG').readOnly = false;
                }
            }else ScreenService.showMessage('Operação bloqueada. Não é possível parcelar título com ajuste.');
        });
    };

    /**
     * Edita Parcela Títulos a Pagar
     * @param widget
     */
    self.editaParcela = function(widget){
        var dataLength = widget.dataSource.data.length;
        var indexRow   = widget.currentRow.__index + 1;

        if(widget.currentRow.NRPARCBOLPAG != '00')
            if (!widget.currentRow.__createdLocal) {
                if (dataLength != indexRow){
                    ScreenService.showMessage('Operação bloqueada. Somente é permitido a editar a última parcela do título.');
                }else General.edit(widget);
            }else{
                widget.currentRow.__is_new     = false;
                widget.getField('DTVENBOLPAG').readOnly = ((widget.currentRow.NRPARCBOLPAG == '01'));
                General.edit(widget);
            }
        else ScreenService.showMessage('Operação bloqueada. Não é possível editar a parcela original.');
    };

    /**
     * Deleta Parcela Títulos a Pagar
     * @param args
     */
    self.deletaParcelamento = function(args){
        var widget     = args.owner.widget;
        var dataLength = widget.dataSource.data.length;
        var indexRow   = widget.currentRow.__index + 1;

        if(dataLength > 1) {
            if (dataLength == indexRow) {
                if (!widget.currentRow.__createdLocal) {
                    ScreenService.confirmMessage('Deseja realmente excluir a parcela?', 'question', function () {
                        Pag00001_TitulosAPagar_Service.deletaParcelamentoTitulo(widget.dataSource.data).then(function () {
                            ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                            ScreenService.goBack();

                            widget.reload();
                            templateManager.container.getWidget('TitulosAPagarGrid').reload();
                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                } else{
                    ScreenService.confirmMessage('Deseja realmente excluir a parcela?', 'question', function () {
                        ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                        General.backOrCancel(args);
                    });
                }
            } else ScreenService.showMessage('Operação bloqueada. Somente é permitido a exclusão da última parcela do título.');
        }else{
            if(widget.currentRow.NRPARCBOLPAG == '01') {
                ScreenService.confirmMessage('Deseja realmente excluir a parcela?', 'question', function () {
                    ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                    ScreenService.goBack();

                    widget.reload();
                });
            }else ScreenService.showMessage('Operação bloqueada. Não é possível excluir a parcela original.');
        }
    };

    /**
     * Salva Parcela Títulos a Pagar
     * @param args
     */
    self.salvaParcela = function(args){
        if(args.owner.widget.isValid()) {
            var widget = args.owner.widget;
            
            widget.view();
            ScreenService.goBack();
            
            widget.getAction('finalizarParcela').isVisible = true;
            widget.getAction('parcelamentoAutomatico').isVisible = false;
            widget.getAction('cancelarParcela').isVisible = true;
            widget.getAction('back_list').isVisible = false;
            
            self.widgetEmEdicao('ParcelamentoGrid', true);
            self.desabilitaAbas('ParcelamentoGrid', true);
        }
    };
    
    /**
     * Desabilita Abas
     * @param widget
     */
    self.desabilitaAbas = function(widget_name, disable){
        var widget = templateManager.container.getWidget(widget_name);
        
        if(disable){
            General.changeWidgetAcessibility(widget.getParent().widgets, !disable);
            General.changeWidgetAcessibility([widget.getParent()], !disable);
            General.changeWidgetAcessibility([widget], disable);
        }else{
            General.changeWidgetAcessibility(widget.getParent().widgets, !disable);
            General.changeWidgetAcessibility([widget.getParent()], !disable);
        }
    };

    /**
     * Finaliza Parcelamento Títulos a Pagar
     * @param widget
     */
    self.finalizaParcelamento = function(widget){
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
        
        self.validaParcelamentoManual(widgetTitulos, widget)
        .then(function(){
            Pag00001_TitulosAPagar_Service.finalizaParcelamentoTitulo(widget.dataSource.data).then(function () {
                ScreenService.showMessage('Registro(s) salvo(s) com sucesso.', 'checked');
                
                self.widgetEmEdicao('ParcelamentoGrid', false);
                self.desabilitaAbas('ParcelamentoGrid', false);
                
                widget.reload();
                widgetTitulos.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        })
        .catch(function(erro){
            if (erro){
                ScreenService.showMessage(erro);
            }
        });
    };
    
    /**
     * Cancela Parcelamento Títulos a Pagar
     * @param args
     */
    self.cancelaParcelamento = function(args){
        General.backOrCancel(args).then(function(){
            var widget = args.owner.widget;
            if(widget.currentRow.NRPARCBOLPAG == '01' && widget.dataSource.data.length == 1)
                widget.reload();
            self.widgetEmEdicao('ParcelamentoGrid', false);
            self.desabilitaAbas('ParcelamentoGrid', false);
        });
    };

    /**
     * Desabilita Ação Finalizar Parcela Títulos a Pagar
     * @param args
     */
    self.desabilitaFinalizacaoParcela = function(args){
        args.owner.getAction('finalizarParcela').isVisible = false;
        args.owner.getAction('parcelamentoAutomatico').isVisible = true;
        args.owner.getAction('cancelarParcela').isVisible = false;
        args.owner.getAction('back_list').isVisible = true;
        self.widgetEmEdicao('ParcelamentoGrid', false);
        self.desabilitaAbas('ParcelamentoGrid', false);
    };

    /** [Fim] Parcelamento Grid */

    /** [Início]Documento Form */

    /**
     * Atualiza Titulo a Pagar/Pagos
     * @param args
     */
    self.atualizaTituloAPagar = function(){
        var widgetTitulosAPagarGrid = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetDocumentosGrid    = templateManager.container.getWidget('DocumentosGrid');
        
        if(widgetDocumentosGrid.isValid()){
            if((moment(widgetDocumentosGrid.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de Emissão deve ser menor ou igual a Data de Entrada.');
                return;
            }
            
            if((moment(widgetDocumentosGrid.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetTitulosAPagarGrid.currentRow.DTVENBOLPAG, 'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de Emissão deve ser menor ou igual a Data de Vencimento.');
                return;
            }
    
            if((widgetDocumentosGrid.currentRow.backupIdstatus   != widgetDocumentosGrid.currentRow.IDSTATUS) ||
                (widgetDocumentosGrid.currentRow.backupDtnotafisc != widgetDocumentosGrid.currentRow.DTENTRSAID)){
                Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetDocumentosGrid.currentRow.CDEMPRESA).then(function (response) {
                    var contopectbl  = response.dataset.contopectbl[0];
    
                    var dtinicctbl  = contopectbl.DTINICCTBL;
                    var dtfinacctbl = contopectbl.DTFINACCTBL;
    
                    if(dtinicctbl && dtfinacctbl) {
                        if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                            (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                            ScreenService.showMessage('Operação bloqueada. Data de entrada fora do período financeiro.');
                            return;
                        }
                    }
    
                    Pag00001_TitulosAPagar_Service.atualizaTituloAPagar(widgetDocumentosGrid.currentRow).then(function () {
                        ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                        ScreenService.goBack();
    
                        widgetTitulosAPagarGrid.currentRow.IDSTATUS = widgetDocumentosGrid.currentRow.IDSTATUS;
    
                        widgetDocumentosGrid.reload();
                    }, function (error) {
                        if (contabilizacaoController.checkErrosContabilizacao(error))
                            ScreenService.showMessage(error.data.error);
                    });
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }else{
                Pag00001_TitulosAPagar_Service.atualizaTituloAPagar(widgetDocumentosGrid.currentRow).then(function () {
                    ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                    ScreenService.goBack();
    
                    widgetTitulosAPagarGrid.currentRow.IDSTATUS = widgetDocumentosGrid.currentRow.IDSTATUS;
    
                    widgetDocumentosGrid.reload();
                }, function (error) {
                    if (contabilizacaoController.checkErrosContabilizacao(error))
                        ScreenService.showMessage(error.data.error);
                });
            }
        }
    };

    /**
     * Busca Parametrização Filial
     */
    self.changeFilial = function() {
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');

        if(widgetDocumentosForm.getField('CDFILIRECE').value()){

            // Tamanho NF
            var nrtamnumnfutdfe = (widgetDocumentosForm.getField('NRTAMNUMNFUTDFE').value() > 0) ? widgetDocumentosForm.getField('NRTAMNUMNFUTDFE').value() : 9;
            widgetDocumentosForm.getField('NRNOTAFISC').maxlength = nrtamnumnfutdfe;

            // Gera Novo Código NF
            var currentRow = {
                NRORG      : getLocalVar('NRORG'),
                CDCONTADOR : 'NRNOTAFISCPAG'
            };
            Pag00001_TitulosAPagar_Service.geraNovoCodigo(currentRow).then(function (novocodigo) {
                var nrnotafisc = novocodigo.dataset.geraNovoCodigo.novocodigo.slice(-nrtamnumnfutdfe);
                widgetDocumentosForm.getField('NRNOTAFISC').setValue(nrnotafisc);
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

            // Busca Padrão Serie NF
            Pag00001_TitulosAPagar_Service.buscaPadraoSerieNF(getLocalVar('CDFILIAL'), widgetDocumentosForm.currentRow.CDFILIRECE).then(function (response_serienf) {
                var serienf = response_serienf.dataset.notafiscal_serieentr[0];

                if(serienf != undefined){
                    widgetDocumentosForm.currentRow.CDSERIE = serienf.CDSERIEENTR;
                    widgetDocumentosForm.currentRow.DSSERIE = serienf.DSSERIEENTR;
                }else{
                    widgetDocumentosForm.currentRow.CDSERIE = null;
                    widgetDocumentosForm.currentRow.DSSERIE = null;
                }

                widgetDocumentosForm.redraw();
                widgetDocumentosForm.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

        }else{
            widgetDocumentosForm.currentRow.CDSERIE = '';
            widgetDocumentosForm.currentRow.DSSERIE = '';
            widgetDocumentosForm.getField('NRNOTAFISC').setValue('');
        }
    };

    /**
     * Busca Parametrização Fornecedor
     * @param args
     */
    self.changeFornecedor = function(args) {
        var widgetDocumentosForm = args.owner.field.widget.container.getWidget('DocumentosForm');

        widgetDocumentosForm.getField('CDTPCTPAGAR').setValue('');
        widgetDocumentosForm.getField('DSTPCTPAGAR').setValue('');
        widgetDocumentosForm.getField('CDCLASSFINA').setValue('');
        widgetDocumentosForm.getField('DSCONTAPAG').setValue('');
        widgetDocumentosForm.getField('DSDEFACTPAG').setValue('');

        var cdforneced = widgetDocumentosForm.getField('CDFORNECED').value();

        if(cdforneced){

            Pag00001_TitulosAPagar_Service.buscaFornecedor(cdforneced).then(function (paramforneced) {
                var fornecedor  = paramforneced.dataset.fornecedor[0];
                var cdtpctpagar = fornecedor.CDTPCTPAGAR;

                if(cdtpctpagar){
                    Pag00001_TitulosAPagar_Service.buscaTipoctpagar(cdtpctpagar).then(function (paramtipoctpagar) {
                        var tipoctpagar = paramtipoctpagar.dataset.tipoctpagar[0];
                        widgetDocumentosForm.getField('CDTPCTPAGAR').setValue(tipoctpagar.CDTPCTPAGAR);
                        widgetDocumentosForm.getField('DSTPCTPAGAR').setValue(tipoctpagar.DSTPCTPAGAR);
                        widgetDocumentosForm.getField('CDCLASSFINA').setValue(tipoctpagar.CDCLASSFINA);
                        widgetDocumentosForm.getField('DSCONTAPAG' ).setValue(tipoctpagar.DSDEFACTPAG);
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }

            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

        }
    };

    /**
     * Busca Parametrização Tipo de Conta
     * @param args
     * @param widget_name
     */
    self.changeTipoConta = function(args, widget_name) {
        var widgetDocumentos = templateManager.container.getWidget(widget_name);

        widgetDocumentos.currentRow.DSCONTAPAG = widgetDocumentos.currentRow.DSDEFACTPAG?widgetDocumentos.currentRow.DSDEFACTPAG:widgetDocumentos.currentRow.DSTPCTPAGAR;
        if(widget_name != 'DocumentosGrid')
            args.owner.field.widget.container.getWidget(widget_name).getField('DSCONTAPAG').reload();
    };

    /**
     * Gera Títulos a Pagar
     * @param widget
     */
    self.geraTituloAPagar = function(widget){
        var documentosForm = widget;
        var widgetTitulosAPagarGrid = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
        var widgetAnexoDocGrid = templateManager.container.getWidget('AnexoDocGrid');

        if(documentosForm.isValid()){
            if(widgetParcelamentoDocGrid.isValid()){
                var filter = [
                    {
                        "name": "CDFORNECED",
                        "operator": "=",
                        "value": documentosForm.currentRow.CDFORNECED
                    },
                    {
                        "name": "CDSERIEENTR",
                        "operator": "=",
                        "value": documentosForm.currentRow.CDSERIE
                    },
                    {
                        "name": "NRNOTAFISC",
                        "operator": "=",
                        "value": documentosForm.currentRow.NRNOTAFISC
                    },
                    {
                        "name": "IDSISTEMA",
                        "operator": "=",
                        "value": 'PAG'
                    }
                ];
    
                General.callRestEngine('fin#/crud/notafiscal', 'FilterData', filter).then(function (notafiscal) {
                    var nota = notafiscal.dataset.notafiscal;
                    
                    if((nota.length > 0)){
                        ScreenService.showMessage('Operação bloqueada. Documento já lançado.');
                        return;
                    }
                
                    if (self.titulosPagarValidos(documentosForm, widgetParcelamentoDocGrid)){
        			    self.validaDataVencimento(widgetParcelamentoDocGrid)
        			    .then(function () {
                            self.validaPeriodoContabil(widgetTitulosAPagarGrid, documentosForm)
                            .then(function(){
                                var dataset = self.preparaDataSetTitulos(widgetParcelamentoDocGrid, documentosForm, widgetAnexoDocGrid);
                                Pag00001_TitulosAPagar_Service.geraTituloAPagarDoc(dataset).then(function (nrlanbolpag) {
                                    nrlanbolpag = nrlanbolpag.dataset.geraTituloAPagarDoc.nrlanbolpag;
                                    ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                                    ScreenService.closeSwipe();
                                    documentosForm.isVisible = false;
                                    self.forceActivate();
                                    widgetTitulosAPagarGrid.reload().then(function(){
                                        var data = widgetTitulosAPagarGrid.dataSource.data.filter(function(data){
                                        	return ((data.CDFILIAL+data.NRLANBOLPAG) == nrlanbolpag);
                                        });
                                        widgetTitulosAPagarGrid.currentRow = data[0]?data[0]:{};
                                    });
                                    self.forceActivate();
                                }, function (error) {
                                    if (contabilizacaoController.checkErrosContabilizacao(error))
                                        ScreenService.showMessage(error.data.error);
                                });
                            })
                            .catch(function (error) {
                                ScreenService.showMessage(error);
                            });
                        }).catch(function(erro){
                            if (erro){
                                ScreenService.showMessage(erro);
                            }
                        });
                    }
                }).catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }else{
                ScreenService.showMessage('Operação bloqueada. Preencha o(s) campo(s) de vencimento(s).');
            }
        }
    };
    
    self.preparaDataSetTitulos = function(widgetParcelamentoDocGrid, documentosForm, widgetAnexoDocGrid){
        var dataParc = widgetParcelamentoDocGrid.dataSource.data;
        dataParc.forEach(function (data){
            if(data.__leftActions != undefined)
                delete data.__leftActions;
            if(data.__rightActions != undefined)
                delete data.__rightActions;
        });
        documentosForm.currentRow.DTATUAVENPAG = widgetParcelamentoDocGrid.dataSource.data[0].DTVENBOLPAG;
        dataset = [
            documentosForm.currentRow,
            dataParc,
            widgetAnexoDocGrid.dataSource.data
        ];
        return dataset;
    };
    
    self.titulosPagarValidos = function(documentosForm, widgetParcelamentoDocGrid){
        var tituloValido = true;
        
        if((moment(documentosForm.currentRow.DTEMISSAO, 'DD/MM/YYYY') > 
            moment(documentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY')) && tituloValido){
            tituloValido = false;
            ScreenService.showMessage('Operação bloqueada. Data de emissão deve ser menor ou igual a data de entrada.');
        }
        
        if((moment(documentosForm.currentRow.DTEMISSAO, 'DD/MM/YYYY') > 
           moment(widgetParcelamentoDocGrid.dataSource.data[0].DTVENBOLPAG, 'DD/MM/YYYY')) && tituloValido){
            tituloValido = false;
            ScreenService.showMessage('Operação bloqueada. Data de emissão deve ser menor ou igual a data de vencimento.');
        }
        
        var vrtotparc = 0;
        widgetParcelamentoDocGrid.dataSource.data.forEach(function(data){
            vrtotparc += self.formataValor(data.VRFINALBPAG);
        });
        vrtotparc = self.formataValor(vrtotparc);
        if((self.formataValor(documentosForm.currentRow.VRNOTAFISC) != vrtotparc) && tituloValido){
            tituloValido = false;
            ScreenService.showMessage('Operação bloqueada. Valor do documento difere do somatório das parcelas.');
        }
        
        return tituloValido;
    };
    
    self.validaPeriodoContabil = function(widgetTitulosAPagarGrid, documentosForm){
        var deferred = ZHPromise.defer();
        var cdempresa = widgetTitulosAPagarGrid.appliedFilter.FILTER_ROW.CDEMPRESA;
        Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(cdempresa)
        .then(function (response) {
            var contopectbl = response.dataset.contopectbl[0];
            if (!contopectbl){
                deferred.reject('Operação bloqueada. Não existe parametrização contábil para a empresa informada.');
            }else{
                var dtinicctbl  = contopectbl.DTINICCTBL;
                var dtfinacctbl = contopectbl.DTFINACCTBL;
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(documentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(documentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    deferred.reject('Operação bloqueada. Data do entrada fora do período financeiro.');
                }else{
                    deferred.resolve();
                }
            }
        })
        .catch(function (error) {
            deferred.reject(error.data.error);
        });
        return deferred.promise;
    };

    /**
     * Edita Documento Grid
     * @param widgetParcelamentoDocGrid
     */
    self.validaDataVencimento = function(widgetParcelamentoDocGrid){
        var deferred = ZHPromise.defer();
        var ultdata    = null;
        var datavalida = true;
    
        widgetParcelamentoDocGrid.dataSource.data.forEach(function(data){
            if(ultdata){
                if(ultdata > moment(data.DTVENBOLPAG, 'DD/MM/YYYY')){
                    datavalida = false;
                    deferred.reject('Operação bloqueada. Datas de vencimento das parcelas devem seguir uma sequencia crescente.');
                }else ultdata = moment(data.DTVENBOLPAG, 'DD/MM/YYYY');
            }else ultdata = moment(data.DTVENBOLPAG, 'DD/MM/YYYY');
        });
        
        if(datavalida)
            deferred.resolve(datavalida);
            
        return deferred.promise;
        
    };

    /**
     * Edita Documento Grid
     * @param args
     */
    self.editaDocumentoGrid = function(args){
        var widget = args.owner.widget;
        var widgetTitulosPagar   = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulosPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosPagar.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado via CNAB não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosPagar.currentRow.IDASSOCBOLPAG == 'S') {
            ScreenService.showMessage('Operação bloqueada. Título associado não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosPagar.currentRow.VRBOLPAG != widgetTitulosPagar.currentRow.VRFINALBPAG) {
            ScreenService.showMessage('Operação bloqueada. Título com ajuste não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosPagar.currentRow.NRPARCBOLPAG != '00') {
            ScreenService.showMessage('Operação bloqueada. Título com parcelado não pode ser alterado o documento.');
            return;
        }

        General.edit(widget);

        widget.currentRow.backupIdstatus   = widget.currentRow.IDSTATUS;
        widget.currentRow.backupDtnotafisc = widget.currentRow.DTENTRSAID;
        //self.changeStatusDocumentoForm();
        self.desabilitaCamposSistema();
    };

    /**
     * Desabilita Campos de acordo com o Sistema
     */
    self.desabilitaCamposSistema = function(){
        var widget = templateManager.container.getWidget('DocumentosGrid');

        if(widget.currentRow.IDSISTEMA == 'PAG') {
            widget.getField('DTEMISSAO').readOnly  = false;
            widget.getField('DTENTRSAID').readOnly = false;
            if(widget.currentRow.IDSTATUS == 'V')
                 widget.getField('VRNOTAFISC').readOnly = false;
            else widget.getField('VRNOTAFISC').readOnly = true;
        }else{
            widget.getField('DTEMISSAO').readOnly  = true;
            widget.getField('DTENTRSAID').readOnly = true;
            widget.getField('VRNOTAFISC').readOnly = true;
        }
    };

    /** [Fim]Documento Form */

    /** [Inicio]Ajuste Conta Grid */

    /**
     * Deleta Ajuste Conta
     * @param args
     */
    self.deletaAjusteConta = function(args){
        var widgetAjusteConta    = args.owner.widget;
        var widgetTitulosPagar   = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetContaPagar     = templateManager.container.getWidget('ContaDocumentosGrid');

        if((widgetTitulosPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulosPagar.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }

        Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulosPagar.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetAjusteConta.currentRow.DTAJUSTEPAG, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetAjusteConta.currentRow.DTAJUSTEPAG, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação bloqueada. Data do ajuste fora do período financeiro.');
                    return;
                }
            }

            var currentRowAjusteConta    = widgetAjusteConta.currentRow;
            var currentRowTitulosPagar   = widgetTitulosPagar.currentRow;
            var currentRowContaPagar     = widgetContaPagar.currentRow;

            var currentRow = Object.assign(currentRowAjusteConta, currentRowTitulosPagar, currentRowContaPagar);

            ScreenService.confirmMessage('Deseja realmente excluir o Ajuste?', 'question', function() {
                Pag00001_TitulosAPagar_Service.deletaAjusteConta(currentRow).then(function () {
                    ScreenService.showMessage('Registro salvo com sucesso.');
                    ScreenService.goBack();
                    widgetAjusteConta.reload();
                    widgetTitulosPagar.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });

    };

    /** [Fim]Ajuste Conta Grid */

    /** [Inicio]Rateio Grid */

    /**
     * Valor Total Rateio Conta Pagar
     * @param args
     */
    var valorTotalRateioConta = function(args){
        var vrTotal = 0;
        for(var i in args.dataSource)
            vrTotal += parseFloat(args.dataSource[i].VRRATCCCTPAG);
        return vrTotal;
    };

    /**
     * Novo Rateio Conta a Pagar
     * @param args
     */
    self.novoRateioConta = function(args){
        var widget  = args.owner.widget;
        var widgetContaPagar = templateManager.container.getWidget('ContaDocumentosGrid');
        var widgetTitulos   = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulos.currentRow.IDSTATBPAG == 'B') || (widgetTitulos.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulos.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulos.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulos.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação bloqueada. Data do entrada fora do período financeiro.');
                    return;
                }
            }

            var vrTotalConta  = parseFloat(widgetContaPagar.currentRow.VRATUAPAG);
            var vrTotalRateio = parseFloat(valorTotalRateioConta(args));
            var vrPerateio    = ((vrTotalRateio * 100) / vrTotalConta);

            widget.newRow();

            vrTotalRateio = vrTotalConta - vrTotalRateio;
            widget.currentRow.VRRATCCCTPAG = (vrTotalRateio >= 0)?vrTotalRateio:0;
            widget.currentRow.VRPERATEIO   = ((vrPerateio >= 0)&&(vrPerateio <= 100))?((vrTotalRateio * 100) / vrTotalConta):0;
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Edita Rateio Títulos a Pagar
     * @param widget
     */
    self.editaRateio = function(widget){
        var widgetTitulos    = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetContaPagar = templateManager.container.getWidget('ContaDocumentosGrid');

        if((widgetTitulos.currentRow.IDSTATBPAG == 'B') || (widgetTitulos.currentRow.IDSTATBPAG == 'P')) {
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulos.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulos.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulos.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação bloqueada. Data do entrada fora do período financeiro.');
                    return;
                }
            }

            widget.currentRow.__is_new = false;
            General.edit(widget);
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });


    };

    /**
     * Cálcula Percentual para Valor Rateio Conta a Pagar
     * @param row
     */
    self.calcPercValorRateio = function(row){
        var widgetConta = templateManager.container.getWidget('ContaDocumentosGrid');
        var vrperateio  = parseFloat(row.VRPERATEIO.replace(',','.'));
        var vratuapag   = parseFloat(widgetConta.currentRow.VRATUAPAG);

        row.VRRATCCCTPAG   = (vratuapag * vrperateio) / 100;
    };

    /**
     * Cálcula Valor para Percentual Rateio Conta a Pagar
     * @param row
     */
    self.calcValorPercRateio = function(row){
        var widgetConta   = templateManager.container.getWidget('ContaDocumentosGrid');
        var vrratccctpag  = parseFloat(row.VRRATCCCTPAG.replace(',','.'));
        var vratuapag     = parseFloat(widgetConta.currentRow.VRATUAPAG);

        row.VRPERATEIO   = (vrratccctpag * 100) / vratuapag;
    };

    /**
     * Deleta Rateio Conta a Pagar
     * @param args
     */
    self.deletaRateioConta = function(args){
        var widget = args.owner.widget;
        var widgetTitulos    = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetContaPagar = templateManager.container.getWidget('ContaDocumentosGrid');

        if((widgetTitulos.currentRow.IDSTATBPAG == 'B') || (widgetTitulos.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulos.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetContaPagar.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Pag00001_TitulosAPagar_Service.buscaContOpeCtbl(widgetTitulos.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaPagar.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação bloqueada. Data do entrada fora do período financeiro.');
                    return;
                }
            }

            ScreenService.confirmMessage('Deseja realmente excluir?', 'question', function () {
                if(!widget.currentRow.__createdLocal) {
                    Pag00001_TitulosAPagar_Service.deletaRateioConta(widget.currentRow).then(function () {
                        ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                        ScreenService.goBack();

                        // Remover Linha
                        widget.dataSource.data.splice(widget.currentRow.__index, 1);

                        if((widget.dataSource.data.length == 0) || (widget.editing == false))
                            widget.reload();
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }else{
                    ScreenService.showMessage('Registro excluído com sucesso.', 'checked');
                    widget.currentRow.__is_new = true;
                    General.backOrCancel(args);

                    if(widget.dataSource.data.length == 0)
                        widget.reload();
                }
            });
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });



    };

    /**
     * Salva Rateio Conta a Pagar
     * @param args
     */
    self.salvaRateioConta = function(args){
        var widget = args.owner.widget;
        if(widget.isValid()) {
            widget.view();
            ScreenService.goBack();
            self.habilitaFinalizacaoRateioConta(widget);
        }
    };

    /**
     * Finaliza Rateio Conta a Pagar
     * @param args
     */
    self.finalizaRateioConta = function(args) {
        var widgetRateioConta = args.owner.widget;
        var widgetContaPagar  = templateManager.container.getWidget('ContaDocumentosGrid');

        var vrTotalRateio     = 0;
        var vrTotalConta      = parseFloat(widgetContaPagar.currentRow.VRATUAPAG);

        // Percorre Dados de Rateio
        for (var i = 0; i < widgetRateioConta.dataSource.data.length; i++){
            // Percorre Dados de Rateio
            for (var j = 0; j < widgetRateioConta.dataSource.data.length; j++) {
                // Verifica Existência de Dados Duplicados
                if ((widgetRateioConta.dataSource.data[i].CDCENTCUST   == widgetRateioConta.dataSource.data[j].CDCENTCUST) &&
                    (widgetRateioConta.dataSource.data[i].CDFIRATCTPAG == widgetRateioConta.dataSource.data[j].CDFIRATCTPAG) &&
                    (i != j)
                ){
                    ScreenService.showMessage('Operação bloqueada. Existem informações duplicadas de Centro de Custo.');
                    return;
                }
            }

            // Soma Valor Total Rateio
            vrTotalRateio += parseFloat(widgetRateioConta.dataSource.data[i].VRRATCCCTPAG);
        }

        // Verifica Valor Total dos Rateios
        if (vrTotalRateio.toFixed(2) != vrTotalConta.toFixed(2)) {
            ScreenService.showMessage("Total do(s) Rateio(s) difere do valor total do Título a Pagar em "+((vrTotalRateio-vrTotalConta).formatFloat())+".");
            return;
        }

        // Declaração de Variáveis
        var nrorg        = widgetContaPagar.currentRow.NRORG;
        var cdfilial     = widgetContaPagar.currentRow.CDFILIAL;
        var nrlanctonf   = widgetContaPagar.currentRow.NRLANCTONF;
        var nrseqvenpag  = widgetContaPagar.currentRow.NRSEQVENPAG;
        var nrparcelapag = widgetContaPagar.currentRow.NRPARCELAPAG;

        // Array Data Source
        var dataSet = [{
            nrorg       : nrorg,
            cdfilial    : cdfilial,
            nrlanctonf  : nrlanctonf,
            nrseqvenpag : nrseqvenpag,
            nrparcelapag: nrparcelapag
        }, widgetRateioConta.dataSource.data];

        // Finaliza Rateio Conta
        Pag00001_TitulosAPagar_Service.finalizaRateioConta(dataSet).then(function () {
            ScreenService.showMessage('Registro(s) salvo(s) com sucesso.', 'checked');
            widgetRateioConta.reload();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Habilita Finalização Rateio Conta Pagar
     * @param widget
     */
    self.habilitaFinalizacaoRateioConta = function(widget){
        widget.getAction('finish').isVisible = true;
        widget.getAction('cancel').isVisible = true;
        widget.getAction('back_list').isVisible = false;
        widget.getParent().editing = true;

        var widgets = [
            templateManager.container.getWidget('ContaDocumentosGrid'),
            templateManager.container.getWidget('AjusteDocumentosGrid')
        ];

        General.changeWidgetAcessibility(widgets, false);
        templateManager.updateTemplate();
    };

    /**
     * Desabilita Finalização Rateio Conta Pagar
     * @param args
     */
    self.desabilitaFinalizacaoRateioConta = function(args){
        args.owner.getAction('finish').isVisible = false;
        args.owner.getAction('cancel').isVisible = false;
        args.owner.getAction('back_list').isVisible = true;
        args.owner.getParent().editing = false;

        var widgets = [
            templateManager.container.getWidget('ContaDocumentosGrid'),
            templateManager.container.getWidget('AjusteDocumentosGrid')
        ];

        General.changeWidgetAcessibility(widgets, true);
        templateManager.updateTemplate();
    };

    /** [Fim]Rateio Grid/Form */
    
    /** [Inicio]Anexo Grid/Form */
    
    self.importAnexoFile = function(args){
        //var anexoWidget = templateManager.container.getWidget('AnexoDocGrid');
        var importWidget = templateManager.container.getWidget('AnexoDocGridImportFile');
        importWidget.newRow();
        return ScreenService.openPopup(importWidget);
    };
    
    self.saveAnexoDocGrif = function(widget){
        widget.currentRow.NRORG = getLocalVar('NRORG');
        widget.currentRow.DTANEXOLANCBOL = moment().format('DD/MM/YYYY');
        
        Pag00001_TitulosAPagar_Service.saveAnexoDocGrif(widget.currentRow).then(function (sucess){
            ScreenService.closePopup();
            widget.getParent().reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.deleteAnexoDocGrif = function(widget){
        Pag00001_TitulosAPagar_Service.deleteAnexoDocGrif(widget.getCheckedRows()).then(function (sucess){
            widget.reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.verificaAnexos = function(widget){
        General.callRestEngine('fin_contaReceber#/deleteAnexoIncompletos', 'FilterData', []).then(function () {
            templateManager.container.getWidget('AnexoDocGrid').reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.adicionaAnexoGrid = function(widget){
        var widgetTitulosAPagarGrid = templateManager.container.getWidget('TitulosAPagarGrid');
        widget.newRow();
        widget.currentRow.NRLANBOLPAG = widgetTitulosAPagarGrid.currentRow.NRLANBOLPAG;
        widget.currentRow.CDFILIAL = widgetTitulosAPagarGrid.currentRow.CDFILIAL;
        widget.currentRow.CDTELA = 'PAG00001';
        widget.currentRow.DTANEXOLANCBOL = moment().format('DD/MM/YYYY');
    };
    
    self.saveAnexoGrid = function(widget){
        widget.currentRow.NRORG = getLocalVar('NRORG');
        
        var row = Util.clone(widget.currentRow, ignoredKeys);
        
        Pag00001_TitulosAPagar_Service.salvaAnexoGrid(row).then(function (sucess){
            ScreenService.goBack();
            widget.reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.deleteAnexoGrid = function(widget){
        ScreenService.confirmMessage('Deseja realmente excluir o(s) anexo(s)?', 'question', function() {
            
            var dataSet = [];
            if(widget.hasCheckedRows()){
                dataSet = widget.getCheckedRows();
                
                dataSet.forEach(function(data){
                	delete data.__rightActions;
                });
                
                Pag00001_TitulosAPagar_Service.deleteAnexoGrid(dataSet).then(
                    function (sucess){
                        widget.reload();
                    }, function(error){
                        ScreenService.showMessage(error.dataset.error);
                    }
                );
            } else{
                if (!widget.hasCheckedRows() && widget.currentRow.NRANEXOLANCBOL){
                    dataSet = [Util.clone(widget.currentRow, ignoredKeys)];
                    Pag00001_TitulosAPagar_Service.deleteAnexoGrid(dataSet).then(
                        function (sucess){
                            ScreenService.goBack();
                            ScreenService.showMessage('Registro excluído com sucesso.');
                            widget.reload();
                        }, function(error){
                            ScreenService.showMessage(error.dataset.error);
                        }
                    );
                }
            }
        });
    };
    
    self.downloadAnexoGrid = function(widget){
        var downloadRoute;
        var fileName;
        var serviceUrl;
        Pag00001_TitulosAPagar_Service.getFileServerUrl().then(
        function(sucess) {
            serviceUrl = sucess.dataset.sucess.fileName;
            if(widget.hasCheckedRows()){
                var rows = widget.getCheckedRows();
                rows.forEach(function(row){
                    downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexo?requestType=Row&row[DSARQUIVO]=";
                    fileName = row.NMPATHANEXO;
                    downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexoFileServer?requestType=Row&row[NMARQUIVO]=";
                    var rowDownload = {
                        'NMANEXOLANCBOL':row.NMANEXOLANCBOL, 
                        'PATH': serviceUrl+fileName
                    };
                    window.open(downloadRoute+rowDownload.NMANEXOLANCBOL+'&row[PATH]='+rowDownload.PATH);
                });
            } 
            else {
                var row = widget.currentRow;
                downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexo?requestType=Row&row[DSARQUIVO]=";
                fileName = row.NMPATHANEXO;
                downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexoFileServer?requestType=Row&row[NMARQUIVO]=";
                var rowDownload = {
                    'NMANEXOLANCBOL':row.NMANEXOLANCBOL, 
                    'PATH': serviceUrl+fileName
                };
                window.open(downloadRoute+rowDownload.NMANEXOLANCBOL+'&row[PATH]='+rowDownload.PATH);
            }
        });
    };
    
    /** [Fim]Anexo Grid */

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     */
    self.changeStatusDocumentoForm = function() {
        var widget = templateManager.container.getWidget('DocumentosGrid');
        
        if(widget.currentRow.IDSTATUS == 'V'){
            if(widget.currentRow.backupIdstatus != 'V')
                widget.currentRow.backupVRNOTAFISC  = widget.getField('VRNOTAFISC').value();
        
             widget.getField('VRNOTAFISC').readOnly = false;
        }else{ 
            if(widget.currentRow.backupVRNOTAFISC)
                widget.getField('VRNOTAFISC').setValue(widget.currentRow.backupVRNOTAFISC);
                
            widget.getField('VRNOTAFISC').readOnly = true;
        }

        /*var idstatus = widget.getField('IDSTATUS').value();

        if(idstatus == 'V') {
            if(widget.currentRow.backupIdstatus != 'V') {
                widget.currentRow.backupDtemissao  = widget.getField('DTEMISSAO').value();
                widget.currentRow.backupDtentrsaid = widget.getField('DTENTRSAID').value();
            }
            widget.getField('DTEMISSAO').readOnly  = false;
            widget.getField('DTENTRSAID').readOnly  = false;
            widget.getField('VRNOTAFISC').readOnly  = false;
        }else{
            if(widget.currentRow.backupDtemissao && widget.currentRow.backupDtentrsaid) {
                widget.getField('DTEMISSAO').setValue(widget.currentRow.backupDtemissao);
                widget.getField('DTENTRSAID').setValue(widget.currentRow.backupDtentrsaid);
            }
            widget.getField('DTEMISSAO').readOnly  = true;
            widget.getField('DTENTRSAID').readOnly = true;
            widget.getField('VRNOTAFISC').readOnly = true;
        }*/
    };

    /**
     * Desabilita Widget Baixa
     */
    self.desabilitaWidgetSwipe = function(){
        ScreenService.closeSwipe();
        templateManager.container.getWidget('TitulosAPagarGrid').activate();
        templateManager.container.getWidget('NewBaixaTabbed').isVisible = false;
    };

    /**
     * Gera Codigo de Barras Itau
     */
    self.geraCodigoBarrasItau = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  3); // Código da Carteira
        codigodebarra += representacaonumerica.substr( 7,  2); // Nosso Numero
        codigodebarra += representacaonumerica.substr(10,  6); // Nosso Numero
        codigodebarra += representacaonumerica.substr(16,  1); // DAC do Nosso Numero
        codigodebarra += representacaonumerica.substr(17,  3); // Código da Agencia
        codigodebarra += representacaonumerica.substr(21,  1); // Código da Agencia
        codigodebarra += representacaonumerica.substr(22,  6); // Código da Conta Corrente
        codigodebarra += representacaonumerica.substr(28,  3); // Fixo Zero

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras HSBC
     */
    self.geraCodigoBarrasHsbc = function(representacaonumerica, codigodebarra){

        var carteira = representacaonumerica.substr( 30,  1);

        if (carteira == '1'){
            codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
            codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
            codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
            codigodebarra += representacaonumerica.substr( 4,  5); // Nosso Numero + DAC do Nosso Numero
            codigodebarra += representacaonumerica.substr(10,  6); // Nosso Numero + DAC do Nosso Numero
            codigodebarra += representacaonumerica.substr(16,  4); // Código da Agencia
            codigodebarra += representacaonumerica.substr(21,  7); // Código da Conta Corrente
            codigodebarra += representacaonumerica.substr(28,  2); // Carteira
            codigodebarra += '1';                                  // Cod. do Aplicativo
        }else{
            codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
            codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
            codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
            codigodebarra += representacaonumerica.substr( 4,  5); // Código do cedente
            codigodebarra += representacaonumerica.substr(10,  2); // Código do cedente
            codigodebarra += representacaonumerica.substr(12,  8); // Número Bancário ou Código do Documento (Nosso Número,sem os dígitos verificadores e o tipo identificador).
            codigodebarra += representacaonumerica.substr(21,  5); // Número Bancário ou Código do Documento (Nosso Número,sem os dígitos verificadores e o tipo identificador).
            codigodebarra += representacaonumerica.substr(26,  4); // Data de Vencimento no Formato Juliano.
            codigodebarra += '2';                                  // Código do Produto CNR, deve preencher o número 2
        }

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Safra
     */
    self.geraCodigoBarrasSafra = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  1); // SISTEMA
        codigodebarra += representacaonumerica.substr( 5,  4); // Código da Agencia
        codigodebarra += representacaonumerica.substr(10, 10); // Código da Conta Corrente
        codigodebarra += representacaonumerica.substr(21, 10); // Nosso Numero + DAC do Nosso Numero

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Unibanco
     */
    self.geraCodigoBarrasUnibanco = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  2); // Código da Transação
        codigodebarra += representacaonumerica.substr( 6,  3); // Data de Vencimento
        codigodebarra += representacaonumerica.substr(10,  3); // Data de Vencimento
        codigodebarra += representacaonumerica.substr(13,  5); // Código da Agencia
        codigodebarra += representacaonumerica.substr(18,  2); // Nosso Numero + DAC do Nosso Numero
        codigodebarra += representacaonumerica.substr(21, 10); // Nosso Numero + DAC do Nosso Numero

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Caixa
     */
    self.geraCodigoBarrasCaixa = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  5);
        codigodebarra += representacaonumerica.substr(10, 10);
        codigodebarra += representacaonumerica.substr(21, 10);

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Banrisul
     */
    self.geraCodigoBarrasBanrisul = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  2); // SISTEMA
        codigodebarra += representacaonumerica.substr( 6,  3); // Código da Agencia
        codigodebarra += representacaonumerica.substr(10,  1); // Código da Agencia
        codigodebarra += representacaonumerica.substr(11,  7); // Código da Conta Corrente
        codigodebarra += representacaonumerica.substr(18,  2); // Nosso Numero + Constante 40 + NC
        codigodebarra += representacaonumerica.substr(21, 10); // Nosso Numero + Constante 40 + NC

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Sicoob
     */
    self.geraCodigoBarrasSicoob = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Fator de Vencimento + Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  1); // Código da carteira
        codigodebarra += representacaonumerica.substr( 5,  4); // Código da agência
        codigodebarra += representacaonumerica.substr(10, 10); // 12ª a 20ª posição do Campo Livre
        codigodebarra += representacaonumerica.substr(21, 10); // 22ª a 31ª posição do Campo Livre

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras Padrao
     */
    self.geraCodigoBarrasPadrao = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0,  4); // Banco(3) + Código da Moeda(1)
        codigodebarra += representacaonumerica.substr(32,  1); // Dígito Verificador do código de barras(1)
        codigodebarra += representacaonumerica.substr(-14   ); // Valor(14)
        codigodebarra += representacaonumerica.substr( 4,  5); // Campo Livre 1
        codigodebarra += representacaonumerica.substr(10, 10); // Campo Livre 2
        codigodebarra += representacaonumerica.substr(21, 10); // Campo Livre 3

        return codigodebarra;
    };

    /**
     * Gera Codigo de Barras 48 Dígitos
     */
    self.geraCodigoBarras48Digitos = function(representacaonumerica, codigodebarra){

        codigodebarra  = representacaonumerica.substr( 0, 11);
        codigodebarra += representacaonumerica.substr(12, 11);
        codigodebarra += representacaonumerica.substr(24, 11);
        codigodebarra += representacaonumerica.substr(36, 11);

        return codigodebarra;
    };

    /**
     * Valida Representação Numérica
     */
    self.validaRepresentacaoNumerica = function(widget_name, codbar_name, line_name, vrbol_name, vrliq_name){
        var deferred = ZHPromise.defer();
        
        var widget       = templateManager.container.getWidget(widget_name);
        var cdbarrapag   = widget.getField(codbar_name);
        
        var valorrepnum = '0000000000';
        var dslinhbarpag = widget.getField(line_name).value();
        var vrbolpag     = widget.getField(vrbol_name).value();
        var vrfinalbpag  = widget.getField(vrliq_name).value();

        if(dslinhbarpag) {
            dslinhbarpag = dslinhbarpag.replace(/\D/g, '');

            //Verifica se a linha digitável possui 48 dígitos, caso sim valida a linha para as contas de consumo
            if (dslinhbarpag.length > 48) {
                ScreenService.showMessage('Representação numérica inválida.');
                cdbarrapag.setValue('');
                deferred.reject();
                return deferred;
            }else if (dslinhbarpag.length < 48) {
                // Verifica o 1º campo da Linha Digitável
                if (self.Mod10(dslinhbarpag.substr(0, 9)) != dslinhbarpag.substr(9, 1)) {
                    ScreenService.showMessage('Primeiro campo inválido. Dígito verificador inconsistente.');
                    cdbarrapag.setValue('');
                    deferred.reject();
                    return deferred;
                }

                // Verifica o 2º campo da Linha Digitável
                if (self.Mod10(dslinhbarpag.substr(10, 10)) != dslinhbarpag.substr(20, 1)) {
                    ScreenService.showMessage('Segundo campo inválido. Dígito verificador inconsistente.');
                    cdbarrapag.setValue('');
                    deferred.reject();
                    return deferred;
                }

                // Verifica o 3º campo da Linha Digitável
                if (self.Mod10(dslinhbarpag.substr(21, 10)) != dslinhbarpag.substr(31, 1)) {
                    ScreenService.showMessage('Terceiro campo inválido. Dígito verificador inconsistente.');
                    cdbarrapag.setValue('');
                    deferred.reject();
                    return deferred;
                }
                // Representação Numérica Inválida
                if (!(dslinhbarpag.substr(0, 9).length) || !(dslinhbarpag.substr(10, 10).length) || !(dslinhbarpag.substr(21, 10).length)) {
                    ScreenService.showMessage('Representação numérica inválida.');
                    cdbarrapag.setValue('');
                    deferred.reject();
                    return deferred;
                }

                // Valor do Título Representação Numérica
                valorrepnum = dslinhbarpag.substr(-10);
            }else{
                // Valor do Título Representação Numérica
                valorrepnum = dslinhbarpag.substr(4,7) + dslinhbarpag.substr(12,4);
            }

            // Valida Valor Representação Numérica
            self.validaValorRepNumerica(valorrepnum, vrfinalbpag, vrbolpag).then(function () {
                // Gera Código de Barras Título
                self.geraCodigoBarras(widget, dslinhbarpag, cdbarrapag);
                deferred.resolve();
                return deferred;
            }).catch(function(){
                deferred.reject();
                return deferred;
            });

        }else cdbarrapag.setValue('');

        deferred.resolve();
        return deferred;
    };

    /**
     * Valida Valor Representação Numérica Título a Pagar
     * @param valorrepnum
     * @param vrfinalbpag
     * @param vrbolpag
     * @returns {*|Deferred}
     */
    self.validaValorRepNumerica = function(valorrepnum, vrfinalbpag, vrbolpag){
        var deferred = ZHPromise.defer();

        if (valorrepnum != '0000000000') {
            // Valor da Representação Numérica
            valorrepnum = ((parseInt(valorrepnum)) / 100).toFixed(2);
            vrfinalbpag = parseFloat(vrfinalbpag).toFixed(2);
            vrbolpag    = parseFloat(vrbolpag).toFixed(2);

            // Verifica se o Valor da Representação Numérica está diferente do Valor do Título no Sitema
            if ((valorrepnum != vrfinalbpag) && (valorrepnum != vrbolpag)) {
                ScreenService.showMessage('Operação bloqueada. O valor informado no código de barras (' + valorrepnum + ') para o título é diferente do valor final (' + vrfinalbpag + ') e valor original (' + vrbolpag + ') do título no sistema.');
                deferred.reject();
            } else deferred.resolve();
        }else{
            deferred.resolve();
        }

        return deferred;
    };

    /**
     * Modulo 10
     * @param codigo
     * @returns {string}
     * @constructor
     */
    self.Mod10 = function(codigo){

        var result = '';

        if (codigo.length > 0){

            var base   = 2;
            var soma   = 0;
            var carac  = 0;
            var aux    = 0;

            for (var i = (codigo.length - 1); i >= 0; i--){

                carac = parseInt(codigo[i]);

                if ((carac * base) > 9){

                    // Se o Nº tiver dois dígitos ou mais, eles são somados separadamente.
                    aux = carac * base;

                    for (var j = 0; j < aux.toString().length; j++)
                        soma = soma + parseInt(aux.toString()[j]);

                }else soma += (carac * base);

                base = base - 1;

                if (base < 1)
                    base = 2;
            }

            // Calcula um multiplo de 10 maior que a soma
            var mult = 10;

            while (mult < soma)
                mult = mult + 10;

            result = mult - soma;

            if (result == 10)
                result = '0';
        }

        return result;
    };

    /**
     * Modulo Concessionario
     * @param codigo
     * @returns {string}
     * @constructor
     */
    self.Mod10_Concessionaria = function(codigo){

        var result = '';

        if (codigo.length > 0){

            var base   = 2;
            var soma   = 0;
            var carac  = 0;

            for (var i = (codigo.length - 1); i >= 0; i--){

                carac = codigo[i];

                if ((carac * base) > 9)
                    soma  = soma + ((carac * base) - 10) + 1;
                else soma += (carac * base);

                if (base == 2)
                    base -= 1;
                else base += 1;
            }

            var resto = soma % 10;

            result = 10 - resto;

            if ((resto) == 0)
                result = '0';
        }

        return result;
    };

    /**
     * Modulo 11 B9
     * @param codigo
     * @param banco
     * @constructor
     * @return {string}
     */
    self.Mod11_B9 = function(codigo, banco){

        var result = '';

        if (codigo.length > 0) {
            var soma = 0;
            var base = 2;
            var carac = 0;

            var barraaux = codigo;

            if (codigo.length >= 44)
            // Retira o dígito verificador do código de barras
                barraaux = codigodebarra.substr(0, 4) + codigodebarra.substr(5, (codigo.length - 5));

            for (var i = (barraaux.length - 1); i >= 0; i--) {

                carac = codigo[i];

                soma += (carac * base);

                // Incrementa a base até chegar a 9
                base += 1;

                if (base > 9)
                    base = 2;
            }

            var dv = 11 - (soma % 11);
            var resto = (soma % 11);

            // BANESTES E NOSSA CAIXA
            if ((banco == '021') || (banco == '151')) { // Banestes e Nossa Caixa
                if ((resto == 0) || (resto == 1) || (resto > 9))
                    result = '1';
                else result = dv;
            } else {
                // MERCANTIL
                if (banco == '389') { // Mercantil
                    if ((resto == 0) || (resto == 1)) {
                        if (codigo.length >= 44)
                            result = '1'; // Dac do Código de barras
                        else result = '0'; // Dac do Nosso numero
                    } else result = dv;
                } else {
                    if (banco == '422') {
                        if (resto == 0)
                            result = '1';
                        else if (resto == 1)
                            result = '0';
                        else result = dv;
                    }
                    // SICREDI
                    else if (banco == '748') {
                        if ((resto == 0) || (resto == 1))
                            result = '0';
                        else result = dv;
                    } else if (banco == 'TRIBUTO') {
                        if ((resto == 1) || (resto == 0))
                            result = '0';
                        else if (resto == 10)
                            result = '1';
                        else result = dv;
                    }
                    // DEMAIS BANCOS
                    else {
                        /*
                         *Esta regra se aplica para o DV do CdBarra da caixa federal,
                         *mas não para o Nosso Número da caixa. Este é retornado na função MOD11_DV
                         */
                        if ((dv == 0) || (dv == 1) || (dv > 9))
                            result = '1';
                        else result = dv;
                    }
                }
            }
        }

        return result;
    };

    /**
     * Gera Codigo de Barras
     */
    self.geraCodigoBarras = function(widget, representacaonumerica, cdbarrapag){

        var codigodebarra = null;
        var banco = representacaonumerica.substr(0, 3);

        if (representacaonumerica.length != 48){
            switch(banco) {
                // BANCO ITAU
                case '341':
                    codigodebarra = self.geraCodigoBarrasItau(representacaonumerica, codigodebarra);
                    break;
                // BANCO HSBC
                case '399':
                    codigodebarra = self.geraCodigoBarrasHsbc(representacaonumerica, codigodebarra);
                    break;
                // BANCO SAFRA
                case '422':
                    codigodebarra = self.geraCodigoBarrasSafra(representacaonumerica, codigodebarra);
                    break;
                // BANCO UNIBANCO
                case '409':
                    codigodebarra = self.geraCodigoBarrasUnibanco(representacaonumerica, codigodebarra);
                    break;
                // BANCO CAIXA
                case '104':
                    codigodebarra = self.geraCodigoBarrasCaixa(representacaonumerica, codigodebarra);
                    break;
                // BANCO BANRISUL
                case '041':
                    codigodebarra = self.geraCodigoBarrasBanrisul(representacaonumerica, codigodebarra);
                    break;
                // BANCO SICOOB
                case '756':
                    codigodebarra = self.geraCodigoBarrasSicoob(representacaonumerica, codigodebarra);
                    break;
                //BANCO PADRAO
                default:
                    codigodebarra = self.geraCodigoBarrasPadrao(representacaonumerica, codigodebarra);
            }
        }else codigodebarra = self.geraCodigoBarras48Digitos(representacaonumerica, codigodebarra);

        cdbarrapag.setValue(codigodebarra);
    };

    /**
     * Valida Código de Barras
     */
    self.validaCodigoBarras = function(widget_name, codbar_name, line_name, vrbol_name, vrliq_name) {
        var deferred = ZHPromise.defer();
        
        var widget        = templateManager.container.getWidget(widget_name);
        var codigodebarra = widget.getField(codbar_name).value();
        var dslinhbarpag  = widget.getField(line_name);
        
        var barra         = '';
        var valorcodbar   = '0000000000';
        
        var vrbolpag     = widget.getField(vrbol_name).value();
        var vrfinalbpag  = widget.getField(vrliq_name).value();

        if (codigodebarra) {

            if (codigodebarra.length != 44) {
                ScreenService.showMessage('Código de barras deve ter 44 posições.');
                deferred.reject();
                return deferred;
            }

            if (codigodebarra.substr(0, 1) == '8') {
                var barraconc = codigodebarra.substr(0, 3) + codigodebarra.substr(4, 40);
                if ((codigodebarra.substr(2, 1) == '6') || (codigodebarra.substr(2, 1) == '7')) {
                    if (self.Mod10_Concessionaria(barraconc) != codigodebarra.substr(3, 1)) {
                        ScreenService.showMessage('Código de barras inválido. Dígito verificador inconsistente.');
                        deferred.reject();
                        return deferred;
                    }
                } else {
                    if (self.Mod11_B9(barraconc, 'TRIBUTO') != codigodebarra.substr(3, 1)) {
                        ScreenService.showMessage('Código de barras inválido. Dígito verificador inconsistente.');
                        deferred.reject();
                        return deferred;
                    }
                }

                // Valor do Título Código de Barras
                valorcodbar = codigodebarra.trim().substr(5, 10);
            } else {
                barra = codigodebarra.substr(0, 4) + codigodebarra.substr(5, 39);
                if (self.Mod11_B9(barra, codigodebarra.substr(0, 3)) != codigodebarra.substr(4, 1)) {
                    ScreenService.showMessage('Código de barras inválido. Dígito verificador inconsistente.');
                    deferred.reject();
                    return deferred;
                }

                // Valor do Título Código de Barras
                valorcodbar = codigodebarra.trim().substr(9, 10);
            }

            // Valida Valor do Código de Barras
            if (valorcodbar != '0000000000') {
                return self.validaValorCodigoBarras(valorcodbar, widget.currentRow.CDFORNECED, vrbolpag, vrfinalbpag).then(function () {
                    dslinhbarpag.setValue('');
                    deferred.resolve();
                    return deferred;
                }).catch(function () {
                    deferred.reject();
                    return deferred;
                });
            }else{
                deferred.resolve();
                return deferred;
            }

        }else {
            dslinhbarpag.setValue('');
            deferred.resolve();
            return deferred;
        }
    };

    /**
     * Valida Valor do Código de Barras
     * @param valorcodbar
     * @param cdforneced
     * @param vrfinalbpag
     * @param vrbolpag
     * @returns {*}
     */
    self.validaValorCodigoBarras = function(valorcodbar, cdforneced, vrfinalbpag, vrbolpag){
        var deferred = ZHPromise.defer();
        
        // Valor do código de barras
        valorcodbar = ((parseInt(valorcodbar))/100).toFixed(2);
        
        valorcodbar = self.formataValor(valorcodbar);
        vrfinalbpag = self.formataValor(vrfinalbpag);
        vrbolpag    = self.formataValor(vrbolpag);


        //Se o valor do código de barras está diferente do valor do boleto no sistema
        if ((valorcodbar != vrfinalbpag) && (valorcodbar != vrbolpag)){

            //Pesquisa se o Fornecedor do boleto possui Bonificação cadastrada.
            return Pag00001_TitulosAPagar_Service.buscaFornecedor(cdforneced).then(function (paramforneced) {
                var fornecedor  = paramforneced.dataset.fornecedor[0];

                //Se o fornecedor possui bonificação e o valor da mesma foi informada: Apenas alerta sobre a diferença.
                if ((fornecedor.IDTPDESCFORN == 'B') && (fornecedor.VRPEDESCFORN > 0)){
                    //Se marcar para não continuar a operação, então essa é abortada.
                    ScreenService.confirmMessage('Operação bloqueada. O valor informado no código de barras ('+valorcodbar+') para o título é diferente do valor final ('+vrfinalbpag+') e valor original ('+vrbolpag+') do título no sistema. Confirmar operação?', 'question', function(){
                        deferred.resolve();
                        return deferred;
                    }, function() {
                        deferred.reject();
                        return deferred;
                    });
                }else{
                    // Se não está marcado para possuir bonificação: Bloqueia Operação
                    ScreenService.showMessage('Operação bloqueada. O valor informado no código de barras ('+valorcodbar+') para o título é diferente do valor final ('+vrfinalbpag+') e valor original ('+vrbolpag+') do título no sistema.');
                    deferred.reject();
                    return deferred;
                }
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

        }else{
            deferred.resolve();
            return deferred;
        }

    };

    /**
     * Valida Adiantamento Fornecedor
     */
    self.validaAdiantamentoFornecedor = function(){

        var widget = templateManager.container.getWidget('TitulosAPagarGrid');

        // Válidações para Adiantamento e Devolução
        if((widget.getLastSelectedRow().IDSTATBPAG == 'B') || (widget.getLastSelectedRow().IDSTATBPAG == 'P')) return;
        if(widget.getLastSelectedRow().IDSITUCNABP != '00') return;
        if(widget.getLastSelectedRow().IDSTATUS == 'V') return;
        if(widget.getLastSelectedRow().IDTPCTPAGAR == '2') return;
        
        var route = 'fin_movimentacaoBancaria#/adiantapagar_adiantamentoPagar';
        var filter = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": widget.getLastSelectedRow().CDEMPRESA
            },
            {
                "name": "CDFORNECED",
                "operator": "=",
                "value": widget.getLastSelectedRow().CDFORNECED
            },
            {
                "name": "NRINSJURFORN",
                "operator": "=",
                "value": widget.getLastSelectedRow().NRINSJURFORN
            }
        ];

        General.callRestEngine(route, 'FilterData', filter).then(function(adiantamento) {
            var vrfinalbpag = self.formataValor(widget.getLastSelectedRow().VRFINALBPAG.toString());
            if (adiantamento.dataset.adiantapagar_adiantamentoPagar.length > 0 && vrfinalbpag > 0) {
                ScreenService.confirmMessage('Atenção! Fornecedor possui crédito de adiantamento. Deseja utilizá-lo?', 'question', function() {
                    var adiantamentoPopup = templateManager.container.getWidget('AdiantamentoTabbed');

                    ScreenService.openPopup(adiantamentoPopup).then(function () {
                        adiantamentoPopup.currentRow.NMRAZSOCFORN = widget.getLastSelectedRow().NMRAZSOCFORN;
                        adiantamentoPopup.currentRow.CDAJUCTPAG   = '';
                        adiantamentoPopup.currentRow.DSAJUCTPAG   = '';
                        adiantamentoPopup.currentRow.TOTALTITULO   = vrfinalbpag;
                        adiantamentoPopup.widgets[0].reload();
                    });
                }, function(){
                    self.validaDevolucaoFornecedor();
                });
            }else self.validaDevolucaoFornecedor();
        });
    };

    /**
     * Valida Devolução Fornecedor
     */
    self.validaDevolucaoFornecedor = function(){

        self.buscaGeralPag().then(function(response_geralpag) {

            var geral_pag   = response_geralpag.dataset.geral_pag[0];
            var idutlnfdevol = geral_pag.IDUTLNFDEVOL;

            // Verifica Utilização de Devolução
            if(idutlnfdevol == 'S') {
                var widget = templateManager.container.getWidget('TitulosAPagarGrid');
                var vrfinalbpag = self.formataValor(widget.getLastSelectedRow().VRFINALBPAG.toString());
                var routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';
                General.callRestEngine(routeDevolucao, 'Row', widget.getLastSelectedRow()).then(function (devolucao) {
                    if (devolucao.dataset.DEVOLUCAO.length > 0 && vrfinalbpag > 0) {
                        ScreenService.confirmMessage('Atenção! Fornecedor possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function () {
                            var devolucaoPopup = templateManager.container.getWidget('DevolucaoTabbed');

                            ScreenService.openPopup(devolucaoPopup).then(function () {
                                devolucaoPopup.currentRow.NMRAZSOCFORN = widget.getLastSelectedRow().NMRAZSOCFORN;
                                devolucaoPopup.currentRow.CDAJUCTPAG  = '';
                                devolucaoPopup.currentRow.DSAJUCTPAG  = '';
                                devolucaoPopup.currentRow.TOTALTITULO = vrfinalbpag;
                                devolucaoPopup.widgets[0].reload();
                            });
                        });
                    }
                });
            }
        });
    };

    /**
     * Cancela Adiantamento Fornecedor
     * @param widget
     */
    self.cancelaAdiantamentoFornecedor = function(widget){

        var widgetAdiantamento = templateManager.container.getWidget('TitulosAPagarGrid');
        var routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';

        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();

        General.callRestEngine(routeDevolucao, 'Row', widgetAdiantamento.getLastSelectedRow()).then(function(devolucao) {
            if (devolucao.length > 0) {
                ScreenService.confirmMessage('Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function(){
                    var devolucaoPopup = templateManager.container.getWidget('devolucaoFornecedorTabbed');

                    ScreenService.openPopup(devolucaoPopup).then(function(){
                        devolucaoPopup.currentRow.NMRAZSOCFORN = widget.getLastSelectedRow().NMRAZSOCFORN;
                    });
                });
            }
        });
    };

    /**
     * Cancela Devolução Fornecedor
     * @param widget
     */
    self.cancelaDevolucaoFornecedor = function(widget){
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
    };

    /**
     * Seta Data Source Popup Fornecedor
     * @param popupFornecedor
     */
    self.setDataSourcePopupFornecedor = function(popupFornecedor){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        var lastSelectedRow = widget.getLastSelectedRow();
        popupFornecedor.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDFORNECED') === 0){
                filtro.value = lastSelectedRow.CDFORNECED;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = lastSelectedRow.CDEMPRESA;
            }else if (filtro.name.localeCompare('NRINSJURFORN') === 0) {
                filtro.value = lastSelectedRow.NRINSJURFORN;
            }
        });
        popupFornecedor.reload();
    };

    /**
     * Seleciona Todos os Registros
     * @param widget
     */
    self.selecionaTodosRegistros = function(widget){
        self.atualizaValorAdiantaDev(widget).then(
            function() {},
            function(error){
                widget.unSelectAll();
                self.unsetStripeDataFooter();
            }
        );
    };

    /**
     * Atualiza Valor Adiantamento Devolução
     * @param widget
     * @returns {*}
     */
    self.atualizaValorAdiantaDev = function(widget) {
        var deferred = ZHPromise.defer();
        var tituloGrid = templateManager.container.getWidget('TitulosAPagarGrid');

        var totalDesconto = 0;
        var valorTitulo   = parseFloat(tituloGrid.getLastSelectedRow().VRFINALBPAG);

        widget.getCheckedRows().forEach(function (data) {
            totalDesconto += parseFloat(data.VRDESCONTO);
        });

        if (totalDesconto > valorTitulo){
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do título.');
            widget.getLastSelectedRow().__isSelected = false;
            deferred.reject();
        } else {
            deferred.resolve();
            widget.getParent().currentRow.TOTALDESCONTO = totalDesconto;
        }
        return deferred.promise;
    };

    /**
     * Calcula Total Adiantamento Disponível
     * @param widget
     */
    self.calculaTotalAdiantamentoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += self.formataValor(data.VRTOTDISP);
        });
        widget.getParent().currentRow.TOTALADIANTAMENTO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    /**
     * Calcula Total Devolução Disponível
     * @param widget
     */
    self.calculaTotalDevolucaoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += self.formataValor(data.VRTOTDISP);
        });
        widget.getParent().currentRow.TOTALDEVOLUCAO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    /**
     * Limita Valor Desconto
     * @param widget
     */
    self.limitaValorDesconto = function(widget){
        var superiorLancamento = false;

        var data   = widget.dataSource.data;
        var indice = widget.currentRow.__index;
        var widgetTitulo = templateManager.container.getWidget('TitulosAPagarGrid');

        var valorTitulo     = self.formataValor(widgetTitulo.getLastSelectedRow().VRFINALBPAG.toString());
        var valorDesconto   = self.formataValor(widget.currentRow.VRDESCONTO_inline.toString());
        var totalDisponivel = self.formataValor(widget.currentRow.VRTOTDISP.toString());
        var totalDesconto   = 0.00;

        for(var i = 0; i < data.length; i++){
            if (data[i].__isSelected){
                if (i != indice){
                    totalDesconto += self.formataValor(data[i].VRDESCONTO.toString());
                }else{
                    totalDesconto += self.formataValor(data[i].VRDESCONTO_inline.toString());
                }
            }
        }

        if (valorDesconto > totalDisponivel) {
            ScreenService.showMessage('Valor do desconto deve ser menor ou igual que o valor disponível.');
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }

         if ((valorDesconto > valorTitulo) || (totalDesconto > valorTitulo)) {
            totalDesconto -= self.formataValor(data[indice].VRDESCONTO_inline.toString());
            data[indice].VRDESCONTO_inline = 0;
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do lançamento.');
        }

        widget.getParent().currentRow.TOTALDESCONTO = totalDesconto;
    };

    /**
     * Confirma Adiantamento
     */
    self.confirmaAdiantamento = function(){
        var adiantamentoTabbed = templateManager.container.getWidget('AdiantamentoTabbed');
        var adiantamentoGrid   = templateManager.container.getWidget('AdiantamentoGrid');
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');

        if(adiantamentoTabbed.isValid()) {
            if ((adiantamentoGrid.getCheckedRows().length <= 0) || (adiantamentoTabbed.currentRow.TOTALDESCONTO <= 0)) {
                ScreenService.showMessage('Operação bloqueada. Selecione o(s) registro(s) e informe o(s) valor(es) a ser(em) descontado(s).');
                return;
            }

            var route = 'fin_movimentacaoBancaria#/adiantapagar_adiantamentoPagar';
            var filter = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": widget.getLastSelectedRow().CDEMPRESA
                },
                {
                    "name": "CDFORNECED",
                    "operator": "=",
                    "value": widget.getLastSelectedRow().CDFORNECED
                },
                {
                    "name": "NRINSJURFORN",
                    "operator": "=",
                    "value": widget.getLastSelectedRow().NRINSJURFORN
                }
            ];
            
            //valida se o adiantamento ainda esta disponivel, impedindo a utilizacao do mesmo em dois boletos diferentes ao mesmo tempo
            General.callRestEngine(route, 'FilterData', filter).then(function(adiantamento) {
                var adiantamentosNaoDisponiveis = [];
                var adiantametoDisponiveis = adiantamento.dataset.adiantapagar_adiantamentoPagar;
                var adiantamentosSelecionados = adiantamentoGrid.getCheckedRows();
                
                adiantametoDisponiveis.forEach(function(valores){
                    let adDisponiveleSelecionado = adiantamentosSelecionados.find(adiantamentosSelecionados => adiantamentosSelecionados.NRNOTAFISC == valores.NRNOTAFISC);
                    if(adDisponiveleSelecionado){
                        if(valores.VRTOTDISP < 0){
                            adiantamentosNaoDisponiveis.push(valores);
                        } 
                    } else {
                        adiantamentosSelecionados.forEach(function(dados){
                            let existe = adiantametoDisponiveis.find(element => element.NRNOTAFISC == dados.NRNOTAFISC);
                            if(!existe){
                                let adicionado = adiantamentosNaoDisponiveis.find(dado => dado.NRNOTAFISC == dados.NRNOTAFISC);
                                if(!adicionado){
                                    adiantamentosNaoDisponiveis.push(valores);
                                }
                            }
                        });
                    }
                });
                
                if(adiantamentosNaoDisponiveis.length > 0){
                       ScreenService.showMessage("Esse adiantamento não está mais disponível para essa operação!");
                       adiantamentoGrid.reload();
                } else{
                   self.geraAdiantamentoFornecedor();
                }
                
                // var adiantamentoFiltrado = adiantamentos.filter(function(valores){
                //   return valores.NRNOTAFISC == adiantamentoGrid.getLastSelectedRow().NRNOTAFISC;
                // });
                
                // if(adiantamentoFiltrado.length > 0){
                //     adiantamentoFiltrado.forEach(function(dados){
                //       if(dados.VRTOTDISP > 0){
                //           self.geraAdiantamentoFornecedor();
                //       }
                //     });
                // }else{
                //     ScreenService.showMessage("Esse adiantamento não está mais disponível para essa operação!");
                //     adiantamentoGrid.reload();
                // }
            });
        }
    };

    /**
     * Confirma Adiantamento
     */
    self.confirmaDevolucao = function(){
        var devolucaoTabbed = templateManager.container.getWidget('DevolucaoTabbed');
        var devolucaoGrid   = templateManager.container.getWidget('DevolucaoGrid');

        if(devolucaoTabbed.isValid()) {
            if ((devolucaoGrid.getCheckedRows().length <= 0) || (devolucaoTabbed.currentRow.TOTALDESCONTO <= 0)) {
                ScreenService.showMessage('Operação bloqueada. Selecione o(s) registro(s) e informe o(s) valor(es) a ser(em) descontado(s).');
                return;
            }

            self.geraDevolucaoFornecedor();
        }
    };

    /**
     * Gera Adiantamento Fornecedor
     */
    self.geraAdiantamentoFornecedor = function(){

        var tituloGrid         = templateManager.container.getWidget('TitulosAPagarGrid');
        var adiantamentoTabbed = templateManager.container.getWidget('AdiantamentoTabbed');
        var adiantamentoGrid   = templateManager.container.getWidget('AdiantamentoGrid');

        var lastSelectedRowTituloGrid    = tituloGrid.getLastSelectedRow();
        var currentRowAdiantamentoTabbed = adiantamentoTabbed.currentRow;
        var checkedrowsAdiantamentoGrid  = adiantamentoGrid.getCheckedRows();

        var dataSet = [lastSelectedRowTituloGrid, currentRowAdiantamentoTabbed, checkedrowsAdiantamentoGrid];

        General.callRestEngine('fin_contaPagar#/geraAdiantamentoFornecedor', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');
            ScreenService.closePopup();
            
            tituloGrid.unSelectAll();
            self.unsetStripeDataFooter();
            tituloGrid.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Gera Devolução Fornecedor
     */
    self.geraDevolucaoFornecedor = function(){
        var tituloGrid      = templateManager.container.getWidget('TitulosAPagarGrid');
        var devolucaoTabbed = templateManager.container.getWidget('DevolucaoTabbed');
        var devolucaoGrid   = templateManager.container.getWidget('DevolucaoGrid');

        var lastSelectedRowTituloGrid = tituloGrid.getLastSelectedRow();
        var currentRowDevolucaoTabbed = devolucaoTabbed.currentRow;
        var checkedrowsDevolucaoGrid  = devolucaoGrid.getCheckedRows();

        var dataSet = [lastSelectedRowTituloGrid, currentRowDevolucaoTabbed, checkedrowsDevolucaoGrid];

        General.callRestEngine('fin_contaPagar#/geraDevolucaoFornecedor', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');
            ScreenService.closePopup();
            tituloGrid.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Consulta Contabilização
     */
    self.consultaContabilizacao = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {
            var contabil = [];
            var contctb = 0;

            for (var i = 0; i < checkedrows.length; i++){
                if (checkedrows[i].NRLANCTBDOC) {
                    contabil.push({});
                    contabil[contctb].CDEMPRESA  = checkedrows[i].CDEMPRESA;
                    contabil[contctb].NRLANCCONT = checkedrows[i].NRLANCTBDOC;
                    contctb++;
                }
            }
            if(contabil.length > 0) {
                contabilizacaoController.exibeContabilizacao(contabil);
            }else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };

    /**
     * Habilita Action Títulos a Pagar
     */
    self.habilitaActionsTitulo = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        var checkedrows = widget.getCheckedRows();
        var cont_assoc = 0;

        var actions = {
            associacao : false,
            desassociar : false,
            prorrogacao : false,
            remessa : false,
            borderoremessa : false,
            consremessa : false,
            cancremessa : false,
            baixa : false,
            cancbaixa : false,
            consctb : false,
            anteforn: false,
            habtitulo: false,
            destitulo: false,
            cancante: false,
            impretorno : true,
            cadauto : true,
            cadguiatrib : false
        };

        if(checkedrows.length > 0) {
            for (var i in checkedrows){

                actions = self.validaActionsSituacaoTitulo(checkedrows[i], actions);

                actions = self.validaActionsRemessaTitulo(checkedrows[i], actions);

                actions = self.validaActionsAssociacaoTitulo(checkedrows[i], actions);
                if(actions.associacao) {
                    cont_assoc++;
                    actions.associacao = cont_assoc > 1;
                }

                actions = self.validaActionsContabilizacaoTitulo(checkedrows[i], actions);

            }
        }

        widget.getAction('Associar').isVisible                = actions.associacao;
        widget.getAction('Desassociar').isVisible             = actions.desassociacao;
        widget.getAction('Prorrogar').isVisible               = actions.prorrogacao;
        widget.getAction('CadastrarGuiaTributo').isVisible    = actions.cadguiatrib;
        widget.getAction('Remessa').isVisible                 = actions.remessa;
        widget.getAction('BorderoRemessa').isVisible          = actions.borderoremessa;
        //widget.getAction('ConsultarRemessa').isVisible        = actions.consremessa;
        widget.getAction('CancelarRemessa').isVisible         = actions.cancremessa;
        widget.getAction('Baixar').isVisible                  = actions.baixa;
        widget.getAction('CancelarBaixa').isVisible           = actions.cancbaixa;
        widget.getAction('ConsultarCtbTitulo').isVisible      = actions.consctb;
        widget.getAction('AntecipacaoFornecedor').isVisible   = actions.anteforn;
        widget.getAction('HabilitarTitulo').isVisible         = actions.habtitulo;
        widget.getAction('DesabilitarTitulo').isVisible       = actions.destitulo;
        widget.getAction('CancelarAntecipacaoForn').isVisible = actions.cancante;
        widget.getAction('ImportarRetornoTitulo').isVisible   = actions.impretorno;
        widget.getAction('CadastramentoAutomatico').isVisible = actions.cadauto;
    };

    /**
     * Valida Actions Situacao Titulo
     */
    self.validaActionsSituacaoTitulo = function(checkedrows, actions){
        var listaIdbaixautil = ['I','G','H','U','V','K','X', 'J'];
        if (checkedrows.IDSTATBPAG == 'A') {
            actions.prorrogacao = true;
            actions.emissao     = true;
            //TO DO mais codigos de impostos
            if(listaIdbaixautil.includes(checkedrows.IDBAIXAUTILP))
                actions.cadguiatrib = true;
            if(idhabibole_geral == 'N')
                actions.baixa       = true;
            else{
                if (checkedrows.IDHABIBPAG == 'N')
                     actions.habtitulo   = true;
                else{ 
                    actions.destitulo   = true;
                    actions.baixa       = true;
                }
            }
            if(!checkedrows.NRLANCANTECIPA)
                 actions.anteforn = true;
            else actions.cancante = true;
        }else if (checkedrows.IDSTATBPAG == 'P'){
            actions.baixa       = true;
            actions.cancbaixa   = true;
        }else if (checkedrows.IDSTATBPAG == 'B'){
            actions.cancbaixa   = true;
        }

        return actions;
    };

    /**
     * Valida Actions Remessa Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsRemessaTitulo = function(checkedrows, actions){
        if (checkedrows.IDSITUCNABP == '00') {
            if(checkedrows.IDSTATBPAG == 'A')
                actions.remessa = true;
        }else if (checkedrows.IDSITUCNABP != '90'){
            actions.borderoremessa = true;
            actions.cancremessa = true;
            actions.consremessa = true;
        }else{ 
            actions.borderoremessa = true;
            actions.consremessa = true;
        }

        return actions;
    };

    /**
     * Valida Actions Associacao Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsAssociacaoTitulo = function(checkedrows, actions){
        if(checkedrows.IDSTATBPAG == 'A')
            if (checkedrows.IDASSOCBOLPAG != 'S')
                actions.associacao = true;
            else actions.desassociacao = true;

        return actions;
    };

    /**
     * Valida Actions Contabilizacao Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsContabilizacaoTitulo = function(checkedrows, actions){
        if (checkedrows.NRLANCTBDOC != null)
            actions.consctb = true;

        return actions;
    };








    /**
     * Simula Parcelamento Automatico
     * @param widget
     */
    self.simulaParcelamentoAutomatico = function(widget){
        if(widget.isValid()){
            var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
            var widgetParcelamento = templateManager.container.getWidget('ParcelamentoGrid');
            var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');

            // Varíaveis dos Campos em Tela
            var nrparcelas  = widget.getField('NRPARCELAS').value();
            var dtprimvenc  = widget.getField('DTPRIMVENC').value();
            var cdmotiparc  = widget.getField('CDMOTIPARC').value();
            var dsmotiparc  = widget.getField('DSMOTIPARC').value();
            var vrfinalbpag = parseFloat(widget.getField('VRFINALBPAG').value()).toFixed(2);
            var idproxvenc  = widget.getField('IDPROXVENC').value();
            var nrdiasvenc  = widget.getField('NRDIASVENC').value();

            // Variavel do DataSource
            var datasource = [];
            var dtvencbolpag = moment(dtprimvenc, 'DD/MM/YYYY').format('DD/MM/YYYY');

            // Calculo de Valores das Parcelas
            var vrparcela = parseFloat(vrfinalbpag / nrparcelas).toFixed(2);
            var vrtotparc = vrparcela * nrparcelas;
            var vrultparc = ((vrtotparc != vrfinalbpag)?
                parseFloat((vrfinalbpag)-(vrtotparc)+parseFloat(vrparcela)).toFixed(2)
                : vrparcela);

            for(var i = 1; i <= nrparcelas; i++){
                datasource[i-1] = {
                    'NRORG'        : widgetTitulos.currentRow.NRORG,
                    'CDFILIAL'     : widgetTitulos.currentRow.CDFILIAL,
                    'NRLANBOLPAG'  : ((i == 1)?
                        widgetTitulos.currentRow.NRLANBOLPAG
                        :null),
                    'NRORIGBOLPAG' : widgetTitulos.currentRow.NRLANBOLPAG,
                    'NRPARCBOLPAG'     : retornaNumeroParcela(i),
                    'NRPARCBOLPAGVISU' : ("00"+ i.toString()).substr(-3),
                    'DTVENBOLPAG'  : dtvencbolpag,
                    'VRFINALBPAG'  : ((i != nrparcelas)?
                        vrparcela
                        : vrultparc),
                    'CDMOTIPARC'  : cdmotiparc,
                    'DSMOTIPARC'  : dsmotiparc
                };

                dtvencbolpag = ((idproxvenc == 'F')?
                    moment(dtvencbolpag, 'DD/MM/YYYY').add(1, 'months').format('DD/MM/YYYY')
                    : moment(dtvencbolpag, 'DD/MM/YYYY').add(nrdiasvenc, 'days').format('DD/MM/YYYY'));
            }

            if(datasource.length > 0) {
                var parcelamentoAutomaticoGrid = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');
                parcelamentoAutomaticoGrid.dataSource.data = datasource;
                widgetParcelamentoAutomatico.getAction('finish').isVisible = true;
                templateManager.updateTemplate();
            }
        }
    };

    /**
     * Altera Campo Proximos Vencimentos
     * @param widget
     */
    self.alteraCampoProximosVencimentos = function(widget){
        var idproxvenc = widget.getField('IDPROXVENC').value();

        if(idproxvenc == 'F') {
            widget.getField('NRDIASVENC').readOnly = true;
            delete widget.getField('NRDIASVENC').validations.required;
            delete widget.getField('NRDIASVENC').validations.min;
            widget.getField('NRDIASVENC').setValue(0);
        }else{
            widget.getField('NRDIASVENC').readOnly = false;
            widget.getField('NRDIASVENC').validations.required = true;
            widget.getField('NRDIASVENC').validations.min = {"limit" : 0};
            widget.getField('NRDIASVENC').setValue(30);
        }
        
        templateManager.updateTemplate();
    };

    /**
     * Abre Parcelamento Automatico
     */
    self.abreParcelamentoAutomatico = function(){
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
        
        self.buscaAjusteTitulo(widgetTitulosAPagar.currentRow.CDFILIAL, widgetTitulosAPagar.currentRow.NRLANBOLPAG).then(function (response_ajbolpag){
            var ajbolpag = response_ajbolpag.dataset.ajbolpag[0];
            if(!ajbolpag) {
                var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoForm');
                widgetParcelamentoAutomatico.isVisible = true;
                widgetParcelamentoAutomatico.editing = true;
                ScreenService.openSwipe(widgetParcelamentoAutomatico);
                self.buscaPadraoParcelamentoAutomatico(widgetParcelamentoAutomatico);
            }else ScreenService.showMessage('Operação bloqueada. Não é possível parcelar título com ajuste.');
        });
    };

    /**
     * Busca Padrão Parcelamento Automático
     */
    self.buscaPadraoParcelamentoAutomatico = function (widget){
        var widgetTitulos = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetParcelamento = templateManager.container.getWidget('ParcelamentoGrid');
        var widgetParcelamentoAutomaticoGrid = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');

        widget.getField('NRPARCELAS').setValue(2);
        widget.getField('CDMOTIPARC').setValue('');
        widget.getField('DSMOTIPARC').setValue('');
        widget.getField('VRFINALBPAG').setValue(widgetParcelamento.dataSource.data[0].VRFINALBPAG);
        widget.getField('DTPRIMVENC').setValue(widgetParcelamento.dataSource.data[0].DTVENBOLPAG);
        widget.getField('IDPROXVENC').setValue('F');
        widget.getField('NRDIASVENC').setValue(0);

        widgetParcelamentoAutomaticoGrid.getAction('finish').isVisible = false;
        widgetParcelamentoAutomaticoGrid.dataSource.data = [];

        templateManager.updateTemplate();
    };

    /**
     * Finaliza Parcelamento Automático Títulos a Pagar/Pagos
     */
    self.finalizaParcelamentoAutomatico = function(){
        var widgetParcelamentoAutomaticoGrid = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');
        var widgetParcelamentoAutomaticoForm = templateManager.container.getWidget('ParcelamentoAutomaticoForm');
        var widgetParcelamentoGrid           = templateManager.container.getWidget('ParcelamentoGrid');
        var widgetTitulosAPagarGrid          = templateManager.container.getWidget('TitulosAPagarGrid');

        for(var i in widgetParcelamentoAutomaticoGrid.dataSource.data) {
            if ($.currency.unmaskString(widgetParcelamentoAutomaticoGrid.dataSource.data[i].VRFINALBPAG) <= 0) {
                ScreenService.showMessage('Operação bloqueada. Existe(m) parcela(s) menor(es) ou igual(is) a zero.');
                return;
            }
        }

        Pag00001_TitulosAPagar_Service.finalizaParcelamentoTitulo(widgetParcelamentoAutomaticoGrid.dataSource.data).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');

            for(var i in widgetParcelamentoAutomaticoGrid.dataSource.data)
                if(widgetParcelamentoAutomaticoGrid.dataSource.data[i].NRLANBOLPAG == widgetTitulosAPagarGrid.currentRow.NRLANBOLPAG)
                    widgetTitulosAPagarGrid.currentRow.VRFINALBPAG = widgetParcelamentoAutomaticoGrid.dataSource.data[i].VRFINALBPAG;
                    widgetTitulosAPagarGrid.currentRow.VRBOLPAG = widgetParcelamentoAutomaticoGrid.dataSource.data[i].VRFINALBPAG;

            ScreenService.closeSwipe();
            widgetParcelamentoAutomaticoForm.editing = false;
            widgetParcelamentoGrid.reload();
            widgetTitulosAPagarGrid.reload();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Nova Parcela Títulos a Pagar
     * @param args
     */
    self.geraParcelamentoAutomatico = function(args){
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetParcelamento  = templateManager.container.getWidget('ParcelamentoGrid');

        if((widgetTitulosAPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosAPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulosAPagar.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação bloqueada. Título previsto não pode ser alterado.');
            return;
        }
        if(widgetParcelamento.dataSource.data.length > 1) {
            ScreenService.showMessage('Operação bloqueada. Título parcelado não pode utilizar o parcelamento automático.');
            return;
        }

        if(!widgetTitulosAPagar.currentRow.NRCNABPAGREM){
            self.abreParcelamentoAutomatico();
        }else{
            var paramsRem  = {
                nrorg       : getLocalVar('NRORG'),
                cdfilial    : getLocalVar('CDFILIAL'),
                cdoperador  : getLocalVar('CDOPERADOR')
            };
            var currentRowRem = [paramsRem, [widgetTitulosAPagar.currentRow]];

            ScreenService.confirmMessage('O título está vinculado a uma remessa. Deseja removê-lo para prosseguir?', 'question', function () {
                Pag00001_TitulosAPagar_Service.excluiTituloPagRemessa(currentRowRem).then(function () {
                    self.abreParcelamentoAutomatico();
                    widgetTitulosAPagar.currentRow.IDSITUCNABP = '00';
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }
    };

    /**
     * Abre Cadastramento Automatico
     */
    self.abreCadastramentoAutomatico = function(){
        if(!templateManager.container.getWidget('TitulosAPagarGrid').appliedFilter.FILTER_ROW.CDEMPRESA) {
            ScreenService.showMessage('Operação bloqueada. Empresa não selecionada no filtro.');
            return;
        }

        var widgetTitulosAPagarGrid       = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetCadastramentoAutomatico = templateManager.container.getWidget('CadastramentoAutomaticoForm');
        widgetCadastramentoAutomatico.isVisible = true;
        ScreenService.openPopup(widgetCadastramentoAutomatico);
        self.buscaPadraoCadastramentoAutomatico(widgetTitulosAPagarGrid, widgetCadastramentoAutomatico);
    };

    /**
     * Busca Padrão Cadastramento Automático
     */
    self.buscaPadraoCadastramentoAutomatico = function (widgetTitulosAPagarGrid, widgetCadastramentoAutomatico){
        var checkedRows = widgetTitulosAPagarGrid.getCheckedRows();
        var lastChecked = checkedRows ? checkedRows[0] : null;

        var cdfilirece      = '';
        var nrtamnumnfutdfe = '';
        var cdempresarece   = '';
        var cdinscestarece  = '';
        var cdinscmunirece  = '';
        var nmfilialrece    = '';
        var cdforneced      = '';
        var nrinsjurforn    = '';
        var idtpijurforn    = '';
        var nmrazsocforn    = '';
        var cdtipooper      = '';
        var nmtipooper      = '';
        var cdserie         = '';
        var dsserie         = '';
        var vrbolpag        = '';
        var dtprimvenc      = moment().format('DD/MM/YYYY');
        var cdtpctpagar     = '';
        var dstpctpagar     = '';
        var dsdefactpag     = '';
        var dscontapag      = '';
        var cdclassfina     = '';

        if(lastChecked) {

            var parfilempr  = self.buscaParamfilialFilialEmprcntrfil(lastChecked.CDEMPRESA, lastChecked.CDFILIRECE);
            var tipooperac  = self.buscaTipooperac(lastChecked.CDTIPOOPER);
            var serieentr   = self.buscaSerieentr(lastChecked.CDSERIE);
            var tipoctpagar = self.buscaTipoctpagar(lastChecked.CDTPCTPAGAR);

            ZHPromise.all([parfilempr, tipooperac, serieentr, tipoctpagar]).then(function(response){
                parfilempr   = response[0].dataset.paramfilial_filial_emprcntrfilpag[0];
                tipooperac   = response[1].dataset.tipooperac[0];
                serieentr    = response[2].dataset.serieentr[0];
                tipoctpagar  = response[3].dataset.tipoctpagar[0];

                cdfilirece      = (parfilempr) ? parfilempr.CDFILIAL   : '';
                nmfilialrece    = (parfilempr) ? parfilempr.NMFILIEMIT : '';
                cdempresarece   = (parfilempr) ? parfilempr.CDEMPRESA  : '';
                cdinscestarece  = (parfilempr) ? parfilempr.CDINSCESTA : '';
                cdinscmunirece  = (parfilempr) ? parfilempr.CDINSCMUNI : '';
                nrtamnumnfutdfe = (parfilempr) ? parfilempr.NRTAMNUMNFUTDFE > 0 ? parfilempr.NRTAMNUMNFUTDFE : 9 : '';
                cdforneced      = lastChecked.CDFORNECED;
                nrinsjurforn    = lastChecked.NRINSJURFORN;
                idtpijurforn    = lastChecked.IDTPIJURFORN;
                nmrazsocforn    = lastChecked.NMRAZSOCFORN;
                nmtipooper      = (tipooperac) ? tipooperac.NMTIPOOPER : '';
                cdtipooper      = (tipooperac) ? tipooperac.CDTIPOOPER : '';
                dsserie         = (serieentr)  ? serieentr.DSSERIEENTR : '';
                cdserie         = (serieentr)  ? serieentr.CDSERIEENTR : '';
                vrbolpag        = lastChecked.VRBOLPAG;
                dtprimvenc      = lastChecked.DTVENBOLPAG;
                cdtpctpagar     = (tipoctpagar) ? tipoctpagar.CDTPCTPAGAR : '';
                dstpctpagar     = (tipoctpagar) ? tipoctpagar.DSTPCTPAGAR : '';
                dsdefactpag     = (tipoctpagar) ? tipoctpagar.DSDEFACTPAG : '';
                dscontapag      = (tipoctpagar) ? tipoctpagar.DSDEFACTPAG : '';
                cdclassfina     = (tipoctpagar) ? tipoctpagar.CDCLASSFINA : '';

                self.setaPadraoCadastramentoAutomatico(
                    widgetCadastramentoAutomatico, nrtamnumnfutdfe, cdempresarece, cdinscestarece, cdinscmunirece,
                    nmfilialrece, cdfilirece, cdforneced, nrinsjurforn, idtpijurforn, nmrazsocforn, nmtipooper,
                    cdtipooper, dsserie, cdserie, vrbolpag, dtprimvenc, dstpctpagar, cdtpctpagar, dsdefactpag,
                    dscontapag, cdclassfina
                );
            });
        }else{
            self.setaPadraoCadastramentoAutomatico(
                widgetCadastramentoAutomatico, nrtamnumnfutdfe, cdempresarece, cdinscestarece, cdinscmunirece,
                nmfilialrece, cdfilirece, cdforneced, nrinsjurforn, idtpijurforn, nmrazsocforn, nmtipooper,
                cdtipooper, dsserie, cdserie, vrbolpag, dtprimvenc, dstpctpagar, cdtpctpagar, dsdefactpag,
                dscontapag, cdclassfina
            );
        }


    };

    /**
     * Seta Padrão Cadastramento Automático
     */
    self.setaPadraoCadastramentoAutomatico = function (
        widgetCadastramentoAutomatico, nrtamnumnfutdfe, cdempresarece, cdinscestarece, cdinscmunirece,
        nmfilialrece, cdfilirece, cdforneced, nrinsjurforn, idtpijurforn, nmrazsocforn, nmtipooper,
        cdtipooper, dsserie, cdserie, vrbolpag, dtprimvenc, dstpctpagar, cdtpctpagar, dsdefactpag,
        dscontapag, cdclassfina
    ){
        widgetCadastramentoAutomatico.getField('NRORG').setValue(getLocalVar('NRORG'));
        widgetCadastramentoAutomatico.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widgetCadastramentoAutomatico.getField('CDOPERACTPG').setValue(getLocalVar('CDOPERADOR'));
        widgetCadastramentoAutomatico.getField('NRTITULOS').setValue(1);
        widgetCadastramentoAutomatico.getField('NRTAMNUMNFUTDFE').setValue(nrtamnumnfutdfe);
        widgetCadastramentoAutomatico.getField('CDEMPRESARECE').setValue(cdempresarece);
        widgetCadastramentoAutomatico.getField('CDINSCESTARECE').setValue(cdinscestarece);
        widgetCadastramentoAutomatico.getField('CDINSCMUNIRECE').setValue(cdinscmunirece);
        widgetCadastramentoAutomatico.getField('NMFILIALRECE').setValue(nmfilialrece);
        widgetCadastramentoAutomatico.getField('CDFILIRECE').setValue(cdfilirece);
        widgetCadastramentoAutomatico.getField('IDSTATUS').setValue('A');
        widgetCadastramentoAutomatico.getField('CDFORNECED').setValue(cdforneced);
        widgetCadastramentoAutomatico.getField('NRINSJURFORN').setValue(nrinsjurforn);
        widgetCadastramentoAutomatico.getField('IDTPIJURFORN').setValue(idtpijurforn);
        widgetCadastramentoAutomatico.getField('NMRAZSOCFORN').setValue(nmrazsocforn);
        widgetCadastramentoAutomatico.getField('NMTIPOOPER').setValue(nmtipooper);
        widgetCadastramentoAutomatico.getField('CDTIPOOPER').setValue(cdtipooper);
        widgetCadastramentoAutomatico.getField('DSSERIE').setValue(dsserie);
        widgetCadastramentoAutomatico.getField('CDSERIE').setValue(cdserie);
        widgetCadastramentoAutomatico.getField('VRNOTAFISC').setValue(vrbolpag);
        widgetCadastramentoAutomatico.getField('DTPRIMVENC').setValue(dtprimvenc);
        widgetCadastramentoAutomatico.getField('IDPROXVENC').setValue('F');
        widgetCadastramentoAutomatico.getField('NRDIASVENC').setValue(0);
        widgetCadastramentoAutomatico.getField('DSTPCTPAGAR').setValue(dstpctpagar);
        widgetCadastramentoAutomatico.getField('DSDEFACTPAG').setValue(dsdefactpag);
        widgetCadastramentoAutomatico.getField('DSCONTAPAG').setValue(dscontapag);
        widgetCadastramentoAutomatico.getField('CDTPCTPAGAR').setValue(cdtpctpagar);
        widgetCadastramentoAutomatico.getField('CDCLASSFINA').setValue(cdclassfina);

        templateManager.updateTemplate();
    };

    /**
     * Gera Cadastramento Automático Títulos a Pagar/Pagos
     */
    self.geraCadastramentoAutomatico = function(){
        var widgetTitulosAPagarGrid     = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetCadastramentoAutomatico = templateManager.container.getWidget('CadastramentoAutomaticoForm');

        if(widgetCadastramentoAutomatico.isValid()) {
            Pag00001_TitulosAPagar_Service.geraCadastramentoAutomatico(widgetCadastramentoAutomatico.currentRow).then(function () {
                ScreenService.showMessage('Registros salvos com sucesso.');
                ScreenService.closePopup(widgetCadastramentoAutomatico);
                ScreenService.closePopup();

                widgetCadastramentoAutomatico.isVisible = false;

                widgetTitulosAPagarGrid.unSelectAll();
                self.unsetStripeDataFooter();
                widgetTitulosAPagarGrid.activate();
                widgetTitulosAPagarGrid.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }
    };

    self.voltarTitulosPagar = function(){
        var titulos = templateManager.container.getWidget('TitulosAPagarGrid');
        ScreenService.goBack().then(function(){
            titulos.reload();
        });
    };

    /**
     * Abre Importação Retorno Título
     */
    self.abreImportacaoRetornoTitulo = function(){
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
        /*widgetTitulosAPagar.appliedFilter.FILTER_ROW.CDEMPRESA;
        widgetTitulosAPagar.appliedFilter.FILTER_ROW.NMEMPRESA;*/

        var widgetImportacaoRetorno = templateManager.container.getWidget('ImportacaoRetornoForm');
        widgetImportacaoRetorno.isVisible = true;
        widgetImportacaoRetorno.editing = true;
        ScreenService.openSwipe(widgetImportacaoRetorno);
        ScreenService.closePopup(true);
        self.buscaPadraoImportacaoRetornoTitulo(widgetTitulosAPagar, widgetImportacaoRetorno);
    };

    /**
     * Busca Padrão Importação Retorno Título
     */
    self.buscaPadraoImportacaoRetornoTitulo = function(widgetTitulosAPagar, widgetImportacaoRetorno){
        widgetImportacaoRetorno.getField('CDEMPRESA').setValue(widgetTitulosAPagar.appliedFilter.FILTER_ROW.CDEMPRESA);
        widgetImportacaoRetorno.getField('NMEMPRESA').setValue(widgetTitulosAPagar.appliedFilter.FILTER_ROW.NMEMPRESA);
        widgetImportacaoRetorno.getField('NMBANCO').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABPAG').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABPAG').readOnly = true;

        self.deleteDataRetorno();

        templateManager.updateTemplate();
    };

    /**
     *
     * @param widgetImportacaoRetorno
     */
    self.validaVisualizacaoImportacao = function(widgetImportacaoRetorno){
        widgetImportacaoRetorno.getField('DSARQCNABPAG').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABPAG').readOnly = !widgetImportacaoRetorno.getField('NMBANCO').value();

        self.deleteDataRetorno();

        templateManager.updateTemplate();
    };

    /**
     * Prepara Arquivo de Retorno
     * @param widget
     */
    self.preparaArquivoRetorno = function(widget){
        var route = 'fin_contaPagar#/carregaRetorno';
        var arqRetorno = widget.currentRow.DSARQCNABPAG[0];
        var conteudoArquivo = arqRetorno.b64File.split(',')[1];
        var empresa = templateManager.container.getWidget('ImportacaoRetornoForm').currentRow.CDEMPRESA;
        var row = {
            'CDEMPRESA': empresa,
            'CDBANCO': widget.currentRow.CDBANCO,
            'CDBANCCNAB': widget.currentRow.CDBANCCNAB,
            'CNABRETORNO': conteudoArquivo
        };

        General.callRestEngine(route, 'Row', row)
            .then(function(response){
                self.setDataSourceRetorno(response.dataset.BOLETOS);
                self.setDataSourceOcorrencias(response.dataset.OCORRENCIAS);
                widget.widgets[0].SITUACAOCNAB = response.dataset.SITUACAOCNAB[0];
            }).catch(function(erro){
                widget.currentRow.DSARQCNABPAG = [];
                ScreenService.showMessage(erro.data.error);
            });
    };

    /**
     * Data Source Retorno
     * @param boletos
     */
    self.setDataSourceRetorno = function(boletos){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        widgetBoletos.dataSource.data = [];
        widgetBoletos.dataSource.data = boletos;
    };

    /**
     * Data Source Ocorrências
     * @param ocorrencias
     */
    self.setDataSourceOcorrencias = function(ocorrencias){
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = ocorrencias;
    };

    /**
     * Insere Retorno Título
     */
    self.insereRetornoTitulo = function(){
        var widgetTitulos     = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetBoletos     = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');

        if (widgetBoletos.getParent().isValid()){
            if (widgetBoletos.dataSource.data.length > 0) {
                if (self.validaSituacaoCnab(widgetBoletos)) {

                    var dataSet = [];
                    var route = 'fin_contaPagar#/insereRetorno';
                    var form = {
                        'DSARQCNABPAG': widgetBoletos.getParent().currentRow.DSARQCNABPAG[0].b64File.split(',')[0],
                        'NOMEARQUIVO': widgetBoletos.getParent().currentRow.DSARQCNABPAG[0].name,
                        'CDBANCCNAB': widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDBANCO': widgetBoletos.getParent().currentRow.CDBANCO,
                        'CDEMPRESA': widgetBoletos.getParent().currentRow.CDEMPRESA
                    };

                    dataSet.push(form);
                    dataSet.push(widgetBoletos.dataSource.data);
                    dataSet.push(widgetOcorrencias.ocorrencias);

                    General.callRestEngine(route, 'DataSet', dataSet)
                        .then(function () {
                            ScreenService.goBack().then(function () {
                                ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                                widgetBoletos.getParent().isVisible = false;
                                widgetTitulos.unSelectAll();
                                self.unsetStripeDataFooter();
                                widgetTitulos.activate();
                                widgetTitulos.reload();
                            });
                        }).catch(function (erro) {
                            ScreenService.showMessage(erro.data.error);
                        });

                } else ScreenService.showMessage('Registro(s) do arquivo de retorno possui(em) ocorrência(s) que não possibilita(m) o salvamento.');
            }else ScreenService.showMessage('Não existe(m) título(s) para importação.');
        }
    };

    /**
     * Valída Situação CNAB
     * @param boletos
     * @returns {boolean}
     */
    self.validaSituacaoCnab = function(boletos){
        var boletoValido = !boletos.dataSource.data.every(function(elem) {return elem.SITUACAOBOLETO == false;});
        return (boletos.SITUACAOCNAB && boletoValido) ? true : false;
    };

    /**
     * Deleta Data Retorno
     */
    self.deleteDataRetorno = function(){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetBoletos.dataSource.data = [];
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = [];
    };

    /**
     * Abre Ocorrências
     * @param widget
     */
    self.abreOcorrencias = function(widget){
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == widget.currentRow.CDLINHA || ocorrencia.GERAL);
        });
        templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid').activate();
    };

    /**
     * Edita Conta Documento
     * @param args
     */
    self.editaContaDocumento = function(args){
        var widgetContaDocumentos = args.owner.widget;
        var widgetTitulosPagar    = templateManager.container.getWidget('TitulosAPagarGrid');

        if((widgetTitulosPagar.currentRow.IDSTATBPAG == 'B') || (widgetTitulosPagar.currentRow.IDSTATBPAG == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado.');
            return;
        }
        if(widgetTitulosPagar.currentRow.IDSITUCNABP != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado, via CNAB, não pode ser alterado.');
            return;
        }

        General.edit(widgetContaDocumentos);
        General.changeWidgetAcessibility(widgetContaDocumentos.widgets, true);
    };

    /**
     * Salca Conta Documento
     * @param args
     */
    self.salvaContaDocumento = function(args){
        var widgetContaDocumentosGrid = args.owner.widget;

        if(widgetContaDocumentosGrid.isValid()){
            Pag00001_TitulosAPagar_Service.salvaContaDocumento(widgetContaDocumentosGrid.currentRow).then(function () {
                ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                ScreenService.goBack();

                widgetContaDocumentosGrid.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Seta Data Souce Stripe
     * @param widget
     */
    self.setStripeData = function(widget) {
        var currentRow = widget.appliedFilter.FILTER_ROW;
        currentRow.DTVENBOLPAG = (currentRow.DTVENBOLPAG) ? currentRow.DTVENBOLPAG : {'start': '', 'end': ''};
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMEMPRESA   = currentRow.CDEMPRESA + ' - ' + currentRow.NMEMPRESA;
        stripeWidget.currentRow.DTVENBOLPAG = 'De ' + currentRow.DTVENBOLPAG.start + ' até ' + currentRow.DTVENBOLPAG.end;
    };
    
    self.setStripeDataFooter = function(status) {
        var widget          = templateManager.container.getWidget("TitulosAPagarGrid");
        var stripeWidget    = templateManager.container.getWidget("stripeFooter");
        var lastSelectedRow = widget.getLastSelectedRow();
        var checkedRows     = widget.getCheckedRows();
        var dstitulo        = '';
        var totalOriginal   = 0;
        var totalAjuste     = 0;
        var totalLiquido    = 0;
        
        checkedRows.forEach(function(item){
            totalOriginal += self.formataValor(item.VRBOLPAG);
            totalLiquido  += self.formataValor(item.VRFINALBPAG);
        });
        
        totalAjuste = totalLiquido - totalOriginal;
        
        if(((Object.keys(lastSelectedRow).length === 0 && lastSelectedRow.constructor === Object) && (checkedRows.length > 1)) != true){ 
            
            var dsboletpag = lastSelectedRow.DSBOLETPAG?" - " +lastSelectedRow.DSBOLETPAG:'';
                stripeWidget.getField('DSTITULO').isVisible = true;
                dstitulo = moment(lastSelectedRow.DTVENBOLPAG, 'DD/MM/YYYY').format('DD/MM/YYYY') + "  -  " +
                                lastSelectedRow.NMRAZSOCFORN + "  -  " +
                                self.numberToReal(lastSelectedRow.VRFINALBPAG) +
                                dsboletpag;
        }
        
        stripeWidget.currentRow.DSTITULO  = dstitulo;
        if (status){
            stripeWidget.getField('DSTITULO').isVisible = false;
            stripeWidget.currentRow.DSTITULO = '';
        }
        stripeWidget.currentRow.NRTITUTOT = checkedRows.length;
        stripeWidget.currentRow.VRORIGTOT = self.numberToReal(totalOriginal);
        stripeWidget.currentRow.VRAJUSTOT = self.numberToReal(totalAjuste);
        stripeWidget.currentRow.VRLIQTOT  = self.numberToReal(totalLiquido);
    };
    
    self.unsetStripeDataFooter = function() {
        var widget        = templateManager.container.getWidget("TitulosAPagarGrid");
        var stripeWidget  = templateManager.container.getWidget("stripeFooter");
        var checkedrows   = widget.getCheckedRows();
        var dstitulo      = '';
        var totalOriginal = 0;
        var totalAjuste   = 0;
        var totalLiquido  = 0;
        
        if(checkedrows.length > 0){
            checkedrows.forEach(function(item){
                totalOriginal += self.formataValor(item.VRBOLPAG);
                totalLiquido  += self.formataValor(item.VRFINALBPAG);
            });
            totalAjuste = totalLiquido - totalOriginal;
        }
        
        stripeWidget.getField('DSTITULO').isVisible = false;
        stripeWidget.currentRow.NRTITUTOT = checkedrows.length;
        stripeWidget.currentRow.VRORIGTOT = self.numberToReal(totalOriginal);
        stripeWidget.currentRow.VRAJUSTOT = self.numberToReal(totalAjuste);
        stripeWidget.currentRow.VRLIQTOT  = self.numberToReal(totalLiquido);
    };
    
    /**
     * Formata Número - Ex. 1500 p/ 1.500,00
     */
    self.numberToReal = function(numero) {
        numero = parseFloat(numero).toFixed(2).split('.');
        numero[0] = numero[0].split(/(?=(?:...)*$)/).join('.');
        return numero.join(',');
    };

    /**
     * Cancela Título Retorno
     */
    self.cancelaTituloRetorno = function(){
        var route = 'fin_contaPagar#/cancelaTituloRetorno';
        var widgetCnabRetorno        = templateManager.container.getWidget('CnabRetornoForm');
        var widgetCnabRetornoTitulo  = templateManager.container.getWidget('CnabRetornoTituloForm');
        var checkedRowsCnabRetTitulo = widgetCnabRetornoTitulo.getCheckedRows();

        for (var i in checkedRowsCnabRetTitulo) {
            if (checkedRowsCnabRetTitulo[i].IDSITUCNABP == '90') {
                ScreenService.showMessage('Operação bloqueada. Título baixado via CNAB não pode ser cancelado do retorno.');
                return;
            }
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o(s) título(s) do retorno?', 'question', function () {
            var dataset = {
                retorno : widgetCnabRetorno.currentRow,
                boletos : widgetCnabRetornoTitulo.getCheckedRows()
            };

            General.callRestEngine(route, 'DataSet', dataset)
                .then(function(){
                    ScreenService.showMessage('Título(s) cancelado(s) do retorno com sucesso.', 'checked');
                    widgetCnabRetornoTitulo.reload();
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        });
    };

    /**
     * Abre Parcelamento Automatico
     */
    self.abreAntecipacaoFornecedor = function(){
        var widgetAntecipacaoFornecedor = templateManager.container.getWidget('AntecipacaoFornecedorForm');
        ScreenService.closePopup(true);
        widgetAntecipacaoFornecedor.isVisible = true;
        widgetAntecipacaoFornecedor.newRow();
        widgetAntecipacaoFornecedor.__isNew = true;
        ScreenService.openSwipe(widgetAntecipacaoFornecedor);
        self.buscaPadraoAntecipacaoFornecedor(widgetAntecipacaoFornecedor);
    };

    /**
     * Busca Padrao Antecipação de Fornecedor
     */
    self.buscaPadraoAntecipacaoFornecedor = function(widget){
        var widgetTitulosAPagar      = templateManager.container.getWidget('TitulosAPagarGrid');
        var checkedRowsTitulosAPagar = widgetTitulosAPagar.getCheckedRows();

        var vrcontaantecipa = 0;
        var vrliquantecipa  = 0;

        for (var i in checkedRowsTitulosAPagar) {
            vrcontaantecipa += checkedRowsTitulosAPagar[i].VRATUAPAG;
            vrliquantecipa += checkedRowsTitulosAPagar[i].VRFINALBPAG;
        }

        widget.getField('NRORG').setValue(getLocalVar('NRORG'));
        widget.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widget.getField('CDEMPRESA').setValue(widgetTitulosAPagar.appliedFilter.FILTER_ROW.CDEMPRESA);
        widget.getField('VRCONTAANTECIPA').setValue(vrcontaantecipa);
        widget.getField('VRLIQUANTECIPA').setValue(vrliquantecipa);
        widget.getField('DTLANCANTECIPA').setValue(moment().format('DD/MM/YYYY'));
        widget.getField('DTVENCANTECIPA').setValue(moment().format('DD/MM/YYYY'));
        widget.getField('DTEMISANTECIPA').setValue(moment().format('DD/MM/YYYY'));
    };

    /**
     * Finaliza Antecipacao de Fornecedor
     * @param widget
     */
    self.finalizaAntecipacaoFornecedor = function(widget){
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');

        if(widget.isValid()){
            var titulosEmAberto = widgetTitulosAPagar.getCheckedRows().filter(function(titulo){
                return ((titulo.IDSTATBPAG == 'A') && (titulo.IDANTECIPAFORN == 'N'));
            });

            var dataSet = [widget.currentRow, titulosEmAberto];

            General.callRestEngine('fin_contaPagar#/finalizaAntecipacaoFornecedor', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Título(s) antecipado(s) com sucesso.', 'checked');
                ScreenService.goBack();
                
                widget.isVisible = false;
                
                widgetTitulosAPagar.unSelectAll();
                self.unsetStripeDataFooter();
                widgetTitulosAPagar.activate();
                widgetTitulosAPagar.reload();
            }).catch(function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Cancela Antecipacao de Fornecedor
     */
    self.cancelaAntecipacaoFornecedor = function(){
        ScreenService.confirmMessage('Deseja realmente cancelar a(s) antecipação(ões)?', 'question', function() {
            var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');

            var titulosAntecipados = widgetTitulosAPagar.getCheckedRows().filter(function(titulo){
                return ((titulo.IDSTATBPAG == 'A') && (titulo.IDANTECIPAFORN == 'S'));
            });

            var dataSet = [titulosAntecipados];

            General.callRestEngine('fin_contaPagar#/cancelaAntecipacaoFornecedor', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Antecipado(s) cancelada(s) com sucesso.', 'checked');
                ScreenService.closePopup();
                widgetTitulosAPagar.reload();
                
                widgetTitulosAPagar.unSelectAll();
                self.unsetStripeDataFooter();
            }).catch(function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    /**
     * Busca Padrão Filial
     * @param widget
     */
    self.buscaPadraoFilial = function(documento) {
        var titulosAPagarGrid = templateManager.container.getWidget('TitulosAPagarGrid');
        var route = 'fin_contaPagar#/validaFilialLogadaPorEmpresaFiltrada';
        var cdempresa = titulosAPagarGrid.appliedFilter.FILTER_ROW.CDEMPRESA;
        General.callRestEngine(route, 'Row', {'CDEMPRESA': cdempresa}).then(function(response_filial){
            if (response_filial.dataset.filial.length > 0){
                var filial   = response_filial.dataset.filial[0];
                documento.currentRow.CDFILIRECE      = filial.CDFILIAL;
                documento.currentRow.NMFILIALRECE    = filial.NMFILIAL;
                documento.currentRow.CDEMPRESARECE   = filial.CDEMPRESA;
                documento.currentRow.CDINSCESTARECE  = filial.CDINSCESTA;
                documento.currentRow.CDINSCMUNIRECE  = filial.CDINSCMUNI;
                documento.currentRow.NRTAMNUMNFUTDFE = filial.NRTAMNUMNFUTDFE;
                self.changeFilial();
            }else{
                documento.currentRow.CDFILIRECE      = null;
                documento.currentRow.NMFILIALRECE    = null;
                documento.currentRow.CDEMPRESARECE   = null;
                documento.currentRow.CDINSCESTARECE  = null;
                documento.currentRow.CDINSCMUNIRECE  = null;
                documento.currentRow.NRTAMNUMNFUTDFE = null;
                documento.currentRow.NRNOTAFISC      = null;
                documento.currentRow.CDSERIE         = null;
                documento.currentRow.DSSERIE         = null;
            }
        }).catch(function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Altera Ajuste Título
     * @param widget
     */
    self.alteraAjusteTitulo = function(widget){
        if(widget.currentRow.DSAJUCTPAG)
            self.alteraValorAjuste(widget);
    };

    /**
     * Altera Valor Desejavel Título
     * @param widget
     */
    self.alteraValorDesejavel = function(widget){
        var vrajbolpag = 0;
    
        if(widget.currentRow.IDAJUCTPAG == 'A') {
            if ((self.formataValor(widget.currentRow.VRDESEBPAG)) < (self.formataValor(widget.currentRow.VRATUABPAG))){
                vrajbolpag = 0;
                widget.currentRow.VRDESEBPAG = widget.currentRow.VRATUABPAG;
                ScreenService.showMessage('O valor desejável não pode ser menor que o valor atual');
            }
            else if (widget.currentRow.VRDESEBPAG == "") {
                vrajbolpag = 0;
                widget.currentRow.VRDESEBPAG = widget.currentRow.VRATUABPAG;
            } else if ((self.formataValor(widget.currentRow.VRDESEBPAG)) >= (self.formataValor(widget.currentRow.VRATUABPAG))) {
                vrajbolpag = ((self.formataValor(widget.currentRow.VRDESEBPAG) - self.formataValor(widget.currentRow.VRATUABPAG)));
            }
        }
        else if(widget.currentRow.IDAJUCTPAG == 'D') {
            if((self.formataValor(widget.currentRow.VRDESEBPAG)) <= (self.formataValor(widget.currentRow.VRATUABPAG))){
                vrajbolpag = (self.formataValor(widget.currentRow.VRATUABPAG) - self.formataValor(widget.currentRow.VRDESEBPAG));
            }
            else if(widget.currentRow.VRDESEBPAG && widget.currentRow.VRDESEBPAG != "") {
                vrajbolpag = 0;
                widget.currentRow.VRDESEBPAG = widget.currentRow.VRATUABPAG;
                ScreenService.showMessage('O valor desejável não pode ser maior que o valor atual');
            } else if(widget.currentRow.VRDESEBPAG == ""){
                widget.currentRow.VRDESEBPAG = widget.currentRow.VRATUABPAG;
            }
        }

        vrajbolpag = vrajbolpag > 0 ? vrajbolpag : 0;

        widget.currentRow.VRAJBOLPAG = parseFloat(vrajbolpag).toFixed(2);
    };
    
    /**
     * Altera Valor Desejavel Título
     * @param widget
     */
    self.alteraValorAjuste = function(widget){
        var vrdesebpag = widget.currentRow.VRATUABPAG;

        if(widget.currentRow.IDAJUCTPAG == 'A') {
            if(widget.currentRow.VRAJBOLPAG && widget.currentRow.VRAJBOLPAG != "") {
                vrdesebpag = (self.formataValor(widget.currentRow.VRATUABPAG) + self.formataValor(widget.currentRow.VRAJBOLPAG));
            } else {
                vrdesebpag = widget.currentRow.VRATUABPAG;
            } 
        }
        else if(widget.currentRow.IDAJUCTPAG == 'D') {
            if((self.formataValor(widget.currentRow.VRAJBOLPAG)) <= (self.formataValor(widget.currentRow.VRATUABPAG))){
                vrdesebpag = (self.formataValor(widget.currentRow.VRATUABPAG) - self.formataValor(widget.currentRow.VRAJBOLPAG));
            } 
            else if(widget.currentRow.VRAJBOLPAG && widget.currentRow.VRAJBOLPAG != "") {
                vrdesebpag = widget.currentRow.VRATUABPAG;
                widget.currentRow.VRAJBOLPAG = 0;
                ScreenService.showMessage('O valor do ajuste não pode ser maior que o valor atual');
            }
        }

        vrdesebpag = vrdesebpag > 0 ? vrdesebpag : 0;

        widget.currentRow.VRDESEBPAG = vrdesebpag.toFixed(2);
    };


    /**
     * Formata Valor
     * @param valor
     */
    self.formataValor = function(valor){
        if(typeof valor == 'string')
            if(valor.indexOf(',') >= 0)
                valor = valor.replace('.','').replace(',','.');
                
        valor = parseFloat(parseFloat(valor).toFixed(2));

        return valor;
    };

    /**
     * Formata Valor
     * @param valor
     */
    self.abreRateioContaUnica = function(){
        var widgetContaDocumentos  = templateManager.container.getWidget('ContaDocumentosGrid');

        if(widgetContaDocumentos.dataSource.data.length == 1){
            if(widgetContaDocumentos.currentRow.NRLANBOLPAG != widgetContaDocumentos.dataSource.data[0].NRLANBOLPAG){
                widgetContaDocumentos.currentRow = widgetContaDocumentos.dataSource.data[0]?widgetContaDocumentos.dataSource.data[0]:{};
                widgetContaDocumentos.afterMoveRow(widgetContaDocumentos);
                widgetContaDocumentos.view();
            }
        }

    };

    /**
     * Altera Descrição Títulos a Pagar
     */
    self.changeDsboletpag = function(change) {
    	var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
    	widgetTitulosAPagar.currentRow.__changeDsboletpag = change;
    };









    /**
     * Busca Parametrização Tipo de Conta
     * @param args
     * @param widget_name
     */
    self.changeTipoContaDoc = function() {
        var widgetDocumentos = templateManager.container.getWidget('DocumentosForm');
        var widgetParcelamentoDoc = templateManager.container.getWidget('ParcelamentoDocGrid');

        var dscontapag = widgetDocumentos.currentRow.DSDEFACTPAG?widgetDocumentos.currentRow.DSDEFACTPAG:widgetDocumentos.currentRow.DSTPCTPAGAR;

        widgetParcelamentoDoc.dataSource.data.forEach(function(data){
            data.DSCONTAPAG = dscontapag;
        });

    };


    /**
     * Adicionar Parcela Títulos a Pagar
     * @param args
     */
    self.adicionarParcela = function(args){
        var widgetParcelamento = args.owner.widget;
        var widgetDocumentos   = args.owner.widget.getParent();
        
        if(widgetDocumentos.currentRow.VRNOTAFISC <= 0) {
            ScreenService.showMessage('Operação bloqueada. Informe o valor do documento.');
            return;
        }
        
        widgetParcelamento.newRow();
        self.habDesCampos('DocumentosForm', true);
        
        if(widgetParcelamento.dataSource.data.length == 2){
            widgetParcelamento.dataSource.data[0].NRPARCBOLPAG = '01';
            widgetParcelamento.dataSource.data[0].NRPARCBOLPAGVISU = '001';
        }
        
        
        var nrparcbolpag     = retornaNumeroParcela(widgetParcelamento.dataSource.data.length);
        var nrparcbolpagvisu = ("00" + (widgetParcelamento.dataSource.data.length)).slice(-3);
        var dscontapag   = widgetParcelamento.dataSource.data[widgetParcelamento.dataSource.data.length - 2].DSCONTAPAG;
        var vrtotalparc  = valorTotalParcelamento(widgetParcelamento);
        var vttotbolpag  = widgetDocumentos.currentRow.VRNOTAFISC;
        var vrdifbolparc = self.formataValor(vttotbolpag - vrtotalparc);
        
        dscontapag = (!dscontapag)?(widgetDocumentos.currentRow.DSDEFACTPAG?widgetDocumentos.currentRow.DSDEFACTPAG:widgetDocumentos.currentRow.DSTPCTPAGAR):dscontapag;

        widgetParcelamento.currentRow.NRPARCBOLPAG = nrparcbolpag;
        widgetParcelamento.currentRow.NRPARCBOLPAGVISU = nrparcbolpagvisu;
        widgetParcelamento.currentRow.DSCONTAPAG   = dscontapag;
        widgetParcelamento.currentRow.IDEXCLUIR    = 'S';
        widgetParcelamento.currentRow.VRFINALBPAG  = (vrdifbolparc >= 0)?vrdifbolparc:0;
        
        widgetParcelamento.dataSource.data[widgetParcelamento.dataSource.data.length - 2].IDEXCLUIR = null;
        
    };
    
    
    
    /**
     * Confirma Nova Parcela Títulos a Pagar
     * @param widget
     * @param widgetTitulosAReceber
     */
    var retornaNumeroParcela = function(nrparcbolpag){
        
        nrparcbolpag = ("000" + nrparcbolpag).slice(-3);
        
        var unidade = nrparcbolpag.substr(-1, 1);
        var centena = nrparcbolpag.substr(-3, 1);
        
        if(centena != '0'){
            if(nrparcbolpag != '360'){
                var letras = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'X', 'W', 'Y', 'Z'];
            
                var posicao = parseInt(nrparcbolpag.substr(-3, 2)) - 10;
                nrparcbolpag = letras[posicao] + unidade;
            }else nrparcbolpag = 'AA';
        }
            
        return nrparcbolpag.slice(-2);
    };
    
    /**
     * Excluir Parcela Títulos a Pagar
     * @param widget
     */
    self.excluirParcela = function(widget){
        widget.list();
        self.habDesCampos('DocumentosForm', false);
        
        widget.dataSource.data.splice(widget.currentRow.__index, 1);
        
        widget.dataSource.data.forEach(function(data){
            if(data.hasOwnProperty('DSCONTAPAG_inline'))
                delete data.DSCONTAPAG_inline;
            if(data.hasOwnProperty('DTVENBOLPAG_inline'))
                delete data.DTVENBOLPAG_inline;
            if(data.hasOwnProperty('VRFINALBPAG_inline'))
                delete data.VRFINALBPAG_inline;
            if(data.hasOwnProperty('changed_DSCONTAPAG'))
                delete data.changed_DSCONTAPAG;
            if(data.hasOwnProperty('changed_VRFINALBPAG'))
                delete data.changed_VRFINALBPAG;
            if(data.hasOwnProperty('changed_DTVENBOLPAG'))
                delete data.changed_DTVENBOLPAG;
            if(data.hasOwnProperty('changed'))
                delete data.changed;
            if(data.hasOwnProperty('__is_new'))
                delete data.__is_new;
        });
        
        var vrtotalparc = 0;
        var vrnotafisc  = widget.getParent().currentRow.VRNOTAFISC;
        
        widget.dataSource.data.forEach(function(data){
            vrtotalparc += self.formataValor(data.VRFINALBPAG);
        });
        
        var vrdifparcnf = (vrnotafisc - vrtotalparc); 
        
        widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBPAG = (vrdifparcnf > 0)?self.formataValor(widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBPAG) + vrdifparcnf:widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBPAG;
        
        if(widget.currentRow.__index == '1')
            widget.dataSource.data[widget.currentRow.__index - 1].NRPARCBOLPAG = "00";
        else widget.dataSource.data[widget.currentRow.__index - 1].IDEXCLUIR = 'S';
    };
    
        /**
     * Excluir Parcela Títulos a Pagar
     * @param widget
     */
    self.cancelarParcela = function(widget){
        widget.list();
        self.habDesCampos('DocumentosForm', false);
        
        widget.dataSource.data.forEach(function (data){
            if(data.__is_new == true)
                widget.dataSource.data.splice(data.__index, 1);
        });
        
        widget.dataSource.data.forEach(function(data){
            if(data.hasOwnProperty('DSCONTAPAG_inline'))
                delete data.DSCONTAPAG_inline;
            if(data.hasOwnProperty('DTVENBOLPAG_inline'))
                delete data.DTVENBOLPAG_inline;
            if(data.hasOwnProperty('VRFINALBPAG_inline'))
                delete data.VRFINALBPAG_inline;
                
            if(data.hasOwnProperty('changed_DSCONTAPAG')){
                data.DSCONTAPAG = data.changed_DSCONTAPAG;
                delete data.changed_DSCONTAPAG;
            }
            if(data.hasOwnProperty('changed_VRFINALBPAG')){
                data.VRFINALBPAG = data.changed_VRFINALBPAG;
                delete data.changed_VRFINALBPAG;
            }
            if(data.hasOwnProperty('changed_DTVENBOLPAG')){
                data.DTVENBOLPAG = data.changed_DTVENBOLPAG;
                delete data.changed_DTVENBOLPAG;
            }
            
            if(data.hasOwnProperty('changed'))
                delete data.changed;
            if(data.hasOwnProperty('__is_new'))
                delete data.__is_new;
        });
        
        if(widget.currentRow.__index == '1')
            widget.dataSource.data[widget.dataSource.data.length - 1].NRPARCBOLPAG = "00";
        else if (widget.currentRow.__index != '0')
            widget.dataSource.data[widget.dataSource.data.length - 1].IDEXCLUIR = 'S';
    };
    
    /**
     * Valida Campos Parcelamento Títulos a Pagar
     * @param widget
     */
    self.validaCamposParcelamento = function(widget){
        var deferred = ZHPromise.defer();
        widget.dataSource.data.forEach(function(data){
            if(!data.DTVENBOLPAG){
                ScreenService.showMessage('Operação bloqueada. Preencha o campo de vencimento.');
                deferred.reject();
            }
            if(data.VRFINALBPAG <= 0){
                ScreenService.showMessage('Operação bloqueada. Valor da parcela deve ser maior que zero.');
                deferred.reject();
            }
        });
        deferred.resolve();
        return deferred.promise;
    };
    
    /**
     * Excluir Parcela Títulos a Pagar
     * @param widget
     */
    self.salvarParcela = function(widget){
        self.validaCamposParcelamento(widget).then(function(){
            self.habDesCampos('DocumentosForm', false);
            widget.list();
            
            widget.dataSource.data.forEach(function(data){
                if(data.DSCONTAPAG_inline != undefined)
                    delete data.DSCONTAPAG_inline;
                if(data.DTVENBOLPAG_inline != undefined)
                    delete data.DTVENBOLPAG_inline;
                if(data.VRFINALBPAG_inline != undefined)
                    delete data.VRFINALBPAG_inline;
                if(data.changed_DSCONTAPAG != undefined)
                    delete data.changed_DSCONTAPAG;
                if(data.changed_VRFINALBPAG != undefined)
                    delete data.changed_VRFINALBPAG;
                if(data.changed_DTVENBOLPAG != undefined)
                    delete data.changed_DTVENBOLPAG;
                if(data.changed != undefined)
                    delete data.changed;
                if(data.__is_new != undefined)
                    delete data.__is_new;
            });
        });
    };

    /**
     * Excluir Parcela Títulos a Pagar
     * @param widget
     */
    self.habDesCampos = function(widget_name, read_only){
        var widget = templateManager.container.getWidget(widget_name);
        
        widget.fields.forEach(function(field){
            field.readOnly = read_only;
        });
    };
    
    this.setaMascCnpjCpfFornecedor = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURFORN.length === 14) inscFormatada = value.NRINSJURFORN.substr(0, 2) + '.' + value.NRINSJURFORN.substr(2, 3) + '.' + value.NRINSJURFORN.substr(5, 3) + '/' + value.NRINSJURFORN.substr(8, 4) + '-' + value.NRINSJURFORN.substr(12, 2);
          else if(value.NRINSJURFORN.length === 11) inscFormatada = value.NRINSJURFORN.substr(0, 3) + '.' + value.NRINSJURFORN.substr(3, 3) + '.' + value.NRINSJURFORN.substr(6, 3) + '-' + value.NRINSJURFORN.substr(9, 2);
          else inscFormatada = null;
          value.NRINSJURFORN = inscFormatada?inscFormatada:value.NRINSJURFORN;
	    });
	};
	
	this.changeHeaderValues = function(widget){
        var codigo; 
        var nmempresa = widget.owner.appliedFilter.FILTER_ROW.NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "é igual a. ".concat(codigo).concat(' - ').concat(nmempresa)
        };
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        newFilter.push(cdempresa);
        filter.forEach(function(x){
            if((x.value != "é igual a.") && (x.label != "Código Empresa")){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a./)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
    };
    
    /**
     * Desabilita PopUp
     * @param widget
     */
    self.desabilitaPopUp = function (widget_name){
        var widget = templateManager.container.getWidget(widget_name);
        widget.isVisible = false;
        ScreenService.closePopup(widget);
    };
    
    
    
    /**
     * Borderô Remessa Titulo a Pagar
     */
    self.borderoRemessa = function(){
        var widgetTitulosAPagar  = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetBorderoTabbed  = templateManager.container.getWidget('BorderoRemessaTabbed');
        var widgetBorderoGrid    = templateManager.container.getWidget('BorderoRemessaGrid');
        
        var titulosRemessa = widgetTitulosAPagar.getCheckedRows().filter(function(titulo){
            return (titulo.NRCNABPAGREM);
        });
        
        var cdempresa = titulosRemessa[0].CDEMPRESA;
        var remessas  = "_";
        
        titulosRemessa.forEach(function(value){
            remessas = remessas.concat(value.NRCNABPAGREM.concat('_'));
        });
        
        widgetBorderoGrid.dataSourceFilter[0].value = cdempresa;
        widgetBorderoGrid.dataSourceFilter[1].value = remessas;
        
        ScreenService.closePopup();
        widgetBorderoTabbed.isVisible = true;
        widgetBorderoTabbed.editing = true;
        ScreenService.openSwipe(widgetBorderoTabbed);
        widgetBorderoGrid.reload();
    };
    
    
    /**
     * Default Inicial
     */
    self.defaultInicial = function(){
        self.buscaGeralPag().then(function(response_geralpag) {

            var geral_pag   = response_geralpag.dataset.geral_pag[0];
            idhabibole_geral       = geral_pag.IDHABIBOLE;
            idutilpadrbolhab_geral = geral_pag.IDUTILPADRBOLHAB;

        });
    };
    
    self.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctpagar = widget.getField('CDTPCTPAGAR');
        
        cdtpctpagar.setValue("");
        cdtpctpagar.dataSourceFilter[0].value = cdempresa;
        cdtpctpagar.reload();
    };
    
    /**
     * Cadastrar Guia de Tributo
     */
    self.cadastrarGuiaTributo = function(){
        
        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
        var checkedrows = widgetTitulosAPagar.getCheckedRows();

        var titulosGuiaTributo = checkedrows.filter(function(boleto){
            return ['B', 'R', 'P', 'G', 'I', 'H', 'J', 'K', 'N', 'Q', 'U', 'V', 'X'].includes(boleto.IDBAIXAUTILP);
        });
        
        if(checkedrows.length > 0) {
            var widgetCadastrarGuiaTributo = templateManager.container.getWidget('CadastrarGuiaTributoForm');
            widgetCadastrarGuiaTributo.isVisible = true;
            
            if(checkedrows.length >= 2){

                widgetCadastrarGuiaTributo.getField('CDRECEITAGUIA').setValue(null);
                widgetCadastrarGuiaTributo.getField('DTAPURACGUIA').setValue(null);
                widgetCadastrarGuiaTributo.getField('NRREFEGUIA').setValue(null);
                widgetCadastrarGuiaTributo.getField('VRPERCENTTTRIB').setValue(null);
                widgetCadastrarGuiaTributo.getField('VROUTRENTGUIA').setValue(null);

            } else{

                var param = {
                  "NRLANBOLPAG":  titulosGuiaTributo[0].NRLANBOLPAG
                };

                General.callRestEngine('fin_contaPagar#/verificaDadosGuiaTributo', 'Row', param).then(function(result){
                    var dadosGuia = result.dataset.dadosGuia;
                    widgetCadastrarGuiaTributo.getField('CDRECEITAGUIA').setValue(dadosGuia[0].CDRECEITAGUIA);
                    widgetCadastrarGuiaTributo.getField('DTAPURACGUIA').setValue(dadosGuia[0].DTAPURACGUIA);
                    widgetCadastrarGuiaTributo.getField('NRREFEGUIA').setValue(dadosGuia[0].NRREFEGUIA);
                    widgetCadastrarGuiaTributo.getField('VRPERCENTTTRIB').setValue(dadosGuia[0].VRPERCENTTRIB);
                    widgetCadastrarGuiaTributo.getField('VROUTRENTGUIA').setValue(dadosGuia[0].VROUTRENTGUIA);
                });
            }
            
            ScreenService.openPopup(widgetCadastrarGuiaTributo);

        }else ScreenService.showMessage('Operação bloqueada. Não há título(s) para cadastrar guia de tributo.');
        
    };
    
    /**
     * Confirmar Cadastro Guia de Tributo
     */
    self.confirmarCadastroGuiaTributo = function(){
        
        var widgetCadastrarGuiaTributo = templateManager.container.getWidget('CadastrarGuiaTributoForm');
        
        if(widgetCadastrarGuiaTributo.isValid()){
            
            var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
            var checkedrows = widgetTitulosAPagar.getCheckedRows();
            var titulosGuiaTributo = checkedrows.filter(function(boleto){
                return ['G','H','I','U','V','K','J','X'].includes(boleto.IDBAIXAUTILP);
            });
            
            var route = 'fin_contaPagar#/cadastrarGuiaTributo';
            
            var dataset = [
                widgetCadastrarGuiaTributo.currentRow,
                titulosGuiaTributo
            ];
            
            General.callRestEngine(route, 'DataSet', dataset).then(function (success) {
                
                ScreenService.showMessage("Cadastro(s) de Guia de Tributo realizado com sucesso.");
                
                widgetCadastrarGuiaTributo.isVisible = false;
                ScreenService.closePopup(widgetCadastrarGuiaTributo);
                ScreenService.closePopup();
                
                widgetTitulosAPagar.unSelectAll();
                self.unsetStripeDataFooter();
                
                
            });
        }
        
    };
    
    /**
     * Cancelar Cadastro Guia de Tributo
     */
    self.cancelarCadastroGuiaTributo = function(){

        var widgetTitulosAPagar = templateManager.container.getWidget('TitulosAPagarGrid');
        var widgetCadastrarGuiaTributo = templateManager.container.getWidget('CadastrarGuiaTributoForm');
        widgetCadastrarGuiaTributo.isVisible = false;
        
        ScreenService.closePopup(widgetCadastrarGuiaTributo);
        ScreenService.closePopup();
        
    };

    /**
     * Novo Ajuste Titulo Baixa
     * @param widget
     */
    self.novoAjusteTituloBaixa = function(){
        var widgetTitulosBaixa      = templateManager.container.getWidget('NewTitulosBaixaGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');

        widgetAjusteTituloBaixa.newRow();
        widgetAjusteTituloBaixa.getField('NRORG').setValue(widgetTitulosBaixa.currentRow.NRORG);
        widgetAjusteTituloBaixa.getField('CDFILIAL').setValue(widgetTitulosBaixa.currentRow.CDFILIAL);
        widgetAjusteTituloBaixa.getField('NRLANBOLPAG').setValue(widgetTitulosBaixa.currentRow.NRLANBOLPAG);
        widgetAjusteTituloBaixa.getField('IDOPERACAO').setValue('I');
        widgetAjusteTituloBaixa.getField('IDORIGAJBPAG').setValue('2');
    };
    
    
    
    /**
     * Salva Ajuste Titulo Baixa
     * @param args
     */
    self.salvaAjusteTituloBaixa = function(){
        var widgetBaixa             = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetTitulosBaixa      = templateManager.container.getWidget('NewTitulosBaixaGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');
        
        if(widgetAjusteTituloBaixa.isValid()){
            var currentRowAjuste = widgetAjusteTituloBaixa.currentRow;
            var dataAjuste = widgetAjusteTituloBaixa.dataSource.data;
            
            var cdajuctpag = dataAjuste.filter(function(filter) {
                return (filter.CDAJUCTPAG == currentRowAjuste.CDAJUCTPAG);
            });
            
            if(cdajuctpag.length == 2){
                ScreenService.showMessage('Operação bloqueado. Ajuste já cadastrado.');
                return;
            }
            
            localStorageEngine.getDataSource('fin_contaPagar#/ajbolpag_tipoajpagar').then(
                function (dataLocal) {
                    dataLocal.push(currentRowAjuste);
                    localStorageEngine.setDataSource('fin_contaPagar#/ajbolpag_tipoajpagar', dataLocal);
                    
                    // Valor de Ajuste do Título
                    var idajuctpag  = widgetAjusteTituloBaixa.currentRow.IDAJUCTPAG;
                    idajuctpag      = (idajuctpag == 'A')?'A':'D';
                    var vrajbolpag  = parseFloat(widgetAjusteTituloBaixa.currentRow.VRAJBOLPAG);
                    vrajbolpag      = (idajuctpag == 'A')?vrajbolpag:-(vrajbolpag);
        
                    // Atualiza Valor Líquido do Título
                    var vrfinalbpag = parseFloat(widgetTitulosBaixa.currentRow.VRFINALBPAG);
                    vrfinalbpag    += vrajbolpag;
                    
                    if(vrfinalbpag < 0){
                        ScreenService.showMessage('Operação bloqueada. Valor do título não pode ficar negativo.');
                        return;
                    }
                    
                    widgetTitulosBaixa.currentRow.VRFINALBPAG = vrfinalbpag;
                    
                    // Atuliza Valor da Baixa
                    var vrbaixapag = parseFloat(widgetBaixa.currentRow.VRBAIXAPAG);
                    vrbaixapag    += vrajbolpag;
                    widgetBaixa.currentRow.VRBAIXAPAG = vrbaixapag;
                    
                    ScreenService.goBack();
                    ScreenService.showMessage('Registro salvo com sucesso.');
                    widgetTitulosBaixa.currentRow.__ajusteAtualizado = true;
                }
            );
            

        }
    };
    
    /**
     * Exclui Ajuste Título Baixa
     * @param args
    */
    self.excluiAjusteTituloBaixa = function(args){
        var widget             = args.owner.widget;
        var widgetBaixa        = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetTitulosBaixa = templateManager.container.getWidget('NewTitulosBaixaGrid');

        if(widget.currentRow.IDORIGAJBPAG == '4') {
            ScreenService.showMessage('Operação bloqueada. Ajuste originado de uma compensação não pode ser excluído.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente excluir o ajuste?', 'question', function() {
            
            localStorageEngine.getDataSource('fin_contaPagar#/ajbolpag_tipoajpagar').then(
                function (dataLocal) {
                    for(var i in dataLocal){
                        if((dataLocal[i].CDFILIAL    == widget.currentRow.CDFILIAL   ) && 
                           (dataLocal[i].NRLANBOLPAG == widget.currentRow.NRLANBOLPAG) && 
                           (dataLocal[i].CDAJUCTPAG  == widget.currentRow.CDAJUCTPAG )){
                            dataLocal[i].IDOPERACAO = 'E';
                            widget.currentRow.IDOPERACAO = 'E';
                        }
                    }
                    
                    localStorageEngine.setDataSource('fin_contaPagar#/ajbolpag_tipoajpagar', dataLocal).then(
                        function () {
                            templateManager.container.getWidget('AjusteTituloBaixaGrid').reload();
                        }
                    );
                    
                     // Valor de Ajuste do Título
                    var idajuctpag  = widget.currentRow.IDAJUCTPAG;
                    idajuctpag      = (idajuctpag == 'A')?'A':'D';
                    var vrajbolpag  = parseFloat(widget.currentRow.VRAJBOLPAG);
                    vrajbolpag      = (idajuctpag == 'A')?-(vrajbolpag):vrajbolpag;
        
                    // Atualiza Valor Líquido do Título
                    var vrfinalbpag = parseFloat(widgetTitulosBaixa.currentRow.VRFINALBPAG);
                    vrfinalbpag    += vrajbolpag;
                    
                    if(vrfinalbpag < 0){
                        ScreenService.showMessage('Operação bloqueada. Valor do título não pode ficar negativo.');
                        return;
                    }
                    
                    widgetTitulosBaixa.currentRow.VRFINALBPAG = vrfinalbpag;
                    
                    // Atuliza Valor da Baixa
                    var vrbaixapag = parseFloat(widgetBaixa.currentRow.VRBAIXAPAG);
                    vrbaixapag    += vrajbolpag;
                    widgetBaixa.currentRow.VRBAIXAPAG = vrbaixapag;
                    
                    ScreenService.goBack();
                    ScreenService.showMessage('Registro excluído com sucesso.');
                    widgetTitulosBaixa.currentRow.__ajusteAtualizado = true;
                }
            );
        });
    };

    /**
     * Altera Tipo de Prorrogação
     */
    self.alteraTipoProrrogacao = function(){
        var widgetProrrogar = templateManager.container.getWidget('ProrrogarForm');
        var idtppror        = widgetProrrogar.getField('IDTPPROR').value();

        if(idtppror == 'F') {
            widgetProrrogar.getField('DTVENBOLPAG').readOnly = false;
            widgetProrrogar.getField('DTVENBOLPAG').validations.required = true;
            widgetProrrogar.getField('DTVENBOLPAG').setValue('');
            
            widgetProrrogar.getField('NRDIASVENC').readOnly  = true;
            delete widgetProrrogar.getField('NRDIASVENC').validations.required;
            delete widgetProrrogar.getField('NRDIASVENC').validations.min;
            widgetProrrogar.getField('NRDIASVENC').clearValue();
        }else{
            widgetProrrogar.getField('DTVENBOLPAG').readOnly = true;
            delete widgetProrrogar.getField('DTVENBOLPAG').validations.required;
            widgetProrrogar.getField('DTVENBOLPAG').setValue('');
            
            widgetProrrogar.getField('NRDIASVENC').readOnly  = false;
            widgetProrrogar.getField('NRDIASVENC').validations.required = true;
            widgetProrrogar.getField('NRDIASVENC').validations.min = {"limit" : 0};
            widgetProrrogar.getField('NRDIASVENC').setValue(30);
        }
        
        templateManager.updateTemplate();
    };
    
    self.validaParcelamentoManual = function(widgetTitulosAPagar, parcelamento){
        var deferred = ZHPromise.defer();
        
        var currentRowTitulosAPagar = widgetTitulosAPagar.currentRow;
        var mensagemValor = false;
        var valortotparc = 0;
        var parcelamentoPermitido = true;
        var parcelas = parcelamento.dataSource.data;
        var totalTitulo = parcelas[0].VRTOTBOLPAG;
        
        for (var i = 0; i < parcelas.length && parcelamentoPermitido; i++){
            if ($.currency.unmaskString(parcelas[i].VRFINALBPAG) <= 0) {
                parcelamentoPermitido = false;
                deferred.reject('Operação bloqueada. Existe(m) parcela(s) menor(es) ou igual(is) a zero.');
            }
            valortotparc += self.formataValor(parcelas[i].VRFINALBPAG);
        }
        
        totalTitulo  = self.formataValor(totalTitulo);
        valortotparc = self.formataValor(valortotparc);

        if((totalTitulo != valortotparc) && (parcelamentoPermitido) ){
            if((currentRowTitulosAPagar.IDSISTEMA == 'PAG') && (currentRowTitulosAPagar.IDSTATBPAG == 'A') && (currentRowTitulosAPagar.IDASSOCBOLPAG == 'N')){
                var mensagemConfirma =
                    'Valor total das parcelas difere do valor total do título.<br/><br/>' +
                    'Valor total do Título: ' + parseFloat(totalTitulo).toFixed(2) + '<br/>' +
                    'Valor total das Parcelas: ' + parseFloat(valortotparc).toFixed(2) + '<br/><br/>' +
                    'Deseja alterar o valor do documento para prosseguir?';
                ScreenService.confirmMessage(mensagemConfirma, 'question',
                    function () {
                        if(parcelas.length < 2){
                            parcelas[0].NRPARCBOLPAG = '00';
                        }
                        deferred.resolve(parcelamentoPermitido);
                    },
                    function() {
                        parcelamentoPermitido = false;
                        deferred.reject(parcelamentoPermitido);
                    }
                );
            }else{
                
                
                parcelamentoPermitido = false;
                deferred.reject(
                    'Operação bloqueada. Valor total das parcelas difere do valor total do título.<br/><br/>' +
                    'Valor total do Título: ' + totalTitulo + '<br/>' +
                    'Valor total das Parcelas: ' + valortotparc
                );
            }
        }else if (parcelamentoPermitido){
            if(parcelas.length < 2){
                ScreenService.showMessage('Operação bloqueada. Continue o parcelamento para finalização.');
                return;
            }
            deferred.resolve(parcelamentoPermitido);
        }
        return deferred.promise;
    };
    
    self.setCdfornecedor = function (widget){
        widget.currentRow.CDFORNECED = widget.dataSourceFilter[0].value;
    };
    
    self.salvarCodigoPix = function(widget){
        var TitulosAPagarGrid = templateManager.container.getWidget('TitulosAPagarGrid');
        if(!widget.isValid()){
            ScreenService.showMessage("Preencha todos os campos obrigatórios!");   
        }else{
            widget.currentRow.CDFORNECED = TitulosAPagarGrid.currentRow.CDFORNECED;
            widget.currentRow.__is_new = true;
            widget.currentRow.CDCHAVEPIX = widget.currentRow.CDCHAVEPIX.trim();
            var chave = {}, CDCHAVEPIX = widget.currentRow.CDCHAVEPIX;
            
            switch (widget.currentRow.IDCHAVEPIX) {
                case '01':
                    chave.valid = CDCHAVEPIX.length > 8 && typeof(parseInt(CDCHAVEPIX)) === 'number';
                    if(!chave.valid){
                        chave.message = "Celular inválido.";
                    }   
                    break;
                case '02':
                    chave = ValidationEngine.mail(CDCHAVEPIX, {});
                    break;
                case '03':
                    if(CDCHAVEPIX.length == 11){
                        chave.valid = General.validateCPF(CDCHAVEPIX);
                    }else if(CDCHAVEPIX.length == 14){
                        chave.valid = General.validateCNPJ(CDCHAVEPIX);
                    }else{
                        chave.valid = false;
                    }
                    if(!chave.valid){
                        chave.message = "CPF/CNPJ inválido.";
                    }   
                    break;
                
                case '04':
                    var ckchex  = new RegExp('^[0-9a-fA-F-]+$');
                    chave.valid = ckchex.test(CDCHAVEPIX);
                    if(!chave.valid){
                        chave.message = "EVP inválido.";
                    }   
                    break;
                
                default:
                    chave.valid = false;
                    chave.message = "Tipo de Chave Pix inválido.";
            }
            if (widget.isValid() && chave.valid){
                ScreenService.openSwipe(widget).then(function(){
                    General.saveCrud(widget).then(function(row){
                        ScreenService.closePopup(true);
                        TitulosAPagarGrid.currentRow.CHAVEPIX = widget.currentRow.CDCHAVEPIX;
                        TitulosAPagarGrid.currentRow.DSCHAVEPIX = widget.currentRow.DSCHAVEPIX;
                        TitulosAPagarGrid.currentRow.IDCHAVEPIX = widget.currentRow.IDCHAVEPIX;
                        TitulosAPagarGrid.edit();
                        TitulosAPagarGrid.activate();
                        if(widget.relacionandoPix){
                            self.confirmaEditaTituloAPagar(TitulosAPagarGrid);
                        }
                    },
                    function(){
                        ScreenService.closeSwipe();
                        widget.activate();
                    });
                });
            }else{
                ScreenService.showMessage(chave.message||"Dados inválidos");
            }
        }
    };
    self.abrePopUpAddPIX = function(widget){
        widget.container = templateManager.container;
        widget = metaDataFactory.widgetFactory(widget);
        ScreenService.openPopup(widget);
    };
    
    self.newRowPIX = function(widget){
        widget.container = templateManager.container;

    };
    

    /**
     * Valida Tipo de Baixa
     */
    self.validaTipoBaixa = function(widget){
        if(widget.currentRow.IDBAIXAUTILP == 'P'){
            ScreenService.showMessage('Operação bloqueada. Tipo de baixa de compensação não pode ser utilizada.');
            
            widget.currentRow.NMTPBAIXAPAG = '';
            widget.currentRow.CDTPBAIXAPAG = '';
            widget.currentRow.IDBAIXAUTILP = '';
        }
    };
    
    /**
     * Altera Vencimento Titulo A Pagar
     * @param args
     */
    self.alteraVencimento = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarGrid');
        
        if((widget.backupRow.DTVENBOLPAG.substr(0, 10) != widget.currentRow.DTVENBOLPAG.substr(0, 10))){
            widget.getField('DSMOTIPROR').readOnly = false;
            if(widget.currentRow.IDSISTEMA != 'PAG'){
                widget.getField('DSMOTIPROR').validations.required = true;
            }else{
                delete widget.getField('DSMOTIPROR').validations.required;
            }
        }else{
            delete widget.getField('DSMOTIPROR').validations.required;
            widget.getField('DSMOTIPROR').readOnly = true;
            widget.getField('DSMOTIPROR').setValue('');
        }
        
        templateManager.updateTemplate();
    };
    
    self.changeChavePix = function(widget){
        if(widget.getField('IDCHAVEPIX').value() == '01'){
            widget.getField('CDCHAVEPIX').setValue('+55');
        }else{
            widget.getField('CDCHAVEPIX').setValue('');
        }
        templateManager.updateTemplate();
    };
    
    self.buildDataSourceKanban = function(){
        var widget = templateManager.container.getWidget('TitulosAPagarKanban');
        widget.dataSource.data.forEach(function (data) {
            
            var dtatual = moment().format('DD/MM/YYYY');
            var dtvenbolpag = moment(data.DTVENBOLPAG, "DD/MM/YYYY HH:mm:ss").format('DD/MM/YYYY');
            
            
            switch (data.IDSTATBPAG) {
                case 'A':
                    data.column = data.IDSTATUSPAG == 'A'? 1: 2;
                    break;
                case 'P':
                    data.column = 3;
                    break;
                case 'B':
                    data.column = 4;
                    break;
                default:
                    data.column = 1;
            }
            
            
            var vrfinalbpag = self.formataValor(data.VRFINALBPAG);
            var valor = vrfinalbpag.toLocaleString('pt-BR', {minimumFractionDigits: 2});
            
            data.label = dtvenbolpag + " - " + data.NMRAZSOCFORN + " - " + valor; 
            
            var nrinsjurforn = "CNPJ/CPF: " + self.formataCnpjCpf(data.NRINSJURFORN);
            var nrparcbolpagvisu = "Parcela: " + data.NRPARCBOLPAGVISU;
            var nmtpbaixapag = "Tipo de Baixa: " + (data.NMTPBAIXAPAG?data.NMTPBAIXAPAG:"");
            var nrdocserie = "Documento: " + data.NRDOCSERIE;
            var idstatus = "Tipo: " + (data.IDSTATUS == 'A'?'Real':'Previsto');
            var vrorigpag = self.formataValor(data.VRORIGPAG);
            vrorigpag = "Valor Original: " + vrorigpag.toLocaleString('pt-BR', {minimumFractionDigits: 2});
            
            data.description = nrinsjurforn + "<br/>" + nrparcbolpagvisu + "<br/>" + nmtpbaixapag + "<br/>" + nrdocserie + "<br/>" + idstatus + "<br/>" + vrorigpag;
            
            
            
            //Object.assign(data, {'due-date': valor});
        });
    };
    
    self.formataCnpjCpf = function(value) {
        if(value.length === 14)
             value = value.substr(0, 2) + '.' + value.substr(2, 3) + '.' + value.substr(5, 3) + '/' + value.substr(8, 4) + '-' + value.substr(12, 2);
        else if(value.length === 11)
             value = value.substr(0, 3) + '.' + value.substr(3, 3) + '.' + value.substr(6, 3) + '-' + value.substr(9, 2);
        
        return value;
	};
	
    self.beforeMoveItem = function(args, e) {
        var widget = templateManager.container.getWidget('TitulosAPagarKanban');
        widget.reload();
        
        ScreenService.showMessage('Operação bloqueda. Não é possível realizar alterações por movimentação no momento, em breve estaremos liberando a operação.');
	};
	
	 /**
     * Edita Baixa Pagamento
     * @param widget
     */
    self.editBaixaPagamento = function(widget){
        General.edit(widget);
        
        var widgetNewBaixaTabbed = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');

        var idbaixautilp = widgetNewBaixaTabbed.getField('IDBAIXAUTILP').value();

        if((idbaixautilp == 'M') || (idbaixautilp == 'A')) {
            widgetNewTitulosBaixaGrid.getField('NMTPBAIXAPAG').readOnly  = true;
        }else{
            widgetNewTitulosBaixaGrid.getField('NMTPBAIXAPAG').readOnly  = false;
        }
    };
	
	
	 /**
     * Calcula Total Baixa Paga
     * @param widget
     */
    self.calculaTotalBaixaPaga = function(){
        var widgetNewBaixaTabbed = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');
        
        var vrtotalbaixapaga = 0;
        
        var dataNewTitulosBaixaGrid = widgetNewTitulosBaixaGrid.getCheckedRows();
        
        dataNewTitulosBaixaGrid.forEach(function (data) {
            vrtotalbaixapaga += self.formataValor(data.VRPAGO);
        });
        
        widgetNewBaixaTabbed.currentRow.VRBAIXAPAG = vrtotalbaixapaga;
    };
	
	 /**
     * Edita Baixa Pagamento
     * @param widget
     */
    self.saveBaixaPagamento = function(args){
        var widget = args.owner.widget;
        
        if(widget.isValid()){
            var vrfinalbpag = self.formataValor(widget.currentRow.VRFINALBPAG);
            var vrpago      = self.formataValor(widget.currentRow.VRPAGO);
            
            if(vrfinalbpag != vrpago){
                if(vrfinalbpag > vrpago){
                        var currentRow = widget.dataSource.data[widget.currentRow.__index];
                        
                        var newRow = null;
                        
                        if(currentRow.NRPARCBOLPAGVISU == '000'){
                            currentRow.NRPARCBOLPAGVISU = String(parseInt(currentRow.NRPARCBOLPAGVISU) + 1).padStart(3, '0');
                            currentRow.NRPARCBOLPAG     = String(parseInt(currentRow.NRPARCBOLPAG) + 1).padStart(2, '0');
                            currentRow.NRORIGBOLPAG     = currentRow.NRLANBOLPAG;
                            
                            newRow = Util.clone(currentRow);
                            
                            newRow.NRPARCBOLPAGVISU =  String(parseInt(newRow.NRPARCBOLPAGVISU) + 1).padStart(3, '0');
                            newRow.NRPARCBOLPAG     =  String(parseInt(newRow.NRPARCBOLPAG) + 1).padStart(2, '0');
                        }else{
                            
                            newRow = Util.clone(currentRow);
                            
                            var filter = [
                                {
                                    "name": "NRORG",
                                    "operator": "=",
                                    "value": currentRow.NRORG
                                },
                                {
                                    "name": "CDFILIAL",
                                    "operator": "=",
                                    "value": currentRow.CDFILIAL
                                },
                                {
                                    "name": "NRORIGBOLPAG",
                                    "operator": "=",
                                    "value": currentRow.NRORIGBOLPAG
                                }
                            ];
                            
                            General.callRestEngine('fin#/crud/boletpagar_maxNrparcbolpag','FilterData', filter).then(function(result){
                                var maxNrparcbolpag = result.dataset.boletpagar_maxNrparcbolpag;
                                
                                if(maxNrparcbolpag.length > 0){
                                    
                                    var old_nrparcbolpagvisu = maxNrparcbolpag[0].NRPARCBOLPAGVISU;
                                    var old_nrparcbolpag = maxNrparcbolpag[0].NRPARCBOLPAG;
                                    
                                    var rowsNrorigbolpag = widget.dataSource.data.filter(function(boleto){
                                        return boleto.NRORIGBOLPAG == currentRow.NRORIGBOLPAG;
                                    });
                                    
                                    
                                    self.getNewNrparcbolpag(rowsNrorigbolpag).then(function(result) {
                                        var new_nrparcbolpagvisu_widget = result[0];
                                        var new_nrparcbolpag_widget = result[0];
                                        
                                        if(new_nrparcbolpagvisu_widget > old_nrparcbolpagvisu){
                                            newRow.NRPARCBOLPAGVISU =  String(parseInt(new_nrparcbolpagvisu_widget) + 1).padStart(3, '0');
                                            newRow.NRPARCBOLPAG     =  String(parseInt(new_nrparcbolpag_widget) + 1).padStart(2, '0');
                                        }else{
                                            newRow.NRPARCBOLPAGVISU =  String(parseInt(old_nrparcbolpagvisu) + 1).padStart(3, '0');
                                            newRow.NRPARCBOLPAG     =  String(parseInt(old_nrparcbolpag) + 1).padStart(2, '0');
                                        }
                                    });
                                }
                            });
                        }
                        
                        currentRow.VRFINALBPAG = currentRow.VRPAGO;
                        currentRow.VRBOLPAG    = currentRow.VRPAGO;
                        currentRow.PARCELADO   = true;
                        currentRow.CDMOTIPARC  = '01';
                        
                        //newRow.NRPARCBOLPAGVISU =  String(parseInt(newRow.NRPARCBOLPAGVISU) + 1).padStart(3, '0');
                        //newRow.NRPARCBOLPAG     =  String(parseInt(newRow.NRPARCBOLPAG) + 1).padStart(2, '0');
                        newRow.VRFINALBPAG      = newRow.VRFINALBPAG - newRow.VRPAGO;
                        newRow.VRBOLPAG         = newRow.VRFINALBPAG;
                        newRow.VRPAGO           = newRow.VRFINALBPAG;
                        newRow.NRORIGBOLPAG     = newRow.NRORIGBOLPAG ? newRow.NRORIGBOLPAG : newRow.NRLANBOLPAG;
                        newRow.NRLANBOLPAG      = null;
                        newRow.CDMOTIPARC       = '01';//TODO PARAMETRO PARA MOTIVO
                        newRow.PARCELADO        = true;
                        newRow.__index = newRow.__index + 1;
                        
                        widget.dataSource.data.splice(newRow.__index, 0, newRow);
                        
                        ScreenService.goBack();
                        ScreenService.showMessage('Registro salvo com sucesso.');
                }else ScreenService.showMessage('Operação bloqueada. Valor pago não pode ser superior ao valor líquido.');
            }else{
                ScreenService.goBack();
                ScreenService.showMessage('Registro salvo com sucesso.');
            }
        }
        
    };
    
     self.getNewNrparcbolpag = function(rowsNrorigbolpag) {
        var deferred = ZHPromise.defer();
        
        var maxNrparcbolpagvisu_widget = 0;
        var maxNrparcbolpag_widget = 0;
                                    
        rowsNrorigbolpag.forEach(function (data) {
            maxNrparcbolpagvisu_widget = (maxNrparcbolpagvisu_widget < parseInt(data.NRPARCBOLPAGVISU)) ? data.NRPARCBOLPAGVISU : maxNrparcbolpagvisu_widget;
            maxNrparcbolpag_widget = (maxNrparcbolpag_widget < parseInt(data.NRPARCBOLPAG)) ? data.NRPARCBOLPAG : maxNrparcbolpag_widget;
        });
        
        var result = [maxNrparcbolpagvisu_widget, maxNrparcbolpag_widget];
        
        deferred.resolve(result);   
        
        return deferred.promise;
    };
    
};

/* global Configuration */
Configuration(function (ContextRegister){
    ContextRegister.register('Pag00001_TitulosAPagar_Controller', Pag00001_TitulosAPagar_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag00200_RetornoPagamento_Controller.js
var Pag00200_RetornoPagamento_Controller = function(ScreenService, templateManager,General, ZHPromise, Pag00200_RetornoPagamento_Service, ZhFilterPreferences){

    /**
     * Variáveis de Inicialização
     **/
    var self = this;

    this.incrementaDataFinal = function(){
        var retorno = templateManager.container.getWidget('retorno');
        retorno.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'DTCNABPAG'){
                filtro.value[1] = moment(filtro.value[1].replace('/', ''), 'DDMMYYYY').add(1, 'days').format('DD/MM/YYYY');
            }
        });

    };

    this.ativaGridOcorrencias = function(widget){
        if (widget.name == 'boletos_retorno'){
            templateManager.container.getWidget('ocorrencias_retorno').activate();
        }else{
            templateManager.container.getWidget('ocorrencias_importacao').activate();
        }
    };

    this.ativaWidgetPrincipal = function(widget){
        widget.activate();
    };

    this.setDataSourceFilterRetorno = function(retorno){
        retorno = templateManager.container.getWidget('retorno');
        var valorInicial = retorno.appliedFilter.FILTER_ROW.VRLIQINI;
        var valorFinal = retorno.appliedFilter.FILTER_ROW.VRLIQFIN;
        var notaInicial = retorno.appliedFilter.FILTER_ROW.NRNOTAFISCINI;
        var notaFinal = retorno.appliedFilter.FILTER_ROW.NRNOTAFISCFIN;
        if (parseFloat(valorInicial) == 0 && parseFloat(valorFinal) == 0){
            valorInicial = null;
            valorInicial = null;
        }
        retorno.dataSourceFilter.forEach(function(data){
            switch(data.name){
                case 'VRLIQINI': data.value = valorInicial;
                    break;
                case 'VRLIQFIN': data.value = valorFinal;
                    break;
                case 'TVALOR': data.value = (valorInicial && valorFinal) ? '' : 'T';
                    break;
                case 'NRNOTAFISCINI': data.value = notaInicial;
                    break;
                case 'NRNOTAFISCFIN': data.value = notaFinal;
                    break;
                case 'TNOTA': data.value = (notaInicial && notaFinal) ? '' : 'T';
                    break;
            }
        });
    };

	this.buscaEmpresa = function (cdempresa){
        return Pag00200_RetornoPagamento_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Filial
     * @return filialLogada
     */
    this.buscaFilial = function (){
        return Pag00200_RetornoPagamento_Service.buscaFilial().then(function(filial) {
            return {dataset : {filial : filial}};
        });
    };

    this.openFilterRetornoPagar = function(retorno){
        this.buscaPadraoEmpresa(retorno).then(function(empresa){
            var data = moment().format('DD/MM/YYYY');
            var rowFilter = {
                'CDEMPRESA': empresa.CDEMPRESA,
                'NMEMPRESA': empresa.NMEMPRESA,
                'DTCNABPAG': {start : data, end : data},
            };
            ZhFilterPreferences.setOriginalFilter(retorno.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(retorno, rowFilter);
        });
    };

    /**
     * Busca Padrão Empresa
     * @param widget
     */
    this.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        this.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            this.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                deferred.resolve(empresa);
            });
        }.bind(this));
        return deferred.promise;
    };

	this.setFilterWidgetBoleto = function(widget){
        var widgetBoletos = templateManager.container.getWidget('boletos_retorno');
        widgetBoletos.dataSourceFilter[0].value = widget.currentRow.CDEMPRESA;
        widgetBoletos.dataSourceFilter[1].value = widget.currentRow.NRCNABPAG;
	};

    this.setFilterWidgetOcorrencia = function(widget){
        var widgetBoletos = templateManager.container.getWidget('ocorrencias_retorno');
        widgetBoletos.dataSourceFilter[0].value = widget.currentRow.CDFILIAL;
        widgetBoletos.dataSourceFilter[1].value = widget.currentRow.NRLANBOLPAG;
    };

    var setGridNaoClicavel = function(){
        var swipe = document.getElementsByClassName('zh-swipe-inner zh-content ng-scope open');
        var div = document.createElement('div');
        div.setAttribute('class', 'swipe-overlay ng-scope');
        div.setAttribute('data-ng-if', 'swipe.editing || swipe.currentWidget.editing');
        swipe[0].appendChild(div);
    };

	this.adicionaRetorno = function(){
        var tabbed = templateManager.container.getWidget('tabbed_import');
        tabbed.isVisible = true;
        ScreenService.openSwipe(tabbed).then(function(){
            window.setTimeout(setGridNaoClicavel, 100);
        });
    };

    this.voltarBoletos = function(){
        ScreenService.goBack().then(function(){
            var tabbed = templateManager.container.getWidget('tabbed_import');
            tabbed.fields.forEach(function(field){
                field.readOnly = false;
            });
        });
    };

	this.cancelRetorno = function(){
		var tabbed = templateManager.container.getWidget('tabbed_import');
		tabbed.isVisible = false;
		ScreenService.closeSwipe().then(function(){
            tabbed.getParent().activate();
            tabbed.widgets[0].dataSource.data = [];
            tabbed.widgets[0].ocorrencias = [];
        });
	};

	this.validaVisualizacaoImport = function(widget){
		if (widget.getField('NMBANCO').value()){
			widget.getField('DSARQCNABPAG').readOnly = false;
		}else{
			widget.getField('DSARQCNABPAG').readOnly = true;
		}
		this.deleteDataRetorno();
	};

    this.deleteDataRetorno = function(){
        var widgetBoletos = templateManager.container.getWidget('boletos_importacao');
        var widgetOcorrencias = templateManager.container.getWidget('ocorrencias_importacao');
        widgetBoletos.dataSource.data = [];
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = [];
    };

	this.preparaArquivoRetorno = function(widget){
		var route = 'fin_contaPagar#/carregaRetorno';
		var arqRetorno = widget.currentRow.DSARQCNABPAG[0];
		var conteudoArquivo = arqRetorno.b64File.split(',')[1];
		var empresa = templateManager.container.getWidget('tabbed_import').currentRow.CDEMPRESA;
		var row = {
			'CDEMPRESA': empresa,
            'CDBANCO': widget.currentRow.CDBANCO,
			'CDBANCCNAB': widget.currentRow.CDBANCCNAB,
			'CNABRETORNO': conteudoArquivo
		};

		General.callRestEngine(route, 'Row', row)
			.then(function(response){
				this.setDataSourceRetorno(response.dataset.BOLETOS);
				this.setDataSourceOcorrencias(response.dataset.OCORRENCIAS);
                widget.widgets[0].SITUACAOCNAB = response.dataset.SITUACAOCNAB[0];
                //this.validaSituacaoCnab(response.dataset.SITUACAOCNAB[0], response.dataset.BOLETOS, widget.widgets[0]);
			}.bind(this))
			.catch(function(erro){
                widget.currentRow.DSARQCNABPAG = [];
				ScreenService.showMessage(erro.data.error);
			});
	};

    this.validaSituacaoCnab = function(boletos){
        var boletoValido = !boletos.dataSource.data.every(function(elem) {return elem.SITUACAOBOLETO == false;});
        return (boletos.SITUACAOCNAB && boletoValido) ? true : false;
    };

	this.setDataSourceRetorno = function(boletos){
		var widgetBoletos = templateManager.container.getWidget('boletos_importacao');
		widgetBoletos.dataSource.data = [];
		widgetBoletos.dataSource.data = boletos;
	};

	this.setDataSourceOcorrencias = function(ocorrencias){
		var widgetOcorrencias = templateManager.container.getWidget('ocorrencias_importacao');
		widgetOcorrencias.dataSource.data = [];
		widgetOcorrencias.ocorrencias = ocorrencias;
	};

	this.openOcorrencias = function(widget){
        var tabbed = templateManager.container.getWidget('tabbed_import');
		var widgetOcorrencias = templateManager.container.getWidget('ocorrencias_importacao');
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == widget.currentRow.CDLINHA || ocorrencia.GERAL);
        });
        tabbed.fields.forEach(function(field){
            field.readOnly = true;
        });
	};

	this.insereRetorno = function(){
		var widgetBoletos = templateManager.container.getWidget('boletos_importacao');
        var retorno = templateManager.container.getWidget('retorno');
		var widgetOcorrencias = templateManager.container.getWidget('ocorrencias_importacao');
        if (widgetBoletos.getParent().isValid()){
            if (widgetBoletos.dataSource.data.length > 0){
                if (this.validaSituacaoCnab(widgetBoletos)){
            		var dataSet = [];
            		var route = 'fin_contaPagar#/insereRetorno';
            		var form = {
            			'DSARQCNABPAG': widgetBoletos.getParent().currentRow.DSARQCNABPAG[0].b64File.split(',')[0],
                        'NOMEARQUIVO': widgetBoletos.getParent().currentRow.DSARQCNABPAG[0].name,
            			'CDBANCCNAB': widgetBoletos.getParent().currentRow.CDBANCCNAB,
            			'CDBANCO': widgetBoletos.getParent().currentRow.CDBANCO,
            			'CDEMPRESA': widgetBoletos.getParent().currentRow.CDEMPRESA
            		};
            		dataSet.push(form);
            		dataSet.push(widgetBoletos.dataSource.data);
            		dataSet.push(widgetOcorrencias.ocorrencias);
            		General.callRestEngine(route, 'DataSet', dataSet)
            			.then(function(){
            				ScreenService.goBack().then(function(){
                                this.deleteDataRetorno();
                                widgetBoletos.getParent().isVisible = false;
                                retorno.activate();
                                retorno.reload();
                            }.bind(this));
            			}.bind(this))
            			.catch(function(erro){
            				ScreenService.showMessage(erro.data.error);
            			});
                }else{
                    ScreenService.showMessage('Registro(s) do arquivo de retorno possui(em) ocorrência(s) que não possibilita(m) o salvamento.');
                }
            }else{
                ScreenService.showMessage('Não existem boletos importados.');
            }
        }
	};

    /**
     * Cancela Título Retorno
     */
    self.cancelaTituloRetorno = function(){
        var route = 'fin_contaPagar#/cancelaTituloRetorno';
        var widgetRetornoGrid = templateManager.container.getWidget('retorno');
        var widgetBoletosGrid = templateManager.container.getWidget('boletos_retorno');

        if(widgetBoletosGrid.currentRow.IDSITUCNABP == '90') {
            ScreenService.showMessage('Operação Bloqueada. Título baixado via CNAB não pode ser cancelado do retorno.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o título do retorno?', 'question', function () {
            var dataset = {
                retorno : widgetRetornoGrid.currentRow,
                boleto  : widgetBoletosGrid.currentRow
            };

            General.callRestEngine(route, 'DataSet', dataset)
                .then(function(){
                    ScreenService.showMessage('Título cancelado do retorno com sucesso.', 'checked');
                    ScreenService.goBack();
                    widgetBoletosGrid.reload();
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        });
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag00200_RetornoPagamento_Controller', Pag00200_RetornoPagamento_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag12200_AdiantamentosDisponiveis_Controllers.js
function Pag12200_AdiantamentosDisponiveis_Controller(General, ScreenService, templateManager, ZhFilterPreferences) {
	
	this.setReadSelect = function(fieldNative, widgetFiltro, nameField){
		var field = widgetFiltro.getField(nameField);
		if(fieldNative.value() === 'L'){
			field.readOnly = false;
			field.validations = {"required": true};
		} else {
			field.setValue(null);
			delete field.validations;
			field.readOnly = true;
			if (field.widget.currentRow.$error) {
        		field.widget.currentRow.$error[nameField] = [];
    		}
		}
	};

	this.checaDisponibilidadePag = function(field, valor){
		if(field.value() === 'N' && valor > 0){
			field.setValue('S');
			ScreenService.showMessage('Operação bloqueada! Adiantamento já foi utilizado.');
		}
	};
	
	this.filter = function(widget) {
		var rowFilter = {
			'TPCDFORNECED': 'T',
			'TPCDFILIORIG': 'T',
			'TPCDFILILANC': 'T',
			'TPFORMAPAG': 'T',
			'TPCDTPCTPAGAR': 'T',
			'STATUS': 'T'
		};
		ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
		ZhFilterPreferences.openFilterWidget(widget, rowFilter);
	};

	this.AjustaFiltro = function(rowFilter){
		var widget = templateManager.container.getWidget('pag12200_adiantamentos_disponiveis_grid');
		var pos = 0;
		var deletePos = null;
		widget.dataSourceFilter.forEach(function(filter) {
			switch(filter.name){
				case 'DOCINICIAL': filter.value = (rowFilter.DOCINICIAL) ? rowFilter.DOCINICIAL : null;
					break;
				case 'DOCFINAL': filter.value = (rowFilter.DOCFINAL) ? rowFilter.DOCFINAL : null;
					break;
				case 'CDFORNECED': filter.value = (rowFilter.TPCDFORNECED == 'L') ? rowFilter.CDFORNECED.join('_') : '';
					break;
				case 'TPCDFORNECED': filter.value = rowFilter.TPCDFORNECED;
					break;
				case 'CDFILIORIG': filter.value = (rowFilter.TPCDFILIORIG == 'L') ? rowFilter.CDFILIORIG.join('_') : '';
					break;
				case 'TPCDFILIORIG': filter.value = rowFilter.TPCDFILIORIG;
					break;
				case 'CDFILILANC': filter.value = (rowFilter.TPCDFILILANC == 'L') ? rowFilter.CDFILILANC.join('_') : '';
					break;
				case 'TPCDFILILANC': filter.value = rowFilter.TPCDFILILANC;
					break;
				case 'FORMAPAG': filter.value = (rowFilter.TPFORMAPAG == 'L') ? rowFilter.FORMAPAG.join('_') : '';
					break;
				case 'TPFORMAPAG': filter.value = rowFilter.TPFORMAPAG;
					break;
				case 'CDTPCTPAGAR': filter.value = (rowFilter.TPCDTPCTPAGAR == 'L') ? rowFilter.CDTPCTPAGAR.join('_') : '';
					break;
				case 'TPCDTPCTPAGAR': filter.value = rowFilter.TPCDTPCTPAGAR;
					break;
				case 'DATAINI': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.start : null;
					break;
				case 'DATAFIN': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.end : null;
					break;
				case 'STATUS': filter.value = rowFilter.STATUS;
					break;
				case 'DATA': deletePos = pos;
					break;
			}
			pos++;
		});
		if (deletePos){
			widget.dataSourceFilter.splice(deletePos, 1);
		}
	};

	this.setRequired = function(widget, nmf1, nmf2){
		var field1 = widget.getField(nmf1);
		var field2 = widget.getField(nmf2);
		if(field1.value() || field2.value()){
			field1.validations.required = true;
			field2.validations.required = true;
		} else {
			delete field1.validations.required;
			delete field2.validations.required;
			if (widget.currentRow.$error) {
        widget.currentRow.$error[nmf1] = [];
        widget.currentRow.$error[nmf2] = [];
      }
		}
	};
	
    this.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctpagar = widget.getField('DSTPCTPAGAR');
        
        cdtpctpagar.setValue("");
        cdtpctpagar.dataSourceFilter[1].value = cdempresa;
        cdtpctpagar.reload();
    };

}

Configuration(function (ContextRegister) {
	ContextRegister.register('Pag12200_AdiantamentosDisponiveis_Controller', Pag12200_AdiantamentosDisponiveis_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag12300_NotasDevolucoesDisponiveis_Controller.js
function Pag12300_NotasDevolucoesDisponiveis_Controller(General, ScreenService, templateManager, ZhFilterPreferences) {

	this.setReadSelect = function(fieldNative, widgetFiltro, nameField){
		var field = widgetFiltro.getField(nameField);
		if(fieldNative.value() === 'L'){
			field.readOnly = false;
			field.validations = {"required": true};
		} else {
			field.setValue(null);
			delete field.validations;
			field.readOnly = true;
			if (field.widget.currentRow.$error) {
        		field.widget.currentRow.$error[nameField] = [];
    		}
		}
	};

	this.filter = function(widget) {
		var rowFilter = {
			'TPCDFORNECED' : 'T',
			'TPCDFILIORIG' : 'T',
			'TPCDTIPOOPER' : 'T',
			'STATUS'       : 'T'
		};
		ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
		ZhFilterPreferences.openFilterWidget(widget, rowFilter);
	};

	this.AjustaFiltro = function(rowFilter, preference){
		var widget = templateManager.container.getWidget('pag12300_notas_devolucoes_disponiveis_grid');
		var pos = 0;
		var deletePos = null;
		widget.dataSourceFilter.forEach(function(filter) {
			switch(filter.name){
				case 'DOCINICIAL': filter.value = (rowFilter.DOCINICIAL) ? rowFilter.DOCINICIAL : null;
					break;
				case 'DOCFINAL': filter.value = (rowFilter.DOCFINAL) ? rowFilter.DOCFINAL : null;
					break;
				case 'CDFORNECED': filter.value = (rowFilter.TPCDFORNECED == 'L') ? rowFilter.CDFORNECED.join('_') : '';
					break;
				case 'TPCDFORNECED': filter.value = rowFilter.TPCDFORNECED;
					break;
				case 'CDFILIORIG': filter.value = (rowFilter.TPCDFILIORIG == 'L') ? rowFilter.CDFILIORIG.join('_') : '';
					break;
				case 'TPCDFILIORIG': filter.value = rowFilter.TPCDFILIORIG;
					break;
				case 'CDTIPOOPER': filter.value = (rowFilter.TPCDTIPOOPER == 'L') ? rowFilter.CDTIPOOPER.join('_') : '';
					break;
				case 'TPCDTIPOOPER': filter.value = rowFilter.TPCDTIPOOPER;
					break;
				case 'DATAINI': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.start : null;
					break;
				case 'DATAFIN': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.end : null;
					break;
				case 'STATUS': filter.value = rowFilter.STATUS;
					break;
				case 'DATA': deletePos = pos;
					break;
			}
			pos++;
		});
		if (deletePos){
			widget.dataSourceFilter.splice(deletePos, 1);
		}
	};
	
	this.setRequired = function(widget, nmf1, nmf2){
		var field1 = widget.getField(nmf1);
		var field2 = widget.getField(nmf2);
		if(field1.value() || field2.value()){
			field1.validations.required = true;
			field2.validations.required = true;
		} else {
			delete field1.validations.required;
			delete field2.validations.required;
			if (widget.currentRow.$error) {
        widget.currentRow.$error[nmf1] = [];
        widget.currentRow.$error[nmf2] = [];
      }
		}
	};
	
	

	this.checaDisponibilidade = function(field, valor){
		if(field.value() === 'N' && valor > 0){
			field.setValue('S');
			ScreenService.showMessage('Operação bloqueada. Nota Fiscal de Devolução já foi utilizada.');
		}
	};
	
	this.setaMascCnpjCpfFornecedor = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURFORN.length === 14) inscFormatada = value.NRINSJURFORN.substr(0, 2) + '.' + value.NRINSJURFORN.substr(2, 3) + '.' + value.NRINSJURFORN.substr(5, 3) + '/' + value.NRINSJURFORN.substr(8, 4) + '-' + value.NRINSJURFORN.substr(12, 2);
          else if(value.NRINSJURFORN.length === 11) inscFormatada = value.NRINSJURFORN.substr(0, 3) + '.' + value.NRINSJURFORN.substr(3, 3) + '.' + value.NRINSJURFORN.substr(6, 3) + '-' + value.NRINSJURFORN.substr(9, 2);
          else inscFormatada = null;
          value.NRINSJURFORN = inscFormatada?inscFormatada:value.NRINSJURFORN;
	    });
	};
	
	this.changeHeaderValues = function(widget){
        /*var codigo; 
        var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "is equal to".concat(codigo).concat(' - ').concat(nmempresa)
        };*/
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        //newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "é igual a."){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
        
    };
}
Configuration(function (ContextRegister) {
	ContextRegister.register('Pag12300_NotasDevolucoesDisponiveis_Controller', Pag12300_NotasDevolucoesDisponiveis_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag16000_EmprestimoLeasing_Controller.js
var Pag16000_EmprestimoLeasing_Controller = function(ScreenService, templateManager, General, ZHPromise, Pag16000_EmprestimoLeasing_Service, contabilizacaoController, ZhFilterPreferences) {

    var self = this;

    this.setFiltroPorEmpresa = function(widgetFiltro){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var tabbed_emprestimo = templateManager.container.getWidget('tabbed_emprestimo');
        emprestimo.getField('NMFILIAL').dataSourceFilter[0].value = widgetFiltro.currentRow.CDEMPRESA;
        tabbed_emprestimo.getField('NMFILIAL').dataSourceFilter[0].value = widgetFiltro.currentRow.CDEMPRESA;
        widgetFiltro.getField('FILIORIG').dataSourceFilter[0].value = widgetFiltro.currentRow.CDEMPRESA;
    };

    //CONTROLE REPARCELAMENTO
    this.cancelarReparcelamento = function (args){
        var reparcelamento = args.owner.widget;
        if (reparcelamento.currentRow.edit){
            reparcelamento.currentRow.edit = false;
            reparcelamento.currentRow = angular.copy(reparcelamento.backupRow);
            args.owner.widget.view();
            var existeNovaParcela = reparcelamento.dataSource.data.filter(function(parcela){
                return parcela.__createdLocal;
            });
            if ((reparcelamento.dataSource.data.length == 1) || ((existeNovaParcela.length == 0) && !(reparcelamento.NOVOREPARCELAMENTO))){
                General.changeWidgetAcessibility(reparcelamento.widgets, true);
            }
        }else{
            General.backOrCancel(args).then(function(){
                if (reparcelamento.dataSource.data.length == 1){
                    reparcelamento.NOVOREPARCELAMENTO = false;
                }
            });
        }
    };

    this.adicionaParcela = function(reparcelamento){
        var parcelas = reparcelamento.getParent().getParent();
        if (this.validaAdicaoParcela(reparcelamento)){
            var sequencial = parcelas.currentRow.NRSEQVENPAG;
            var vencimento = parcelas.currentRow.DTATUAVENPAG;
            var numeroParcela = reparcelamento.dataSource.data.length.toString().padStart(2, "0");
            var descricao = parcelas.currentRow.DSCONTAPAG;
            this.novoParcelamento(reparcelamento).then(function(){
                var valoresNovoParcelamento = this.calculaNovosValoresReparcelamento(reparcelamento);
                diferencaValorConta = valoresNovoParcelamento[0];
                diferencaValorBoleto = valoresNovoParcelamento[1];
                diferencaAjuste = valoresNovoParcelamento[2];
                    this.setRowReparcelamento(reparcelamento, diferencaValorBoleto, diferencaAjuste, sequencial, numeroParcela, vencimento, descricao, diferencaValorConta);
                    reparcelamento.currentRow.edit = false;
            }.bind(this));
        }
    };

    this.setRowReparcelamento = function(reparcelamento, diferencaValorBoleto, diferencaAjuste, sequencial, numeroParcela, vencimento, descricao, diferencaValorConta){
        reparcelamento.NOVOREPARCELAMENTO = true;
        reparcelamento.currentRow.VRFINALBPAG = diferencaValorBoleto;
        reparcelamento.currentRow.VRAJUSTE = diferencaAjuste;
        reparcelamento.currentRow.NRSEQVENPAG = sequencial;
        reparcelamento.currentRow.NRPARCELAPAG = numeroParcela;
        reparcelamento.currentRow.DTATUAVENPAG = vencimento;
        reparcelamento.currentRow.DSCONTAPAG = descricao;
        reparcelamento.currentRow.VRATUAPAG = diferencaValorConta;
        reparcelamento.backupRow = {};
        reparcelamento.backupRow.VRATUAPAG = diferencaValorConta;
        reparcelamento.backupRow.VRAJUSTE = diferencaAjuste;
        reparcelamento.backupRow.VRFINALBPAG = diferencaValorBoleto;
    };

    this.calculaNovosValoresReparcelamento = function(reparcelamento){
        var parcelas = reparcelamento.getParent().getParent();
        var sequencial = parcelas.currentRow.NRSEQVENPAG;
        var totalConta = 0;
        var totalAjusteParcela = 0;
        var totalContaReparcelamento = 0;
        var totalAjusteReparcelamento = 0;
        var diferencaValorConta = 0;
        var diferencaValorBoleto = 0;
        var diferencaAjuste = 0;
        parcelas.dataSource.data.forEach(function(data){
            if (data.NRSEQVENPAG == sequencial){
                var valorConta = data.VRATUAPAG;
                valorConta = (valorConta.toString().search(',') == -1) ? parseFloat(valorConta) : parseFloat(valorConta.toString().replace('.', '').replace(',', '.'));
                totalConta = (parseFloat(totalConta) + parseFloat(valorConta)).toFixed(2);
                var valorAjuste = (data.VRAJUSTE) ? parseFloat(data.VRAJUSTE) : 0;
                totalAjusteParcela = (parseFloat(totalAjusteParcela) + valorAjuste).toFixed(2);
            }
        });
        reparcelamento.dataSource.data.forEach(function(data){
            if (data.NRSEQVENPAG == sequencial){
                var valorContaReparcelamento = data.VRATUAPAG;
                valorContaReparcelamento = (valorContaReparcelamento.toString().search(',') == -1) ? parseFloat(valorContaReparcelamento) : parseFloat(valorContaReparcelamento.toString().replace('.', '').replace(',', '.'));
                totalContaReparcelamento = (parseFloat(totalContaReparcelamento) + valorContaReparcelamento).toFixed(2);
                var valorAjusteReparcelamento = data.VRAJUSTE;
                valorAjusteReparcelamento = (valorAjusteReparcelamento.toString().search(',') == -1) ? parseFloat(valorAjusteReparcelamento) : parseFloat(valorAjusteReparcelamento.toString().replace('.', '').replace(',', '.'));
                totalAjusteReparcelamento = (parseFloat(totalAjusteReparcelamento) + valorAjusteReparcelamento).toFixed(2);
            }
        });
        if (parseFloat(totalConta) != parseFloat(totalContaReparcelamento)){
            diferencaValorConta = (parseFloat(totalConta) - parseFloat(totalContaReparcelamento)).toFixed(2);
            diferencaValorBoleto = (parseFloat(totalConta) - parseFloat(totalContaReparcelamento) + (parseFloat(totalAjusteParcela) - parseFloat(totalAjusteReparcelamento))).toFixed(2);
            diferencaAjuste = (parseFloat(totalAjusteParcela) - parseFloat(totalAjusteReparcelamento)).toFixed(2);
            diferencaValorConta = (diferencaValorConta < 0) ? 0 : diferencaValorConta;
        }
        return [diferencaValorConta, diferencaValorBoleto, diferencaAjuste];
    };

    /*this.recalcularParcelasPosteriores = function(reparcelamento){
        var indice = reparcelamento.currentRow.__index;
        var data = reparcelamento.dataSource.data.filter(function(parc){
            return parc.__index > indice;
        });
        var totalContaPagar = 0;
        var totalAjuste = 0;
        reparcelamento.dataSource.data.forEach(function(parc){
            if (parc.__index <= indice){
                totalContaPagar = parseFloat(totalContaPagar) + parseFloat(parc.VRATUAPAG);
                totalAjuste = parseFloat(totalAjuste) + parseFloat(parc.VRAJUSTE);
            }
        });
        data.forEach(function(data){
            reparcelamento.dataSource.data.forEach(function(parc){
                if (parc.__index <= indice){
                    totalContaPagar = parseFloat(totalContaPagar) + parseFloat(parc.VRATUAPAG);
                    totalAjuste = parseFloat(totalAjuste) + parseFloat(parc.VRAJUSTE);
                }
            });
            var valorContaAtual = data.VRATUAPAG;
            var percentual = (totalContaPagar == 0) ? 0 : parseFloat((valorContaAtual * 100 / totalContaPagar).toFixed(2));
            data.VRAJUSTE = (totalAjuste * percentual / 100).toFixed(2);
            data.VRFINALBPAG = (parseFloat(valorContaAtual) + parseFloat(data.VRAJUSTE)).toFixed(2);
            totalContaPagar = 0;
            totalAjuste = 0;
            indice++;
        });
    };*/

    this.setValoresAjusteEBoleto = function(reparcelamento){
        var valorContaAtual = parseFloat(reparcelamento.currentRow.VRATUAPAG.toString().replace('.', '').replace(',', '.'));
        var valorContaAnterior = (reparcelamento.backupRow.VRATUAPAG) ? parseFloat(reparcelamento.backupRow.VRATUAPAG) : 0;
        var percentual = (valorContaAnterior == 0) ? 0 : parseFloat((valorContaAtual * 100 / valorContaAnterior).toFixed(2));
        var valorAjuste = reparcelamento.backupRow.VRAJUSTE;
        valorAjuste = (valorAjuste) ? parseFloat(reparcelamento.backupRow.VRAJUSTE) : 0.00;
        reparcelamento.currentRow.VRAJUSTE = (valorAjuste * percentual / 100).toFixed(2);
        reparcelamento.currentRow.VRFINALBPAG = (valorContaAtual + parseFloat(reparcelamento.currentRow.VRAJUSTE)).toFixed(2);
    };

    this.validaAdicaoParcela = function(reparcelamento){
        var adicaoPermitida = true;
        var parcelasPersistidas = reparcelamento.dataSource.data.filter(function(f){return !f.__is_new;});
        if (!parcelasPersistidas[parcelasPersistidas.length - 1].hasOwnProperty('changed')){
            adicaoPermitida = false;
            ScreenService.showMessage("Operação Bloqueada. Inclusão permitida somente após edição da última parcela.");
        }
        return adicaoPermitida;
    };

    this.salvarParcelas = function(reparcelamento){
        var valorConta = reparcelamento.currentRow.VRATUAPAG;
        valorConta = (valorConta.toString().search(',') == -1) ? parseFloat(valorConta) : parseFloat(valorConta.toString().replace('.', '').replace(',', '.'));
        if (reparcelamento.currentRow.edit){
            reparcelamento.currentRow.edit = false;
            reparcelamento.view();
            reparcelamento.NOVOREPARCELAMENTO = true;
            this.validaVisibilidadeDeleteParcelas(reparcelamento);
            //this.recalcularParcelasPosteriores(reparcelamento);
        }else{
            if (valorConta > 0){
                ScreenService.closeSwipe().then(function(){
                    reparcelamento.NOVOREPARCELAMENTO = true;
                    reparcelamento.list();
                    reparcelamento.currentRow.edit = false;
                    this.validaVisibilidadeDeleteParcelas(reparcelamento);
                }.bind(this));
            }else{
                ScreenService.showMessage("Valor da conta deve ser maior que zero.");
            }
        }
    };

    this.editarReparcelamento = function(reparcelamento){
        var data = reparcelamento.dataSource.data;
        var route = 'fin_contaPagar#/validaPermissaoEdicaoTitulo';
        var DataPersistida = data.filter(function(data){
            return !data.hasOwnProperty('__createdLocal');
        });
        var ultimaParcelaPersistida = DataPersistida.length - 1;
        if (data.length > 1){
            if (reparcelamento.currentRow.hasOwnProperty('__createdLocal')){
                this.executaEdit(reparcelamento);
            }else{
                General.callRestEngine(route, 'Row', reparcelamento.currentRow).then(function(){
                    if (reparcelamento.currentRow.__index == ultimaParcelaPersistida){
                        this.executaEdit(reparcelamento);
                    }else{
                        ScreenService.showMessage('Operação Bloqueada. É permitido a edição apenas da última parcela do título.');
                    }
                }.bind(this))
                .catch(function(error){
                    ScreenService.showMessage(error.data.error);
                });
            }
        }else{
            if (reparcelamento.currentRow.hasOwnProperty('__createdLocal')){
                this.executaEdit(reparcelamento);
            }else{
                General.callRestEngine(route, 'Row', reparcelamento.currentRow)
                .then(function(){
                    this.executaEdit(reparcelamento);
                }.bind(this))
                .catch(function(error){
                    ScreenService.showMessage(error.data.error);
                });
            }
        }
    };

    this.deleteReparcelamentoEmprestimoPagar = function (args) {
        var reparcelamento = templateManager.container.getWidget('reparcelamento');
        if (reparcelamento.dataSource.data.length > 1){
            if (!reparcelamento.currentRow.__createdLocal) {
                var ultimaParcela = reparcelamento.dataSource.data.length - 1;
                if (reparcelamento.currentRow.__index == ultimaParcela){
                    this.removeUltimaParcela(reparcelamento);
                }else{
                    ScreenService.showMessage('Operação Bloqueada. Somente é permitido a exclusão da última parcela do título.');
                }
            }else{
                this.remocaoLogicaParcelas(args);
            }
        }else{
            ScreenService.showMessage('Operação Bloqueada. Não é possível excluir a parcela original.');
        }
    };

    this.remocaoLogicaParcelas = function(args){
        General.backOrCancel(args);
    };

    this.removeUltimaParcela = function(reparcelamento){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var route = 'fin_contaPagar#/deleteReparcelamentoEmprestimoPagar';
        var parcelas = reparcelamento.getParent().getParent();
        this.emprestimoRealizadoNoPeriodoContabil(emprestimo).then(function(){
            ScreenService.confirmMessage(ScreenService.i18n('Confirmar exclusão da última parcela?'), 'question', function () {
                reparcelamento.dataSource.data.forEach(function(data){
                    data.CDAJUCTPAG = emprestimo.currentRow.CDAJUCTPAG;
                });
                General.callRestEngine(route, 'DataSet', reparcelamento.dataSource.data)
                    .then(function () {
                        var nrseqvenpag = reparcelamento.currentRow.NRSEQVENPAG;
                        ScreenService.closeSwipe().then(function(){
                            parcelas.reload().then(function(){
                                var rowsParcela = parcelas.dataSource.data.filter(function(parcela){
                                    return parcela.NRSEQVENPAG == nrseqvenpag;
                                });
                                var sequenciaisParcela = rowsParcela.map(function(m){return m.NRPARCELAPAG;});
                                var parcelaReferencia = sequenciaisParcela.reduce(function(a, b) {
                                    return Math.max(a, b);
                                });
                                var currentRow = rowsParcela.filter(function(parcela){
                                    return parcela.NRPARCELAPAG == parcelaReferencia.toString().padStart(2, "0");
                                })[0];
                                parcelas.currentRow = currentRow;
                            });
                            reparcelamento.reload();
                            ScreenService.showMessage('Registro excluído com sucesso!');
                            this.validaVisibilidadeDeleteParcelas(reparcelamento);
                        }.bind(this));
                    }.bind(this))
                    .catch(function (response) {
                        ScreenService.showMessage(response.data.error);
                    });
            }.bind(this));
        }.bind(this))
        .catch(function (error) {
            ScreenService.showMessage(error);
        });

    };
    //CONTROLE REPARCELAMENTO
    this.setEmpresaStripe = function(filtro){
        var stripe = templateManager.container.getWidget('stripe');
        stripe.currentRow.EMPRESA = ": " + filtro.FILTER_ROW.CDEMPRESA + " - " + filtro.FILTER_ROW.NMEMPRESA;
    };

    this.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                var cdempresa = empresa.CDEMPRESA;
                var nmempresa = empresa.NMEMPRESA;
                deferred.resolve(empresa);
            }.bind(this));
        }.bind(this));
        return deferred.promise;
    };

    this.setPrimeiroVencimento = function(parcelasGrid){
        var tabbed_emprestimo = parcelasGrid.getParent();
        if (parcelasGrid.currentRow.__index == 0){
            var data = parcelasGrid.currentRow.DTATUAVENPAG_inline;
            if (data.toString().search('/') == -1){
                parcelasGrid.currentRow.DTATUAVENPAG_inline = data.substr(0, 2) + '/' + data.substr(2, 2) + '/' + data.substr(4, 4);
            }
            tabbed_emprestimo.currentRow.DTPRIVENCEMP = parcelasGrid.currentRow.DTATUAVENPAG_inline;
        }
    };

    this.setQtdeParcelas = function(parcelas){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        emprestimo.currentRow.PARCELAS = parcelas.dataSource.data.length;
    };

    this.controlaVisibilidadeActionContabilizacao = function(widget){
        if (widget.currentRow.NRLCTBMDEB){
            widget.getAction('ConsultarCtbEmprestimoDebito').isVisible = true;
        }else{
            widget.getAction('ConsultarCtbEmprestimoDebito').isVisible = false;
        }
        if (widget.currentRow.NRLCTBMCRED){
            widget.getAction('ConsultarCtbEmprestimoCredito').isVisible = true;
        }else{
            widget.getAction('ConsultarCtbEmprestimoCredito').isVisible = false;
        }
    };

    this.controlaVisibilidadeActionEdicao = function(widget){
        if (widget.dataSource.data > 0){
            widget.getAction('editar').isVisible = true;
        }else{
            widget.getAction('editar').isVisible = false;
        }
    };

    this.controlaVisibilidadeContabilizacaoPagamento = function(widget){
        if (widget.currentRow.NRLANCTBDOC){
            widget.getAction('ConsultarCtbEmprestimoPagamento').isVisible = true;
        }else{
            widget.getAction('ConsultarCtbEmprestimoPagamento').isVisible = false;
        }
    };

    this.consultaContabilizacaoEmprestimoPagamento = function(){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var contabil = [{
            "CDEMPRESA": emprestimo.currentRow.CDEMPRESA,
            "NRLANCCONT": emprestimo.currentRow.NRLANCTBDOC
        }];
        contabilizacaoController.exibeContabilizacao(contabil);
    };

    this.consultaContabilizacaoEmprestimoDebito = function(){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var credito = templateManager.container.getWidget('lancamentoIof');
        var contabil = [{
            "CDEMPRESA": emprestimo.currentRow.CDEMPRESA,
            "NRLANCCONT": credito.currentRow.NRLCTBMDEB
        }];
        contabilizacaoController.exibeContabilizacao(contabil);
    };

    this.consultaContabilizacaoEmprestimoCredito = function(){
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var credito = templateManager.container.getWidget('lancamentoIof');
        var contabil = [{
            "CDEMPRESA": emprestimo.currentRow.CDEMPRESA,
            "NRLANCCONT": credito.currentRow.NRLCTBMCRED
        }];
        contabilizacaoController.exibeContabilizacao(contabil);
    };

    this.setDescricaoCredito = function(widget){
        if (!widget.currentRow.DSMOVICRED){
            widget.currentRow.DSMOVICRED = widget.currentRow.DSTPCREDBANC;
        }
    };

    this.setDescricaoDebito = function(widget){
        if (!widget.currentRow.DSMOVIDEBI){
            widget.currentRow.DSMOVIDEBI = widget.currentRow.DSTPDEBIBANC;
        }
    };

    this.editarCredito = function(widget){
        General.widgetEdit(widget);
        if (widget.getField('DEBIOF').value() == 'S'){
            widget.getField('DEBIOF').readOnlyOnEdit = false;
            widget.getField('NRDOCMBANC').readOnlyOnEdit = false;
            widget.getField('DSMOVIDEBI').readOnlyOnEdit = false;
        }else{
            widget.getField('DEBIOF').setValue('N');
            widget.getField('DEBIOF').readOnlyOnEdit = true;
            widget.getField('NRDOCMBANC').readOnlyOnEdit = true;
            widget.getField('DSMOVIDEBI').readOnlyOnEdit = true;
        }
        if (widget.getField('CREDITO').value() == 'S'){
            widget.getField('CREDITO').readOnlyOnEdit = false;
            widget.getField('NRDOCMBANCCRED').readOnlyOnEdit = false;
            widget.getField('DSMOVICRED').readOnlyOnEdit = false;
        }else{
            widget.getField('CREDITO').setValue('N');
            widget.getField('CREDITO').readOnlyOnEdit = true;
            widget.getField('NRDOCMBANCCRED').readOnlyOnEdit = true;
            widget.getField('DSMOVICRED').readOnlyOnEdit = true;
        }
    };

    this.setContaCorrenteCredito = function(widget){
        if (widget.getField('DSCONTCORR').value()){
            widget.getField('DSCONTCORRCRED').setValue(widget.getField('DSCONTCORR').value());
            widget.getField('CDAGENCIACRED').setValue(widget.getField('CDAGENCIA').value());
            widget.getField('CDCONTCORRCRED').setValue(widget.getField('CDCONTCORR').value());
            widget.getField('CDBANCOCRED').setValue(widget.getField('CDBANCO').value());
            widget.currentRow.CDREDUCONTCRED = widget.currentRow.CDREDUCONT;
            widget.currentRow.CDCONTCTBLCRED = widget.currentRow.CDCONTCTBL;
            widget.currentRow.IDTIPOCCORRCRED = widget.currentRow.IDTIPOCCORR;
            widget.getField('DSCONTCORRCRED').readOnlyOnEdit = true;
            widget.getField('CDAGENCIACRED').readOnlyOnEdit = true;
            widget.getField('CDCONTCORRCRED').readOnlyOnEdit = true;
            widget.getField('CDBANCOCRED').readOnlyOnEdit = true;
        }else{
            widget.getField('DSCONTCORRCRED').readOnlyOnEdit = false;
            widget.getField('CDAGENCIACRED').readOnlyOnEdit = false;
            widget.getField('CDCONTCORRCRED').readOnlyOnEdit = false;
            widget.getField('CDBANCOCRED').readOnlyOnEdit = false;
        }
    };

    this.saveCreditoEmprestimoPagar = function(widget){
        var route = 'fin_contaPagar#/saveCreditoEmprestimoPagar';
        General.callRestEngine(route, 'Row', widget.currentRow).then(function () {
            if (General.formWidgetSwitchTemplate(widget) && widget.getParent()) {
                var widgets = widget.getParent().widgets;
                General.changeWidgetAcessibility(widgets.concat(widget.getParent()), true);
            }
            widget.currentRow.changed = false;
            delete widget.backupRow;
            widget.view();
        }.bind(this), function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    this.voltarECancelReparcelamento = function(reparcelamento){
        ScreenService.goBack().then(function(){
            reparcelamento.NOVOREPARCELAMENTO = false;
            General.changeWidgetAcessibility(reparcelamento.widgets, true);
            reparcelamento.getParent().getParent().activate();
        });
    };

    this.iniciaFiltroPagamento = function(widget){
        this.buscaPadraoEmpresa(widget).then(function(empresa){
            var data = moment().format('DD/MM/YYYY');
            var start = moment().startOf('month').format('DD/MM/YYYY');
            var end   = moment().endOf('month').format('DD/MM/YYYY');
            var rowFilter = {};
            if (empresa.hasOwnProperty('CDEMPRESA')){
                rowFilter = {
                    'DTPRIVENCEMP': {start : start, end : end},
                    'DTEMPRESTIMO': {start : start, end : end},
                    'CDEMPRESA': empresa.CDEMPRESA,
                    'NMEMPRESA': empresa.NMEMPRESA,
                    'IDSTATPAG'   : 'T',
                    'SITUACAO'    : '',
                    'IDSTATUS'    : 'T'
                };
            }else{
                rowFilter = {
                    'DTPRIVENCEMP': {start : start, end : end},
                    'DTEMPRESTIMO': {start : start, end : end},
                    'IDSTATPAG'   : 'T',
                    'SITUACAO'    : '',
                    'IDSTATUS'    : 'T'
                };
            }
            widget.getField('DSTPCTPAGAR').dataSourceFilter[1].value = empresa.CDEMPRESA;
            widget.getField('DSTPCTPAGAR').reload();
            
            this.openFiltro(widget, rowFilter);
        }.bind(this));
    };

    this.openFiltro = function(widget, rowFilter){
        var deferred = ZHPromise.defer();
        ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
        ZhFilterPreferences.openFilterWidget(widget, rowFilter);
        deferred.resolve();
        return deferred.promise;
    };

    this.validaVisibilidadeDeleteParcelas = function(widget){
        if (widget.NOVOREPARCELAMENTO){
            widget.getAction('CONFIRMAR').isVisible = true;
        }else{
            widget.getAction('CONFIRMAR').isVisible = false;
        }
    };

    this.validaDisponibilidadeDebCredIOF = function(widget, agrupamento){
        var naoValidar = ['CDBANCO', 'CDAGENCIA', 'CDCONTCORR', 'CDBANCOCRED', 'CDAGENCIACRED', 'CDCONTCORRCRED', 'NRDOCMBANC', 'NRDOCMBANCCRED', agrupamento];
        var somenteLeitura = (widget.getField(agrupamento).value() != 'S');
        var validations = (widget.getField(agrupamento).value() == 'S') ? {"required": true} : {};
        widget.fields.forEach(function(field){
            if (!naoValidar.includes(field.name) && field.fieldGroup == agrupamento){
                field.readOnlyOnEdit = somenteLeitura;
                field.validations = validations;
            }
        });
        if (agrupamento == 'CREDITO' ){
            widget.currentRow.DTMOVICRED = (somenteLeitura) ? null : moment().format('DD/MM/YYYY');
            if (widget.getField(agrupamento).value() == 'S'){
                widget.getField('NRDOCMBANCCRED').readOnlyOnEdit = false;
                widget.getField('DSCONTCORRCRED').readOnlyOnEdit = false;
            }else{
                widget.getField('NRDOCMBANCCRED').readOnlyOnEdit = true;
            }
        }
        if (agrupamento == 'DEBIOF' ){
            if (widget.getField(agrupamento).value() == 'S'){
                widget.getField('NRDOCMBANC').readOnlyOnEdit = false;
                widget.getField('DSCONTCORRCRED').readOnlyOnEdit = true;
            }else{
                widget.getField('NRDOCMBANC').readOnlyOnEdit = true;
                if (widget.getField('CREDITO').value() == 'S'){
                    widget.getField('DSCONTCORRCRED').readOnlyOnEdit = false; 
                 }
            }
            widget.currentRow.DTMOVIDEBI = (somenteLeitura) ? null : moment().format('DD/MM/YYYY');
            
             
        }
        this.setValorCredito();
        
        templateManager.updateTemplate();
    };

    this.setDocumento = function(args) {
        var emprestimo = args.owner.field.widget.container.getWidget('tabbed_emprestimo');

       /* emprestimo.getField('CDSERIE').value('');
        emprestimo.getField('DSSERIE').value('');
        emprestimo.getField('NRNOTAFISC').value('');*/

        var nrtamnumnfut = (emprestimo.currentRow.NRTAMNUMNFUTDFE > 0) ? emprestimo.currentRow.NRTAMNUMNFUTDFE : 9;
        emprestimo.getField('NRNOTAFISC').maxlength = nrtamnumnfut;

        if(emprestimo.getField('NMFILIAL').value()){
            var currentRow = {
                NRORG      : getLocalVar('NRORG'),
                CDCONTADOR : 'NRNOTAFISCPAG'
            };
            var route = 'fin_contaPagar#/geraNovoCodigo';
            General.callRestEngine(route, 'Row', currentRow).then(function (novocodigo) {
                var nrnotafisc = novocodigo.dataset.geraNovoCodigo.novocodigo.slice(-nrtamnumnfut);
                emprestimo.getField('NRNOTAFISC').setValue(nrnotafisc);
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    this.validaDatacontratoMenorVencimento = function(widget){
        var dataValida = true;
        var arrayVencimento = widget.currentRow.DTPRIVENCEMP.split('/');
        var arrayContrato = widget.currentRow.DTEMPRESTIMO.split('/');
        var anoVencimento = arrayVencimento.pop();
        var mesVencimento = arrayVencimento.pop();
        var diaVencimento = arrayVencimento.pop();
        var anoContrato = arrayContrato.pop();
        var mesContrato = arrayContrato.pop();
        var diaContrato = arrayContrato.pop();
        var dataVencimento = parseInt(anoVencimento.concat(mesVencimento.concat(diaVencimento)));
        var dataContrato = parseInt(anoContrato.concat(mesContrato.concat(diaContrato)));
        if (dataVencimento < dataContrato){
            dataValida = false;
            ScreenService.showMessage('Data do Primeiro Vencimento deve ser maior ou igual a data do Contrato.');
        }
        return dataValida;
    };

    this.carregaGridParcelas = function(){
        var parcelasGrid = templateManager.container.getWidget('parcelasGrid');
        var emprestimo = templateManager.container.getWidget('tabbed_emprestimo');
        var numeroParcelas = parseFloat(emprestimo.currentRow.PARCELAS);
        var totalEmprestimo = emprestimo.currentRow.VRTOTALEMP;
        if (totalEmprestimo){
            totalEmprestimo = (totalEmprestimo.toString().search(',') != -1) ? parseFloat(totalEmprestimo.toString().replace('.', '').replace(',', '.')) : parseFloat(totalEmprestimo);
        }else{
            totalEmprestimo = 0;
        }
        var tipoConta = emprestimo.currentRow.DSTPCTPAGAR;
        var dataIncrement = '';
        var data = emprestimo.currentRow.DTPRIVENCEMP.split('/');
        if (data){
             var dia = data[0].toString();
            var mes = data[1].toString();
            var ano = data[2].toString();
            dataIncrement = moment(mes + "/" + dia + "/" + ano);
        }
        emprestimo.VALORPARCELA = (totalEmprestimo/numeroParcelas);
        var sistemaAmortizacao = emprestimo.currentRow.IDTIPOEMP;
        var taxaJuros = self.formataValoresDecimais(emprestimo.currentRow.VRTXJUROSEMP);

        if ((numeroParcelas) && (totalEmprestimo > 0) && (tipoConta)){
             if (this.emprestimoValido(emprestimo)){
                if (taxaJuros == 0){
                    emprestimo.currentRow.VRTOTALAJUSTEMP = 0;
                }
                var listaParcelas = Pag16000_EmprestimoLeasing_Service.efetuaCalculoParcelas(
                    parseFloat(numeroParcelas), parseFloat(totalEmprestimo), tipoConta, parseFloat(emprestimo.currentRow.VRTOTALAJUSTEMP), dataIncrement, sistemaAmortizacao, taxaJuros
                    );
                parcelasGrid.dataSource.data = listaParcelas;
                var totalJuros = 0;
                listaParcelas.forEach(function(parcelas){
                    totalJuros = parseFloat(totalJuros) + parseFloat(parcelas.VRAJUSTE);
                });
                emprestimo.currentRow.VRTOTALAJUSTEMP = totalJuros;
            }
        }else{
            parcelasGrid.dataSource.data = [];
        }
    };

    this.emprestimoValido = function(emprestimo){
        var emprestimoValido = true;
        var numeroParcelas = parseFloat(emprestimo.currentRow.PARCELAS);
        var totalEmprestimo = parseFloat(emprestimo.currentRow.VRTOTALEMP.toString().replace('.', '').replace(',', '.')).toFixed(2);
        var valorMinimoParcelas = 0.01;
        if ((totalEmprestimo / numeroParcelas) < valorMinimoParcelas){
            emprestimoValido = false;
            ScreenService.showMessage('Valor total do empréstimo não pode ser dividido para o número de parcelas informadas.');
        }

        return emprestimoValido;
    };

    this.setValorCredito = function(){
        var tabbed_emprestimo = templateManager.container.getWidget('tabbed_emprestimo');
        var credito = templateManager.container.getWidget('credito');
        if (credito.getField('CREDITO').value() == 'S'){
            credito.getField('VRMOVICRED').setValue(tabbed_emprestimo.getField('VRTOTALEMP').value());
        }
        if (tabbed_emprestimo.getField('VRTOTALEMP').value() == 0){
            ScreenService.showMessage('Valor total do empréstimo deve ser maior do que 0.');
        }
    };

    this.setValoresAjusteEBoleto = function(reparcelamento){
        var valorContaAtual = parseFloat(reparcelamento.currentRow.VRATUAPAG.toString().replace('.', '').replace(',', '.'));
        var valorContaAnterior = (reparcelamento.backupRow.VRATUAPAG) ? parseFloat(reparcelamento.backupRow.VRATUAPAG) : 0;
        var percentual = (valorContaAnterior == 0) ? 0 : parseFloat((valorContaAtual * 100 / valorContaAnterior).toFixed(2));
        var valorAjuste = reparcelamento.backupRow.VRAJUSTE;
        valorAjuste = (valorAjuste) ? parseFloat(reparcelamento.backupRow.VRAJUSTE) : 0.00;
        reparcelamento.currentRow.VRAJUSTE = (valorAjuste * percentual / 100).toFixed(2);
        reparcelamento.currentRow.VRFINALBPAG = (valorContaAtual + parseFloat(reparcelamento.currentRow.VRAJUSTE)).toFixed(2);
    };

    this.novoParcelamento = function(reparcelamento){
        var deferred = ZHPromise.defer();
        reparcelamento.newRow();
        deferred.resolve();
        return deferred.promise;
    };

    this.alteraValorBoletoConta = function(widget){
        var indice = widget.currentRow.__index;
        var data = widget.dataSource.data;
        var valorConta = widget.currentRow.VRATUAPAG_inline;
        valorConta = (valorConta.toString().search(',') == -1) ? parseFloat(valorConta) : parseFloat(valorConta.toString().replace('.', '').replace(',', '.'));
        data[indice].VRFINALBPAG = (parseFloat(data[indice].VRAJUSTE) + parseFloat(valorConta)).toFixed(2);
    };

    this.alteraValorBoletoAjuste = function(widget){
        var indice = widget.currentRow.__index;
        var data = widget.dataSource.data;
        data[indice].VRFINALBPAG = (parseFloat(widget.currentRow.VRAJUSTE_inline.toString().replace('.', '').replace(',', '.')) + parseFloat(data[indice].VRATUAPAG)).toFixed(2);
    };

    this.aplicaJurosAoAjusteEBoleto = function(field){
        var emprestimo = templateManager.container.getWidget('tabbed_emprestimo');
        var valorTotalEmprestimo = emprestimo.currentRow.VRTOTALEMP;
        if (valorTotalEmprestimo){
            if (valorTotalEmprestimo > 0){
                var parcelasGrid = templateManager.container.getWidget('parcelasGrid');
                if (emprestimo.getField('IDTIPOEMP').value() == "O"){
                    if (field.name == 'VRTOTALAJUSTEMP'){
                        emprestimo.currentRow.VRTXJUROSEMP = (parseFloat(emprestimo.currentRow.VRTOTALAJUSTEMP).toFixed(2) * 100 / parseFloat(emprestimo.currentRow.VRTOTALEMP).toFixed(2));
                    }else{
                        valorTotalEmprestimo = (valorTotalEmprestimo.toString().search(',') != -1) ? parseFloat(valorTotalEmprestimo.replace('.', '').replace(',', '.')) : parseFloat(valorTotalEmprestimo);
                        emprestimo.currentRow.VRTOTALAJUSTEMP = ((parseFloat(emprestimo.currentRow.VRTXJUROSEMP) * valorTotalEmprestimo).toFixed(2) / 100).toFixed(2);
                    }
                }
                this.carregaGridParcelas();
            }
        }
    };

    this.validaValorAjuste = function(widget){
        if (widget.currentRow.IDTIPOEMP == 'O'){
            widget.getField('VRTOTALAJUSTEMP').readOnlyOnEdit = false;
        }else{
            widget.getField('VRTOTALAJUSTEMP').readOnlyOnEdit = true;
        }
        widget.getField('VRTXJUROSEMP').setValue(0);
        widget.getField('VRTOTALAJUSTEMP').setValue(0);
    };

    this.setValidationsTipoAjuste = function(widget){
        var valorJuros = widget.getField('VRTOTALAJUSTEMP');
        var tipoAjuste = widget.getField('DSAJUCTPAG');
        if (valorJuros.value() > 0){
            tipoAjuste.validations = {"required": true};
            tipoAjuste.readOnlyOnEdit =  false;
        }else{
            tipoAjuste.validations = {};
            tipoAjuste.readOnlyOnEdit = true;
        }
    };

    this.aplicaFiltroEStripe = function(emprestimo){
        this.aplicaFiltro(emprestimo, emprestimo.appliedFilter).then(function(){
            this.setEmpresaStripe(emprestimo.appliedFilter);
        }.bind(this));
    };

    this.aplicaFiltro = function (popupFilter) {
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var deferred = ZHPromise.defer();
        var dtemissao    = false;
        var dtentrada    = false;
        var dtprivencemp = false;
        var dtemprestimo = false;
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTEMISSAO')){
            if (popupFilter.FILTER_ROW.DTEMISSAO.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTEMISSAO.start))
                dtemissao = true;
        }
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTENTRADA')){
            if (popupFilter.FILTER_ROW.DTENTRADA.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTENTRADA.start))
                dtentrada = true;
        }
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTPRIVENCEMP')){
            if (popupFilter.FILTER_ROW.DTPRIVENCEMP.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTPRIVENCEMP.start))
                dtprivencemp = true;
        }
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTEMPRESTIMO')){
            if (popupFilter.FILTER_ROW.DTEMPRESTIMO.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTEMPRESTIMO.start))
                dtemprestimo = true;
        }
        emprestimo.dataSourceFilter.forEach(function(filter) {
            switch(filter.name) {
                case 'DTEMISSAOINI': filter.value = (dtemissao) ? popupFilter.FILTER_ROW.DTEMISSAO.start : null;
                    break;
                case 'DTEMISSAOFIN': filter.value = (dtemissao) ? popupFilter.FILTER_ROW.DTEMISSAO.end : null;
                    break;
                case 'TDTEMISSAO': filter.value = (dtemissao) ? null : 'T';
                    break;
                case 'DTENTRADAINI': filter.value = (dtentrada) ? popupFilter.FILTER_ROW.DTENTRADA.start : null;
                    break;
                case 'DTENTRADAFIN': filter.value = (dtentrada) ? popupFilter.FILTER_ROW.DTENTRADA.end : null;
                    break;
                case 'TDTENTRADA': filter.value = (dtentrada) ? null : 'T';
                    break;
                case 'DTVENCINI': filter.value = (dtprivencemp) ? popupFilter.FILTER_ROW.DTPRIVENCEMP.start : null;
                    break;
                case 'DTVENCFIN': filter.value = (dtprivencemp) ? popupFilter.FILTER_ROW.DTPRIVENCEMP.end : null;
                    break;
                case 'TDTATUAVENPAG': filter.value = (dtprivencemp) ? null : 'T';
                    break;
                case 'DTCONTRATOINI': filter.value = (dtemprestimo) ? popupFilter.FILTER_ROW.DTEMPRESTIMO.start : null;
                    break;
                case 'DTCONTRATOFIN': filter.value = (dtemprestimo) ? popupFilter.FILTER_ROW.DTEMPRESTIMO.end : null;
                    break;
                case 'TDTCONTRATOEMP': filter.value = (dtemprestimo) ? null : 'T';
                    break;
                case 'TNOTA': filter.value = (popupFilter.FILTER_ROW.NRNOTAFISCINI && popupFilter.FILTER_ROW.NRNOTAFISCFIN) ? null : 'T';
                    break;
                case 'TFILIORIG': filter.value = (popupFilter.FILTER_ROW.FILIORIG) ? null : 'T';
                    break;
                case 'TFILILANC': filter.value = (popupFilter.FILTER_ROW.FILILANC) ? null : 'T';
                    break;
                case 'TVRDOC': filter.value = (popupFilter.FILTER_ROW.VRDOCUINI && popupFilter.FILTER_ROW.VRDOCUFIN) ? null : 'T';
                    break;
                case 'TCDTPCTPAGAR': filter.value = (popupFilter.FILTER_ROW.CDTPCTPAGAR) ? null : 'T';
                    break;
                case 'TVRORIGPAG': filter.value = ((!popupFilter.FILTER_ROW.VRORIGINI) && (!popupFilter.FILTER_ROW.VRORIGFIN)) ? 'T' : null;
                    break;
                case 'TVRNOTAFISC': filter.value = ((!popupFilter.FILTER_ROW.VRLIQUINI) && (!popupFilter.FILTER_ROW.VRLIQUFIN)) ? 'T' : null;
                    break;
                case 'TVRTOTALEMP': filter.value = ((!popupFilter.FILTER_ROW.VREMPRINI) && (!popupFilter.FILTER_ROW.VREMPRFIN)) ? 'T' : null;
                    break;
                case 'TIDSTATPAG': filter.value = ((popupFilter.FILTER_ROW.IDSTATPAG) && (popupFilter.FILTER_ROW.IDSTATPAG != 'T')) ? null : 'T';
                    break;
                case 'TCDFORNECED': filter.value = (popupFilter.FILTER_ROW.CDFORNECED) ? null : 'T';
                    break;
                case 'SITUACAOVENCIDAS': filter.value = (popupFilter.FILTER_ROW.SITUACAO == '1') ? null : 'T';
                    break;
                case 'SITUACAOAVENCER': filter.value = (popupFilter.FILTER_ROW.SITUACAO == '0') ? null : 'T';
                    break;
                case 'IDSTATUS': var statusNota = '';
                    if (popupFilter.FILTER_ROW.IDSTATUS == 'A'){
                        statusNota = ['A', 'E'];
                    } else if (popupFilter.FILTER_ROW.IDSTATUS == 'V'){
                        statusNota = ['V'];
                    } else {
                        statusNota = null;
                    }
                    filter.value = statusNota;
                    break;
                case 'TIDSTATUS': filter.value = ( (!popupFilter.FILTER_ROW.hasOwnProperty(IDSTATUS)) || (popupFilter.FILTER_ROW.IDSTATUS == 'T')) ? 'T' : null;
                    break;
                default: break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.setValorFinal = function(widget, valorInicial, valorFinal){
        var novoValorInicial = (valorInicial.value()) ? valorInicial.value() : 0;
        valorInicial.setValue(novoValorInicial);
        widget.getField(valorFinal).setValue(valorInicial.value());
        /*var novoValorInicial = (valorInicial.value()) ? valorInicial.value() : 0;
        valorInicial.value(novoValorInicial);
        var min = (valorInicial.toString().search(',') == -1) ? parseFloat(valorInicial.value()) : parseFloat(valorInicial.value().toString().replace('.', '').replace(',', '.'));
        widget.getField(valorFinal).validations.range.min = min;*/
    };

    this.validaIsReload = function(componente){
        if (componente.isReload){
            componente.reload().then(function(){
                this.validaReloadComponente(componente, false);
            }.bind(this));
        }
    };

    this.validaReloadComponente = function(componente, status){
        componente.isReload = status;
    };

    this.openReparcelamento = function(){
        var tabbed_reparcelamento = templateManager.container.getWidget('tabbed_reparcelamento');
        var reparcelamento = templateManager.container.getWidget('reparcelamento');
        reparcelamento.dataSourceFilter[0].value = tabbed_reparcelamento.getParent().currentRow.CDFILIAL;
        reparcelamento.dataSourceFilter[1].value = tabbed_reparcelamento.getParent().currentRow.NRLANCTONF;
        reparcelamento.dataSourceFilter[2].value = tabbed_reparcelamento.getParent().currentRow.NRSEQVENPAG;
        ScreenService.openSwipe(tabbed_reparcelamento).then(function(){
            reparcelamento.reload().then(function(){
                reparcelamento.dataExclusao = [angular.copy(reparcelamento.dataSource.data[reparcelamento.dataSource.data.length - 1])];
            });
            reparcelamento.NOVOREPARCELAMENTO = false;
            this.validaVisibilidadeDeleteParcelas(reparcelamento);
        }.bind(this));
    };

    this.executaEdit = function(reparcelamento){
        General.widgetEdit(reparcelamento);
        reparcelamento.backupRow = angular.copy(reparcelamento.currentRow);
        reparcelamento.currentRow.edit = true;
    };

    this.validaSituacao = function(widget){
        var idstatpag = widget.getField('IDSTATPAG');
        var situacao = widget.getField('SITUACAO');
        if (idstatpag.value() == 'B'){
            situacao.readOnly = true;
        }else{
            situacao.readOnly = false;
        }
    };

    this.setDescricaoAjusteEmprestimo = function(widget){
        widget.currentRow.DSAJUSTEEMP = widget.currentRow.DSAJUCTPAG;
    };

    this.salvarEmprestimo = function () {
        var emprestimo = templateManager.container.getWidget('tabbed_emprestimo');
        var parcelas = templateManager.container.getWidget('parcelasGrid');
        var credito = templateManager.container.getWidget('credito');
        var route = 'fin_contaPagar#/saveEmprestimoPagar/save';
        var dataSet = [];
        if (emprestimo.isValid())
        {
            if (credito.isValid()){
                if (this.validaCondicoesEmprestimo(emprestimo, parcelas, credito)){
                    emprestimo.currentRow.CDTPGARANTIAEMP = (emprestimo.currentRow.hasOwnProperty('CDTPGARANTIAEMP')) ? emprestimo.currentRow.CDTPGARANTIAEMP : null;
                    dataSet.push(emprestimo.currentRow);
                    dataSet.push(parcelas.dataSource.data);
                    dataSet.push(credito.currentRow);
                    this.emprestimoRealizadoNoPeriodoContabil(emprestimo, emprestimo.currentRow.DTEMPRESTIMO).then(function(){
                        General.callRestEngine(route, 'DataSet', dataSet)
                        .then(
                            function () {
                                ScreenService.closeSwipe().then(function(){
                                    this.deleteValoresInsercaoEmprestimo();
                                    parcelas.getParent().isVisible = false;
                                    emprestimo.getParent().activate();
                                    emprestimo.getParent().reload();
                                    ScreenService.showMessage('Registro salvo com sucesso!');
                                }.bind(this));
                            }.bind(this)
                        )
                        .catch(
                            function (error) {
                                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                                    ScreenService.showMessage(error.data.error);
                                }
                            }
                        );
                    }.bind(this))
                    .catch(function(error){
                        ScreenService.showMessage(error);
                    });
                }
            }else{
                credito.activate();
            }
        }
    };

    this.emprestimoRealizadoNoPeriodoContabil = function(emprestimo, dataReferencia){
        var deferred = ZHPromise.defer();
        var filtroContopectbl = [{
            "name": "CDEMPRESA",
            "operator": "=",
            "value": emprestimo.currentRow.CDEMPRESA
        }];
        General.callRestEngine('fin#/crud/contopectbl', 'FilterData', filtroContopectbl).then(function(response){
            var contopectbl = response.dataset.contopectbl[0];
            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;
            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(dataReferencia, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(dataReferencia, 'DD/MM/YYYY'))) {
                    deferred.reject('Operação Bloqueada. Data de entrada fora do período financeiro.');
                }else{
                    deferred.resolve();
                }
            }
        }.bind(this));
        return deferred.promise;
    };

    this.validaDatas = function(dataInicial, dataFinal){
        var inicial = dataInicial.split('/');
        var final = dataFinal.split('/');
        var anoIni = inicial.pop();
        var mesIni = inicial.pop();
        var diaIni = inicial.pop();
        var anoFim = final.pop();
        var mesFim = final.pop();
        var diaFim = final.pop();
        dataInicial = parseInt(anoIni.concat(mesIni.concat(diaIni)));
        dataFinal = parseInt(anoFim.concat(mesFim.concat(diaFim)));
        if (dataInicial <= dataFinal){
            return true;
        }else{
            return false;
        }
    };

    this.validaCondicoesEmprestimo = function(emprestimo, parcelas, credito){
        var emprestimoValido = true;
        var totalAjusteEmp = emprestimo.currentRow.VRTOTALAJUSTEMP;
        totalAjusteEmp = (totalAjusteEmp.toString().search(',') != -1) ? parseFloat(totalAjusteEmp.toString().replace('.', '').replace(',', '.')) : parseFloat(totalAjusteEmp);
        var valorEmprestimo = emprestimo.currentRow.VRTOTALEMP;
        valorEmprestimo = (valorEmprestimo.toString().search(',') != -1) ? parseFloat(valorEmprestimo.toString().replace('.', '').replace(',', '.')) : parseFloat(valorEmprestimo);
        var totalEmprestimo = (valorEmprestimo + totalAjusteEmp).toFixed(2);
        var totalParcelas = 0;
        var totalAjuste = 0;
        parcelas.dataSource.data.forEach(function(data){
            totalParcelas = parseFloat(totalParcelas) + parseFloat(data.VRFINALBPAG);
            totalAjuste = parseFloat(totalAjuste) + parseFloat(data.VRAJUSTE);
        });
        totalParcelas = totalParcelas.toFixed(2);
        totalAjuste = totalAjuste.toFixed(2);
        if ((totalEmprestimo != totalParcelas) && (emprestimoValido)){
            emprestimoValido = false;
            ScreenService.showMessage("A soma das parcelas difere do valor total do empréstimo.");
        }
        if ((parseFloat(totalAjusteEmp).toFixed(2) != totalAjuste) && (emprestimoValido)){
            emprestimoValido = false;
            ScreenService.showMessage("A soma dos ajustes das contas difere do total de juros do empréstimo.");
        }
        if ((credito.currentRow.CREDITO == 'S') && (emprestimoValido)){
            if (credito.currentRow.DTMOVICRED){
                if (!this.validaDatas(emprestimo.currentRow.DTEMPRESTIMO, credito.currentRow.DTMOVICRED)){
                    emprestimoValido = false;
                    ScreenService.showMessage("Data do lançamento de crédito deve ser maior ou igual à data do empréstimo");
                }
            }
        }
        if ((credito.currentRow.DEBIOF == 'S') && (emprestimoValido)){
            if (credito.currentRow.DTMOVIDEBI){
                if (!this.validaDatas(emprestimo.currentRow.DTEMPRESTIMO, credito.currentRow.DTMOVIDEBI)){
                    emprestimoValido = false;
                    ScreenService.showMessage("Data do lançamento de débito deve ser maior ou igual à data do empréstimo");
                }
            }
        }
        if (credito.currentRow.DEBIOF && emprestimoValido){
            if (parseFloat(credito.currentRow.VRMOVIDEBI) > parseFloat(valorEmprestimo)){
                emprestimoValido = false;
                ScreenService.showMessage("Valor do débito deve ser menor que o valor do empréstimo.");
            }
        }
        if (emprestimoValido){
            emprestimoValido = this.validaDatacontratoMenorVencimento(emprestimo);
        }
        return emprestimoValido;
    };

    this.validaCondicoesReparcelamento = function(parcelas, reparcelamento){
        var reparcelamentoValido = true;
        var totalAjuste = 0;
        var totalConta = 0;
        var dataReparcelamento = reparcelamento.dataSource.data.filter(function(data){
            return data.__createdLocal || data.changed;
        });
        dataReparcelamento.forEach(function(parcela){
            var valorConta = parcela.VRATUAPAG;
            valorConta = (valorConta.toString().search(',') == -1) ? parseFloat(valorConta) : parseFloat(valorConta.toString().replace('.', '').replace(',', '.'));
            totalConta = (parseFloat(totalConta) + parseFloat(valorConta)).toFixed(2);
            valorAjuste = parcela.VRAJUSTE;
            valorAjuste = (valorAjuste.toString().search(',') == -1) ? parseFloat(valorAjuste) : parseFloat(valorAjuste.toString().replace('.', '').replace(',', '.'));
            totalAjuste = (parseFloat(totalAjuste) + parseFloat(valorAjuste)).toFixed(2);
        });

        if ((parseFloat(parcelas.currentRow.VRAJUSTE) != parseFloat(totalAjuste)) && reparcelamentoValido){
            reparcelamentoValido = false;
            ScreenService.showMessage("A soma dos ajustes dos boletos difere do total de juros da conta.");
        }
        var totalTitulo = parseFloat(parcelas.currentRow.VRATUAPAG);
        totalConta = parseFloat(totalConta);
        if (totalTitulo != totalConta){
            reparcelamentoValido = false;
            var diferencaValores =
                'Total das parcelas difere do valor total do título.<br/><br/>Valor Total Título: ' + totalTitulo + '<br/>'+
                'Valor Total Parcelas: ' + totalConta + '<br/>' + 'Diferença de Valor: ' + (totalTitulo - totalConta).toFixed(2) + '.';
            ScreenService.showMessage(diferencaValores);
        }

        return reparcelamentoValido;
    };

    this.deleteValoresInsercaoEmprestimo = function(){
        templateManager.container.getWidget('tabbed_emprestimo').currentRow = {};
        templateManager.container.getWidget('parcelasGrid').dataSource.data = [];
        templateManager.container.getWidget('credito').currentRow = {};
    };



    this.salvarReparcelamentoEmprestimo = function () {
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var parcelas = templateManager.container.getWidget('parcelas');
        var reparcelamento = templateManager.container.getWidget('reparcelamento');
        var route = 'fin_contaPagar#/saveReparcelamentoEmprestimoPagar/save';
        var dataSet = [];
        if (this.validaCondicoesReparcelamento(parcelas, reparcelamento)){
            dataSet = this.preparaDataSetReparcelamento(emprestimo, parcelas, reparcelamento);
            General.callRestEngine(route, 'DataSet', dataSet)
            .then(
                function () {
                    reparcelamento.NOVOREPARCELAMENTO = true;
                    ScreenService.closeSwipe().then(function(){
                        reparcelamento.reload();
                        parcelas.reload();
                        parcelas.activate();
                        General.changeWidgetAcessibility(reparcelamento.widgets, true);
                        ScreenService.showMessage('Parcelamento salvo com sucesso!');
                    }.bind(this));
                }
            )
            .catch(
                function (error) {
                    if (contabilizacaoController.checkErrosContabilizacao(error)) {
                        ScreenService.showMessage(error.data.error);
                    }
                }
            );
        }
    };

    this.preparaDataSetReparcelamento = function(emprestimo, parcelas, reparcelamento){
        var dataSet = [];
        dataSet.push(emprestimo.currentRow);
        reparcelamento.dataExclusao.forEach(function(data){
            data.CDAJUCTPAG = emprestimo.currentRow.CDAJUCTPAG;
            data.CDEMPRESA = emprestimo.currentRow.CDEMPRESA;
            data.CDFIMOVFLPAG = parcelas.currentRow.CDFIMOVFLPAG;
            data.CDFILIAL = parcelas.currentRow.CDFILIAL;
            data.CDCLASSFINA = parcelas.currentRow.CDCLASSFINA;
            data.NRLANCTONF = parcelas.currentRow.NRLANCTONF;
        });
        dataSet.push(reparcelamento.dataExclusao);
        var dataInclusao = reparcelamento.dataSource.data.filter(function(parcela){
            return parcela.__createdLocal || parcela.changed;
        });
        dataInclusao.forEach(function(data){
            data.CDAJUCTPAG = emprestimo.currentRow.CDAJUCTPAG;
            data.CDEMPRESA = emprestimo.currentRow.CDEMPRESA;
            data.CDFIMOVFLPAG = parcelas.currentRow.CDFIMOVFLPAG;
            data.CDFILIAL = parcelas.currentRow.CDFILIAL;
            data.CDCLASSFINA = parcelas.currentRow.CDCLASSFINA;
            data.NRLANCTONF = parcelas.currentRow.NRLANCTONF;
        });
        dataSet.push(dataInclusao);
        return dataSet;
    };

    /*this.deleteEmprestimo = function (widget) {
        var route = 'fin_contaPagar#/deleteEmprestimoPagar';
        var dataSet = [];
        ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir Empréstimo?'), 'question', function () {
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function () {
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                        ScreenService.showMessage('Registro excluído com sucesso!');
                    });
                })
                .catch(function (response) {
                    ScreenService.showMessage(response.data.error);
                });
        });
    };*/

    this.deleteDocumentoEmprestimoPagar = function (widget) {
        var route = 'fin_contaPagar#/deleteDocumentoEmprestimoPagar';
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var parcelas = templateManager.container.getWidget('parcelas');
        var credito = templateManager.container.getWidget('lancamentoIof');
        var rowCredito = {};
        var rowDebIof = {};
        rowCredito = this.preparaRowCredito(credito.currentRow, rowCredito);
        rowDebIof = this.preparaRowDebito(credito.currentRow, rowDebIof);
        var dataSet = [];
        dataSet.push(emprestimo.currentRow);
        dataSet.push(parcelas.dataSource.data);
        dataSet.push(rowCredito);
        dataSet.push(rowDebIof);
        if (this.validaStatusEmprestimo(emprestimo)){
            this.emprestimoRealizadoNoPeriodoContabil(emprestimo).then(function(){
                ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir o Empréstimo?'), 'question', function () {
                    General.callRestEngine(route, 'DataSet', dataSet)
                        .then(function () {
                            ScreenService.closeSwipe().then(function(){
                                widget.reload();
                                ScreenService.showMessage('Registro excluído com sucesso!');
                            });
                        })
                        .catch(function (response) {
                            ScreenService.showMessage(response.data.error);
                        });
                });
            }.bind(this))
            .catch(function (error) {
                ScreenService.showMessage(error);
            });

        }
    };

    this.validaStatusEmprestimo = function($emprestimo){
        var statusValido = true;
        if ($emprestimo.currentRow.IDBPAGREAL == 'S'){
            statusValido = false;
            ScreenService.showMessage('Conta relacionada a um boleto real não pode ser alterada');
        }
        return statusValido;
    };

    this.deleteParcelaEmprestimoPagar = function () {
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var parcelas = templateManager.container.getWidget('parcelas');
        var route = 'fin_contaPagar#/deleteParcelaEmprestimoPagar';
        var routeGrupoOperadores = 'fin_contaPagar#/verificaGrupoOperadores';
        this.emprestimoRealizadoNoPeriodoContabil(emprestimo).then(function(){
            if (this.validaCondicoesExclusaoParcelas(parcelas)){
                var dataDelete = angular.copy(parcelas.getCheckedRows());
                dataDelete.forEach(function(data){
                    data.CDAJUCTPAG = emprestimo.currentRow.CDAJUCTPAG;
                });
                General.callRestEngine(routeGrupoOperadores, 'Row', {}).then(function(grupoOper){
                    if (grupoOper.dataset.STATUS[0]){
                        this.validaStatusPagamento(dataDelete.IDSTATPAG);
                    }else{
                        ScreenService.confirmMessage(ScreenService.i18n('Confirmar exclusão da(s) parcela(s)?'), 'question', function () {
                            General.callRestEngine(route, 'DataSet', dataDelete)
                            .then(function () {
                                parcelas.reload().then(function(){
                                    var totalParcelas = 0;
                                    parcelas.dataSource.data.forEach(function(parcela){
                                        totalParcelas = parseFloat(totalParcelas) + parseFloat(parcela.VRATUAPAG);
                                    });
                                    emprestimo.currentRow.VRTOTALEMP = (totalParcelas).toFixed(2);
                                    //emprestimo.reload([], false);
                                }.bind(this));
                                ScreenService.showMessage('Registro excluído com sucesso!');
                            }.bind(this))
                            .catch(function (response) {
                                ScreenService.showMessage(response.data.error);
                            });
                        });
                    }
                }.bind(this));
            }
        }.bind(this))
        .catch(function (error) {
            ScreenService.showMessage(error);
        });
    };

    this.validaCondicoesExclusaoParcelas = function(parcelas){
        var condicaoValida = true;
        var qtdeRowsMarcadas = parcelas.getCheckedRows().length;
        var qtdeRows = parcelas.dataSource.data.length;
        if (qtdeRows == 1){
            condicaoValida = false;
            ScreenService.showMessage('Operação bloqueada! Empréstimo possui apenas um vencimento.');
        }
        if ((qtdeRows == qtdeRowsMarcadas) && (condicaoValida)){
            condicaoValida = false;
            ScreenService.showMessage('Operação bloqueada! Não é possível excluir todos os vencimentos.');
        }
        return condicaoValida;
    };

    this.validaStatusPagamento = function(idstatpag){
        if ((idstatpag == 'A') || (idstatpag == 'E')){
            ScreenService.showMessage('Operação bloqueada! Operador não possui permissão para excluir conta do tipo real.');
        }else{
            if (idstatpag == 'V'){
                ScreenService.showMessage('Operação bloqueada! Operador não possui permissão para excluir conta do tipo previsto.');
            }else{
                ScreenService.showMessage('Operação bloqueada! Operador não possui permissão excluir o tipo de conta.');
            }
        }
    };

    this.adicionaEmprestimo = function(){
        var tabbed = templateManager.container.getWidget('tabbed_emprestimo');
        var emprestimo = templateManager.container.getWidget('emprestimo');
        var credito = templateManager.container.getWidget('credito');
        var parcelasGrid = templateManager.container.getWidget('parcelasGrid');
        credito.getField('DSCONTCORR').dataSourceFilter[0].value = emprestimo.appliedFilter.FILTER_ROW.CDEMPRESA;
        credito.getField('DSCONTCORRCRED').dataSourceFilter[0].value = emprestimo.appliedFilter.FILTER_ROW.CDEMPRESA;
        ScreenService.openSwipe(tabbed).then(function(){
            tabbed.isVisible = true;
            //parcelasGrid.edit();
            credito.edit();
            tabbed.currentRow.CDEMPRESA = emprestimo.appliedFilter.FILTER_ROW.CDEMPRESA;
            tabbed.getField('DTEMPRESTIMO').setValue(moment().format('DD/MM/YYYY'));
            tabbed.getField('DTPRIVENCEMP').setValue(moment().format('DD/MM/YYYY'));
            tabbed.getField('IDMOVFLUXO').setValue('E');
            tabbed.getField('IDTIPOEMP').setValue('O');
            tabbed.getField('NMFILIAL').dataSourceFilter[0].value = emprestimo.appliedFilter.FILTER_ROW.CDEMPRESA;
            tabbed.getField('VRTOTALAJUSTEMP').setValue(0);
            tabbed.getField('VRTXJUROSEMP').setValue(0);
            tabbed.getField('IDTIPOEMP').setValue('O');
            credito.getField('DTMOVICRED').setValue(moment().format('DD/MM/YYYY'));
            credito.getField('DTMOVIDEBI').setValue(moment().format('DD/MM/YYYY'));
            credito.getField('DEBIOF').setValue('S');
            credito.getField('CREDITO').setValue('S');
            this.validaDisponibilidadeDebCredIOF(credito, 'CREDITO');
            this.validaDisponibilidadeDebCredIOF(credito, 'DEBIOF');
            tabbed.redraw();
        }.bind(this));
    };

    this.cancelEmprestimo = function(){
        var tabbed = templateManager.container.getWidget('tabbed_emprestimo');
        ScreenService.closeSwipe().then(function(){
            this.deleteValoresInsercaoEmprestimo();
            tabbed.isVisible = false;
            tabbed.getParent().activate();
            tabbed.widgets[0].dataSource.data = [];
        }.bind(this));
    };

    this.aplicaFiltroParcelas = function(emprestimo){
        var parcelas = templateManager.container.getWidget('parcelas');
        var lancamentoIof = templateManager.container.getWidget('lancamentoIof');
        parcelas.dataSourceFilter[0].value = emprestimo.currentRow.CDEMPRESA;
        parcelas.dataSourceFilter[1].value = emprestimo.currentRow.NRSEQUENCIALEMP;
        parcelas.dataSourceFilter[2].value = emprestimo.currentRow.CDFILIAL;
        parcelas.reload();
        lancamentoIof.dataSourceFilter[0].value = emprestimo.currentRow.CDEMPRESA;
        lancamentoIof.dataSourceFilter[1].value = emprestimo.currentRow.NRSEQUENCIALEMP;
        lancamentoIof.dataSourceFilter[2].value = emprestimo.currentRow.CDFILIPROC;
        lancamentoIof.dataSource.data = [];
        lancamentoIof.reload([], false).then(function(){
            this.setLancamentoCreditoDebitoIof(lancamentoIof);
        }.bind(this));
    };

    this.setLancamentoCreditoDebitoIof = function(lancamentoIof){
        var row = {};
        if (lancamentoIof.dataSource.data.length > 0){
            if (lancamentoIof.dataSource.data.length > 1){
                lancamentoIof.dataSource.data[0].DEBIOF = 'S';
                lancamentoIof.dataSource.data[1].CREDITO = 'S';
                row = this.preparaRowDebito(lancamentoIof.dataSource.data[0], row);
                row = this.preparaRowCredito(lancamentoIof.dataSource.data[1], row);
            }else{
                if (lancamentoIof.dataSource.data[0].VRMOVIDEBI){
                    lancamentoIof.dataSource.data[0].DEBIOF = 'S';
                    lancamentoIof.dataSource.data[0].CREDITO = 'N';
                    row = this.preparaRowDebito(lancamentoIof.dataSource.data[0], row);
                }else{
                    lancamentoIof.dataSource.data[0].DEBIOF = 'N';
                    lancamentoIof.dataSource.data[0].CREDITO = 'S';
                    row = this.preparaRowCredito(lancamentoIof.dataSource.data[0], row);
                }
            }
        }else{
            row.DEBIOF = 'N';
            row.CREDITO = 'N';
            row.NRLCTBMCRED = null;
            row.NRLCTBMDEB = null;
        }
        lancamentoIof.currentRow = row;
    };

    this.preparaRowCredito = function(currentRow, row){
        row.CREDITO = currentRow.CREDITO;
        row.CDCONTCTBLCRED = currentRow.CDCONTCTBLCRED;
        row.CDREDUCONTCRED = currentRow.CDREDUCONTCRED;
        row.DTMOVICRED = currentRow.DTMOVICRED;
        row.NRLCTBMCRED = currentRow.NRLCTBMCRED;
        row.DSCONTCORRCRED = currentRow.DSCONTCORRCRED;
        row.CDBANCOCRED = currentRow.CDBANCOCRED;
        row.CDAGENCIACRED = currentRow.CDAGENCIACRED;
        row.CDCONTCORRCRED = currentRow.CDCONTCORRCRED;
        row.NRDOCMBANCCRED = currentRow.NRDOCMBANCCRED;
        row.DSMOVICRED = currentRow.DSMOVICRED;
        row.DSTPCREDBANC = currentRow.DSTPCREDBANC;
        row.VRMOVICRED = currentRow.VRMOVICRED;
        row.NRSEQMOVREBACRED = currentRow.NRSEQMOVREBACRED;
        row.CDSEQUEXBACRED = currentRow.CDSEQUEXBACRED;
        row.NRMOVBANCCRED = currentRow.NRMOVBANCCRED;
        return row;
    };

    this.preparaRowDebito = function(currentRow, row){
        row.DEBIOF = currentRow.DEBIOF;
        row.CDREDUCONT = currentRow.CDREDUCONT;
        row.CDCONTCTBL = currentRow.CDCONTCTBL;
        row.DTMOVIDEBI = currentRow.DTMOVIDEBI;
        row.NRLCTBMDEB = currentRow.NRLCTBMDEB;
        row.DSCONTCORR = currentRow.DSCONTCORR;
        row.CDBANCO = currentRow.CDBANCO;
        row.CDAGENCIA = currentRow.CDAGENCIA;
        row.CDCONTCORR = currentRow.CDCONTCORR;
        row.NRDOCMBANC = currentRow.NRDOCMBANC;
        row.DSMOVIDEBI = currentRow.DSMOVIDEBI;
        row.DSTPDEBIBANC = currentRow.DSTPDEBIBANC;
        row.VRMOVIDEBI = currentRow.VRMOVIDEBI;
        row.NRSEQMOVREBA = currentRow.NRSEQMOVREBA;
        row.CDSEQUEXBA = currentRow.CDSEQUEXBA;
        row.NRMOVBANC = currentRow.NRMOVBANC;
        return row;
    };

    self.formataValoresDecimais = function(valor){
        var valorFormatado = 0;
        valorFormatado = (valor.toString().search(',') != '-1') ?
            ((parseFloat(valor.toString().split(',')[0]) > 0) ? parseFloat(valor.toString().replace('.', '').replace(',', '.')) : parseFloat(valor.toString().replace(',', '.'))) :
            parseFloat(valor);
        return valorFormatado;
    };

    this.setaMascCnpjCpfFornecedor = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURFORN.length === 14) inscFormatada = value.NRINSJURFORN.substr(0, 2) + '.' + value.NRINSJURFORN.substr(2, 3) + '.' + value.NRINSJURFORN.substr(5, 3) + '/' + value.NRINSJURFORN.substr(8, 4) + '-' + value.NRINSJURFORN.substr(12, 2);
          else if(value.NRINSJURFORN.length === 11) inscFormatada = value.NRINSJURFORN.substr(0, 3) + '.' + value.NRINSJURFORN.substr(3, 3) + '.' + value.NRINSJURFORN.substr(6, 3) + '-' + value.NRINSJURFORN.substr(9, 2);
          else inscFormatada = null;
          value.NRINSJURFORN = inscFormatada?inscFormatada:value.NRINSJURFORN;
	    });
	};
	
	this.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctpagar = widget.getField('CDTPCTPAGAR');
        
        cdtpctpagar.setValue("");
        cdtpctpagar.dataSourceFilter[1].value = cdempresa;
        cdtpctpagar.reload();
    };
	
	self.editarEmprestimoLeasing = function(){
	    var emprestimoWidget = templateManager.container.getWidget('emprestimo');
	    var parcelasWidget = templateManager.container.getWidget('parcelas');
        var creditoWidget = templateManager.container.getWidget('lancamentoIof');
        
        parcelasWidget.enabled = false;
        creditoWidget.enabled = false;
        General.edit(emprestimoWidget);
	};
	
	self.salvarEditEmprestimoLeasing = function(){
	    var parcelasWidget = templateManager.container.getWidget('parcelas');
        var creditoWidget = templateManager.container.getWidget('lancamentoIof');
        
        parcelasWidget.enabled = true;
        creditoWidget.enabled = true;
	};
	
	self.CancelarEditEmprestimoLeasing = function(){
	    var parcelasWidget = templateManager.container.getWidget('parcelas');
        var creditoWidget = templateManager.container.getWidget('lancamentoIof');
        
        parcelasWidget.enabled = true;
        creditoWidget.enabled = true;
	};
	
	self.VoltarButtonParcelasAndCredito = function(){
	    var emprestimoWidget = templateManager.container.getWidget('emprestimo');
	    
	    ScreenService.goBack();
        emprestimoWidget.activate();
	};
	
	self.setStripe = function(){
	   var emprestimoWidget = templateManager.container.getWidget('emprestimo');
	   var stripeWidget = templateManager.container.getWidget('stripe');
	   
	   stripeWidget.currentRow.EMPRESA  = emprestimoWidget.appliedFilter.FILTER_ROW.NMEMPRESA ;
	   
	};

};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag16000_EmprestimoLeasing_Controller', Pag16000_EmprestimoLeasing_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag18000_CompensacaoContaPagar_Controller.js
var Pag18000_CompensacaoContaPagar_Controller = function(ScreenService, templateManager, General, ZHPromise, ZhFilterPreferences) {

    var wasFiltered = false;
    /*
     * Gets e Sets
     */
     
    this.openFilterCompensacao = function(compensacaoContasPagar){
        var data = moment().format('DD/MM/YYYY');
        var rowFilter = {'DTLANCCOMPPAG': {start : data, end : data}};
        ZhFilterPreferences.setOriginalFilter(compensacaoContasPagar.id, rowFilter);
        ZhFilterPreferences.openFilterWidget(compensacaoContasPagar, rowFilter);
    };

    this.validaDatasBaixaCompensacao = function(widget, fieldClear){
        var arrayBaixa = widget.currentRow.DTBAIXACOMP.split('/');
        var arrayComp = widget.currentRow.DTCOMPENCTPAG.split('/');
        var anoBaixa = arrayBaixa.pop();
        var mesBaixa = arrayBaixa.pop();
        var diaBaixa = arrayBaixa.pop();
        var anoComp = arrayComp.pop();
        var mesComp = arrayComp.pop();
        var diaComp = arrayComp.pop();
        var dataBaixa = parseInt(anoBaixa.concat(mesBaixa.concat(diaBaixa)));
        var dataCompensacao = parseInt(anoComp.concat(mesComp.concat(diaComp)));
        if (dataBaixa <= dataCompensacao){
            ScreenService.showMessage('Data da baixa deve ser igual ou maior à data da compensação.');
            fieldClear.setValue('');
        }
    };

    this.adicionaCompensacao = function(widget){
        this.novaCompensacao(widget).then(function(){
            this.validaItemCompensacao(widget);
        }.bind(this));
    };

    this.novaCompensacao = function(widget){
        var deferred = ZHPromise.defer();
        General.widgetNewRow(widget);
        widget.getField('IDTPITUTLCOMPAG').setValue('0');
        widget.getField('CONTARECEBER').isVisible = true;
        widget.getField('CONTACORRENTE').isVisible = true;
        widget.getField('CONTAPAGAR').isVisible = true;
        widget.getField('VRTOTALCOMPPAG').class = '3';
        widget.getField('SOMACONTACORRENTE').class = '3';
        templateManager.container.getWidget('TitulosCompensadosReceber').isVisible = false;
        templateManager.container.getWidget('ContasCorrentesCompensadas').isVisible = false;
        deferred.resolve();
        return deferred.promise;
    };

    this.cancelaCompensacao = function(widget){
        General.widgetCancelForm(widget);
        widget.getField('CONTARECEBER').isVisible = false;
        widget.getField('CONTACORRENTE').isVisible = false;
        widget.getField('CONTAPAGAR').isVisible = false;
        widget.getField('VRTOTALCOMPPAG').class = '12';
        widget.getField('SOMACONTACORRENTE').class = '12';
        templateManager.container.getWidget('TitulosCompensadosReceber').isVisible = true;
        templateManager.container.getWidget('ContasCorrentesCompensadas').isVisible = true;
    };

    this.getEmpresa = function () {
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.CDEMPRESA;
    };

    this.getFilial = function () {
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.CDFILIAL;
    };

    this.getNrorg = function () {
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.NRORG;
    };

    /*
     * Validações de FrontEnd
     */

    this.aplicaFiltroCompensacaoPagar = function () {
        var compensacao = templateManager.container.getWidget('CompensacaoContasPagar');
        var filtro = compensacao.appliedFilter;
        var operador = '';
        var cdforneced = '';
        if (filtro.FILTER_ROW.hasOwnProperty('DTLANCCOMPPAG')){
            if (filtro.FILTER_ROW.DTLANCCOMPPAG.hasOwnProperty('start')){
                var lancIni = filtro.FILTER_ROW.DTLANCCOMPPAG.start;
                var lancFin = filtro.FILTER_ROW.DTLANCCOMPPAG.end;
            }
        }
        if (filtro.FILTER_ROW.hasOwnProperty('DTCOMPENCTPAG')){
            if (filtro.FILTER_ROW.DTCOMPENCTPAG.hasOwnProperty('start')){
                var compIni = filtro.FILTER_ROW.DTCOMPENCTPAG.start;
                var compFin = filtro.FILTER_ROW.DTCOMPENCTPAG.end;
            }
        }
        var operadorMulti = filtro.FILTER_ROW.CDOPERADOR;
        var fornecedorMulti = filtro.FILTER_ROW.CDFORNECED;
        if (operadorMulti) {
            operadorMulti.forEach(function (filter) {
                operador += "_" + filter;
            });
            operador += "_";
        }
        if (fornecedorMulti) {
            fornecedorMulti.forEach(function (filter) {
                cdforneced += "_" + filter;
            });
            cdforneced += "_";
        }
        compensacao.dataSourceFilter.forEach(function (data) {
            switch (data.name) {
                case 'CDEMPRESA':
                    data.value = filtro.FILTER_ROW.CDEMPRESA;
                    break;
                case 'CDOPERADOR':
                    data.value = operador;
                    break;
                case 'TCDOPERADOR':
                    data.value = (operador) ? null : 'T';
                    break;
                case 'CDFORNECED':
                    data.value = cdforneced;
                    break;
                case 'TCDFORNECED':
                    data.value = (cdforneced) ? null : 'T';
                    break;
                case 'DTCOMPENSACAO_INI': data.value = compIni;
                    break;
                case 'DTCOMPENSACAO_FIN': data.value = compFin;
                    break;
                case 'TCOMPENSACAO': data.value = (compIni && compFin) ? null : 'T';
                    break;
                case 'DTLANCAMENTO_INI': data.value = lancIni;
                    break;
                case 'DTLANCAMENTO_FIN': data.value = lancFin;
                    break;
                case 'TLANCAMENTO': data.value = (lancIni && lancFin) ? null : 'T';
                    break;
            }
        });
    };


    this.validaCamposAtivos = function (args) {
        var field = args.owner.field.name;
        var row = args.owner.field.widget.currentRow;

        switch (field) {
            case 'NMEMPRESA':
                args.owner.field.widget.getField('NMRAZSOCFORN').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('NMTPBAIXAREC').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('DSAJUCTREC').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('CONTACORRENTE').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('CONTARECEBER').readOnly = !row.NMEMPRESA;
                break;
            case 'NMRAZSOCFORN':
                args.owner.field.widget.getField('CONTAPAGAR').readOnly = !row.NMRAZSOCFORN;
                if ((row.CDFORNECED != args.owner.field.widget.FORNECEDOR) && (row.CDFORNECED)){
                    row.CONTAPAGAR = null;
                    row.DTVENBOLPAG = null;
                    row.DOCUMENTO = null;
                    row.NRINSJURFORN = null;
                    row.NRSEQVENPAG = null;
                    row.NRPARCELAPAG = null;
                    row.VRORIGPAG = null;
                    row.VRFINALBPAG = null;
                }
                args.owner.field.widget.getField('CONTAPAGAR').readOnly = !row.NMRAZSOCFORN;
                break;
            case 'CDREDUCONT':
                args.owner.field.widget.getField('CONTACORRENTE').readOnly = !row.CDREDUCONT;
                break;
            default:
                var aux = 'teste';
        }
    };

    this.setValueAntFornecedor = function (widget){
        widget.FORNECEDOR = widget.currentRow.CDFORNECED;
    };

    this.validaItemCompensacao = function (widget) {
        if (widget.currentRow.IDTPITUTLCOMPAG == '0') {
            widget.fieldGroups.forEach(function (group) {
                if (group.name == 'contaReceber') {
                    group.isVisible = true;
                    widget.getField('CONTACORRENTE').validations = null;
                    widget.getField('CONTARECEBER').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('NMTPBAIXAREC').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('DSAJUCTREC').validations = {'required': {'message': 'Campo obrigatório'}};
                }
                if (group.name == 'creditoContaCorrente') {
                    group.isVisible = false;
                }
            });
        } else {
            widget.fieldGroups.forEach(function (group) {
                if (group.name == 'contaReceber') {
                    group.isVisible = false;
                }
                if (group.name == 'creditoContaCorrente') {
                    group.isVisible = true;
                    widget.getField('CONTACORRENTE').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('CONTARECEBER').validations = null;
                    widget.getField('NMTPBAIXAREC').validations = null;
                    widget.getField('DSAJUCTREC').validations = null;
                }
            });
        }
    };

    this.validaTipoCompensacao = function(widget){
        var receberCompensados = templateManager.container.getWidget('TitulosCompensadosReceber');
        var contasCompensadas = templateManager.container.getWidget('ContasCorrentesCompensadas');
        if (widget.currentRow.IDTPITUTLCOMPAG == '0'){
            receberCompensados.isVisible = true;
            receberCompensados.enabled = true;
            contasCompensadas.isVisible = false;
            this.setFiltroTitulosRecCompensados(widget, receberCompensados).then(function(){
                receberCompensados.reload();
            });
        }else{
            contasCompensadas.isVisible = true;
            contasCompensadas.enabled = true;
            receberCompensados.isVisible = false;
            this.setFiltroContasCorrentesCompensadas(widget, contasCompensadas).then(function(){
                contasCompensadas.reload();
            });
        }
    };

    this.setFiltroTitulosRecCompensados = function (widget, receberCompensados) {
        var deferred = ZHPromise.defer();
        receberCompensados.dataSourceFilter.forEach(function (data) {
            switch(data.name) {
                case 'NRLANCCOMPPAG': data.value = widget.currentRow.NRLANCCOMPPAG;
                    break;
                case 'CDEMPRESA': data.value = widget.currentRow.CDEMPRESA;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.setFiltroContasCorrentesCompensadas = function (widget, contasCompensadas) {
        var deferred = ZHPromise.defer();
        contasCompensadas.dataSourceFilter.forEach(function (data) {
            switch(data.name) {
                case 'NRLANCCOMPPAG': data.value = widget.currentRow.NRLANCCOMPPAG;
                    break;
                case 'CDREDUCONT': data.value = widget.currentRow.CDREDUCONT;
                    break;
                case 'NRORG': data.value = widget.currentRow.NRORG;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };
    this.validaFieldLancContaCorrente = function (args) {
        /*var field = args.owner.field.name;

         switch(field) {
         case 'CDREDUCONT':
         var row = args.owner.field.widget.currentRow;
         args.owner.field.widget.getField('CONTACORRENTE').readOnly = !row.CDREDUCONT;
         break;
         default:
         var aux = 'teste';
         }*/
    };

    this.validaValorTotalPagar = function (origin) {
        /*try{
         var widget = templateManager.container.getWidget('CompensacaoContasReceber');

         var somaTotal = 0;
         var VRORIGPAG = widget.currentRow.CP_VRORIGPAG;

         if (VRORIGPAG) {
         for (var i=0; i<VRORIGPAG.length; i++) {
         somaTotal = somaTotal + VRORIGPAG[i];
         }
         };

         widget.currentRow.VRTOTALCOMREC = somaTotal;

         if(somaTotal > widget.currentRow.VRORIGREC){
         if(origin){
         ScreenService.showMessage("Valor total a compensar deve ser menor ou igual ao valor líquido da conta a receber.");
         }
         return false;
         }else{
         return true;
         }

         }catch(e){
         ScreenService.showMessage("Valor total a compensar deve ser menor ou igual ao valor líquido da conta informada.");
         }*/
    };

    this.getDefaulInterval = function () {
        var obj = {
            "start": moment().startOf('month').format('DD/MM/YYYY'),
            "end": moment().endOf('month').format('DD/MM/YYYY')
        };
        return obj;
    };

    /*
     * Fim Validações de FrontEnd
     */

    this.deleteCompensacao = function (widget) {
        var route = 'fin_contaPagar#/deleteCompensacaoPagar/delete';
        var dataSet = [];
        dataSet = this.preparaRowDeleteCompensacao(widget, dataSet);
        if (widget.currentRow.IDTPITUTLCOMPAG == '0'){
            var widgetTitulos = templateManager.container.getWidget('TitulosCompensadosReceber');
            this.preparaRowDeleteTitulos(widgetTitulos, dataSet);
        }else{
            var widgetContas = templateManager.container.getWidget('ContasCorrentesCompensadas');
            this.preparaRowDeleteContas(widgetContas, dataSet);
        }
        ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir Compensação?'), 'question', function () {
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function () {
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                        ScreenService.showMessage('Registro excluído com sucesso!');
                    });
                })
                .catch(function (response) {
                    ScreenService.showMessage(response.data.error);
                });
        });
    };

    this.preparaRowDeleteTitulos = function(widget, dataSet){
        widget.dataSource.data.forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = {};
            dataSet[indice].CDFILIAL = data.CDFILIAL;
            dataSet[indice].NRLANBOLREC = data.NRLANBOLREC;
            dataSet[indice].CDFILIBOLREC = data.CDFILIBOLREC;
            dataSet[indice].VRFINALBREC = data.VRFINALBREC;
            dataSet[indice].CDEMPRESA = data.CDEMPRESA;
            dataSet[indice].CDFIMOVFLREC = data.CDFIMOVFLREC;
            dataSet[indice].CDCLASSFINA = data.CDCLASSFINA;
            dataSet[indice].CDFILIAL = data.CDFILIAL;
            dataSet[indice].NRLANCTONF = data.NRLANCTONF;
            dataSet[indice].NRSEQVENREC = data.NRSEQVENREC;
            dataSet[indice].NRPARCELAREC = data.NRPARCELAREC;
            dataSet[indice].VRORIGREC = data.VRORIGREC;
            dataSet[indice].VRITEMCOMPPAG = data.VRITEMCOMPPAG;
            dataSet[indice].DTMOVFLUXO = data.DTMOVFLUXO;
        });
        return dataSet;
    };

    this.preparaRowDeleteContas = function(widget, dataSet){
        widget.dataSource.data.forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = {};
            dataSet[indice].CDREDUCONT = data.CDREDUCONT;
            dataSet[indice].NRMOVIDEBI = data.NRMOVIDEBI;
        });
        return dataSet;
    };

    this.preparaRowDeleteCompensacao = function(widget, dataSet){
        var indice = dataSet.length;
        dataSet[indice] = {};
        dataSet[indice].VRFINALBPAG = widget.currentRow.VRFINALBPAG;
        dataSet[indice].CDFILIBPAG = widget.currentRow.CDFILIBPAG;
        dataSet[indice].NRLANBOLPAG = widget.currentRow.NRLANBOLPAG;
        dataSet[indice].VRTOTALCOMPPAG = widget.currentRow.VRTOTALCOMPPAG;
        dataSet[indice].CDAJUCTPAG = widget.currentRow.CDAJUCTPAG;
        dataSet[indice].CDAJUCTREC = widget.currentRow.CDAJUCTREC;
        dataSet[indice].CDFIMOVFLPAG = widget.currentRow.CDFIMOVFLPAG;
        dataSet[indice].CDCLASSFINA = widget.currentRow.CDCLASSFINA;
        dataSet[indice].CDFILIAL = widget.currentRow.CDFILIAL;
        dataSet[indice].NRLANCTONF = widget.currentRow.NRLANCTONF;
        dataSet[indice].NRSEQVENPAG = widget.currentRow.NRSEQVENPAG;
        dataSet[indice].NRPARCELAPAG = widget.currentRow.NRPARCELAPAG;
        dataSet[indice].VRORIGPAG = widget.currentRow.VRORIGPAG;
        dataSet[indice].DTMOVFLUXO = widget.currentRow.DTMOVFLUXO;
        dataSet[indice].NRORG = widget.currentRow.NRORG;
        dataSet[indice].DTBAIXACOMP = widget.currentRow.DTBAIXACOMP;
        dataSet[indice].NRBAIXAPAG = widget.currentRow.NRBAIXAPAG;
        dataSet[indice].FILIPROC = widget.currentRow.FILIPROC;
        dataSet[indice].CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
        dataSet[indice].IDTPITUTLCOMPAG = widget.currentRow.IDTPITUTLCOMPAG;
        dataSet[indice].NRLANCCOMPPAG = widget.currentRow.NRLANCCOMPPAG;
        dataSet[indice].NRLCTBBPAG = widget.currentRow.NRLCTBBPAG;
        dataSet[indice].CDFILIBAIXAREC = widget.currentRow.CDFILIBAIXAREC;
        dataSet[indice].NRBAIXAREC = widget.currentRow.NRBAIXAREC;
        dataSet[indice].NRLCTBBREC = widget.currentRow.NRLCTBBREC;
        dataSet[indice].CDEMPRESA = widget.currentRow.CDEMPRESA;
        return dataSet;
    };

    this.saveCompensacao = function (widget) {
        var route = 'fin_contaPagar#/saveCompensacaoPagar/save';
        var dataSet = [];
        var valorCompensar = 0;

        if (widget.isValid())
        {
            dataSet = this.preparaRowCompensacaoPagar(widget, dataSet);
            var valorContaPagar = parseFloat(widget.currentRow.VRFINALBPAG);
            //Verifica se compensa com lançamentos em conta ou contas a receber.
            if (widget.currentRow.IDTPITUTLCOMPAG == '0') {
                valorCompensar = parseFloat(widget.currentRow.VRTOTALCOMPPAG);
                dataSet = this.preparaRowContasReceber(widget, dataSet);
            } else {
                valorCompensar = parseFloat(widget.currentRow.SOMACONTACORRENTE);
                dataSet = this.preparaRowLancamentoReceber(widget, dataSet);
            }
            if (valorCompensar != '0'){
                if (valorCompensar <= valorContaPagar){
                    General.callRestEngine(route, 'DataSet', dataSet)
                    .then(
                        function () {
                            widget.getField('CONTARECEBER').isVisible = false;
                            widget.getField('CONTAPAGAR').isVisible = false;
                            widget.getField('CONTACORRENTE').isVisible = false;
                            widget.getField('VRTOTALCOMPPAG').class = '12';
                            widget.getField('SOMACONTACORRENTE').class = '12';
                            ScreenService.closeSwipe().then(function(){
                                widget.reload();
                                ScreenService.showMessage('Registro salvo com sucesso!');
                            });
                        }
                    )
                    .catch(
                        function(erro){
                            ScreenService.showMessage(erro.data.error);
                        }
                    );
                }else{
                    ScreenService.showMessage("Valor total a compensar deve ser menor ou igual ao valor líquido da conta informada.");
                }
            }else{
                ScreenService.showMessage("Valor total a compensar deve ser maior que zero.");
            }
        }
    };

    this.preparaRowCompensacaoPagar = function (widget, dataSetPagar) {
        var empresa = widget.currentRow.CDEMPRESA;
        dataSetPagar[0] = {};
        dataSetPagar[0].__is_new = widget.currentRow.__is_new;
        dataSetPagar[0].changed = widget.currentRow.hasOwnProperty('changed');
        dataSetPagar[0].DOCUMENTO = widget.currentRow.DOCUMENTO;
        dataSetPagar[0].CDFORNECED = widget.currentRow.CDFORNECED;
        dataSetPagar[0].NRINSFURFORN = widget.currentRow.NRINSFURFORN;
        dataSetPagar[0].NRSEQVENPAG = widget.currentRow.NRSEQVENPAG;
        dataSetPagar[0].NRPARCELAPAG = widget.currentRow.NRPARCELAPAG;
        dataSetPagar[0].VRORIGINAL = widget.currentRow.VRORIGINAL;
        dataSetPagar[0].VRLIQUIDO = widget.currentRow.VRLIQUIDO;
        dataSetPagar[0].VRORIGPAG = widget.currentRow.VRORIGPAG;
        dataSetPagar[0].VRFINALBPAG = widget.currentRow.VRFINALBPAG;
        dataSetPagar[0].CDFIMOVFLPAG = widget.currentRow.CDFIMOVFLPAG;
        dataSetPagar[0].CDCLASSFINA = widget.currentRow.CDCLASSFINA;
        dataSetPagar[0].DTMOVFLUXO = widget.currentRow.DTMOVFLUXO;
        dataSetPagar[0].CDEMPRESA = empresa;
        dataSetPagar[0].NMFILIAL = widget.currentRow.NMFILIAL;
        dataSetPagar[0].CDFILIBPAG = widget.currentRow.CDFILIBPAG;
        dataSetPagar[0].NRLANBOLPAG = widget.currentRow.NRLANBOLPAG;
        dataSetPagar[0].IDTPINURFORN = widget.currentRow.IDTPINURFORN;
        dataSetPagar[0].CDFILIAL = widget.currentRow.CDFILIAL;
        dataSetPagar[0].NRLANCTONF = widget.currentRow.NRLANCTONF;
        dataSetPagar[0].CDTPBAIXAREC = widget.currentRow.CDTPBAIXAREC;
        dataSetPagar[0].DSBAIXAREC = widget.currentRow.NMTPBAIXAREC;
        dataSetPagar[0].CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
        dataSetPagar[0].CDAJUCTPAG = widget.currentRow.CDAJUCTPAG;
        dataSetPagar[0].CDREDUCONT = widget.currentRow.CDREDUCONT;
        dataSetPagar[0].DTBAIXACOMP = widget.currentRow.DTBAIXACOMP;
        dataSetPagar[0].IDTPITUTLCOMPAG = widget.currentRow.IDTPITUTLCOMPAG;
        dataSetPagar[0].CDAJUCTREC = widget.currentRow.CDAJUCTREC;
        if (widget.currentRow.IDTPITUTLCOMPAG == '0') {
            //Dados Conta Receber
            dataSetPagar[0].VRTOTALCOMPPAG = widget.currentRow.VRTOTALCOMREC;
            dataSetPagar[0].VRTOTALCOMPPAG = widget.currentRow.VRTOTALCOMPPAG;
        } else {
            //Dados Conta Receber
            dataSetPagar[0].VRTOTALCOMPPAG = widget.currentRow.SOMACONTACORRENTE;
        }
        return dataSetPagar;
    };

    this.preparaRowLancamentoReceber = function (widget, dataSet) {
        var empresa = widget.currentRow.CDEMPRESA;
        var contaCorrente = templateManager.container.getWidget('CompensacaoContasPagar').getField('CONTACORRENTE');
        contaCorrente.dataSource.getCheckedRows().forEach(function (data) {
            var indice = dataSet.length;
            dataSet[indice] = {};
            dataSet[indice].NRLANBOLREC = data.NRLANBOLREC;
            dataSet[indice].VRFINALBREC = data.VRFINALBREC;
            dataSet[indice].CDAJUCTREC = widget.currentRow.CDAJUCTREC;
            dataSet[indice].CDTPBAIXAREC = widget.currentRow.CDTPBAIXAREC;
            dataSet[indice].NRSEQVENREC = data.NRSEQVENREC;
            dataSet[indice].NRPARCELAREC = data.NRPARCELAREC;
            dataSet[indice].VRTOTALCOMPPAG = data.SOMACONTACORRENTE;
            dataSet[indice].IDLANCOMCONTPAG = data.IDLANCOMCONTPAG;
            dataSet[indice].CDREDUCONT= data.CDREDUCONT;
            dataSet[indice].NRMOVIDEBI= data.NRMOVIDEBI;
            dataSet[indice].VRMOVIDEBI= data.VRMOVIDEBI;
            dataSet[indice].CDFIMOVFLREC= data.CDFIMOVFLREC;
            dataSet[indice].CDEMPRESA = empresa;
        });
        return dataSet;
    };

    this.preparaRowContasReceber = function (widget, dataSet) {
        var empresa = widget.currentRow.CDEMPRESA;
        var contaReceber = templateManager.container.getWidget('CompensacaoContasPagar').getField('CONTARECEBER');
        contaReceber.dataSource.getCheckedRows().forEach(function (data) {
            var indice = dataSet.length;
            dataSet[indice] = {};
            dataSet[indice].CDEMPRESA = empresa;
            dataSet[indice].CDTPBAIXAREC = widget.currentRow.CDTPBAIXAREC;
            dataSet[indice].CDFILIAL = data.CDFILIAL;
            dataSet[indice].CDCLASSFINA = data.CDCLASSFINA;
            dataSet[indice].CDFILIBOLREC = data.CDFILIBOLREC;
            dataSet[indice].NRLANBOLREC = data.NRLANBOLREC;
            dataSet[indice].VRFINALBREC = data.VRFINALBREC;
            dataSet[indice].CDFIMOVFLREC = data.CDFIMOVFLREC;
            dataSet[indice].NRLANCTONF = data.NRLANCTONF;
            dataSet[indice].NRSEQVENREC = data.NRSEQVENREC;
            dataSet[indice].NRPARCELAREC = data.NRPARCELAREC;
            dataSet[indice].VRORIGREC = data.VRORIGREC;
            dataSet[indice].DTMOVFLUXO = moment().format('DD/MM/YYYY');
            dataSet[indice].NMRAZSOCCLIE = data.NMRAZSOCCLIE;
        });
        return dataSet;
    };

    this.ativaFiltroContasReceber = function (args) {
        if (!wasFiltered) {
            var filtroContaReceber = templateManager.container.getWidget('filtroContaReceber');
            ScreenService.openPopup(filtroContaReceber).then(function () {
                filtroContaReceber.activate();
            });
            args.e.preventDefault();
        } else {
            wasFiltered = false;
        }
    };

    this.filtraContasReceber = function (filtroContasReceber) {
        var compensacaoPagar = filtroContasReceber.getParent();
        var contaReceber = compensacaoPagar.getField('CONTARECEBER');
        ScreenService.closePopup(false);
        wasFiltered = true;
        this.aplicaFiltroContasReceber(filtroContasReceber, contaReceber).then(function () {
            contaReceber.reload();
            $("#CONTARECEBER").scope().openField(contaReceber);
        });
    };

    this.aplicaFiltroContasReceber = function (filtroContasReceber, contaReceber) {
        var deferred = ZHPromise.defer();
        //var compensacaoPagar = filtroContasReceber.getParent();
        var clienteConcat, cdfiliemit, cdfilial = '';
        if (!filtroContasReceber.CLIENTEFILTRO) {
            if (filtroContasReceber.currentRow.hasOwnProperty('CLIENTEFILTRO') &&
                filtroContasReceber.currentRow.CLIENTEFILTRO.length > 0){
                    clienteConcat = filtroContasReceber.currentRow.CLIENTEFILTRO.join("_");
            }else{
                clienteConcat = [];
            }
        } else {
            clienteConcat = [];
        }
        if (Array.isArray(filtroContasReceber.getField('CDFILIEMIT').value())) {
            filtroContasReceber.getField('CDFILIEMIT').value().forEach(function (data) {
                cdfiliemit += '_' + data;
            });
            cdfiliemit = (cdfiliemit) ? cdfiliemit + "_" : null;
        }
        if (Array.isArray(filtroContasReceber.getField('CDFILIALANC').value())) {
            filtroContasReceber.getField('CDFILIALANC').value().forEach(function (data) {
                cdfilial += '_' + data;
            });
            cdfilial = (cdfilial) ? cdfilial + "_" : null;
        }
        var valorIni = filtroContasReceber.getField('VRFINALBREC_INI').value();
        var valorFin = filtroContasReceber.getField('VRFINALBREC_FIN').value();
        contaReceber.dataSourceFilter.forEach(function (filtro) {
            switch (filtro.name) {
                case 'CDCLIENTE':
                    filtro.value = (clienteConcat.length != 0) ? clienteConcat : null;
                    break;
                case 'TCDCLIENTE':
                    filtro.value = (clienteConcat.length != 0) ? null : 'T';
                    break;
                case 'DOCINICIAL':
                    filtro.value = filtroContasReceber.getField('DOCINICIAL').value();
                    break;
                case 'DOCFINAL':
                    filtro.value = filtroContasReceber.getField('DOCFINAL').value();
                    break;
                case 'FILIORIG':
                    filtro.value = cdfiliemit;
                    break;
                case 'FILILANC':
                    filtro.value = cdfilial;
                    break;
                case 'CDNOSNUMREC':
                    filtro.value = filtroContasReceber.getField('CDNOSNUMREC').value();
                    break;
                case 'IDPROTESTREC':
                    filtro.value = filtroContasReceber.getField('IDPROTESTREC').value();
                    break;
                case 'DATAINI':
                    filtro.value = filtroContasReceber.getField('DTLANCCOMPPAG').value().start;
                    break;
                case 'DATAFIN':
                    filtro.value = filtroContasReceber.getField('DTLANCCOMPPAG').value().end;
                    break;
                case 'CDTPBAIXAREC':
                    filtro.value = filtroContasReceber.currentRow.CDTPBAIXAREC;
                    break;
                case 'NRDUPLICATA':
                    filtro.value = filtroContasReceber.currentRow.NRDUPLICATA;
                    break;
                case 'STATUS':
                    filtro.value = filtroContasReceber.getField('STATUS').value();
                    break;
                case 'VALORINI':
                    filtro.value = valorIni;
                    break;
                case 'VALORFIN':
                    filtro.value = valorFin;
                    break;
                case 'TVALOR':
                    if (valorIni && valorFin) {
                        if (valorIni == 0 && valorFin == 0) {
                            filtro.value = 'T';
                        } else {
                            filtro.value = null;
                        }
                    } else {
                        filtro.value = 'T';
                    }

                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.ativaFiltroContaCorrente = function (args) {
        if (!wasFiltered) {
            var filtroContaCorrente = templateManager.container.getWidget('filtroContaCorrente');
            ScreenService.openPopup(filtroContaCorrente).then(function () {
                filtroContaCorrente.activate();
            });
            args.e.preventDefault();
        }
        else {
            wasFiltered = false;
        }
    };

    this.filtraContaCorrente = function (filtroContaCorrente) {
        var compensacaoPagar = filtroContaCorrente.getParent();
        var contaCorrente = compensacaoPagar.getField('CONTACORRENTE');
        ScreenService.closePopup(false);
        wasFiltered = true;
        this.aplicaFiltroContaCorrente(filtroContaCorrente, contaCorrente).then(function () {
            contaCorrente.reload();
            $("#CONTACORRENTE").scope().openField(contaCorrente);
        });
    };

    this.aplicaFiltroContaCorrente = function (filtroContaCorrente, contaCorrente) {
        var deferred = ZHPromise.defer();
        var compensacaoPagar = filtroContaCorrente.getParent();
        var dataInicial, dataFinal, valorInicial, valorFinal = null;
        var tipoDebito = '';
        if (filtroContaCorrente.currentRow.TIPODEBITO){
            filtroContaCorrente.currentRow.TIPODEBITO.forEach(function (value) {
                tipoDebito += "_" + value;
            });
            tipoDebito = (tipoDebito) ? tipoDebito + "_" : null;
        }
        var contas = '';
        if (!filtroContaCorrente.CONTAFILTRO) {
            if (filtroContaCorrente.currentRow.hasOwnProperty('CONTAFILTRO') &&
                filtroContaCorrente.currentRow.CONTAFILTRO.length > 0){
                filtroContaCorrente.currentRow.CONTAFILTRO.forEach(function (value) {
                    contas += "_" + value;
                });
                contas = (contas) ? contas + "_" : null;
            }else{
                contas = [];
            }
        }else{
            contas = [];
        }
        contaCorrente.dataSourceFilter.forEach(function (data) {
            switch (data.name) {
                case 'CDEMPRESA':
                    data.value = compensacaoPagar.currentRow.CDEMPRESA;
                    break;
                case 'CDREDUCONT':
                    data.value = (contas.length != 0) ? contas : null;
                    break;
                case 'TCDREDUCONT':
                    data.value = (contas.length != 0) ? null : 'T';
                    break;
                case 'DATAINICIAL':
                    dataInicial = (filtroContaCorrente.currentRow.DATALANCAMENTO.start) ? filtroContaCorrente.currentRow.DATALANCAMENTO.start : null;
                    data.value = dataInicial;
                    break;
                case 'DATAFINAL':
                    dataFinal = (filtroContaCorrente.currentRow.DATALANCAMENTO.end) ? filtroContaCorrente.currentRow.DATALANCAMENTO.end : null;
                    data.value = dataFinal;
                    break;
                case 'TDATA':
                    data.value = (dataInicial && dataFinal) ? null : 'T';
                    break;
                case 'VALORINICIAL':
                    valorInicial = (filtroContaCorrente.currentRow.VALORINICIAL) ? filtroContaCorrente.currentRow.VALORINICIAL : null;
                    data.value = valorInicial;
                    break;
                case 'VALORFINAL':
                    valorFinal = (filtroContaCorrente.currentRow.VALORFINAL) ? filtroContaCorrente.currentRow.VALORFINAL : null;
                    data.value = valorFinal;
                    break;
                case 'TVALOR':
                    if (valorInicial && valorFinal) {
                        if (valorInicial == 0 && valorFinal == 0) {
                            data.value = 'T';
                        } else {
                            data.value = null;
                        }
                    } else {
                        data.value = 'T';
                    }

                    break;
                case 'CDTPDEBIBANC':
                    data.value = (tipoDebito) ? tipoDebito : null;
                    break;
                case 'TTIPO':
                    data.value = (tipoDebito) ? null : 'T';
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.setTodasContasCorrentes = function (value) {
        var filtroContaCorrente = templateManager.container.getWidget('filtroContaCorrente');
        filtroContaCorrente.CONTAFILTRO = value;
    };

    this.setTodosClientes = function (value) {
        var filtroContaReceber = templateManager.container.getWidget('filtroContaReceber');
        filtroContaReceber.CLIENTEFILTRO = value;
    };

    this.somaValorCompensarContaCorrente = function (contaCorrente) {
        var compensacao = templateManager.container.getWidget('CompensacaoContasPagar');
        var valorCompensar = 0;
        if (compensacao.isActive()){
            contaCorrente.value().forEach(function (data) {
                valorCompensar += parseFloat(data);
            });
        }else{
            contaCorrente.dataSource.getCheckedRows().forEach(function (data) {
                valorCompensar += parseFloat(data.VRMOVIDEBI);
            });
        }
        compensacao.currentRow.SOMACONTACORRENTE = valorCompensar;
        compensacao.activate();
    };

    this.somaValorCompensarContasReceber = function (contaReceber) {
        var compensacao = templateManager.container.getWidget('CompensacaoContasPagar');
        var valorCompensar = 0;
        if (compensacao.isActive()){
            contaReceber.value().forEach(function (data) {
                valorCompensar += parseFloat(data);
            });
        }else{
            contaReceber.dataSource.getCheckedRows().forEach(function (data) {
                valorCompensar += parseFloat(data.VRFINALBREC);
            });
        }
        compensacao.currentRow.VRTOTALCOMPPAG = valorCompensar;
        compensacao.activate();
    };

    this.cancelarFiltro = function () {
        ScreenService.closePopup();
        templateManager.container.getWidget('CompensacaoContasPagar').activate();
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag18000_CompensacaoContaPagar_Controller', Pag18000_CompensacaoContaPagar_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag20000_PosicaoTitulosAPagar_Controller.js
var pag20000_PosicaoTitulosAPagar_Controller = function(templateManager, ScreenService, General, ZHPromise) {
	
	var self = this;
	
	var emAtraso      = [];
	var emAtrasoValor = 0;
	
	var emAberto      = [];
	var emAbertoValor = 0;
	
	var planejado      = [];
	var planejadoValor = 0;
	
	var baixado       = [];
	var baixadoValor  = 0;
	
	var total         = [];
	var totalValor    = 0;
	
	var widgetAtiva = "TotalNotification";
	
	self.initializeCards = function(){
        self.resetColorCards('TotalNotification');
	};
	
	self.refreshView = function(){
	    self.aplicaFiltro();
	};
	
	self.resetColorCards = function(cardAtual){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		
		var widgetAtrasoNotification = templateManager.container.getWidget('AtrasoNotification');
		var widgetAbertoNotification = templateManager.container.getWidget('AbertoNotification');
		var widgetPlanejadoNotification = templateManager.container.getWidget('PlanejadoNotification');
		var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
		var widgetTotalNotification = templateManager.container.getWidget('TotalNotification');
		
		switch(cardAtual) {
            /*case 'AtrasoNotification':
            		widgetAtrasoNotification.color = '#D34A40'; // muda
            		$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#D34A40');
            		widgetAbertoNotification.color = '#808080';
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
            	break;
            case 'AbertoNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#086A87');
            		widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
                break;
            case 'RecebidoNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#298A08'; // muda
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#298A08');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
                break;
             
            case 'TotalNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#0B2F3A'; // muda
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#0B2F3A');
                break;*/
                
                case 'AtrasoNotification':
            		//widgetAtrasoNotification.color = '#D34A40'; // muda
            		$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '1.0');
            		//widgetAbertoNotification.color = '#808080';
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
            	break;
            	
            	case 'AbertoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '1.0');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
                
            case 'PlanejadoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '1.0');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
                
            case 'RecebidoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#298A08'; // muda
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '1.0');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
             
            case 'TotalNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#0B2F3A'; // muda
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '1.0');
                break;
                
            default:
                
        }
		
		setTimeout(function(){window.dispatchEvent(new Event('resize'));}, 10);

	};

	self.aplicaFiltroStatusAtrasadoPag = function(){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = emAtraso;
		widgetPosicaoTitulosGrid.label = 'Em Atraso';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBPAG')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBPAG')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'A';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('AtrasoNotification');
	};
	
	self.aplicaFiltroStatusAbertoPag = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = emAberto;
		widgetPosicaoTitulosGrid.label = 'Em Aberto';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBPAG')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBPAG')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'A_O';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('AbertoNotification');
	};
	
	self.aplicaFiltroStatusPlanejadoPag = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = planejado;
		widgetPosicaoTitulosGrid.label = 'Planejado';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBPAG')
                filtro.value = 'P';
            if(filtro.name == 'TIDSTATBPAG')
                filtro.value = '';
            if(filtro.name == 'IDSTATUS')
                filtro.value = '';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = 'T';
        });
		
		self.resetColorCards('PlanejadoNotification');
	};
	
	self.aplicaFiltroStatusRecebidoPag = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = baixado;
		widgetPosicaoTitulosGrid.label = 'Baixado';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBPAG')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBPAG')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'B';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('RecebidoNotification');
	};
	
	self.aplicaFiltroStatusTotalPag = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = total;
		widgetPosicaoTitulosGrid.label = 'Total do Período';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBPAG')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBPAG')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = '';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = 'T';
        });
		
		self.resetColorCards('TotalNotification');
	};
	
	self.aplicaFiltroGrid = function(){
		var deferred = ZHPromise.defer();
		
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		
		var cdforneced = widgetPopUpStripe.currentRow.CDFORNECED;
	    var tcdforneced = 'T';
		
		 if(widgetPopUpStripe.currentRow.hasOwnProperty('CDFORNECED')){
	    	if(Array.isArray(cdforneced)){
	    		if(cdforneced.length > 0){
		    		cdforneced = cdforneced.join("_");
		    		tcdforneced = '';
	    		}
		    }
	    }
		
		var dataSourceFilter = [
			{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.CDEMPRESA
            },
			{
                "name": "DTINICIAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.start
            },
			{
                "name": "DTFINAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.end
            },
            {
                "name": "CDFORNECED",
                "operator": "=",
                "value": cdforneced
            },
            {
                "name": "TCDFORNECED",
                "operator": "=",
                "value": tcdforneced
            },
            {
                "name": "IDSTATBPAG",
                "operator": "=",
                "value": ''
            },
            {
                "name": "TIDSTATBPAG",
                "operator": "=",
                "value": 'T'
            },
            {
                "name": "IDSTATUS",
                "operator": "=",
                "value": ''
            },
            {
                "name": "TIDSTATUS",
                "operator": "=",
                "value": 'T'
            }
        ];
        
        widgetPosicaoTitulosGrid.dataSourceFilter = dataSourceFilter;
        widgetPosicaoTitulosGrid.reload().then(function(){
        	emAtraso = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS == 'A';
            });
            
            emAberto = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS != 'B';
            });
            
            planejado = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATBPAG == 'P';
            });
            
            baixado = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS == 'B';
            });
            
            total = widgetPosicaoTitulosGrid.dataSource.data;
        });
        
        var dataSourceFilterNotification = [
			{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.CDEMPRESA
            },
			{
                "name": "DTINICIAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.start
            },
			{
                "name": "DTFINAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.end
            },
            {
                "name": "CDFORNECED",
                "operator": "=",
                "value": cdforneced
            },
            {
                "name": "TCDFORNECED",
                "operator": "=",
                "value": tcdforneced
            }
       ];
        
		General.callRestEngine('fin#/crud/boletpagar_posicao_titulos', 'FilterData', dataSourceFilterNotification, {"itemsPerPage": 500}).then(
	        function(data){
	        	var valores = data.dataset.boletpagar_posicao_titulos;
	        	var widgetAtrasoNotification   = templateManager.container.getWidget('AtrasoNotification');
	        	var widgetAbertoNotification   = templateManager.container.getWidget('AbertoNotification');
	        	var widgetPlanejadoNotification   = templateManager.container.getWidget('PlanejadoNotification');
	        	var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
	        	var widgetTotalNotification    = templateManager.container.getWidget('TotalNotification');
	        	
	        	if(valores.length > 0){
	        		widgetAtrasoNotification.notification    = self.formataValorBRL(valores[0].ATRASO);
		            widgetAbertoNotification.notification    = self.formataValorBRL(valores[0].ABERTO);
		            widgetPlanejadoNotification.notification = self.formataValorBRL(valores[0].PLANEJADO);
		            widgetRecebidoNotification.notification  = self.formataValorBRL(valores[0].RECEBIDO);
		            widgetTotalNotification.notification     = self.formataValorBRL(valores[0].TOTAL);
	        	}else{
	        		widgetAtrasoNotification.notification   = self.formataValorBRL(0);
		            widgetAbertoNotification.notification   = self.formataValorBRL(0);
		            widgetPlanejadoNotification.notification = self.formataValorBRL(0);
		            widgetRecebidoNotification.notification = self.formataValorBRL(0);
		            widgetTotalNotification.notification    = self.formataValorBRL(0);
	        	}
	        		
        		widgetPosicaoTitulosGrid.activate();
	        	deferred.resolve();
	        });
        
		return deferred.promise;
	   
	};
	
    self.verificaStatusTitulos = function (data) {
       var deferred = ZHPromise.defer();

        data.dataset.boletpagar_cliente_posicao_titulos.forEach(function(boleto){
    		total.push(boleto);
    		totalValor += parseFloat(boleto.VRFINALBPAG); 
    		
    		// Em Atraso
            if(boleto.IDSTATUS == 'A'){
            	emAtraso.push(boleto);
            	emAtrasoValor += parseFloat(boleto.VRFINALBPAG);
            }
            	
            // Baixado
            if(boleto.IDSTATUS == 'B'){
            	baixado.push(boleto);
            	baixadoValor += parseFloat(boleto.VRFINALBPAG);
            }
            
            // Em Aberto	
            if(boleto.IDSTATUS != 'B'){
	            emAberto.push(boleto);
	            emAbertoValor += parseFloat(boleto.VRFINALBPAG);
            }
            
            // Planejado	
            if(boleto.IDSTATBPAG == 'P'){
	            planejado.push(boleto);
	            planejadoValor += parseFloat(boleto.VRFINALBPAG);
            }
        });
        
        deferred.resolve();
        
        return deferred.promise;
    };
	
    self.atualizaValorNotificacao = function (widget) {
        if (widget.dataSource.data.length > 0)
             widget.notification = widget.dataSource.data.shift().NRCOUNT;
        else widget.notification = "0";
    };
	
	self.aplicaFiltro = function(){
	    var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
	    
	    if(widgetPopUpStripe.isValid()){
		    
		    self.aplicaStripe();
		    
		    self.aplicaValoresNotificacoes();
    
		    self.aplicaFiltroGrid().then(function(){
		    	self.aplicaFiltroStatusTotalPag();
		    });
		    
		    
	    	ScreenService.closePopup(widgetPopUpStripe);
	    }
	};
	
	self.aplicaStripe = function(){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetStripe      = templateManager.container.getWidget('stripe');
		
		var msgcdfornecedor = 'Todos os fornecedores filtrados.';
		
		if(widgetPopUpStripe.currentRow.hasOwnProperty('CDFORNECED')){
			var cdfornecedor = widgetPopUpStripe.currentRow.CDFORNECED;
			
	    	if(Array.isArray(cdfornecedor)){
	    		if(cdfornecedor.length > 0){
	    			if(cdfornecedor.length > 1){
		    			msgcdfornecedor = 'Lista de '+cdfornecedor.length+' fornecedores filtrados.';
	    			}else{
	    				msgcdfornecedor = widgetPopUpStripe.currentRow.NMRAZSOCFORN;
	    			}
	    		}
		    }
	    }
	    	
        widgetStripe.currentRow.NMEMPRESA = widgetPopUpStripe.currentRow.CDEMPRESA + ' - ' + widgetPopUpStripe.currentRow.NMEMPRESA;
        widgetStripe.currentRow.DATA      = widgetPopUpStripe.currentRow.DATA.start + ' a ' + widgetPopUpStripe.currentRow.DATA.end;
        widgetStripe.currentRow.CDFORNECED = msgcdfornecedor;

	    widgetStripe.reload();
	};
	
	self.aplicaValoresNotificacoes = function(){
	    /*var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
	    
	    if(widgetPopUpStripe.currentRow.hasOwnProperty('CDCLIENTE')){
	    	if(Array.isArray(cdcliente)){
	    		if(cdcliente.length > 0){
		    		cdcliente = cdcliente.join("_");
		    		tcdcliente = '';
	    		}
		    }
	    }
	    
	    
	    var dataSourceFilter = [
	    	{
	    		"name"     : "CDEMPRESA",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.CDEMPRESA
	    	},
	    	{
	    		"name"     : "DTINICIAL",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.DATA.start
	    	},
	    	{
	    		"name"     : "DTFINAL",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.DATA.end
	    	},
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": cdcliente
            },
            {
                "name": "TCDCLIENTE",
                "operator": "=",
                "value": tcdcliente
            }
    	];
	    
	    General.callRestEngine('fin#/crud/boletreceber_posicao_titulos', 'FilterData', dataSourceFilter).then(
	        function(success){
	        	
	        	var widgetAtrasoNotification   = templateManager.container.getWidget('AtrasoNotification');
	        	var widgetAbertoNotification   = templateManager.container.getWidget('AbertoNotification');
	        	var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
	        	var widgetTotalNotification    = templateManager.container.getWidget('TotalNotification');
	        	
	        	if(success.dataset.boletreceber_posicao_titulos.length > 0){
		        	
		            widgetAtrasoNotification.notification   = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].ATRASO);
		            widgetAbertoNotification.notification   = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].ABERTO);
		            widgetRecebidoNotification.notification = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].RECEBIDO);
		            widgetTotalNotification.notification    = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].TOTAL);
	        	}else {
	        		widgetAtrasoNotification.notification   = self.formataValorBRL(0);
		            widgetAbertoNotification.notification   = self.formataValorBRL(0);
		            widgetRecebidoNotification.notification = self.formataValorBRL(0);
		            widgetTotalNotification.notification    = self.formataValorBRL(0);
	        	}
	        });*/
	};
	
	self.abreFiltro = function(){
	    var widget = templateManager.container.getWidget('popUpStripe');
	    ScreenService.openPopup(widget);
	    widget.activate();
	};
	
	self.formataValorBRL = function (valor){
		return parseFloat(valor).toLocaleString('pt-BR');
	};
	
	self.cancelar = function(){
	    var widget = templateManager.container.getWidget('PosicaoTitulosGrid');
	    ScreenService.closePopup(widget);
	    widget.activate();
	};
	
};

/* global Configuration */
Configuration(function(ContextRegister) {
	ContextRegister.register("Pag20000_PosicaoTitulosAPagar_Controller", pag20000_PosicaoTitulosAPagar_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag30000_RetornoDDA_Controller.js
var Pag30000_RetornoDDA_Controller = function(ScreenService, templateManager,General, ZHPromise, Pag00200_RetornoPagamento_Service){

	this.importaRetornoDDA = function(widget){
		if (widget.isValid()){
			var route = 'fin_contaPagar#/importaRetornoDDA';
			var arqRetorno = widget.currentRow.DSARQUIVO[0];
			var conteudoArquivo = arqRetorno.b64File.split(',')[1];
			var row = {
        	    'CDBANCO': widget.currentRow.CDBANCO,
        	    'CDBANCCNAB': widget.currentRow.CDBANCCNAB,
				'ARQUIVO': conteudoArquivo
			};
			General.callRestEngine(route, 'Row', row)
				.then(function(response){
					if (response.dataset.retornoDDA[0].length > 0){
						ScreenService.showMessage('Atenção! Existem inconsistências no arquivo de retorno, verifique na tela de Validação DDA.');
						/*var report = 'InconsistenciasDDA';
        	            var routeReport = 'fin_contaPagar#/' + report;
        	            var row = {};
						General.openReport(routeReport, report, row);*/
					}else{
                        ScreenService.showMessage('Não existem inconsistências no arquivo de retorno.');
                    }
				}.bind(this))
				.catch(function(erro){
					ScreenService.showMessage(erro.data.error);
				});
		}
	};
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag30000_RetornoDDA_Controller', Pag30000_RetornoDDA_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag41200_TipoAjusteContaPagarController.js
var Pag41200_TipoAjusteContaPagarController = function(templateManager, Pag41200_TipoAjusteContaPagarService){

   this.findCompleteLevel = function(widget) {
        var verificaNivel = widget.getField('CDPICTCCTB').dataSource.data[0].CDPICTCCTB;
        verificaNivel = verificaNivel.split('.');
        verificaNivel = verificaNivel.length;
        widget.getField('CDCONTCTBL').dataSourceFilter[0].value = verificaNivel;
        widget.getField('CDCONTCTBL').reload();
    };

    this.setContctblMask = function() {

      Pag41200_TipoAjusteContaPagarService.getContctblMask().then(function(response){
          var paramgeral = response.dataset.paramgeral[0];
          var contctblMask = paramgeral.CDPICTCCTB;

          templateManager.container.getWidget('pag41200_tipo_ajuste_conta_pagar_grid').getField('CDCONTCTBL').mask.params.mask = contctblMask;
          templateManager.container.getWidget('pag41200_tipo_ajuste_conta_pagar_grid').getField('CDCONTCTBL').fields[0].mask.params.mask = contctblMask;

        });

    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag41200_TipoAjusteContaPagarController', Pag41200_TipoAjusteContaPagarController);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag43000_ParametrosContasPaga_Controller.js
var  Pag43000_ParametrosContasPaga_Controller = function(ScreenService, localStorageEngine){

	var widget1;
	var widget2;

	this.setCampos = function(widgetTabbed){
		widget1 = widgetTabbed.widgets[0];
        var data = widget1.dataSource.data[0];
        if (!data){
            widget1.currentRow = {};
            widget1.currentRow.__is_new = true;
            widget1.currentRow.CDGERALPAG = '';
            widget1.currentRow.NRORG = '';
            widget1.currentRow.DSMULTAPAG = '';
            widget1.currentRow.DSMORAPAG = '';
            widget1.currentRow.DSCREPAG = '';
            widget1.currentRow.DSJUCTPAG = '';
            widget1.currentRow.NMANTAUTO = '';
            widget1.currentRow.DSUANTAUTO = '';
            widget1.currentRow.NMCREDCONT = '';
            widget1.currentRow.NMBAIXADOC = '';
            widget1.currentRow.NMBAIXATED = '';
            widget1.currentRow.IDUTLNFDEVOL = '';
            widget1.currentRow.DSTPCREDBANC = '';
            widget1.currentRow.IDHABIBOLE = 'N';
            widget1.currentRow.IDUTIAUTDESP = 'N';
            widget1.currentRow.IDEXTELACALAJUBX = 'N';
            widget1.currentRow.QTDIAANTBAIP = '';
            widget1.currentRow.QTDIAAPOBAIP = '';
            widget1.currentRow.QTDIAANTBOLP = '';
            widget1.currentRow.QTDIAAPOBOLP = '';
        }else{
            widget1.currentRow = data;
            widget1.currentRow.__is_new = false;
        }
		widget2 = widgetTabbed.widgets[1];
        widget2.currentRow = {};
        widget2.currentRow.QTDIAANTBAIP = widget1.currentRow.QTDIAANTBAIP;
        widget2.currentRow.QTDIAAPOBAIP = widget1.currentRow.QTDIAAPOBAIP;
        widget2.currentRow.QTDIAANTBOLP = widget1.currentRow.QTDIAANTBOLP;
        widget2.currentRow.QTDIAAPOBOLP = widget1.currentRow.QTDIAAPOBOLP;
	};

	this.salvar = function(param){
        if (widget1.currentRow.__is_new){
            this.setCurrentRow();
        }else{
            this.setIntervaloDatas();
        }
		widget1.saveRow().then(function(success){
			widget1.template = widget1.templateToView;
			widget1.view();
			widget2.template = widget2.templateToView;
			widget2.view();
            widget1.reload();
            widget2.reload();
			if (param){
				widget2.activate();
			} else {
				widget1.activate();
			}
		},  function(callback){
			ScreenService.showMessage(callback.data.error);
		});
	};

    this.setIntervaloDatas = function(){
        widget1.currentRow.__is_new = false;
        widget1.currentRow.QTDIAANTBAIP = widget2.currentRow.QTDIAANTBAIP;
        parseInt(widget1.currentRow.QTDIAANTBAIP);
        widget1.currentRow.QTDIAAPOBAIP = widget2.currentRow.QTDIAAPOBAIP;
        parseInt(widget1.currentRow.QTDIAAPOBAIP);
        widget1.currentRow.QTDIAANTBOLP = widget2.currentRow.QTDIAANTBOLP;
        parseInt(widget1.currentRow.QTDIAANTBOLP);
        widget1.currentRow.QTDIAAPOBOLP = widget2.currentRow.QTDIAAPOBOLP;
        parseInt(widget1.currentRow.QTDIAAPOBOLP);
    };

    this.retornaOperadorLogado = function() {
        var filter = [
            {
                "name": "CDOPERADOR",
                "operator": "=",
                "value": "OPERADOR_LOGADO"
            }
        ];
        return localStorageEngine.download("fin#/crud/operador", filter);
    };

    this.setCurrentRow = function(){
        var nrorg = null;
        this.retornaOperadorLogado().then(function(operador){
            nrorg = operador[0].NRORG;
        });
        widget1.currentRow = {};
        widget1.currentRow.__is_new = true;
        widget1.currentRow.CDGERALPAG = 1;
        widget1.currentRow.NRORG = nrorg;
        widget1.currentRow.DSMULTAPAG = '';
        widget1.currentRow.DSMORAPAG = '';
        widget1.currentRow.DSCREPAG = '';
        widget1.currentRow.DSJUCTPAG = '';
        widget1.currentRow.NMANTAUTO = '';
        widget1.currentRow.DSUANTAUTO = '';
        widget1.currentRow.NMCREDCONT = '';
        widget1.currentRow.NMBAIXADOC = '';
        widget1.currentRow.NMBAIXATED = '';
        widget1.currentRow.IDUTLNFDEVOL = '';
        widget1.currentRow.DSTPCREDBANC = '';
        widget1.currentRow.IDHABIBOLE = 'N';
        widget1.currentRow.IDUTIAUTDESP = 'N';
        widget1.currentRow.IDEXTELACALAJUBX = 'N';
        widget1.currentRow.QTDIAANTBAIP = '';
        widget1.currentRow.QTDIAAPOBAIP = '';
        widget1.currentRow.QTDIAANTBOLP = '';
        widget1.currentRow.QTDIAAPOBOLP = '';
    };

	this.editar = function(){
		widget1.template = widget1.templateToForm;
		widget2.template = widget2.templateToForm;
		widget1.edit();
		widget2.edit();
		widget1.backupRow = (widget1.currentRow) ? angular.copy(widget1.currentRow): {};
		widget2.backupRow = (widget2.currentRow) ? angular.copy(widget2.currentRow): {};
	};

	this.cancelar = function(){
		widget1.currentRow = (widget1.currentRow) ? widget1.backupRow : null;
		widget1.template = widget1.templateToView;
		widget1.view();
		widget2.currentRow = (widget1.currentRow) ? widget2.backupRow : null;
		widget2.template = widget2.templateToView;
		widget2.view();
	};

};

Configuration(function(ContextRegister){
    ContextRegister.register('Pag43000_ParametrosContasPaga_Controller', Pag43000_ParametrosContasPaga_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag47000_tipoGarantiaEmprestimo_Controller.js
function Pag47000_tipoGarantiaEmprestimo_Controller(General,ScreenService,templateManager,ZhFilterPreferences) {
	this.salvar = function(args) {
		var Widget = args.owner.widget;
        General.saveCrud(Widget).then(function(Success) {
            templateManager.container.getWidget('grid_tipoDeGarantiaDeEmprestimo').getAction('voltarPagina').isVisible=true;
        }, function(callback){
            ScreenService.showMessage(callback.data.error);
        });
    };
    this.excluir = function(widget){
   		General.deleteCrud(widget);
    };

     this.cancelar = function(args){
        var Widget = args.owner.widget;
        var currentRow = Widget.currentRow;
      	General.backOrCancel(args);
        templateManager.container.getWidget('grid_tipoDeGarantiaDeEmprestimo').getAction('voltarPagina').isVisible=true;
    };
    
    this.ajustaFiltro = function(widget) {
        widget.dataSourceFilter = [];
        if(widget.appliedFilter.FILTER_ROW.CDTPGARANTIAEMP) {
            widget.dataSourceFilter = [
                {
                    "name": "CDTPGARANTIAEMP",
                    "operator": "=",
                    "value": widget.appliedFilter.FILTER_ROW.CDTPGARANTIAEMP
                }
            ];
        } 
    };
    
}

Configuration(function(ContextRegister) {
	ContextRegister.register('Pag47000_tipoGarantiaEmprestimo_Controller', Pag47000_tipoGarantiaEmprestimo_Controller);
});

// FILE: ./assets/modules/contaPagar/assets/js/controllers/Pag49000_ValidacaoDDA_Controller.js
var Pag49000_ValidacaoDDA_Controller = function(ScreenService, templateManager,General, ZHPromise, Pag00200_RetornoPagamento_Service, ZhFilterPreferences){

    /**
     * Seta DataSource Filter
     */
    this.setDataSourceFilter = function(validacaoDDAGrid, preference){
        validacaoDDAGrid.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA'    : filter.value = preference.FILTER_ROW.CDEMPRESA;
                    break;
                case 'CDBANCO'      : filter.value = preference.FILTER_ROW.CDBANCO;
                    break;
                case 'NRINSJURFORN' : filter.value = (preference.FILTER_ROW.NRINSJURFORN) ? preference.FILTER_ROW.NRINSJURFORN : null;
                    break;
                case 'DTINICIAL'    : filter.value = preference.FILTER_ROW.DTINTERVALO.start;
                    break;
                case 'DTFINAL'      : filter.value = preference.FILTER_ROW.DTINTERVALO.end;
                    break;
            }
        });
        
        this.removeFieldsClausulaWhere(validacaoDDAGrid, preference);
    };

    this.removeFieldsClausulaWhere = function(validacaoDDAGrid, preference){
        var filtrosRemoviveis = ['DTINTERVALO'];
        var dataSourceFilter = validacaoDDAGrid.dataSourceFilter;
        for (var i = dataSourceFilter.length - 1; i >= 0; i--) {
            if (filtrosRemoviveis.includes(dataSourceFilter[i].name)){
	        	dataSourceFilter.splice(i, 1);
	        }
        }
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        for (var key in preference.CUSTOM_FILTER_ROW) {
            if (filtrosRemoviveis.includes(key)){
                delete preference.CUSTOM_FILTER_ROW[key];
            }
        }
    };
    
    /**
     * Busca Empresa
     * @return cdempresa
     */
	this.buscaEmpresa = function (cdempresa){
        return Pag00200_RetornoPagamento_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Filial
     * @return filial
     */
    this.buscaFilial = function (){
        return Pag00200_RetornoPagamento_Service.buscaFilial().then(function(filial) {
            return {dataset : {filial : filial}};
        });
    };
    
    /**
     * Busca Padrão Empresa
     */
    this.buscaPadraoEmpresa = function() {
        var deferred = ZHPromise.defer();
        this.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            this.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                deferred.resolve(empresa);
            });
        }.bind(this));
        return deferred.promise;
    };

    /**
     * Abre Filtro
     * @param widget
     */
    this.openFilter = function(widget){
        this.buscaPadraoEmpresa().then(function(empresa){
            var rowFilter = {
                'CDEMPRESA'  : empresa.CDEMPRESA,
                'NMEMPRESA'  : empresa.NMEMPRESA,
                'DTINTERVALO': {start : moment().format('DD/MM/YYYY'), end : moment().format('DD/MM/YYYY')}
            };
            ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(widget, rowFilter);
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Pag49000_ValidacaoDDA_Controller', Pag49000_ValidacaoDDA_Controller);
});

// FILE: ./assets/modules/relatoriosBancarios/assets/js/controllers/bco00600_comparativo_previsto_realizado_Controller.js
function bco00600_comparativo_previsto_realizado_Controller(General, ScreenService, templateManager, ZhFilterPreferences, localStorageEngine) {

    var self = this;

    this.openFilter = function(widget){
        General.callRestEngine( "fin#/paramgeral_geralrec", 'FilterData', []).then(function (paramgeral) {
            var rowsToSet = {};
            paramgeral = paramgeral.dataset.paramgeral_geralrec[0] || [];
            if(paramgeral.IDTPORCAFINA == 'F'){
                widget.getField('CDFILIAL').isVisible = true;
                
                
                if(widget.getField('CDFILIAL').validations){
                    widget.getField('CDFILIAL').validations = {};
                    widget.getField('CDFILIAL').validations.required = true;
                }
                rowsToSet.CDFILIAL = null;
            }else{
                widget.getField('CDFILIAL').isVisible = false;
                rowsToSet.CDFILIAL = getLocalVar('CDFILIAL');
                
                if(widget.getField('CDFILIAL').validations){
                    delete widget.getField('CDFILIAL').validations.required;    
                }
            }
            rowsToSet.TPORCAFINA = paramgeral.IDTPORCAFINA;

            ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
            
        });
        

    };

    this.changeHeaderValues = function(widget, data){
        var newFilter = widget.e.metadata.filter;
        
        newFilter.push({"label": 'Período', "value": widget.owner.dataSourceFilter[0].value + ' a ' + widget.owner.dataSourceFilter[1].value });

    };
    
    this.preparaFiltroMovimentacao = function(args) {
        var widget    = templateManager.container.getWidget('comparativo_grid');
    	var filterRow = args.e.preference.FILTER_ROW;
    	var itemErradoIndex = null;
    	
    	for(var index = 0, filters = widget.dataSourceFilter; index< filters.length ; index ++){
    	   var item = filters[index];
            if(item.name == 'DTINI'){
    	        item.value = filterRow.DTMES ? filterRow.DTMES.start : null;
    	    }else if(item.name == 'DTFIM'){
    	        item.value = filterRow.DTMES ? filterRow.DTMES.end : null;
    	    }else if(item.name == 'CDEMPRESA'){
    	        item.value = item.value?(item.value.length > 0 && Array.isArray(item.value)?item.value.join('_'):'T'):'T';
    	    }else if(!filterRow[item.name]){
    	        item.value = 'T';
    	    }else if(filterRow[item.name] && (!item.value|| Array.isArray(item.value))){
    	        item.value = filterRow[item.name];
    	    }
    	    if(item.name == 'DTMES' || item.name == 'NMEMPRESA'){
    	        itemErradoIndex = index;
    	        break;
    	    }
    	}
        if(itemErradoIndex != null){
            widget.dataSourceFilter.splice(itemErradoIndex, 1);
            self.preparaFiltroMovimentacao(args);    
        }else{
            widget.reload();    
        }
    	
    };

    self.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var FILIAL = widget.getField('CDFILIAL');
        if(FILIAL.isVisible){
            FILIAL.setValue("");
            if(cdempresa.length){  
                FILIAL.dataSourceFilter[0].value = cdempresa;
                FILIAL.readOnly = false;
                FILIAL.reload();    
            }else{
                FILIAL.readOnly = true;
            }    
        }
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('bco00600_comparativo_previsto_realizado_Controller', bco00600_comparativo_previsto_realizado_Controller);
});

// FILE: ./assets/modules/relatoriosBancarios/assets/js/controllers/bco21100ExtratoDeMovimentacaoBancariaController.js
function bco21100ExtratoDeMovimentacaoBancariaController(General, ScreenService, templateManager, ZhFilterPreferences) {

    var self = this;

    this.openFilter = function(widget){
        var rowsToSet = {};
        rowsToSet.DTMOVBANC = { 
            start: moment().format('DD/MM/YYYY'),
            end: moment().format('DD/MM/YYYY')
        };
        rowsToSet.P_SALDOINICIAL = "0";
        ZhFilterPreferences.openFilterWidget(widget, rowsToSet);
    };

    self.buildFilter = function (widget, preference) {
        var filterRow = preference.FILTER_ROW;
        
        var filtro = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": filterRow.CDEMPRESA ? filterRow.CDEMPRESA : 'T'
            },
            {
                "name": "P_SALDOINICIAL",
                "operator": "=",
                "value": filterRow.P_SALDOINICIAL ? filterRow.P_SALDOINICIAL : '0'
            },
            {
                "name": "DTPERIODOINI",
                "operator": "=",
                "value": filterRow.DTMOVBANC.start
            },
            {
                "name": "DTPERIODOFIN",
                "operator": "=",
                "value": filterRow.DTMOVBANC.end
            }
        ];
        
        if(filterRow.CDREDUCONT){
            filtro.push(
                {
                    "name": "CDREDUCONT",
                    "operator": "=",
                    "value": filterRow.CDREDUCONT
                }
            );
        }
        
        widget.dataSourceFilter = filtro; 
        
        widget.reload();
        
    };
    
    /*self.changeDateFields = function(widget) {
        var currentRow = widget.row;
        currentRow.DTPERIODOINI = currentRow.DTMOVBANC.start;
        currentRow.DTPERIODOFIN = currentRow.DTMOVBANC.end;
    };*/

    //Limpa os nomes dos campos invterval do dataSource
    var getDataSourceWithoutIntervalFieldNames = function(filtro){
        for(var i = 0; i < filtro.length; i++) {
            if(filtro[i].name === 'DTMOVBANC') {
                filtro.splice(i, 1);
                i--;
            }
        }
        return filtro;
    };
   
    this.changeHeaderValues = function(widget, data){
        var newFilter = widget.e.metadata.filter;
        
        var correctFilter = [];
        var aux_arr = [];
        var value;
        newFilter.forEach(function(filter){
            if(filter.label == 'Saldo Inicial'){
                data.getField("P_SALDOINICIAL").filterProperties.dataSource.data.forEach(function(saldo){
                    if(saldo.value == filter.value.substr(10,1)){
                        value = saldo.name;
                    }
                });
                filter.value = value+'.'; 
                correctFilter.push(filter);
            } else if(filter.label == 'Data'){
                filter.label = 'Período';
                correctFilter.push(filter);
            }
        });
        
        correctFilter.forEach(function(aux){
            if((aux.value.match(/is equal to/)) || (aux.value.match(/é igual a/))){
                value = aux.value.substr(11);
                aux.value = value;
                aux_arr.push(aux);
            } else if((aux.value.match(/is between/)) || (aux.value.match(/está entre/))){
                value = aux.value.substr(11);
                value = value.replace('e','a');
                aux.value = value;
                aux_arr.push(aux);
            } else {
                aux_arr.push(aux);
            }
        });
        
        correctFilter = aux_arr;
        
        var nmempresa = data.dataSource.data[0].NMEMPRESA;
        
        data.dataSourceFilter.forEach(function(filter){
            if((filter.name == "CDEMPRESA") && (filter.value != "T")){
                correctFilter.push({
                    label: 'EMPRESA',
                    value: filter.value+' - '+nmempresa
                });
            } else if((filter.name == "CDREDUCONT") && filter.value){
                General.callRestEngine(
                    'fin#/contcorr_banco_masks',
                    'FilterData', 
                    [
                        {
                            "name": "CDOPERADOR",
                            "operator": "=",
                            "value": getLocalVar('CDOPERADOR')
                        },
                        {
                            "name": "CDREDUCONT",
                            "operator": "=",
                            "value": filter.value
                        }
                    ]
                ).then(function(response){
                    response = response.dataset.contcorr_banco_masks[0];
                    correctFilter.push({
                        label: 'CONTA CORRENTE',
                        value: (response.CDAGENCIA+'/'+response.CDCONTCORRMASK+' '+response.DSCONTCORR)
                    });
                });
            }
        }.bind(this));
        
        
        widget.e.metadata.filter = correctFilter;    

    };
    
    this.preparaFiltroMovimentacao = function(args) {
    	var widget    = templateManager.container.getWidget('movimentacao_grid');
    	var filterRow = args.e.preference.FILTER_ROW;
    	
    	args.e.preference.FILTER_ROW = {};
    	args.e.preference.CUSTOM_FILTER_ROW = {};
    	
    	General.callRestEngine('fin#/gerarRelatorioMovimentacaoBancaria', 'Row', filterRow).then(function (response) {
    		var nrseqmovbanc = response.dataset.movimentacaoBancaria[0];
    		
    		var filtro = [
    			{
    				"name": "NRSEQMOVBANC",
    				"operator": "=",
    				"value": nrseqmovbanc
    			}
    		];
    		
    		args.e.preference.FILTER_ROW = {"NRSEQMOVBANC" : nrseqmovbanc};
    		widget.dataSourceFilter = filtro;
    		widget.reload();
    	}).catch(function(erro){
    		ScreenService.showMessage(erro.data.error);
    	});
    };

}

Configuration(function (ContextRegister) {
	ContextRegister.register('bco21100ExtratoDeMovimentacaoBancariaController', bco21100ExtratoDeMovimentacaoBancariaController);
});

// FILE: ./assets/modules/relatoriosReceber/assets/js/controllers/Rec01400_Relatorio_Customizavel_Curva_ABC_Clientes_Controller.js
var Rec01400_Relatorio_Customizavel_Curva_ABC_Clientes = function(templateManager, ScreenService, General, ZhFilterPreferences, restEngine){
    /**
     * Variáveis de Inicialização
     **/
    var self = this;
    
    /**
     * Abertura do filtro de relatório ao carregar a tela
     * @param widget 
    */
     
    self.openFilterRelatorio = function(widget){
      ZhFilterPreferences.openFilterWidget(widget);
    };

    /**
     * Função para habilitar o campo de Data de Baixa
     * @param widget 
    */

    self.openFilterDate = function(widget){
      var fieldStatus = widget.getField("STATUS");
      var fieldVencimento = widget.getField("DTATUAVENREC");
      var fieldDataBaixa = widget.getField("DTBAIXAREC");
      if (fieldStatus.value() == 'B') {
        fieldDataBaixa.isVisible = true;
        fieldVencimento.class = 4;
        fieldStatus.class = 4;
        fieldDataBaixa.class = 4;
       } else {
        fieldDataBaixa.isVisible = false;
        fieldVencimento.class = 9;
        fieldStatus.class = 3;
      }
      widget.reload();
    };
    
    /**
     * Apliaca o filtro para a requisição dos dados do relatório
     * @param widget, preference
    */
    
    self.filterRelatorio = function(widget, preference) {
      
      var currentRowFiltro = preference.FILTER_ROW;
      var cdEmpresa = currentRowFiltro.EMPRESA;
      var cdCliente = currentRowFiltro.NMRAZSOCCLIE;
      var cdTipoConta = currentRowFiltro.DSTPCTRECEB;
      var cdStatus = currentRowFiltro.STATUS;
      var vencimento = currentRowFiltro.DTATUAVENREC;
      var dtBaixa = currentRowFiltro.DTBAIXAREC;
      var curvaGeral = currentRowFiltro.TIPOCURVA;

      if(dtBaixa === undefined) {
        dtBaixa = ({
          "end": '',
          "start": ''
        });
      }
      
      // Parametros do filtro do frontEnd
      var filter = [
          {
            'name': 'P_EMPRESA',
            'operator': '=',
            'value': cdEmpresa
          },
          {
            'name': 'P_CLIENTE',
            'operator': '=',
            'value': cdCliente
          },
          {
            'name': 'P_TIPOCONTA',
            'operator': '=',
            'value': cdTipoConta
          },
          {
            'name': 'P_STATUS',
            'operator': '=',
            'value': cdStatus
          },
          {
            'name': 'P_DTINIVENC',
            'operator': '=',
            'value': vencimento.start
          },
          {
            'name': 'P_DTFINVENC',
            'operator': '=',
            'value': vencimento.end
          },
          {
            'name': 'P_DTINIBAIXA',
            'operator': '=',
            'value': dtBaixa.start
          },
          {
            'name': 'P_DTFINBAIXA',
            'operator': '=',
            'value': dtBaixa.end
          },
          {
            'name': 'P_TDTBAIXAREC',
            'operator': '=',
            'value': 'T'
          },
          {
            'name': 'P_IDCURVAGERAL',
            'operator': '=',
            'value': curvaGeral
          }
        ];
        
      // rota backend, passando como parametro os dados do filtro da tela
      restEngine.post('/executaProcedureCurvaAbcCliente', {
            "requestType": "Row",
            "row": filter
        }).then((response) => {
          var dados = response.dataset.relatorio_curva_abc_de_clientes.relatorio_curva_abc_de_clientes;
          widget.dataSourceFilter = {};
          widget.dataSourceFilter = [
              {
                'name': 'P_NRSEQCURVAABCCLI',
                'operator': '=',
                'value': dados
              }
            ];
            widget.reload();
        });
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Rec01400_Relatorio_Customizavel_Curva_ABC_Clientes', Rec01400_Relatorio_Customizavel_Curva_ABC_Clientes);
});

// FILE: ./assets/modules/relatoriosReceber/assets/js/controllers/Rec11001_RelatorioCustomizavelTitulosReceberRecebidos_Controller.js
function Rec11001_RelatorioCustomizavelTitulosReceberRecebidos_Controller(General, ScreenService, templateManager, ZhFilterPreferences, ZHPromise) {
    
    var self = this;
    
    self.changeInputValues = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        
        validateStartEndValues(filterRow);
        parseCurrencyValues(filterRow);
    };
    
    self.buildFilter = function (widget, preference) {
        var filtro = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;
        
        validateStartEndValues(filterRow);
        parseCurrencyValues(filterRow);
        
        filtro.forEach(function(fil) {
            /*if (Array.isArray(fil.value)) {
                fil.value = fil.value.join("_");
            }*/
            switch (fil.name) {
                case 'CDEMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_") : null;
                    break;
                case 'DTEMISSAOINI':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.start : null;
                    break;
                case 'DTEMISSAOFIN':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.end : null;
                    break;
                case 'DTENTRSAIDINI':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.start : null;
                    break;
                case 'DTENTRSAIDFIN':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.end : null;
                    break;
                case 'DTCNABRECINI':
                    fil.value = filterRow.DTCNABREC ? filterRow.DTCNABREC.start : null;
                    break;
                case 'DTCNABRECFIN':
                    fil.value = filterRow.DTCNABREC ? filterRow.DTCNABREC.end : null;
                    break;
                case 'DTVENBOLRECINI':
                    fil.value = filterRow.DTVENBOLREC ? filterRow.DTVENBOLREC.start : null;
                    break;
                case 'DTVENBOLRECFIN':
                    fil.value = filterRow.DTVENBOLREC ? filterRow.DTVENBOLREC.end : null;
                    break;
                case 'DTBAXINICIAL':
                    fil.value = filterRow.DTBAIXAREC ? filterRow.DTBAIXAREC.start : null;
                    break;
                case 'DTBAXFINAL':
                    fil.value = filterRow.DTBAIXAREC ? filterRow.DTBAIXAREC.end : null;
                    break;
                case 'TDTBAIXA':
                    fil.value = filterRow.DTBAIXAREC ? filterRow.DTBAIXAREC.start ? null : 'T' : 'T';
                    break;
                case 'DTLBAINICIAL':
                    fil.value = filterRow.DTLANCBAIXAR ? filterRow.DTLANCBAIXAR.start : null;
                    break;
                case 'DTLBAFINAL':
                    fil.value = filterRow.DTLANCBAIXAR ? filterRow.DTLANCBAIXAR.end : null;
                    break;
                case 'NRNOTAFISCINI':
                    fil.value = filterRow.NRNOTAFISCINI ? filterRow.NRNOTAFISCINI.padStart(9, '0') : null;
                    break;
                case 'NRNOTAFISCFIN':
                    fil.value = filterRow.NRNOTAFISCFIN ? filterRow.NRNOTAFISCFIN.padStart(9, '0') : null;
                    break;
                case 'TDTLBAIXA':
                    fil.value = filterRow.DTLANCBAIXAR ? filterRow.DTLANCBAIXAR.start ? null : 'T' : 'T';
                    break;
                case 'VRBAIXARECINI':
                    fil.value = filterRow.VRBAIXARECINI && filterRow.VRBAIXARECFIN ? filterRow.VRBAIXARECINI : null;
                    break; 
                case 'VRBAIXARECFIN':
                    fil.value = filterRow.VRBAIXARECINI && filterRow.VRBAIXARECFIN ? filterRow.VRBAIXARECFIN : null;
                    break;
                case 'TVRBAIXA':
                    fil.value = filterRow.VRBAIXARECINI && filterRow.VRBAIXARECFIN ? null : 'T';
                    break;
            }
        });
        for(var i = 0; i < filtro.length; i++) {
            if(filtro[i].name === 'DTEMISSAO' ||
               filtro[i].name === 'DTENTRSAID' ||
               filtro[i].name === 'DTVENBOLREC' ||
               filtro[i].name === 'DTCNABREC' ||
               filtro[i].name === 'DTBAIXAREC' ||
               filtro[i].name === 'DTLANCBAIXAR' ) {
                filtro.splice(i, 1);
                i--;
            }
        }
        
        preference.CUSTOM_FILTER_ROW = {
            'CDCLIENTE'    : preference.FILTER_ROW.CDCLIENTE,
            'IDSTATUSREC'  : preference.FILTER_ROW.IDSTATUSREC,
            'IDSTATBREC'   : preference.FILTER_ROW.IDSTATBREC,
            'IDSTATUSEMIT' : preference.FILTER_ROW.IDSTATUSEMIT,
            'IDPROTESTREC' : preference.FILTER_ROW.IDPROTESTREC,
            'CDTIPOOPER'   : preference.FILTER_ROW.CDTIPOOPER,
            'CDFILIEMIT'   : preference.FILTER_ROW.CDFILIEMIT,
            'CDFILIALNF'   : preference.FILTER_ROW.CDFILIALNF,
            'CDTPCTRECEB'  : preference.FILTER_ROW.CDTPCTRECEB,
            'CDCLASSFINA'  : preference.FILTER_ROW.CDCLASSFINA,
            'CDVENDEDOR'   : preference.FILTER_ROW.CDVENDEDOR,
            'IDSTATUS'     : preference.FILTER_ROW.IDSTATUS,
            'IDSITUCNABR'  : preference.FILTER_ROW.IDSITUCNABR,
            'CDTPBAIXAREC' : preference.FILTER_ROW.CDTPBAIXAREC,
            'IDSTATUSBREC' : preference.FILTER_ROW.IDSTATUSBREC,
            'NRNOTAFISCINI': preference.FILTER_ROW.NRNOTAFISCINI,
            'NRNOTAFISCFIN': preference.FILTER_ROW.NRNOTAFISCFIN
        };
    };
    
    /*
     * Verificação para igualar valores iniciais e finais do filtro, no caso
     * do usuário preencher um e deixar o outro em branco. Isso irá fazer
     * com que a query retorne somente resultados que sejam exatamente os  
     * mesmos valores inseridos em um dos campos.
     */
    var validateStartEndValues = function(filterRow) {
        // Verifica valor do boleto inicial e final.
        filterRow.VRBOLRECINI = filterRow.VRBOLRECINI || filterRow.VRBOLRECFIN || null;
        filterRow.VRBOLRECFIN = filterRow.VRBOLRECFIN || filterRow.VRBOLRECINI;
        
        // Verifica valor líquido inicial e final.
        filterRow.VRFINALBRECINI = filterRow.VRFINALBRECINI || filterRow.VRFINALBRECFIN || null;
        filterRow.VRFINALBRECFIN = filterRow.VRFINALBRECFIN || filterRow.VRFINALBRECINI;
        
        // Verifica nota fiscal inicial e final.
        filterRow.NRNOTAFISCINI = filterRow.NRNOTAFISCINI || filterRow.NRNOTAFISCFIN || null;
        filterRow.NRNOTAFISCFIN = filterRow.NRNOTAFISCFIN || filterRow.NRNOTAFISCINI;
        
        // Verifica valor da baixa inicial e final.
        filterRow.VRBAIXARECINI = filterRow.VRBAIXARECINI || filterRow.VRBAIXARECFIN || null;
        filterRow.VRBAIXARECFIN = filterRow.VRBAIXARECFIN || filterRow.VRBAIXARECINI;
        
        // Verifica número remessa inicial e final.
        filterRow.NRCNABRECREMINI = filterRow.NRCNABRECREMINI || filterRow.NRCNABRECREMFIN || null;
        filterRow.NRCNABRECREMFIN = filterRow.NRCNABRECREMFIN || filterRow.NRCNABRECREMINI;
        
        // Verifica número retorno inicial e final.
        filterRow.NRCNABRECRETINI = filterRow.NRCNABRECRETINI || filterRow.NRCNABRECRETFIN || null;
        filterRow.NRCNABRECRETFIN = filterRow.NRCNABRECRETFIN || filterRow.NRCNABRECRETINI;
        
        // Verifica valor remessa inicial e final.
        filterRow.VRTOTBOLCNABRECINI = filterRow.VRTOTBOLCNABRECINI || filterRow.VRTOTBOLCNABRECFIN || null;
        filterRow.VRTOTBOLCNABRECFIN = filterRow.VRTOTBOLCNABRECFIN || filterRow.VRTOTBOLCNABRECINI;
    };
    
    /*
     * Função para realizar um parse nos valores monetários inseridos no filtro.
     */
    var parseCurrencyValues = function(filterRow) {
        // Realiza o parse nos valores do boleto inicial e final.
        filterRow.VRBOLRECINI = (filterRow.VRBOLRECINI !== null) ? $.currency.unmaskString(filterRow.VRBOLRECINI) : null; 
        filterRow.VRBOLRECFIN = (filterRow.VRBOLRECFIN !== null) ? $.currency.unmaskString(filterRow.VRBOLRECFIN) : null; 
        
        // Realiza o parse nos valores líquidos inicial e final.
        filterRow.VRFINALBRECINI = (filterRow.VRFINALBRECINI !== null) ? $.currency.unmaskString(filterRow.VRFINALBRECINI) : null; 
        filterRow.VRFINALBRECFIN = (filterRow.VRFINALBRECFIN !== null) ? $.currency.unmaskString(filterRow.VRFINALBRECFIN) : null; 
        
        // Realiza o parse nos valores da baixa inicial e final.
        filterRow.VRBAIXARECINI = (filterRow.VRBAIXARECINI !== null) ? $.currency.unmaskString(filterRow.VRBAIXARECINI) : null; 
        filterRow.VRBAIXARECFIN = (filterRow.VRBAIXARECFIN !== null) ? $.currency.unmaskString(filterRow.VRBAIXARECFIN) : null; 
        // Realiza o parse nos valores de remessa inicial e final.
        filterRow.VRTOTBOLCNABRECINI = (filterRow.VRTOTBOLCNABRECINI !== null) ? $.currency.unmaskString(filterRow.VRTOTBOLCNABRECINI) : null; 
        filterRow.VRTOTBOLCNABRECFIN = (filterRow.VRTOTBOLCNABRECFIN !== null) ? $.currency.unmaskString(filterRow.VRTOTBOLCNABRECFIN) : null; 
    };
    
    self.openFilterTitulosAReceber = function(widgetTitulosAReceber){
        self.buscaDadosIniciaisFiltro(widgetTitulosAReceber).then(function(){
            var rowSetFilter = {
                'DTVENBOLREC' : widgetTitulosAReceber.DTVENBOLREC,
                'CDEMPRESA'   : [widgetTitulosAReceber.empresa.CDEMPRESA]
            };
            
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter).then( function(sucess){
                widgetTitulosAReceber.getField('CDTPCTRECEB').dataSourceFilter[0].value = widgetTitulosAReceber.empresa.CDEMPRESA;
                widgetTitulosAReceber.getField('CDTPCTRECEB').reload();
            });
        });
    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAReceber) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAReceber).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAReceber).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralrec     = self.buscaGeralRec();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralrec, paramestoque]).then(function(response){
            geralrec     = response[0].dataset.geral_rec[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralrec?geralrec.QTDIAANTLANR?geralrec.QTDIAANTLANR:0:0;
            var qtdiaapolanr = geralrec?geralrec.QTDIAAPOLANR?geralrec.QTDIAAPOLANR:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLREC = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    self.buscaGeralRec = function(){
        return General.callRestEngine(
            'fin#/crud/geral_rec',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        );
    };
    
    
    this.changeHeaderValues = function(widget){
        var codigo; 
        /*var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "é igual a".concat(codigo).concat(' - ').concat(nmempresa)
        };*/
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        //newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "é igual a."){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
        
    };
    
    self.changeTipoConta = function(widget){
        var filtroEmpresa = "";
        widget.getField('CDEMPRESA').value().forEach(function(empresa){
            filtroEmpresa = filtroEmpresa+"_"+empresa;
        });
            
        widget.getField('CDTPCTRECEB').dataSourceFilter[0].value = filtroEmpresa;
        widget.getField('CDTPCTRECEB').clearValue();
        widget.getField('CDTPCTRECEB').reload();
    };
    
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec11001_RelatorioCustomizavelTitulosReceberRecebidos_Controller', Rec11001_RelatorioCustomizavelTitulosReceberRecebidos_Controller);
});

// FILE: ./assets/modules/relatoriosReceber/assets/js/controllers/Rec11002_RelatorioCustomizavelTitulosReceberRecebidos_Controller.js
function Rec11002_RelatorioCustomizavelTitulosReceberRecebidos_Controller (General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    var self = this; 
    
    self.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };
     
    self.openFilterTitulosAReceber = function(widgetTitulosAReceber){
        self.buscaDadosIniciaisFiltro(widgetTitulosAReceber).then(function(){
            var rowSetFilter = {
                //'DTVENBOLREC' : widgetTitulosAReceber.DTVENBOLREC,
                'CDEMPRESA'   : widgetTitulosAReceber.empresa.CDEMPRESA,
                'NMEMPRESA'   : widgetTitulosAReceber.empresa.NMEMPRESA
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAReceber) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAReceber).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAReceber).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralRec = function(){
        return General.callRestEngine(
            'fin#/crud/geral_rec',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        );
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralrec     = self.buscaGeralRec();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralrec, paramestoque]).then(function(response){
            geralrec     = response[0].dataset.geral_rec[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralrec?geralrec.QTDIAANTBOLR?geralrec.QTDIAANTBOLR:0:0;
            var qtdiaapolanr = geralrec?geralrec.QTDIAAPOBOLR?geralrec.QTDIAAPOBOLR:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            //widget.DTVENBOLREC = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    
    self.buildFilter = function (widget, preference) {
        var filtro = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;
        filtro.forEach(function(fil) {
            if (Array.isArray(fil.value)) {
                fil.value = fil.value.join("_");
            }
            switch (fil.name) {
                
                case 'VALORINI':
                    fil.value = filterRow.VALORINI ? filterRow.VALORINI : null;
                    break;
                
                case 'VALORFIN':
                    fil.value = filterRow.VALORFIN ? filterRow.VALORFIN : null;
                    break;
                
                case 'DATENTR':
                    fil.value = filterRow.ENTRADA ? filterRow.ENTRADA.start : null;
                    break;
                
                case 'DATSAID':
                    fil.value = filterRow.ENTRADA ? filterRow.ENTRADA.end : null;
                    break;
                
                case 'DATEMISINI':
                    fil.value = filterRow.EMISSAO ? filterRow.EMISSAO.start : null;
                    break;
                
                case 'DATEMIFIN':
                    fil.value = filterRow.EMISSAO ? filterRow.EMISSAO.end : null;
                    break;
                
                case 'DTATUAVENRECINI':
                    fil.value = filterRow.VENCIMENTO ? filterRow.VENCIMENTO.start : null;
                    break;
                
                case 'DTATUAVENRECFIN':
                    fil.value = filterRow.VENCIMENTO ? filterRow.VENCIMENTO.end : null;
                    break;
                
                case 'IDTPCTRECEBER':
                    fil.value = filterRow.IDTPCTRECEBER ? filterRow.IDTPCTRECEBER : null;
                    break;
                
                case 'CODFILILANC':
                    fil.value = filterRow.CDFILIAL ? filterRow.CDFILIAL : 'T';
                    break;
                
                case 'STATUS':
                    fil.value = filterRow.STATUS ? filterRow.STATUS : 'T';
                    break;
                
                case 'TIPO':
                    fil.value = filterRow.IDSTATUS ? filterRow.IDSTATUS : 'A_E'; 
                    break;
                
                case 'TIPOCONTA':
                    fil.value = filterRow.CDTPCTRECEB ? filterRow.CDTPCTRECEB : 'T';
                    break;
                
                case 'P_CLASSFINA':
                    fil.value = filterRow.CDCLASSFINA ? filterRow.CDCLASSFINA: 'T';
                    break;
                
                case 'IDBLOQNFFIN':
                    if(filterRow.IDBLOQNFFIN == null){
                        filterRow.IDBLOQNFFIN = 'L';
                    }
                    fil.value = filterRow.IDBLOQNFFIN ? filterRow.IDBLOQNFFIN : 'L';
                    break;
                
                case 'P_FORMA':
                    fil.value = filterRow.CDTPBAIXAREC ? filterRow.CDTPBAIXAREC : 'T';
                    break;
                
                case 'P_CCORR':
                    fil.value = filterRow.CDREDUCONT ? filterRow.CDREDUCONT : 'T';
                    break;
                                
                case 'P_TOPERLANC':
                    fil.value = filterRow.CDOPERACTPG ? null : 'T';
                    break;
                
                case 'P_OPERLANC':
                    fil.value = filterRow.CDOPERACTPG ? filterRow.CDOPERACTPG : null;
                    break;
                
                 case 'P_TCDTIPOOPER':
                    fil.value = filterRow.CDTIPOOPER ? null : 'T';
                    break;
                
                case 'P_CDTIPOOPER':
                    fil.value = filterRow.CDTIPOOPER ? filterRow.CDTIPOOPER : null;
                    break;
                    
                case 'IDSISTEMA':
                    fil.value = filterRow.IDSISTEMA ? filterRow.IDSISTEMA : 'T';
                    break;
                    
                case 'P_EMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_"): null;
                    break;
            }
            
        });
        delete filterRow.CDEMPRESA;
        delete filterRow.NMEMPRESA;
        filtro = getDataSourceWithoutIntervalFieldNames(filtro);
    };
    
    self.changeInputValues = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        validateStartEndValues(filterRow);
    };
    
    //Limpa os nomes dos campos invterval do dataSource
    var getDataSourceWithoutIntervalFieldNames = function(filtro){
        for(var i = 0; i < filtro.length; i++) {
            if(filtro[i].name === 'DTEMISSAO' ||
               filtro[i].name === 'DTENTRSAID' ||
               filtro[i].name === 'DTVENBOLREC' ||
               filtro[i].name === 'DTBAIXAREC' ) {
                filtro.splice(i, 1);
                i--;
            }
        } 
        return filtro;
    };
    
    /*
     * Verificação para igualar valores iniciais e finais do filtro, no caso
     * do usuário preencher um e deixar o outro em branco. Isso irá fazer
     * com que a query retorne somente resultados que sejam exatamente os  
     * mesmos valores inseridos em um dos campos.
     */
    var validateStartEndValues = function(filterRow) {
        // Verifica valor líquido inicial e final.
        filterRow.VRFINALBRECINI = filterRow.VRFINALBRECINI || filterRow.VRFINALBRECFIN || null;
        filterRow.VRFINALBRECFIN = filterRow.VRFINALBRECFIN || filterRow.VRFINALBRECINI;
        
        // Verifica nota fiscal inicial e final.
        filterRow.NRNOTAFISCINI = filterRow.NRNOTAFISCINI || filterRow.NRNOTAFISCFIN || null;
        filterRow.NRNOTAFISCFIN = filterRow.NRNOTAFISCFIN || filterRow.NRNOTAFISCINI;
        
        // Verifica valor da baixa inicial e final.
        filterRow.VRBAIXARECINI = filterRow.VRBAIXARECINI || filterRow.VRBAIXARECFIN || null;
        filterRow.VRBAIXARECFIN = filterRow.VRBAIXARECFIN || filterRow.VRBAIXARECINI;
        
        // Verifica número remessa inicial e final.
        // Verifica valor remessa inicial e final.
        filterRow.VRTOTBOLCNABRECINI = filterRow.VRTOTBOLCNABRECINI || filterRow.VRTOTBOLCNABRECFIN || null;
        filterRow.VRTOTBOLCNABRECFIN = filterRow.VRTOTBOLCNABRECFIN || filterRow.VRTOTBOLCNABRECINI;
    };
    
    
    
    this.changeHeaderValues = function(widget){
        /*var codigo; 
        var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "is equal to".concat(codigo).concat(' - ').concat(nmempresa)
        };*/
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        //newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "é igual a."){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
        
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec11002_RelatorioCustomizavelTitulosReceberRecebidos_Controller', Rec11002_RelatorioCustomizavelTitulosReceberRecebidos_Controller);
});

// FILE: ./assets/modules/relatoriosReceber/assets/js/controllers/Rec11004_RelatorioCustomizavelRateiosTitulosReceberRecebido_Controller.js
function Rec11004_RelatorioCustomizavelRateiosTitulosReceberRecebido_Controller(General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    
    var self = this;
    
    self.openFilterTitulosARecebido = function(widgetTitulosAReceber){
        self.buscaDadosIniciaisFiltro(widgetTitulosAReceber).then(function(){
            var rowSetFilter = {
                'DTVENBOLREC' : widgetTitulosAReceber.DTVENBOLREC,
                'CDEMPRESA'   : widgetTitulosAReceber.empresa.CDEMPRESA,
                'NMEMPRESA'   : widgetTitulosAReceber.empresa.NMEMPRESA
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
            
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAReceber) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAReceber).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAReceber).then(function(){
                deferred.resolve();
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa_filial_filioper';
            var filterEmpresa = [
                {"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA},
                {"name": "CDOPERADOR", "operator": "=", "value": getLocalVar('CDOPERADOR')}
            ];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa_filial_filioper[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralRec = function(){
        return General.callRestEngine(
            'fin#/crud/geral_rec',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        ); 
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralrec     = self.buscaGeralRec();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralrec, paramestoque]).then(function(response){
            geralrec     = response[0].dataset.geral_rec[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralrec?geralrec.QTDIAANTBOLP?geralrec.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralrec?geralrec.QTDIAAPOBOLP?geralrec.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLREC = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec11004_RelatorioCustomizavelRateiosTitulosReceberRecebido_Controller', Rec11004_RelatorioCustomizavelRateiosTitulosReceberRecebido_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec00001_TitulosAReceber_Controller.js
var rec00001_TitulosAReceber_Controller = function(ZHPromise, General, templateManager, ScreenService, contabilizacaoController, Rec00001_TitulosAReceber_Service, ZhFilterPreferences, localStorageEngine)
{
    /** Variáveis de Inicialização */
    var self = this;
    
    var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
    
    /** [Início]Título a Receber/Recebidos[Filtro] - Título a Receber/Recebidos[Botão Filtrar] */

    self.voltarParcelamento = function(){
        var titulos = templateManager.container.getWidget('TitulosAReceberGrid');
        ScreenService.goBack().then(function(){
            titulos.reload();
        });
    };
    
    /**
     * Force Activate
     */
    self.forceActivate = function(){
        templateManager.container.getWidget('TitulosAReceberGrid').activate();
    };

    /**
     * Busca Parâmetros de Estoque
     * @return dataProcessamento
     */
    self.buscaEmpresa = function (cdempresa){
        return Rec00001_TitulosAReceber_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Filial
     * @return filialLogada
     */
    self.buscaFilial = function (){
        return Rec00001_TitulosAReceber_Service.buscaFilial().then(function(filial) {
            return filial;
        });
    };

    /**
     * Busca Parâmetros de Estoque
     * @return dataProcessamento
     */
    self.buscaParamEstoque = function (){
        return Rec00001_TitulosAReceber_Service.buscaParamEstoque().then(function(paramestoque) {
            return paramestoque;
        });
    };

    /**
     * Busca Parametrização Geral Receber
     * @return paramGeral
     */
    self.buscaGeralRec = function (){
        return Rec00001_TitulosAReceber_Service.buscaGeralRec().then(function(geralrec) {
            return geralrec;
        });
    };

    /**
     * Busca Parametrização Expedição
     * @return paramGeral
     */
    self.buscaParExpedicao = function (cdfilial){
        return Rec00001_TitulosAReceber_Service.buscaParExpedicao(cdfilial).then(function(parexpedicao) {
            return parexpedicao;
        });
    };

    /**
     * Busca Cliente
     * @return paramGeral
     */
    self.buscaCliente = function (cdcliente){
        return Rec00001_TitulosAReceber_Service.buscaCliente(cdcliente).then(function(cliente) {
            return cliente;
        });
    };

    /**
     * Busca Parametro por Filial - Filial - Empresa Controla Filial
     * @param cdempresa
     * @param cdfilial
     * @return paramfilial_filial_emprcntrfil
     */
    self.buscaParamfilialFilialEmprcntrfil = function (cdempresa, cdfilial){
        return Rec00001_TitulosAReceber_Service.buscaParamfilialFilialEmprcntrfil(cdempresa, cdfilial).then(function(paramfilial_filial_emprcntrfil) {
            return paramfilial_filial_emprcntrfil;
        });
    };

    /**
     * Busca Tipo de Operação
     * @param cdtipooper
     * @return tipooperac
     */
    self.buscaTipooperac = function (cdtipooper){
        return Rec00001_TitulosAReceber_Service.buscaTipooperac(cdtipooper).then(function(tipooperac) {
            return tipooperac;
        });
    };

    /**
     * Busca Série NF - Filial
     * @param cdempresa
     * @param cdfilial
     * @param cdserie
     * @returns serienf_filial
     */
    self.buscaSerienfFilial = function (cdempresa, cdfilial, cdserie){
        return Rec00001_TitulosAReceber_Service.buscaSerienfFilial(cdempresa, cdfilial, cdserie).then(function(serienf_filial) {
            return serienf_filial;
        });
    };

    /**
     * Busca Tipo de Conta a Receber
     * @param cdtpctreceb
     * @returns serienf_filial
     */
    self.buscaTipoctreceber = function (cdtpctreceb){
        return Rec00001_TitulosAReceber_Service.buscaTipoctreceber(cdtpctreceb).then(function(tipoctreceber) {
            return tipoctreceber;
        });
    };
    
    self.openFilterTitulosAReceber = function(widgetTitulosAReceber){
        self.buscaDadosIniciaisFiltro(widgetTitulosAReceber).then(function(){
            var rowSetFilter = {};
            if((sessionStorage.getItem('DASHBOARD_NAME') == 'widgetTitulosReceber') || (sessionStorage.getItem('DASHBOARD_NAME') == 'clientInadimplenteWidget')){
                rowSetFilter = {
                    'DTVENBOLREC': {
                        "start": sessionStorage.getItem('DTVENBOLREC.start'),
                        "end": sessionStorage.getItem('DTVENBOLREC.end'),
                    },    
                    'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                    'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                    'IDSTATBREC': ["A", "E"]
                };
                
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
            } else if((sessionStorage.getItem('DASHBOARD_NAME') == 'boletReceberVenc') || (sessionStorage.getItem('DASHBOARD_NAME') == 'boletReceberAberto') || (sessionStorage.getItem('DASHBOARD_NAME') == 'boletReceberBaixa')){
                rowSetFilter = {
                    'DTVENBOLREC': {
                        "start": sessionStorage.getItem('DTVENBOLREC.start'),
                        "end": sessionStorage.getItem('DTVENBOLREC.end'),
                    },    
                    'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                    'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                    'IDSTATBREC': sessionStorage.getItem('IDSTATBREC').split(",")
                };
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
            } else {
                rowSetFilter = {
                    'DTVENBOLREC': widgetTitulosAReceber.DTVENBOLREC,
                    'CDEMPRESA': widgetTitulosAReceber.empresa.CDEMPRESA,
                    'NMEMPRESA': widgetTitulosAReceber.empresa.NMEMPRESA
                };
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
            }
            
            if((sessionStorage.getItem('DASHBOARD_NAME') == 'rastreioContabil')){
                var cliente = [];
                cliente.push(sessionStorage.getItem('CDCLIENTE'));
                if(sessionStorage.getItem('ORIGEM') == 'BAIXA_REC'){
                    rowSetFilter = {   
                        'DTVENBOLREC': {start : '', end : ''},
                        'DTBAIXAREC': {start : sessionStorage.getItem('DTBAIXAREC'), 
                                         end : sessionStorage.getItem('DTBAIXAREC')},
                        'VRBAXINICIAL': sessionStorage.getItem('VRBAXINICIAL'),
                        'VRBAXFINAL': sessionStorage.getItem('VRBAXFINAL'),
                        'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                        'NMEMPRESA': sessionStorage.getItem('NMEMPRESA')
                    };
                }else{
                    rowSetFilter = {   
                        'DTVENBOLREC': {start : '', end : ''},
                        'NRINICIAL': sessionStorage.getItem('NRNOTAFISCINI'),
                        'NRFINAL': sessionStorage.getItem('NRNOTAFISCFIN'),
                        'CDEMPRESA': sessionStorage.getItem('CDEMPRESA'),
                        'NMEMPRESA': sessionStorage.getItem('NMEMPRESA'),
                        'CDCLIENTE': cliente
                    };
                }
                
                ZhFilterPreferences.setOriginalFilter(widgetTitulosAReceber.id, rowSetFilter);
                ZhFilterPreferences.openFilterWidget(widgetTitulosAReceber, rowSetFilter);
            }
            
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAReceber) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAReceber).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAReceber).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };

    /**
     * Busca Padrão Empresa
     * @param widget
     */
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        self.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            self.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        return deferred.promise;
    };

    /**
     * Busca Padrão Intervalo Vencimento
     * @returns {{start: *, end: *}}
     */
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralrec     = self.buscaGeralRec();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralrec, paramestoque]).then(function(response){
            geralrec     = response[0].dataset.geral_rec[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralrec?geralrec.QTDIAANTLANR?geralrec.QTDIAANTLANR:0:0;
            var qtdiaapolanr = geralrec?geralrec.QTDIAAPOLANR?geralrec.QTDIAAPOLANR:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLREC = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };

    /**
     * Filtro Títulos a ReceberR
     * @param widget
     */
    self.filtroTitulosAReceber = function(widget, preference, currentRowFiltro){

        var vrbaxinicial    = currentRowFiltro.VRBAXINICIAL;
        var vrbaxfinal      = currentRowFiltro.VRBAXFINAL;
        var vrcnminicial    = currentRowFiltro.VRCNMINICIAL;
        var vrcnmfinal      = currentRowFiltro.VRCNMFINAL;
        var nrinicial       = currentRowFiltro.NRINICIAL;
        var nrfinal         = currentRowFiltro.NRFINAL;
        var vrinicial       = currentRowFiltro.VRINICIAL;
        var vrfinal         = currentRowFiltro.VRFINAL;
        var vrbinicial      = currentRowFiltro.VRBINICIAL;
        var vrbfinal        = currentRowFiltro.VRBFINAL;
        var nrcnabrecremini = currentRowFiltro.NRCNABRECREMINI;
        var nrcnabrecremfin = currentRowFiltro.NRCNABRECREMFIN;
        var nrcnabrecretini = currentRowFiltro.NRCNABRECRETINI;
        var nrcnabrecretfin = currentRowFiltro.NRCNABRECRETFIN;
        var cdreducont      = currentRowFiltro.CDREDUCONT;
        var cdnosnumrec     = currentRowFiltro.CDNOSNUMREC;
        var cdfilialnf      = currentRowFiltro.CDFILIALNF;
        var cdtiporece      = currentRowFiltro.CDTIPORECE;
        
        delete preference.FILTER_ROW.CDTIPORECE;
        preference.CUSTOM_FILTER_ROW = preference.FILTER_ROW;

        widget.dataSourceFilter.forEach(function (filtro) {
            switch(filtro.name){
                case 'VRBAXINICIAL': filtro.value = vrbaxinicial < 0 ? null : vrbaxinicial;
                    break;
                case 'VRBAXFINAL': filtro.value = vrbaxfinal < 0 ? null : vrbaxfinal;
                    break;
                case 'VRBAXTOTAL': filtro.value = (((vrbaxinicial <= 0) && (vrbaxfinal <= 0)) || ((vrbaxinicial == null) && (vrbaxfinal == null))) ? 'T' : null;
                    break;
                case 'VRCNMINICIAL': filtro.value = vrcnminicial < 0 ? null : vrcnminicial;
                    break;
                case 'VRCNMFINAL': filtro.value = vrcnmfinal < 0 ? null : vrcnmfinal;
                    break;
                case 'VRCNMTOTAL': filtro.value = (((vrcnminicial <= 0) && (vrcnmfinal <= 0)) || ((vrcnminicial == null) && (vrcnmfinal == null))) ? 'T' : null;
                    break;
                case 'NRINICIAL': filtro.value = nrinicial;
                    break;
                case 'NRFINAL': filtro.value = nrfinal;
                    break;
                case 'NRTOTAL': filtro.value = (nrinicial && nrfinal) ? null : 'T';
                    break;
                case 'VRINICIAL': filtro.value = vrinicial <= 0 ? null : vrinicial;
                    break;
                case 'VRFINAL': filtro.value = vrfinal <= 0 ? null : vrfinal;
                    break;
                case 'VRTOTAL': filtro.value = ((vrinicial > 0) && (vrfinal > 0)) ? null : 'T';
                    break;
                case 'VRBINICIAL': filtro.value = vrbinicial <= 0 ? null : vrbinicial;
                    break;
                case 'VRBFINAL': filtro.value = vrbfinal <= 0 ? null : vrbfinal;
                    break;
                case 'VRBTOTAL': filtro.value = ((vrbinicial > 0) && (vrbfinal > 0)) ? null : 'T';
                    break;
                case 'NRCNABRECREMINI': filtro.value = nrcnabrecremini;
                    break;
                case 'NRCNABRECREMFIN': filtro.value = nrcnabrecremfin;
                    break;
                case 'NRCNABRECREMT': filtro.value = (nrcnabrecremini && nrcnabrecremfin) ? null : 'T';
                    break;
                case 'NRCNABRECRETINI': filtro.value = nrcnabrecretini;
                    break;
                case 'NRCNABRECRETFIN': filtro.value = nrcnabrecretfin;
                    break;
                case 'NRCNABRECRETT': filtro.value = (nrcnabrecretini && nrcnabrecretfin) ? null : 'T';
                    break;
                case 'CDREDUCONT': filtro.value = cdreducont;
                    break;
                case 'TCDREDUCONT': filtro.value = (cdreducont) ? null : 'T';
                    break;
                case 'CDNOSNUMREC': filtro.value = cdnosnumrec;
                    break;
                case 'TCDNOSNUMREC': filtro.value = (cdnosnumrec) ? null : 'T';
                    break;
                case 'CDFILIALNF': filtro.value = (cdfilialnf) ? cdfilialnf.join('_') : null;
                    break;
                case 'CDTIPORECE': filtro.value = (cdtiporece) ? cdtiporece.join('_') : 'T';
                    break;
            }
        });
        if(preference.CUSTOM_FILTER_ROW.NRINICIAL != "" && preference.CUSTOM_FILTER_ROW.NRINICIAL != null){
            var nrInicial = preference.CUSTOM_FILTER_ROW.NRINICIAL;
            nrInicial.padStart(9, '0');
            preference.CUSTOM_FILTER_ROW.NRINICIAL = nrInicial;
        }
        if(preference.CUSTOM_FILTER_ROW.NRFINAL != "" && preference.CUSTOM_FILTER_ROW.NRFINAL != null){
            var nrFinal = preference.CUSTOM_FILTER_ROW.NRFINAL;
            nrFinal.padStart(9, '0');
            preference.CUSTOM_FILTER_ROW.NRFINAL = nrFinal;
        }
    };
    
    /**
     * Seta Data Souce Stripe
     * @param widget
     */
    self.setStripeData = function(widget) {
        var currentRow = widget.appliedFilter.FILTER_ROW;
        currentRow.DTVENBOLREC = (currentRow.DTVENBOLREC) ? currentRow.DTVENBOLREC : {'start': '', 'end': ''};
        var stripeWidget = templateManager.container.getWidget("stripe");

        stripeWidget.currentRow = {};
        stripeWidget.currentRow.NMEMPRESA   = currentRow.CDEMPRESA + ' - ' + currentRow.NMEMPRESA;
        stripeWidget.currentRow.DTVENBOLREC = 'De ' + currentRow.DTVENBOLREC.start + ' até ' + currentRow.DTVENBOLREC.end;
    };
    
    self.setStripeDataFooter = function(status) {
        var widget          = templateManager.container.getWidget("TitulosAReceberGrid");
        var stripeWidget    = templateManager.container.getWidget("stripeFooter");
        var lastSelectedRow = widget.getLastSelectedRow();
        var checkedRows     = widget.getCheckedRows();
        var dstitulo        = '';
        var totalOriginal   = 0;
        var totalAjuste     = 0;
        var totalLiquido    = 0;
       
        checkedRows.forEach(function(item){
            totalOriginal += parseFloat(parseFloat(item.VRBOLREC).toFixed(2));
            totalLiquido  += parseFloat(parseFloat(item.VRFINALBREC).toFixed(2));
        });
        
        totalAjuste = totalLiquido - totalOriginal;
        
        if(((Object.keys(lastSelectedRow).length === 0 && lastSelectedRow.constructor === Object) && (checkedRows.length > 1)) != true){ 
            
            var dsboletrec = lastSelectedRow.DSBOLETREC?" - " +lastSelectedRow.DSBOLETREC:'';
            stripeWidget.getField('DSTITULO').isVisible = true;
            dstitulo = moment(lastSelectedRow.DTVENBOLREC, 'DD/MM/YYYY').format('DD/MM/YYYY') + "  -  " +
                            lastSelectedRow.NMRAZSOCCLIE + "  -  " +
                            self.numberToReal(lastSelectedRow.VRFINALBREC) +
                            dsboletrec;
           
        }
        stripeWidget.currentRow.DSTITULO  = dstitulo;
        if (status){
            stripeWidget.getField('DSTITULO').isVisible = false;
            stripeWidget.currentRow.DSTITULO = '';
        }
        stripeWidget.currentRow.NRTITUTOT = checkedRows.length;
        stripeWidget.currentRow.VRORIGTOT = self.numberToReal(totalOriginal.toFixed(2));
        stripeWidget.currentRow.VRAJUSTOT = self.numberToReal(totalAjuste.toFixed(2));
        stripeWidget.currentRow.VRLIQTOT  = self.numberToReal(totalLiquido.toFixed(2));
    };
    
    self.unsetStripeDataFooter = function() {
        var widget        = templateManager.container.getWidget("TitulosAReceberGrid");
        var stripeWidget  = templateManager.container.getWidget("stripeFooter");
        var checkedrows   = widget.getCheckedRows();
        var dstitulo      = '';
        var totalOriginal = 0;
        var totalAjuste   = 0;
        var totalLiquido  = 0;
        
        if(checkedrows.length > 0){
            checkedrows.forEach(function(item){
                totalOriginal += self.formataValor(item.VRBOLREC);
                totalLiquido  += self.formataValor(item.VRFINALBREC);
            });
            totalAjuste = totalLiquido - totalOriginal;
        }
        
        stripeWidget.getField('DSTITULO').isVisible = false;
        stripeWidget.currentRow.NRTITUTOT = checkedrows.length;
        stripeWidget.currentRow.VRORIGTOT = self.numberToReal(totalOriginal);
        stripeWidget.currentRow.VRAJUSTOT = self.numberToReal(totalAjuste);
        stripeWidget.currentRow.VRLIQTOT  = self.numberToReal(totalLiquido);
    };
    

    /**
     * Formata Número - Ex. 1500 p/ 1.500,00
     */
    self.numberToReal = function(numero) {
        numero = parseFloat(numero).toFixed(2).split('.');
        numero[0] = numero[0].split(/(?=(?:...)*$)/).join('.');
        return numero.join(',');
    };

    /** [Fim] Título a Receber/Recebidos[Filtro] - Título a Receber/Recebidos[Botão Filtrar] */
    /** [Início] Título a Receber/Recebidos */

    /**
     * Novo Título a Receber
     */
    self.novoTituloAReceber = function(){
        if(!templateManager.container.getWidget('TitulosAReceberGrid').appliedFilter.FILTER_ROW.CDEMPRESA) {
            ScreenService.showMessage('Operação bloqueada. Empresa não selecionada no filtro.');
            return;
        }
        
        General.callRestEngine('fin#/crud/geral_rec','FilterData', []).then(function(result){
            if(result.dataset.geral_rec.length == 0){
                ScreenService.showMessage('Operação bloqueada. Favor reveja as parametrizações na tela de Parâmetros de Títulos a Receber.');
                return;
            }
            
            self.validaTipoContaEmpresa().then(function(){
                var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
                widgetDocumentosForm.isVisible = true;
                widgetDocumentosForm.getField('NRLICENCAUSO').isVisible = false;
                widgetDocumentosForm.getField('NMRAZSOCVEN').isVisible  = false;
                widgetDocumentosForm.editing = true;
                
                ScreenService.openSwipe(widgetDocumentosForm).then(function(){
                    templateManager.updateTemplate();
                    self.buscaPadraoTitulo();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                    return;
                });
            }, function (error) {
                ScreenService.showMessage('Operação bloqueada. Empresa selecionada no filtro não possui tipo de conta relacionada na tela Tipo de Título a Receber.');
                return;
            });
            
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            return;
        });

        
    };
    
    self.validaTipoContaEmpresa = function(){
        var deferred = ZHPromise.defer();
        
        var filter = [
            {
                "name": "CDEMPRESA",
                "operator": "=",
                "value": "T"
            }
        ];
        
        General.callRestEngine('fin#/crud/emprtpctrec_empresa','FilterData', filter).then(function(empresas){
            empresas = empresas.dataset.emprtpctrec_empresa;
            
            if(empresas.length > 0){
                var cdempresa = templateManager.container.getWidget('TitulosAReceberGrid').appliedFilter.FILTER_ROW.CDEMPRESA;
                
                filter = [
                    {
                        "name": "CDEMPRESA",
                        "operator": "=",
                        "value": cdempresa
                    }
                ];
                
                General.callRestEngine('fin#/crud/emprtpctrec_empresa','FilterData', filter).then(function(empresas){
                    empresas = empresas.dataset.emprtpctrec_empresa;
                    
                    if(empresas.length > 0)
                       deferred.resolve();
                    else deferred.reject();
                });
                
            }else{
                deferred.resolve();
            }
           
        });
        
        return deferred.promise;
    };
    
    self.alteraAjusteTitulo = function(widget){
        if(widget.currentRow.DSAJUCTREC)
            self.alteraValorAjusteRec(widget);
    };

    /**
     * Altera Valor Desejavel Título
     * @param widget
     */
    self.alteraValorDesejavel = function(widget){
        var vrajbolrec = 0;
    
        if(widget.currentRow.IDAJUCTREC == 'A') {
            if ((self.formataValor(widget.currentRow.VRDESEBREC)) < (self.formataValor(widget.currentRow.VRATUABREC))){
                vrajbolrec = 0;
                widget.currentRow.VRDESEBREC = widget.currentRow.VRATUABREC;
                ScreenService.showMessage('O valor desejável não pode ser menor que o valor atual');
            }
            else if (widget.currentRow.VRDESEBREC == "") {
                vrajbolrec = 0;
                widget.currentRow.VRDESEBREC = widget.currentRow.VRATUABREC;
            } else if ((self.formataValor(widget.currentRow.VRDESEBREC)) >= (self.formataValor(widget.currentRow.VRATUABREC))) {
                vrajbolrec = ((self.formataValor(widget.currentRow.VRDESEBREC) - self.formataValor(widget.currentRow.VRATUABREC)));
            }
            
        }
        else if(widget.currentRow.IDAJUCTREC == 'D') {
            if((self.formataValor(widget.currentRow.VRDESEBREC)) <= (self.formataValor(widget.currentRow.VRATUABREC))){
                vrajbolrec = (self.formataValor(widget.currentRow.VRATUABREC) - self.formataValor(widget.currentRow.VRDESEBREC));
            }
            else if(widget.currentRow.VRDESEBREC && widget.currentRow.VRDESEBREC != "") {
                vrajbolrec = 0;
                widget.currentRow.VRDESEBREC = widget.currentRow.VRATUABREC;
                ScreenService.showMessage('O valor desejável não pode ser maior que o valor atual');
            } else if(widget.currentRow.VRDESEBREC == ""){
                widget.currentRow.VRDESEBREC = widget.currentRow.VRATUABREC;
            }
        }

        vrajbolrec = vrajbolrec > 0 ? vrajbolrec : 0;

        widget.currentRow.VRAJBOLREC = vrajbolrec.toFixed(2);
    };
    
    /**
     * Altera Valor Ajuste Título
     * @param widget
     */
    self.alteraValorAjusteRec = function(widget){
        var vrdesebrec = widget.currentRow.VRATUABREC;

        if(widget.currentRow.IDAJUCTREC == 'A') {
            if(widget.currentRow.VRAJBOLREC && widget.currentRow.VRAJBOLREC != "") {
                vrdesebrec = (self.formataValor(widget.currentRow.VRATUABREC) + self.formataValor(widget.currentRow.VRAJBOLREC));
            } else {
                vrdesebrec = widget.currentRow.VRATUABREC;
            } 
        }
        else if(widget.currentRow.IDAJUCTREC == 'D') {
            if((self.formataValor(widget.currentRow.VRAJBOLREC)) <= (self.formataValor(widget.currentRow.VRATUABREC))){
                vrdesebrec = (self.formataValor(widget.currentRow.VRATUABREC) - self.formataValor(widget.currentRow.VRAJBOLREC));
            } 
            else if(widget.currentRow.VRAJBOLREC && widget.currentRow.VRAJBOLREC != "") {
                vrdesebrec = widget.currentRow.VRATUABREC;
                widget.currentRow.VRAJBOLREC = 0;
                ScreenService.showMessage('O valor do ajuste não pode ser maior que o valor atual');
            }
        }

        vrdesebrec = vrdesebrec > 0 ? vrdesebrec : 0;

        widget.currentRow.VRDESEBREC = parseFloat(vrdesebrec).toFixed(2);
    };
    
    
    /**
     * Busca Padrão Título
     */
    self.buscaPadraoTitulo = function (){
        // Documento
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
        
        self.buscaPadraoFilial(widgetDocumentosForm);
        widgetDocumentosForm.getField('NRORG').setValue(getLocalVar('NRORG'));
        widgetDocumentosForm.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widgetDocumentosForm.getField('IDSTATUS').setValue('A');
        widgetDocumentosForm.getField('DTEMISSAO').setValue(moment().format('DD/MM/YYYY'));
        widgetDocumentosForm.getField('DTENTRSAID').setValue(moment().format('DD/MM/YYYY'));
        widgetDocumentosForm.getField('DTATUAVENREC').setValue(moment().format('DD/MM/YYYY'));
        widgetDocumentosForm.getField('CDCLIENTE').setValue(null);
        widgetDocumentosForm.getField('NMRAZSOCCLIE').setValue(null);
        widgetDocumentosForm.getField('CDTIPOOPER').setValue(null);
        widgetDocumentosForm.getField('NMTIPOOPER').setValue(null);
        widgetDocumentosForm.getField('CDTPCTRECEB').setValue(null);
        widgetDocumentosForm.getField('DSTPCTRECEB').setValue(null);
        widgetDocumentosForm.getField('CDSERIE').setValue(null);
        widgetDocumentosForm.getField('DSSERIE').setValue(null);
        widgetDocumentosForm.getField('NRNOTAFISC').setValue(null);
        widgetDocumentosForm.getField('VRNOTAFISC').setValue(null);
        
        // Parcelamento
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
        
        widgetParcelamentoDocGrid.currentRow = {};
        widgetParcelamentoDocGrid.dataSource.data = [];
    };

    /**
     * Edição Titulo A Receber
     * @param args
     */
    self.edicaoTituloAReceber = function(args){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'E') {
            ScreenService.showMessage('Operação bloqueada. Título emitido não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        General.widgetEdit(args.owner.widget);
    };

    /**
     * Edita Titulo A Receber
     * @param args
     */
    self.editarTituloAReceber = function(args){
        var widget = args.owner.widget;

        if(widget.isValid()){
            Rec00001_TitulosAReceber_Service.editarTituloAReceber(widget.currentRow).then(function () {
                General.changeWidgetAcessibility(widget.widgets, true);
                ScreenService.showMessage('Registro salvo com sucesso.');
                args.owner.widget.view();
                widget.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Deleta Titulos a Receber
     */
    self.deletaTituloReceber = function(){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }


        ScreenService.showCustomDialog('O que deseja excluir', 'question', 'question', [{
            label: 'Vencimento',
            code: self.deletaVencTituloReceber
        }, {
            label: 'Documento',
            code: self.deletaDocTituloReceber
        }, {
            label: 'Cancelar',
            code: ''
        }]);

    };

    /**
     * Deleta Vencimento Titulos a Receber
     */
    self.deletaVencTituloReceber = function(){

        ScreenService.confirmMessage('Deseja realmente excluir o Vencimento?', 'question', function() {

            var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
            
            if(widgetTitulosAReceber.currentRow.NRPARCBOLREC != '00'){
                ScreenService.showMessage('Operação bloqueada. Título parcelado deve ser excluído na aba Parcelamento.');
                return;
            }

            Rec00001_TitulosAReceber_Service.deletaVencTituloReceber(widgetTitulosAReceber.currentRow).then(function () {
                ScreenService.showMessage('Registro excluído com sucesso.');
                ScreenService.goBack();
                widgetTitulosAReceber.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });

        });

    };

    /**
     * Deleta Documento Titulos a Receber
     */
    self.deletaDocTituloReceber = function(){

        ScreenService.confirmMessage('Deseja realmente excluir o Documento?', 'question', function() {

            var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');

            Rec00001_TitulosAReceber_Service.deletaDocTituloReceber(widgetTitulosAReceber.currentRow).then(function () {
                ScreenService.showMessage('Registro excluído com sucesso.');
                ScreenService.goBack();
                widgetTitulosAReceber.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

        });

    };




    /**
     * Cálcula Valor para Percentual
     * @param row
     */
    var valueToPerc = function(valor, valorfinal){
        valor       = parseFloat(valor.replace(',','.'));
        valorfinal  = parseFloat(valorfinal);

        return parseFloat((valor * 100)/valorfinal).toFixed(2);
    };
    
    /**
     * Cálcula Percentual para Valor
     * @param row
     */
    var percToValue = function(percentual, valorfinal){
        if(percentual == ''){
            percentual = '0';
        }
        percentual  = parseFloat(percentual.replace(',','.'));
        valorfinal  = parseFloat(valorfinal);

        return parseFloat((percentual * valorfinal)/100).toFixed(2);
    };
    
    
    
    
    
    
    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMultaTotal = function(row){
        row.VRPEMULBREC = valueToPerc(row.VRMULTABREC, row.VRFINALBREC);
        row.VRMULTACALCBREC = parseFloat(row.VRFINALBREC) + parseFloat(row.VRMULTABREC.replace(',','.'));
    };
    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMoraTotal = function(row){
        row.VRPEMORBREC = valueToPerc(row.VRMORDIABREC, row.VRFINALBREC);
        row.VRMORDIACALCBREC = parseFloat(row.VRFINALBREC) + parseFloat(row.VRMORDIABREC.replace(',','.'));
    };

    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMultaTotal = function(row){
        row.VRMULTABREC = percToValue(row.VRPEMULBREC, row.VRFINALBREC);
        row.VRMULTACALCBREC = parseFloat(row.VRFINALBREC) + parseFloat(row.VRMULTABREC);
    };

    /**
     * Cálcula Valor para Percentual Mora Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMoraTotal = function(row){
        row.VRMORDIABREC = percToValue(row.VRPEMORBREC, row.VRFINALBREC);
        row.VRMORDIACALCBREC = parseFloat(row.VRFINALBREC) + parseFloat(row.VRMORDIABREC);
    };





    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMulta = function(row){
        row.VRPEMULBREC = valueToPerc(row.VRMULTABREC, row.VRFINALBREC);
    };
    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMora = function(row){
        row.VRPEMORBREC = valueToPerc(row.VRMORDIABREC, row.VRFINALBREC);
    };

    /**
     * Cálcula Percentual para Valor Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMulta = function(row){
        row.VRMULTABREC = percToValue(row.VRPEMULBREC, row.VRFINALBREC);
    };

    /**
     * Cálcula Percentual para Valor Mora Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMora = function(row){
        row.VRMORDIABREC = percToValue(row.VRPEMORBREC, row.VRFINALBREC);
    };
    
    
    
    
    
    

    /** [Fim] Título a Receber/Recebidos */
    /** [Início] Ajuste - Título a Receber/Recebidos > Ajuste */

    /**
     * Busca Ajuste Titulo a Receber
     * @param cdfilial
     * @param nrlanbolrec
     */
    self.buscaAjusteTitulo= function(cdfilial, nrlanbolrec){
        return Rec00001_TitulosAReceber_Service.buscaAjusteTitulo(cdfilial, nrlanbolrec).then(function (ajbolrec) {
            return ajbolrec;
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    self.novoAjusteBoleto = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetAjusteGrid = templateManager.container.getWidget('AjusteGrid');
        self.validaAlteracaoTitulo(widgetTitulosAReceber, true)
        .then(function(operacao){
            self.confirmaNovoAjusteBoleto(widgetAjusteGrid, widgetTitulosAReceber, operacao.mantemnossonum);
        })
        .catch(function(erro){
            if (erro.message){
                ScreenService.showMessage(erro.message);
            }
        });
    };

    self.cancelarParcelamentoBoleto = function(args){
        General.backOrCancel(args).then(function(){
            var widget = args.owner.widget;
            
            var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
            //widgetTitulosAReceber.currentRow.NRCNABRECRET = widgetTitulosAReceber.currentRow.NRCNABRECRETANT;
            widgetTitulosAReceber.currentRow.NRCNABRECREM = widgetTitulosAReceber.currentRow.NRCNABRECREMANT;
            widgetTitulosAReceber.currentRow.IDSTATUSEMIT = widgetTitulosAReceber.currentRow.IDSTATUSEMITANT;
            //widgetTitulosAReceber.currentRow.NRCNABRECRETANT = null;
            widgetTitulosAReceber.currentRow.NRCNABRECREMANT = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = null;
            
            if(widget.currentRow.NRPARCBOLREC == '01' && widget.dataSource.data.length == 1)
                widget.reload();
            
        });
    };

    self.confirmaNovoAjusteBoleto = function(widget, widgetTitulosAReceber, mantemnossonum){
        widget.newRow();
        widgetTitulosAReceber.currentRow.MANTEMNOSSONUM = mantemnossonum;
        widget.getField('CDFILIAL').setValue(widgetTitulosAReceber.currentRow.CDFILIAL);
        widget.getField('NRLANBOLREC').setValue(widgetTitulosAReceber.currentRow.NRLANBOLREC);
        widget.getField('VRATUABREC').setValue(widgetTitulosAReceber.currentRow.VRFINALBREC);
        widget.getField('VRAJBOLREC').setValue(0);
        widget.getField('VRDESEBREC').setValue(widgetTitulosAReceber.currentRow.VRFINALBREC);
    };

    self.cancelarAjusteBoleto = function(args){
        General.backOrCancel(args).then(function(){
            var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
            //widgetTitulosAReceber.currentRow.NRCNABRECRET = widgetTitulosAReceber.currentRow.NRCNABRECRETANT;
            if (widgetTitulosAReceber.currentRow.NRCNABRECREMANT !== undefined)
                widgetTitulosAReceber.currentRow.NRCNABRECREM = widgetTitulosAReceber.currentRow.NRCNABRECREMANT;
            if (widgetTitulosAReceber.currentRow.IDSTATUSEMITANT !== undefined)
                widgetTitulosAReceber.currentRow.IDSTATUSEMIT = widgetTitulosAReceber.currentRow.IDSTATUSEMITANT;
            //widgetTitulosAReceber.currentRow.NRCNABRECRETANT = null;
            widgetTitulosAReceber.currentRow.NRCNABRECREMANT = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = null;
        });
    };

    /**
     * Novo Ajuste Boleto
     * @param args
     */
    self.salvarAjusteBoleto = function(args){
        var widgetAjusteGrid = templateManager.container.getWidget('AjusteGrid');
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var dataAjuste = [];

        dataAjuste.push(widgetTitulosAReceber.currentRow);
        dataAjuste.push(widgetAjusteGrid.currentRow);
        Rec00001_TitulosAReceber_Service.salvarAjusteBoleto(dataAjuste)
        .then(function () {
            var idajuctrec  = widgetAjusteGrid.currentRow.IDAJUCTREC;
            var vrajbolrec  = parseFloat(widgetAjusteGrid.currentRow.VRAJBOLREC);
            vrajbolrec      = (idajuctrec == 'A')?vrajbolrec:-(vrajbolrec);
            
            // Atualiza Valor Líquido do Título
            var vrfinalbrec = parseFloat(widgetTitulosAReceber.currentRow.VRFINALBREC);
            vrfinalbrec    += vrajbolrec;
            widgetTitulosAReceber.currentRow.VRFINALBREC = vrfinalbrec;
            
            ScreenService.closeSwipe();
            
            widgetTitulosAReceber.currentRow.NRCNABRECREM = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMIT = 'N';
            widgetTitulosAReceber.currentRow.NRCNABRECREMANT = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = 'N';
            ScreenService.showMessage('Registro salvo com sucesso.');
            widgetAjusteGrid.reload();
            
            
        })
        .catch(function(error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Deleta Ajuste Boleto
     * @param args
     */
    self.deletarAjustes = function(args){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }

        var widget = args.owner.widget;
        if(widget.currentRow.IDORIGAJBREC == '4') {
            ScreenService.showMessage('Operação bloqueada. Ajuste originado de uma compensação não pode ser excluído.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente excluir o Ajuste?', 'question', function() {
            // Valor de Ajuste do Título
            var idajuctrec  = widget.currentRow.IDAJUCTREC;
            idajuctrec      = (idajuctrec == 'A')?'A':'D';
            var vrajbolrec  = parseFloat(widget.currentRow.VRAJBOLREC);
            vrajbolrec      = (idajuctrec == 'A')?-(vrajbolrec):vrajbolrec;

            // Atualiza Valor Líquido do Título
            var vrfinalbrec = parseFloat(widgetTitulosReceber.currentRow.VRFINALBREC);
            vrfinalbrec    += vrajbolrec;
            widgetTitulosReceber.currentRow.VRFINALBREC = vrfinalbrec;

            widget.removeRow();
            ScreenService.goBack();
            //widgetTitulosReceber.reload();
        });
    };

    /** [Fim] Ajuste - Título a Receber/Recebidos > Ajuste */
    /** [Início] Tipo de Recebimento - Título a Receber/Recebidos > Tipo de Recebimento */

    /**
     * Valor Total Tipo de Recebimento
     * @param args
     * @returns {number}
     */
    var valorTotalTipoRecebimento = function(args){
        var vrTotal = 0;

        for(var i in args.dataSource.data){
            vrTotal += parseFloat(args.dataSource.data[i].VRTPRECBOLR) ? parseFloat(args.dataSource.data[i].VRTPRECBOLR): 0;
        }

        return vrTotal;
    };

    /**
     * Altera Campo Tipo de Recebimento
     * @param widget
     */
    self.alteraCampoTipoRecebimento = function(widget){

        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        var idtiporece = widget.getField('IDTIPORECE').value();
        var vrtprecbolr = widgetTitulosAReceber.currentRow.VRFINALBREC - (valorTotalTipoRecebimento(widget) - widget.getField('VRTPRECBOLR').value());

        widget.getField('NRCHEQBOLREC').setValue('');
        widget.getField('CDBANCBOLREC').setValue('');
        widget.getField('CDAGENBOLREC').setValue('');
        widget.getField('CDCONTBOLREC').setValue('');
        widget.getField('NMCARTAO').setValue('');
        widget.getField('NRCARCREBOLR').setValue('');
        widget.getField('VRTPRECBOLR').setValue(vrtprecbolr);

        switch(idtiporece) {
            case '1':
            case '2':
                widget.getField('NRCHEQBOLREC').readOnly = true;
                widget.getField('CDBANCBOLREC').readOnly = false;
                widget.getField('CDAGENBOLREC').readOnly = false;
                widget.getField('CDCONTBOLREC').readOnly = false;
                widget.getField('NMCARTAO').readOnly = false;
                widget.getField('NRCARCREBOLR').readOnly = false;
                break;
            case '3':
                widget.getField('NRCHEQBOLREC').readOnly = false;
                widget.getField('CDBANCBOLREC').readOnly = true;
                widget.getField('CDAGENBOLREC').readOnly = true;
                widget.getField('CDCONTBOLREC').readOnly = true;
                widget.getField('NMCARTAO').readOnly = true;
                widget.getField('NRCARCREBOLR').readOnly = true;
                break;
            default:
                widget.getField('CDBANCBOLREC').readOnly = true;
                widget.getField('NRCHEQBOLREC').readOnly = true;
                widget.getField('CDAGENBOLREC').readOnly = true;
                widget.getField('CDCONTBOLREC').readOnly = true;
                widget.getField('NMCARTAO').readOnly = true;
                widget.getField('NRCARCREBOLR').readOnly = true;
        }

    };

    /**
     * Edição Tipo de Recebimento
     * @param args
     */
    self.edicaoTipoRecebimento = function(args){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        General.widgetEdit(args.owner.widget);
    };

    /**
     * Novo Tipo de Recebimento
     * @param args
     */
    self.novoTipoRecebimento = function(args){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        // Variaveis de inicialização
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var cdfilial    = widgetTitulosAReceber.currentRow.CDFILIAL;
        var nrlanbolrec = widgetTitulosAReceber.currentRow.NRLANBOLREC;

        // Código Sequencial do Tipo de Recebimento
        var nrseqtprecbo = ("00" + (args.dataSource.data.length + 1)).slice(-2);

        // Novo Registro
        args.newRow();
        //ScreenService.openSwipe(args);

        // Valor do Tipo de Recebimento
        var vrtprecbolr = widgetTitulosAReceber.currentRow.VRFINALBREC - valorTotalTipoRecebimento(args);

        // Desabilita Campos
        args.getField('CDBANCBOLREC').readOnly = true;
        args.getField('NRCHEQBOLREC').readOnly = true;
        args.getField('CDAGENBOLREC').readOnly = true;
        args.getField('CDCONTBOLREC').readOnly = true;

        // Default Values
        args.getField('CDFILIAL').setValue(cdfilial);
        args.getField('NRLANBOLREC').setValue(nrlanbolrec);
        args.getField('NRSEQTPRECBO').setValue(nrseqtprecbo);
        args.getField('VRTPRECBOLR').setValue(vrtprecbolr);
    };

    /**
     * Salva Tipo de Recebimento Títulos a Receber/Recebidos
     * @param args
     */
    self.salvarTipoRecebimento = function(args){
        if(args.owner.widget.isValid()) {
            args.owner.widget.view();
            ScreenService.goBack();
            
            self.widgetEmEdicao('TipoRecebimentoGrid', true);
            self.desabilitaAbas('TipoRecebimentoGrid', true);
            
            args.owner.widget.getAction('cancelaTipoRecebimento').isVisible = true;
            args.owner.widget.getAction('back_list').isVisible = false;
            args.owner.widget.getAction('finalize').isVisible = true;
        }
    };

    /**
     * Finaliza Tipo de Recebimento
     * @param args
     */
    self.finalizaTipoRecebimento = function(args){
        var widget = args.owner.widget;
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var error = false;

        for (var i = 0; i < widget.dataSource.data.length; i++)
            for (var j = i+1; j < widget.dataSource.data.length; j++)
                if (widget.dataSource.data[i].CDTIPORECE == widget.dataSource.data[j].CDTIPORECE)
                    error = true;

        if(!error) {
            if (valorTotalTipoRecebimento(widget) != widgetTitulosAReceber.currentRow.VRFINALBREC) {
                ScreenService.showMessage('Operação bloqueada. Total do(s) tipo(s) de recebimento difere do valor total do título.');
            }
            else {
                if (widget.isValid()) {

                    var nrorg = widgetTitulosAReceber.currentRow.NRORG;
                    var cdfilial = widgetTitulosAReceber.currentRow.CDFILIAL;
                    var nrlanbolrec = widgetTitulosAReceber.currentRow.NRLANBOLREC;
                    var dataSource = [{
                        nrorg: nrorg,
                        cdfilial: cdfilial,
                        nrlanbolrec: nrlanbolrec
                    }, widget.dataSource.data];

                    Rec00001_TitulosAReceber_Service.finalizaTipoRecebimento(dataSource).then(function () {
                        ScreenService.showMessage('Registro salvo com sucesso.');
                        widget.reload();

                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }
            }
        }else ScreenService.showMessage('Operação bloqueada. Existem tipos de recebimento duplicados.');
    };

    /**
     * Desabilita Ação Finalizar Tipo Recebimento Títulos a Receber/Recebidos
     * @param args
     */
    self.desabilitaFinalizacaoTipoRecebimento = function(args){
        self.widgetEmEdicao('TipoRecebimentoGrid', false);
        self.desabilitaAbas('TipoRecebimentoGrid', false);
        
        args.owner.getAction('cancelaTipoRecebimento').isVisible = false;
        args.owner.getAction('back_list').isVisible = true;
        args.owner.getAction('finalize').isVisible = false;
    };

    /**
     * Deleta Tipo Recebimento Títulos a Receber/Recebidos
     * @param args
     */
    self.deletaTipoRecebimento = function(args){
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        var widget = args.owner.widget;
        var dataLength = widget.dataSource.data.length;
        var indexRow   = widget.currentRow.__index + 1;
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if (dataLength == indexRow) {
            if (!widget.currentRow.__createdLocal) {
                var nrorg       = widgetTitulosAReceber.currentRow.NRORG;
                var cdfilial    = widgetTitulosAReceber.currentRow.CDFILIAL;
                var nrlanbolrec = widgetTitulosAReceber.currentRow.NRLANBOLREC;
                var dataSource = [{nrorg: nrorg, cdfilial: cdfilial, nrlanbolrec: nrlanbolrec}, widget.dataSource.data];

                ScreenService.confirmMessage('Deseja realmente excluir?', 'question', function () {
                    Rec00001_TitulosAReceber_Service.deletaTipoRecebimento(dataSource).then(function () {
                        ScreenService.goBack();
                        ScreenService.showMessage('Registro excluído com sucesso.');
                        widget.reload();
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                });
            } else General.backOrCancel(args);

        } else ScreenService.showMessage('Operação Bloqueada. Somente é permitido a exclusão do último tipo de recebimento do título.');

    };

    /** [Fim] Tipo de Recebimento - Título a Receber/Recebidos > Tipo de Recebimento */
    /** [Início] Parcelamento - Título a Receber/Recebidos > Parcelamento */

    /**
     * Valor Total Parcelamento Títulos a Receber/Recebidos
     * @param args
     */
    var valorTotalParcelamento = function(args){
        var vrTotal = 0;
        for(var i in args.dataSource.data){
            vrTotal += args.dataSource.data[i].VRFINALBREC?parseFloat(args.dataSource.data[i].VRFINALBREC):0;
        }
        return vrTotal;
    };

    /**
     * Nova Parcela Títulos a Receber/Recebidos
     * @param args
     */
    self.novaParcela = function(args){
        var widgetParcelamento    = args.owner.widget;
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        
        self.validaAlteracaoTitulo(widgetTitulosAReceber, false)
        .then(function(operacao){
            self.confirmaNovaParcela(widgetParcelamento, widgetTitulosAReceber, operacao.mantemnossonum);
        })
        .catch(function(erro){
            if (erro.message){
                ScreenService.showMessage(erro.message);
            }
        });
    };

    /**
     * Confirma Nova Parcela Títulos a Receber/Recebidos
     * @param widget
     * @param widgetTitulosAReceber
     */
    self.confirmaNovaParcela = function(widget, widgetTitulosAReceber){
        self.buscaAjusteTitulo(widgetTitulosAReceber.currentRow.CDFILIAL, widgetTitulosAReceber.currentRow.NRLANBOLREC).then(function (response_ajbolrec){
            var ajbolrec = response_ajbolrec.dataset.ajbolrec[0];
            if(!ajbolrec) {
                widget.currentRow = widget.dataSource.data[0];

                if (widget.currentRow.NRPARCBOLREC == "00") {
                    widget.afterMoveRow();
                    widget.edit();
                    widget.currentRow.NRPARCBOLREC     = "01";
                    widget.currentRow.NRPARCBOLRECVISU = "001";
                } else {

                    var nrparcbolrec     = retornaNumeroParcela(widget.dataSource.data.length + 1);
                    var nrparcbolrecvisu = ("00" + (widget.dataSource.data.length + 1)).slice(-3);
                    var dtvenbolrec      = widget.dataSource.data[widget.dataSource.data.length - 1].DTVENBOLREC;
                    var vrtotalparc      = valorTotalParcelamento(widget);
                    var vrfinalbrec      = widget.currentRow.VRTOTBOLREC - vrtotalparc;

                    widget.newRow();

                    widget.currentRow.NRPARCBOLREC     = nrparcbolrec;
                    widget.currentRow.NRPARCBOLRECVISU = nrparcbolrecvisu;
                    widget.currentRow.DTVENBOLREC      = dtvenbolrec;
                    widget.currentRow.VRFINALBREC      = (vrfinalbrec > 0)?vrfinalbrec:0;
                    widget.currentRow.IDSITUCNABR      = "00";
                }
                
            }else ScreenService.showMessage('Operação Bloqueada. Não é possível parcelar título com ajuste.');
        });
    };
    
    /**
     * Confirma Nova Parcela Títulos a Receber/Recebidos
     * @param widget
     * @param widgetTitulosAReceber
     */
    var retornaNumeroParcela = function(nrparcbolrec){
        
        nrparcbolrec = ("000" + nrparcbolrec).slice(-3);
        
        var unidade = nrparcbolrec.substr(-1, 1);
        var centena = nrparcbolrec.substr(-3, 1);
        
        if(centena != '0'){
            if(nrparcbolrec != '360'){
                var letras = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'X', 'W', 'Y', 'Z'];
            
                var posicao = parseInt(nrparcbolrec.substr(-3, 2)) - 10;
                nrparcbolrec = letras[posicao] + unidade;
            }else nrparcbolrec = 'AA';
        }
            
        return nrparcbolrec.slice(-2);
    };

    /**
     * Edita Parcela Títulos a Receber/Recebidos
     * @param widget
     */
    self.editaParcela = function(widget){
        if(widget.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widget.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }

        var dataLength = widget.dataSource.data.length;
        var indexRow   = widget.currentRow.__index + 1;

        if(widget.currentRow.NRPARCBOLREC != '00'){
            if (!widget.currentRow.__createdLocal) {
                if (dataLength != indexRow){
                    ScreenService.showMessage('Operação Bloqueada. Somente é permitido a editar a última parcela do título.');
                }else General.edit(widget);
            }else{
                widget.currentRow.__is_new     = false;
                widget.getField('DTVENBOLREC').readOnly = ((widget.currentRow.NRPARCBOLREC == '01'));
                General.edit(widget);
            }
        }else ScreenService.showMessage('Operação Bloqueada. Não é possível editar a parcela original.');
    };

    /**
     * Salva Parcela Títulos a Receber/Recebidos
     * @param args
     */
    self.salvaParcela = function(args){
        if(args.owner.widget.isValid()) {
            var widget = args.owner.widget;
            
            widget.view();
            ScreenService.goBack();
            
            widget.getAction('finalizarParcela').isVisible = true;
            widget.getAction('parcelamentoAutomatico').isVisible = false;
            widget.getAction('cancelaParcelamento').isVisible = true;
            widget.getAction('back_list').isVisible = false;
            
            self.widgetEmEdicao('ParcelamentoGrid', true);
            self.desabilitaAbas('ParcelamentoGrid', true);
        }
    };

    /**
     * Finaliza Parcelamento Títulos a Receber/Recebidos
     * @param widget
     */
    self.finalizaParcelamento = function(widgetParcelamento){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        self.validaParcelamentoManual(widgetTitulosAReceber, widgetParcelamento)
        .then(function(){
            self.finalizaParcelamentoTitulos(widgetParcelamento);
        })
        .catch(function(erro){
            if (erro){
                ScreenService.showMessage(erro);
            }
        });
    };

    self.finalizaParcelamentoTitulos = function(widgetParcelamento){
        var widgetTitulosAReceber  = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetParcelamentoGrid = templateManager.container.getWidget('ParcelamentoGrid');
        
        
        var dataParcelamentoAuto = [];
        dataParcelamentoAuto.push(widgetTitulosAReceber.currentRow);
        dataParcelamentoAuto.push(widgetParcelamento.dataSource.data);
        
        for(var i in widgetParcelamento.dataSource.data) {
            if ($.currency.unmaskString(widgetParcelamento.dataSource.data[i].VRFINALBREC) <= 0) {
                ScreenService.showMessage('Operação bloqueada. Existe(m) parcela(s) menor(es) ou igual(is) a zero.');
                return;
            }
        }

        Rec00001_TitulosAReceber_Service.finalizaParcelamentoTitulos(dataParcelamentoAuto).then(function () {
            
            ScreenService.showMessage('Registro salvo com sucesso.');
            General.changeWidgetAcessibility(widgetTitulosAReceber.widgets, false);
            templateManager.updateTemplate();
            
            self.widgetEmEdicao('ParcelamentoGrid', false);
            self.desabilitaAbas('ParcelamentoGrid', false);

            for(var i in widgetParcelamento.dataSource.data)
                if(widgetParcelamento.dataSource.data[i].NRLANBOLREC == widgetTitulosAReceber.currentRow.NRLANBOLREC)
                    widgetTitulosAReceber.currentRow.VRFINALBREC = widgetParcelamento.dataSource.data[i].VRFINALBREC;
            
            widgetTitulosAReceber.currentRow.NRCNABRECREM    = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMIT    = 'N';
            widgetTitulosAReceber.currentRow.NRCNABRECREMANT = null;
            widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = 'N';
            
            ScreenService.closeSwipe();
            
            widgetParcelamentoGrid.reload();
            widgetTitulosAReceber.reload();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    self.geraParcelamentoAutomatico = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetParcelamentoGrid = templateManager.container.getWidget('ParcelamentoGrid');
        self.validaAlteracaoTitulo(widgetTitulosAReceber, true)
        .then(function(operacao){
            if(widgetParcelamentoGrid.dataSource.data.length > 1) {
                ScreenService.showMessage('Operação bloqueada. Título parcelado não pode utilizar o parcelamento automático.');
            }else{
                self.abreParcelamentoAutomatico(widgetTitulosAReceber, operacao);
            }
        })
        .catch(function(erro){
            if (erro.message){
                ScreenService.showMessage(erro.message);
            }
        });
    };

     /**
     * Abre Parcelamento Automatico
     */
    self.abreParcelamentoAutomatico = function(widgetTitulosAReceber, operacao){
        self.buscaAjusteTitulo(widgetTitulosAReceber.currentRow.CDFILIAL, widgetTitulosAReceber.currentRow.NRLANBOLREC).then(function (response_ajbolrec){
            var ajbolrec = response_ajbolrec.dataset.ajbolrec[0];
            if(!ajbolrec) {
                var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoForm');
                widgetParcelamentoAutomatico.isVisible = true;
                widgetParcelamentoAutomatico.editing = true;
                ScreenService.openSwipe(widgetParcelamentoAutomatico).then(function(){
                    widgetTitulosAReceber.currentRow.MANTEMNOSSONUM = operacao.mantemnossonum;
                });
                self.buscaPadraoParcelamentoAutomatico(widgetParcelamentoAutomatico);
            }else ScreenService.showMessage('Operação Bloqueada. Não é possível parcelar título com ajuste.');
        });
    };

    self.validaParcelamentoManual = function(widgetTitulosAReceber, parcelamento){
        var deferred = ZHPromise.defer();
        var currentRowTitulosAReceber = widgetTitulosAReceber.currentRow;
        var mensagemValor = false;
        var valortotparc = 0;
        var parcelamentoPermitido = true;
        var parcelas = parcelamento.dataSource.data;
        var totalTitulo = parcelas[0].VRTOTBOLREC;
        
        for (var i = 0; i < parcelas.length && parcelamentoPermitido; i++){
            if ($.currency.unmaskString(parcelas[i].VRFINALBREC) <= 0) {
                parcelamentoPermitido = false;
                deferred.reject('Operação bloqueada. Existe(m) parcela(s) menor(es) ou igual(is) a zero.');
            }
            valortotparc += self.formataValor(parcelas[i].VRFINALBREC);
        }
        
        totalTitulo  = self.formataValor(totalTitulo);
        valortotparc = self.formataValor(valortotparc);

        if((totalTitulo != valortotparc) && (parcelamentoPermitido) ){
            if((currentRowTitulosAReceber.IDSISTEMA == 'REC') && (currentRowTitulosAReceber.IDSTATBREC == 'A') && (currentRowTitulosAReceber.IDASSOCBOLREC == 'N')){
                var mensagemConfirma =
                    'Valor total das parcelas difere do valor total do título.<br/><br/>' +
                    'Valor total do Título: ' + parseFloat(totalTitulo).toFixed(2) + '<br/>' +
                    'Valor total das Parcelas: ' + parseFloat(valortotparc).toFixed(2) + '<br/><br/>' +
                    'Deseja alterar o valor do documento para prosseguir?';
                ScreenService.confirmMessage(mensagemConfirma, 'question',
                    function () {
                        if(parcelas.length < 2){
                            parcelas[0].NRPARCBOLREC = '00';
                        }
                        deferred.resolve(parcelamentoPermitido);
                    },
                    function() {
                        parcelamentoPermitido = false;
                        deferred.reject(parcelamentoPermitido);
                    }
                );
            }else{
                
                
                parcelamentoPermitido = false;
                deferred.reject(
                    'Operação bloqueada. Valor total das parcelas difere do valor total do título.<br/><br/>' +
                    'Valor total do Título: ' + totalTitulo + '<br/>' +
                    'Valor total das Parcelas: ' + valortotparc
                );
            }
        }else if (parcelamentoPermitido){
            if(parcelas.length < 2){
                ScreenService.showMessage('Operação bloqueada. Continue o parcelamento para finalização.');
                return;
            }
            deferred.resolve(parcelamentoPermitido);
        }
        return deferred.promise;
    };

    self.validaAlteracaoTitulo = function(widgetTitulosAReceber, validaPrevisto){
        var deferred = ZHPromise.defer();
        var boletoEmitido = widgetTitulosAReceber.currentRow.IDSTATUSEMIT;
        var boletoEmRemessa = widgetTitulosAReceber.currentRow.NRCNABRECREM;
        var operacao = {valida: true, mantemnossonum: false};
        if(widgetTitulosAReceber.currentRow.IDSTATBREC == 'B' && operacao.valida) {
            operacao.valida = false;
            operacao.message = 'Operação bloqueada. Título baixado não pode ser alterado.';
            deferred.reject(operacao);
        }
        if(widgetTitulosAReceber.currentRow.IDSTATUS == 'V' && operacao.valida && validaPrevisto == true) {
            operacao.valida = false;
            operacao.message = 'Operação Bloqueada. Título previsto não pode ser alterado.';
            deferred.reject(operacao);
        }
        if (boletoEmRemessa && operacao.valida){
            ScreenService.confirmMessage('O Título será excluído da remessa. Confirma cancelamento?', 'question',
                function () {
                    if (boletoEmitido == 'S' && operacao.valida){
                        ScreenService.confirmMessage('Atenção! Título já foi emitido. A confirmação desta operação fará com que a emissão seja cancelada. Deseja continuar?', 'question',
                            function () {
                                ScreenService.confirmMessage('Deseja manter o(s) nosso(s) números(s) do(s) título(s)?', 'question', function () {
                                    operacao.mantemnossonum = true;
                                    deferred.resolve(operacao);
                                },
                                function(){
                                    operacao.mantemnossonum = false;
                                    deferred.resolve(operacao);
                                });
                                //widgetTitulosAReceber.currentRow.NRCNABRECRETANT = widgetTitulosAReceber.currentRow.NRCNABRECRET;
                                widgetTitulosAReceber.currentRow.NRCNABRECREMANT = widgetTitulosAReceber.currentRow.NRCNABRECREM;
                                widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = widgetTitulosAReceber.currentRow.IDSTATUSEMIT;
                                //widgetTitulosAReceber.currentRow.NRCNABRECRET = null;
                                widgetTitulosAReceber.currentRow.NRCNABRECREM = null;
                                widgetTitulosAReceber.currentRow.IDSTATUSEMIT = 'N';
                            },
                            function () {
                                operacao.valida = false;
                                operacao.message = false;
                                deferred.reject(operacao);
                            }
                        );
                    }else{
                        operacao.mantemnossonum = false;
                        deferred.resolve(operacao);
                    }
                },
                function () {
                    operacao.valida = false;
                    operacao.message = false;
                    deferred.reject(operacao);
                }
            );
        }else if ((boletoEmitido == 'S') && operacao.valida){
            ScreenService.confirmMessage('Atenção! Título já foi emitido. A confirmação desta operação fará com que a emissão seja cancelada. Deseja continuar?', 'question',
                function () {
                    ScreenService.confirmMessage('Deseja manter o(s) nosso(s) números(s) do(s) título(s)?', 'question', function () {
                        operacao.mantemnossonum = true;
                        deferred.resolve(operacao);
                    },
                    function(){
                        operacao.mantemnossonum = false;
                        deferred.resolve(operacao);
                    });
                    widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = widgetTitulosAReceber.currentRow.IDSTATUSEMIT;
                    widgetTitulosAReceber.currentRow.IDSTATUSEMIT = 'N';
                },
                function () {
                    operacao.valida = false;
                    operacao.message = false;
                    deferred.reject(operacao);
                }
            );
        }else{
            deferred.resolve(operacao);
        }

        return deferred.promise;
    };

    /**
     * Desabilita Ação Finalizar Parcela Títulos a Receber/Recebidos
     * @param args
     */
    self.desabilitaFinalizacaoParcela = function(args){
        args.owner.getAction('finalizarParcela').isVisible = false;
        args.owner.getAction('cancelaParcelamento').isVisible = false;
        args.owner.getAction('back_list').isVisible = true;
        args.owner.getAction('parcelamentoAutomatico').isVisible = true;
        self.widgetEmEdicao('ParcelamentoGrid', false);
        self.desabilitaAbas('ParcelamentoGrid', false);
    };

    /**
     * Deleta Parcela Títulos a Receber/Recebidos
     * @param args
     */
    self.deletaParcela = function(args){
        var widget = args.owner.widget;
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if(widget.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widget.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }

        var dataLength = widget.dataSource.data.length;
        var indexRow   = widget.currentRow.__index + 1;

        if(dataLength > 1) {
            if (dataLength == indexRow) {
                if (!widget.currentRow.__createdLocal) {
                    ScreenService.confirmMessage('Deseja realmente excluir?', 'question', function () {
                        Rec00001_TitulosAReceber_Service.deletaParcelamentoTitulos(widget.dataSource.data).then(function () {
                            ScreenService.goBack();
                            ScreenService.showMessage('Registro excluído com sucesso.');

                            widget.reload();
                            widgetTitulosAReceber.reload();

                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                } else General.backOrCancel(args);
            } else ScreenService.showMessage('Operação Bloqueada. Somente é permitido a exclusão da última parcela do título.');
        }else{
            if(widget.currentRow.NRPARCBOLREC == '01') {
                ScreenService.goBack();
                widget.reload();
            }else ScreenService.showMessage('Operação Bloqueada. Não é possível excluir a parcela original.');
        }
    };

    /** [Fim] Parcelamento - Título a Receber/Recebidos > Parcelamento */
    /** [Início] Documento[Grade] - Título a Receber/Recebidos > Documento */

    /**
     * Busca Parametrização Tipo de Conta
     * @param args
     * @param widget_name
     */
    self.changeTipoConta = function(args, widget_name) {
        var widgetDocumentos = templateManager.container.getWidget(widget_name);

        widgetDocumentos.currentRow.DSCONTAREC = widgetDocumentos.currentRow.DSTPCTRECEB;

        args.owner.field.widget.container.getWidget(widget_name).getField('DSCONTAREC').reload();
    };

    /** [Fim] Documento[Grade] - Título a Receber/Recebidos > Documento */
    /** [Início] Documento[Formulário] - Título a Receber/Recebidos[Botão Adicionar] */

    /**
     * Busca Parametrização Filial
     */
    self.changeFilial = function() {
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
    
        if(widgetDocumentosForm.getField('CDFILIEMIT').value()){
    
            // Tamanho NF
            var nrtamnumnfut = (widgetDocumentosForm.getField('NRTAMNUMNFUT').value() > 0) ? widgetDocumentosForm.getField('NRTAMNUMNFUT').value() : 7;
            widgetDocumentosForm.getField('NRNOTAFISC').mask.params.maxlength = nrtamnumnfut;
            widgetDocumentosForm.getField('NRNOTAFISC').maxlength = nrtamnumnfut;

            // Gera Novo Código NF
            var currentRow = {
                NRORG      : getLocalVar('NRORG'),
                CDCONTADOR : 'NRNOTAFISCREC',
                NRTAMNUMNFUT: nrtamnumnfut
            };
            Rec00001_TitulosAReceber_Service.geraNovoCodigo(currentRow).then(function (novocodigo) {
                var nrnotafisc = novocodigo.dataset.geraNovoCodigo.novocodigo.slice(-nrtamnumnfut);
                widgetDocumentosForm.getField('NRNOTAFISC').setValue(nrnotafisc);
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
    
            // Serie NF
            Rec00001_TitulosAReceber_Service.buscaPadraoSerieNF(getLocalVar('CDFILIAL'), widgetDocumentosForm.currentRow.CDFILIEMIT).then(function (response_serienf) {
                var serienf = response_serienf.dataset.notafiscal_serie[0];
    
                if(serienf != undefined){
                    widgetDocumentosForm.currentRow.CDSERIE = serienf.CDSERIE;
                    widgetDocumentosForm.currentRow.DSSERIE = serienf.DSSERIE;
                }else{
                    widgetDocumentosForm.currentRow.CDSERIE = null;
                    widgetDocumentosForm.currentRow.DSSERIE = null;
                }
    
                widgetDocumentosForm.redraw();
                widgetDocumentosForm.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
    
            // Habilita Vendedor
            self.habilitaVendedorDocumento(widgetDocumentosForm);
        }else{
            widgetDocumentosForm.currentRow.CDSERIE = '';
            widgetDocumentosForm.currentRow.DSSERIE = '';
            widgetDocumentosForm.getField('NRNOTAFISC').setValue('');
        }
    };

    /**
     * Busca Parametrização Cliente
     * @param args
     */
    self.changeCliente = function(args) {
        var widgetDocumentosForm = args.owner.field.widget.container.getWidget('DocumentosForm');

        widgetDocumentosForm.getField('CDTPCTRECEB').setValue('');
        widgetDocumentosForm.getField('DSTPCTRECEB').setValue('');
        widgetDocumentosForm.getField('DSCONTAREC').setValue('');

        var cdcliente = widgetDocumentosForm.getField('CDCLIENTE').value();

        if(cdcliente){

            Rec00001_TitulosAReceber_Service.buscaCliente(cdcliente).then(function (paramcliente) {
                var cliente     = paramcliente.dataset.cliente[0];
                var cdtpctreceb = cliente.CDTPCTRECEB;

                if(cdtpctreceb){
                    self.buscaTipoctreceber(cdtpctreceb).then(function (paramtipoctreceber) {
                        var tipoctreceber = paramtipoctreceber.dataset.tipoctreceber[0];

                        widgetDocumentosForm.getField('CDTPCTRECEB').setValue((tipoctreceber)?tipoctreceber.CDTPCTRECEB:'');
                        widgetDocumentosForm.getField('DSTPCTRECEB').setValue((tipoctreceber)?tipoctreceber.DSTPCTRECEB:'');
                        widgetDocumentosForm.getField('CDCLASSFINA').setValue((tipoctreceber)?tipoctreceber.CDCLASSFINA:'');
                        widgetDocumentosForm.getField('DSCONTAREC' ).setValue((tipoctreceber)?tipoctreceber.DSTPCTRECEB:'');
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }

            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });

            self.habilitaLicensaUsoDocumento(widgetDocumentosForm);

        }
    };

    /**
     * Gera Títulos a Receber
     * @param args
     */
    self.geraTituloAReceber = function(widget){
        var widgetDocumentosForm      = templateManager.container.getWidget('DocumentosForm');
        var widgetTitulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
    	var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
    	var widgetAnexoDocGrid        = templateManager.container.getWidget('AnexoDocGrid');
    
        if(widgetDocumentosForm.isValid()){
            if(widgetParcelamentoDocGrid.isValid()){
                var filter = [
                    {
                        "name": "CDCLIENTE",
                        "operator": "=",
                        "value": widgetDocumentosForm.currentRow.CDCLIENTE
                    },
                    {
                        "name": "CDSERIE",
                        "operator": "=",
                        "value": widgetDocumentosForm.currentRow.CDSERIE
                    },
                    {
                        "name": "NRNOTAFISC",
                        "operator": "=",
                        "value": widgetDocumentosForm.currentRow.NRNOTAFISC
                    },
                    {
                        "name": "IDSISTEMA",
                        "operator": "=",
                        "value": 'REC'
                    }
                ];
    
                General.callRestEngine('fin#/crud/notafiscal', 'FilterData', filter).then(function (notafiscal) {
                    var nota = notafiscal.dataset.notafiscal;
                    
                    if(nota.length > 0){
                        ScreenService.showMessage('Operação bloqueada. Documento já lançado.');
                        return;
                    }

                    if((moment(widgetDocumentosForm.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetDocumentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY'))){
                        ScreenService.showMessage('Operação bloqueada. Data de emissão deve ser menor ou igual a data de entrada.');
                        return;
                    }
            
                    if((moment(widgetDocumentosForm.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetParcelamentoDocGrid.dataSource.data[0].DTVENBOLREC, 'DD/MM/YYYY'))){
                        ScreenService.showMessage('Operação bloqueada. Data de emissão deve ser menor ou igual a data de vencimento.');
                        return;
                    }
                    
                    var vrtotparc = 0;
                    
                    widgetParcelamentoDocGrid.dataSource.data.forEach(function(data){
                        vrtotparc += self.formataValor(data.VRFINALBREC);
                    });
            
            		self.validaDataVencimento(widgetParcelamentoDocGrid).then(function () {
                    
                        widgetDocumentosForm.currentRow.DTVENBOLREC = widgetParcelamentoDocGrid.dataSource.data[0];
                        
                        if(self.formataValor(widgetDocumentosForm.currentRow.VRNOTAFISC) != self.formataValor(vrtotparc)){
                            ScreenService.showMessage('Operação bloqueada. Valor do documento difere do somatório das parcelas.');
                            return;
                        }
            
                        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulosAReceberGrid.appliedFilter.FILTER_ROW.CDEMPRESA).then(function (response) {
                            var contopectbl  = response.dataset.contopectbl[0];
                            
                            if(!contopectbl){
                                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                                return;
                            }
            
                            var dtinicctbl  = contopectbl.DTINICCTBL;
                            var dtfinacctbl = contopectbl.DTFINACCTBL;
            
                            if(dtinicctbl && dtfinacctbl) {
                                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetDocumentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetDocumentosForm.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                                    ScreenService.showMessage('Operação Bloqueada. Data do entrada fora do período financeiro.');
                                    return;
                                }
                            
                                var dataParc = widgetParcelamentoDocGrid.dataSource.data;
                                
                                dataParc.forEach(function (data){
                                    if(data.__leftActions != undefined)
                                        delete data.__leftActions;
                                    if(data.__rightActions != undefined)
                                        delete data.__rightActions;
                                });
                                
                                var dataset = [
                                    widgetDocumentosForm.currentRow,
                                    dataParc,
                                    widgetAnexoDocGrid.dataSource.data
                                ];
                
                                Rec00001_TitulosAReceber_Service.geraTituloAReceberDoc(dataset).then(function (nrlanbolrec) {
                                    nrlanbolrec = nrlanbolrec.dataset.geraTituloAReceberDoc.nrlanbolrec;
                                    ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                                    ScreenService.closeSwipe();
                
                                    widgetDocumentosForm.isVisible = false;
                                    
                                    self.forceActivate();
                                    widgetTitulosAReceberGrid.reload().then(function(){
                                        var data = widgetTitulosAReceberGrid.dataSource.data.filter(function(data){
                                        	return ((data.CDFILIAL+data.NRLANBOLREC) == nrlanbolrec);
                                        });
                
                                        widgetTitulosAReceberGrid.currentRow = data[0]?data[0]:{};
                                    });
                                    self.forceActivate();
                                }, function (error) {
                                    if (contabilizacaoController.checkErrosContabilizacao(error))
                                        ScreenService.showMessage(error.data.error);
                                });
                            }
                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    }).catch(function(erro){
                        if (erro){
                            ScreenService.showMessage(erro);
                        }
                    });
                }).catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }else ScreenService.showMessage('Operação bloqueada. Preencha o(s) campo(s) de vencimento(s).');
        }
    };

    /**
     * Associar Títulos a Receber
     * @param widget
     */
    self.associarTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        var valortotal = 0;

        if(checkedrows.length > 1) {
            for(var i in checkedrows) {
                if((checkedrows[i].IDSTATBREC == 'B') || (checkedrows[i].IDSTATBREC == 'P')){
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) não pode(m) ser alterado(s).');
                    return;
                }
                if (checkedrows[i].IDSITUCNABR != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
                if (checkedrows[i].IDSITUCNABR == 'E') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) emitido(s) não pode(m) ser associados(s).');
                    return;
                }
                if (checkedrows[i].IDSTATUS == 'V') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) previsto(s) não pode(m) ser alterado(s).');
                    return;
                }

                valortotal += parseFloat(checkedrows[i].VRFINALBREC);
            }

            Rec00001_TitulosAReceber_Service.validaAssociarTituloAReceber(checkedrows).then(function () {
                var widgetAssociar = templateManager.container.getWidget('AssociarForm');
                widgetAssociar.isVisible = true;
                ScreenService.openPopup(widgetAssociar);
                widgetAssociar.getField('VRFINALBREC').setValue(valortotal.toFixed(2));
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }else ScreenService.showMessage('Operação bloqueada. Marque no mínimo dois títulos para associação.');
    };

    /**
     * Confirmar Associacao Títulos a Receber/Recebidos
     * @param args
     */
    self.confirmarAssociacaoTitulo = function(args){
        var widget = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        var params = {nrorg: getLocalVar('NRORG'), cdfilial: getLocalVar('CDFILIAL'), dtvenbolrec: widget.currentRow.DTVENBOLREC, cdreducont: widget.currentRow.CDREDUCONT, cdtpbaixarec: widget.currentRow.CDTPBAIXAREC};
        var checkedrows = [params, widgetTitulos.getCheckedRows()];

        if(widget.isValid()) {
            Rec00001_TitulosAReceber_Service.confirmarAssociacaoTitulo(checkedrows).then(function () {
                ScreenService.showMessage('Títulos associados com sucesso.');
                widgetTitulos.unSelectAll();
                self.unsetStripeDataFooter();

                widget.currentRow.DTVENBOLREC  = '';
                widget.currentRow.DSCONTCORR   = '';
                widget.currentRow.NMTPBAIXAREC = '';

                ScreenService.closePopup(widget);
                ScreenService.closePopup();
                widget.isVisible = false;

                widgetTitulos.reload();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Cancela Associação Títulos a Receber/Recebidos
     * @param widget
     */
    self.cancelaAssociacaoTitulo = function(widget){
        var params = {nrorg: getLocalVar('NRORG')};
        var checkedrows = widget.getCheckedRows();
        if(checkedrows.length > 0) {
            for(var i in checkedrows) {
                if (checkedrows[i].IDSTATBREC == 'B') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) baixado(s) não pode(m) ser alterado(s).');
                    return;
                }
                if (checkedrows[i].IDSITUCNABR != '00') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                    return;
                }
                if (checkedrows[i].IDSITUCNABR == 'E') {
                    ScreenService.showMessage('Operação bloqueada. Título(s) emitido(s) não pode(m) ser associados(s).');
                    return;
                }
            }

            ScreenService.confirmMessage('Deseja realmente desassociar o(s) título(s)?', 'question', function () {
                
                checkedrows = [params, widget.getCheckedRows()];
                Rec00001_TitulosAReceber_Service.cancelaAssociarTituloAReceber(checkedrows).then(function() {
                    ScreenService.showMessage('Título(s) desassociado(s) com sucesso.');
                    widget.unSelectAll();
                    self.unsetStripeDataFooter();
                    
                    ScreenService.closePopup();
                    widget.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            });
        }else ScreenService.showMessage('Marque o(s) Título(s) a Receber para Desassociação.');
    };

    /**
     * Prorrogar Títulos a Receber/Recebidos
     * @param widget
     */
    self.prorrogarTitulo = function(widgetTitulosAReceber){
        var widgetProrrogar = templateManager.container.getWidget('ProrrogarForm');
        var checkedrows     = widgetTitulosAReceber.getCheckedRows();

        Rec00001_TitulosAReceber_Service.validaProrrogarTituloAReceber(checkedrows).then(function (callback) {
            var cnab = callback.dataset.validaProrrogar.cnab;
                
            if(!cnab) {
                widgetProrrogar.isVisible = true;
                ScreenService.openPopup(widgetProrrogar);
                widgetProrrogar.getField('IDTPPROR').setValue('F');
                self.alteraTipoProrrogacao();
            }else{
                ScreenService.confirmMessage('Existem títulos marcados enviados via CNAB. Ao confirmar a operação os mesmos serão excluídos da remessa. Deseja realmente continuar?', 'question', function() {
                    widgetProrrogar.isVisible = true;
                    ScreenService.openPopup(widgetProrrogar);
                    widgetProrrogar.getField('IDTPPROR').setValue('F');
                    self.alteraTipoProrrogacao();
                });
            }
            
            templateManager.updateTemplate();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Confirmar Prorrogacao Títulos a Receber/Recebidos
     */
    self.confirmarProrrogacaoTitulo = function(){
        var widgetProrrogar = templateManager.container.getWidget('ProrrogarForm');
        var widgetTitulos   = templateManager.container.getWidget('TitulosAReceberGrid');
        
        if(widgetProrrogar.isValid()) {
            var checkedrows = [widgetProrrogar.currentRow, widgetTitulos.getCheckedRows()];
        
            Rec00001_TitulosAReceber_Service.confirmarProrrogacaoTitulo(checkedrows).then(function () {
                ScreenService.showMessage('Título(s) prorrogado(s) com sucesso.');

                widgetProrrogar.currentRow.DTVENBOLREC  = '';
                widgetProrrogar.currentRow.DSMOTIPROR   = '';
                widgetProrrogar.isVisible = false;

                ScreenService.closePopup(widgetProrrogar);
                ScreenService.closePopup();
                
                widgetTitulos.reload();
                widgetTitulos.dataSource.unSelectAll();
                
                self.unsetStripeDataFooter();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Emitir Títulos a Receber/Recebidos
     * @param args
     * @param widget
     */
    self.emitirTitulo = function(args, widget){
        var params = {nrorg: getLocalVar('NRORG')};
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {
            ScreenService.confirmMessage('Deseja realmente emitir o(s) título(s)?', 'question', function () {

                for(var z in checkedrows) {
                    if (!checkedrows[z].CDREDUCONT) {
                        ScreenService.showMessage('Operação bloqueada. Título(s) está(ão) sem conta cobrança.');
                        return;
                    }
                }

                checkedrows = [params, widget.getCheckedRows()];

                var emitido = false;

                for (var i = 0; i < checkedrows[1].length; i++) {
                    if(checkedrows[1][i].IDSTATBREC == 'E') {
                        emitido = true;
                        i = checkedrows.length;
                    }
                }

                if(emitido){
                    ScreenService.confirmMessage('Existe(m) título(s) emitido(s). Deseja reemití-lo(s)?', 'question', function () {
                        self.confirmaEmissaoTitulo(widget, checkedrows);
                    });
                }else {
                    self.confirmaEmissaoTitulo(widget, checkedrows);
                }
            });
        }else ScreenService.showMessage('Marque o(s) Título(s) a Receber para emissão.');
    };

    /**
     * Confirma Emissão Título
     * @param widget
     * @param checkedrows
     */
    self.confirmaEmissaoTitulo = function(widget, checkedrows){
        Rec00001_TitulosAReceber_Service.emitirTituloAReceber(checkedrows).then(function (relatorios) {
            relatorios = relatorios.dataset.EmitirTituloAReceber.relatorios;
            for (var i = 0; i < relatorios.length; i++) {
                var relatorio = relatorios[i];
                var report = relatorio.RELATORIO;
                var routes = 'fin_contaReceber#/' + report;
                var row = {
                    CDFILIAL: relatorio.CDFILIAL,
                    NRLANBOLREC: relatorio.NRLANBOLREC,
                    NRORG: relatorio.NRORG
                };
                General.openReport(routes, report, row);
                ScreenService.closePopup();
                ScreenService.showMessage('Título(s) emitido(s) com sucesso.');
                widget.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
                widget.reload();
            }
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Cancela Emissão Títulos a Receber/Recebidos
     * @param widget
     */
    self.cancelaEmissaoTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        if (checkedrows.length > 0) {

            ScreenService.confirmMessage('Deseja realmente cancelar a(s) emissão(ões) do(s) título(s)?', 'question', function () {
                // Verifica situação CNAB
                if(self.verificaSituacaoCnab(widget.getCheckedRows())) {
                    ScreenService.confirmMessage('Existe(m) título(s) marcado(s) enviado(s) via CNAB. Ao confirmar a operação o(s) mesmo(s) será(ão) excluído(s) da(s) remessa(s). Confirma o cancelamento?', 'question', function () {

                        ScreenService.confirmMessage('Deseja manter o(s) nosso(s) números(s) do(s) título(s)?', 'question', function () {
                            var params  = {nrorg: getLocalVar('NRORG'), mantemnossonum: true};
                            checkedrows = [params, widget.getCheckedRows()];

                            // Cancela Emissão Título
                            Rec00001_TitulosAReceber_Service.cancelaEmissaoTituloAReceber(checkedrows).then(function () {
                                ScreenService.showMessage('Título(s) emitido(s) cancelado(s) com sucesso.');
                                widget.dataSource.unSelectAll();
                                self.unsetStripeDataFooter();
                                widget.reload();
                            }, function (error) {
                                ScreenService.showMessage(error.data.error);
                            });

                        }, function (){
                            var params  = {nrorg: getLocalVar('NRORG'), mantemnossonum: false};
                            checkedrows = [params, widget.getCheckedRows()];

                            // Cancela Emissão Título
                            Rec00001_TitulosAReceber_Service.cancelaEmissaoTituloAReceber(checkedrows).then(function () {
                                ScreenService.showMessage('Título(s) emitido(s) cancelado(s) com sucesso.');
                                widget.dataSource.unSelectAll();
                                self.unsetStripeDataFooter();
                                widget.reload();
                            }, function (error) {
                                ScreenService.showMessage(error.data.error);
                            });
                        });

                    });
                }else{
                    ScreenService.confirmMessage('Deseja manter o(s) nosso(s) números(s) do(s) título(s)?', 'question', function () {
                        var params  = {nrorg: getLocalVar('NRORG'), mantemnossonum: true};
                        checkedrows = [params, widget.getCheckedRows()];

                        // Cancela Emissão Título
                        Rec00001_TitulosAReceber_Service.cancelaEmissaoTituloAReceber(checkedrows).then(function () {
                            ScreenService.showMessage('Título(s) emitido(s) cancelado(s) com sucesso.');
                            widget.dataSource.unSelectAll();
                            self.unsetStripeDataFooter();
                            widget.reload();
                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });

                    }, function (){
                        var params  = {nrorg: getLocalVar('NRORG'), mantemnossonum: false};
                        checkedrows = [params, widget.getCheckedRows()];

                        // Cancela Emissão Título
                        Rec00001_TitulosAReceber_Service.cancelaEmissaoTituloAReceber(checkedrows).then(function () {
                            ScreenService.showMessage('Título(s) emitido(s) cancelado(s) com sucesso.');
                            widget.dataSource.unSelectAll();
                            self.unsetStripeDataFooter();
                            widget.reload();
                        }, function (error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                }
                ScreenService.closePopup();
            });

        } else ScreenService.showMessage('Marque o(s) Título(s) a Receber para Cancelar Emissão.');
    };

    /**
     * Busca Dados CNAB Títulos a Receber/Recebidos
     * @param row
     */
    self.buscaDadosCnab = function(widgetName){
        var widget = templateManager.container.getWidget(widgetName);
        
        if(widget.currentRow.CDREDUCONT){
            if(widget.currentRow.CDINS1COBBOL)
                Rec00001_TitulosAReceber_Service.buscaInstCobranca(widget.currentRow.CDBANCO, widget.currentRow.CDINS1COBBOL).then(function(instrucaocobranca) {
                    var instcobranca = instrucaocobranca.dataset.instcobranca[0];
                    if(instcobranca != undefined)
                        //widget.getField('NMINST1COBR').setValue(instcobranca.NMINSTCOBR);
                        widget.currentRow.NMINST1COBR = instcobranca.NMINSTCOBR;
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            else{
                //widget.getField('NMINST1COBR').setValue();
                widget.currentRow.NMINST1COBR = null;
            }
            
            if(widget.currentRow.CDINS2COBBOL)
                Rec00001_TitulosAReceber_Service.buscaInstCobranca(widget.currentRow.CDBANCO, widget.currentRow.CDINS2COBBOL).then(function(instrucaocobranca) {
                    var instcobranca = instrucaocobranca.dataset.instcobranca[0];
                    if(instcobranca != undefined)
                        //widget.getField('NMINST2COBR').setValue(instcobranca.NMINSTCOBR);
                        widget.currentRow.NMINST2COBR = instcobranca.NMINSTCOBR;
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            else{
                //widget.getField('NMINST2COBR').setValue();
                widget.currentRow.NMINST2COBR = null;
            }
        }else{
            //widget.getField('DSCONTCORR').setValue(null);
            //widget.getField('IDFORMCADBREC').setValue('1');
            //widget.getField('NMINST1COBR').setValue(null);
            //widget.getField('NMINST2COBR').setValue(null);
            
            widget.currentRow.DSCONTCORR     = null;
            widget.currentRow.IDFORMCADBREC  = '1';
            widget.currentRow.NMINST1COBR    = null;
            widget.currentRow.NMINST2COBR    = null;
            
        }
    };

    /**
     * Verifica Situação CNAB Títulos a Receber/Recebidos
     * @param checkedrows
     */
    self.verificaSituacaoCnab = function(checkedrows){
        Rec00001_TitulosAReceber_Service.verificaCnabTituloAReceber(checkedrows).then(function (datasource) {
            return datasource.dataset.verificaCnabTituloAReceber.exibiconf[0];
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Busca Padrão Baixa Título
     * @param widgetBaix
     * @param widgetTitulosBaixa
     * @param titulosEmAberto
     * @returns {{start: *, end: *}}
     */
    self.buscaPadraoBaixaTitulo = function (widgetBaixa, widgetTitulosBaixa, titulosEmAberto){
        var deferred = ZHPromise.defer();

        var emissaomax = null;
        var emissaoatu = null;
        var vrbaixarec  = 0;
        
        for(var i in titulosEmAberto){
            emissaoatu = moment(titulosEmAberto[i].DTEMISSAO, "DD/MM/YYYY HH:mm:ss");
            emissaomax = emissaomax ? emissaoatu > emissaomax ? emissaoatu : emissaomax : emissaoatu;
            vrbaixarec += parseFloat(titulosEmAberto[i].VRFINALBREC);
        }

        widgetBaixa.getField('CDEMPRESA').setValue(templateManager.container.getWidget('TitulosAReceberGrid').appliedFilter.FILTER_ROW.CDEMPRESA||null);
        widgetBaixa.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widgetBaixa.getField('DTLANCBAIXAR').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixa.getField('DTBAIXAREC').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixa.getField('IDUNICABAIXA').setValue('S');
        widgetBaixa.getField('IDSTATUSBREC').setValue('E');
        widgetBaixa.getField('DSCONTCORR').setValue('');
        widgetBaixa.getField('CDBANCO').setValue('');
        widgetBaixa.getField('CDAGENCIA').setValue('');
        widgetBaixa.getField('CDCONTCORR').setValue('');
        widgetBaixa.getField('NMTPBAIXAREC').setValue('');
        widgetBaixa.getField('DSBAIXAREC').setValue('');
        widgetBaixa.getField('VRBAIXAREC').setValue(vrbaixarec.toFixed(2));
        widgetBaixa.getField('DTEMISSAO').setValue(emissaomax.format('DD/MM/YYYY'));
        
        localStorageEngine.setDataSource("fin#/baixa_titulos", titulosEmAberto).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    /**
     * Busca Padrao Baixa Titulo Descontado
     * @param widgetBaixaDescontado
     */
    self.buscaPadraoBaixaTituloDescontado = function (widgetBaixaDescontado){
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid').getCheckedRows();
        var vrbaixarec  = 0;

        for(var i in widgetTitulos)
            vrbaixarec += parseFloat(widgetTitulos[i].VRFINALBREC);

        widgetBaixaDescontado.getField('CDEMPRESA').setValue(templateManager.container.getWidget('TitulosAReceberGrid').appliedFilter.FILTER_ROW.CDEMPRESA||null);
        widgetBaixaDescontado.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widgetBaixaDescontado.getField('DTLANCBAIXAR').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixaDescontado.getField('DTBAIXAREC').setValue(moment().format('DD/MM/YYYY'));
        widgetBaixaDescontado.getField('IDSTATUSBREC').setValue('E');
        widgetBaixaDescontado.getField('CDREDUCONT').setValue(widgetTitulos[0].CDREDUCONT);
        widgetBaixaDescontado.getField('CDBANCO').setValue(widgetTitulos[0].CDBANCO);
        widgetBaixaDescontado.getField('CDAGENCIA').setValue(widgetTitulos[0].CDAGENCIA);
        widgetBaixaDescontado.getField('DSCONTCORR').setValue(widgetTitulos[0].DSCONTCORR);
        widgetBaixaDescontado.getField('NMTPBAIXAREC').setValue('');
        widgetBaixaDescontado.getField('VRBAIXAREC').setValue(vrbaixarec.toFixed(2));
    };

    /**
     * Confirmar Baixa Titulo a Receber
     * @param args
     */
    self.confirmaBaixaTitulo = function(args){
        var widget = templateManager.container.getWidget('NewBaixaTabbed');
        
        if(widget.isValid()) {
            var dtemissao = moment(widget.currentRow.DTEMISSAO, "DD/MM/YYYY HH:mm:ss");
            var dtbaixarec = moment(widget.currentRow.DTBAIXAREC, "DD/MM/YYYY HH:mm:ss");
            
            if(dtbaixarec < dtemissao){
                ScreenService.showMessage('Operação bloqueada. Data da baixa inferior a data de emissão do(s) título(s).');
                return;
            }
            
            self.geraAjusteMultaMoraBaixa().then(function(dataLocal){
                        
                var widgetTitulos    = templateManager.container.getWidget('TitulosAReceberGrid');
                var widgetNewTitulos = templateManager.container.getWidget('NewTitulosBaixaGrid');
                var checkedrows      = widgetNewTitulos.dataSource.data;
                var cdempresa        = widgetTitulos.appliedFilter.FILTER_ROW.CDEMPRESA;
                
                self.buscaEmpresa(cdempresa).then(function (response_empresa){
                    var empresa = response_empresa.dataset.empresa[0];

                    var currentRow = {
                        nrorg           : getLocalVar('NRORG')   ,
                        cdfilial        : getLocalVar('CDFILIAL'),
                        baixaboleto     : widget.currentRow      ,
                        boletosmarcados : checkedrows            ,
                        ajustesboletos  : dataLocal
                    };
        
                    if(empresa.IDCTBBAIXAEMPR == 'S' && widget.currentRow.IDCTBLBXREC == 'S') {
                        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulos.appliedFilter.FILTER_ROW.CDEMPRESA).then(function (response) {
                            var contopectbl  = response.dataset.contopectbl[0];
                    
                            if(!contopectbl){
                                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                                return;
                            }
        
                            var dtinicctbl  = contopectbl.DTINICCTBL;
                            var dtfinacctbl = contopectbl.DTFINACCTBL;
        
                            if(dtinicctbl && dtfinacctbl) {
                                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widget.currentRow.DTBAIXAREC, 'DD/MM/YYYY')) ||
                                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widget.currentRow.DTBAIXAREC, 'DD/MM/YYYY'))) {
                                    ScreenService.showMessage('Operação bloqueada. Data da baixa fora do período contábil financeiro.');
                                    return;
                                }
                            }
        
                            self.confirmaBaixaTituloSave(currentRow, widget, widgetTitulos);
                        }, function (error) {
                            if (contabilizacaoController.checkErrosContabilizacao(error)) {
                                ScreenService.showMessage(error.data.error);
                            }
                        });
                    }else self.confirmaBaixaTituloSave(currentRow, widget, widgetTitulos);
                });
            }
        );
        }
    };

    /**
     * Confirma Baixa Titulo a Receber Salvamento
     * @param currentRow
     * @param widget
     * @param widgetTitulos
     */
    self.confirmaBaixaTituloSave = function(currentRow, widget, widgetTitulos){
        
        localStorage.setItem('baixaRecRow',JSON.stringify(currentRow));
        localStorage.setItem('TIPOBAIXA','REC');

        Rec00001_TitulosAReceber_Service.baixaTituloReceber(currentRow).then(function () {
            ScreenService.showMessage('Título(s) baixado(s) com sucesso.');

            widget.currentRow.DTBAIXAREC   = '';
            widget.currentRow.IDSTATUSBREC = '';
            widget.currentRow.NMSTATUSBREC = '';
            widget.currentRow.CDREDUCONT   = '';
            widget.currentRow.DSCONTCORR   = '';
            widget.currentRow.CDTPBAIXAREC = '';
            widget.currentRow.NMTPBAIXAREC = '';
            widget.currentRow.VRBAIXAREC   = '';
            widget.currentRow.DSBAIXAREC   = '';

            ScreenService.closeSwipe(widget);
            ScreenService.closeSwipe();
            widget.isVisible = false;
            widgetTitulos.reload();
            
            widgetTitulos.dataSource.unSelectAll();
            self.unsetStripeDataFooter();
        }, function (error) {
            if (contabilizacaoController.checkErrosContabilizacao(error)) {
                ScreenService.showMessage(error.data.error);
            }
        });
    };

    /**
     * Exclusão Baixa Titulo a Receber
     */
    self.exclusaoBaixaTitulo = function(){
        ScreenService.showCustomDialog('Excluir...', 'question', 'question', [{
            label: 'Baixa',
            code: self.confirmaExclusaoBaixaTitulo
        }, {
            label: 'Título',
            code: self.confirmaExclusaoBoletoBaixaTitulo
        }, {
            label: 'Cancelar',
            code: ''
        }]);
    };

    /**
     * Confirmar Exclusão Baixa Recebida
     */
    self.confirmaExclusaoBaixaTitulo = function(){
        ScreenService.confirmMessage('Deseja realmente excluir a(s) baixa(s)?', 'question', function() {
            var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
            
            var titulosBaixados = widgetTitulos.getCheckedRows().filter(function(titulo){
                return ((titulo.IDSTATBREC == 'B') || (titulo.IDSTATBREC == 'P'));
            });

            var currentRow = {
                nrorg           : getLocalVar('NRORG'),
                cdfilial        : getLocalVar('CDFILIAL'),
                boletosmarcados : titulosBaixados
            };

            Rec00001_TitulosAReceber_Service.excluiBaixaTituloReceber(currentRow).then(function () {
                ScreenService.showMessage('Baixa(s) excluída(s) com sucesso.');
                ScreenService.closePopup();

                self.reloadTitulosAReceber();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    /**
     * Recarrega Titulos a Receber
     */
    self.reloadTitulosAReceber = function(){
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        widgetTitulos.reload();
        widgetTitulos.dataSource.unSelectAll();
        self.unsetStripeDataFooter();
    };

    /**
     * Confirmar Exclusão Boleto Baixa Titulo a Receber
     */
    self.confirmaExclusaoBoletoBaixaTitulo = function(){
        ScreenService.confirmMessage('Deseja realmente excluir o(s) título(s) da(s) baixa(s)?', 'question', function() {
            var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');

            var currentRow = {
                nrorg: getLocalVar('NRORG'),
                cdfilial: getLocalVar('CDFILIAL'),
                boletosmarcados: widgetTitulos.getCheckedRows()
            };

            Rec00001_TitulosAReceber_Service.excluiBoletoBaixaTituloReceber(currentRow).then(function () {
                ScreenService.showMessage('Título(s) excluído(s) da(s) baixa(s) com sucesso.');

                ScreenService.closePopup();

                widgetTitulos.reload();
                widgetTitulos.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        });
    };

    /** [Fim] Documento[Formulário] - Título a Receber/Recebidos[Botão Adicionar] */
    
    /** [Inicio]Anexo Grid/Form */
    
    self.importAnexoFile = function(args){
        //var anexoWidget = templateManager.container.getWidget('AnexoDocGrid');
        var importWidget = templateManager.container.getWidget('AnexoDocGridImportFile');
        importWidget.newRow();
        return ScreenService.openPopup(importWidget);
    };
    
    self.saveAnexoDocGrid = function(widget){
        widget.currentRow.NRORG = getLocalVar('NRORG');
        widget.currentRow.DTANEXOLANCBOL = moment().format('DD/MM/YYYY');
        
        Rec00001_TitulosAReceber_Service.saveAnexoDocGrid(widget.currentRow).then(function (sucess){
            ScreenService.closePopup();
            widget.getParent().reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.deleteAnexoDocGrid = function(widget){
        Rec00001_TitulosAReceber_Service.deleteAnexoDocGrid(widget.getCheckedRows()).then(function (sucess){
            widget.reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.verificaAnexos = function(widget){
        General.callRestEngine('fin_contaReceber#/deleteAnexoIncompletosRec', 'FilterData', []).then(function () {
            templateManager.container.getWidget('AnexoDocGrid').reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.adicionaAnexoGrid = function(widget){
        var widgetTitulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
        widget.newRow();
        widget.currentRow.NRLANBOLREC = widgetTitulosAReceberGrid.currentRow.NRLANBOLREC;
        widget.currentRow.CDFILIAL = widgetTitulosAReceberGrid.currentRow.CDFILIAL;
        widget.currentRow.CDTELA = 'REC00001';
        widget.currentRow.DTANEXOLANCBOL = moment().format('DD/MM/YYYY');
    };
    
    self.saveAnexoGrid = function(widget){
        widget.currentRow.NRORG = getLocalVar('NRORG');
        
        var row = Util.clone(widget.currentRow, ignoredKeys);
        
        Rec00001_TitulosAReceber_Service.salvaAnexoGrid(row).then(function (sucess){
            ScreenService.goBack();
            widget.reload();
        }, function(error){
            ScreenService.showMessage(error.dataset.error);
        });
    };
    
    self.deleteAnexoGrid = function(widget){
        ScreenService.confirmMessage('Deseja realmente excluir o(s) anexo(s)?', 'question', function() {
        
            var dataSet = [];
            
            if(widget.hasCheckedRows()){
                dataSet = widget.getCheckedRows();
                
                dataSet.forEach(function(data){
                	delete data.__rightActions;
                });
                
                Rec00001_TitulosAReceber_Service.deleteAnexoGrid(dataSet).then(
                    function (sucess){
                        widget.reload();
                    }, function(error){
                        ScreenService.showMessage(error.dataset.error);
                    }
                );
            } else{
                if (widget.currentRow.NRANEXOLANCBOL){
                    dataSet = [Util.clone(widget.currentRow, ignoredKeys)];
                    Rec00001_TitulosAReceber_Service.deleteAnexoGrid(dataSet).then(
                        function (sucess){
                            ScreenService.goBack();
                            ScreenService.showMessage('Registro excluído com sucesso.');
                            widget.reload();
                        }, function(error){
                            ScreenService.showMessage(error.dataset.error);
                        }
                    );
                }
            }
        });
    };
    
    self.downloadAnexoGrid = function(widget){
        var downloadRoute;
        var fileName;
        var serviceUrl;
        Rec00001_TitulosAReceber_Service.getFileServerUrl().then(
        function(sucess) {
            serviceUrl = sucess.dataset.sucess.fileName;
            if(widget.hasCheckedRows()){
                var rows = widget.getCheckedRows();
                rows.forEach(function(row){
                    downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexo?requestType=Row&row[DSARQUIVO]=";
                    fileName = row.NMPATHANEXO;
                    downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexoFileServer?requestType=Row&row[NMARQUIVO]=";
                    var rowDownload = {
                        'NMANEXOLANCBOL':row.NMANEXOLANCBOL, 
                        'PATH': serviceUrl+fileName
                    };
                    window.open(downloadRoute+rowDownload.NMANEXOLANCBOL+'&row[PATH]='+rowDownload.PATH);
                });
            } 
            else {
                var row = widget.currentRow;
                downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexo?requestType=Row&row[DSARQUIVO]=";
                fileName = row.NMPATHANEXO;
                downloadRoute = window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl + "/downloadArquivoAnexoFileServer?requestType=Row&row[NMARQUIVO]=";
                var rowDownload = {
                    'NMANEXOLANCBOL':row.NMANEXOLANCBOL, 
                    'PATH': serviceUrl+fileName
                };
                window.open(downloadRoute+rowDownload.NMANEXOLANCBOL+'&row[PATH]='+rowDownload.PATH);
            }
        });
    };
    
    /** [Fim]Anexo Grid */
    
    /** [Início] Rotina Ajuste[Conta] - Título a Receber/Recebidos > Documentos > Contas > Ajuste */

    /**
     * Deleta Ajuste Conta
     * @param args
     */
    self.deletaAjusteConta = function(args){
        var widgetAjusteConta    = args.owner.widget;
        var widgetContaReceber   = templateManager.container.getWidget('ContaDocumentosGrid');
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');


        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetContaReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulosReceber.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];
                    
            if(!contopectbl){
                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                return;
            }

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetAjusteConta.currentRow.DTAJUSTEREC, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetAjusteConta.currentRow.DTAJUSTEREC, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação Bloqueada. Data do ajuste fora do período contábil financeiro.');
                    return;
                }
            }

            var currentRowAjusteConta    = widgetAjusteConta.currentRow;
            var currentRowTitulosReceber = widgetTitulosReceber.currentRow;
            var currentRowContaReceber   = widgetContaReceber.currentRow;

            var currentRow = Object.assign(currentRowAjusteConta, currentRowTitulosReceber, currentRowContaReceber);

            ScreenService.confirmMessage('Deseja realmente excluir o Ajuste?', 'question', function() {

                Rec00001_TitulosAReceber_Service.deletaAjusteConta(currentRow).then(function () {
                    ScreenService.showMessage('Registro salvo com sucesso.');
                    ScreenService.goBack();
                    widgetAjusteConta.reload();
                    widgetTitulosReceber.reload();
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });

            });
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });

    };

    /** [Fim] Rotina Ajuste[Conta] - Título a Receber/Recebidos > Documentos > Contas > Ajuste */
    /** [Início] Rotina Rateio - Títulos a Receber/Recebidos > Documentos > Contas > Rateio */

    /**
     * Valor Total Rateio Conta a Receber
     * @param args
     */
    var valorTotalRateioConta = function(args){
        var vrTotal = 0;
        for(var i in args.dataSource)
            vrTotal += parseFloat(args.dataSource[i].VRRATCCCTREC);
        return vrTotal;
    };

    /**
     * Novo Rateio Conta a Receber
     * @param args
     */
    self.novoRateioConta = function(args){
        var widgetRateioConta    = args.owner.widget;
        var widgetContaReceber   = templateManager.container.getWidget('ContaDocumentosGrid');
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetContaReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulosReceber.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];
                    
            if(!contopectbl){
                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                return;
            }

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação Bloqueada. Data de saída fora do período contábil financeiro.');
                    return;
                }
            }
            
            var vrTotalConta       = parseFloat(widgetContaReceber.currentRow.VRATUAREC);
            var vrTotalRateio      = parseFloat(valorTotalRateioConta(args));
            
            widgetRateioConta.newRow();

            vrTotalRateio = vrTotalConta - vrTotalRateio;
            widgetRateioConta.currentRow.VRRATCCCTREC = vrTotalRateio;
            widgetRateioConta.currentRow.VRPERATEIO   = (vrTotalRateio * 100) / vrTotalConta;
            
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Edita Rateio Conta a Receber
     * @param widget
     */
    self.editaRateioConta = function(widget){
        var widgetContaReceber   = templateManager.container.getWidget('ContaDocumentosGrid');
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetContaReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulosReceber.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];
                    
            if(!contopectbl){
                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                return;
            }

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação Bloqueada. Data do saída fora do período contábil financeiro.');
                    return;
                }
            }

            widget.edit();

        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Salva Rateio Conta a Receber
     * @param args
     */
    self.salvaRateioConta = function(args){
        if(args.owner.widget.isValid()) {
            args.owner.widget.view();
            ScreenService.goBack();
            
            self.habilitaFinalizacaoRateioConta(args.owner.widget);
        }
    };

    /**
     * Finaliza Rateio Conta a Receber
     * @param args
     */
    self.finalizaRateioConta = function(args) {
        var vrTotalRateio      = 0;
        var widgetRateioConta  = args.owner.widget;
        var widgetContaReceber = templateManager.container.getWidget('ContaDocumentosGrid');
        var vrTotalConta       = parseFloat(widgetContaReceber.currentRow.VRATUAREC);

        // Percorre Dados de Rateio
        for (var i = 0; i < widgetRateioConta.dataSource.data.length; i++){
            // Percorre Dados de Rateio
            for (var j = 0; j < widgetRateioConta.dataSource.data.length; j++) {
                // Verifica Existência de Dados Duplicados
                if ((widgetRateioConta.dataSource.data[i].CDCENTCUST   == widgetRateioConta.dataSource.data[j].CDCENTCUST) &&
                    (widgetRateioConta.dataSource.data[i].CDFIRATCTREC == widgetRateioConta.dataSource.data[j].CDFIRATCTREC) &&
                    (i != j)
                ){
                    ScreenService.showMessage('Operação bloqueada. Existem informações duplicadas de Centro de Custo.');
                    return;
                }
            }
            vrTotalRateio += parseFloat(widgetRateioConta.dataSource.data[i].VRRATCCCTREC);
        }

        // Verifica Valor Total dos Rateios
        if (vrTotalRateio.toFixed(2) != vrTotalConta.toFixed(2)) {
            ScreenService.showMessage("Operação bloqueada. Total do(s) rateio(s) difere do valor total do título em "+((vrTotalRateio-vrTotalConta).formatFloat())+".");
            return;
        }

        var nrorg        = widgetContaReceber.currentRow.NRORG;
        var cdfilial     = widgetContaReceber.currentRow.CDFILIAL;
        var nrlanctonf   = widgetContaReceber.currentRow.NRLANCTONF;
        var nrseqvenrec  = widgetContaReceber.currentRow.NRSEQVENREC;
        var nrparcelarec = widgetContaReceber.currentRow.NRPARCELAREC;

        var dataSource = [{
            nrorg       : nrorg,
            cdfilial    : cdfilial,
            nrlanctonf  : nrlanctonf,
            nrseqvenrec : nrseqvenrec,
            nrparcelarec: nrparcelarec
        }, widgetRateioConta.dataSource.data];

        Rec00001_TitulosAReceber_Service.finalizaRateioConta(dataSource).then(function () {
            ScreenService.showMessage('Registro(s) salvo(s) com sucesso.');
            widgetRateioConta.reload();
            
            self.widgetEmEdicao("RateioDocumentosGrid", false);
            self.desabilitaAbas('RateioDocumentosGrid', false);
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Deleta Rateio Conta a Receber
     * @param args
     */
    self.deletaRateioConta = function(args){
        var widget = args.owner.widget;
        var widgetContaReceber   = templateManager.container.getWidget('ContaDocumentosGrid');
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if(widgetTitulosReceber.currentRow.IDSTATBREC == 'B') {
            ScreenService.showMessage('Operação bloqueada. Título baixado não pode ser alterado.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado.');
            return;
        }
        if(widgetContaReceber.currentRow.IDSTATUS == 'V') {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return;
        }

        Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulosReceber.currentRow.CDEMPRESA).then(function (response) {
            var contopectbl  = response.dataset.contopectbl[0];
                    
            if(!contopectbl){
                ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                return;
            }

            var dtinicctbl  = contopectbl.DTINICCTBL;
            var dtfinacctbl = contopectbl.DTFINACCTBL;

            if(dtinicctbl && dtfinacctbl) {
                if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                    (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetContaReceber.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                    ScreenService.showMessage('Operação Bloqueada. Data de saida fora do período contábil financeiro.');
                    return;
                }
            }

            ScreenService.confirmMessage('Deseja realmente excluir?', 'question', function () {
                if(!widget.currentRow.__createdLocal) {
                    Rec00001_TitulosAReceber_Service.deletaRateioConta(widget.currentRow).then(function () {
                        ScreenService.showMessage('Registro excluído com sucesso.');
                        ScreenService.goBack();
                        widget.removeRow();
                    }, function (error) {
                        ScreenService.showMessage(error.data.error);
                    });
                }else{
                    ScreenService.showMessage('Registro excluído com sucesso.');
                    General.backOrCancel(args);
                }
            });

        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Cálcula Percentual para Valor Rateio Conta a Receber
     * @param row
     */
    self.calcPercValorRateio = function(row){
        var widgetConta = templateManager.container.getWidget('ContaDocumentosGrid');
        var vrperateio  = parseFloat(row.VRPERATEIO.replace(',','.'));
        var vratuarec   = parseFloat(widgetConta.currentRow.VRATUAREC);

        row.VRRATCCCTREC   = (vratuarec * vrperateio) / 100;
    };

    /**
     * Cálcula Valor para Percentual Rateio Conta a Receber
     * @param row
     */
    self.calcValorPercRateio = function(row){
        var widgetConta   = templateManager.container.getWidget('ContaDocumentosGrid');
        var vrratccctrec  = parseFloat(row.VRRATCCCTREC.replace(',','.'));
        var vratuarec     = parseFloat(widgetConta.currentRow.VRATUAREC);

        row.VRPERATEIO   = (vrratccctrec * 100) / vratuarec;
    };

    /** [Fim] Rotina Rateio  - Títulos a Receber/Recebidos > Documentos > Contas > Rateio */

    /**
     * Remessa Titulo a Receber
     * @param widget
     */
    self.remessaTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {

            for (var i = 0; i < checkedrows.length; i++) {
                if(checkedrows[i].IDSITUCNABR != '00') {
                    ScreenService.showMessage('Operação Bloqueada. Título(s) marcado(s) já enviado(s).');
                    return;
                }
                //if(checkedrows[i].IDSTATUSEMIT != 'S') {
                    //ScreenService.showMessage('Operação Bloqueada. Título(s) marcado(s) não emitido(s).');
                  //  return;
                //}
                if((checkedrows[i].IDSTATBREC != 'E') && (checkedrows[i].IDSTATBREC != 'A')) {
                    ScreenService.showMessage('Operação Bloqueada. Marque somente título(s) em status "Em Aberto".');
                    return;
                }
            }

            var widgetRemessa = templateManager.container.getWidget('RemessaForm');
            widgetRemessa.isVisible = true;
            ScreenService.openPopup(widgetRemessa);
            self.buscaPadraoRemessaTitulo(widgetRemessa);

        }else ScreenService.showMessage('Marque o(s) Título(s) a Receber para fazer a remessa.');
    };

    /**
     * Busca Padrão Remessa Título
     */
    self.buscaPadraoRemessaTitulo = function (widget){
        var nrremessarec  = 0;
        var vrremessarec  = 0;
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetTitulosFilter = widgetTitulos.appliedFilter;

        var cdempresa     = widgetTitulosFilter.FILTER_ROW.CDEMPRESA||null;
        var checkedrows   = widgetTitulos.getCheckedRows();

        for(var i in checkedrows) {
            vrremessarec += self.formataValor(checkedrows[i].VRFINALBREC);
            nrremessarec += 1;
        }

        widget.getField('CDEMPRESA').setValue(cdempresa);
        widget.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widget.getField('DTCNABREC').setValue(moment().format('DD/MM/YYYY'));
        widget.getField('IDOPECNABREC').setValue('I');
        widget.getField('IDTIPCNABREC').setValue('0');
        widget.getField('VRTOTBOLCNABREC').setValue(vrremessarec);
        widget.getField('QTBOLENVCNABREC').setValue(nrremessarec);
        widget.getField('DSARQCNABREC').setValue('/tmp/');

        templateManager.updateTemplate();
    };

    /**
     * Alteração Conta Corrente Remessa
     */
    self.alteraContCorrRem = function (widget){
        widget = widget.owner.field.widget;
        
        var cdReduCont = widget.getField('CDREDUCONT').value();
        this.preencheFilial(cdReduCont);
        
        var cdbanccnab = widget.getField('CDBANCCNAB').value();

        if((cdbanccnab == '353') || (cdbanccnab == '008') || (cdbanccnab == '033')) {
            widget.getField('DSREDUCONTCOBR').readOnly = false;
            widget.getField('DSREDUCONTCOBR').validations.required = true;
        }else{
            widget.getField('DSREDUCONTCOBR').readOnly = true;
            delete widget.getField('DSREDUCONTCOBR').validations.required;
        }

        if((cdbanccnab == '237') || (cdbanccnab == '001') || (cdbanccnab == '637')) {
            widget.getField('IDTIPCNABREC').readOnly = false;
            widget.getField('IDTIPCNABREC').validations.required = true;
        }else{
            widget.getField('IDTIPCNABREC').readOnly = true;
            delete widget.getField('IDTIPCNABREC').validations.required;
        }
    };

    /**
     * Confirmar Remessa Titulo a Receber
     * @param args
     */
    self.confirmaRemessaTitulo = function(args){
        var widget = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        widget.currentRow.CDREDUCONTCOBR = widget.currentRow.CDREDUCONTCOBR||null;
    
        var currentRow = {
            nrorg           : getLocalVar('NRORG')   ,
            cdfilial        : getLocalVar('CDFILIAL'),
            remessaboleto   : widget.currentRow      ,
            boletosmarcados : widgetTitulos.getCheckedRows()
        };

        if(widget.isValid()) {
            Rec00001_TitulosAReceber_Service.remessaTituloReceber(currentRow).then(function (dsarqcnabrec) {
                var arquivo = dsarqcnabrec.dataset.remessaTituloReceber.dsarqcnabrec;

                ScreenService.showMessage('Remessa gerada com sucesso.');

                widget.currentRow.CDEMPRESA       = '';
                widget.currentRow.NRCNABREC       = '';
                widget.currentRow.CDOPERADOR      = '';
                widget.currentRow.DTCNABREC       = '';
                widget.currentRow.IDOPECNABREC    = '';
                widget.currentRow.NMOPECNABREC    = '';
                widget.currentRow.CDBANCCNAB      = '';
                widget.currentRow.CDREDUCONT      = '';
                widget.currentRow.DSCONTCORR      = '';
                widget.currentRow.CDREDUCONTCOBR  = '';
                widget.currentRow.DSREDUCONTCOBR  = '';
                widget.currentRow.IDTIPCNABREC    = '';
                widget.currentRow.NMTIPCNABREC    = '';
                widget.currentRow.QTBOLENVCNABREC = '';
                widget.currentRow.VRTOTBOLCNABREC = '';

                ScreenService.closePopup(widget);
                ScreenService.closePopup();
                widget.isVisible = false;

                self.downloadRemessaTitulo(arquivo);

                widgetTitulos.reload();
                widgetTitulos.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    /**
     * Atualiza Formulário
     */
    self.atualizaForm = function(widget_name) {
        var widget = templateManager.container.getWidget(widget_name);
        widget.currentRow = widget.dataSource.data[0]?widget.dataSource.data[0]:{};
        templateManager.updateTemplate();
    };

    /**
     * Download Remessa Titulos a Receber
     */
    self.downloadRemessaTitulo = function(arquivo) {
        window.open(window.serviceUrls.filter(function(x){return x.package == 'fin';})[0].baseUrl+"/donwloadArquivo?requestType=DataSet&dataset[0][DSARQUIVO]="+arquivo);
    };

    /**
     * Exclusão Baixa Titulo a Receber
     */
    self.exclusaoRemessaTitulo = function(){
        var checkedrows = templateManager.container.getWidget('TitulosAReceberGrid').getCheckedRows();

        if(checkedrows.length > 0) {
            for (var i = 0; i < checkedrows.length; i++) {
                if(checkedrows[i].IDSITUCNABR == '00') {
                    ScreenService.showMessage('Operação Bloqueada. Título(s) marcado(s) não enviado(s).');
                    return;
                }
            }

            ScreenService.showCustomDialog('O que deseja excluir?', 'question', 'question', [{
                label: 'Remessa',
                code: self.excluiRemessaRec
            }, {
                label: 'Título',
                code: self.excluiTituloRemessaRec
            }, {
                label: 'Cancelar',
                code: ''
            }]);

        }else ScreenService.showMessage('Marque o(s) Título(s) a Receber para exclusão da Remessa.');
    };

    /**
     * Exclui Titulos da Remessa Receber
     */
    self.excluiTituloRemessaRec = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');

        var paramsRem = {
            nrorg     : getLocalVar('NRORG'),
            cdfilial  : getLocalVar('CDFILIAL'),
            cdoperador: getLocalVar('CDOPERADOR')
        };
        var checkedrows = [paramsRem, widget.getCheckedRows()];

        ScreenService.confirmMessage('Deseja realmente excluir o(s) título(s) da(s) remessa(s).', 'question', function () {
            Rec00001_TitulosAReceber_Service.excluiTituloRecRemessa(checkedrows).then(function () {
                ScreenService.showMessage('Título(s) excluído(s) da(s) remessa(s) com sucesso.');
                ScreenService.closePopup();
                widget.reload();
                widget.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });

    };

    /**
     * Exclui Remessa Receber
     */
    self.excluiRemessaRec = function(){
        var widget      = templateManager.container.getWidget('TitulosAReceberGrid');

        var paramsRem = {
            nrorg     : getLocalVar('NRORG'),
            cdfilial  : getLocalVar('CDFILIAL'),
            cdoperador: getLocalVar('CDOPERADOR')
        };
        var checkedrows = [paramsRem, widget.getCheckedRows()];

        ScreenService.confirmMessage('Deseja realmente excluir a(s) remessa(s).', 'question', function () {
            Rec00001_TitulosAReceber_Service.excluiRemessaRec(checkedrows).then(function () {
                ScreenService.showMessage('Remessa(s) excluída(s) com sucesso.');
                ScreenService.closePopup();
                widget.reload();
                widget.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }, function (error) {
                ScreenService.showMessage(error.data.error);
            });
        });
    };

    /**
     * Busca Parâmetros de Contabilização
     * @return contOpeCtbl
     */
    self.buscaContOpeCtbl = function (cdempresa){
        return Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(cdempresa).then(function(contopectbl) {
            return contopectbl;
        });
    };

    /**
     * Busca Conta a Receber
     * @param cdfilial
     * @param nrlanctonf
     */
    self.buscaContaReceber = function(cdfilial, nrlanctonf){
        return Rec00001_TitulosAReceber_Service.buscaContaReceber(cdfilial, nrlanctonf).then(function (contareceber) {
            return contareceber;
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    /**
     * Edita Documento Grid
     * @param args
     */
    self.editaDocumentoGrid = function(args){
        var widget = args.owner.widget;
        var widgetTitulosReceber = templateManager.container.getWidget('TitulosAReceberGrid');

        if((widgetTitulosReceber.currentRow.IDSTATBREC == 'B') || (widgetTitulosReceber.currentRow.IDSTATBREC == 'P')){
            ScreenService.showMessage('Operação bloqueada. Título baixado ou com baixa planejada não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosReceber.currentRow.IDSITUCNABR != '00') {
            ScreenService.showMessage('Operação bloqueada. Título enviado ao banco, via CNAB, não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosReceber.currentRow.NRPARCBOLREC != '00'){
            ScreenService.showMessage('Operação bloqueada. Título parcelado não pode ser alterado o documento.');
            return;
        }

        if(widgetTitulosReceber.currentRow.IDASSOCBOLREC == 'S') {
            ScreenService.showMessage('Operação bloqueada. Título associado não pode ser alterado o documento.');
            return;
        }
        if(widgetTitulosReceber.currentRow.VRBOLREC != widgetTitulosReceber.currentRow.VRFINALBREC) {
            ScreenService.showMessage('Operação bloqueada. Título com ajuste não pode ser alterado o documento.');
            return;
        }
        
        General.widgetEdit(args.owner.widget);

        widget.currentRow.backupIdstatus   = widget.currentRow.IDSTATUS;
        widget.currentRow.backupDtnotafisc = widget.currentRow.DTENTRSAID;
        
        
        
        self.changeStatusDocumentoForm();
    };

    /**
     * Habilita/Desabilita campos de acordo com Tipo de Baixa
     */
    self.changeStatusDocumentoForm = function() {
        var widget = templateManager.container.getWidget('DocumentosGrid');

        if(widget.currentRow.IDSISTEMA == 'REC') {
            widget.getField('DTEMISSAO').readOnly  = false;
            widget.getField('DTENTRSAID').readOnly = false;
        }else{
            widget.getField('DTEMISSAO').readOnly  = true;
            widget.getField('DTENTRSAID').readOnly = true;
        }
    };

    /**
     * Atualiza Titulo a Receber
     */
    self.atualizaTituloAReceber = function(){
        var widgetTitulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetDocumentosGrid      = templateManager.container.getWidget('DocumentosGrid');
        
        if(widgetDocumentosGrid.isValid()){
            if((moment(widgetDocumentosGrid.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de Emissão deve ser menor ou igual a Data de Saída.');
                return;
            }
            
            if((moment(widgetDocumentosGrid.currentRow.DTEMISSAO, 'DD/MM/YYYY')) > (moment(widgetTitulosAReceberGrid.currentRow.DTVENBOLREC, 'DD/MM/YYYY'))){
                ScreenService.showMessage('Data de Emissão deve ser menor ou igual a Data de Vencimento.');
                return;
            }
    
            if((widgetDocumentosGrid.currentRow.backupIdstatus   != widgetDocumentosGrid.currentRow.IDSTATUS) ||
               (widgetDocumentosGrid.currentRow.backupDtnotafisc != widgetDocumentosGrid.currentRow.DTENTRSAID)){
                Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetDocumentosGrid.currentRow.CDEMPRESA).then(function (response) {
                    var contopectbl  = response.dataset.contopectbl[0];
                        
                    if(!contopectbl){
                        ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                        return;
                    }
    
                    var dtinicctbl  = contopectbl.DTINICCTBL;
                    var dtfinacctbl = contopectbl.DTFINACCTBL;
    
                    if(dtinicctbl && dtfinacctbl) {
                        if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY')) ||
                            (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widgetDocumentosGrid.currentRow.DTENTRSAID, 'DD/MM/YYYY'))) {
                            ScreenService.showMessage('Operação Bloqueada. Data do saída fora do período contábil financeiro.');
                            return;
                        }
                    }
    
                    Rec00001_TitulosAReceber_Service.atualizaTituloAReceber(widgetDocumentosGrid.currentRow).then(function () {
                        General.changeWidgetAcessibility(widgetDocumentosGrid.widgets, true);
                        ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                        ScreenService.goBack();
    
                        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
                        widgetTitulos.currentRow.CDTPCTRECEB = widgetDocumentosGrid.currentRow.CDTPCTRECEB;
                        widgetTitulos.currentRow.IDSTATUS = widgetDocumentosGrid.currentRow.IDSTATUS;
    
                        widgetDocumentosGrid.reload();
                    }, function (error) {
                        if (contabilizacaoController.checkErrosContabilizacao(error))
                            ScreenService.showMessage(error.data.error);
                    });
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }else{
                Rec00001_TitulosAReceber_Service.atualizaTituloAReceber(widgetDocumentosGrid.currentRow).then(function () {
                    General.changeWidgetAcessibility(widgetDocumentosGrid.widgets, true);
                    ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                    ScreenService.goBack();
    
                    var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
                    widgetTitulos.currentRow.CDTPCTRECEB = widgetDocumentosGrid.currentRow.CDTPCTRECEB;
                    widgetDocumentosGrid.reload();
                }, function (error) {
                    if (contabilizacaoController.checkErrosContabilizacao(error))
                        ScreenService.showMessage(error.data.error);
                });
            }
        }
    };

    /**
     * Baixa Titulo
     * @param widget
     */
    self.baixaTitulo = function(widget){
        var checkedrows = widget.getCheckedRows();
        if(checkedrows.length > 0) {
            if (self.statusTituloPermiteRealizarBaixa(checkedrows)){

                var boletosBaixaPlanejada = checkedrows.filter(function(boleto){
                    return boleto.IDSTATBREC == 'P';
                });
                var boletosEmAberto = checkedrows.filter(function(boleto){
                    return (boleto.IDSTATBREC == 'A')||(boleto.IDSTATBREC == 'E');
                });
                var boletosDescontado = checkedrows.filter(function(boleto){
                    return (boleto.IDSTATBREC == 'D')&&(boleto.DTBAIXAREC);
                });

                if((boletosBaixaPlanejada.length > 0) || (boletosDescontado.length > 0)){
                    if(boletosDescontado.length > 0){
                        ScreenService.confirmMessage('Existe(m) título(s) marcado(s) descontado(s). Deseja realizar a baixa real do(s) mesmo(s)?', 'question', function () {
                            self.baixarTitulosMarcadosOuTitulosBaixaDescontado();
                        }, function () {
                            if(boletosBaixaPlanejada.length > 0) {
                                ScreenService.confirmMessage('Existe(m) título(s) marcado(s) com baixa(s) planejada(s).Deseja transfomá-la(s) em baixa(s) real(is)?', 'question', function () {
                                    self.baixarTitulosMarcadosOuTitulosBaixa();
                                }, function () {
                                    if (boletosEmAberto.length > 0) {
                                        self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                                    } else {
                                        ScreenService.closePopup();
                                    }
                                });
                            }
                        });
                    }else{
                        ScreenService.confirmMessage('Existe(m) título(s) marcado(s) com baixa(s) planejada(s).Deseja transfomá-la(s) em baixa(s) real(is)?', 'question', function () {
                            self.baixarTitulosMarcadosOuTitulosBaixa();
                        }, function () {
                            if (boletosEmAberto.length > 0) {
                                self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                            } else {
                                ScreenService.closePopup();
                            }
                        });
                    }
                }else self.efetivarBaixaTitulosAbertos(boletosEmAberto);
            }else ScreenService.closePopup();
        }else ScreenService.showMessage('Marque o(s) título(s) para baixá-lo(s).');
    };

    /**
     * Titulos Baixados
     * @param boletoMarcado
     * @returns {boolean}
     */
    function titulosBaixados(boletoMarcado){
        return boletoMarcado.IDSTATBREC != "B";
    }

    /**
     * Titulos Previstos
     * @param boletoMarcado
     * @returns {boolean}
     */
    function titulosPrevistos(boletoMarcado){
        return boletoMarcado.IDSTATUS != "V";
    }

    /**
     * Titulos Abertos e Planejados
     * @param boletoMarcado
     * @returns {boolean}
     */
    function titulosAbertosEPlanejados(boletoMarcado){
        return ((boletoMarcado.IDSTATBREC == "A") || (boletoMarcado.IDSTATBREC == "E") || (boletoMarcado.IDSTATBREC == "P") || (boletoMarcado.IDSTATBREC == "D"));
    }

    /**
     * Status do Título que Permite Realização da Baixa
     * @param checkedrows
     * @returns {boolean}
     */
    self.statusTituloPermiteRealizarBaixa = function(checkedrows){
        if(!checkedrows.every(titulosBaixados)) {
            ScreenService.showMessage('Operação Bloqueada. Título(s) baixado(s) não pode(m) ser alterado(s).');
            return false;
        }
        if(!checkedrows.every(titulosPrevistos)) {
            ScreenService.showMessage('Operação Bloqueada. Título previsto não pode ser alterado.');
            return false;
        }
        if(!checkedrows.every(titulosAbertosEPlanejados)) {
            ScreenService.showMessage('Operação Bloqueada. Marque somente título(s) com status "Em Aberto" ou "Planejado".');
            return false;
        }
        return true;
    };

    /**
     * Baixa Titulos Marcados ou Titulos Baixa
     */
    self.baixarTitulosMarcadosOuTitulosBaixa = function(){
        ScreenService.showCustomDialog('Transformar em real(is)?', 'question', 'question', [{
            label: 'Baixa',
            code: self.efetivaTitulosDaBaixa
        }, {
            label: 'Título',
            code: self.efetivaTitulosMarcados
        }, {
            label: 'Cancelar',
            code: self.verificaTitulosEmAberto
        }]);
    };

    /**
     * Baixa Titulos Marcados ou Titulos Baixa
     */
    self.baixarTitulosMarcadosOuTitulosBaixaDescontado = function(){
        var widgetBaixaDescontado = templateManager.container.getWidget('BaixaDescontadoForm');
        widgetBaixaDescontado.isVisible = true;
        ScreenService.openPopup(widgetBaixaDescontado);
        self.buscaPadraoBaixaTituloDescontado(widgetBaixaDescontado);
    };

    /**
     * Baixa Titulo
     */
    self.verificaTitulosEmAberto = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widget.getCheckedRows();

        var boletosEmAberto = checkedrows.filter(function(boleto){
            return (boleto.IDSTATBREC == 'A')||(boleto.IDSTATBREC == 'E');
        });

        if(boletosEmAberto.length > 0)
            self.efetivarBaixaTitulosAbertos(boletosEmAberto);
        else ScreenService.closePopup();
    };

    /**
     * Efetiva Titulos Marcados
     */
    self.efetivaTitulosMarcados = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var boletosBaixaPlanejada = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBREC == 'P';
        });
        var boletosEmAberto = widget.getCheckedRows().filter(function(boleto){
            return (boleto.IDSTATBREC == 'A')||(boleto.IDSTATBREC == 'E');
        });
        var baixatitulo = self.preparaDataSourceBoletosPlanejados(boletosBaixaPlanejada, 'E');
        baixatitulo.forEach(function (baixa) {
            self.efetivaBaixaTitulo(baixa).then(function () {
                if (boletosEmAberto.length > 0) {
                    self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                } else {
                    ScreenService.closePopup();
                    widget.dataSource.unSelectAll();
                    self.unsetStripeDataFooter();
                    widget.reload().then(function () {
                        ScreenService.showMessage('Título(s) transformado(s) em baixa(s) real(is) com sucesso.');
                    });
                }
            });
        });
    };

    /**
     * Para Data Source Boletos Planejados
     * @param boletosBaixaPlanejada
     * @param statusBaixaReal
     * @returns {Array}
     */
    self.preparaDataSourceBoletosPlanejados = function(boletosBaixaPlanejada, statusBaixaReal){
        var baixaTitulo = [];
        boletosBaixaPlanejada.forEach(function(boletoMarcado){
            var baixa = baixaTitulo.filter(function(baixa){
                return baixa.NRBAIXAREC == boletoMarcado.NRBAIXAREC;
            });
            if (baixa.length > 0){
                baixa[0].BOLETOS.push(boletoMarcado);
            }else{
                baixaTitulo[baixaTitulo.length] = {
                    'NRBAIXAREC': boletoMarcado.NRBAIXAREC,
                    'CDEMPRESA': boletoMarcado.CDEMPRESA,
                    'CDOPERADOR': boletoMarcado.CDOPERADOR,
                    'CDTPBAIXAREC': boletoMarcado.CDTPBAIXAREC,
                    'DTBAIXAREC': boletoMarcado.DTBAIXAREC,
                    'IDBAIXAUTILR': boletoMarcado.IDBAIXAUTILR,
                    'DTLANCBAIXAR': moment().format('DD/MM/YYYY'),
                    'IDSTATUSBREC': statusBaixaReal,
                    'CDREDUCONT': boletoMarcado.CDREDUCONTBAIXA,
                    'DSBAIXAREC': boletoMarcado.DSBAIXAREC,
                    'BOLETOS': [boletoMarcado]
                };
            }
        });
        return baixaTitulo;
    };

    /**
     * Efetiva Titulos da Baixa
     */
    self.efetivaTitulosDaBaixa = function(){
        var route = 'fin#/crud/boletreceber';
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var boletosBaixaPlanejada = widget.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBREC == 'P';
        });
        var boletosEmAberto = widget.getCheckedRows().filter(function(boleto){
            return (boleto.IDSTATBREC == 'A')||(boleto.IDSTATBREC == 'E');
        });
        var baixatitulo = self.preparaDataSourceBoletosPlanejados(boletosBaixaPlanejada, 'E');
        baixatitulo.forEach(function(baixa){
            var filter = [
                {
                    "name": "CDFILIBAIREC",
                    "operator": "=",
                    "value": baixa.BOLETOS[0].CDFILIBAIREC
                },
                {
                    "name": "NRBAIXAREC",
                    "operator": "=",
                    "value": baixa.NRBAIXAREC
                },
                {
                    "name": "NRORG",
                    "operator": "=",
                    "value": baixa.BOLETOS[0].NRORG
                }
            ];

            General.callRestEngine(route, 'FilterData', filter).then(function (boletos) {
                baixa.BOLETOS = boletos.dataset.boletreceber;
                self.efetivaBaixa(baixa).then(function () {
                    if (boletosEmAberto.length > 0) {
                        self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                    } else {
                        ScreenService.closePopup();
                        widget.dataSource.unSelectAll();
                        self.unsetStripeDataFooter();
                        widget.reload().then(function () {
                            ScreenService.showMessage('Título(s) transformado(s) em baixa(s) real(is) com sucesso.');
                        });
                    }
                });
            });
        });
    };

    /**
     * Efetiva Baixa
     * @param baixa
     * @returns {*}
     */
    self.efetivaBaixa = function(baixa){
        var deferred = ZHPromise.defer();

        var currentRow = {
            nrorg: getLocalVar('NRORG'),
            cdfilial: getLocalVar('CDFILIAL'),
            baixaboleto: baixa,
            boletosmarcados: baixa.BOLETOS
        };

        Rec00001_TitulosAReceber_Service.transformaPlanejadoTitulo(currentRow).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });

        return deferred.promise;
    };

    /**
     * Efetiva Baixa Titulo
     * @param baixa
     * @returns {*}
     */
    self.efetivaBaixaTitulo = function(baixa){
        var deferred = ZHPromise.defer();

        var currentRow = {
            nrorg: getLocalVar('NRORG'),
            cdfilial: getLocalVar('CDFILIAL'),
            baixaboleto: baixa,
            boletosmarcados: baixa.BOLETOS
        };

        Rec00001_TitulosAReceber_Service.transformaBoletoPlanejadoTitulo(currentRow).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });

        return deferred.promise;
    };

    /**
     * Efetiva Baixa Titulos Abertos
     * @param titulosEmAberto
     */
    self.efetivarBaixaTitulosAbertos = function(titulosEmAberto){
        ScreenService.confirmMessage('Deseja realmente realizar baixa do(s) título(s) "Em Aberto" marcados?', 'question', function () {
            var widgetBaixa = templateManager.container.getWidget('NewBaixaTabbed');
            var widgetTitulosBaixa = templateManager.container.getWidget('NewTitulosBaixaGrid');
            
            ScreenService.closePopup();
            
            widgetBaixa.isVisible = true;
            self.widgetEmEdicao("NewBaixaTabbed", true);
            
            self.buscaPadraoBaixaTitulo(widgetBaixa, widgetTitulosBaixa, titulosEmAberto).then(
                function (dataLocal) {
                    ScreenService.openSwipe(widgetBaixa);
                    widgetTitulosBaixa.reload();
                });
            self.dataSourceAjusteTituloBaixo();
        });
    };

    /**
     * Valida Adiantamento Cliente
     */
    self.validaAdiantamentoCliente = function(){

        var widget = templateManager.container.getWidget('TitulosAReceberGrid');

        // Válidações para Adiantamento e Devolução
        if(widget.getLastSelectedRow().IDSTATBREC    !=  'A') return;
        if(widget.getLastSelectedRow().IDSITUCNABR   != '00') return;
        if(widget.getLastSelectedRow().IDSTATUS      ==  'V') return;
        if(widget.getLastSelectedRow().IDTPCTRECEBER ==  '2') return;

        var routeAdiantamento = 'fin_movimentacaoBancaria#/validaAdiantamentoCliente';

        General.callRestEngine(routeAdiantamento, 'Row', widget.getLastSelectedRow()).then(function(adiantamento) {
            var vrfinalbrec = self.formataValor(widget.getLastSelectedRow().VRFINALBREC.toString());
            if (adiantamento.dataset.ADIANTAMENTO.length > 0) {
                ScreenService.confirmMessage('Atenção! Cliente possui crédito de adiantamento. Deseja utilizá-lo?', 'question', function() {
                    var adiantamentoPopup = templateManager.container.getWidget('AdiantamentoTabbed');

                    ScreenService.openPopup(adiantamentoPopup).then(function () {
                        adiantamentoPopup.currentRow.NMRAZSOCCLIE = widget.getLastSelectedRow().NMRAZSOCCLIE;
                        adiantamentoPopup.currentRow.CDAJUCTREC   = '';
                        adiantamentoPopup.currentRow.DSAJUCTREC   = '';
                        adiantamentoPopup.currentRow.TOTALTITULO   = vrfinalbrec;
                        adiantamentoPopup.widgets[0].reload();
                    });
                }, function(){
                    self.validaDevolucaoCliente();
                });
            }else self.validaDevolucaoCliente();
        });
    };

    /**
     * Valida Devolução Cliente
     */
    self.validaDevolucaoCliente = function(){

        self.buscaGeralRec().then(function(response_geralrec) {

            var geral_rec   = response_geralrec.dataset.geral_rec[0];
            var idutlnfdevol = geral_rec.IDUTLNFDEVOLREC;

            // Verifica Utilização de Devolução
            if(idutlnfdevol == 'S') {
                var widget = templateManager.container.getWidget('TitulosAReceberGrid');
                var routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';

                General.callRestEngine(routeDevolucao, 'Row', widget.getLastSelectedRow()).then(function (devolucao) {
                    if (devolucao.dataset.DEVOLUCAO.length > 0) {
                        ScreenService.confirmMessage('Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function () {
                            var devolucaoPopup = templateManager.container.getWidget('DevolucaoTabbed');

                            ScreenService.openPopup(devolucaoPopup).then(function () {
                                devolucaoPopup.currentRow.NMRAZSOCCLIE = widget.getLastSelectedRow().NMRAZSOCCLIE;
                                devolucaoPopup.currentRow.CDAJUCTREC = '';
                                devolucaoPopup.currentRow.DSAJUCTREC = '';
                                devolucaoPopup.widgets[0].reload();
                            });
                        });
                    }
                });
            }
        });
    };

    /**
     * Cancela Adiantamento Cliente
     * @param widget
     */
    self.cancelaAdiantamentoCliente = function(widget){

        var widgetAdiantamento = templateManager.container.getWidget('TitulosAReceberGrid');
        var routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';

        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();

        General.callRestEngine(routeDevolucao, 'Row', widgetAdiantamento.getLastSelectedRow()).then(function(devolucao) {
            if (devolucao.length > 0) {
                ScreenService.confirmMessage('Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function(){
                    var devolucaoPopup = templateManager.container.getWidget('devolucaoClienteTabbed');

                    ScreenService.openPopup(devolucaoPopup).then(function(){
                        devolucaoPopup.currentRow.NMRAZSOCCLIE = widget.getLastSelectedRow().NMRAZSOCCLIE;
                    });
                });
            }
        });
    };

    /**
     * Cancela Devolução Cliente
     * @param widget
     */
    self.cancelaDevolucaoCliente = function(widget){
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
    };

    /**
     * Seta Data Source Popup Cliente
     * @param popupCliente
     */
    self.setDataSourcePopupCliente = function(popupCliente){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var lastSelectedRow = widget.getLastSelectedRow();
        popupCliente.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDCLIENTE') === 0){
                filtro.value = lastSelectedRow.CDCLIENTE;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = lastSelectedRow.CDEMPRESA;
            }else if (filtro.name.localeCompare('NRINSJURCLIE') === 0) {
                filtro.value = lastSelectedRow.NRINSJURCLIE;
            }
        });
        popupCliente.reload();
    };

    /**
     * Seleciona Todos os Registros
     * @param widget
     */
    self.selecionaTodosRegistros = function(widget){
        self.atualizaValorAdiantaDev(widget).then(
            function() {},
            function(error){
                widget.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
            }
        );
    };

    /**
     * Atualiza Valor Adiantamento Devolução
     * @param widget
     * @returns {*}
     */
    self.atualizaValorAdiantaDev = function(widget) {
        var deferred = ZHPromise.defer();
        var tituloGrid = templateManager.container.getWidget('TitulosAReceberGrid');

        var totalDesconto = 0;
        var valorTitulo   = parseFloat(tituloGrid.getLastSelectedRow().VRFINALBREC);

        widget.getCheckedRows().forEach(function (data) {
            totalDesconto += parseFloat(data.VRDESCONTO);
        });

        if (totalDesconto > valorTitulo){
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do título.');
            widget.getLastSelectedRow().__isSelected = false;
            deferred.reject();
        } else {
            deferred.resolve();
            widget.getParent().currentRow.TOTALDESCONTO = totalDesconto;
        }
        return deferred.promise;
    };

    /**
     * Calcula Total Adiantamento Disponível
     * @param widget
     */
    self.calculaTotalAdiantamentoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += self.formataValor(data.VRTOTDISP);
        });
        widget.getParent().currentRow.TOTALADIANTAMENTO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    /**
     * Calcula Total Devolução Disponível
     * @param widget
     */
    self.calculaTotalDevolucaoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += self.formataValor(data.VRTOTDISP);
        });
        widget.getParent().currentRow.TOTALDEVOLUCAO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    /**
     * Limita Valor Desconto
     * @param widget
     */
    self.limitaValorDesconto = function(widget){
        var superiorDisponivel, superiorLancamento = false;

        var data   = widget.dataSource.data;
        var indice = widget.currentRow.__index;
        var widgetTitulo = templateManager.container.getWidget('TitulosAReceberGrid');

        var valorTitulo     = parseFloat(widgetTitulo.currentRow.VRFINALBREC);
        var valorDesconto   = parseFloat(widget.currentRow.VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
        var totalDisponivel = parseFloat(widget.currentRow.VRTOTDISP.toString().replace('.', '').replace(',', '.'));
        var totalDesconto   = 0.00;

        for(var i = 0; i < data.length; i++)
            if (i != indice && data[i].__isSelected)
                totalDesconto += parseFloat(data[i].VRDESCONTO);

        if ((valorDesconto + totalDesconto) > totalDisponivel) {
            superiorDisponivel = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }

        if (valorDesconto > valorTitulo) {
            superiorLancamento = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }

        if (widget.currentRow.__isSelected) {
            var desconto = parseFloat(totalDesconto) + parseFloat(widget.dataSource.data[indice].VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
            widget.getParent().currentRow.TOTALDESCONTO = desconto;
        }else widget.getParent().currentRow.TOTALDESCONTO = parseFloat(totalDesconto);

        if (superiorDisponivel)
            ScreenService.showMessage('Valor do desconto deve ser menor ou igual que o valor disponível.');

        if (superiorLancamento)
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do lançamento.');

    };

    /**
     * Confirma Adiantamento
     */
    self.confirmaAdiantamento = function(){
        var adiantamentoTabbed = templateManager.container.getWidget('AdiantamentoTabbed');
        var adiantamentoGrid   = templateManager.container.getWidget('AdiantamentoGrid');
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');

        if(adiantamentoTabbed.isValid()) {
            if ((adiantamentoGrid.getCheckedRows().length <= 0) || (adiantamentoTabbed.currentRow.TOTALDESCONTO <= 0)) {
                ScreenService.showMessage('Operação bloqueada. Selecione o(s) registro(s) e informe o(s) valor(es) a ser(em) descontado(s).');
                return;
            }
            
            var route = 'fin_movimentacaoBancaria#/adiantareceb_adiantamentoReceber';
            var filter = [
                {
                    "name": "CDEMPRESA",
                    "operator": "=",
                    "value": widget.getLastSelectedRow().CDEMPRESA
                },
                {
                    "name": "CDCLIENTE",
                    "operator": "=",
                    "value": widget.getLastSelectedRow().CDCLIENTE
                }
            ];

            //valida se o adiantamento ainda esta disponivel, impedindo a utilizacao do mesmo em dois boletos diferentes ao mesmo tempo 
            
            General.callRestEngine(route, 'FilterData', filter).then(function(adiantamento) {
                var adiantamentos = adiantamento.dataset.adiantareceb_adiantamentoReceber;
    
                var adiantamentoFiltrado = adiantamentos.filter(function(valores){
                    return valores.NRLANCTONF == adiantamentoGrid.getLastSelectedRow().NRLANCTONF;
                 });
                 
                 if(adiantamentoFiltrado.length > 0){
                     adiantamentoFiltrado.forEach(function(dados){
                        if(dados.VRTOTDISP > 0){
                            self.geraAdiantamentoCliente();
                        }
                     });
                 }else{
                     ScreenService.showMessage("Esse adiantamento não está mais disponível para essa operação!");
                     adiantamentoGrid.reload();
                 }
            });
        }
    };

    /**
     * Confirma Adiantamento
     */
    self.confirmaDevolucao = function(){
        var devolucaoTabbed = templateManager.container.getWidget('DevolucaoTabbed');
        var devolucaoGrid   = templateManager.container.getWidget('DevolucaoGrid');

        if(devolucaoTabbed.isValid()) {
            if ((devolucaoGrid.getCheckedRows().length <= 0) || (devolucaoTabbed.currentRow.TOTALDESCONTO <= 0)) {
                ScreenService.showMessage('Operação bloqueada. Selecione o(s) registro(s) e informe o(s) valor(es) a ser(em) descontado(s).');
                return;
            }

            self.geraDevolucaoCliente();
        }
    };

    /**
     * Gera Adiantamento Cliente
     */
    self.geraAdiantamentoCliente = function(){

        var tituloGrid         = templateManager.container.getWidget('TitulosAReceberGrid');
        var adiantamentoTabbed = templateManager.container.getWidget('AdiantamentoTabbed');
        var adiantamentoGrid   = templateManager.container.getWidget('AdiantamentoGrid');

        var lastSelectedRowTituloGrid    = tituloGrid.getLastSelectedRow();
        var currentRowAdiantamentoTabbed = adiantamentoTabbed.currentRow;
        var checkedrowsAdiantamentoGrid  = adiantamentoGrid.getCheckedRows();

        var dataSet = [lastSelectedRowTituloGrid, currentRowAdiantamentoTabbed, checkedrowsAdiantamentoGrid];

        General.callRestEngine('fin_contaReceber#/geraAdiantamentoCliente', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');
            ScreenService.closePopup();
            
            tituloGrid.dataSource.unSelectAll();
            self.unsetStripeDataFooter();
            self.forceActivate();
            
            tituloGrid.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Gera Devolução Cliente
     */
    self.geraDevolucaoCliente = function(){

        var tituloGrid      = templateManager.container.getWidget('TitulosAReceberGrid');
        var devolucaoTabbed = templateManager.container.getWidget('DevolucaoTabbed');
        var devolucaoGrid   = templateManager.container.getWidget('DevolucaoGrid');

        var lastSelectedRowTituloGrid = tituloGrid.getLastSelectedRow();
        var currentRowDevolucaoTabbed = devolucaoTabbed.currentRow;
        var checkedrowsDevolucaoGrid  = devolucaoGrid.getCheckedRows();

        var dataSet = [lastSelectedRowTituloGrid, currentRowDevolucaoTabbed, checkedrowsDevolucaoGrid];

        General.callRestEngine('fin_contaReceber#/geraDevolucaoCliente', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');
            ScreenService.closePopup();
            tituloGrid.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Habilita Licensa de Uso Documento
     * @param widget
     */
    self.habilitaLicensaUsoDocumento = function(widget){
        var cdcliente = widget.currentRow.CDCLIENTE;

        self.buscaCliente(cdcliente).then(function (response_cliente){
            var cliente = response_cliente.dataset.cliente[0];

            if(cliente)
                widget.getField('NRLICENCAUSO').isVisible = cliente.IDUTLICEUSO == 'S';
            else widget.getField('NRLICENCAUSO').isVisible = false;
        });
    };

    /**
     * Habilita Vendedor Documento
     * @param widget
     */
    self.habilitaVendedorDocumento = function(widget){
        var cdfilial = widget.currentRow.CDFILIEMIT;

        self.buscaParExpedicao(cdfilial).then(function (response_parexpedicao){
            var parexpedicao = response_parexpedicao.dataset.parexpedicao[0];

            if(parexpedicao)
                widget.getField('NMRAZSOCVEN').isVisible = parexpedicao.IDCTRLVENEXP == 'S';
            else widget.getField('NMRAZSOCVEN').isVisible = false;
        });
    };

    /**
     *Consulta Contabilização
     */
    self.consultaContabilizacao = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widget.getCheckedRows();

        if(checkedrows.length > 0) {
            var contabil = [];

            for (var i in checkedrows){
                if (checkedrows[i].NRLANCTBDOC) {
                    contabil.push({});
                    contabil[i].CDEMPRESA  = checkedrows[i].CDEMPRESA;
                    contabil[i].NRLANCCONT = checkedrows[i].NRLANCTBDOC;
                }
            }
            if(contabil.length > 0) {
                contabilizacaoController.exibeContabilizacao(contabil);
            }else ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
        }else ScreenService.showMessage('Marque o(s) título(s) para consulta da(s) contabilização(ões).');
    };
    
    /**
     *Consulta Contabilização
     */
    self.consultaContabilizacaoBaixa = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var currentRow = widget.currentRow;
        var contabil = [];
        
        contabil.push({});
        contabil[0].CDEMPRESA  = currentRow.CDEMPRESA;
        contabil[0].NRLANCCONT = currentRow.NRLANCTBDOC;
        
        if(contabil.length > 0) {
            contabilizacaoController.exibeContabilizacao(contabil);
        }else ScreenService.showMessage('Título não possui(em) contabilização.');
    };


    /**
     * Habilita Action Títulos a Receber
     */
    self.habilitaActionsTitulo = function(){
        var widget = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widget.getCheckedRows();
        var cont_assoc = 0;
        var cont_prept = 0;

        var actions = {
            associacao : false, desassociacao : false, prorrogacao : false, emissao : false,
            cancemi : false, remessa : false, cancremessa : false, baixa : false,
            cancbaixa : false, consctb : false, envio : false, enviocarta : false, cadauto : true,
            desctitulo : false, efetrec : false, cancdesc : false, cancrec : false, cancrenfin: false,
            preptitulos : false
        };

        if(checkedrows.length > 0) {
            for (var i in checkedrows){

                actions = self.validaActionsSituacaoTitulo(checkedrows[i], actions);
                if(actions.preptitulos) {
                    cont_prept++;
                    actions.preptitulos = cont_prept > 1;
                }

                actions = self.validaActionsRemessaTitulo(checkedrows[i], actions);

                actions = self.validaActionsAssociacaoTitulo(checkedrows[i], actions);
                if(actions.associacao) {
                    cont_assoc++;
                    actions.associacao = cont_assoc > 1;
                }

                actions = self.validaActionsRenegociacaoTitulo(checkedrows[i], actions);

                actions = self.validaActionsContabilizacaoTitulo(checkedrows[i], actions);

            }
        }

        widget.getAction('Associar').isVisible                 = actions.associacao;
        widget.getAction('Desassociar').isVisible              = actions.desassociacao;
        widget.getAction('Prorrogar').isVisible                = actions.prorrogacao;
        widget.getAction('Emitir').isVisible                   = actions.emissao;
        widget.getAction('CancelarEmissao').isVisible          = actions.cancemi;
        widget.getAction('Remessa').isVisible                  = actions.remessa;
        widget.getAction('CancelaRemessa').isVisible           = actions.cancremessa;
        widget.getAction('Baixar').isVisible                   = actions.baixa;
        widget.getAction('CancelarBaixa').isVisible            = actions.cancbaixa;
        widget.getAction('ConsultaCtbTitulo').isVisible        = actions.consctb;
        widget.getAction('EnviarTitulo').isVisible             = actions.envio;
        widget.getAction('EnviarCarta').isVisible              = actions.enviocarta;
        widget.getAction('CadastramentoAutomatico').isVisible  = actions.cadauto;
        widget.getAction('RenegociarTitulo').isVisible         = actions.renfin;
        widget.getAction('CancelarRenegociarTitulo').isVisible = actions.cancrenfin;
        widget.getAction('DescontaTitulo').isVisible           = actions.desctitulo;
        widget.getAction('EfetivaRecebimento').isVisible       = actions.efetrec;
        widget.getAction('CancelaRecebimento').isVisible       = actions.cancrec;
        widget.getAction('CancelaTituloDescontado').isVisible  = actions.cancdesc;
        widget.getAction('PrepararTitulos').isVisible          = actions.preptitulos;

    };

    /**
     * Valida Actions Situacao Titulo
     */
    self.validaActionsSituacaoTitulo = function(checkedrows, actions){
        if (checkedrows.IDSTATBREC == 'A') {
            actions.prorrogacao  = true;
            actions.emissao      = true;
            actions.baixa        = true;
            actions.envio        = true;
            actions.desctitulo   = true;
            actions.renfin       = true;
            actions.preptitulos  = true;
            if (checkedrows.IDSTATUSREC == 'A'){
                actions.enviocarta   = true;
            }
        }else if (checkedrows.IDSTATBREC == 'E'){
            actions.baixa        = true;
            actions.envio        = true;
            actions.emissao      = true;
            actions.desctitulo   = true;
            if (checkedrows.IDSTATUSREC == 'A'){
                actions.enviocarta   = true;
            }
            if(checkedrows.IDSITUCNABR == '00') {
                actions.cancemi  = true;
            }
        }else if (checkedrows.IDSTATBREC == 'P'){
            actions.baixa        = true;
            actions.cancbaixa    = true;
            if (checkedrows.IDSTATUSREC == 'A'){
                actions.enviocarta   = true;
            }
        }else if (checkedrows.IDSTATBREC == 'B'){
            actions.cancbaixa    = true;
        }else if (checkedrows.IDSTATBREC == 'D'){
            actions.baixa        = true;
            if(!checkedrows.DTBAIXAREC) {
                  actions.efetrec  = true;
                  actions.cancdesc = true;
            }else actions.cancrec  = true;
        }
        
        if (checkedrows.IDSTATUSREC == 'A'){
            actions.enviocarta   = true;
        }

        return actions;
    };

    /**
     * Valida Actions Remessa Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsRemessaTitulo = function(checkedrows, actions){
        if (checkedrows.IDSITUCNABR == '00') {
            if(checkedrows.IDSTATBREC == 'A' || checkedrows.IDSTATBREC == 'E')
                actions.remessa = true;
        }else if (checkedrows.IDSITUCNABR != '90'){
            actions.cancremessa = true;
            actions.consremessa = true;
        }else actions.consremessa = true;

        return actions;
    };

    /**
     * Valida Actions Renegociação Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsRenegociacaoTitulo = function(checkedrows, actions){
        if((checkedrows.IDSTATBREC == 'A') && (checkedrows.IDRENEGOCFIN == 'N'))
            actions.renfin     = true;
        else if((checkedrows.IDSTATBREC != 'B') && (checkedrows.IDRENEGOCFIN == 'S'))
            actions.cancrenfin = true;
        return actions;
    };

    /**
     * Valida Actions Associacao Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsAssociacaoTitulo = function(checkedrows, actions){
        if(checkedrows.IDSTATBREC == 'A')
            if (checkedrows.IDASSOCBOLREC != 'S')
                actions.associacao = true;
            else actions.desassociacao = true;

        return actions;
    };

    /**
     * Valida Actions Contabilizacao Titulo
     * @param checkedrows
     * @param actions
     * @returns {*}
     */
    self.validaActionsContabilizacaoTitulo = function(checkedrows, actions){
        if (checkedrows.NRLANCTBDOC != null)
            actions.consctb = true;

        return actions;
    };

    /**
     * Abre Envio de Título por E-mail
     */
    self.abreEnvioTituloPorEmail = function(){
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedRows   = widgetTitulos.getCheckedRows();
        var emitirTitulo  = false;

        if(checkedRows.length > 0) {
            for (var i = 0; i < checkedRows.length; i++) {
                if ((checkedRows[i].IDSTATBREC != 'A') && (checkedRows[i].IDSTATBREC != 'E')) {
                    ScreenService.showMessage('Operação bloqueada. Marque somente título(s) em status "Em Aberto".');
                    return;
                }
                if (!(checkedRows[i].CDREDUCONT)) {
                    ScreenService.showMessage('Operação bloqueada. Existe(m) títulos(s) marcado(s) sem conta cobrança.');
                    return;
                }
                if(checkedRows[i].IDSTATBREC == 'A'){
                    emitirTitulo = true;
                }
            }

            if(emitirTitulo){
                ScreenService.confirmMessage('Existe(m) título(s) marcado(s) pendente de emissão. Deseja emití-lo(s) para prosseguir com o envio de e-mail?', 'question', function() {
                    self.confirmaAberturaEnvioTituloPorEmail();
                }, function (){
                    ScreenService.closePopup();
                });
            }else{
                self.confirmaAberturaEnvioTituloPorEmail();
            }
        }else ScreenService.showMessage('Marque o(s) título(s) para envio por e-mail.');
    };

    /**
     * Confirma Abertura Envio de Título por E-mail
     */
    self.confirmaAberturaEnvioTituloPorEmail = function(){
        var widgetEnviaTitulo = templateManager.container.getWidget('EnviaTituloPorEmailForm');
        widgetEnviaTitulo.isVisible = true;
        ScreenService.closePopup();
        ScreenService.openSwipe(widgetEnviaTitulo);
        self.buscaPadraoEnvioTituloPorEmail(widgetEnviaTitulo);
    };

    /**
     * Busca Padrão Envio Titulo Por Email
     */
    self.buscaPadraoEnvioTituloPorEmail = function (widget){
        Rec00001_TitulosAReceber_Service.buscaEndefili(getLocalVar('CDFILIAL'), 'P').then(function (response_endefili){
            var endefili    = response_endefili.dataset.endefili[0];
            var dsemailfili = endefili.DSEMAILFILI;

            var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');

            var assunto  = '[EMPRESA] - Emissão de Boleto Bancário - Vencimento [VENCIMENTO]';
            var mensagem =
                'Prezado(a) Senhor(a): [CLIENTE]\n\n' +
                'A [EMPRESA] vem por meio deste comunicar a emissão de um novo boleto bancário para V.Sa.\n\n' +
                'Para imprimir, abra o documento em anexo no e-mail.\n\n' +
                'Em caso de dúvidas entre em contato conosco.\n\n' +
                'Atenciosamente,\n' +
                '[EMPRESA]';

            widget.getField('DSEMAILFILI').setValue(dsemailfili);
            widget.getField('DSASSUEMAIL').setValue(assunto);
            widget.getField('DSMENSEMAIL').setValue(mensagem);

            var destinatarioEnviaTituloPorEmailGrid = templateManager.container.getWidget('DestinatarioEnviaTituloPorEmailGrid');
            var datasource = [];

            for (var i = 0; i < widgetTitulos.getCheckedRows().length; i++) {
                datasource[i] = {
                    NRINSJURCLIE   : widgetTitulos.getCheckedRows()[i].NRINSJURCLIE,
                    NMRAZSOCCLIE   : widgetTitulos.getCheckedRows()[i].NMRAZSOCCLIE,
                    DSEMAILCLIE    : widgetTitulos.getCheckedRows()[i].DSEMAILCLIE,
                    DSEMAILCLIECOP : widgetTitulos.getCheckedRows()[i].DSEMAILCLIECOP
                };
            }

            destinatarioEnviaTituloPorEmailGrid.dataSource.data = datasource;
            templateManager.updateTemplate();
        });
    };

    /**
     * Simula Parcelamento Automatico
     * @param widget
     */
    self.simulaParcelamentoAutomatico = function(widget){
        // VERIFICAR SE POSSUI AJUSTE OU ESTÁ ASSOCIADO!!!!
        if(widget.isValid()){
            var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
            var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');

            // Varíaveis dos Campos em Tela
            var nrparcelas  = widget.getField('NRPARCELAS').value();
            var dtprimvenc  = widget.getField('DTPRIMVENC').value();
            var cdmotiparc  = widget.getField('CDMOTIPARC').value();
            var dsmotiparc  = widget.getField('DSMOTIPARC').value();
            var vrfinalbrec = parseFloat(widget.getField('VRFINALBREC').value()).toFixed(2);
            var idproxvenc  = widget.getField('IDPROXVENC').value();
            var nrdiasvenc  = widget.getField('NRDIASVENC').value();

            // Variavel do DataSource
            var datasource = [];
            var dtvencbolrec = moment(dtprimvenc, 'DD/MM/YYYY').format('DD/MM/YYYY');

            // Calculo de Valores das Parcelas
            var vrparcela = parseFloat(vrfinalbrec / nrparcelas).toFixed(2);
            var vrtotparc = vrparcela * nrparcelas;
            var vrultparc = ((vrtotparc != vrfinalbrec)?
                parseFloat((vrfinalbrec)-(vrtotparc)+parseFloat(vrparcela)).toFixed(2)
                : vrparcela);

            for(var i = 1; i <= nrparcelas; i++){
                datasource[i-1] = {
                    'NRORG'        : widgetTitulos.currentRow.NRORG,
                    'CDFILIAL'     : widgetTitulos.currentRow.CDFILIAL,
                    'NRLANBOLREC'  : ((i == 1)?
                        widgetTitulos.currentRow.NRLANBOLREC
                        :null),
                    'NRORIGBOLREC'     : widgetTitulos.currentRow.NRLANBOLREC,
                    'NRPARCBOLREC'     : retornaNumeroParcela(i),
                    'NRPARCBOLRECVISU' : ("00"+ i.toString()).substr(-3),
                    'DTVENBOLREC'      : dtvencbolrec,
                    'VRFINALBREC'      : ((i != nrparcelas)? vrparcela : vrultparc),
                    'CDMOTIPARC'       : cdmotiparc,
                    'DSMOTIPARC'       : dsmotiparc,
                    'IDSTATBREC'       : 'A'
                };

                dtvencbolrec = ((idproxvenc == 'F')?
                    moment(dtvencbolrec, 'DD/MM/YYYY').add(1, 'months').format('DD/MM/YYYY')
                    : moment(dtvencbolrec, 'DD/MM/YYYY').add(nrdiasvenc, 'days').format('DD/MM/YYYY'));
            }

            if(datasource.length > 0) {
                var parcelamentoAutomaticoGrid = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');
                parcelamentoAutomaticoGrid.dataSource.data = datasource;
                widgetParcelamentoAutomatico.getAction('finish').isVisible = true;
                templateManager.updateTemplate();
            }
        }
    };

    /**
     * Altera Campo Proximos Vencimentos
     * @param widget
     */
    self.alteraCampoProximosVencimentos = function(widget){
        var idproxvenc = widget.getField('IDPROXVENC').value();

        if(idproxvenc == 'F') {
            widget.getField('NRDIASVENC').readOnly = true;
            delete widget.getField('NRDIASVENC').validations.required;
            delete widget.getField('NRDIASVENC').validations.min;
            widget.getField('NRDIASVENC').setValue(0);
        }else{
            widget.getField('NRDIASVENC').readOnly = false;
            widget.getField('NRDIASVENC').validations.required = true;
            widget.getField('NRDIASVENC').validations.min = {"limit" : 0};
            widget.getField('NRDIASVENC').setValue(30);
        }
        
        templateManager.updateTemplate();
    };

    /**
     * Busca Padrão Parcelamento Automático
     */
    self.buscaPadraoParcelamentoAutomatico = function (widget){
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetParcelamento = templateManager.container.getWidget('ParcelamentoGrid');
        var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoGrid');

        widget.getField('NRPARCELAS').setValue(2);
        widget.getField('CDMOTIPARC').setValue('');
        widget.getField('DSMOTIPARC').setValue('');
        widget.getField('VRFINALBREC').setValue(widgetParcelamento.dataSource.data[0].VRFINALBREC);
        widget.getField('DTPRIMVENC').setValue(widgetParcelamento.dataSource.data[0].DTVENBOLREC);
        widget.getField('IDPROXVENC').setValue('F');
        widget.getField('NRDIASVENC').setValue(0);

        widgetParcelamentoAutomatico.getAction('finish').isVisible = false;
        widgetParcelamentoAutomatico.dataSource.data = [];

        templateManager.updateTemplate();
    };

    /**
     * Envia Título por E-mail
     */
    self.enviaTituloPorEmail = function(){
        var widgetTitulos     = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetEnviaTitulo = templateManager.container.getWidget('EnviaTituloPorEmailForm');

        var dataSet = [widgetTitulos.getCheckedRows(), widgetEnviaTitulo.currentRow];

        General.callRestEngine('fin_contaReceber#/enviaTituloPorEmail', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Título(s) enviado(s) por e-mail sucesso.');
            ScreenService.closeSwipe();

            widgetEnviaTitulo.isVisible = false;

            widgetTitulos.dataSource.unSelectAll();
            self.unsetStripeDataFooter();
            self.forceActivate();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Abre Cadastramento Automatico
     */
    self.abreCadastramentoAutomatico = function(){
        if(!templateManager.container.getWidget('TitulosAReceberGrid').appliedFilter.FILTER_ROW.CDEMPRESA) {
            ScreenService.showMessage('Operação bloqueada. Empresa não selecionada no filtro.');
            return;
        }

        var widgetTitulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetCadastramentoAutomatico = templateManager.container.getWidget('CadastramentoAutomaticoForm');
        widgetCadastramentoAutomatico.isVisible = true;
        ScreenService.openPopup(widgetCadastramentoAutomatico);
        self.buscaPadraoCadastramentoAutomatico(widgetTitulosAReceberGrid, widgetCadastramentoAutomatico);
    };
    
    /*
     * Função responsável por alterar o  valor de CDINSCMUNIEMIT e CDINSCESTAEMIT.
     * Sera atribuido o valor que retorna no datasource do campo DSSERIE
     */
    self.changeSerie = function(widget){
        
        var serie =  widget.getField("CDSERIE");
        var dsSerie =  widget.getField("DSSERIE");

        var serieFiltrada = dsSerie.dataSource.data.filter(function(dados){
                return serie.value() == dados.CDSERIE;
        });

        widget.getField("CDINSCMUNIEMIT").setValue(serieFiltrada[0].CDINSCMUNI);
        widget.getField('CDINSCESTAEMIT').setValue(serieFiltrada[0].CDINSCESTA);
        
        console.log(serieFiltrada); 
    };

    /**
     * Busca Padrão Cadastramento Automático
     */
    self.buscaPadraoCadastramentoAutomatico = function (widgetTitulosAReceberGrid, widgetCadastramentoAutomatico){
        var checkedRows = widgetTitulosAReceberGrid.getCheckedRows();
        var lastChecked = checkedRows ? checkedRows[0] : null;
        
        if(checkedRows.length > 0){
            var dataDadosRecebimento = widgetTitulosAReceberGrid.dataSource.data;
            var dataDadosRecebimentoFiltrado = dataDadosRecebimento.filter(function(dados){
                return checkedRows[0].NRNOTAFISC == dados.NRNOTAFISC;
            });
        }

        var cdfiliemit      = '';
        var nrtamnumnfut    = '';
        var cdempresaemit   = '';
        var cdinscestaemit  = '';
        var cdinscmuniemit  = '';
        var nmfilialemit    = '';
        var cdcliente       = '';
        var nrinsjurclie    = '';
        var idtpijurclie    = '';
        var nmrazsocclie    = '';
        var cdtipooper      = '';
        var nmtipooper      = '';
        var cdserie         = '';
        var dsserie         = '';
        var vrbolrec        = '';
        var dtprimvenc      = moment().format('DD/MM/YYYY');
        var cdtpctreceb     = '';
        var dstpctreceb     = '';
        var dsdefactrec     = '';
        var dscontarec      = '';
        var cdclassfina     = '';

        if(lastChecked) {

            var parfilempr  = self.buscaParamfilialFilialEmprcntrfil(lastChecked.CDEMPRESA, lastChecked.CDFILIEMIT);
            var tipooperac  = self.buscaTipooperac(lastChecked.CDTIPOOPER);
            var serienffili = self.buscaSerienfFilial(lastChecked.CDEMPRESA, lastChecked.CDFILIEMIT, lastChecked.CDSERIE);
            var tipoctreceb = self.buscaTipoctreceber(lastChecked.CDTPCTRECEB);

            ZHPromise.all([parfilempr, tipooperac, serienffili, tipoctreceb]).then(function(response){
                parfilempr   = response[0].dataset.paramfilial_filial_emprcntrfil[0];
                tipooperac   = response[1].dataset.tipooperac[0];
                serienffili  = response[2].dataset.serienf_filial[0];
                tipoctreceb  = response[3].dataset.tipoctreceber[0];

                cdfiliemit      = (parfilempr) ? parfilempr.CDFILIEMIT : '';
                nmfilialemit    = (parfilempr) ? parfilempr.NMFILIEMIT : '';
                cdempresaemit   = (parfilempr) ? parfilempr.CDEMPRESA  : '';
                cdinscestaemit  = (serienffili) ? serienffili.CDINSCESTA : '';
                cdinscmuniemit  = (serienffili) ? serienffili.CDINSCMUNI : '';
                nrtamnumnfut    = (parfilempr) ? parfilempr.NRTAMNUMNFUT > 0 ? parfilempr.NRTAMNUMNFUT : 9 : '';
                cdcliente       = lastChecked.CDCLIENTE;
                nrinsjurclie    = lastChecked.NRINSJURCLIE;
                idtpijurclie    = lastChecked.IDTPIJURCLIE;
                nmrazsocclie    = lastChecked.NMRAZSOCCLIE;
                nmtipooper      = (tipooperac) ? tipooperac.NMTIPOOPER : '';
                cdtipooper      = (tipooperac) ? tipooperac.CDTIPOOPER : '';
                dsserie         = (serienffili) ? serienffili.DSSERIE : '';
                cdserie         = (serienffili) ? serienffili.CDSERIE : '';
                vrbolrec        = lastChecked.VRBOLREC;
                dtprimvenc      = lastChecked.DTVENBOLREC;
                cdtpctreceb     = (tipoctreceb) ? tipoctreceb.CDTPCTRECEB : '';
                dstpctreceb     = (tipoctreceb) ? tipoctreceb.DSTPCTRECEB : '';
                dsdefactrec     = (tipoctreceb) ? tipoctreceb.DSDEFACTREC : '';
                
                // dscontarec      = (tipoctreceb) ? tipoctreceb.DSDEFACTREC : '';
                if(dataDadosRecebimentoFiltrado[0].DSBOLETREC){
                    dscontarec      = (tipoctreceb) ? dataDadosRecebimentoFiltrado[0].DSBOLETREC : '';
                } else {
                    dscontarec      = (tipoctreceb) ? tipoctreceb.DSDEFACTREC : '';
                }
                
                cdclassfina     = (tipoctreceb) ? tipoctreceb.CDCLASSFINA : '';

                self.setaPadraoCadastramentoAutomatico(
                    widgetCadastramentoAutomatico, nrtamnumnfut, cdempresaemit, cdinscestaemit, cdinscmuniemit,
                    nmfilialemit, cdfiliemit, cdcliente, nrinsjurclie, idtpijurclie, nmrazsocclie, nmtipooper,
                    cdtipooper, dsserie, cdserie, vrbolrec, dtprimvenc, dstpctreceb, cdtpctreceb, dsdefactrec,
                    dscontarec, cdclassfina
                );
            });
        }else {
            self.setaPadraoCadastramentoAutomatico(
                widgetCadastramentoAutomatico, nrtamnumnfut, cdempresaemit, cdinscestaemit, cdinscmuniemit,
                nmfilialemit, cdfiliemit, cdcliente, nrinsjurclie, idtpijurclie, nmrazsocclie, nmtipooper,
                cdtipooper, dsserie, cdserie, vrbolrec, dtprimvenc, dstpctreceb, cdtpctreceb, dsdefactrec,
                dscontarec, cdclassfina
            );
        }
    };

    /**
     * Seta Padrão Cadastramento Automático
     */
    self.setaPadraoCadastramentoAutomatico = function (
        widgetCadastramentoAutomatico, nrtamnumnfut, cdempresaemit, cdinscestaemit, cdinscmuniemit,
        nmfilialemit, cdfiliemit, cdcliente, nrinsjurclie, idtpijurclie, nmrazsocclie, nmtipooper,
        cdtipooper, dsserie, cdserie, vrbolrec, dtprimvenc, dstpctreceb, cdtpctreceb, dsdefactrec,
        dscontarec, cdclassfina
    ){

        widgetCadastramentoAutomatico.getField('NRORG').setValue(getLocalVar('NRORG'));
        widgetCadastramentoAutomatico.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widgetCadastramentoAutomatico.getField('NRTITULOS').setValue(1);
        widgetCadastramentoAutomatico.getField('NRTAMNUMNFUT').setValue(nrtamnumnfut);
        widgetCadastramentoAutomatico.getField('CDEMPRESAEMIT').setValue(cdempresaemit);
        widgetCadastramentoAutomatico.getField('CDINSCESTAEMIT').setValue(cdinscestaemit);
        widgetCadastramentoAutomatico.getField('CDINSCMUNIEMIT').setValue(cdinscmuniemit);
        widgetCadastramentoAutomatico.getField('NMFILIALEMIT').setValue(nmfilialemit);
        widgetCadastramentoAutomatico.getField('CDFILIEMIT').setValue(cdfiliemit);
        widgetCadastramentoAutomatico.getField('IDSTATUS').setValue('A');
        widgetCadastramentoAutomatico.getField('CDCLIENTE').setValue(cdcliente);
        widgetCadastramentoAutomatico.getField('NRINSJURCLIE').setValue(nrinsjurclie);
        widgetCadastramentoAutomatico.getField('IDTPIJURCLIE').setValue(idtpijurclie);
        widgetCadastramentoAutomatico.getField('NMRAZSOCCLIE').setValue(nmrazsocclie);
        widgetCadastramentoAutomatico.getField('CDTIPOOPER').setValue(cdtipooper);
        widgetCadastramentoAutomatico.getField('NMTIPOOPER').setValue(nmtipooper);
        widgetCadastramentoAutomatico.getField('CDSERIE').setValue(cdserie);
        widgetCadastramentoAutomatico.getField('DSSERIE').setValue(dsserie);
        widgetCadastramentoAutomatico.getField('VRNOTAFISC').setValue(vrbolrec);
        widgetCadastramentoAutomatico.getField('DTPRIMVENC').setValue(dtprimvenc);
        widgetCadastramentoAutomatico.getField('IDPROXVENC').setValue('F');
        widgetCadastramentoAutomatico.getField('NRDIASVENC').setValue(0);
        widgetCadastramentoAutomatico.getField('CDTPCTRECEB').setValue(cdtpctreceb);
        widgetCadastramentoAutomatico.getField('DSTPCTRECEB').setValue(dstpctreceb);
        widgetCadastramentoAutomatico.getField('DSDEFACTREC').setValue(dsdefactrec);
        widgetCadastramentoAutomatico.getField('DSCONTAREC').setValue(dscontarec);
        widgetCadastramentoAutomatico.getField('CDCLASSFINA').setValue(cdclassfina);

        templateManager.updateTemplate();
    };

    /**
     * Gera Cadastramento Automático Títulos a Receber/Recebidos
     */
    self.geraCadastramentoAutomatico = function(){
        var widgetTitulosAReceberGrid     = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetCadastramentoAutomatico = templateManager.container.getWidget('CadastramentoAutomaticoForm');

        if(widgetCadastramentoAutomatico.isValid()) {
            Rec00001_TitulosAReceber_Service.geraCadastramentoAutomatico(widgetCadastramentoAutomatico.currentRow).then(function () {
                ScreenService.showMessage('Registros salvos com sucesso.');
                ScreenService.closePopup(widgetCadastramentoAutomatico);
                ScreenService.closePopup();

                widgetCadastramentoAutomatico.isVisible = false;
                
                widgetTitulosAReceberGrid.unSelectAll();
                self.unsetStripeDataFooter();
                
                widgetTitulosAReceberGrid.activate();
                widgetTitulosAReceberGrid.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }
    };

    /**
     * Abre Cadastramento Automatico
     */
    self.abreDescontaTitulo = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widgetTitulosAReceber.getCheckedRows();
        var cdreducont  = null;

        for(var i in checkedrows) {
            if ((checkedrows[i].IDSTATBREC != 'A') && (checkedrows[i].IDSTATBREC != 'E')) {
                ScreenService.showMessage('Operação bloqueada. Marque somente título(s) em status "Em Aberto".');
                return;
            }
            if (checkedrows[i].IDSITUCNABR != '00') {
                ScreenService.showMessage('Operação bloqueada. Título(s) enviado(s), via CNAB, não pode(m) ser alterado(s).');
                return;
            }
            if (checkedrows[i].IDSTATUS == 'V') {
                ScreenService.showMessage('Operação bloqueada. Título(s) previsto(s) não pode(m) ser alterado(s).');
                return;
            }
            if (!checkedrows[i].CDREDUCONT) {
                ScreenService.showMessage('Operação bloqueada. Título(s) está(ão) sem conta cobrança.');
                return;
            }
            if((cdreducont) && (cdreducont != checkedrows[i].CDREDUCONT)) {
                ScreenService.showMessage('Operação bloqueada. Título(s) deve(m) pertencer a mesma conta cobrança.');
                return;
            }else cdreducont = checkedrows[i].CDREDUCONT;
        }

        var widgetDescontaTitulo  = templateManager.container.getWidget('DescontaTituloForm');
        widgetDescontaTitulo.isVisible = true;
        ScreenService.openPopup(widgetDescontaTitulo);
        self.buscaPadraoDescontaTitulo(widgetTitulosAReceber, widgetDescontaTitulo);
    };

    /**
     * Busca Padrão Cadastramento Automático
     */
    self.buscaPadraoDescontaTitulo = function (widgetTitulosAReceber, widgetDescontaTitulo){
        widgetDescontaTitulo.getField('NRORG').setValue(getLocalVar('NRORG'));
        widgetDescontaTitulo.getField('CDFILIAL').setValue(getLocalVar('CDFILIAL'));
        widgetDescontaTitulo.getField('CDOPERADOR').setValue(getLocalVar('CDOPERADOR'));
        widgetDescontaTitulo.getField('CDEMPRESA').setValue(widgetTitulosAReceber.appliedFilter.FILTER_ROW.CDEMPRESA);
        widgetDescontaTitulo.getField('DTMOVICRED').setValue(moment().format('DD/MM/YYYY'));
        widgetDescontaTitulo.getField('DSTPCREDBANC').setValue('');
        widgetDescontaTitulo.getField('VRMOVICRED').setValue('');
        widgetDescontaTitulo.getField('DSCONTCORR').setValue('');
        widgetDescontaTitulo.getField('NRDOCMBANC').setValue('');
        widgetDescontaTitulo.getField('DSMOVICRED').setValue('');
    };

    /**
     * Envia Título por E-mail
     */
    self.salvaTituloDescontado = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetDescontaTitulo  = templateManager.container.getWidget('DescontaTituloForm');

        if(widgetDescontaTitulo.isValid()) {
            var dataSet = [widgetDescontaTitulo.currentRow, widgetTitulosAReceber.getCheckedRows()];

            General.callRestEngine('fin_contaReceber#/salvaTituloDescontado', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Título(s) descontado(s) com sucesso.');
                ScreenService.closePopup(widgetDescontaTitulo);
                ScreenService.closePopup();

                widgetDescontaTitulo.isVisible = false;
                
                self.unsetStripeDataFooter();
                
                widgetTitulosAReceber.dataSource.unSelectAll();
                widgetTitulosAReceber.reload();
                widgetTitulosAReceber.activate();
            }).catch(function (erro) {
                ScreenService.showMessage(erro.data.error);
            });
        }
    };

    /**
     * Envia Título por E-mail
     */
    self.cancelaTituloDescontado = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widgetTitulosAReceber.getCheckedRows();

        for(var i in checkedrows) {
            if ((checkedrows[i].IDSTATBREC != 'D')) {
                ScreenService.showMessage('Operação bloqueada. Marque somente título(s) em status "Descontado".');
                return;
            }
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o(s) título(s) descontado(s) marcado(s)?', 'question', function() {

            General.callRestEngine('fin_contaReceber#/cancelaTituloDescontado', 'DataSet', [checkedrows]).then(function () {
                ScreenService.showMessage('Título(s) descontado(s) cancelado(s) com sucesso.');
                ScreenService.closePopup();

                widgetTitulosAReceber.activate();
                widgetTitulosAReceber.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
                widgetTitulosAReceber.reload();
            }).catch(function (erro) {
                ScreenService.showMessage(erro.data.error);
            });
        });
    };

    /**
     * Envia Título por E-mail
     */
    self.efetivarRecebimentoDescontado = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widgetTitulosAReceber.getCheckedRows();
        var cdreducont = null;

        for(var i in checkedrows) {
            if ((checkedrows[i].IDSTATBREC != 'D')) {
                ScreenService.showMessage('Operação bloqueada. Marque somente título(s) em status "Descontado".');
                return;
            }
            if ((checkedrows[i].DTBAIXAREC)) {
                ScreenService.showMessage('Operação bloqueada. Existe(m) titulo(s) selecionado(s) com recebimento já efetivado.');
                return;
            }
            if((cdreducont) && (cdreducont != checkedrows[i].CDREDUCONT)) {
                ScreenService.showMessage('Operação bloqueada. Título(s) deve(m) pertencer a mesma conta cobrança.');
                return;
            }else cdreducont = checkedrows[i].CDREDUCONT;
        }

        ScreenService.confirmMessage('Deseja realmente efetivar recebimento do(s) título(s) descontado(s) marcado(s)?', 'question', function() {
            General.callRestEngine('fin_contaReceber#/efetivarRecebimentoDescontado', 'DataSet', [checkedrows]).then(function () {
                ScreenService.showMessage('Título(s) descontado(s) efetivado recebimento com sucesso.');
                ScreenService.closePopup();

                widgetTitulosAReceber.activate();
                widgetTitulosAReceber.reload();
            }).catch(function (erro) {
                ScreenService.showMessage(erro.data.error);
            });
        });
    };

    /**
     * Cancela Título Retorno
     */
    self.cancelaTituloRetorno = function(){
        var route = 'fin_contaReceber#/cancelaTituloRetornoCob';
        var widgetCnabRetorno        = templateManager.container.getWidget('CnabRetornoForm');
        var widgetCnabRetornoTitulo  = templateManager.container.getWidget('CnabRetornoTituloForm');
        var checkedRowsCnabRetTitulo = widgetCnabRetornoTitulo.getCheckedRows();

        for (var i in checkedRowsCnabRetTitulo) {
            if (checkedRowsCnabRetTitulo[i].IDSITUCNABR == '90') {
                ScreenService.showMessage('Operação Bloqueada. Título(s) marcado(s) com baixa via CNAB não pode(m) ser cancelado(s) do retorno.');
                return;
            }
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o(s) título(s) do retorno?', 'question', function () {
            var dataset = {
                retorno : widgetCnabRetorno.currentRow,
                boletos : widgetCnabRetornoTitulo.getCheckedRows()
            };

            General.callRestEngine(route, 'DataSet', dataset)
                .then(function(){
                    ScreenService.showMessage('Título(s) cancelado(s) do retorno com sucesso.', 'checked');
                    widgetCnabRetornoTitulo.dataSource.unSelectAll();
                    self.unsetStripeDataFooter();
                    widgetCnabRetornoTitulo.reload();
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        });
    };

    /**
     * Abre Importação Retorno Título
     */
    self.abreImportacaoRetornoTitulo = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        /*widgetTitulosAReceber.appliedFilter.FILTER_ROW.CDEMPRESA;
        widgetTitulosAReceber.appliedFilter.FILTER_ROW.NMEMPRESA;*/

        var widgetImportacaoRetorno = templateManager.container.getWidget('ImportacaoRetornoForm');
        widgetImportacaoRetorno.isVisible = true;
        self.widgetEmEdicao('ImportacaoRetornoForm', true);
        ScreenService.openSwipe(widgetImportacaoRetorno);
        ScreenService.closePopup(true);
        self.buscaPadraoImportacaoRetornoTitulo(widgetTitulosAReceber, widgetImportacaoRetorno);
    };

    /**
     * Widget em modo de Edição
     * @param widget_name
     */
    self.widgetEmEdicao = function(widget_name, editing){
        var widget = templateManager.container.getWidget(widget_name);
        widget.editing = editing;
        templateManager.updateTemplate();
    };

    /**
     * Desabilita Abas
     * @param widget
     */
    self.desabilitaAbas = function(widget_name, disable){
        var widget = templateManager.container.getWidget(widget_name);
        
        if(disable){
            General.changeWidgetAcessibility(widget.getParent().widgets, !disable);
            General.changeWidgetAcessibility([widget.getParent()], !disable);
            General.changeWidgetAcessibility([widget], disable);
        }else{
            General.changeWidgetAcessibility(widget.getParent().widgets, !disable);
            General.changeWidgetAcessibility([widget.getParent()], !disable);
        }
    };

    /**
     * Busca Padrão Importação Retorno Título
     */
    self.buscaPadraoImportacaoRetornoTitulo = function(widgetTitulosAReceber, widgetImportacaoRetorno){
        widgetImportacaoRetorno.getField('CDEMPRESA').setValue(widgetTitulosAReceber.appliedFilter.FILTER_ROW.CDEMPRESA);
        widgetImportacaoRetorno.getField('NMEMPRESA').setValue(widgetTitulosAReceber.appliedFilter.FILTER_ROW.NMEMPRESA);
        widgetImportacaoRetorno.getField('NMBANCO').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABREC').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABREC').readOnly = true;

        self.deleteDataRetorno();

        templateManager.updateTemplate();
    };

    /**
     *
     * @param widgetImportacaoRetorno
     */
    self.validaVisualizacaoImportacao = function(widgetImportacaoRetorno){
        widgetImportacaoRetorno.getField('DSARQCNABREC').setValue('');
        widgetImportacaoRetorno.getField('DSARQCNABREC').readOnly = !widgetImportacaoRetorno.getField('NMBANCO').value();

        self.deleteDataRetorno();

        templateManager.updateTemplate();
    };

    /**
     * Prepara Arquivo de Retorno
     * @param widget
     */
    self.preparaArquivoRetorno = function(widget){
        var route = 'fin_contaReceber#/carregaRetornoCobranca';
        
        var arqRetorno      = widget.currentRow.DSARQCNABREC[0];
        var conteudoArquivo = arqRetorno.b64File.split(',')[1];
        var empresa         = widget.currentRow.CDEMPRESA;
        
        var row = {
            'CDEMPRESA': empresa,
            'CDBANCO': widget.currentRow.CDBANCO,
            'CDBANCCNAB': widget.currentRow.CDBANCCNAB,
            'CNABRETORNO': conteudoArquivo
        };

        General.callRestEngine(route, 'Row', row)
            .then(function(response){
                self.setDataSourceRetorno(response.dataset.BOLETOS);
                self.setDataSourceOcorrencias(response.dataset.OCORRENCIAS);
                widget.widgets[0].SITUACAOCNAB = response.dataset.SITUACAOCNAB[0];
            }).catch(function(erro){
                widget.currentRow.DSARQCNABREC = [];
                ScreenService.showMessage(erro.data.error);
            });
    };

    /**
     * Data Source Retorno
     * @param boletos
     */
    self.setDataSourceRetorno = function(boletos){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        widgetBoletos.dataSource.data = [];
        widgetBoletos.dataSource.data = boletos;
    };

    /**
     * Data Source Ocorrências
     * @param ocorrencias
     */
    self.setDataSourceOcorrencias = function(ocorrencias){
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = ocorrencias;
    };

    /**
     * Insere Retorno Título
     */
    self.insereRetornoTitulo = function(){
        var widgetTitulos     = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetBoletos     = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');

        if (widgetBoletos.getParent().isValid()){
            if (widgetBoletos.dataSource.data.length > 0) {
                if (self.validaSituacaoCnab(widgetBoletos)) {
                    var dataSet = [];
                    var route = 'fin_contaReceber#/insereRetornoCobranca';

                    var form = {
                        'NOMEARQUIVO' : widgetBoletos.getParent().currentRow.DSARQCNABREC[0].name,
                        'CDBANCCNAB'  : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDBANCO'     : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDEMPRESA'   : widgetBoletos.getParent().currentRow.CDEMPRESA
                    };

                    dataSet.push(form);
                    dataSet.push(widgetBoletos.dataSource.data);
                    dataSet.push(widgetOcorrencias.ocorrencias);

                    General.callRestEngine(route, 'DataSet', dataSet)
                        .then(function () {
                            ScreenService.goBack().then(function () {
                                ScreenService.showMessage('Registro salvo com sucesso.', 'checked');
                                widgetBoletos.getParent().isVisible = false;
                                widgetTitulos.activate();
                                widgetTitulos.reload();
                            });
                        }).catch(function (erro) {
                            ScreenService.showMessage(erro.data.error);
                        });

                } else ScreenService.showMessage('Registro(s) do arquivo de retorno possui(em) ocorrência(s) que não possibilita(m) o salvamento.');
            }else ScreenService.showMessage('Não existe(m) título(s) para importação.');
        }
    };

    /**
     * Valída Situação CNAB
     * @param boletos
     * @returns {boolean}
     */
    self.validaSituacaoCnab = function(boletos){
        var boletoValido = !boletos.dataSource.data.every(function(elem) {return elem.SITUACAOBOLETO == false;});
        return (boletos.SITUACAOCNAB && boletoValido) ? true : false;
    };

    /**
     * Deleta Data Retorno
     */
    self.deleteDataRetorno = function(){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoRetornoTituloGrid');
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetBoletos.dataSource.data = [];
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = [];
    };

    /**
     * Abre Ocorrências
     * @param widget
     */
    self.abreOcorrencias = function(widget){
        var widgetOcorrencias = templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid');
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == widget.currentRow.CDLINHA || ocorrencia.GERAL);
        });
        //templateManager.container.getWidget('OcorrenciaImportacaoRetornoGrid').activate();
    };

    /**
     * Abre Renegociação Financeira
     */
    self.abreRenegociacaoFinanceira = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widgetTitulosAReceber.getCheckedRows();
        var cdcliente   = null;
        var cdtpctreceb = null;

        self.buscaGeralRec().then(function(response_geralrec) {

            var geral_rec = response_geralrec.dataset.geral_rec[0];

            if(!geral_rec.CDTPBXRENEGO){
                ScreenService.showMessage('Operação bloqueada. Tipo de baixa padrão para renegociação não parametrizado nos parâmetros de títulos a receber.');
                return;
            }

            if(!geral_rec.CDAJACRERENEGO){
                ScreenService.showMessage('Operação bloqueada. Tipo de ajuste de acréscimo padrão para renegociação não parametrizado nos parâmetros de títulos a receber.');
                return;
            }

            if(!geral_rec.CDAJDESCRENEGO){
                ScreenService.showMessage('Operação bloqueada. Tipo de ajuste de desconto padrão para renegociação não parametrizado nos parâmetros de títulos a receber.');
                return;
            }

            for(var i in checkedrows) {
                if(checkedrows[i].IDRENEGOCFIN == 'S') {
                    ScreenService.showMessage('Operação Bloqueada. Existe(m) título(s) marcado(s) já renegociado(s).');
                    return;
                }

                if(checkedrows[i].IDSTATUS == 'V') {
                    ScreenService.showMessage('Operação Bloqueada. Título(s) previsto(s) não pode(m) ser renegociado(s).');
                    return;
                }

                if(checkedrows[i].IDSTATBREC != 'A') {
                    ScreenService.showMessage('Operação Bloqueada. Somente título(s) em status "Em Aberto" pode(m) ser renegociado(s).');
                    return;
                }

                // Valida Cliente
                if(cdcliente){
                    if(cdcliente != checkedrows[i].CDCLIENTE){
                        ScreenService.showMessage('Operação bloqueada. Títulos devem pertencer ao mesmo cliente.');
                        return;
                    }
                }else{
                    cdcliente = checkedrows[i].CDCLIENTE;
                }

                // Valida Tipo de Conta
                if(cdtpctreceb){
                    if(cdtpctreceb != checkedrows[i].CDTPCTRECEB){
                        ScreenService.showMessage('Operação bloqueada. Títulos devem conter o mesmo tipo de conta.');
                        return;
                    }
                }else{
                    cdtpctreceb = checkedrows[i].CDTPCTRECEB;
                }

            }

            var widgetRenegociacaoFin = templateManager.container.getWidget('RenegociacaoFinanceiraForm');

            widgetRenegociacaoFin.isVisible = true;
            ScreenService.openSwipe(widgetRenegociacaoFin);
            ScreenService.closePopup(true);

            self.buscaPadraoRenegociacaoFinanceira(widgetTitulosAReceber, widgetRenegociacaoFin, geral_rec);
        });
    };

    /**
     * Busca Padrão Renegociação Financeira
     */
    self.buscaPadraoRenegociacaoFinanceira = function (widgetTitulosAReceber, widgetRenegociacaoFin, geral_rec){

        var vrliquido   = 0;
        var vrcalculado = 0;
        var nrdiasvenc  = 0;
        var dtatual     = moment().format('DD/MM/YYYY');
        var dtvenbolrec = moment().format('DD/MM/YYYY');
        var dataatual, datavenci;

        widgetTitulosAReceber.getCheckedRows().forEach(function (data) {
            vrliquido += data.VRFINALBREC;

            dtvenbolrec = moment(data.DTVENBOLREC, 'DD/MM/YYYY').format('DD/MM/YYYY');
            dataatual = moment(dtatual, 'DD/MM/YYYY');
            datavenci = moment(dtvenbolrec, 'DD/MM/YYYY');

            if(datavenci.isBefore(dataatual)){
                nrdiasvenc = moment(dtatual, 'DD/MM/YYYY').diff(moment(dtvenbolrec, 'DD/MM/YYYY'), 'days');
                vrcalculado += parseFloat(data.VRFINALBREC + (data.VRMULTABREC + (nrdiasvenc * (data.VRFINALBREC * data.VRPEMORBREC / 100))));
            }else vrcalculado += parseFloat(data.VRFINALBREC);

        });

        var checkedRows = widgetTitulosAReceber.getCheckedRows();
        var lastChecked = checkedRows ? checkedRows[0] : null;

        var nrorg        = lastChecked.NRORG;
        var cdfilial     = lastChecked.CDFILIAL;
        var cdempresa    = lastChecked.CDEMPRESA;
        var cdfiliemit   = lastChecked.CDFILIEMIT;
        var cdcliente    = lastChecked.CDCLIENTE;
        var nrinsjurclie = lastChecked.NRINSJURCLIE;
        var idtpijurclie = lastChecked.IDTPIJURCLIE;
        var cdserie      = lastChecked.CDSERIE;
        var cdtipooper   = lastChecked.CDTIPOOPER;
        var cdtpctreceb  = lastChecked.CDTPCTRECEB;
        var cdtpbxrenego = geral_rec.CDTPBXRENEGO;

        var geralrec    = self.buscaGeralRec();
        var parfilempr  = self.buscaParamfilialFilialEmprcntrfil(cdempresa, cdfiliemit);
        var tipooperac  = self.buscaTipooperac(cdtipooper);
        var serienffili = self.buscaSerienfFilial(cdempresa, cdfiliemit, cdserie);
        var tipoctreceb = self.buscaTipoctreceber(cdtpctreceb);

        ZHPromise.all([geralrec, parfilempr, tipooperac, serienffili, tipoctreceb]).then(function(response){


            geralrec     = response[0].dataset.geral_rec[0];
            parfilempr   = response[1].dataset.paramfilial_filial_emprcntrfil[0];
            tipooperac   = response[2].dataset.tipooperac[0];
            serienffili  = response[3].dataset.serienf_filial[0];
            tipoctreceb  = response[4].dataset.tipoctreceber[0];

            var cdtpbxrenego    = (geralrec) ? geralrec.CDTPBXRENEGO  : '';
            var cdajacrerenego  = (geralrec) ? geralrec.CDAJACRERENEGO  : '';
            var cdajdescrenego  = (geralrec) ? geralrec.CDAJDESCRENEGO  : '';
            var cdempresaemit   = (parfilempr) ? parfilempr.CDEMPRESA  : '';
            var cdinscestaemit  = (serienffili) ? serienffili.CDINSCESTA : '';
            var cdinscmuniemit  = (serienffili) ? serienffili.CDINSCMUNI : '';
            var nrtamnumnfut    = (parfilempr) ? parfilempr.NRTAMNUMNFUT > 0 ? parfilempr.NRTAMNUMNFUT : 9 : '';
            var dstpctreceb     = (tipoctreceb) ? tipoctreceb.DSTPCTRECEB : '';
            var cdclassfina     = (tipoctreceb) ? tipoctreceb.CDCLASSFINA : '';
            var dtprimvenc      = moment().format('DD/MM/YYYY');

            self.setaPadraoRenegociacaoFinanceira(
                widgetRenegociacaoFin, nrorg, cdfilial, cdfiliemit, cdempresaemit, cdinscestaemit, cdinscmuniemit,
                nrtamnumnfut, cdcliente, nrinsjurclie, idtpijurclie, cdtipooper, cdserie, cdtpctreceb, dstpctreceb,
                cdclassfina, vrliquido, vrcalculado, dtprimvenc, cdtpbxrenego, cdajacrerenego, cdajdescrenego
            );
            
            widgetRenegociacaoFin.getField('NRDIASVENC').readOnly = true;

            var renegociacaoFinanceira = templateManager.container.getWidget('RenegociacaoFinanceiraGrid');
            renegociacaoFinanceira.dataSource.data = [];

            templateManager.updateTemplate();
        });
    };

    /**
     * Seta Padrão Renegociação Financeira
     */
    self.setaPadraoRenegociacaoFinanceira = function (
        widgetRenegociacaoFin, nrorg, cdfilial, cdfiliemit, cdempresaemit, cdinscestaemit, cdinscmuniemit,
        nrtamnumnfut, cdcliente, nrinsjurclie, idtpijurclie, cdtipooper, cdserie, cdtpctreceb, dstpctreceb,
        cdclassfina, vrliquido, vrcalculado, dtprimvenc, cdtpbxrenego, cdajacrerenego, cdajdescrenego
    ){

        widgetRenegociacaoFin.currentRow.NRORG          = nrorg;
        widgetRenegociacaoFin.currentRow.CDFILIAL       = cdfilial;
        widgetRenegociacaoFin.currentRow.CDOPERADOR     = getLocalVar('CDOPERADOR');
        widgetRenegociacaoFin.currentRow.NRTAMNUMNFUT   = nrtamnumnfut;
        widgetRenegociacaoFin.currentRow.CDEMPRESAEMIT  = cdempresaemit;
        widgetRenegociacaoFin.currentRow.CDINSCESTAEMIT = cdinscestaemit;
        widgetRenegociacaoFin.currentRow.CDINSCMUNIEMIT = cdinscmuniemit;
        widgetRenegociacaoFin.currentRow.CDFILIEMIT     = cdfiliemit;
        widgetRenegociacaoFin.currentRow.IDSTATUS       = 'A';
        widgetRenegociacaoFin.currentRow.CDCLIENTE      = cdcliente;
        widgetRenegociacaoFin.currentRow.NRINSJURCLIE   = nrinsjurclie;
        widgetRenegociacaoFin.currentRow.IDTPIJURCLIE   = idtpijurclie;
        widgetRenegociacaoFin.currentRow.CDTIPOOPER     = cdtipooper;
        widgetRenegociacaoFin.currentRow.CDSERIE        = cdserie;
        widgetRenegociacaoFin.currentRow.DTPRIMVENC     = dtprimvenc;
        widgetRenegociacaoFin.currentRow.DSCONTCORR     = '';
        widgetRenegociacaoFin.currentRow.DSRENEGOFIN    = '';
        widgetRenegociacaoFin.currentRow.IDAJUSTE       = 'A';
        widgetRenegociacaoFin.currentRow.NRPARCELAS     = 1;
        widgetRenegociacaoFin.currentRow.VRLIQUIDO      = vrliquido;
        widgetRenegociacaoFin.currentRow.VRCALCULADO    = vrcalculado.toFixed(2);
        widgetRenegociacaoFin.currentRow.VRNEGOCIADO    = vrcalculado.toFixed(2);
        widgetRenegociacaoFin.currentRow.IDPROXVENC     = 'F';
        widgetRenegociacaoFin.currentRow.NRDIASVENC     = 0;
        widgetRenegociacaoFin.currentRow.VRAJUSTE       = "0.00";
        widgetRenegociacaoFin.currentRow.VRPERAJUSTE    = "0.00";
        widgetRenegociacaoFin.currentRow.CDTPCTRECEB    = cdtpctreceb;
        widgetRenegociacaoFin.currentRow.DSTPCTRECEB    = dstpctreceb;
        widgetRenegociacaoFin.currentRow.CDCLASSFINA    = cdclassfina;
        widgetRenegociacaoFin.currentRow.CDTPBXRENEGO   = cdtpbxrenego;
        widgetRenegociacaoFin.currentRow.CDAJACRERENEGO = cdajacrerenego;
        widgetRenegociacaoFin.currentRow.CDAJDESCRENEGO = cdajdescrenego;

        templateManager.updateTemplate();
    };

    /**
     * Cálcula Valor para Percentual Ajuste Títulos a Receber/Recebidos
     * @param args
     */
    self.calcValorPercAjusteId = function(args){
        var row = args.row;
        var vrajuste    = parseFloat(row.VRAJUSTE.replace(',','.'));
        var vrcalculado = parseFloat(row.VRCALCULADO);

        row.VRPERAJUSTE = parseFloat((vrajuste * 100)/vrcalculado).toFixed(2);

        if(row.IDAJUSTE == 'A')
             row.VRNEGOCIADO = parseFloat(vrcalculado + vrajuste).toFixed(2);
        else row.VRNEGOCIADO = parseFloat(vrcalculado - vrajuste).toFixed(2);
    };

    /**
     * Cálcula Valor para Percentual Ajuste Títulos a Receber/Recebidos
     * @param args
     */
    self.calcValorPercAjuste = function(args){
        var row = args.row;
        var vrajuste    = parseFloat(row.VRAJUSTE.replace('.','').replace(',','.'));
        var vrcalculado = parseFloat(row.VRCALCULADO);

        row.VRPERAJUSTE = parseFloat((vrajuste * 100)/vrcalculado).toFixed(2);

        if(row.IDAJUSTE == 'A')
             row.VRNEGOCIADO = parseFloat(vrcalculado + vrajuste).toFixed(2);
        else row.VRNEGOCIADO = parseFloat(vrcalculado - vrajuste).toFixed(2);
    };

    /**
     * Cálcula Percentual para Valor Ajuste Títulos a Receber/Recebidos
     * @param args
     */
    self.calcPercValorAjuste = function(args){
        var row = args.row;
        var vrperajuste = parseFloat(row.VRPERAJUSTE.replace(',','.'));
        var vrcalculado = parseFloat(row.VRCALCULADO);

        row.VRAJUSTE    = parseFloat((vrperajuste * vrcalculado)/100).toFixed(2);

        if(row.IDAJUSTE == 'A')
             row.VRNEGOCIADO = parseFloat(vrcalculado + parseFloat(row.VRAJUSTE)).toFixed(2);
        else row.VRNEGOCIADO = parseFloat(vrcalculado - parseFloat(row.VRAJUSTE)).toFixed(2);
    };

    /**
     * Simula Renegociação Financeira
     * @param widget
     */
    self.simulaRenegociacaoFinanceira = function(widget){
        if(widget.isValid()){
            var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');

            // Varíaveis dos Campos em Tela
            var nrparcelas  = widget.getField('NRPARCELAS').value();
            var dtprimvenc  = widget.getField('DTPRIMVENC').value();
            var idproxvenc  = widget.getField('IDPROXVENC').value();
            var nrdiasvenc  = widget.getField('NRDIASVENC').value();
            var vrnegociado = parseFloat(widget.getField('VRNEGOCIADO').value()).toFixed(2);

            // Variavel do DataSource
            var datasource = [];
            var dtvencbolrec = moment(dtprimvenc, 'DD/MM/YYYY').format('DD/MM/YYYY');

            // Calculo de Valores das Parcelas
            var vrparcela = parseFloat(vrnegociado / nrparcelas).toFixed(2);
            var vrtotparc = vrparcela * nrparcelas;
            var vrultparc = ((vrtotparc != vrnegociado)?
                parseFloat((vrnegociado)-(vrtotparc)+parseFloat(vrparcela)).toFixed(2)
                : vrparcela);

            for(var i = 1; i <= nrparcelas; i++){
                datasource[i-1] = {
                    'NRORG'        : widget.getField('NRORG').value(),
                    'CDFILIAL'     : widget.getField('CDFILIAL').value(),
                    'NRPARCBOLREC' : ("0"+ i.toString()).substr(-2),
                    'DTVENBOLREC'  : dtvencbolrec,
                    'VRFINALBREC'  : ((i != nrparcelas)?vrparcela:vrultparc)
                };

                dtvencbolrec = ((idproxvenc == 'F')?
                    moment(dtvencbolrec, 'DD/MM/YYYY').add(1, 'months').format('DD/MM/YYYY')
                    : moment(dtvencbolrec, 'DD/MM/YYYY').add(nrdiasvenc, 'days').format('DD/MM/YYYY'));
            }

            if(datasource.length > 0) {
                var renegociacaoFinanceira = templateManager.container.getWidget('RenegociacaoFinanceiraGrid');
                renegociacaoFinanceira.dataSource.data = datasource;
                templateManager.updateTemplate();
            }
        }
    };

    /**
     * Confirma Renegociação Financeira
     */
    self.confirmaRenegociacaoFinanceira = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetRenegFinanForm  = templateManager.container.getWidget('RenegociacaoFinanceiraForm');
        var widgetRenegFinanGrid  = templateManager.container.getWidget('RenegociacaoFinanceiraGrid');
        var vrtotalreneg = 0;

        if(widgetRenegFinanGrid.dataSource.data.length < 1){
            ScreenService.showMessage('Operação bloqueada. Simule a renegociação financeira para confirmar a operação.');
            return;
        }

        for(var i in widgetRenegFinanGrid.dataSource.data) {
            if ($.currency.unmaskString(widgetRenegFinanGrid.dataSource.data[i].VRFINALBREC) <= 0) {
                ScreenService.showMessage('Operação bloqueada. Existe(m) parcela(s) menor(es) ou igual(is) a zero.');
                return;
            }

            vrtotalreneg += parseFloat(widgetRenegFinanGrid.dataSource.data[i].VRFINALBREC);
        }

        if(parseFloat(vrtotalreneg).toFixed(2) != parseFloat(widgetRenegFinanForm.currentRow.VRNEGOCIADO).toFixed(2)){
            ScreenService.showMessage('Operação bloqueada. Valor negociado direfente do somatório das parcelas.');
            return;
        }

        var dataSource = [
            widgetRenegFinanForm.currentRow,
            widgetRenegFinanGrid.dataSource.data,
            widgetTitulosAReceber.getCheckedRows()
            ];

        General.callRestEngine('fin_contaReceber#/confirmaRenegociacaoFinanceira', 'DataSet', dataSource).then(function () {
            ScreenService.showMessage('Registro salvo com sucesso.');
            ScreenService.closeSwipe();

            widgetRenegFinanForm.isVisible = false;

            widgetTitulosAReceber.dataSource.unSelectAll();
            widgetTitulosAReceber.activate();
            widgetTitulosAReceber.reload();
            self.forceActivate();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /*
     * Confirmar Baixa Titulo a Receber
     * @param args
     */
    self.confirmaBaixaTituloDescontado = function(args){
        var widget = args.owner.widget;
        var widgetTitulos = templateManager.container.getWidget('TitulosAReceberGrid');

        var checkedrows     = widgetTitulos.getCheckedRows();
        var checkedrows_des = checkedrows.filter(function(filtro){return (filtro.IDSTATBREC == 'D');});

        var currentRow = {
            nrorg           : getLocalVar('NRORG')   ,
            cdfilial        : getLocalVar('CDFILIAL'),
            baixaboleto     : widget.currentRow      ,
            boletosmarcados : checkedrows_des
        };

        if(widget.isValid()) {

            if(widget.currentRow.IDCTBLBXREC == 'S') {
                Rec00001_TitulosAReceber_Service.buscaContOpeCtbl(widgetTitulos.getCheckedRows()[0].CDEMPRESA).then(function (response) {
                    var contopectbl  = response.dataset.contopectbl[0];
                    
                    if(!contopectbl){
                        ScreenService.showMessage('Operação bloqueada. Período contábil financeiro não parametrizado.');
                        return;
                    }
                    
                    var dtinicctbl  = contopectbl.DTINICCTBL;
                    var dtfinacctbl = contopectbl.DTFINACCTBL;

                    if(dtinicctbl && dtfinacctbl) {
                        if ((moment(dtinicctbl, 'DD/MM/YYYY')) > (moment(widget.currentRow.DTBAIXAREC, 'DD/MM/YYYY')) ||
                            (moment(dtfinacctbl, 'DD/MM/YYYY')) < (moment(widget.currentRow.DTBAIXAREC, 'DD/MM/YYYY'))) {
                            ScreenService.showMessage('Operação bloqueada. Data da baixa fora do período contábil financeiro.');
                            return;
                        }
                    }

                    self.confirmaBaixaTituloDescontadoSave(currentRow, widget, widgetTitulos);
                }, function (error) {
                    ScreenService.showMessage(error.data.error);
                });
            }else{
                self.confirmaBaixaTituloDescontadoSave(currentRow, widget, widgetTitulos);
            }
        }
    };

    /**
     *
     * @param currentRow
     * @param widget
     * @param widgetTitulos
     */
    self.confirmaBaixaTituloDescontadoSave = function(currentRow, widget, widgetTitulos){
        var boletosEmAberto = widgetTitulos.getCheckedRows().filter(function(boleto){
            return (widgetTitulos.IDSTATBREC == 'A')||(widgetTitulos.IDSTATBREC == 'E');
        });
        var boletosBaixaPlanejada = widgetTitulos.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBREC == 'P';
        });

        Rec00001_TitulosAReceber_Service.baixaTituloReceber(currentRow).then(function () {
            if(boletosBaixaPlanejada.length > 0) {
                ScreenService.confirmMessage('Existe(m) título(s) marcado(s) com baixa(s) planejada(s).Deseja transfomá-la(s) em baixa(s) real(is)?', 'question', function () {
                    self.baixarTitulosMarcadosOuTitulosBaixa();
                }, function () {
                    if (boletosEmAberto.length > 0) {
                        self.efetivarBaixaTitulosAbertos(boletosEmAberto);
                    } else {
                        ScreenService.closePopup();
                    }
                });
            }else if(boletosEmAberto.length > 0){
                self.efetivarBaixaTitulosAbertos(boletosEmAberto);
            }else {
                ScreenService.showMessage('Título(s) descontados baixado(s) com sucesso.');
                ScreenService.closePopup(widget);
                widget.isVisible = false;
                ScreenService.closePopup();
                widgetTitulos.reload();
            }
        }, function (error) {
            if (contabilizacaoController.checkErrosContabilizacao(error)) {
                ScreenService.showMessage(error.data.error);
            }
        });
    };

    /**
     * Envia Título por E-mail
     */
    self.cancelarRecebimentoDescontado = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows = widgetTitulosAReceber.getCheckedRows();

        for(var i in checkedrows) {
            if ((checkedrows[i].IDSTATBREC != 'D')) {
                ScreenService.showMessage('Operação bloqueada. Marque somente título(s) em status "Descontado".');
                return;
            }
            if ((!checkedrows[i].DTBAIXAREC)) {
                ScreenService.showMessage('Operação bloqueada. Existe(m) titulo(s) selecionado(s) com recebimento não efetivado(s).');
                return;
            }
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o recebimento do(s) título(s) descontado(s) marcado(s)?', 'question', function() {
            General.callRestEngine('fin_contaReceber#/cancelarRecebimento', 'DataSet', [checkedrows]).then(function () {
                ScreenService.showMessage('Título(s) descontado(s) cancelado recebimento com sucesso.');
                ScreenService.closePopup();

                widgetTitulosAReceber.activate();
                widgetTitulosAReceber.dataSource.unSelectAll();
                self.unsetStripeDataFooter();
                widgetTitulosAReceber.reload();
            }).catch(function (erro) {
                ScreenService.showMessage(erro.data.error);
            });
        });
    };

    /**
     * Cancela Renegociação Financeira
     */
    self.cancelaRenegociacaoFinanceira = function(){
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var checkedrows           = widgetTitulosAReceber.getCheckedRows();
        var checkedrows_reneg     = checkedrows.filter(function(filtro){return (filtro.IDRENEGOCFIN == 'S')||(filtro.IDSTATBREC != 'B');});

        ScreenService.confirmMessage('Deseja realmente cancelar a(s) renegociação(ões) financeira(s) do(s) título(s) marcado(s)?', 'question', function() {

            General.callRestEngine('fin_contaReceber#/cancelaRenegociacaoFinanceira', 'DataSet', [checkedrows_reneg]).then(function () {
                ScreenService.showMessage('Renegociação(ões) cancelada(s) com sucesso.');
                ScreenService.closePopup();

                widgetTitulosAReceber.activate();
                widgetTitulosAReceber.reload();
            }).catch(function (erro) {
                ScreenService.showMessage(erro.data.error);
            });
        });
    };


    /**
     * Ativa Histórico de Renegociação Financeira
     */
    self.ativaHistoritoRenegociacao = function(widget){
        if(widget.currentRow.IDRENEGOCFIN == 'S')
             templateManager.container.getWidget('RenegociacaoTabbed').isVisible = true;
        else templateManager.container.getWidget('RenegociacaoTabbed').isVisible = false;
    };

    self.validaRangeFiltroCnab = function(cnabInicial, cnabFinal, cnabAlterada){
        if (cnabInicial.value() > cnabFinal.value()){
            cnabAlterada.setValue('');
            ScreenService.showMessage(cnabInicial.label + ' deve ser menor que o  ' + cnabFinal.label);
        }
    };

    /**
     * Fecha PopUp Parcelamento Automatico
     * @param widget
     */
    self.closeSwipeParcelamento = function (widget_name){
        var widget = templateManager.container.getWidget(widget_name);
        var widgetTitulosAReceber = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetParcelamentoAutomatico = templateManager.container.getWidget('ParcelamentoAutomaticoForm');
        
        widget.isVisible = false;
        ScreenService.closeSwipe();
        widgetParcelamentoAutomatico.editing = false;
        
        //widgetTitulosAReceber.currentRow.NRCNABRECRET = widgetTitulosAReceber.currentRow.NRCNABRECRETANT;
        widgetTitulosAReceber.currentRow.NRCNABRECREM = widgetTitulosAReceber.currentRow.NRCNABRECREMANT;
        widgetTitulosAReceber.currentRow.IDSTATUSEMIT = widgetTitulosAReceber.currentRow.IDSTATUSEMITANT;
        //widgetTitulosAReceber.currentRow.NRCNABRECRETANT = null;
        widgetTitulosAReceber.currentRow.NRCNABRECREMANT = null;
        widgetTitulosAReceber.currentRow.IDSTATUSEMITANT = null;
        
    };
    
    /**
     * Abre Calculo de Ajuste - Título a Receber
     */
    self.abreCalculoAjuste = function(){
        var widgetCalculoAjuste = templateManager.container.getWidget('CalculoAjusteForm');
        widgetCalculoAjuste.editing   = true;
        widgetCalculoAjuste.isVisible = true;
        ScreenService.openSwipe(widgetCalculoAjuste);
        self.buscaPadraoCalculoAjuste(widgetCalculoAjuste);
        templateManager.updateTemplate();
    };
    
    /**
     * Busca Padrao Calculo de Ajuste
     * @param widgetCalculoAjuste
     */
    self.buscaPadraoCalculoAjuste = function (widgetCalculoAjuste){
        self.buscaGeralRec().then(function(response_geralrec) {
            var currentRowTitulo = templateManager.container.getWidget('NewTitulosBaixaGrid').getLastSelectedRow();
            
            var vrpemulbrec      = currentRowTitulo.VRPEMULBREC ? currentRowTitulo.VRPEMULBREC : 0;     
            var vrmultabrec      = currentRowTitulo.VRMULTABREC ? currentRowTitulo.VRMULTABREC : 0;     
            var vrmultacalcbrec  = parseFloat(parseFloat(currentRowTitulo.VRFINALBREC) + parseFloat(currentRowTitulo.VRMULTABREC)).toFixed(2);
            
            var vrpemorbrec      = currentRowTitulo.VRPEMORBREC  ? currentRowTitulo.VRPEMORBREC  : 0;     
            var vrmordiabrec     = currentRowTitulo.VRMORDIABREC ? currentRowTitulo.VRMORDIABREC : 0;    
            var vrmordiacalcbrec = parseFloat(parseFloat(currentRowTitulo.VRFINALBREC) + parseFloat(currentRowTitulo.VRMORDIABREC)).toFixed(2);
            
            widgetCalculoAjuste.getField('IDJUROS').setValue('S');
            widgetCalculoAjuste.getField('IDJUROSCOMP').setValue('P');
            widgetCalculoAjuste.getField('VRPEMULBREC').setValue(vrpemulbrec);
            widgetCalculoAjuste.getField('VRMULTABREC').setValue(vrmultabrec);
            widgetCalculoAjuste.getField('VRMULTACALCBREC').setValue(vrmultacalcbrec);
            widgetCalculoAjuste.getField('VRPEMORBREC').setValue(vrpemorbrec);
            widgetCalculoAjuste.getField('VRMORDIABREC').setValue(vrmordiabrec);
            widgetCalculoAjuste.getField('VRMORDIACALCBREC').setValue(vrmordiacalcbrec);
            widgetCalculoAjuste.getField('VRPERCENTUAL').setValue(0.0);
            widgetCalculoAjuste.getField('NMINDICE').setValue(null);
            widgetCalculoAjuste.getField('CDINDICE').setValue(null);
            widgetCalculoAjuste.getField('DSAJUCTREC').setValue(null);
            widgetCalculoAjuste.getField('CDAJUCTREC').setValue(null);
            widgetCalculoAjuste.getField('CDAJUMULCTREC').setValue(null);
            widgetCalculoAjuste.getField('CDAJUMULCTREC').setValue(null); 
            
            
            widgetCalculoAjuste.getField('DSAJUMULCTREC').readOnly = false;
            widgetCalculoAjuste.getField('VRPEMULBREC').readOnly   = false;
            widgetCalculoAjuste.getField('VRMULTABREC').readOnly   = false;
            widgetCalculoAjuste.getField('DSAJUMORCTREC').readOnly = false;
            widgetCalculoAjuste.getField('VRPEMORBREC').readOnly   = false;
            widgetCalculoAjuste.getField('VRMORDIABREC').readOnly  = false;
            widgetCalculoAjuste.getField('CDAJUMORCTREC').readOnly = false;
            
            widgetCalculoAjuste.getField('DSAJUCTREC').readOnly    = true;
            widgetCalculoAjuste.getField('IDJUROSCOMP').readOnly   = true;
            widgetCalculoAjuste.getField('VRPERCENTUAL').readOnly  = true;
            widgetCalculoAjuste.getField('NMINDICE').readOnly      = true;
            
            widgetCalculoAjuste.currentRow.VRFINALBREC = widgetCalculoAjuste.getParent().getLastSelectedRow().VRFINALBREC;
            
            var geral_rec = response_geralrec.dataset.geral_rec[0];
            
            var cdajmultarec = geral_rec.CDAJMULTAREC?geral_rec.CDAJMULTAREC:null;
            var cdajmorarec  = geral_rec.CDAJMULTAREC?geral_rec.CDAJMORAREC:null;
            
            if(cdajmultarec){
        		var filter_multa = [
        			{
        				"name": "CDAJUCTREC",
        				"operator": "=",
        				"value": cdajmultarec
        			}
        		];
        
        		General.callRestEngine('fin#/crud/tipoajreceber', 'FilterData', filter_multa).then(function (tipoajreceber) {
        		    var dsajmultarec  = tipoajreceber.dataset.tipoajreceber[0].DSAJUCTREC;
        		    var idajumulctrec = tipoajreceber.dataset.tipoajreceber[0].IDAJUCTREC;
        		    
        		    widgetCalculoAjuste.getField('CDAJUMULCTREC').setValue(cdajmultarec);
        			widgetCalculoAjuste.getField('DSAJUMULCTREC').setValue(dsajmultarec);
        			widgetCalculoAjuste.getField('IDAJUMULCTREC').setValue(idajumulctrec);
        		});
            }
    		
    		if(cdajmorarec){
        		var filter_mora = [
        			{
        				"name": "CDAJUCTREC",
        				"operator": "=",
        				"value": cdajmorarec
        			}
        		];
        		
        		General.callRestEngine('fin#/crud/tipoajreceber', 'FilterData', filter_mora).then(function (tipoajreceber) {
        		    var dsajumorctrec = tipoajreceber.dataset.tipoajreceber[0].DSAJUCTREC;
        		    var idajumorctrec = tipoajreceber.dataset.tipoajreceber[0].IDAJUCTREC;
        		    
        		    widgetCalculoAjuste.getField('CDAJUMORCTREC').setValue(cdajmorarec);
        			widgetCalculoAjuste.getField('DSAJUMORCTREC').setValue(dsajumorctrec);
        			widgetCalculoAjuste.getField('IDAJUMORCTREC').setValue(idajumorctrec);
        		});
            }
            
            
        });
    };
    
    /**
     * Busca Parametrização Tipo de Conta
     * @param args
     * @param widget_name
     */
    self.alteraTipoAjuste = function() {
        var widgetCalculoAjuste = templateManager.container.getWidget('CalculoAjusteForm');
        
        widgetCalculoAjuste.getField('VRPEMULBREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRMULTABREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRMULTACALCBREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRPEMORBREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRMORDIABREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRMORDIACALCBREC').setValue(0.0);
        widgetCalculoAjuste.getField('VRPERCENTUAL').setValue(0.0);
        widgetCalculoAjuste.getField('NMINDICE').setValue(null);
        widgetCalculoAjuste.getField('CDINDICE').setValue(null);
        widgetCalculoAjuste.getField('DSAJUCTREC').setValue(null);
        widgetCalculoAjuste.getField('CDAJUCTREC').setValue(null);
        widgetCalculoAjuste.getField('DSAJUMULCTREC').setValue(null);
        widgetCalculoAjuste.getField('CDAJUMULCTREC').setValue(null); 
        widgetCalculoAjuste.getField('DSAJUMORCTREC').setValue(null); 
        widgetCalculoAjuste.getField('CDAJUMORCTREC').setValue(null); 
        widgetCalculoAjuste.getField('IDJUROSCOMP').setValue("P"); 

        if(widgetCalculoAjuste.getField('IDJUROS').value() == 'S'){
            widgetCalculoAjuste.getField('DSAJUMULCTREC').readOnly = false;
            widgetCalculoAjuste.getField('VRPEMULBREC').readOnly   = false;
            widgetCalculoAjuste.getField('VRMULTABREC').readOnly   = false;
            widgetCalculoAjuste.getField('DSAJUMORCTREC').readOnly = false;
            widgetCalculoAjuste.getField('VRPEMORBREC').readOnly   = false;
            widgetCalculoAjuste.getField('VRMORDIABREC').readOnly  = false;
            widgetCalculoAjuste.getField('CDAJUMORCTREC').readOnly = false;
            
            widgetCalculoAjuste.getField('DSAJUCTREC').readOnly    = true;
            widgetCalculoAjuste.getField('IDJUROSCOMP').readOnly   = true;
            widgetCalculoAjuste.getField('VRPERCENTUAL').readOnly  = true;
            widgetCalculoAjuste.getField('NMINDICE').readOnly      = true;
            
            templateManager.updateTemplate();
        }else{
            widgetCalculoAjuste.getField('DSAJUMULCTREC').readOnly = true;
            widgetCalculoAjuste.getField('VRPEMULBREC').readOnly   = true;
            widgetCalculoAjuste.getField('VRMULTABREC').readOnly   = true;
            widgetCalculoAjuste.getField('DSAJUMORCTREC').readOnly = true;
            widgetCalculoAjuste.getField('VRPEMORBREC').readOnly   = true;
            widgetCalculoAjuste.getField('VRMORDIABREC').readOnly  = true;
            widgetCalculoAjuste.getField('CDAJUMORCTREC').readOnly = true;
            widgetCalculoAjuste.getField('NMINDICE').readOnly      = true;
            
            widgetCalculoAjuste.getField('DSAJUCTREC').readOnly    = false;
            widgetCalculoAjuste.getField('IDJUROSCOMP').readOnly   = false;
            widgetCalculoAjuste.getField('VRPERCENTUAL').readOnly  = false;
            templateManager.updateTemplate();
        }
    };

    /**
     * Desabilita PopUp
     * @param widget
     */
    self.desabilitaPopUp = function (widget_name){
        var widget = templateManager.container.getWidget(widget_name);
        widget.isVisible = false;
        ScreenService.closePopup(widget);
    };
    
    
    /**
     * Desabilita Swipe
     * @param widget
     */
    self.desabilitaSwipe = function (widget_name){
        var widget = templateManager.container.getWidget(widget_name);
        widget.isVisible = false;
        ScreenService.closeSwipe(widget);
    };

    /**
     * Busca Padrão Filial
     * @param widget
     */
    self.buscaPadraoFilial = function(documento) {
        var titulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
        var route = 'fin_contaReceber#/validaFilialLogadaPorEmpresaFiltradaRec';
        var cdempresa = titulosAReceberGrid.appliedFilter.FILTER_ROW.CDEMPRESA;
        General.callRestEngine(route, 'Row', {'CDEMPRESA': cdempresa}).then(function(response_filial){
            if (response_filial.dataset.filial.length > 0){
                var filial   = response_filial.dataset.filial[0];
                documento.currentRow.CDFILIEMIT     = filial.CDFILIAL;
                documento.currentRow.NMFILIALEMIT   = filial.NMFILIAL;
                documento.currentRow.CDEMPRESAEMIT  = filial.CDEMPRESA;
                documento.currentRow.CDINSCESTAEMIT = filial.CDINSCESTA;
                documento.currentRow.CDINSCMUNIEMIT = filial.CDINSCMUNI;
                documento.currentRow.NRTAMNUMNFUT   = filial.NRTAMNUMNFUT;
                self.changeFilial();
            }else{
                documento.currentRow.CDFILIEMIT     = null;
                documento.currentRow.NMFILIALEMIT   = null;
                documento.currentRow.CDEMPRESAEMIT  = null;
                documento.currentRow.CDINSCESTAEMIT = null;
                documento.currentRow.CDINSCMUNIEMIT = null;
                documento.currentRow.NRTAMNUMNFUT   = null;
                documento.currentRow.NRNOTAFISC     = null;
                documento.currentRow.CDSERIE        = null;
                documento.currentRow.DSSERIE        = null;
            }
        }).catch(function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };
    
    /**
     * Cancela Parcelamento
     * @param widget
     */
    self.cancelaParcelamento = function(widget){
        widget.reload();
        templateManager.updateTemplate();
        self.widgetEmEdicao('ParcelamentoGrid', false);
        self.desabilitaAbas('ParcelamentoGrid', false);
    };
    
    /**
     * Habilita Finalização Rateio Conta
     * @param widget
     */
    self.habilitaFinalizacaoRateioConta = function(widget){
        widget.getAction('finish').isVisible = true;
        widget.getAction('back_list').isVisible = false;
        widget.getAction('cancel').isVisible = true;

        self.widgetEmEdicao("RateioDocumentosGrid", true);
        self.desabilitaAbas('RateioDocumentosGrid', true);
        
        templateManager.updateTemplate();
    };

    /**
     * Desabilita Finalização Rateio Conta
     * @param args
     */
    self.desabilitaFinalizacaoRateioConta = function(args){
        args.owner.getAction('finish').isVisible = false;
        args.owner.getAction('back_list').isVisible = true;
        args.owner.getAction('cancel').isVisible = false;
        
        self.widgetEmEdicao("RateioDocumentosGrid", false);
        self.desabilitaAbas('RateioDocumentosGrid', false);
        
        templateManager.updateTemplate();
    };

    /**
     * Abre Rateio Conta Unica
     * @param valor
     */
    self.abreRateioContaUnica = function(){
        var widgetContaDocumentos  = templateManager.container.getWidget('ContaDocumentosGrid');

        if(widgetContaDocumentos.dataSource.data.length == 1){
            if(widgetContaDocumentos.currentRow.NRLANBOLREC != widgetContaDocumentos.dataSource.data[0].NRLANBOLREC){
                widgetContaDocumentos.currentRow = widgetContaDocumentos.dataSource.data[0]?widgetContaDocumentos.dataSource.data[0]:{};
                widgetContaDocumentos.afterMoveRow(widgetContaDocumentos);
                widgetContaDocumentos.view();
            }
        }

    };
    
    /**
     * Cancela Parcelamento
     * @param widget
     */
    self.cancelaRateio = function(widget){
        widget.reload();
        self.widgetEmEdicao('RateioDocumentosGrid', false);
        self.desabilitaAbas('RateioDocumentosGrid', false);
        templateManager.updateTemplate();
    };
    
    /**
     * Formata Valor
     * @param valor
     */
    self.formataValor = function(valor){
        if(typeof valor == 'string')
            if(valor.indexOf(',') >= 0)
                valor = valor.replace('.','').replace(',','.');
                
        valor = parseFloat(parseFloat(valor).toFixed(2));

        return valor;
    };
    
    /**
     * Busca Padrão Título
     */
    self.changeValorDoc = function (){
        // Documento
        var widgetDocumentosForm = templateManager.container.getWidget('DocumentosForm');
        
        // Parcelamento
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
        
        if(self.formataValor(widgetDocumentosForm.currentRow.VRNOTAFISC) > 0){
            widgetParcelamentoDocGrid.currentRow.__index          = 0;
            widgetParcelamentoDocGrid.currentRow.NRPARCBOLREC     = '00';
            widgetParcelamentoDocGrid.currentRow.NRPARCBOLRECVISU = '000';
            widgetParcelamentoDocGrid.currentRow.IDEXCLUIR        = null;
            widgetParcelamentoDocGrid.currentRow.DSCONTAREC       = widgetDocumentosForm.currentRow.DSDEFACTREC?widgetDocumentosForm.currentRow.DSDEFACTREC:widgetDocumentosForm.currentRow.DSTPCTRECEB;
            widgetParcelamentoDocGrid.currentRow.VRFINALBREC      = widgetDocumentosForm.currentRow.VRNOTAFISC;
            widgetParcelamentoDocGrid.dataSource.data             = [widgetParcelamentoDocGrid.currentRow];
        }else{
            widgetParcelamentoDocGrid.currentRow = {};
            widgetParcelamentoDocGrid.dataSource.data = [];
        }
    };
    
    
    /**
     * Busca Parametrização Tipo de Conta
     * @param args
     * @param widget_name
     */
    self.changeTipoContaDoc = function() {
        var widgetDocumentos = templateManager.container.getWidget('DocumentosForm');
        var widgetParcelamentoDoc = templateManager.container.getWidget('ParcelamentoDocGrid');

        var dscontarec = widgetDocumentos.currentRow.DSDEFACTREC?widgetDocumentos.currentRow.DSDEFACTREC:widgetDocumentos.currentRow.DSDEFACTREC;

        widgetParcelamentoDoc.dataSource.data.forEach(function(data){
            data.DSCONTAREC = dscontarec;
        });
        
    };
    
    /**
     * Adicionar Parcela Títulos a Receber
     * @param args
     */
    self.adicionarParcela = function(args){
        var widgetParcelamento = args.owner.widget;
        var widgetDocumentos   = args.owner.widget.getParent();
        
        if(widgetDocumentos.currentRow.VRNOTAFISC <= 0) {
            ScreenService.showMessage('Operação bloqueada. Informe o valor do documento.');
            return;
        }
        
        widgetParcelamento.newRow();
        self.habDesCampos('DocumentosForm', true);
        
        if(widgetParcelamento.dataSource.data.length == 2){
            widgetParcelamento.dataSource.data[0].NRPARCBOLREC     = '01';
            widgetParcelamento.dataSource.data[0].NRPARCBOLRECVISU = '001';
        }
        
        var nrparcbolrec     = retornaNumeroParcela(widgetParcelamento.dataSource.data.length);
        var nrparcbolrecvisu = ("00" + (widgetParcelamento.dataSource.data.length)).slice(-3);
        var dscontarec       = widgetParcelamento.dataSource.data[widgetParcelamento.dataSource.data.length - 2].DSCONTAREC;
        var vrtotalparc      = valorTotalParcelamento(widgetParcelamento);
        var vttotbolrec      = widgetDocumentos.currentRow.VRNOTAFISC;
        
        dscontarec = (!dscontarec)?(widgetDocumentos.currentRow.DSDEFACTREC?widgetDocumentos.currentRow.DSDEFACTREC:widgetDocumentos.currentRow.DSTPCTRECEB):dscontarec;

        widgetParcelamento.currentRow.NRPARCBOLREC     = nrparcbolrec;
        widgetParcelamento.currentRow.NRPARCBOLRECVISU = nrparcbolrecvisu;
        widgetParcelamento.currentRow.DSCONTAREC       = dscontarec;
        widgetParcelamento.currentRow.IDEXCLUIR        = 'S';
        widgetParcelamento.currentRow.VRFINALBREC      = ((vttotbolrec - vrtotalparc) >= 0)?(vttotbolrec - vrtotalparc):0;
        
        widgetParcelamento.dataSource.data[widgetParcelamento.dataSource.data.length - 2].IDEXCLUIR = null;
    };
    
    /**
     * Excluir Parcela Títulos a Receber
     * @param widget
     */
    self.excluirParcela = function(widget){
        widget.list();
        self.habDesCampos('DocumentosForm', false);
        
        widget.dataSource.data.splice(widget.currentRow.__index, 1);
        
        var vrtotalparc = 0;
        var vrnotafisc  = widget.getParent().currentRow.VRNOTAFISC;
        
        widget.dataSource.data.forEach(function(data){
            vrtotalparc += self.formataValor(data.VRFINALBREC);
        });
        
        var vrdifparcnf = (vrnotafisc - vrtotalparc); 
        
        widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBREC = (vrdifparcnf > 0)?self.formataValor(widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBREC) + vrdifparcnf:widget.dataSource.data[widget.currentRow.__index - 1].VRFINALBREC;
        
        if(widget.currentRow.__index == '1'){
            widget.dataSource.data[widget.currentRow.__index - 1].NRPARCBOLREC     = "00";
            widget.dataSource.data[widget.currentRow.__index - 1].NRPARCBOLRECVISU = "000";
        }else widget.dataSource.data[widget.currentRow.__index - 1].IDEXCLUIR = 'S';
    };
    
        /**
     * Excluir Parcela Títulos a Receber
     * @param widget
     */
    self.cancelarParcela = function(widget){
        widget.list();
        self.habDesCampos('DocumentosForm', false);
        
        widget.dataSource.data.forEach(function (data){
            if(data.__is_new == true)
                widget.dataSource.data.splice(data.__index, 1);
        });
        
        widget.dataSource.data.forEach(function(data){
            if(data.hasOwnProperty('DSCONTAREC_inline'))
                delete data.DSCONTAREC_inline;
            if(data.hasOwnProperty('DTVENBOLREC_inline'))
                delete data.DTVENBOLREC_inline;
            if(data.hasOwnProperty('VRFINALBREC_inline'))
                delete data.VRFINALBREC_inline;
                
            if(data.hasOwnProperty('changed_DSCONTAREC')){
                data.DSCONTAREC = data.changed_DSCONTAREC;
                delete data.changed_DSCONTAREC;
            }
            if(data.hasOwnProperty('changed_VRFINALBREC')){
                data.VRFINALBREC = data.changed_VRFINALBREC;
                delete data.changed_VRFINALBREC;
            }
            if(data.hasOwnProperty('changed_DTVENBOLREC')){
                data.DTVENBOLREC = data.changed_DTVENBOLREC;
                delete data.changed_DTVENBOLREC;
            }
            
            if(data.hasOwnProperty('changed'))
                delete data.changed;
            if(data.hasOwnProperty('__is_new'))
                delete data.__is_new;
        });
        
        if(widget.currentRow.__index == '1')
            widget.dataSource.data[widget.dataSource.data.length - 1].NRPARCBOLREC = "00";
        else if (widget.currentRow.__index != '0')
            widget.dataSource.data[widget.dataSource.data.length - 1].IDEXCLUIR = 'S';
    };
    
    /**
     * Excluir Parcela Títulos a Receber
     * @param widget
     */
    self.salvarParcela = function(widget){
        var widgetParcelamentoDocGrid = templateManager.container.getWidget('ParcelamentoDocGrid');
    
        if(widgetParcelamentoDocGrid.isValid()){
            self.habDesCampos('DocumentosForm', false);
            
            if(widget.currentRow.VRFINALBREC <= 0){
                ScreenService.showMessage('Operação bloqueada. Valor da parcela deve ser maior que zero.');
                return;
            }
            
            widget.list();

            widget.dataSource.data.forEach(function(data){
                if(data.hasOwnProperty('DSCONTAREC_inline'))
                    delete data.DSCONTAREC_inline;
                if(data.hasOwnProperty('DTVENBOLREC_inline'))
                    delete data.DTVENBOLREC_inline;
                if(data.hasOwnProperty('VRFINALBREC_inline'))
                    delete data.VRFINALBREC_inline;
                if(data.hasOwnProperty('changed_DSCONTAREC'))
                    delete data.changed_DSCONTAREC;
                if(data.hasOwnProperty('changed_VRFINALBREC'))
                    delete data.changed_VRFINALBREC;
                if(data.hasOwnProperty('changed_DTVENBOLREC'))
                    delete data.changed_DTVENBOLREC;
                if(data.hasOwnProperty('changed'))
                    delete data.changed;
                if(data.hasOwnProperty('__is_new'))
                    delete data.__is_new;
            });
        }else{
            ScreenService.showMessage('Operação bloqueada. Preencha o(s) campo(s) de vencimento(s).');
        }
    };

    /**
     * Excluir Parcela Títulos a Receber
     * @param widget
     */
    self.habDesCampos = function(widget_name, read_only){
        var widget = templateManager.container.getWidget(widget_name);
        
        widget.fields.forEach(function(field){
            field.readOnly = read_only;
        });
    };
    
    
    /**
     * Edita Documento Grid
     * @param widgetParcelamentoDocGrid
     */
    self.validaDataVencimento = function(widgetParcelamentoDocGrid){
        var deferred = ZHPromise.defer();
        var ultdata    = null;
        var datavalida = true;
    
        widgetParcelamentoDocGrid.dataSource.data.forEach(function(data){
            if(ultdata){
                if(ultdata > moment(data.DTVENBOLREC, 'DD/MM/YYYY')){
                    datavalida = false;
                    deferred.reject('Operação bloqueada. Datas de vencimento das parcelas devem seguir uma sequencia crescente.');
                }else ultdata = moment(data.DTVENBOLREC, 'DD/MM/YYYY');
            }else ultdata = moment(data.DTVENBOLREC, 'DD/MM/YYYY');
        });
        
        if(datavalida)
            deferred.resolve(datavalida);
            
        return deferred.promise;
        
    };
    
    /**
     * Novo Ajuste Titulo Baixa
     * @param widget
     */
    self.novoAjusteTituloBaixa = function(){
        var widgetTitulosBaixa      = templateManager.container.getWidget('NewTitulosBaixaGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');

        widgetAjusteTituloBaixa.newRow();
        widgetAjusteTituloBaixa.getField('NRORG').setValue(widgetTitulosBaixa.currentRow.NRORG);
        widgetAjusteTituloBaixa.getField('CDFILIAL').setValue(widgetTitulosBaixa.currentRow.CDFILIAL);
        widgetAjusteTituloBaixa.getField('NRLANBOLREC').setValue(widgetTitulosBaixa.currentRow.NRLANBOLREC);
        widgetAjusteTituloBaixa.getField('IDOPERACAO').setValue('I');
        widgetAjusteTituloBaixa.getField('IDORIGAJBREC').setValue('2');
    };
    
    /**
     * Salva Ajuste Titulo Baixa
     * @param args
     */
    self.salvaAjusteTituloBaixa = function(){
        var widgetBaixa             = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetTitulosBaixa      = templateManager.container.getWidget('NewTitulosBaixaGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');
        
        if(widgetAjusteTituloBaixa.isValid()){
            var currentRowAjuste = widgetAjusteTituloBaixa.currentRow;
            var dataAjuste = widgetAjusteTituloBaixa.dataSource.data;
            
            var cdajuctrec = dataAjuste.filter(function(filter) {
                return (filter.CDAJUCTREC == currentRowAjuste.CDAJUCTREC);
            });
            
            if(cdajuctrec.length == 2){
                ScreenService.showMessage('Operação bloqueado. Ajuste já cadastrado.');
                return;
            }
            
            localStorageEngine.getDataSource('fin_contaReceber#/ajbolrec_tipoajreceber').then(
                function (dataLocal) {
                    dataLocal.push(currentRowAjuste);
                    localStorageEngine.setDataSource('fin_contaReceber#/ajbolrec_tipoajreceber', dataLocal);
                    
                    // Valor de Ajuste do Título
                    var idajuctrec  = widgetAjusteTituloBaixa.currentRow.IDAJUCTREC;
                    idajuctrec      = (idajuctrec == 'A')?'A':'D';
                    var vrajbolrec  = parseFloat(widgetAjusteTituloBaixa.currentRow.VRAJBOLREC);
                    vrajbolrec      = (idajuctrec == 'A')?vrajbolrec:-(vrajbolrec);
        
                    // Atualiza Valor Líquido do Título
                    var vrfinalbrec = parseFloat(widgetTitulosBaixa.currentRow.VRFINALBREC);
                    vrfinalbrec    += vrajbolrec;
                    
                    if(vrfinalbrec < 0){
                        ScreenService.showMessage('Operação bloqueada. Valor do título não pode ficar negativo.');
                        return;
                    }
                    
                    widgetTitulosBaixa.currentRow.VRFINALBREC = vrfinalbrec;
                    widgetTitulosBaixa.currentRow.VRFINALBRECCALC = vrfinalbrec;
                    
                    // Atuliza Valor da Baixa
                    var vrbaixarec = parseFloat(widgetBaixa.currentRow.VRBAIXAREC);
                    vrbaixarec    += vrajbolrec;
                    widgetBaixa.currentRow.VRBAIXAREC = vrbaixarec;
                    
                    ScreenService.goBack();
                    ScreenService.showMessage('Registro salvo com sucesso.');
                    widgetTitulosBaixa.currentRow.__ajusteAtualizado = true;
                    
                    self.recalculaJurosTitulos();
                }
            );
            

        }
    };
    
    /**
     * Exclui Ajuste Título Baixa
     * @param args
    */
    self.excluiAjusteTituloBaixa = function(args){
        var widget = args.owner.widget;
        var widgetBaixa = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetTitulosBaixa = templateManager.container.getWidget('NewTitulosBaixaGrid');

        if(widget.currentRow.IDORIGAJBREC == '4') {
            ScreenService.showMessage('Operação bloqueada. Ajuste originado de uma compensação não pode ser excluído.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente excluir o ajuste?', 'question', function() {
            
            localStorageEngine.getDataSource('fin_contaReceber#/ajbolrec_tipoajreceber').then(
                function (dataLocal) {
                    for(var i in dataLocal){
                        if((dataLocal[i].CDFILIAL    == widget.currentRow.CDFILIAL   ) && 
                           (dataLocal[i].NRLANBOLREC == widget.currentRow.NRLANBOLREC) && 
                           (dataLocal[i].CDAJUCTREC  == widget.currentRow.CDAJUCTREC )){
                            dataLocal[i].IDOPERACAO = 'E';
                            widget.currentRow.IDOPERACAO = 'E';
                        }
                    }
                    
                    localStorageEngine.setDataSource('fin_contaReceber#/ajbolrec_tipoajreceber', dataLocal).then(
                        function () {
                            templateManager.container.getWidget('AjusteTituloBaixaGrid').reload();
                        }
                    );
                    
                     // Valor de Ajuste do Título
                    var idajuctrec  = widget.currentRow.IDAJUCTREC;
                    idajuctrec      = (idajuctrec == 'A')?'A':'D';
                    var vrajbolrec  = parseFloat(widget.currentRow.VRAJBOLREC);
                    vrajbolrec      = (idajuctrec == 'A')?-(vrajbolrec):vrajbolrec;
        
                    // Atualiza Valor Líquido do Título
                    var vrfinalbrec = parseFloat(widgetTitulosBaixa.currentRow.VRFINALBREC);
                    vrfinalbrec    += vrajbolrec;
                    
                    if(vrfinalbrec < 0){
                        ScreenService.showMessage('Operação bloqueada. Valor do título não pode ficar negativo.');
                        return;
                    }
                    
                    widgetTitulosBaixa.currentRow.VRFINALBREC = vrfinalbrec;
                    
                    // Atuliza Valor da Baixa
                    var vrbaixarec = parseFloat(widgetBaixa.currentRow.VRBAIXAREC);
                    vrbaixarec    += vrajbolrec;
                    widgetBaixa.currentRow.VRBAIXAREC = vrbaixarec;
                    
                    ScreenService.goBack();
                    ScreenService.showMessage('Registro excluído com sucesso.');
                    widgetTitulosBaixa.currentRow.__ajusteAtualizado = true;
                }
            );
        });
    };
    
    /**
     * Data Source Ajuste Titulo Baixado
     */
    self.dataSourceAjusteTituloBaixo = function(){
        var widgetTitulosAReceber   = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetAjusteTituloBaixa = templateManager.container.getWidget('AjusteTituloBaixaGrid');
        
        var route = 'fin_contaReceber#/buscaTodosAjustesBoletosRec';
        
        General.callRestEngine(route, 'DataSet', widgetTitulosAReceber.getCheckedRows()).then(function (ajustesboletos) {
            var dadosDeTodosAjustes = ajustesboletos.dataset.buscaTodosAjustesBoletosRec;
            
            localStorageEngine.setDataSource('fin_contaReceber#/ajbolrec_tipoajreceber', dadosDeTodosAjustes);
            
            widgetAjusteTituloBaixa.reload();
        });
    };
    
    this.setaMascCnpjCpfCliente = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURCLIE.length === 14) inscFormatada = value.NRINSJURCLIE.substr(0, 2) + '.' + value.NRINSJURCLIE.substr(2, 3) + '.' + value.NRINSJURCLIE.substr(5, 3) + '/' + value.NRINSJURCLIE.substr(8, 4) + '-' + value.NRINSJURCLIE.substr(12, 2);
          else if(value.NRINSJURCLIE.length === 11) inscFormatada = value.NRINSJURCLIE.substr(0, 3) + '.' + value.NRINSJURCLIE.substr(3, 3) + '.' + value.NRINSJURCLIE.substr(6, 3) + '-' + value.NRINSJURCLIE.substr(9, 2);
          else inscFormatada = null;
          value.NRINSJURCLIE = inscFormatada?inscFormatada:value.NRINSJURCLIE;
	    });
	};
	
	/**
     * Cancela Tipo de Recebimento
     * @param widget
     */
    self.cancelaTipoRecebimento = function(widget){
        widget.reload();
        templateManager.updateTemplate();
        self.widgetEmEdicao('TipoRecebimentoGrid', false);
        self.desabilitaAbas('TipoRecebimentoGrid', false);
    };
    
    this.changeHeaderValues = function(widget){
        var codigo; 
        var nmempresa = widget.owner.appliedFilter.FILTER_ROW.NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "é igual a. ".concat(codigo).concat(' - ').concat(nmempresa)
        };
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        newFilter.push(cdempresa);
        filter.forEach(function(x){
            if((x.value != "é igual a.") && (x.label != "Código Empresa") && (x.label != "Organização")){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a./)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
    };
    
    self.diasDeAtraso = function(){
        var titulos = templateManager.container.getWidget('TitulosAReceberGrid');
        titulos.dataSource.data.forEach(function(data){
        	var dataVencimento = data.DTVENBOLREC;
        	var today = self.getCurrentDate();
            var split;
    
    		split = dataVencimento.split('/');
            dataVencimento = split[1] + "/" +split[0]+"/"+split[2];
    
    		dataVencimento = self.days_between(today, dataVencimento);
    		if(dataVencimento > 0 && data.IDSTATBREC != 'B')
        		data.DTATRVALREC = dataVencimento;
        	else
        	    data.DTATRVALREC = '';
        });
    };
    
    self.getCurrentDate = function(){
        var today = new Date();
        var dd = String(today.getDate()).padStart(2, '0');
        var mm = String(today.getMonth() + 1).padStart(2, '0'); //January is 0!
        var yyyy = today.getFullYear();
        
        today = mm + '/' + dd + '/' + yyyy;
        
        return today;
    };
    
    self.days_between = function(date1, date2) {
        date1 = new Date(date1);
        date2 = new Date(date2);

        // The number of milliseconds in one day
        var ONE_DAY = 1000 * 60 * 60 * 24;
    
        // Convert both dates to milliseconds
        var date1_ms = date1.getTime();
        var date2_ms = date2.getTime();
    
        // Calculate the difference in milliseconds
        var difference_ms = Math.abs(date1_ms - date2_ms);
    
        // Convert back to days and return
        return Math.round(difference_ms/ONE_DAY);
    
    };
    
    self.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctreceb = widget.getField('CDTPCTRECEB');
        
        cdtpctreceb.setValue("");
        cdtpctreceb.dataSourceFilter[0].value = cdempresa;
        cdtpctreceb.reload();
    };
    
     /**
     * Busca Parametrização Tipo de Juros Compostos
     * @param args
     * @param widget_name
     */
    self.alteraTipoJurosCompostos = function() {
        var widgetCalculoTipoJurosCompostos = templateManager.container.getWidget('CalculoAjusteForm');
        widgetCalculoTipoJurosCompostos.getField('NMINDICE').setValue(null);
        widgetCalculoTipoJurosCompostos.getField('CDINDICE').setValue(null);
        widgetCalculoTipoJurosCompostos.getField('VRPERCENTUAL').setValue(0.0);
        
        if(widgetCalculoTipoJurosCompostos.getField('IDJUROSCOMP').value() == 'I'){
            widgetCalculoTipoJurosCompostos.getField('NMINDICE').readOnly     = false;
            
            widgetCalculoTipoJurosCompostos.getField('VRPERCENTUAL').readOnly = true;
            templateManager.updateTemplate();
        }else{
            widgetCalculoTipoJurosCompostos.getField('VRPERCENTUAL').readOnly = false;
            
            widgetCalculoTipoJurosCompostos.getField('NMINDICE').readOnly     = true;
            templateManager.updateTemplate();
        }
    };
    
    /**
     * Confirma Cálculo de Ajuste
     * @param args
     * @param widget_name
     */
    self.confirmaCalculoAjuste = function() {
        var calculoAjusteForm = templateManager.container.getWidget('CalculoAjusteForm');
        var currentRowTitulo  = calculoAjusteForm.getParent().getLastSelectedRow();
        var currentRowAjuste  = calculoAjusteForm.currentRow;
        
        if(calculoAjusteForm.isValid()){
            
            var route = 'fin_contaReceber#/confirmaCalculoAjuste';
            var dataSet = [
                currentRowTitulo,
                currentRowAjuste
            ];
            
            General.callRestEngine(route, 'DataSet', dataSet).then(
                function (result) {
                    ScreenService.closeSwipe();
                    calculoAjusteForm.isVisible = false;
                    
                    var valorcalculado = parseFloat(parseFloat(currentRowTitulo.VRFINALBREC) + parseFloat(currentRowAjuste.VRMULTABREC) + parseFloat(currentRowAjuste.VRMORDIABREC)).toFixed(2);
                    
                    currentRowTitulo.VRFINALBREC = valorcalculado;
                    
                    ScreenService.showMessage('Cálculo de juros realizado com sucesso.');
                }
            );
        }
    };

    /**
     * Altera Tipo de Prorrogação
     */
    self.alteraTipoProrrogacao = function(){
        var widgetProrrogar = templateManager.container.getWidget('ProrrogarForm');
        var idtppror        = widgetProrrogar.getField('IDTPPROR').value();

        if(idtppror == 'F') {
            widgetProrrogar.getField('DTVENBOLREC').readOnly = false;
            widgetProrrogar.getField('DTVENBOLREC').validations.required = true;
            widgetProrrogar.getField('DTVENBOLREC').setValue('');
            
            widgetProrrogar.getField('NRDIASVENC').readOnly  = true;
            delete widgetProrrogar.getField('NRDIASVENC').validations.required;
            delete widgetProrrogar.getField('NRDIASVENC').validations.min;
            widgetProrrogar.getField('NRDIASVENC').clearValue();
        }else{
            widgetProrrogar.getField('DTVENBOLREC').readOnly = true;
            delete widgetProrrogar.getField('DTVENBOLREC').validations.required;
            widgetProrrogar.getField('DTVENBOLREC').setValue('');
            
            widgetProrrogar.getField('NRDIASVENC').readOnly  = false;
            widgetProrrogar.getField('NRDIASVENC').validations.required = true;
            widgetProrrogar.getField('NRDIASVENC').validations.min = {"limit" : 0};
            widgetProrrogar.getField('NRDIASVENC').setValue(30);
        }
        
        templateManager.updateTemplate();
    };
    
    
    
    
    
    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMultaBaixa = function(row){
        row.VRMULTABREC        = row.VRMULTABREC_inline;
        row.VRPEMULBREC        = valueToPerc(row.VRMULTABREC_inline, row.VRFINALBREC);
        row.VRPEMULBREC_inline = row.VRPEMULBREC;
        
        self.calculaValorLiquidoFinal(row);
    };

    /**
     * Cálcula Percentual para Valor Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMultaBaixa = function(row){
        row.VRPEMULBREC        = row.VRPEMULBREC_inline;
        row.VRMULTABREC        = percToValue(row.VRPEMULBREC_inline, row.VRFINALBREC);
        row.VRMULTABREC_inline = row.VRMULTABREC;
        
        self.calculaValorLiquidoFinal(row);
    };
    
    
    /**
     * Cálcula Valor para Percentual Multa Títulos a Receber/Recebidos
     * @param row
     */
    self.calcValorPercMoraBaixa = function(row){
        row.VRMORDIABREC       = row.VRMORDIABREC_inline;
        row.VRPEMORBREC        = valueToPerc(row.VRMORDIABREC_inline, row.VRFINALBREC);
        row.VRPEMORBREC_inline = row.VRPEMORBREC;
        
        self.calculaValorLiquidoFinal(row);
    };

    /**
     * Cálcula Percentual para Valor Mora Títulos a Receber/Recebidos
     * @param row
     */
    self.calcPercValorMoraBaixa = function(row){
        row.VRPEMORBREC         = row.VRPEMORBREC_inline.replace(',','.');
        row.VRMORDIABREC        = percToValue(row.VRPEMORBREC_inline, row.VRFINALBREC);
        row.VRMORDIABREC_inline = row.VRMORDIABREC;
        
        self.calculaValorLiquidoFinal(row);
    };
    
    self.calculaValorLiquidoFinal = function(row){
       var widgetNewBaixaTabbed = templateManager.container.getWidget('NewBaixaTabbed');
        
        row.NRDIASATRASO = self.verificaDiasEmAtraso(widgetNewBaixaTabbed.currentRow.DTBAIXAREC, row.DTVENBOLREC);
        
        row.VRMORDIABREC_inline = row.VRMORDIABREC_inline ? row.VRMORDIABREC_inline : row.VRMORDIABREC;
        row.VRMULTABREC_inline  = row.VRMULTABREC_inline  ? row.VRMULTABREC_inline  : row.VRMULTABREC;
        
        if((row.NRDIASATRASO > 0) && (row.IDJUROS)){
            if(row.IDJUROS == 'S'){
                row.VRMORDIABRECTOT = parseFloat(row.VRMORDIABREC_inline*row.NRDIASATRASO);
            }else{
                row.VRPEMORBREC = row.VRPEMORBREC ? row.VRPEMORBREC : 0;
                row.VRMORDIABRECTOT = (parseFloat(row.VRFINALBREC)*(Math.pow((1 + parseFloat(row.VRPEMORBREC) / 100), parseInt(row.NRDIASATRASO)))) - parseFloat(row.VRFINALBREC);
            }
            
            row.VRFINALBRECCALC        = parseFloat(parseFloat(row.VRFINALBREC) + parseFloat(row.VRMORDIABRECTOT) + parseFloat(row.VRMULTABREC_inline)).toFixed(2);
            row.VRFINALBRECCALC_inline = parseFloat(parseFloat(row.VRFINALBREC) + parseFloat(row.VRMORDIABRECTOT) + parseFloat(row.VRMULTABREC_inline)).toFixed(2);
        }else{
            row.VRFINALBRECCALC        = parseFloat(row.VRFINALBREC).toFixed(2);
            row.VRFINALBRECCALC_inline = parseFloat(row.VRFINALBREC).toFixed(2);
        }
    };
    
    self.verificaDiasEmAtraso = function(dataProcessamento, dataVencimento){
       var diasEmAtraso = (moment(dataProcessamento, 'DD/MM/YYYY') - moment(dataVencimento, 'DD/MM/YYYY')) / 86400000;
       diasEmAtraso = diasEmAtraso > 0?diasEmAtraso:0;
       
       return diasEmAtraso;
    };
    
    self.recalculaJurosTitulos = function(){
        var widgetNewBaixaTabbed      = templateManager.container.getWidget('NewBaixaTabbed');
        var widgetNewTitulosBaixaGrid = templateManager.container.getWidget('NewTitulosBaixaGrid');
        
        if(widgetNewBaixaTabbed.currentRow.DTBAIXAREC){
            widgetNewTitulosBaixaGrid.dataSource.data.forEach(function(row){
                self.calculaValorLiquidoFinal(row);
            });
        }
    };
    
    /**
     * Gera Ajuste de Multa e Mora na Baixa
     * @param args
     */
    self.geraAjusteMultaMoraBaixa = function(){
        var deferred = ZHPromise.defer();
        var widgetTitulosBaixa = templateManager.container.getWidget('NewTitulosBaixaGrid');
        
        // Busca Ajustes dos Títulos a Receber/Recebidos
        localStorageEngine.getDataSource('fin_contaReceber#/ajbolrec_tipoajreceber').then(function (dataLocal) {
                
            // Busca Parâmetros de Títulos a Receber/Recebidos
            self.buscaGeralRec().then(function(response_geralrec) {

                var geral_rec = response_geralrec.dataset.geral_rec[0];
            
                self.insereAjusteMultaMoraBaixa(widgetTitulosBaixa, geral_rec, dataLocal).then(function(dataLocal){
                    localStorageEngine.setDataSource('fin_contaReceber#/ajbolrec_tipoajreceber', dataLocal);
                    deferred.resolve(dataLocal);
                });
                
            });
        });
        
        return deferred.promise;
    };
    
    /**
     * Gera Ajuste de Multa e Mora na Baixa
     * @param args
     */
    self.insereAjusteMultaMoraBaixa = function(widgetTitulosBaixa, geral_rec, dataLocal){
        var deferred = ZHPromise.defer();
        
        // Percorre Títulos a Receber/Recebidos
        widgetTitulosBaixa.dataSource.data.forEach(function(titulo){
                
            // Verifica Dias em Atraso e Tipo de Juros
            if((titulo.NRDIASATRASO > 0) && (titulo.IDJUROS)){
                
                // Verifica Parametrização dos Tipos de Ajuste da Baixa
                if((!geral_rec.CDAJMULTAREC) || (!geral_rec.CDAJMORAREC)){
                    ScreenService.showMessage('Operação bloqueada. Parâmetros de Tipos de Ajuste da Baixa, Multa e/ou Juros de Mora, não cadastrados na tela de Parâmetros de Títulos a Receber.');
                    deferred.reject();
                }else{
                
                    // Verifica Valor de Multa
                    if(titulo.VRMULTABREC > 0){
                        var currentRowMulta = {
                            "CDAJUCTREC"   : geral_rec.CDAJMULTAREC,
                            "DSAJUCTREC"   : "",
                            "IDAJUCTREC"   : "A",
                            "VRAJBOLREC"   : titulo.VRMULTABREC,
                            "NRORG"        : titulo.NRORG,
                            "CDFILIAL"     : titulo.CDFILIAL,
                            "NRLANBOLREC"  : titulo.NRLANBOLREC,
                            "IDORIGAJBREC" : "2",
                            "IDOPERACAO"   : "I"
                        };
                        
                        // Insere Ajuste Multa
                        dataLocal.push(currentRowMulta);
                    }
                    
                    // Verifica Valor de Mora
                    if(titulo.VRMORDIABRECTOT > 0){
                        var currentRowMora = {
                            "CDAJUCTREC"   : geral_rec.CDAJMORAREC,
                            "DSAJUCTREC"   : "",
                            "IDAJUCTREC"   : "A",
                            "VRAJBOLREC"   : titulo.VRMORDIABRECTOT,
                            "NRORG"        : titulo.NRORG,
                            "CDFILIAL"     : titulo.CDFILIAL,
                            "NRLANBOLREC"  : titulo.NRLANBOLREC,
                            "IDORIGAJBREC" : "2",
                            "IDOPERACAO"   : "I"
                        };
                        
                        // Insere Ajuste Mora
                        dataLocal.push(currentRowMora);
                    }
                }
            }
        });
        
        deferred.resolve(dataLocal);
        
        return deferred.promise;
    };
    
    
    
    
    
    
    /**
     * Abre Envio de Carta de Cobrança
     */
    self.abreEnvioCartaCobranca = function(){
        var widgetCartaCobrancaTabbed = templateManager.container.getWidget('CartaCobrancaTabbed');
        widgetCartaCobrancaTabbed.isVisible = true;
        
        ScreenService.closePopup();
        ScreenService.openSwipe(widgetCartaCobrancaTabbed);
        
        self.buscaPadraoCartaCobranca();
    };

    /**
     * Busca Padrão de Envio de Carta de Cobrança
     */
    self.buscaPadraoCartaCobranca = function (){
        Rec00001_TitulosAReceber_Service.buscaEndefili(getLocalVar('CDFILIAL'), 'P').then(function (response_endefili){
            var widgetCartaCobrancaTabbed = templateManager.container.getWidget('CartaCobrancaTabbed');
            
            var endefili    = response_endefili.dataset.endefili[0];
            var dsemailfili = endefili.DSEMAILFILI;


            var assunto  = '[EMPRESA] - Carta de Cobrança - Vencimento [VENCIMENTO]';
            var mensagem =
                '[MUNICIPIO], [DIA] de [MES] de [ANO].\n\n' +
                'Prezado(a) Sr(a) [CLIENTE],\n\n' +
                'Consta em nossos registros que o pagamento referente ao título com vencimento para o dia [VENCIMENTO], no valor de [VALOR], encontra-se em aberto.\n\n' +
                'Desta forma, fazemos uso do presente para cientificá-lo(a) do débito pendente e solicitar sua mais breve regularização.\n\n' +
                'Qualquer dúvida, pedimos a gentileza de entrar em contato conosco para juntos resolvemos esta situação.\n\n' +
                'Atenciosamente,\n'+
                '[EMPRESA]';

            widgetCartaCobrancaTabbed.getField('DSEMAILFILI').setValue(dsemailfili);
            widgetCartaCobrancaTabbed.getField('DSASSUEMAIL').setValue(assunto);
            widgetCartaCobrancaTabbed.getField('DSMENSEMAIL').setValue(mensagem);

            var widgetTitulosAReceberGrid = templateManager.container.getWidget('TitulosAReceberGrid');
            var titulos = widgetTitulosAReceberGrid.getCheckedRows().filter(function(data){
            	return data.IDSTATBREC != 'B' && data.IDSTATUSREC == 'A';
            });
            var datasource = [];

            titulos.forEach(function(titulo){
                datasource.push(
                    {
                        NRINSJURCLIE   : titulo.NRINSJURCLIE,
                        NMRAZSOCCLIE   : titulo.NMRAZSOCCLIE,
                        DSEMAILCLIE    : titulo.DSEMAILCLIE,
                        DSEMAILCLIECOP : titulo.DSEMAILCLIECOP
                    }
                );
            });
            
            var widgetCartaCobrancaGrid   = templateManager.container.getWidget('CartaCobrancaGrid');
            widgetCartaCobrancaGrid.dataSource.data = datasource;
            
            templateManager.updateTemplate();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Envia Título por E-mail
     */
    self.enviaCartaPorEmail = function(){
        var widgetTitulos     = templateManager.container.getWidget('TitulosAReceberGrid');
        var widgetEnviaTitulo = templateManager.container.getWidget('CartaCobrancaTabbed');

        var titulos = widgetTitulos.getCheckedRows().filter(function(data){
            return data.IDSTATBREC != 'B' && data.IDSTATUSREC == 'A';
        });
        
        var dataSet = [titulos, widgetEnviaTitulo.currentRow];

        General.callRestEngine('fin_contaReceber#/enviaCartaCobrancaPorEmail', 'DataSet', dataSet).then(function () {
            ScreenService.showMessage('Carta(s) de cobrança enviada(s) por e-mail com sucesso.');
            ScreenService.closeSwipe();

            widgetEnviaTitulo.isVisible = false;

            widgetTitulos.dataSource.unSelectAll();
            self.unsetStripeDataFooter();
            self.forceActivate();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    /**
     * Valida Tipo de Baixa
     */
    self.validaTipoBaixa = function(widget){
        if(widget.currentRow.IDBAIXAUTILR == 'P'){
            ScreenService.showMessage('Operação bloqueada. Tipo de baixa de compensação não pode ser utilizada.');
            
            widget.currentRow.NMTPBAIXAREC = '';
            widget.currentRow.CDTPBAIXAREC = '';
            widget.currentRow.IDBAIXAUTILR = '';
        }
    };
    
    /**
     * Verifica se existe filial parametrizada na tela de conta corrente, caso tenha insere automaticamente
     * ao novo campo de filial a filial filtrada;
     */
     
     self.preencheFilial =  function(cdReduCont){
         var contaCorrente = {
            cdReduCont: cdReduCont,
        };
          var grid =  templateManager.container.getWidget('RemessaForm');
          Rec00001_TitulosAReceber_Service.buscaContaCorrenteVinculada(contaCorrente).then(function(contaCorrente) {
              if(contaCorrente.dataset.filial_conta_corrente.filial_conta_corrente.length === 1) {
                  var filial = contaCorrente.dataset.filial_conta_corrente.filial_conta_corrente[0];
                  grid.getField('CDFILIAL').setValue(filial.CDFILIAL);
              } else {
                  grid.getField('CDFILIAL').setValue("");
                  grid.getField('CDFILIAL').reload();
                  
              }
        });
     };
     
    /**
     * Abre Tela Preparação Titulos a Receber
     */
     self.abrirPreparacaoTitulos =  function(){
        var widgetPreparacaoTitulosForm =  templateManager.container.getWidget('PreparacaoTitulosForm');
        var widgetPreparacaoTitulosGrid =  templateManager.container.getWidget('PreparacaoTitulosGrid');
        var widgetTitulosAReceberGrid   =  templateManager.container.getWidget('TitulosAReceberGrid');
        
        var titulosEmAberto = widgetTitulosAReceberGrid.getCheckedRows().filter(function(boleto){
            return boleto.IDSTATBREC == 'A';
        });
        
        self.buscaPadraoPreparacaoTitulos(widgetPreparacaoTitulosForm, widgetPreparacaoTitulosGrid, titulosEmAberto).then(
            function () {
                ScreenService.closePopup();
                widgetPreparacaoTitulosForm.isVisible = true;
                ScreenService.openSwipe(widgetPreparacaoTitulosForm).then(
                    function () {
                        widgetPreparacaoTitulosForm.editing = true;
                        widgetPreparacaoTitulosGrid.reload();
                    });
            });
     };
     
     /**
     * Busca Padrão Preparação Títulos a Receber
     */
    self.buscaPadraoPreparacaoTitulos = function (widgetPreparacaoTitulosForm, widgetPreparacaoTitulosGrid, titulosEmAberto){
        var deferred = ZHPromise.defer();
        
        widgetPreparacaoTitulosForm.currentRow.NMTPBAIXAREC = '';
        widgetPreparacaoTitulosForm.currentRow.DSCONTCORR   = '';
        widgetPreparacaoTitulosForm.currentRow.CDBANCO      = '';
        widgetPreparacaoTitulosForm.currentRow.CDAGENCIA    = '';
        widgetPreparacaoTitulosForm.currentRow.CDCONTCORR   = '';
        widgetPreparacaoTitulosGrid.dataSource.data = [];
        
        localStorageEngine.setDataSource("fin#/preparacao_titulos", titulosEmAberto).then(function () {
            deferred.resolve();
        }, function (error) {
            ScreenService.showMessage(error.data.error);
            deferred.reject();
        });
        
        return deferred.promise;
    };

    /**
     * Confirma Preparação Títulos a Receber
     */
    self.confirmaPreparacaoTitulos = function(){
        
        var widgetPreparacaoTitulosForm =  templateManager.container.getWidget('PreparacaoTitulosForm');
        var widgetPreparacaoTitulosGrid =  templateManager.container.getWidget('PreparacaoTitulosGrid');
        var widgetTitulosAReceberGrid   =  templateManager.container.getWidget('TitulosAReceberGrid');
        
        if(widgetPreparacaoTitulosForm.isValid()){
            var dataSet = [widgetPreparacaoTitulosForm.currentRow, widgetPreparacaoTitulosGrid.dataSource.data];
    
            General.callRestEngine('fin_contaReceber#/preparacaoTitulosReceber', 'DataSet', dataSet).then(function () {
                ScreenService.showMessage('Títulos preparados com sucesso.');
                
                ScreenService.closeSwipe();
                widgetPreparacaoTitulosForm.editing = false;
                widgetPreparacaoTitulosForm.isVisible = false;
                
                self.resetTituloAReceber();
            }).catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
        }
    };
    
    // Reset as Configurações da tela de Títulos a Receber
    self.resetTituloAReceber = function(){
        var widget   =  templateManager.container.getWidget('TitulosAReceberGrid');
        widget.dataSource.unSelectAll();
        self.unsetStripeDataFooter();
        self.forceActivate();
        widget.reload();
    };
    
    // Verifica se teve alteração do campo de unidade, caso ocorra limpa o campo de série para vincular a série desejada
    self.verificaAlteracaoUnidade =  function(widget){
        widget.getField('DSSERIE').setValue('');
        widget.getField('DSSERIE').reload();
    };
    
    self.setValueDescricao =  function(widget){
        var descricaoCadastramento = widget.getField("DSCONTAREC");
        var descricaoDocumento = widget.getField("DSDEFACTREC");
        var titulosGrid =  templateManager.container.getWidget("TitulosAReceberGrid").getCheckedRows();
        if(titulosGrid.length == 0){
            descricaoCadastramento.setValue(descricaoDocumento.value());
        }
    };

};

/* global Configuration */
Configuration(function (ContextRegister)
{
    ContextRegister.register('Rec00001_TitulosAReceber_Controller', rec00001_TitulosAReceber_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec00200_RetornoCobranca_Controller.js
var rec00200_RetornoCobranca_Controller = function(ZHPromise, General, templateManager, ScreenService, Rec00200_RetornoCobranca_Service, ZhFilterPreferences)
{
    /** Variáveis de Inicialização */
    var self = this;

    self.incrementaDataFinal = function(){
        var retorno = templateManager.container.getWidget('RetornoGrid');
        retorno.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'DTCNABREC'){
                filtro.value[1] = moment(filtro.value[1].replace('/', ''), 'DDMMYYYY').add(1, 'days').format('DD/MM/YYYY');
            }
        });

    };
    
    self.openFilterRetornoCobranca = function(retorno){
        self.buscaPadraoEmpresa(retorno).then(function(empresa){
            var data = moment().format('DD/MM/YYYY');
            var rowFilter = {
                'CDEMPRESA': empresa.CDEMPRESA,
                'NMEMPRESA': empresa.NMEMPRESA,
                'DTCNABREC': {start : data, end : data},
            };
            ZhFilterPreferences.setOriginalFilter(retorno.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(retorno, rowFilter).then( function(sucess){
                retorno.getField('DSTPCTRECEB').dataSourceFilter[0].value = empresa.CDEMPRESA;
                retorno.getField('DSTPCTRECEB').reload();
            });
        });
    };

    self.voltarBoletos = function(){
        ScreenService.goBack().then(function(){
            var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
            tabbed.fields.forEach(function(field){
                field.readOnly = false;
            });
        });
    };

    self.ativaGridOcorrencias = function(widget){
        if (widget.name == 'BoletosGrid'){
            templateManager.container.getWidget('OcorrenciasGrid').activate();
        }else{
            templateManager.container.getWidget('ImportacaoOcorrenciasTabbed').activate();
        }
        //self.setGridNaoClicavel();
    };

    self.ativaWidgetPrincipal = function(widget){
        widget.activate();
    };
    /**
     * Busca Parâmetros de Estoque
     * @return dataProcessamento
     */
    self.buscaEmpresa = function (cdempresa){
        return Rec00200_RetornoCobranca_Service.buscaEmpresa(cdempresa).then(function(empresa) {
            return empresa;
        });
    };

    /**
     * Busca Filial
     * @return filialLogada
     */
    self.buscaFilial = function (){
        return Rec00200_RetornoCobranca_Service.buscaFilial().then(function(filial) {
            return {dataset : {filial : filial}};
        });
    };

    /**
     * Busca Padrão Empresa
     * @param widget
     */
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        self.buscaFilial().then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;

            self.buscaEmpresa(cdempresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                deferred.resolve(empresa);
            });
        });
        return deferred.promise;
    };

    var setGridNaoClicavel = function(){
        var swipe = document.getElementsByClassName('zh-swipe-inner zh-content ng-scope open');
        var div = document.createElement('div');
        div.setAttribute('class', 'swipe-overlay ng-scope');
        div.setAttribute('data-ng-if', 'swipe.editing || swipe.currentWidget.editing');
        swipe[0].appendChild(div);
    };

    self.adicionaRetorno = function(){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        tabbed.isVisible = true;
        ScreenService.openSwipe(tabbed).then(function(){
            window.setTimeout(setGridNaoClicavel, 100);
        });
    };

    /**
     * Cancela Retorno
     */
    self.cancelaRetorno = function(){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        tabbed.isVisible = false;
        ScreenService.closeSwipe().then(function(){
            tabbed.getParent().activate();
            tabbed.widgets[0].dataSource.data = [];
            tabbed.widgets[0].ocorrencias     = [];
        });
    };

    /**
     * Valída Visualização Importacao
     * @param widget
     */
    self.validaVisualizacaoImportacao = function(widget){
        widget.getField('DSARQCNABREC').readOnly = !widget.getField('NMBANCO').value();

        self.deleteDataRetorno();
    };

    /**
     * Deleta Data Source Retorno
     */
    self.deleteDataRetorno = function(){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetBoletos.dataSource.data = [];
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = [];
    };

    /**
     * Prepara Arquivo CNAB
     * @param widget
     */
    self.preparaArquivoRetorno = function(widget){
        var route = 'fin_contaReceber#/carregaRetornoCobranca';

        var cdempresa   = templateManager.container.getWidget('ImportacaoTabbed').currentRow.CDEMPRESA;
        var cdbanccnab  = widget.currentRow.CDBANCCNAB;
        var cnabretorno = widget.currentRow.DSARQCNABREC[0].b64File.split(',')[1];

        var row = {
            'CDEMPRESA'  : cdempresa,
            'CDBANCCNAB' : cdbanccnab,
            'CNABRETORNO': cnabretorno
        };

        General.callRestEngine(route, 'Row', row)
            .then(function(response){
                this.setDataSourceRetorno(response.dataset.BOLETOS);
                this.setDataSourceOcorrencias(response.dataset.OCORRENCIAS);
                widget .widgets[0].SITUACAOCNAB = response.dataset.SITUACAOCNAB[0];
            }.bind(this))
            .catch(function(erro){
                widget.currentRow.DSARQCNABREC = [];
                ScreenService.showMessage(erro.data.error);
            });
    };

    /**
     * Data Source Boletos
     * @param boletos
     */
    self.setDataSourceRetorno = function(boletos){
        var widgetBoletos = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        widgetBoletos.dataSource.data = [];
        widgetBoletos.dataSource.data = boletos;
    };

    /**
     * Data Source Ocorrências
     * @param ocorrencias
     */
    self.setDataSourceOcorrencias = function(ocorrencias){
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetOcorrencias.dataSource.data = [];
        widgetOcorrencias.ocorrencias = ocorrencias;
    };

    /**
     * Abre Ocorrências
     * @param widget
     */
    self.openOcorrencias = function(widget){
        var tabbed = templateManager.container.getWidget('ImportacaoTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == widget.currentRow.CDLINHA || ocorrencia.GERAL);
        });
        tabbed.fields.forEach(function(field){
            field.readOnly = true;
        });
    };

    /**
     * Prepara Ocorrências
     * @param widgetOcorrencias
     * @param nrlanbolrec
     * @param cdlinha
     * @returns {*}
     */
    self.preparaOcorrenciasBoleto = function(widgetOcorrencias, nrlanbolrec, cdlinha){
        var deferred = ZHPromise.defer();
        widgetOcorrencias.dataSource.data = widgetOcorrencias.ocorrencias.filter(function(ocorrencia){
            return (ocorrencia.CDLINHA == cdlinha || ocorrencia.GERAL);
        });
        deferred.resolve();
        return deferred.promise;
    };

    /**
     * Insere Retorno CNAB
     */
    self.insereRetorno = function(){
        var widgetBoletos     = templateManager.container.getWidget('ImportacaoBoletosTabbed');
        var widgetOcorrencias = templateManager.container.getWidget('ImportacaoOcorrenciasTabbed');
        var retorno = templateManager.container.getWidget('RetornoGrid');
        if (widgetBoletos.getParent().isValid()){
            if (widgetBoletos.dataSource.data.length > 0){
                if (self.validaSituacaoCnab(widgetBoletos)){
                    var dataSet = [];
                    var route = 'fin_contaReceber#/insereRetornoCobranca';

                    var form = {
                        'NOMEARQUIVO' : widgetBoletos.getParent().currentRow.DSARQCNABREC[0].name,
                        'CDBANCCNAB'  : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDBANCO'     : widgetBoletos.getParent().currentRow.CDBANCCNAB,
                        'CDEMPRESA'   : widgetBoletos.getParent().currentRow.CDEMPRESA
                    };

                    dataSet.push(form);
                    dataSet.push(widgetBoletos.dataSource.data);
                    dataSet.push(widgetOcorrencias.ocorrencias);

                    General.callRestEngine(route, 'DataSet', dataSet)
                        .then(function(response){
                            ScreenService.goBack().then(function(){
                                this.deleteDataRetorno();
                                widgetBoletos.getParent().isVisible = false;
                                retorno.activate();
                                retorno.reload();
                            }.bind(this));
                        }.bind(this))
                        .catch(function(erro){
                            ScreenService.showMessage(erro.data.error);
                        });
                }else{
                    ScreenService.showMessage('Registro(s) do arquivo de retorno possui(em) ocorrência(s) que não possibilita(m) o salvamento.');
                }
            }else{
                ScreenService.showMessage('Não existem boletos importados.');
            }
        }
    };

    /**
     * Valida Situação CNAB
     * @param boletos
     * @returns {boolean}
     */
    self.validaSituacaoCnab = function(boletos){
        function arrayValidation(element, index, array) {
            return element.SITUACAOBOLETO == false;
        }
        var boletoValido = !boletos.dataSource.data.every(arrayValidation);
        return (boletos.SITUACAOCNAB && boletoValido) ? true : false;
    };

    /**
     * Cancela Título Retorno
     */
    self.cancelaTituloRetorno = function(){
        var route = 'fin_contaReceber#/cancelaTituloRetornoCob';
        var widgetRetornoGrid = templateManager.container.getWidget('RetornoGrid');
        var widgetBoletosGrid = templateManager.container.getWidget('BoletosGrid');

        if(widgetBoletosGrid.currentRow.IDSITUCNABR == '90') {
            ScreenService.showMessage('Operação Bloqueada. Título baixado via CNAB não pode ser cancelado do retorno.');
            return;
        }

        ScreenService.confirmMessage('Deseja realmente cancelar o título do retorno?', 'question', function () {
            var dataset = {
                retorno : widgetRetornoGrid.currentRow,
                boleto  : widgetBoletosGrid.currentRow
            };

            General.callRestEngine(route, 'DataSet', dataset)
                .then(function(){
                    ScreenService.showMessage('Título cancelado do retorno com sucesso.', 'checked');
                    ScreenService.goBack();
                    widgetBoletosGrid.reload();
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        });
    };
    
    self.changeTipoConta = function(widget){
        widget.getField('CDTPCTRECEB').dataSourceFilter[0].value = widget.getField('CDEMPRESA').value();
        widget.getField('CDTPCTRECEB').clearValue();
        widget.getField('CDTPCTRECEB').reload();
    };
    
};

Configuration(function (ContextRegister)
{
    ContextRegister.register('Rec00200_RetornoCobranca_Controller', rec00200_RetornoCobranca_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec12200_AdiantamentosDisponiveis_Controller.js
function Rec12200_AdiantamentosDisponiveis_Controller(ZhFilterPreferences, General, ScreenService, templateManager) {
	
	this.setReadSelect = function(value, widgetFiltro, nameField){
		var field = widgetFiltro.getField(nameField);
		if(value === 'L'){
			field.readOnly = false;
			field.validations.required = true;
		} else {
			delete field.validations.required;
			field.readOnly = true;
			field.setValue('');
		}
	};

	this.filter = function(widget) {
		var rowFilter = {
			'TPCDCLIENTE'   : 'T',
			'TPCDFILIORIG'  : 'T',
			'TPCDFILILANC'  : 'T',
			'TPFORMAREC'    : 'T',
			'TPCDTPCTRECEB' : 'T',
			'STATUS'        : 'T'
		};
		ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
		ZhFilterPreferences.openFilterWidget(widget, rowFilter);
	};

	this.checaListas = function(field, field2, widget){
		field.setValue(field.value() || 'T');
		if(field.value() === 'L'){
			var string = field2.value().join(',');
			widget.currentRow[field.name] = string;
			field2.setValue(null);
		}
	};

	this.checaDisponibilidadeRec = function(field, valor){
		if(field.value() === 'N' && valor > 0){
			field.setValue('S');
			ScreenService.showMessage('Operação bloqueada. Adiantamento já foi utilizado.');
		}
	};
	
	this.AjustaFiltro = function(rowFilter){
		var widget = templateManager.container.getWidget('rec12200_adiantamentos_disponiveis_grid');
		var pos = 0;
		var deletePos = null;
		widget.dataSourceFilter.forEach(function(filter) {
			switch(filter.name){
				case 'DOCINICIAL': filter.value = (rowFilter.DOCINICIAL) ? rowFilter.DOCINICIAL : null;
					break;
				case 'DOCFINAL': filter.value = (rowFilter.DOCFINAL) ? rowFilter.DOCFINAL : null;
					break;
				case 'CDCLIENTE': filter.value = (rowFilter.TPCDCLIENTE == 'L') ? rowFilter.CDCLIENTE.join('_') : '';
					break;
				case 'TPCDCLIENTE': filter.value = rowFilter.TPCDCLIENTE;
					break;
				case 'CDFILIORIG': filter.value = (rowFilter.TPCDFILIORIG == 'L') ? rowFilter.CDFILIORIG.join('_') : '';
					break;
				case 'TPCDFILIORIG': filter.value = rowFilter.TPCDFILIORIG;
					break;
				case 'CDFILILANC': filter.value = (rowFilter.TPCDFILILANC == 'L') ? rowFilter.CDFILILANC.join('_') : '';
					break;
				case 'TPCDFILILANC': filter.value = rowFilter.TPCDFILILANC;
					break;
				case 'FORMAREC': filter.value = (rowFilter.TPFORMAREC == 'L') ? rowFilter.FORMAREC.join('_') : '';
					break;
				case 'TPFORMAREC': filter.value = rowFilter.TPFORMAREC;
					break;
				case 'CDTPCTRECEB': filter.value = (rowFilter.TPCDTPCTRECEB == 'L') ? rowFilter.CDTPCTRECEB.join('_') : '';
					break;
				case 'TPCDTPCTRECEB': filter.value = rowFilter.TPCDTPCTRECEB;
					break;
				case 'DATAINI': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.start : null;
					break;
				case 'DATAFIN': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.end : null;
					break;
				case 'STATUS': filter.value = rowFilter.STATUS;
					break;
				case 'DATA': deletePos = pos;
					break;
			}
			pos++;
		});
		if (deletePos){
			widget.dataSourceFilter.splice(deletePos, 1);
		}
	};

	

	this.setMinMax = function(MaxOrMin){
		var widget = templateManager.container.getWidget('rec12200_adiantamentos_disponiveis_grid').appliedFilter;
		if(MaxOrMin === 'MAX'){
			var maxDate = widget.getField('DATAFIN').value();
			var fieldInidial = widget.getField('DATAINI');
			fieldInidial.maxDate = maxDate;
		} else {
			var minDate = widget.getField('DATAINI').value();
			var fieldFinal = widget.getField('DATAFIN');
			fieldFinal.minDate = minDate;
		}
	};

	this.setRequired = function(widget, nmf1, nmf2){
		var field1 = widget.getField(nmf1);
		var field2 = widget.getField(nmf2);
		if(field1.value() || field2.value()){
			field1.validations.required = true;
			field2.validations.required = true;
		} else {
			delete field1.validations.required;
			delete field2.validations.required;
			if (widget.currentRow.$error) {
        widget.currentRow.$error[nmf1] = [];
        widget.currentRow.$error[nmf2] = [];
      }
		}
	};
	
    this.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctrec = widget.getField('DSTPCTRECEB');
        
        cdtpctrec.setValue("");
        cdtpctrec.dataSourceFilter[1].value = cdempresa;
        cdtpctrec.reload();
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec12200_AdiantamentosDisponiveis_Controller', Rec12200_AdiantamentosDisponiveis_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec12300_NotasDevolucaoDisponiveis_Controller.js
function Rec12300_NotasDevolucaoDisponiveis_Controller(General, ScreenService, templateManager, ZhFilterPreferences) {
	this.setReadSelect = function(value, devolucoesDisponiveis, nameField){
		var field = devolucoesDisponiveis.getField(nameField);
		var name = field.name;
		if(value === 'L'){
			field.readOnly = false;
			field.validations.required = true;
		} else {
			delete field.validations.required;
			field.readOnly = true;
		}
	};

	this.checaDisponibilidade = function(field, valor){
		if(field.value() === 'N' && valor > 0){
			field.setValue('S');
			ScreenService.showMessage('Operação bloqueada. Nota Fiscal de Devolução já foi utilizada.');
		}
	};
	
	this.filter = function(widget) {
		var rowFilter = {
			'TPCDCLIENTE'  : 'T',
			'TPCDFILIORIG' : 'T',
			'TPCDTIPOOPER' : 'T',
			'STATUS'       : 'T'
		};
		ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
		ZhFilterPreferences.openFilterWidget(widget, rowFilter);
	};

	this.AjustaFiltro = function(rowFilter){
		var widget = templateManager.container.getWidget('rec12300_notas_devolucao_disponiveis_grid');
		var pos = 0;
		var deletePos = null;
		widget.dataSourceFilter.forEach(function(filter) {
			switch(filter.name){
				case 'DOCINICIAL': filter.value = (rowFilter.DOCINICIAL) ? rowFilter.DOCINICIAL : null;
					break;
				case 'DOCFINAL': filter.value = (rowFilter.DOCFINAL) ? rowFilter.DOCFINAL : null;
					break;
				case 'CDCLIENTE': filter.value = (rowFilter.TPCDCLIENTE == 'L') ? rowFilter.CDCLIENTE.join('_') : '';
					break;
				case 'TPCDCLIENTE': filter.value = rowFilter.TPCDCLIENTE;
					break;
				case 'CDFILIORIG': filter.value = (rowFilter.TPCDFILIORIG == 'L') ? rowFilter.CDFILIORIG.join('_') : '';
					break;
				case 'TPCDFILIORIG': filter.value = rowFilter.TPCDFILIORIG;
					break;
				case 'CDTIPOOPER': filter.value = (rowFilter.TPCDTIPOOPER == 'L') ? rowFilter.CDTIPOOPER.join('_') : '';
					break;
				case 'TPCDTIPOOPER': filter.value = rowFilter.TPCDTIPOOPER;
					break;
				case 'DATAINI': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.start : null;
					break;
				case 'DATAFIN': filter.value = (rowFilter.hasOwnProperty('DATA')) ? rowFilter.DATA.end : null;
					break;
				case 'STATUS': filter.value = rowFilter.STATUS;
					break;
				case 'DATA': deletePos = pos;
					break;
			}
			pos++;
		});
		if (deletePos){
			widget.dataSourceFilter.splice(deletePos, 1);
		}
	};
	
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Rec12300_NotasDevolucaoDisponiveis_Controller', Rec12300_NotasDevolucaoDisponiveis_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec15000_ChequeRecebidos_Controller.js
var Rec15000_ChequeRecebidos_Controller = function(Rec15000_ChequeRecebidos_Service, ZhFilterPreferences, templateManager,ScreenService, General, ZHPromise){
    
    var mensagens = {
        valorFinal:  'Valor inicial deve ser menor ou igual ao valor final',
        chequeFinal: 'Cheque inicial deve ser maior ou igual ao cheque final.',
        favorecidoP: 'A operação não foi realizada para um ou mais cheques marcados, pois estes não possuem conta corrente do favorecido.',
        favorecidoT: 'A operação não foi realizada. Nenhum dos cheques marcados possui conta do favorecido.',
        favorecido:  'Conta corrente do favorecido deve ser informada.',
        selecioneRows: 'Selecione algum registro e tente novamente.',
        apresentacaoSucesso: 'Apresentação gerada com sucesso',
        erroBoletos: 'Operação bloqueada! O(s) cheque(s) não possuem boleto(s)',
        erroApresentacao: 'A operação não foi realizada para um ou mais cheques marcados, pois estes já estão apresentados.',
        erroDevolucao: 'A operação não foi realizada. Somente cheques apresentados podem ser devolvidos.',
        devolucaoSucesso: 'Devolução gerada com sucesso',
        situarCustodia: 'Confirma situar o(s) cheque(s) selecionado(s) em custódia?',
        retirarCustodia: 'Confirma retirada do(s) cheques(s) selecionado(s)?',
        erroSCustodia: 'A operação não foi realizada para um ou mais cheques marcados, pois estes já estão "Em custódia".',
        erroRCustodia: 'A operação não foi realizada para um ou mais cheques marcados, pois estes não estão "Em custódia".',
    	favorecidoDiferente: 'Operação bloqueada! A conta dos favorecidos é diferente.',
    	erroChequeCompensado: 'Operação bloqueada! Um ou mais cheque(s) já foram compensados.',
    	compensacaoSucesso: 'Compensação gerada com sucesso.', 
    	opBlocked: 'Operação bloqueada! O cheque já foi compensado.',
    	opBlockedBoleto: 'Operação bloqueada! O cheque possui boletos relacionados.',
    	empresa: 'Selecione uma empresa no filtro antes de adicionar.',
    	confirmDelete: 'Excluir registro?',
    	deleteBlocked: 'Operação bloqueada! O status do cheque não permite esta operação.'
    };
    
    var BKPcurrentRow;

	// Abre o form para um novo registro e inicializa valores
	this.adicionar = function(widget) {
		if(!widget.appliedFilter.FILTER_ROW.P_CDEMPRESA) ScreenService.showMessage(mensagens.empresa);
		else {
			var widgetPopup = templateManager.container.getWidget('leitor_cheque');
			widgetPopup.activate();
			widgetPopup.edit();
			ScreenService.openPopup(widgetPopup);
		}
	};
	
	this.editar = function(widget) {
		if(widget.currentRow.IDSITCHEQREC === 'C'){
			ScreenService.showMessage(mensagens.opBlocked);
		}else if(widget.currentRow.IDSITCHEQREC === 'P'){
			ScreenService.showMessage(mensagens.deleteBlocked);
		}else{
			General.widgetEdit(widget);
			BKPcurrentRow = angular.copy(widget.currentRow);
		}
	};
	
	this.excluir = function(widget) {
		var widgetBoleto = templateManager.container.getWidget('grid_boleto');
		if(widget.currentRow.IDSITCHEQREC === 'C') ScreenService.showMessage(mensagens.opBlocked);
		else {
			widgetBoleto.reload().then(function() {
				if(widgetBoleto.dataSource.data[0]) {
					ScreenService.showMessage(mensagens.opBlockedBoleto);
				}
				else {
					ScreenService.confirmMessage(ScreenService.i18n(mensagens.confirmDelete), 'question', function (success) {
						General.callRestEngine('fin_contaReceber#/deletarCheque', 'Row', widget.currentRow).then(function(Success){
							ScreenService.closeSwipe();
							widget.reload();
							widget.activate();
						}.bind(this),
						function (error) {
							if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
								ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
							} else {
								ScreenService.showMessage(error.data.error);
							}
						});
					}.bind(this));
				}
			});
		}
	};
	
	
	this.setDadosCheque = function(widget, cancel) {
		var widgetCheque = templateManager.container.getWidget('grid_cheques_recebidos');
		var widgetBoleto = widgetCheque.widgets[0];
		var codigos;
		if(widget.currentRow.CODCHEQUE && cancel === false) {
			codigos = Rec15000_ChequeRecebidos_Service.getCodLeitor(widget.currentRow.CODCHEQUE);
			widgetCheque.currentRow.NRCHEQUERECE = codigos[0];
		
		}
		ScreenService.closePopup(widget);
		widget.isVisible = false;
		widgetCheque.activate();
		widgetCheque.reload();
		widgetCheque.newRow();
		widgetCheque.currentRow.IDSITCHEQREC = 'A';
		widgetCheque.currentRow.IDTIPCHEQREC = 'P';
		var dataAtual = Rec15000_ChequeRecebidos_Service.getDataAtual();
		widgetCheque.currentRow.DTRECECHEQUE = dataAtual;
		widgetCheque.currentRow.DTVENCHEQREC = dataAtual;
		if(widget.currentRow.CODCHEQUE && cancel === false) {
			if(codigos[0]) widgetCheque.currentRow.NRCHEQUERECE = codigos[0];
			if(codigos[1]) widgetCheque.currentRow.NRCOMCHEQREC = codigos[1];
			if(codigos[2]) widgetCheque.currentRow.CDBANCHEQREC = codigos[2];
			if(codigos[3]) widgetCheque.currentRow.CDAGECHEQREC = codigos[3];
			if(codigos[4]) widgetCheque.currentRow.CDCNTCHEQREC = codigos[4];
		}
		widgetCheque.widgets[0].enabled  = false;
		widgetCheque.widgets[1].enabled  = false;
		widgetCheque.redraw();
		widgetBoleto.actions[0].isVisible = true;
		widgetBoleto.actions[3].isVisible = false;
		widgetBoleto.dataSource.name      = 'fin_contaReceber#/crud/boletreceber_relacionamentocheque';
		widgetBoleto.dataSourceFilter     = [];
		widget.currentRow				  = {};
	};

	// Abre o filtro e inicializa valores
    this.filter = function(widget) {
    	var rowFilter = {};
		rowFilter.IDTIPCHEQREC  = 'A';
		rowFilter.IDSITCHEQREC  = 'T';
		rowFilter.P_TPCONTCORR  = 'T';
		rowFilter.P_TPCDCLIENTE = 'T';
    	
    	ZhFilterPreferences.setOriginalFilter(widget.id, rowFilter);
		ZhFilterPreferences.openFilterWidget(widget, rowFilter);
	};
	
    // Altera o readOnly dos fields de acordo com tipo de seleção 
    this.setReadSelect = function(fieldParent, nameField, widget){
		var field = widget.getField(nameField);
		var value = fieldParent.value();
		
		if(value === 'L'){
			field.readOnly = false;
			field.validations.required = true;
		} else {
			var name = field.name;
			delete field.validations.required;
			field.readOnly = true;
			field.setValue('');
		}
		
		templateManager.updateTemplate();
	};
	
	// Valida que o valor do campo final é menor que o inicial
	this.validaIntervalo = function(widget, fieldName, fieldName2, apaga) {
		var field = widget.getField(fieldName);
		var field2 = widget.getField(fieldName2);
		if(field.value() && field2.value()) {
			if(parseFloat(field.value()) > parseFloat(field2.value())) {
				if(apaga === 1) field.setValue(undefined);
				else field2.setValue(undefined);
				if(field.name === 'P_VRINI' || field.name === 'P_VRFIN') ScreenService.showMessage(mensagens.valorFinal);
				else ScreenService.showMessage(mensagens.chequeFinal);
			}
	    }
	    this.setRequired(widget, fieldName, fieldName2);
	};
	
	// Altera o required de um campo de acordo com outro campo
	this.setRequired = function(widget, nmf1, nmf2){
		var field1 = widget.getField(nmf1);
		var field2 = widget.getField(nmf2);
		if(field1.value() || field2.value()){
			field1.validations.required = true;
			field2.validations.required = true;
		} else {
			delete field1.validations.required;
			delete field2.validations.required;
			if (widget.currentRow.$error) {
        widget.currentRow.$error[nmf1] = [];
        widget.currentRow.$error[nmf2] = [];
      }
		}
	};
	
	// Ajusta os campos do filtro para a query
	this.ajustaFiltro = function(args) {
		var widget = templateManager.container.getWidget("grid_cheques_recebidos");
		var rowFilter = args.e.preference.FILTER_ROW;
		
		var filtro = [
			{
				"name": "P_CDEMPRESA",
				"operator": "=",
				"value": rowFilter.P_CDEMPRESA
			},
			{
				"name": "P_NRCHEINI",
				"operator": "=",
				"value": rowFilter.P_NRCHEINI || null	
			},
			{
				"name": "P_NRCHEFIN",
				"operator": "=",
				"value": rowFilter.P_NRCHEFIN || null
			},
			{
				"name": "P_VRINICHE",
				"operator": "=",
				"value": rowFilter.P_VRINI || null
			},
			{
				"name": "P_VRFINCHE",
				"operator": "=",
				"value": rowFilter.P_VRFIN || null
			},
			{
				"name": "IDTIPCHEQREC",
				"operator": "=",
				"value": rowFilter.IDTIPCHEQREC || 'A'
			},
			{
				"name": "IDSITCHEQREC",
				"operator": "=",
				"value": rowFilter.IDSITCHEQREC || 'T'
			},
			{
				"name": "P_DTVENINI",
				"operator": "=",
				"value": rowFilter.P_DTVEN.start
			},
			{
				"name": "P_DTVENFIN",
				"operator": "=",
				"value": rowFilter.P_DTVEN.end
			},
			{
				"name": "P_DTRECINI",
				"operator": "=",
				"value": rowFilter.P_DTREC.start
			},
			{
				"name": "P_DTRECFIN",
				"operator": "=",
				"value": rowFilter.P_DTREC.end
			}
		];
		
	    if (rowFilter.P_TPCDCLIENTE === 'T'){
	    	filtro.push({
				"name": "P_TPCDCLIENTE",
				"operator": "=",
				"value": rowFilter.P_TPCDCLIENTE 
			});
	    } else {
	    	filtro.push(
			{
				"name": "P_CDCLIENTE",
				"operator": "=",
				"value": concatValuesToFilter(rowFilter.P_TPCDCLIENTE, rowFilter.P_CDCLIENTE)
			});
	    }
	    if (rowFilter.P_TPCONTCORR === 'T'){
	    	filtro.push({
				"name": "P_TPCONTCORR",
				"operator": "=",
				"value": rowFilter.P_TPCONTCORR
			});
	    } else {
	    	filtro.push(
			{
				"name": "P_CONTCORR",
				"operator": "=",
				"value": concatValuesToFilter(rowFilter.P_TPCONTCORR, rowFilter.CDREDUCONT)
			});
	    }
	    
	    widget.dataSourceFilter = filtro;
	    args.e.preference.CUSTOM_FILTER_ROW = args.e.preference.FILTER_ROW;
	};
	
	this.ajustaFiltroOfCdreducont = function (args){
		var widget = templateManager.container.getWidget("grid_cheques_recebidos");
		var rowFilter = args.e.preference.FILTER_ROW;
		
		widget.getField('CDREDUCONT').dataSourceFilter[0].value = rowFilter.P_CDEMPRESA;
		widget.getField('CDREDUCONT').reload();
	};
	
	function concatValuesToFilter (selectNative, selectMultipleCodes){
		if (selectNative === 'T')
			return null;
		else {
			var codes = '_';
			selectMultipleCodes.forEach(function(code){
				codes = codes.concat(code).concat('_');
			});
			return codes;
		}
	}
	
	// Altera as mascaras de agencia e conta 
	this.setMasksConta = function(widget) {
		if(widget.currentRow.CDPICTAGEN) {
			var mascaraAgencia =  widget.currentRow.CDPICTAGEN.replace(/9/g, '*');
			widget.getField('CDAGENCIAMASK').mask.params.mask  = mascaraAgencia;
		}
		if(widget.currentRow.CDPICTCONT) {
			var mascaraConta   = widget.currentRow.CDPICTCONT.replace(/9/g, '*');
			widget.getField('CDCONTCORRMASK').mask.params.mask = mascaraConta;
		}
		widget.redraw();
	};
	
	// Altera a mascara de inscrição de acordo com o tipo(IDTPICEDCHEQ)
	this.setMaskInscricao = function(widget) {
		if(widget.currentRow.IDTPICEDCHEQ === 'J') { 
			widget.getField('NRCEDCHEQREC').mask = 
			{"type": "fix",
            "params": {
                "mask": "99.999.999/9999-99"
            }};
		}
        else if(widget.currentRow.IDTPICEDCHEQ === 'F') {
        	widget.getField('NRCEDCHEQREC').mask = 
			{"type": "fix",
            "params": {
                "mask": "999.999.999-99",
                "acceptOnlyNumbers": true
            }};
        }
        else {
        	widget.getField('NRCEDCHEQREC').mask = {"type": "number", "params": {"acceptOnlyNumbers": true}};
        }
        widget.redraw();
	};
	
	// Altera o label de alguns campos de acordo com parametro tipo
	// Altera a visibilidade da action de acordo com IDSITCHEQREC da row
	this.alteraLabelCampos = function(widget, tipo) {
			widget.getField('CDBANCHEQREC').label = 'C.C. Débito / Banco';
			widget.getField('CDAGECHEQREC').label = 'Agência';
			widget.getField('CDCNTCHEQREC').label = 'Conta';
			widget.getField('CDREDUCONT').label   = 'C.C. do Favorecido';
			widget.getField('CDBANCO').label = 'Banco';
			widget.getField('CDAGENCIAMASK').label = 'Agência';
			widget.getField('CDCONTCORRMASK').label = 'Conta';
			if(widget.currentRow.IDSITCHEQREC === 'P') widget.actions[3].isVisible = true;
			else widget.actions[3].isVisible = false;
		
		widget.redraw();
	};
	
	// Gera apresentação do cheque
	this.gerarApresentacao = function(widget) {
		var rows = widget.getParent().getCheckedRows();
		if(!rows[0])ScreenService.showMessage(mensagens.selecioneRows);
		else {
			Rec15000_ChequeRecebidos_Service.verificaContaFavorecido(rows).then(function(resul) {
				var rowsToUpdate = resul[0];
				var erro = resul[1];
				if(erro === 1 || erro === 2) ScreenService.showMessage(mensagens.favorecido);
				else {
					var apresentacao = widget.currentRow;
					var obj = {
						apresentacao: apresentacao,
						dados: rowsToUpdate
					};
					var dataSet = [obj];
					General.callRestEngine('fin_contaReceber#/gerarApresentacao', 'DataSet', dataSet).then(function(Success){ 
						this.closePopup('FORMAPRESENTACAO', 'grid_cheques_recebidos');
						if(Success.dataset.gerarApresentacao.erroStatus[0] === true) {
							ScreenService.showMessage(mensagens.erroApresentacao);
						}
						ScreenService.goBack();
						ScreenService.showMessage(mensagens.apresentacaoSucesso);
						ScreenService.closePopup();
						widget.getParent().unSelectAll();
					}.bind(this),
					function(callback){
						ScreenService.showMessage(callback.data.error);
					});
				}
			}.bind(this));
		}
	};
	
	// Gera devolução do cheque
	this.gerarDevolucao = function(widget) {
		if(widget.getParent().getCheckedRows().filter(function (filter){
			return filter.IDSITCHEQREC !== 'P';
		}).length === widget.getParent().getCheckedRows().length) ScreenService.showMessage(mensagens.erroDevolucao);
		else {
			var apresentacao = widget.currentRow;
			var obj = {
				apresentacao: apresentacao,
				dados: widget.getParent().getCheckedRows().filter(function (filter){
							return filter.IDSITCHEQREC === 'P';
						})
			};
			var dataSet = [obj];
			General.callRestEngine('fin_contaReceber#/gerarDevolucao', 'DataSet', dataSet).then(function(Success){
				this.closePopup('FORMDEVOLUCAO', 'grid_cheques_recebidos');
				ScreenService.goBack();
				ScreenService.showMessage(mensagens.devolucaoSucesso);
				ScreenService.closePopup();
			}.bind(this),
			function(callback){
				ScreenService.showMessage(callback.data.error);
			});
		}
	};
	
	// Gera compensação do cheque
	this.gerarCompensacao = function(widget) {
		if(widget.isValid()){
			var widgetGridChequesRecebidos = templateManager.container.getWidget("grid_cheques_recebidos");
			var cheques = widget.widgets[1].dataSource.data;
			var obj = {
				compensacao: widget.currentRow,
				cheques: cheques
			};
			var dataSet = [obj];
			this.alteraLabelCampos(widgetGridChequesRecebidos, false);
			General.callRestEngine('fin_contaReceber#/gerarCompensacao', 'DataSet', dataSet).then(function(Success){ 
				this.closePopup('FORMCOMPENSACAO', 'grid_cheques_recebidos');
				ScreenService.goBack();
				ScreenService.showMessage(mensagens.compensacaoSucesso);
				ScreenService.closePopup();
				widgetGridChequesRecebidos.unSelectAll();
				widgetGridChequesRecebidos.reload();
			}.bind(this),
			function(callback){
				ScreenService.showMessage(callback.data.error);
			});	
		}
	};
	
	// Altera o status do cheque para "Em Custódia" ou "A compensar" de acordo com o parametro tipo
	this.setCustodia = function(widget, tipo) {
		// tipo = B -> Situar em custódia("Em custódia")
		// tipo = A -> Retirar de custódia("A compensar")
		var confirmMessage;
		if(tipo === 'A') confirmMessage = mensagens.retirarCustodia;
		else if(tipo === 'B') confirmMessage = mensagens.situarCustodia;
		var rows = widget.getCheckedRows();
		if(!rows[0])ScreenService.showMessage(mensagens.selecioneRows);
		else {
			ScreenService.confirmMessage(ScreenService.i18n(confirmMessage), 'question', function (success) {
				Rec15000_ChequeRecebidos_Service.verificaContaFavorecido(rows).then(function(resul) {
					var rowsToUpdate = resul[0];
					var erro = resul[1];
					var obj = {
						tipo: tipo,
						dados: rowsToUpdate
					};
					if(erro === 2) ScreenService.showMessage(mensagens.favorecidoT);
					else {
						var dataSet = [obj];
						General.callRestEngine('fin_contaReceber#/setCustodia', 'DataSet', dataSet).then(function(Success){
							if(Success.dataset.setCustodia.erroStatus[0] === true) {
								if(tipo ==='B')	ScreenService.showMessage(mensagens.erroSCustodia);
								if(tipo ==='A')	ScreenService.showMessage(mensagens.erroRCustodia);
							}
							else if(erro === 1) ScreenService.showMessage(mensagens.favorecidoP);
							ScreenService.goBack();
							widget.reload();
							widget.unSelectAll();
							ScreenService.closePopup();
						}.bind(this),
						function(callback){
							ScreenService.showMessage(callback.data.error);
						});
					}
				}.bind(this));
			}.bind(this));
		}
	};
	
	// Abre e ativa a widget equivalente ao popupName
	this.openPopup = function(widget, popupName, verificaRows) {
		var rows;
		if(verificaRows) rows = widget.getCheckedRows();
		if(rows || !verificaRows) {
			var widgetPopup = templateManager.container.getWidget(popupName);
			if(popupName === 'FORMAPRESENTACAO') {
				widgetPopup.currentRow.DTAPRCHEQREC = Rec15000_ChequeRecebidos_Service.getDataAtual();
			}
			else if(popupName === 'FORMDEVOLUCAO') {
				widgetPopup.currentRow.DTDEVCHEQREC = Rec15000_ChequeRecebidos_Service.getDataAtual();
			}
			else if(popupName === 'FORMCOMPENSACAO') {
				this.openCompensacao(widget, widgetPopup);
				this.setObservacao(widgetPopup);
				widgetPopup.currentRow.IDSTATUSBREC = 'P';
				widgetPopup.currentRow.DTCOMPENSACAO = Rec15000_ChequeRecebidos_Service.getDataAtual();
			}
			if(popupName === 'FORMAPRESENTACAO' || popupName === 'FORMDEVOLUCAO') {
				ScreenService.openPopup(widgetPopup);
				widgetPopup.isVisible = true;
				widgetPopup.activate();
				widgetPopup.edit();
				widgetPopup.redraw();
			}
		}
		else {
			ScreenService.showMessage(mensagens.selecioneRows);
		}
    };
    
    // Ajusta os valores para o filtro de boleto e cheque a serem compensados e abre popup
    this.openCompensacao = function(widget, widgetPopup){
    	var rows = widget.getCheckedRows();
		if(!rows[0])ScreenService.showMessage(mensagens.selecioneRows);
		else {
			Rec15000_ChequeRecebidos_Service.verificaContaFavorecido(rows).then(function(resul) {
				var erro = resul[1];
				var cdreducont = rows[0].CDREDUCONT;
				var erroDif = false;
				var erroCompens = false;
		    	rows.forEach(function(row){
					if(row.CDREDUCONT !== cdreducont) erroDif = true;
					if(row.IDSITCHEQREC === 'C') erroCompens = true;
				});
				if(erro === 1 || erro === 2) ScreenService.showMessage(mensagens.favorecido);
				else if(erroDif === true) ScreenService.showMessage(mensagens.favorecidoDiferente);
				else if(erroCompens === true) ScreenService.showMessage(mensagens.erroChequeCompensado);
				else {
			    	var widgetBoleto = widgetPopup.widgets[0];
			    	var widgetCheque = widgetPopup.widgets[1];
			    	var cheques = [];
			    	var bancos = [];
			    	var agencias = [];
			    	var contas = [];
			    	rows.forEach(function(row){
			    		cheques.splice(cheques.length, 1, row.NRCHEQUERECE);
			    		bancos.splice(bancos.length, 1, row.CDBANCHEQREC);
			    		agencias.splice(agencias.length, 1, row.CDAGECHEQREC);
			    		contas.splice(contas.length, 1, row.CDCNTCHEQREC);
			    	});
			    	widgetBoleto.dataSourceFilter[0].value = cheques;
			    	widgetBoleto.dataSourceFilter[1].value = bancos;
			    	widgetBoleto.dataSourceFilter[2].value = agencias;
			    	widgetBoleto.dataSourceFilter[3].value = contas;
			    	
			    	widgetCheque.dataSourceFilter[0].value = cheques;
			    	widgetCheque.dataSourceFilter[1].value = bancos;
			    	widgetCheque.dataSourceFilter[2].value = agencias;
			    	widgetCheque.dataSourceFilter[3].value = contas;
			    	
			    	widgetBoleto.reload().then(function() {
			    		if(!widgetBoleto.dataSource.data[0]) {
							ScreenService.showMessage(mensagens.erroBoletos);
							ScreenService.goBack();
							ScreenService.closePopup();
			    		}
			    		else {
							ScreenService.openPopup(widgetPopup);
							widgetPopup.isVisible = true;
							widgetPopup.activate();
							widgetPopup.edit();
							widgetPopup.redraw();
			    			widgetCheque.reload();
			    		}
			    	});
				}
			});
		}
    };
    
    // Define o texto do campo obs
    this.setObservacao = function(widget) {
    	widget.currentRow.OBS = 'Atenção! Todos os cheques dos boletos abaixo, serão compensados.'; 
    };
    
    // Fecha a widget equivalente ao popupName
    // Abre e ativa a widget equivalente ao widgetName
    this.closePopup = function(popupName, widgetName) {
    	var widget = templateManager.container.getWidget(popupName);
        widget.currentRow = {};
        var widgetActivate = templateManager.container.getWidget(widgetName);
        ScreenService.closePopup(widget);
		ScreenService.goBack();
        widgetActivate.activate();
        widgetActivate.reload();
        widget.isVisible = false;
    };
	
	// Salva um novo cheque recebido
	this.save = function(widget) {
		if(widget.isValid()) {
			var widgetBoleto = templateManager.container.getWidget('grid_boleto');
			widgetBoleto.reload().then(function() {
				if(widgetBoleto.dataSource.data[0] && parseFloat(widget.currentRow.VRCHEQUERECE) != parseFloat(BKPcurrentRow.VRCHEQUERECE)) {
					ScreenService.showMessage(mensagens.opBlockedBoleto);
				}else {
					if (parseFloat(widget.currentRow.VRCHEQUERECE) <= 0){
						ScreenService.showMessage("O valor do cheque deve ser maior que 0");
					} else {
						widget.currentRow.CDEMPRESA    = widget.appliedFilter.FILTER_ROW.P_CDEMPRESA;
						if(!widget.currentRow.NMCEDCHEQREC) widget.currentRow.NMCEDCHEQREC = widget.currentRow.NMRAZSOCCLIE.substring(0,40);
						if(!widget.currentRow.NRCEDCHEQREC && widget.currentRow.IDTIPCHEQREC === 'P') {
							widget.currentRow.NRCEDCHEQREC = widget.currentRow.NRINSJURCLIE;
							widget.currentRow.IDTPICEDCHEQ = changeIdtpicedcheqValue(widget);
						}
						if(widget.currentRow.NMCEDCHEQREC && !widget.currentRow.IDTPICEDCHEQ) widget.currentRow.IDTPICEDCHEQ = changeIdtpicedcheqValue(widget); 
						General.saveCrud(widget).then(function(Success){
							var widgetBoleto =  templateManager.container.getWidget('grid_boleto');
							widgetBoleto.dataSource.name = 'fin_contaReceber#/crud/boletreceber_tprecboleto_cliente';
							widgetBoleto.dataSourceFilter = [
			                    {
			                        "name": "NRCHEQUE",
			                        "operator": "=",
			                        "value": General.getFieldValue('NRCHEQUERECE', 'grid_cheques_recebidos')
			                    },
			                    {
			                        "name": "CDBANCO",
			                        "operator": "=",
			                        "value": General.getFieldValue('CDBANCHEQREC', 'grid_cheques_recebidos')
			                    },
			                    {
			                        "name": "CDAGENCIA",
			                        "operator": "=",
			                        "value": General.getFieldValue('CDAGECHEQREC', 'grid_cheques_recebidos')
			                    },
			                    {
			                        "name": "CDCONTCORR",
			                        "operator": "=",
			                        "value": General.getFieldValue('CDCNTCHEQREC', 'grid_cheques_recebidos')
			                	}
			                ];
			                widgetBoleto.actions[2].isVisible = false;
			                widgetBoleto.actions[3].isVisible = true;
			                widgetBoleto.reload();
			                widget.reload([], false);
			                General.changeWidgetAcessibility(widget.widgets, true);
						},
						function(callback) {
							 ScreenService.showMessage(callback.data.error);
						});
					}
				}
			});
		}
	};
	
	function changeIdtpicedcheqValue (widget){
		switch (widget.currentRow.IDTPIJURCLIE){
			case "F":
				return widget.currentRow.IDTPIJURCLIE;
			case "J":
				return widget.currentRow.IDTPIJURCLIE;
			default :
				return null;
		}
	}
	
	this.openFilterBoletosCheque = function(widget) {
		// Alterar o numero da widget recebida depois
		var widgetPopup = widget.widgets[1];
		widgetPopup.isVisible = true;
		widgetPopup.activate();
		widgetPopup.edit();
		
		widgetPopup.currentRow.CDEMPRESA = widget.getParent().appliedFilter.FILTER_ROW.P_CDEMPRESA;
		widgetPopup.currentRow.CDCLIENTE = widget.getParent().currentRow.CDCLIENTE;
		widgetPopup.currentRow.TCDFILIAL = 'T';
		widgetPopup.currentRow.TCDREDUCONT = 'T';
		widgetPopup.currentRow.TCDTPBAIXAREC = 'T';
		widgetPopup.currentRow.DTVENBOLREC = '';
		widgetPopup.currentRow.DTEMISSAO   = '';
		ScreenService.openPopup(widgetPopup);
	};
	
	this.changeReadOnlyLista = function(field) {
		var widget = field.widget;
		var fieldChange;
		switch(field.name) {
			case 'TCDFILIAL':
				fieldChange = widget.getField('NMFILIAL');
				fieldChange.readOnly = (field.value() == 'T');
				if(field.value() == 'T') fieldChange.setValue('');
				break;
			case 'TCDREDUCONT':
				fieldChange = widget.getField('CDREDUCONT');
				fieldChange.readOnly = (field.value() == 'T');
				if(field.value() == 'T') fieldChange.setValue('');
				break;
			case 'TCDTPBAIXAREC':
				fieldChange = widget.getField('NMTPBAIXAREC');	
				fieldChange.readOnly = (field.value() == 'T');
				if(field.value() == 'T') fieldChange.setValue('');
				break;
		}
	};
	
	this.cancelar = function(args) {
		var widgetBoleto = args.owner.widget.widgets[0];
        this.alteraLabelCampos(args.owner.widget, false);
		widgetBoleto.dataSourceFilter = [
            {
                "name": "NRCHEQUE",
                "operator": "=",
                "value": "@General.getFieldValue('NRCHEQUERECE', 'grid_cheques_recebidos')@"
            },
            {
                "name": "CDBANCO",
                "operator": "=",
                "value": "@General.getFieldValue('CDBANCHEQREC', 'grid_cheques_recebidos')@"
            },
            {
                "name": "CDAGENCIA",
                "operator": "=",
                "value": "@General.getFieldValue('CDAGECHEQREC', 'grid_cheques_recebidos')@"
            },
            {
                "name": "CDCONTCORR",
                "operator": "=",
                "value": "@General.getFieldValue('CDCNTCHEQREC', 'grid_cheques_recebidos')@"
            }
        ];
        General.backOrCancel(args).then(function(){
        	args.owner.widget.reload();
        	widgetBoleto.dataSource.name = 'fin_contaReceber#/crud/boletreceber_tprecboleto_cliente';
			widgetBoleto.actions[2].isVisible = false;
            widgetBoleto.actions[3].isVisible = true;
            widgetBoleto.reload();
            General.changeWidgetAcessibility(args.owner.widget.widgets, true);
        });
	};
	
	this.cancelarFiltroBoletos = function(widget) {
		var widgetBoletos = widget.getParent();
		widgetBoletos.showCheckbox = false;
		ScreenService.closePopup(widget);
		widget.isVisible = false;
		widgetBoletos.activate();	
	};
	
	this.filtrarBoletosCheque = function(widget) {
		var widgetBoletos = templateManager.container.getWidget('grid_boleto');
		var filter = [
			{
				"name": "CDEMPRESA",
				"operator": "=",
				"value": widget.currentRow.CDEMPRESA
			},
			{
				"name": "CDCLIENTE",
				"operator": "=",
				"value": widget.currentRow.CDCLIENTE
			},
			{
				"name": "TDTVENC",
				"operator": "=",
				"value": (!widget.currentRow.DTVENBOLREC.start && !widget.currentRow.DTVENBOLREC.end) ? 'T' : ''
			},
			{
				"name": "DTVENCINI",
				"operator": "=",
				"value": widget.currentRow.DTVENBOLREC.start || ''
			},
			{
				"name": "DTVENCFIN",
				"operator": "=",
				"value": widget.currentRow.DTVENBOLREC.end || ''
			},
			{
				"name": "TNRDOC",
				"operator": "=",
				"value": (!widget.currentRow.NRDOCINI && !widget.currentRow.NRDOCFIN) ? 'T' : ''
			},
			{
				"name": "NRDOCINI",
				"operator": "=",
				"value": widget.currentRow.NRDOCINI || ''
			},
			{
				"name": "NRDOCFIN",
				"operator": "=",
				"value": widget.currentRow.NRDOCFIN || ''
			},
			{
				"name": "TDTEMISSAO",
				"operator": "=",
				"value": (!widget.currentRow.DTEMISSAO.start && !widget.currentRow.DTEMISSAO.end) ? 'T' : ''
			},
			{
				"name": "DTVENCINI",
				"operator": "=",
				"value": widget.currentRow.DTEMISSAO.start || ''
			},
			{
				"name": "DTVENCFIN",
				"operator": "=",
				"value": widget.currentRow.DTEMISSAO.end || ''
			},
			{
				"name": "TVRLIQ",
				"operator": "=",
				"value": (!widget.currentRow.VRLIQINI && !widget.currentRow.VRLIQFIN) ? 'T' : ''
			},
			{
				"name": "VRLIQINI",
				"operator": "=",
				"value": widget.currentRow.VRLIQINI || ''
			},
			{
				"name": "VRLIQFIN",
				"operator": "=",
				"value": widget.currentRow.VRLIQFIN || ''
			}
		];
		if(widget.currentRow.TCDFILIAL == 'L') filter.push({
				"name": "CDFILIEMIT",
				"operator": "IN",
				"value": widget.currentRow.CDFILIEMIT
			});
		if(widget.currentRow.TCDREDUCONT == 'L') filter.push({
				"name": "CDREDUCONT",
				"operator": "IN",
				"value": widget.currentRow.CDREDUCONT
			});
		if(widget.currentRow.TCDTPBAIXAREC == 'L') filter.push({
				"name": "CDTPBAIXAREC",
				"operator": "IN",
				"value": widget.currentRow.CDTPBAIXAREC
			});
		widgetBoletos.dataSourceFilter = filter;
		widgetBoletos.reload().then(function(success){
			widgetBoletos.showCheckbox = true;
			widgetBoletos.redraw();
			ScreenService.closePopup(widget);
			widget.isVisible = false;
			widgetBoletos.activate();
		});
	};
	
	this.openWidgetBoletosCheque = function(widget) {
		var widgetParent = templateManager.container.getWidget('grid_cheques_recebidos');
		
		if (widgetParent.currentRow.IDSITCHEQREC != 'C' && widgetParent.currentRow.IDSITCHEQREC != 'B' && widgetParent.currentRow.IDSITCHEQREC != 'R' && widgetParent.currentRow.IDSITCHEQREC != 'P'){
			
			var form_new_boleto = templateManager.container.getWidget('form_new_boleto');
			
			ScreenService.openSwipe(form_new_boleto, false).then(function (){
				form_new_boleto.fields.forEach(function(field){
					widget.getParent().fields.forEach(function(fieldParent){
						if(field.name === fieldParent.name){
							field.setValue(fieldParent.value());
						}
					});
				});
				var valor = 0;
				widget.dataSource.data.forEach(function (data){
					valor = valor + data.VRFINALBREC;
				});
				valor = form_new_boleto.getField('VRCHEQUERECE').value() - valor;
				if (valor > 0){
					form_new_boleto.getField('VRRESTANTECHEQUE').setValue(valor);
				} else {
					form_new_boleto.getField('VRRESTANTECHEQUE').setValue(0);
				}
				//form_new_boleto.edit();
				
				//form_new_boleto.formTemplate = "widget/tabbedWidget.html";
				
				templateManager.container.getWidget('grid_new_boleto').dataSource.data = [];
				//templateManager.container.getWidget('grid_new_boleto').getAction('FiltroBoletos').isVisible = false;
				
				//templateManager.container.getWidget('grid_new_boleto').template = templateManager.container.getWidget('grid_new_boleto').formTemplate;
				templateManager.updateTemplate();
			});
		}else ScreenService.showMessage(mensagens.deleteBlocked);
	};
	
	this.activateFilterOnAddBoleto = function (widget){
		//if(General.getFieldValue('NMTIPORECE', 'form_new_boleto')){
			//widget.widgets[0].getAction('FiltroBoletos').isVisible = true;
			widget.widgets[0].dataSource.name = 'fin_contaReceber#/crud/boletreceber_relacionamentocheque_filter';
		//} else {
			//widget.widgets[0].getAction('FiltroBoletos').isVisible = false;
		//}
	};
	
	this.filtrarBoletosChequeOnAdd = function(widget) {
		var widgetBoletos = templateManager.container.getWidget('grid_new_boleto');
		var filter = [
			{
				"name": "CDEMPRESA",
				"operator": "=",
				"value": widget.currentRow.CDEMPRESA
			},
			{
				"name": "CDCLIENTE",
				"operator": "=",
				"value": widget.currentRow.CDCLIENTE
			},
			{
				"name": "DTVENCINI",
				"operator": "=",
				"value": widget.currentRow.DTVENBOLREC.start || ''
			},
			{
				"name": "DTVENCFIN",
				"operator": "=",
				"value": widget.currentRow.DTVENBOLREC.end || ''
			},
			{
				"name": "TDTVENC",
				"operator": "=",
				"value": (!widget.currentRow.DTVENBOLREC.start && !widget.currentRow.DTVENBOLREC.end) ? 'T' : ''
			},
			{
				"name": "CDFILIORIG",
				"operator": "=",
				"value": concatValuesToFilter(widget.currentRow.TCDFILIAL, widget.currentRow.CDFILIEMIT)
			},
			{
				"name": "TCDFILIORIG",
				"operator": "=",
				"value": widget.currentRow.TCDFILIAL === 'T' ? 'T' : ''
			},
			{
				"name": "NRDOCINI",
				"operator": "=",
				"value": widget.currentRow.NRDOCINI || ''
			},
			{
				"name": "NRDOCFIN",
				"operator": "=",
				"value": widget.currentRow.NRDOCFIN || ''
			},
			{
				"name": "TNRDOC",
				"operator": "=",
				"value": (!widget.currentRow.NRDOCINI && !widget.currentRow.NRDOCFIN) ? 'T' : ''
			},
			{
				"name": "DTEMISINI",
				"operator": "=",
				"value": widget.currentRow.DTEMISSAO.start || '30/12/1899'
			},
			{
				"name": "DTEMISFIN",
				"operator": "=",
				"value": widget.currentRow.DTEMISSAO.end || '30/12/1899'
			},
			{
				"name": "TDTEMISSAO",
				"operator": "=",
				"value": (!widget.currentRow.DTEMISSAO.start && !widget.currentRow.DTEMISSAO.end) ? 'T' : ''
			},
			{
				"name": "CONTACORR",
				"operator": "=",
				"value": concatValuesToFilter(widget.currentRow.TCDREDUCONT, widget.currentRow.CDREDUCONT)
			},
			{
				"name": "TCONTACORR",
				"operator": "=",
				"value": widget.currentRow.TCDREDUCONT === 'T' ? 'T' : ''
			},
			{
				"name": "TPBX",
				"operator": "=",
				"value":concatValuesToFilter(widget.currentRow.TCDTPBAIXAREC, widget.currentRow.CDTPBAIXAREC)
			},
			{
				"name": "TTPBX",
				"operator": "=",
				"value": widget.currentRow.TCDTPBAIXAREC === 'T' ? 'T' : ''
			},
			{
				"name": "VRLIQINI",
				"operator": "=",
				"value": widget.currentRow.VRLIQINI || '0'
			},
			{
				"name": "VRLIQFIN",
				"operator": "=",
				"value": widget.currentRow.VRLIQFIN || '0'
			},
			{
				"name": "TVRLIQ",
				"operator": "=",
				"value": (!widget.currentRow.VRLIQINI && !widget.currentRow.VRLIQFIN) ? 'T' : ''
			}
		];
		widgetBoletos.dataSourceFilter = filter;
		widgetBoletos.reload().then(function(success){
			widgetBoletos.showCheckbox = true;
			widgetBoletos.redraw();
			ScreenService.closePopup(widget);
			widget.isVisible = false;
			widgetBoletos.activate();
		});
	};
	
	this.openFilterBoletosChequeOnAdd = function(widget) {
		// Alterar o numero da widget recebida depois
		var widgetPopup = widget.widgets[1];
		widgetPopup.isVisible = true;
		widgetPopup.activate();
		widgetPopup.edit();
		
		if(Util.isEmptyOrBlank(widgetPopup.currentRow.CDEMPRESA)){
			widgetPopup.currentRow.TCDFILIAL = 'T';
			widgetPopup.currentRow.CDFILIAL = '';
			widgetPopup.currentRow.NMFILIAL = '';
			widgetPopup.currentRow.TCDREDUCONT = 'T';
			widgetPopup.currentRow.CDREDUCONT = '';
			widgetPopup.currentRow.NMREDUCONT = '';
			widgetPopup.currentRow.TCDTPBAIXAREC = 'T';
			widgetPopup.currentRow.CDTPBAIXAREC = '';
			widgetPopup.currentRow.NMTPBAIXAREC = '';
			widgetPopup.currentRow.DTVENBOLREC = '';
			widgetPopup.currentRow.DTEMISSAO   = '';
		}
		
		widgetPopup.currentRow.CDEMPRESA = widget.getParent().getParent().getParent().appliedFilter.FILTER_ROW.P_CDEMPRESA;
		widgetPopup.currentRow.CDCLIENTE = widget.getParent().getParent().getParent().currentRow.CDCLIENTE;
		
		ScreenService.openPopup(widgetPopup);
	};
	
	this.updateRemainingValueOnCheck = function (widget){
		widget.currentRow = widget.getLastSelectedRow();
		
		if(widget.getParent().currentRow.VRRESTANTECHEQUE >= widget.currentRow.VRFINALBREC) {
			widget.currentRow.VRTPRECBOLR = widget.currentRow.VRFINALBREC;
			widget.getParent().currentRow.VRRESTANTECHEQUE = calculaValorRestanteNoCheque(widget);
		} else {
			widget.currentRow.__isSelected = false;
			widget.currentRow.VRTPRECBOLR = 0;
			widget.redraw();
			var rows = [widget.currentRow];
			widget.unSelectAll(rows);
			ScreenService.showMessage('Cheque não possui valores disponìveis para este boleto');
		}
		widget.currentRow = {};
		widget.updateCheckedRows();
	};
	
	this.updateRemainingValueOnUncheck = function (widget){
		widget.dataSource.data.forEach(function(data){
			if(data.__isSelected === false && data.VRTPRECBOLR != 0){
				data.VRTPRECBOLR = 0;
			}
		});
		widget.updateCheckedRows();
		widget.getParent().currentRow.VRRESTANTECHEQUE = calculaValorRestanteNoCheque(widget);
	};
	
	function calculaValorRestanteNoCheque (widget){
		var valorRestante = 0;
		widget.getParent().getParent().dataSource.data.forEach(function(data){
			valorRestante += formatFloat(data.VRFINALBREC);
		});
		widget.getCheckedRows().forEach(function(row){
			valorRestante += formatFloat(row.VRFINALBREC);
		});
		valorRestante = widget.getParent().currentRow.VRCHEQUERECE - valorRestante;
		return valorRestante;
	}
	
	this.setVisibilityAddBoleto = function (widget){
		var widgetParent = widget.getParent();
		if (widgetParent.currentRow.IDSITCHEQREC != 'C' && widgetParent.currentRow.IDSITCHEQREC != 'B' && widgetParent.currentRow.IDSITCHEQREC != 'R' && widgetParent.currentRow.IDSITCHEQREC != 'P'){
			widget.getAction('Adicionar').isVisible = true;
		} else {
			widget.getAction('Adicionar').isVisible = false;
		}
	};
	
	this.deleteBoleto = function (widget){
		var widgetCheque = templateManager.container.getWidget('grid_cheques_recebidos');
		if(widgetCheque.currentRow.IDSITCHEQREC === 'C') ScreenService.showMessage(mensagens.opBlocked);
		else {
			if(widgetCheque.currentRow.IDSITCHEQREC === 'R'|| widgetCheque.currentRow.IDSITCHEQREC ==='B' || widgetCheque.currentRow.IDSITCHEQREC ==='T') ScreenService.showMessage(mensagens.deleteBlocked);
			else{
				ScreenService.confirmMessage(ScreenService.i18n(mensagens.confirmDelete), 'question', function (success) {
					General.callRestEngine('fin_contaReceber#/deleteBoleto', 'Row', widget.currentRow).then(function(Success){
						ScreenService.closeSwipe();
						widget.reload();
						widget.activate();
					}.bind(this),
					function (error) {
						if (error.data && error.data.error.indexOf('ORA-02292') !== -1) {
							ScreenService.showMessage('Exclusão não permitida! O registro possui registros filhos.');
						} else {
							ScreenService.showMessage(error.data.error);
						}
					});
				}.bind(this));
			}
		}
	};
	
	this.saveBoleto = function (args){
		var deferred = ZHPromise.defer();
		var widget = args.owner.widget;
		var widgetCheque = widget.getParent().getParent().getParent();
		var widgetBoleto = widget.getParent().getParent();
		var checkedRows;

		if (widget.getParent().isValid()) {
			if (widget.isValid()) {
				widget.currentRow.changed = true;
				checkedRows = widget.getCheckedRows().filter(function(currentRow){
					return currentRow.__isSelected === true;
				});
				checkedRows.forEach(function(currentRow){
					if(currentRow.__isSelected === true){
						currentRow.CDTIPORECE = widget.getParent().currentRow.CDTIPORECE;
						currentRow.NRSEQTPRECBO = '01';
						currentRow.NRCHEQBOLREC = widgetCheque.currentRow.NRCHEQUERECE;
						currentRow.CDBANCBOLREC = widgetCheque.currentRow.CDBANCHEQREC;
						currentRow.CDAGENBOLREC = widgetCheque.currentRow.CDAGECHEQREC;
						currentRow.CDCONTBOLREC = widgetCheque.currentRow.CDCNTCHEQREC;
					}
				});
	
				General.callRestEngine('fin_contaReceber#/saveBoleto', 'DataSet', checkedRows).then(function (response) {
					ScreenService.closePopup(templateManager.container.getActiveWidget());
					General.backOrCancel(args);
					widgetBoleto.activate();
					widgetBoleto.reload();
					widget.unSelectAll();
					deferred.resolve(response);
				}, function (error) {
					var errorMessage = 'Timeout';
	
					if (error.data) errorMessage = error.data.error;
	
					ScreenService.showMessage(errorMessage);
					deferred.reject(errorMessage);
				});
			} else deferred.reject("Preencha todos os campos corretamente!");
		} else deferred.reject("Preencha todos os campos corretamente!");

		return deferred;
			
	};
	
	this.reloadWidgets = function (widget){
		widget.reload();
		widget.widgets.forEach(function(widgetFilha){
			widgetFilha.reload();
		});
	};
	
	function formatFloat (value){
        value = parseFloat(value.replace(',','.'));
        return value;
	}
	
	this.changeInscrcaoMask = function (widget){
		if (widget.currentRow.NRINSJURCLIE !== undefined){
			if (widget.currentRow.NRINSJURCLIE.length === 14){
				widget.getField('NRINSJURCLIE').mask = {
						"type": "fix",
			                "params": {
			                	"mask": "99.999.999/9999-99"
			                }
					};
			}else {
				if (widget.currentRow.NRINSJURCLIE.length === 11){
					 widget.getField('NRINSJURCLIE').mask = {
							"type": "fix",
			                    "params": {
			                    	"mask": "999.999.999-99"
			                    }
						};					
				} else {
				    widget.getField('NRINSJURCLIE').mask = {"type": "number"};
				}
			}
		}
	};
	
    this.habilitaActions = function(){
        var widget = templateManager.container.getWidget('grid_cheques_recebidos');
        var checkedrows = widget.getCheckedRows();

        var actions = {
        	situacustodia  : false,
			retiracustodia : false,
			apresentacao   : false,
			devolucao      : false,
			compensacao    : false
        };

        if(checkedrows.length > 0) {
            for (var i in checkedrows){

                 if (checkedrows[i].IDSITCHEQREC == 'A') {
		            actions.situacustodia = true;
		            actions.apresentacao  = true;
		            actions.compensacao   = true;
                 }else if(checkedrows[i].IDSITCHEQREC == 'P'){
                 	actions.devolucao     = true;
		            actions.compensacao   = true;
                 }else if(checkedrows[i].IDSITCHEQREC == 'D'){
                 	actions.apresentacao  = true;
		            actions.compensacao   = true;
		            actions.situacustodia = true;
                 }else if(checkedrows[i].IDSITCHEQREC == 'B'){
                 	actions.apresentacao  = true;
		            actions.compensacao   = true;
		            actions.retiracustodia = true;
                 }else if(checkedrows[i].IDSITCHEQREC == 'C'){
                 }

            }
        }

        widget.getAction('SCUSTODIA').isVisible = actions.situacustodia;
        widget.getAction('RCUSTODIA').isVisible = actions.retiracustodia;
        widget.getAction('AP').isVisible        = actions.apresentacao;
        widget.getAction('GD').isVisible        = actions.devolucao;
        widget.getAction('GC').isVisible        = actions.compensacao;
    };
    
    this.setaMascCnpjCpfCliente = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    data.forEach(function(value){
	        if(value.NRINSJURCLIE.length === 14) inscFormatada = value.NRINSJURCLIE.substr(0, 2) + '.' + value.NRINSJURCLIE.substr(2, 3) + '.' + value.NRINSJURCLIE.substr(5, 3) + '/' + value.NRINSJURCLIE.substr(8, 4) + '-' + value.NRINSJURCLIE.substr(12, 2);
          else if(value.NRINSJURCLIE.length === 11) inscFormatada = value.NRINSJURCLIE.substr(0, 3) + '.' + value.NRINSJURCLIE.substr(3, 3) + '.' + value.NRINSJURCLIE.substr(6, 3) + '-' + value.NRINSJURCLIE.substr(9, 2);
          else inscFormatada = null;
          value.NRINSJURCLIE = inscFormatada?inscFormatada:value.NRINSJURCLIE;
	    });
	};
	
};
/*global Configuration*/
Configuration(function (ContextRegister){
    ContextRegister.register('Rec15000_ChequeRecebidos_Controller', Rec15000_ChequeRecebidos_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec18000_Compensacao_Conta_Receber.js
var Rec18000_Compensacao_Conta_Receber = function(ScreenService, templateManager, General, Rec18000_CompensacaoContaReceber_Service, ZHPromise, ZhFilterPreferences){

    var wasFiltered = false;

    this.openFilterCompensacaoReceber = function(compensacaoContasReceber){
        var data = moment().format('DD/MM/YYYY');
        var rowFilter = {'DTCOMPCONTREC': {start : data, end : data}};
        this.setDefaultValueFromFilter().then(function(filialLogada){
            rowFilter.CDFILIAL = filialLogada.CDFILIAL;
            rowFilter.NRORG    = filialLogada.NRORG;
            ZhFilterPreferences.setOriginalFilter(compensacaoContasReceber.id, rowFilter);
            ZhFilterPreferences.openFilterWidget(compensacaoContasReceber, rowFilter); 
        });
    };
    
    this.setDefaultValueFromFilter = function (){
        var deferred = ZHPromise.defer();
        Rec18000_CompensacaoContaReceber_Service.getFilialLogada().then(function(filialLogada) {
            deferred.resolve(filialLogada[0]);
        });
        return deferred.promise;
    };

    this.getDefaulInterval = function (){
        var obj = {
            "start": moment().startOf('month').format('DD/MM/YYYY'),
            "end": moment().endOf('month').format('DD/MM/YYYY')
        };
        return obj;
    };

    this.getCurrentField = function(widgetName, fieldName) {
        return General.getFieldValue(fieldName,widgetName);
    };

    this.validaDatasBaixaCompensacao = function(widget, fieldClear){
        var arrayBaixa = widget.currentRow.DTBAIXACOMP.split('/');
        var arrayComp = widget.currentRow.DTCOMPENCTPAG.split('/');
        var anoBaixa = arrayBaixa.pop();
        var mesBaixa = arrayBaixa.pop();
        var diaBaixa = arrayBaixa.pop();
        var anoComp = arrayComp.pop();
        var mesComp = arrayComp.pop();
        var diaComp = arrayComp.pop();
        var dataBaixa = parseInt(anoBaixa.concat(mesBaixa.concat(diaBaixa)));
        var dataCompensacao = parseInt(anoComp.concat(mesComp.concat(diaComp)));
        if (dataBaixa <= dataCompensacao){
            ScreenService.showMessage('Data da baixa deve ser igual ou maior à data da compensação.');
            fieldClear.setValue('');
        }
    };

    this.adicionaCompensacao = function(widget){
        this.novaCompensacao(widget).then(function(){
            this.validaItemCompensacao(widget);
        }.bind(this));
    };

    this.novaCompensacao = function(widget){
        var deferred = ZHPromise.defer();
        General.widgetNewRow(widget);
        widget.getField('IDTPITUTLCOMREC').setValue('0');
        widget.getField('CONTAPAGAR').isVisible = true;
        widget.getField('CONTACORRENTE').isVisible = true;
        widget.getField('CONTARECEBER').isVisible = true;
        widget.getField('VRTOTALCOMREC').class = '3';
        widget.getField('SOMACONTACORRENTE').class = '3';
        templateManager.container.getWidget('TitulosCompensadosPagar').isVisible = false;
        templateManager.container.getWidget('ContasCorrentesCompensadas').isVisible = false;
        deferred.resolve();
        return deferred.promise;
    };

    this.cancelaCompensacao = function(widget){
        General.widgetCancelForm(widget);
        widget.getField('CONTAPAGAR').isVisible = false;
        widget.getField('CONTACORRENTE').isVisible = false;
        widget.getField('CONTARECEBER').isVisible = false;
        widget.getField('VRTOTALCOMREC').class = '12';
        widget.getField('SOMACONTACORRENTE').class = '12';
        templateManager.container.getWidget('TitulosCompensadosPagar').isVisible = true;
        templateManager.container.getWidget('ContasCorrentesCompensadas').isVisible = true;
    };

    this.ativaFiltroContasPagar = function (args) {
        if (!wasFiltered) {
            var filtroContaPagar = templateManager.container.getWidget('filtroContaPagar');
            ScreenService.openPopup(filtroContaPagar).then(function () {
                filtroContaPagar.activate();
            });
            args.e.preventDefault();
        } else {
            wasFiltered = false;
        }
    };

    this.aplicaFiltroCompensacaoReceber = function () {
        var compensacao = templateManager.container.getWidget('CompensacaoContasReceber');
        var filtro = compensacao.appliedFilter;
        var operador = '';
        var cdforneced = '';
        var lancIni = filtro.FILTER_ROW.DTLANCCOMPREC.start;
        var lancFin = filtro.FILTER_ROW.DTLANCCOMPREC.end;
        var compIni = filtro.FILTER_ROW.DTCOMPENCTREC.start;
        var compFin = filtro.FILTER_ROW.DTCOMPENCTREC.end;
        var operadorMulti = filtro.FILTER_ROW.CDOPERADOR;
        var fornecedorMulti = filtro.FILTER_ROW.CDCLIENTE;
        if (operadorMulti) {
            operadorMulti.forEach(function (filter) {
                operador += "_" + filter;
            });
            operador += "_";
        }
        if (fornecedorMulti) {
            fornecedorMulti.forEach(function (filter) {
                cdforneced += "_" + filter;
            });
            cdforneced += "_";
        }
        compensacao.dataSourceFilter.forEach(function (data) {
            switch (data.name) {
                case 'CDEMPRESA':
                    data.value = filtro.FILTER_ROW.CDEMPRESA;
                    break;
                case 'CDOPERADOR':
                    data.value = operador;
                    break;
                case 'TCDOPERADOR':
                    data.value = (operador) ? null : 'T';
                    break;
                case 'CDCLIENTE':
                    data.value = cdforneced;
                    break;
                case 'TCDCLIENTE':
                    data.value = (cdforneced) ? null : 'T';
                    break;
                case 'DTCOMPENSACAO_INI': data.value = compIni;
                    break;
                case 'DTCOMPENSACAO_FIN': data.value = compFin;
                    break;
                case 'TCOMPENSACAO': data.value = (compIni && compFin) ? null : 'T';
                    break;
                case 'DTLANCAMENTO_INI': data.value = lancIni;
                    break;
                case 'DTLANCAMENTO_FIN': data.value = lancFin;
                    break;
                case 'TLANCAMENTO': data.value = (lancIni && lancFin) ? null : 'T';
                    break;
            }
        });
    };

    this.validaItemCompensacao = function (widget) {
        if (widget.currentRow.IDTPITUTLCOMREC == '0') {
            widget.fieldGroups.forEach(function (group) {
                if (group.name == 'contaPagar') {
                    group.isVisible = true;
                    widget.getField('CONTACORRENTE').validations = null;
                    widget.getField('CONTAPAGAR').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('NMTPBAIXAPAG').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('DSAJUCTPAG').validations = {'required': {'message': 'Campo obrigatório'}};
                }
                if (group.name == 'creditoContaCorrente') {
                    group.isVisible = false;
                }
            });
        } else {
            widget.fieldGroups.forEach(function (group) {
                if (group.name == 'contaPagar') {
                    group.isVisible = false;
                }
                if (group.name == 'creditoContaCorrente') {
                    group.isVisible = true;
                    widget.getField('CONTACORRENTE').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('CONTAPAGAR').validations = null;
                    widget.getField('NMTPBAIXAPAG').validations = null;
                    widget.getField('DSAJUCTPAG').validations = null;
                }else{
                    widget.getField('CONTACORRENTE').validations = null;
                    widget.getField('CONTAPAGAR').validations = widget.getField('CONTAPAGAR').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('NMTPBAIXAPAG').validations = widget.getField('NMTPBAIXAPAG').validations = {'required': {'message': 'Campo obrigatório'}};
                    widget.getField('DSAJUCTPAG').validations = widget.getField('DSAJUCTPAG').validations = {'required': {'message': 'Campo obrigatório'}};
                }
            });
        }
    };

    this.validaTipoCompensacao = function(widget){
        var titulosCompensados = templateManager.container.getWidget('TitulosCompensadosPagar');
        var contasCompensadas = templateManager.container.getWidget('ContasCorrentesCompensadas');
        if (widget.currentRow.IDTPITUTLCOMREC == '0'){
            titulosCompensados.isVisible = true;
            titulosCompensados.enabled = true;
            contasCompensadas.isVisible = false;
            this.setFiltroTitulosPagCompensados(widget, titulosCompensados).then(function(){
                titulosCompensados.reload();
            });
        }else{
            contasCompensadas.isVisible = true;
            contasCompensadas.enabled = true;
            titulosCompensados.isVisible = false;
            this.setFiltroContasCorrentesCompensadas(widget, contasCompensadas).then(function(){
                contasCompensadas.reload();
            });
        }
    };

    this.setFiltroTitulosPagCompensados = function (widget, pagarCompensados) {
        var deferred = ZHPromise.defer();
        pagarCompensados.dataSourceFilter.forEach(function (data) {
            switch(data.name) {
                case 'NRLANCCOMPREC': data.value = widget.currentRow.NRLANCCOMPREC;
                    break;
                case 'CDEMPRESA': data.value = widget.currentRow.CDEMPRESA;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.setFiltroContasCorrentesCompensadas = function (widget, contasCompensadas) {
        var deferred = ZHPromise.defer();
        contasCompensadas.dataSourceFilter.forEach(function (data) {
            switch(data.name) {
                case 'NRLANCCOMPREC': data.value = widget.currentRow.NRLANCCOMPREC;
                    break;
                case 'NRORG': data.value = widget.currentRow.NRORG;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.getEmpresa = function(){
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.CDEMPRESA;
    };

    this.getFilial = function(){
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.CDFILIAL;
    };

    this.getNrorg = function(){
        return templateManager.container.getWidget('CompensacaoContasReceber').currentRow.NRORG;
    };



    this.validaCamposAtivos = function (args){
        var field = args.owner.field.name;
        var row;
        switch(field) {
            case 'NMEMPRESA':
                row = args.owner.field.widget.currentRow;
                args.owner.field.widget.getField('NMRAZSOCCLIE').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('NMTPBAIXAPAG').readOnly = !row.NMEMPRESA;
                args.owner.field.widget.getField('DSAJUCTPAG').readOnly = !row.NMEMPRESA;
                break;
            case 'NMRAZSOCCLIE':
                row = args.owner.field.widget.currentRow;
                args.owner.field.widget.getField('CONTARECEBER').readOnly = !row.NMRAZSOCCLIE;
                break;
            case 'CDREDUCONT':
                row = args.owner.field.widget.currentRow;
                args.owner.field.widget.getField('CONTACORRENTE').readOnly = !row.CDREDUCONT;
                break;
            default:
                var aux = 'teste';
        }
    };

    this.validaFieldLancContaCorrente = function (args){
        var field = args.owner.field.name;

        switch(field) {
            case 'CDREDUCONT':
                var row = args.owner.field.widget.currentRow;
                args.owner.field.widget.getField('CONTACORRENTE').readOnly = !row.CDREDUCONT;
                break;
            default:
                var aux = 'teste';
        }
    };

    this.saveCompensacao = function (widget) {
        var route = 'fin_contaReceber#/saveCompensacaoCobranca';
        var dataSet = [];
        var valorCompensar = 0;

        if (widget.isValid())
        {
            dataSet = this.preparaRowCompensacaoReceber(widget, dataSet);
            var valorContaReceber = parseFloat(widget.currentRow.VRFINALBREC);
            //Verifica se compensa com lançamentos em conta ou contas a receber.
            if (widget.currentRow.IDTPITUTLCOMREC == '0') {
                valorCompensar = parseFloat(widget.currentRow.VRTOTALCOMREC);
                dataSet = this.preparaRowContasPagar(widget, dataSet);
            } else {
                valorCompensar = parseFloat(widget.currentRow.SOMACONTACORRENTE);
                dataSet = this.preparaRowLancamentoPagar(widget, dataSet);
            }
            if (valorCompensar != '0'){
                if (valorCompensar <= valorContaReceber){
                    General.callRestEngine(route, 'DataSet', dataSet)
                    .then(
                        function () {
                            widget.getField('CONTARECEBER').isVisible = false;
                            widget.getField('CONTAPAGAR').isVisible = false;
                            widget.getField('CONTACORRENTE').isVisible = false;
                            widget.getField('NMRAZSOCCLIE').readOnly = true;
                            widget.getField('VRTOTALCOMREC').class = '12';
                            widget.getField('SOMACONTACORRENTE').class = '12';
                            ScreenService.closeSwipe().then(function(){
                                widget.reload();
                                ScreenService.showMessage('Registro salvo com sucesso!');
                            });
                        }
                    )
                    .catch(
                        function(erro){
                            ScreenService.showMessage(erro.data.error);
                        }
                    );
                }else{
                    ScreenService.showMessage("Valor total a compensar deve ser menor ou igual ao valor líquido da conta informada.");
                }
            }else{
                ScreenService.showMessage("Valor total a compensar deve ser maior que zero.");
            }
        }
    };

    this.preparaRowCompensacaoReceber = function (widget, dataSetPagar) {
        dataSetPagar[0] = widget.currentRow;
        if (widget.currentRow.IDTPITUTLCOMREC == '0') {
            //Dados Conta Receber
            dataSetPagar[0].VRTOTALCOMREC = widget.currentRow.VRTOTALCOMREC;
            dataSetPagar[0].DSBAIXAPAG = widget.currentRow.NMTPBAIXAPAG;
        } else {
            //Dados Conta Receber
            dataSetPagar[0].VRTOTALCOMREC = widget.currentRow.SOMACONTACORRENTE;
        }
        return dataSetPagar;
    };

    this.preparaRowContasPagar = function (widget, dataSet) {
        var contaPagar = templateManager.container.getWidget('CompensacaoContasReceber').getField('CONTAPAGAR');
        contaPagar.dataSource.getCheckedRows().forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = data;
            dataSet[indice].CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
            dataSet[indice].CDEMPRESA = widget.currentRow.CDEMPRESA;
        });

        return dataSet;
    };

    this.preparaRowLancamentoPagar = function (widget, dataSet) {
        var contaCorrente = templateManager.container.getWidget('CompensacaoContasReceber').getField('CONTACORRENTE');
        contaCorrente.dataSource.getCheckedRows().forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = data;
            dataSet[indice].VRTOTALCOMREC = data.SOMACONTACORRENTE;
            dataSet[indice].CDAJUCTPAG = widget.currentRow.CDAJUCTPAG;
            dataSet[indice].CDTPBAIXAPAG = widget.currentRow.CDTPBAIXAPAG;
        });
        return dataSet;
    };


    this.filtraContasPagar = function (filtroContasPagar) {
        var compensacaoPagar = filtroContasPagar.getParent();
        var contaPagar = compensacaoPagar.getField('CONTAPAGAR');
        ScreenService.closePopup(false);
        wasFiltered = true;
        this.aplicaFiltroContasPagar(filtroContasPagar, contaPagar).then(function () {
            contaPagar.reload();
            $("#CONTAPAGAR").scope().openField(contaPagar);
        });
    };

    this.aplicaFiltroContasPagar = function (filtroContasPagar, contaPagar) {
        var deferred = ZHPromise.defer();
        var compensacaoPagar = filtroContasPagar.getParent();

        var fornecedorConcat = '';
        var cdfiliemit = '';
        var cdfilial = '';
        if (filtroContasPagar.currentRow.FORNECEDORFILTRO){
            if (filtroContasPagar.currentRow.FORNECEDORFILTRO.length > 0){
                filtroContasPagar.currentRow.FORNECEDORFILTRO.forEach(function (value) {
                    fornecedorConcat += "_" + value;
                });
                fornecedorConcat = (fornecedorConcat) ? fornecedorConcat + "_" : null;
            }else{
               fornecedorConcat = null;
           }
        }else{
            fornecedorConcat = null;
        }
        if (Array.isArray(filtroContasPagar.getField('CDFILIEMIT').value())) {
            filtroContasPagar.getField('CDFILIEMIT').value().forEach(function (data) {
                cdfiliemit += '_' + data;
            });
            cdfiliemit = (cdfiliemit) ? cdfiliemit + "_" : null;
        }
        if (Array.isArray(filtroContasPagar.getField('CDFILIALANC').value())) {
            filtroContasPagar.getField('CDFILIALANC').value().forEach(function (data) {
                cdfilial += '_' + data;
            });
            cdfilial = (cdfilial) ? cdfilial + "_" : null;
        }
        var valorIni = (filtroContasPagar.getField('VRFINALBPAG_INI').value() === undefined)? null : filtroContasPagar.getField('VRFINALBPAG_INI').value();
        var valorFin = (filtroContasPagar.getField('VRFINALBPAG_FIN').value() === undefined)? null : filtroContasPagar.getField('VRFINALBPAG_FIN').value();
        contaPagar.dataSourceFilter.forEach(function (filtro) {
            switch (filtro.name) {
                case 'CDEMPRESA':
                    filtro.value = (compensacaoPagar.currentRow.CDEMPRESA) ? compensacaoPagar.currentRow.CDEMPRESA : null;
                    break;
                case 'CDFORNECED':
                    filtro.value = fornecedorConcat;
                    break;
                case 'DOCINICIAL':
                    filtro.value = filtroContasPagar.getField('DOCINICIAL').value();
                    break;
                case 'DOCFINAL':
                    filtro.value = filtroContasPagar.getField('DOCFINAL').value();
                    break;
                case 'FILIORIG':
                    filtro.value = cdfiliemit;
                    break;
                case 'FILILANC':
                    filtro.value = cdfilial;
                    break;
                case 'DATAINI':
                    filtro.value = filtroContasPagar.getField('DTVENBOLPAG').value().start;
                    break;
                case 'DATAFIN':
                    filtro.value = filtroContasPagar.getField('DTVENBOLPAG').value().end;
                    break;
                case 'CDTPBAIXAPAG':
                    filtro.value = filtroContasPagar.currentRow.TIPOBAIXAPAGAR;
                    break;
                case 'VALORINI':
                    filtro.value = valorIni;
                    break;
                case 'VALORFIN':
                    filtro.value = valorFin;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    /*
     * INICIO - GERA BAIXA PLANEJADA
     */

    this.deleteCompensacao = function (widget) {
        var route = 'fin_contaReceber#/deleteCompensacaoCobranca';
        var dataSet = [];
        dataSet = this.preparaRowDeleteCompensacao(widget, dataSet);
        if (widget.currentRow.IDTPITUTLCOMREC == '0'){
            var widgetTitulos = templateManager.container.getWidget('TitulosCompensadosPagar');
            this.preparaRowDeleteTitulos(widgetTitulos, dataSet);
        }else{
            var widgetContas = templateManager.container.getWidget('ContasCorrentesCompensadas');
            this.preparaRowDeleteContas(widgetContas, dataSet);
        }
        ScreenService.confirmMessage(ScreenService.i18n('Deseja excluir Compensação?'), 'question', function () {
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function () {
                    ScreenService.closeSwipe().then(function(){
                        widget.reload();
                        ScreenService.showMessage('Registro excluído com sucesso!');
                    });
                })
                .catch(function (response) {
                    ScreenService.showMessage(response.data.error);
                });
        });
    };

    this.preparaRowDeleteCompensacao = function(widget, dataSet){
        var indice = dataSet.length;
        dataSet[indice] = widget.currentRow;
        return dataSet;
    };

    this.preparaRowDeleteTitulos = function(widget, dataSet){
        widget.dataSource.data.forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = data;
        });
        return dataSet;
    };

    this.preparaRowDeleteContas = function(widget, dataSet){
        widget.dataSource.data.forEach(function(data){
            var indice = dataSet.length;
            dataSet[indice] = {};
            dataSet[indice].CDREDUCONT = data.CDREDUCONT;
            dataSet[indice].NRMOVICRED = data.NRMOVICRED;
        });
        return dataSet;
    };

    this.ativaFiltroContaCorrente = function (args) {
        if (!wasFiltered) {
            var filtroContaCorrente = templateManager.container.getWidget('filtroContaCorrente');
            ScreenService.openPopup(filtroContaCorrente).then(function () {
                filtroContaCorrente.activate();
            });
            args.e.preventDefault();
        }
        else {
            wasFiltered = false;
        }
    };

    this.filtraContaCorrente = function (filtroContaCorrente) {
        var compensacaoReceber = filtroContaCorrente.getParent();
        var contaCorrente = compensacaoReceber.getField('CONTACORRENTE');
        ScreenService.closePopup(false);
        wasFiltered = true;
        this.aplicaFiltroContaCorrente(filtroContaCorrente, contaCorrente).then(function () {
            contaCorrente.reload();
            $("#CONTACORRENTE").scope().openField(contaCorrente);
        });
    };

    this.aplicaFiltroContaCorrente = function (filtroContaCorrente, contaCorrente) {
        var deferred = ZHPromise.defer();
        var compensacaoPagar = filtroContaCorrente.getParent();
        var dataInicial, dataFinal, valorInicial, valorFinal = null;
        var tipoCredito = '';
        if (filtroContaCorrente.currentRow.TIPOCREDITO){
            filtroContaCorrente.currentRow.TIPOCREDITO.forEach(function (value) {
                tipoCredito += "_" + value;
            });
            tipoCredito = (tipoCredito) ? tipoCredito + "_" : null;
        }
        var contas = '';
        if (!filtroContaCorrente.CONTAFILTRO) {
            if (filtroContaCorrente.currentRow.hasOwnProperty('CONTAFILTRO') &&
                filtroContaCorrente.currentRow.CONTAFILTRO.length > 0){
                filtroContaCorrente.currentRow.CONTAFILTRO.forEach(function (value) {
                    contas += "_" + value;
                });
                contas = (contas) ? contas + "_" : null;
            }else{
                contas = [];
            }
        }else{
            contas = [];
        }
        contaCorrente.dataSourceFilter.forEach(function (data) {
            switch (data.name) {
                case 'CDEMPRESA':
                    data.value = compensacaoPagar.currentRow.CDEMPRESA;
                    break;
                case 'CDREDUCONT':
                    data.value = (contas.length != 0) ? contas : null;
                    break;
                case 'TCDREDUCONT':
                    data.value = (contas.length != 0) ? null : 'T';
                    break;
                case 'DATAINICIAL':
                    dataInicial = (filtroContaCorrente.currentRow.DATALANCAMENTO.start) ? filtroContaCorrente.currentRow.DATALANCAMENTO.start : null;
                    data.value = dataInicial;
                    break;
                case 'DATAFINAL':
                    dataFinal = (filtroContaCorrente.currentRow.DATALANCAMENTO.end) ? filtroContaCorrente.currentRow.DATALANCAMENTO.end : null;
                    data.value = dataFinal;
                    break;
                case 'TDATA':
                    data.value = (dataInicial && dataFinal) ? null : 'T';
                    break;
                case 'VALORINICIAL':
                    valorInicial = (filtroContaCorrente.currentRow.VALORINICIAL) ? filtroContaCorrente.currentRow.VALORINICIAL : null;
                    data.value = valorInicial;
                    break;
                case 'VALORFINAL':
                    valorFinal = (filtroContaCorrente.currentRow.VALORFINAL) ? filtroContaCorrente.currentRow.VALORFINAL : null;
                    data.value = valorFinal;
                    break;
                case 'TVALOR':
                    if (valorInicial && valorFinal) {
                        if (valorInicial == 0 && valorFinal == 0) {
                            data.value = 'T';
                        } else {
                            data.value = null;
                        }
                    } else {
                        data.value = 'T';
                    }

                    break;
                case 'CDTPCREDBANC':
                    data.value = (tipoCredito) ? tipoCredito : null;
                    break;
                case 'TTIPO':
                    data.value = (tipoCredito) ? null : 'T';
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.setTodasContasCorrentes = function (value) {
        var filtroContaCorrente = templateManager.container.getWidget('filtroContaCorrente');
        filtroContaCorrente.CONTAFILTRO = value;
    };

    this.somaValorCompensarContaCorrente = function (contaCorrente) {
        var compensacao = templateManager.container.getWidget('CompensacaoContasReceber');
        var valorCompensar = 0;
        if (compensacao.isActive()){
            contaCorrente.value().forEach(function (data) {
                valorCompensar += parseFloat(data);
            });
        }else{
            contaCorrente.dataSource.getCheckedRows().forEach(function (data) {
                valorCompensar += parseFloat(data.VRMOVICRED);
            });
        }
        compensacao.currentRow.SOMACONTACORRENTE = valorCompensar;
        compensacao.activate();
    };

    this.somaValorCompensarContasReceber = function (contaPagar) {
        var compensacao = templateManager.container.getWidget('CompensacaoContasReceber');
        var valorCompensar = 0;
        if (compensacao.isActive()){
            contaPagar.value().forEach(function (data) {
                valorCompensar += parseFloat(data);
            });
        }else{
            contaPagar.dataSource.getCheckedRows().forEach(function (data) {
                valorCompensar += parseFloat(data.VRFINALBPAG);
            });
        }
        compensacao.currentRow.VRTOTALCOMREC = valorCompensar;
        compensacao.activate();
    };

    this.cancelarFiltro = function () {
        ScreenService.closePopup();
        templateManager.container.getWidget('CompensacaoContasReceber').activate();
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Rec18000_Compensacao_Conta_Receber', Rec18000_Compensacao_Conta_Receber);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec20000_PosicaoTitulosAReceber_Controller.js
var rec20000_PosicaoTitulosAReceber_Controller = function(templateManager, ScreenService, General, ZHPromise) {
	
	var self = this;
	
	var emAtraso      = [];
	var emAtrasoValor = 0;
	
	var emAberto      = [];
	var emAbertoValor = 0;
	
	var planejado      = [];
	var planejadoValor = 0;
	
	var baixado       = [];
	var baixadoValor  = 0;
	
	var total         = [];
	var totalValor    = 0;
	
	var widgetAtiva = "TotalNotification";
	
	self.initializeCards = function(){
        self.resetColorCards('TotalNotification');
	};
	
	self.refreshView = function(){
	    self.aplicaFiltro();
	};
	
	self.resetColorCards = function(cardAtual){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		
		var widgetAtrasoNotification = templateManager.container.getWidget('AtrasoNotification');
		var widgetAbertoNotification = templateManager.container.getWidget('AbertoNotification');
		var widgetPlanejadoNotification = templateManager.container.getWidget('PlanejadoNotification');
		var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
		var widgetTotalNotification = templateManager.container.getWidget('TotalNotification');
		
		switch(cardAtual) {
            /*case 'AtrasoNotification':
            		widgetAtrasoNotification.color = '#D34A40'; // muda
            		$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#D34A40');
            		widgetAbertoNotification.color = '#808080';
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
            	break;
            case 'AbertoNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#086A87');
            		widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
                break;
            case 'RecebidoNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#298A08'; // muda
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#298A08');
            		widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#808080');
                break;
             
            case 'TotalNotification':
                	widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('background-color', '#808080');
            		widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('background-color', '#808080');
            		widgetRecebidoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('background-color', '#808080');
            		widgetTotalNotification.color = '#0B2F3A'; // muda
            		$('#masonry-item-' + widgetTotalNotification.id).css('background-color', '#0B2F3A');
                break;*/
                
                case 'AtrasoNotification':
            		//widgetAtrasoNotification.color = '#D34A40'; // muda
            		$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '1.0');
            		//widgetAbertoNotification.color = '#808080';
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
            	break;
            	
            	case 'AbertoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '1.0');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
                
            case 'PlanejadoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#086A87'; // muda
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '1.0');
            		//widgetRecebidoNotification.color = '#808080';
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
                
            case 'RecebidoNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#298A08'; // muda
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '1.0');
            		//widgetTotalNotification.color = '#808080';
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '0.4');
                break;
             
            case 'TotalNotification':
                	//widgetAtrasoNotification.color = '#808080'; 
                	$('#masonry-item-' + widgetAtrasoNotification.id).css('opacity', '0.4');
            		//widgetAbertoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetAbertoNotification.id).css('opacity', '0.4');
            		$('#masonry-item-' + widgetPlanejadoNotification.id).css('opacity', '0.4');
            		//widgetRecebidoNotification.color = '#808080'; 
            		$('#masonry-item-' + widgetRecebidoNotification.id).css('opacity', '0.4');
            		//widgetTotalNotification.color = '#0B2F3A'; // muda
            		$('#masonry-item-' + widgetTotalNotification.id).css('opacity', '1.0');
                break;
                
            default:
                
        }
        
		//setTimeout(function(){templateManager.updateTemplate();}, 1);
		setTimeout(function(){window.dispatchEvent(new Event('resize'));}, 10);

	};

	self.aplicaFiltroStatusAtrasadoRec = function(){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = emAtraso;
		widgetPosicaoTitulosGrid.label = 'Em Atraso';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBREC')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBREC')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'A';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('AtrasoNotification');
	};
	
	self.aplicaFiltroStatusAbertoRec = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = emAberto;
		widgetPosicaoTitulosGrid.label = 'Em Aberto';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBREC')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBREC')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'A_O';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('AbertoNotification');
	};
	
	self.aplicaFiltroStatusPlanejadoRec = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = planejado;
		widgetPosicaoTitulosGrid.label = 'Planejado';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBREC')
                filtro.value = 'P';
            if(filtro.name == 'TIDSTATBREC')
                filtro.value = '';
            if(filtro.name == 'IDSTATUS')
                filtro.value = '';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = 'T';
        });
		
		self.resetColorCards('PlanejadoNotification');
	};
	
	self.aplicaFiltroStatusRecebidoRec = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = baixado;
		widgetPosicaoTitulosGrid.label = 'Baixado';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBREC')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBREC')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = 'B';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = '';
        });
		
		self.resetColorCards('RecebidoNotification');
	};
	
	self.aplicaFiltroStatusTotalRec = function(){
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		widgetPosicaoTitulosGrid.dataSource.data = total;
		widgetPosicaoTitulosGrid.label = 'Total do Período';
		
		widgetPosicaoTitulosGrid.dataSourceFilter.forEach(function (filtro) {
            if(filtro.name == 'IDSTATBREC')
                filtro.value = '';
            if(filtro.name == 'TIDSTATBREC')
                filtro.value = 'T';
            if(filtro.name == 'IDSTATUS')
                filtro.value = '';
            if(filtro.name == 'TIDSTATUS')
                filtro.value = 'T';
        });
		
		self.resetColorCards('TotalNotification');
	};
	
	self.aplicaFiltroGrid = function(){
		var deferred = ZHPromise.defer();
		
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetPosicaoTitulosGrid = templateManager.container.getWidget('PosicaoTitulosGrid');
		
		var cdcliente = widgetPopUpStripe.currentRow.CDCLIENTE;
	    var tcdcliente = 'T';
		
		 if(widgetPopUpStripe.currentRow.hasOwnProperty('CDCLIENTE')){
	    	if(Array.isArray(cdcliente)){
	    		if(cdcliente.length > 0){
		    		cdcliente = cdcliente.join("_");
		    		tcdcliente = '';
	    		}
		    }
	    }
		
		var dataSourceFilter = [
			{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.CDEMPRESA
            },
			{
                "name": "DTINICIAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.start
            },
			{
                "name": "DTFINAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.end
            },
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": cdcliente
            },
            {
                "name": "TCDCLIENTE",
                "operator": "=",
                "value": tcdcliente
            },
            {
                "name": "IDSTATBREC",
                "operator": "=",
                "value": ''
            },
            {
                "name": "TIDSTATBREC",
                "operator": "=",
                "value": 'T'
            },
            {
                "name": "IDSTATUS",
                "operator": "=",
                "value": ''
            },
            {
                "name": "TIDSTATUS",
                "operator": "=",
                "value": 'T'
            }
        ];
        
        widgetPosicaoTitulosGrid.dataSourceFilter = dataSourceFilter;
        widgetPosicaoTitulosGrid.reload().then(function(){
        	emAtraso = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS == 'A';
            });
            
            emAberto = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS != 'B';
            });
            
            planejado = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATBREC == 'P';
            });
            
            baixado = widgetPosicaoTitulosGrid.dataSource.data.filter(function(data){
            	return data.IDSTATUS == 'B';
            });
            
            total = widgetPosicaoTitulosGrid.dataSource.data;
        });
        
        var dataSourceFilterNotification = [
			{
                "name": "CDEMPRESA",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.CDEMPRESA
            },
			{
                "name": "DTINICIAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.start
            },
			{
                "name": "DTFINAL",
                "operator": "=",
                "value": widgetPopUpStripe.currentRow.DATA.end
            },
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": cdcliente
            },
            {
                "name": "TCDCLIENTE",
                "operator": "=",
                "value": tcdcliente
            }
       ];
        
		General.callRestEngine('fin#/crud/boletreceber_posicao_titulos', 'FilterData', dataSourceFilterNotification, {"itemsPerPage": 500}).then(
	        function(data){
	        	var valores = data.dataset.boletreceber_posicao_titulos;
	        	var widgetAtrasoNotification   = templateManager.container.getWidget('AtrasoNotification');
	        	var widgetAbertoNotification   = templateManager.container.getWidget('AbertoNotification');
	        	var widgetPlanejadoNotification   = templateManager.container.getWidget('PlanejadoNotification');
	        	var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
	        	var widgetTotalNotification    = templateManager.container.getWidget('TotalNotification');
	        	
	        	if(valores.length > 0){
	        		widgetAtrasoNotification.notification   = self.formataValorBRL(valores[0].ATRASO);
		            widgetAbertoNotification.notification   = self.formataValorBRL(valores[0].ABERTO);
		            widgetPlanejadoNotification.notification = self.formataValorBRL(valores[0].PLANEJADO);
		            widgetRecebidoNotification.notification = self.formataValorBRL(valores[0].RECEBIDO);
		            widgetTotalNotification.notification    = self.formataValorBRL(valores[0].TOTAL);
	        	}else{
	        		widgetAtrasoNotification.notification   = self.formataValorBRL(0);
		            widgetAbertoNotification.notification   = self.formataValorBRL(0);
		            widgetPlanejadoNotification.notification = self.formataValorBRL(0);
		            widgetRecebidoNotification.notification = self.formataValorBRL(0);
		            widgetTotalNotification.notification    = self.formataValorBRL(0);
	        	}
	        		
        		widgetPosicaoTitulosGrid.activate();
	        	deferred.resolve();
	        });
        
		return deferred.promise;
	   
	};
	
    self.verificaStatusTitulos = function (data) {
       var deferred = ZHPromise.defer();

        data.dataset.boletreceber_cliente_posicao_titulos.forEach(function(boleto){
    		total.push(boleto);
    		totalValor += parseFloat(boleto.VRFINALBREC); 
    		
    		// Em Atraso
            if(boleto.IDSTATUS == 'A'){
            	emAtraso.push(boleto);
            	emAtrasoValor += parseFloat(boleto.VRFINALBREC);
            }
            	
            // Baixado
            if(boleto.IDSTATUS == 'B'){
            	baixado.push(boleto);
            	baixadoValor += parseFloat(boleto.VRFINALBREC);
            }
            
            // Em Aberto	
            if(boleto.IDSTATUS != 'B'){
	            emAberto.push(boleto);
	            emAbertoValor += parseFloat(boleto.VRFINALBREC);
            }
            
            // Planejado	
            if(boleto.IDSTATBREC == 'P'){
	            planejado.push(boleto);
	            planejadoValor += parseFloat(boleto.VRFINALBREC);
            }
        });
        
        deferred.resolve();
        
        return deferred.promise;
    };
	
    self.atualizaValorNotificacao = function (widget) {
        if (widget.dataSource.data.length > 0)
             widget.notification = widget.dataSource.data.shift().NRCOUNT;
        else widget.notification = "0";
    };
	
	self.aplicaFiltro = function(){
	    var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
	    
	    if(widgetPopUpStripe.isValid()){
		    
		    self.aplicaStripe();
		    
		    self.aplicaValoresNotificacoes();
    
		    self.aplicaFiltroGrid().then(function(){
		    	self.aplicaFiltroStatusTotalRec();
		    });
		    
		    
	    	ScreenService.closePopup(widgetPopUpStripe);
	    }
	};
	
	self.aplicaStripe = function(){
		var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
		var widgetStripe      = templateManager.container.getWidget('stripe');
		
		var msgcdcliente = 'Todos os clientes filtrados.';
		
		if(widgetPopUpStripe.currentRow.hasOwnProperty('CDCLIENTE')){
			var cdcliente = widgetPopUpStripe.currentRow.CDCLIENTE;
			
	    	if(Array.isArray(cdcliente)){
	    		if(cdcliente.length > 0){
	    			if(cdcliente.length > 1){
		    			msgcdcliente = 'Lista de '+cdcliente.length+' clientes filtrados.';
	    			}else{
	    				msgcdcliente = widgetPopUpStripe.currentRow.NMRAZSOCCLIE;
	    			}
	    		}
		    }
	    }
	    	
        widgetStripe.currentRow.NMEMPRESA = widgetPopUpStripe.currentRow.CDEMPRESA + ' - ' + widgetPopUpStripe.currentRow.NMEMPRESA;
        widgetStripe.currentRow.DATA      = widgetPopUpStripe.currentRow.DATA.start + ' a ' + widgetPopUpStripe.currentRow.DATA.end;
        widgetStripe.currentRow.CDCLIENTE = msgcdcliente;

	    widgetStripe.reload();
	};
	
	self.aplicaValoresNotificacoes = function(){
	    /*var widgetPopUpStripe = templateManager.container.getWidget('popUpStripe');
	    
	    if(widgetPopUpStripe.currentRow.hasOwnProperty('CDCLIENTE')){
	    	if(Array.isArray(cdcliente)){
	    		if(cdcliente.length > 0){
		    		cdcliente = cdcliente.join("_");
		    		tcdcliente = '';
	    		}
		    }
	    }
	    
	    
	    var dataSourceFilter = [
	    	{
	    		"name"     : "CDEMPRESA",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.CDEMPRESA
	    	},
	    	{
	    		"name"     : "DTINICIAL",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.DATA.start
	    	},
	    	{
	    		"name"     : "DTFINAL",
	    		"operator" : "=",
	    		"value"    : widgetPopUpStripe.currentRow.DATA.end
	    	},
            {
                "name": "CDCLIENTE",
                "operator": "=",
                "value": cdcliente
            },
            {
                "name": "TCDCLIENTE",
                "operator": "=",
                "value": tcdcliente
            }
    	];
	    
	    General.callRestEngine('fin#/crud/boletreceber_posicao_titulos', 'FilterData', dataSourceFilter).then(
	        function(success){
	        	
	        	var widgetAtrasoNotification   = templateManager.container.getWidget('AtrasoNotification');
	        	var widgetAbertoNotification   = templateManager.container.getWidget('AbertoNotification');
	        	var widgetRecebidoNotification = templateManager.container.getWidget('RecebidoNotification');
	        	var widgetTotalNotification    = templateManager.container.getWidget('TotalNotification');
	        	
	        	if(success.dataset.boletreceber_posicao_titulos.length > 0){
		        	
		            widgetAtrasoNotification.notification   = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].ATRASO);
		            widgetAbertoNotification.notification   = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].ABERTO);
		            widgetRecebidoNotification.notification = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].RECEBIDO);
		            widgetTotalNotification.notification    = self.formataValorBRL(success.dataset.boletreceber_posicao_titulos[0].TOTAL);
	        	}else {
	        		widgetAtrasoNotification.notification   = self.formataValorBRL(0);
		            widgetAbertoNotification.notification   = self.formataValorBRL(0);
		            widgetRecebidoNotification.notification = self.formataValorBRL(0);
		            widgetTotalNotification.notification    = self.formataValorBRL(0);
	        	}
	        });*/
	};
	
	self.abreFiltro = function(){
	    var widget = templateManager.container.getWidget('popUpStripe');
	    ScreenService.openPopup(widget);
	    widget.activate();
	};
	
	self.formataValorBRL = function (valor){
		return parseFloat(valor).toLocaleString('pt-BR');
	};
	
	self.cancelar = function(){
	    var widget = templateManager.container.getWidget('PosicaoTitulosGrid');
	    ScreenService.closePopup(widget);
	    widget.activate();
	};
	
};

/* global Configuration */
Configuration(function(ContextRegister) {
	ContextRegister.register("Rec20000_PosicaoTitulosAReceber_Controller", rec20000_PosicaoTitulosAReceber_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/Rec43000_ParametroContasReceber_Controller.js
var Rec43000_ParametroContasReceber_Controller = function(ScreenService, General, templateManager, Rec43000_ParametroContasReceber_Service){
    
    var self = this;
    
    self.afterLoadSource = function(widget){
        var data = widget.dataSource.data[0];
        var dataEmpty = {
            'CDAJACREREC':'',
            'CDAJACRERENEGO':'',
            'CDAJDESCREC':'',
            'CDAJDESCRENEGO':'',
            'CDAJMORAREC':'',
            'CDAJMULTAREC':'',
            'CDAJTXCARTCNAB':'',
            'CDGERALREC':'',
            'CDGROPERCNTR':'',
            'CDMOTIPROR':'',
            'CDTPBXRENEGO':'',
            'DSAJACREREC':'',
            'DSAJACRERENEGO':'',
            'DSAJDESCREC':'',
            'DSAJDESCRENEGO':'',
            'DSAJMORAREC':'',
            'DSAJMULTAREC':'',
            'DSAJTXCARTCNAB':'',
            'DSMOTIPROR':'',
            'DSTPBXRENEGO':'',
            'IDBLOQALTBOLREC':'N',
            'IDBLOQINCNTR':'',
            'IDCOMPCHQBAIXA':'S',
            'IDEXTELACALAJUBX':'B',
            'IDTPCNTRLIB':'',
            'IDUTBCOPREFCLIE':'N',
            'IDUTCTRLCANREMR':'',
            'IDUTLNFDEVOLREC':'N',
            'IDUTLPRORVCREC':'',
            'NRORG':'',
            'QTDIAANTBAIR':'',
            'QTDIAANTBOLR':'',
            'QTDIAANTLANR':'',
            'QTDIAAPOBAIR':'',
            'QTDIAAPOBOLR':'',
            'QTDIAAPOLANR':'',
            'IDEXCLAJBBXR':'N',
            'IDUTENVBOLAUTO':'N',
            'QTDIAANTENVBOL':'',
            'IDUTENVCARTAUTO':'N',
            'QTDIAPOSENVCAR':'',
            'CDTPBAIRENVBOL':'',
            'IDUTVISUBOLOPEL':'N',
            'CDOPEPERVISUBOL':'',
            'IDUTLPROCREC': '',
            'CDGRUPOPERCONT': '',
            'CDGRUPOPEREXCLUSAOBAIXA': '',
            'CDGRUPOPERCANREMBOL': '',
            'DSGROPERPROR': '',
            'IDCONTROLARECCONTRATO': '',
            'CDGROPERPROR':''
            
        };
        var dados = null;
        if (data) {
            dados = data;
            dados.__is_new = false;
            
            data.CDTPBAIRENVBOL_multi = data.CDTPBAIRENVBOL?data.CDTPBAIRENVBOL.split('_'):'';
            widget.getField('CDTPBAIRENVBOL_multi').reload();
            
            data.CDOPEPERVISUBOL_multi = data.CDOPEPERVISUBOL?data.CDOPEPERVISUBOL.split('_'):'';
            widget.getField('CDOPEPERVISUBOL_multi').reload();
            widget.redraw();
        }else{
            dados = dataEmpty;
            dados.__is_new = true;
        }
        
        widget.currentRow = dados;
        
        var intervaloDatasForm = templateManager.container.getWidget('intervaloDatasForm');
        intervaloDatasForm.currentRow.QTDIAANTLANR = dados.QTDIAANTLANR;
        intervaloDatasForm.currentRow.QTDIAAPOLANR = dados.QTDIAAPOLANR;
        intervaloDatasForm.currentRow.QTDIAANTBAIR = dados.QTDIAANTBAIR;
        intervaloDatasForm.currentRow.QTDIAAPOBAIR = dados.QTDIAAPOBAIR;
        intervaloDatasForm.currentRow.QTDIAANTBOLR = dados.QTDIAANTBOLR;
        intervaloDatasForm.currentRow.QTDIAAPOBOLR = dados.QTDIAAPOBOLR;
        
        var grupoOperadoresForm = templateManager.container.getWidget('grupoOperadoresForm');
        grupoOperadoresForm.currentRow.IDBLOQINCNTR = dados.IDBLOQINCNTR;
        grupoOperadoresForm.currentRow.CDGROPERPROR = dados.CDGROPERPROR;
        grupoOperadoresForm.currentRow.DSGROPERPROR = dados.DSGROPERPROR;
        grupoOperadoresForm.currentRow.IDTPCNTRLIB = dados.IDTPCNTRLIB;
        grupoOperadoresForm.currentRow.IDUTLPROCREC = dados.IDUTLPROCREC;
        grupoOperadoresForm.currentRow.DSGROPERCNTR = dados.DSGROPERCNTR;
        grupoOperadoresForm.currentRow.CDGROPERCNTR = dados.CDGROPERCNTR;
        grupoOperadoresForm.currentRow.IDUTLPRORVCREC = dados.IDUTLPRORVCREC;
        grupoOperadoresForm.currentRow.IDCONTROLARECCONTRATO = dados.IDCONTROLARECCONTRATO;
        grupoOperadoresForm.currentRow.DSGROPERRECCONTR = dados.DSGROPERRECCONTR;
        grupoOperadoresForm.currentRow.CDGROPERRECCONTR = dados.CDGROPERRECCONTR;
        grupoOperadoresForm.currentRow.IDUTCTRLCANREMR = dados.IDUTCTRLCANREMR;
        
        if(dados.CDGRUPOPERCONT != null){
            grupoOperadoresForm.currentRow.CDGRUPOPERCONT = dados.CDGRUPOPERCONT.split(",");
            grupoOperadoresForm.getField('CDGRUPOPERCONT').reload();
        }
        
        if(dados.CDGRUPOPEREXCLUSAOBAIXA != null){
            grupoOperadoresForm.currentRow.CDGRUPOPEREXCLUSAOBAIXA = dados.CDGRUPOPEREXCLUSAOBAIXA.split(",");
            grupoOperadoresForm.getField('CDGRUPOPEREXCLUSAOBAIXA').reload();
        }
        
        if(dados.CDGRUPOPERCANREMBOL != null){
            grupoOperadoresForm.currentRow.CDGRUPOPERCANREMBOL = dados.CDGRUPOPERCANREMBOL.split(",");
            grupoOperadoresForm.getField('CDGRUPOPERCANREMBOL').reload();
        }
        
        templateManager.updateTemplate();
    };

    self.saveTabeds = function(){
        var widgetsTabed = templateManager.container.getWidget('tabbedParametrosContasReceber').widgets;
        var rowsToSave   = [];
        
        widgetsTabed[0].currentRow.CDTPBAIRENVBOL  = widgetsTabed[0].currentRow.CDTPBAIRENVBOL_multi?widgetsTabed[0].currentRow.CDTPBAIRENVBOL_multi.join('_'):'';
        widgetsTabed[0].currentRow.CDOPEPERVISUBOL = widgetsTabed[0].currentRow.CDOPEPERVISUBOL_multi?widgetsTabed[0].currentRow.CDOPEPERVISUBOL_multi.join('_'):'';

        for (var i = 0; i < widgetsTabed.length; i++) {
            var widget = widgetsTabed[i];
            rowsToSave.push(widget.currentRow);
        }
        
        Rec43000_ParametroContasReceber_Service.saveWidgets(rowsToSave, rowsToSave[0].__is_new).then(function () {
            widgetsTabed[0].view();
            widgetsTabed[0].template = widgetsTabed[0].templateToView;
            widgetsTabed[1].view();
            widgetsTabed[1].template = widgetsTabed[1].templateToView;
            widgetsTabed[2].view();
            widgetsTabed[2].template = widgetsTabed[2].templateToView;
            widgetsTabed[0].reload();
            widgetsTabed[1].reload();
            widgetsTabed[2].reload();
            widgetsTabed[0].currentRow.__is_new = false;
            self.afterLoadSource(widgetsTabed[0]);
        }, function (error) {
            ScreenService.showMessage(error.data.error);
        });
    };

    self.editMode = function(){
        var widgetsTabed = templateManager.container.getWidget('tabbedParametrosContasReceber').widgets;
        for (var i = 0; i < widgetsTabed.length; i++) {
            var widget = widgetsTabed[i];
            General.edit(widget);
            widget.template = widget.templateToForm;
        }
    };

    self.backToView = function(){
        var widgetsTabed = templateManager.container.getWidget('tabbedParametrosContasReceber').widgets;
        for (var i = 0; i < widgetsTabed.length; i++) {
            var widget = widgetsTabed[i];
            widget.view();
            widget.currentRow = widget.backupRow;
            widget.template = widget.templateToView;
        }
    };
    
    this.HabilitaCampos = function(widget) { 
        
        var grupoOperadoresForm = templateManager.container.getWidget('grupoOperadoresForm');
        
        var idbloqincntr =  grupoOperadoresForm.getField("IDBLOQINCNTR");
        var dsgropercntr = grupoOperadoresForm.getField("DSGROPERCNTR");
        var cdgropercntr = grupoOperadoresForm.getField("CDGROPERCNTR");
        var idtpcntrlib = grupoOperadoresForm.getField("IDTPCNTRLIB");
        
        var idutlprocrec = grupoOperadoresForm.getField("IDUTLPROCREC");
        var cdgroperpror = grupoOperadoresForm.getField("CDGROPERPROR");
        var dsgroperpror = grupoOperadoresForm.getField("DSGROPERPROR");
        
        
        var cdgrupopercont = grupoOperadoresForm.getField("CDGRUPOPERCONT");
        var idutlprorvcrec = grupoOperadoresForm.getField("IDUTLPRORVCREC");
        
        var idcontrolareccontrato  = grupoOperadoresForm.getField("IDCONTROLARECCONTRATO");
        var dsgroperreccontr = grupoOperadoresForm.getField("DSGROPERRECCONTR");
        var cdgroperreccontr = grupoOperadoresForm.getField("CDGROPERRECCONTR");
        
        var idutctrlcanremr  = grupoOperadoresForm.getField("IDUTCTRLCANREMR");
        var cdgrupopercanrembol = grupoOperadoresForm.getField("CDGRUPOPERCANREMBOL");
        
        
        if(idbloqincntr.value() == "S"){
            dsgropercntr.readOnly = false;
            idtpcntrlib.readOnly = false;
            dsgropercntr.validations.required =  true;
        } else{
            dsgropercntr.readOnly = true;
            idtpcntrlib.readOnly = true;
            dsgropercntr.setValue("");
            cdgropercntr.setValue("");
            idtpcntrlib.setValue("N");
            delete(dsgropercntr.validations.required);
        }
        
        if(idutlprocrec.value() == "S"){
          dsgroperpror.readOnly = false;
          dsgroperpror.validations.required =  true;
        }else{
          dsgroperpror.readOnly = true;
          dsgroperpror.setValue("");
          cdgroperpror.setValue("");
          delete(dsgroperpror.validations.required);
        }
        
        if(idutlprorvcrec.value() == "S"){
          cdgrupopercont.readOnly = false;
          cdgrupopercont.validations.required =  true;
        }else{
          cdgrupopercont.readOnly = true;
          cdgrupopercont.setValue("");
          delete(cdgrupopercont.validations.required);
        }
        
        if(idcontrolareccontrato.value() == "S"){
          dsgroperreccontr.readOnly = false;
          dsgroperreccontr.validations.required =  true;
        }else{
          dsgroperreccontr.readOnly = true;
          cdgroperreccontr.setValue("");
          dsgroperreccontr.setValue("");
          delete(dsgroperreccontr.validations.required);
        }
        
        if(idutctrlcanremr.value() == "S"){
          cdgrupopercanrembol.readOnly = false;
          cdgrupopercanrembol.validations.required =  true;
        }else{
          cdgrupopercanrembol.readOnly = true;
          cdgrupopercanrembol.setValue("");
          delete(cdgrupopercanrembol.validations.required);
        }
        grupoOperadoresForm.reload();
    };
    
    this.saveTabedsOperadores = function(widget){
        var dataSet = { row:widget.currentRow};
        
        if(widget.isValid()){
            Rec43000_ParametroContasReceber_Service.saveTabedsOperadores(dataSet).then(
                function(sucess) {
                ScreenService.showMessage('Grupo de Operadores salvo com sucesso!');
                widget.view();
                widget.reload();
            },
            function(callback){
                if(callback.data){
                    ScreenService.showMessage("Ocorreu um erro inesperado ao salvar o grupo de operadores");
                }
            }
            );
        }
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Rec43000_ParametroContasReceber_Controller', Rec43000_ParametroContasReceber_Controller);
});

// FILE: ./assets/modules/contaReceber/assets/js/controllers/rec41200_tipoAjusteReceberController.js
/**
 * Created by leonardooliveira on 07/03/2016.
 */
var TipoAjusteReceberController = function(General, ScreenService, templateManager, TipoAjusteReceberService){

    var enableAll = [
        "CDAJUCTREC", "DSAJUCTREC", "IDAJUCTREC", "CDCONTCTBL", "DSHISTPADR", "IDTPAJRECEBER", "IDCTBAJCTREC"
    ];

    /*this.editarTipoDeAjuste = function(args){
        args.owner.widget.edit();
        args.owner.widget.getField('IDNOVO').value('N');
        enableDisable(args.owner.widget, enableAll, false);

        enableDisable(args.owner.widget, ["CDAJUCTREC", "IDAJUCTREC", "IDTPAJRECEBER", "IDCTBAJCTREC"], true);

    };*/

    this.novoTipoDeAjuste = function(args){
        //args.owner.widget.currentRow = {};
        widget = args.owner.widget;
        //widget.template = "widget/form.html";
        widget.edit();
        widget.newRow();
        widget.getField('NRORG').value(getLocalVar('NRORGTRAB'));
        widget.getField('IDAJUCTREC').value('A');
        widget.getField('IDTPAJRECEBER').value('N');
        widget.getField('IDCTBAJCTREC').value('S');
        widget.getField('IDNOVO').value('S');
        //widget.edit();
        //ScreenService.openSwipe(args.owner.widget);
        enableDisable(args.owner.widget, enableAll, false);
    };

    this.salvarTipoAjuste = function(args){
        /*var tipoAjusteReceberGrid = args.owner.widget;
        if(!tipoAjusteReceberGrid.isValid()){
            ScreenService.alertNotification('Preencha todos os campos obrigatórios.', 5000);
        }else {
            TipoAjusteReceberService.salvarTipoAjuste(tipoAjusteReceberGrid.currentRow).then(function () {
                ScreenService.successNotification('Registro salvo com sucesso', 5000);
                tipoAjusteReceberGrid.reload();
                tipoAjusteReceberGrid.view();
                ScreenService.goBack();
            }, function (error) {
                ScreenService.notificationMessage(error.data['error'], 'error', 5000);
            });
        }*/
        
        widget = args.owner.widget;
        if(widget.isValid()){
            General.saveCrud(widget).then(
                function success(success) {
                    /*widget.reload().then(function(){
                        if(widget.isNewRow()){
                            widget.reload().then(function(){
                                ScreenService.goBack();
                                //ScreenService.showMessage(ScreenService.i18n('Salvo com sucesso.'));
                            });
                        }
                    });*/
                    //ScreenService.showMessage(ScreenService.i18n('Salvo com sucesso.'));
                }, function error(error){
                    console.log(error.data.error.indexOf('ORA-01400'));
                    if (error.data && error.data.error.indexOf('ORA-00001') == -1) {
                        ScreenService.showMessage(ScreenService.i18n("Informação já cadastrada."));
                    } else if (error.data && error.data.error.indexOf('ORA-01400') !== -1){
                        ScreenService.showMessage(ScreenService.i18n("Existem campos em branco que precisam ser preenchidos."));
                    }else if (error.data.error) {
                        ScreenService.showMessage(ScreenService.i18n(error.data.error));
                    } else {
                        ScreenService.showMessage(ScreenService.i18n("Verifique sua conexão e tente novamente"));
                    }
                }
            );
        }
        //ScreenService.goBack();
    };

    this.deletarTipoAjuste = function(tipoAjusteReceberGrid){
        //ScreenService.confirmMessage('Deseja excluir o tipo de ajuste?').then(function(){
            General.deleteCrud(tipoAjusteReceberGrid).then(function(){
                //ScreenService.successNotification('Registro excluído com sucesso', 5000);
                tipoAjusteReceberGrid.reload();
                tipoAjusteReceberGrid.view();
                ScreenService.goBack();
            }, function(error){
                ScreenService.notificationMessage(error.data.error, 'error', 5000);
            });
            /*TipoAjusteReceberService.deletarTipoAjuste(tipoAjusteReceberGrid.currentRow).then(function(){
                ScreenService.successNotification('Registro excluído com sucesso', 5000);
                tipoAjusteReceberGrid.reload();
                tipoAjusteReceberGrid.view();
                ScreenService.goBack();
            }, function(error){
                ScreenService.notificationMessage(error.data['error'], 'error', 5000);
            });*/
        //});        
    };

    var enableDisable = function(widget, fields, enableDisable){
        var i = 0;
        if(enableDisable) {
            for (i = 0; i < fields.length; i++) {
                widget.getField(fields[i]).readOnly = true;
            }
        }else{
            for (i = 0; i < fields.length; i++) {
                widget.getField(fields[i]).readOnly = false;
            }
        }
    };
    
    var self = this;
    
    this.configContctblField = function() {
      
      self.setContctblMask();
      self.setContctblFilter();
      
    };
    
    this.setContctblMask = function() {
        
        TipoAjusteReceberService.getContctblMask().then(function(response){
            
            var paramgeral = response.dataset.paramgeral[0];
            var contctblMask = paramgeral.CDPICTCCTB;
            
            //fields[0] = CDCONTCTBL
            templateManager.container.getWidget('tipoAjusteReceberGrid').getField('CDCONTCTBL').fields[0].mask.params.mask = contctblMask;
            templateManager.container.getWidget('tipoAjusteReceberGrid').getField('CDCONTCTBL').mask.params.mask = contctblMask;
            
        });
        
    };
    
    this.setContctblFilter = function() {
      
      TipoAjusteReceberService.getNvMaxContctbl().then(function(response) {
        var NVMAXCONTCTBL = response.dataset.paramgeral.NVMAXCONTCTBL;
        
        //dataSourceFilter[0]: {name: "CDNVCONTCTBL", operator: "="}
        templateManager.container.getWidget('tipoAjusteReceberGrid').getField('CDCONTCTBL').dataSourceFilter[0].value = NVMAXCONTCTBL;
        
      });
      
      
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('TipoAjusteReceberController', TipoAjusteReceberController);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco00500_OrcamentoERecalculoFluxoCaixa_Controller.js
var Bco00500_OrcamentoERecalculoFluxoCaixa_Controller = function(ScreenService, ZHPromise, templateManager, General, ZhFilterPreferences) {

    this.recalcularOrcamentoFluxoCaixa = function(popup){
        var classfinanc = templateManager.container.getWidget('classificacao_financeira');
        var row = null;
        if (popup.getField('NMFILIAL').isVisible){
            row = {
                'CDEMPRESA': popup.currentRow.CDEMPRESA,
                'CDFILIAL': popup.currentRow.CDFILIAL,
                'DTORCAFINA_INI': popup.currentRow.DTORCAFINA.start,
                'DTORCAFINA_FIN': popup.currentRow.DTORCAFINA.end
            };
        }else{
            row = {
                'CDEMPRESA': popup.currentRow.CDEMPRESA,
                'DTORCAFINA_INI': popup.currentRow.DTORCAFINA.start,
                'DTORCAFINA_FIN': popup.currentRow.DTORCAFINA.end
            };
        }
        if (popup.isValid()){
            var classificacoesMarcadas = classfinanc.getCheckedRows();
            var dataSet = [];
            dataSet.push(row);
            dataSet.push(classificacoesMarcadas);
            var route = 'fin_movimentacaoBancaria#/recalcularOrcamentoFluxoCaixa';
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function(response){
                    ScreenService.closePopup(true);
                    classfinanc.activate();
                    ScreenService.showMessage('Recálculo realizado com sucesso.');
                    classfinanc.unSelectAll();
                }.bind(this))
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        }
    };

    this.salvarOrcamento = function(orcamento){
        if (orcamento.getParent().isValid()){
            var rowsAlteradas = orcamento.dataSource.data.filter(function(filtro){
                return filtro.changed;
            });
            var dataSet = [];
            var row = null;
             if (orcamento.getParent().getField('NMFILIAL').isVisible){
                row = {
                    'CDEMPRESA': orcamento.getParent().currentRow.CDEMPRESA,
                    'CDFILIAL': orcamento.getParent().currentRow.CDFILIAL,
                    'CDCLASSFINA': orcamento.getParent().getParent().currentRow.CDCLASSFINA,
                    'DTMES': orcamento.getParent().currentRow.DTMES
                };
            }else{
                row = {
                    'CDEMPRESA': orcamento.getParent().currentRow.CDEMPRESA,
                    'CDCLASSFINA': orcamento.getParent().getParent().currentRow.CDCLASSFINA,
                    'DTMES': orcamento.getParent().currentRow.DTMES
                };
            }
            dataSet.push(row);
            dataSet.push(rowsAlteradas);
            var route = 'fin_movimentacaoBancaria#/saveOrcamentoFluxoCaixa';
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function(response){
                    orcamento.reload().then(function(){
                        orcamento.list();
                        ScreenService.showMessage('Orçamento realizado com sucesso.');
                    });

                }.bind(this))
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        }
    };

    this.cadastramentoAutomaticoOrcamentoFluxoCaixa = function(popup){
        var orcamento = templateManager.container.getWidget('orcamento_fluxo_caixa');
        var rowOrigem = null;
        var rowDestino = null;
        if (popup.getField('NMFILIALORIGEM').isVisible){
            rowOrigem = {
                'CDEMPRESA': popup.currentRow.CDEMPRESAORIGEM,
                'CDFILIAL': popup.currentRow.CDFILIALORIGEM
            };
            rowDestino = {
                'CDEMPRESA': popup.currentRow.CDEMPRESADESTINO,
                'CDFILIAL': popup.currentRow.CDFILIALDESTINO
            };
        }else{
            rowOrigem = {
                'CDEMPRESA': popup.currentRow.CDEMPRESAORIGEM
            };
            rowDestino = {
                'CDEMPRESA': popup.currentRow.CDEMPRESADESTINO
            };
        }
        var dataSet = [];
        var rowOrcamento = {
            'OPCAOOPERACAO': popup.currentRow.OPCAOOPERACAO,
            'DTMES': popup.currentRow.DTMES
        };

        dataSet.push(rowOrcamento);
        dataSet.push(rowOrigem);
        dataSet.push(rowDestino);

        if (popup.isValid()){
            var route = 'fin_movimentacaoBancaria#/cadastramentoAutomatico';
            General.callRestEngine(route, 'DataSet', dataSet)
                .then(function(response){
                    ScreenService.closePopup(true);
                    orcamento.getParent().getParent().activate();
                    ScreenService.showMessage('Cadastramento Automático realizado com sucesso.');
                }.bind(this))
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
        }
    };

    this.setParamGeral = function(){
        var route = 'fin#/crud/paramgeral';
        General.callRestEngine(route, 'FilterData', [])
            .then(function(response){
                this.validaVisibilidadeFilialEmpresa(response.dataset.paramgeral[0]);
            }.bind(this))
            .catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
    };

    this.validaVisibilidadeFilialEmpresa = function(paramGeral){
        var recalculo = templateManager.container.getWidget('recalculo');
        var orcamento = templateManager.container.getWidget('tabbed_orcamento_fluxo_caixa');
        var cadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomatico');
        
        if (paramGeral.IDTPORCAFINA == 'E'){
            this.setFieldsParametricacaoEmpresa(recalculo, orcamento, cadastramentoAutomatico);
        }else{
            recalculo.getField('NMFILIAL').isVisible = true;
            recalculo.getField('DTORCAFINA').class = '12';
            orcamento.getField('NMFILIAL').isVisible = true;
            orcamento.getField('NMEMPRESA').class = '5';
            orcamento.getField('DTMES').class = '2';
            cadastramentoAutomatico.getField('NMEMPRESAORIGEM').class = '6';
            cadastramentoAutomatico.getField('NMEMPRESADESTINO').class = '6';
            cadastramentoAutomatico.getField('NMFILIALORIGEM').isVisible = true;
            cadastramentoAutomatico.getField('CDFILIALDESTINO').isVisible = true;
        }
    };

    this.setFieldsParametricacaoEmpresa = function(recalculo, orcamento, cadastramentoAutomatico){
        recalculo.getField('NMFILIAL').isVisible = false;
        recalculo.getField('DTORCAFINA').class   = '6';
        orcamento.getField('NMFILIAL').isVisible = false;
        orcamento.getField('NMEMPRESA').class    = '8';
        orcamento.getField('DTMES').class        = '4';
        cadastramentoAutomatico.getField('NMEMPRESAORIGEM').class     = '12';
        cadastramentoAutomatico.getField('NMEMPRESADESTINO').class    = '12';
        cadastramentoAutomatico.getField('NMFILIALORIGEM').isVisible  = false;
        cadastramentoAutomatico.getField('CDFILIALDESTINO').isVisible = false;
    };

    this.openPopupRecalculoOrcamento = function(){
        var recalculo = templateManager.container.getWidget('recalculo');
        ScreenService.openPopup(recalculo).then(function(){
            recalculo.isVisible = true;
            if (!recalculo.getField('NMEMPRESA').value()){
                this.buscaPadraoEmpresa(recalculo);
            }
        }.bind(this));
    };

    this.cancelarRecalculo = function(recalculo){
        ScreenService.closePopup();
        recalculo.isVisible = false;
    };

    this.openPopupCadastramentoAutomatico = function(){
        var cadastramentoAutomatico = templateManager.container.getWidget('cadastramentoAutomatico');
        ScreenService.openPopup(cadastramentoAutomatico).then(function(){
            cadastramentoAutomatico.isVisible = true;
            cadastramentoAutomatico.getField('OPCAOOPERACAO').setValue("I");
            if (!cadastramentoAutomatico.getField('NMEMPRESAORIGEM').value()){
                this.buscaPadraoEmpresa(cadastramentoAutomatico);
            }
        }.bind(this));
    };

    this.cancelarCadastramentoAutomatica = function(cadastramentoAutomatico){
        ScreenService.closePopup();
        cadastramentoAutomatico.isVisible = false;
    };

    this.openSwipeOrcamentoFluxoCaixa = function(classificacaoFinanceira){
        var orcamentoFinanceiro = templateManager.container.getWidget('orcamento_fluxo_caixa');
        ScreenService.openSwipe(orcamentoFinanceiro.getParent()).then(function(){
            if (!orcamentoFinanceiro.getParent().getField('CDEMPRESA').value()){
                this.buscaPadraoEmpresa(orcamentoFinanceiro.getParent()).then(function(){
                    this.redefineGridOrcamento(orcamentoFinanceiro);
                }.bind(this));
            }else{
                if (orcamentoFinanceiro.getParent().getField('CDFILIAL').value() &&
                    orcamentoFinanceiro.getParent().getField('DTMES').value()){

                    this.redefineGridOrcamento(orcamentoFinanceiro);
                }else{
                    orcamentoFinanceiro.dataSource.data = [];
                }
            }
        }.bind(this));
    };

    this.redefineGridOrcamento = function(orcamentoFinanceiro){
        if (orcamentoFinanceiro.getParent().getField('DTMES').value() &&
            orcamentoFinanceiro.getParent().getField('CDFILIAL').value() &&
            orcamentoFinanceiro.getParent().getField('CDEMPRESA').value()){
            this.setFiltroOrcamentoFluxoCaixa(orcamentoFinanceiro).then(function(){
                this.carregaGridOrcamento(orcamentoFinanceiro);
            }.bind(this));
        }else{
            orcamentoFinanceiro.dataSource.data = [];
        }
    };

    this.carregaGridOrcamento = function(orcamentoFinanceiro){
        if (orcamentoFinanceiro.getParent().isValid()){
            orcamentoFinanceiro.reload();
        }
    };

    this.voltar = function(){
        var classificacao_financeira = templateManager.container.getWidget('classificacao_financeira');
        ScreenService.goBack().then(function(){
            classificacao_financeira.activate();
        });
    };

    this.setFiltroFilial = function(widget, empresa, filial){
        var valorEmpresa = widget.getField(empresa).value();
        
        if (valorEmpresa){
            widget.getField(filial).readOnly = false;
            widget.getField(filial).setValue('');
            widget.getField(filial).dataSourceFilter[0].value = valorEmpresa;
        }else{
            widget.getField(filial).readOnly = true;
            widget.getField(filial).setValue('');
            widget.getField(filial).dataSourceFilter[0].value = '';
        }
        
        if (widget.name == 'tabbed_orcamento_fluxo_caixa')
            templateManager.container.getWidget('orcamento_fluxo_caixa').dataSource.data = [];
        
        widget.getField(filial).reload();
    };

    this.setFiltroOrcamentoFluxoCaixa = function(orcamentoFinanceiro){
        var deferred = ZHPromise.defer();
        var dtmes = orcamentoFinanceiro.getParent().currentRow.DTMES;
        orcamentoFinanceiro.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA': filter.value = orcamentoFinanceiro.getParent().currentRow.CDEMPRESA;
                    break;
                case 'CDFILIAL': filter.value = orcamentoFinanceiro.getParent().currentRow.CDFILIAL;
                    break;
                case 'CDCLASSFINA': filter.value = orcamentoFinanceiro.getParent().getParent().currentRow.CDCLASSFINA;
                    break;
                case 'ANOINI': filter.value = '01/01/' + dtmes;
                    break;
                case 'ANOFIM': filter.value = '31/12/' + dtmes;
                    break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };

    this.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                var cdempresa = empresa.CDEMPRESA;
                var nmempresa = empresa.NMEMPRESA;
                if (widget.name == 'cadastramentoAutomatico'){
                    widget.getField('CDFILIALORIGEM').setValue(filial.CDFILIAL);
                    widget.getField('NMFILIALORIGEM').setValue(filial.NMFILIAL);
                    widget.getField('NMFILIALORIGEM').dataSourceFilter[0].value = cdempresa;
                    widget.getField('CDFILIALORIGEM').readOnly = false;
                    widget.getField('NMFILIALORIGEM').readOnly = false;
                    widget.getField('CDEMPRESAORIGEM').setValue(cdempresa);
                    widget.getField('NMEMPRESAORIGEM').setValue(nmempresa);
                }else{
                    widget.getField('CDFILIAL').setValue(filial.CDFILIAL);
                    widget.getField('NMFILIAL').setValue(filial.NMFILIAL);
                    widget.getField('NMFILIAL').dataSourceFilter[0].value = cdempresa;
                    widget.getField('CDFILIAL').readOnly = false;
                    widget.getField('NMFILIAL').readOnly = false;
                    widget.getField('CDEMPRESA').setValue(cdempresa);
                    widget.getField('NMEMPRESA').setValue(nmempresa);
                }
                if (widget.name == 'recalculo'){
                    if (!widget.getField('DTORCAFINA').value()){
                        var start = moment().format('DD/MM/YYYY');
                        var end   = moment().format('DD/MM/YYYY');
                        widget.getField('DTORCAFINA').setValue({start : start, end : end});
                    }
                }else{
                    if (!widget.getField('DTMES').value()){
                        widget.getField('DTMES').setValue(parseInt(moment().format('YYYY')));
                    }
                }
                deferred.resolve();
            }.bind(this));
        }.bind(this));
        return deferred.promise;
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco00500_OrcamentoERecalculoFluxoCaixa_Controller', Bco00500_OrcamentoERecalculoFluxoCaixa_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco01400_RecalculoLancamentoContaCorrente_Controller.js
var Bco01400_RecalculoLancamentoContaCorrente_Controller = function(Bco01400_RecalculoLancamentoContaCorrente_Service, ScreenService, contabilizacaoController) {
  
  var LANC_INVALIDO_CODE = 1;
  var LANC_FORA_DO_PERIODO_CODE = 2;
  
  this.formatFilter = function(widget) {
    
    var currentFilter = widget.appliedFilter.FILTER_ROW;
    var params = {};
    
    params = Bco01400_RecalculoLancamentoContaCorrente_Service.buildParamsAccordingToFilter(currentFilter);
    
    widget.dataSourceFilter = Bco01400_RecalculoLancamentoContaCorrente_Service.buildDataSourceFromParams(params);
  };
  
  this.validateCheckedRow = function(widget) {
    
    var checkedRow = widget.selectedRow;
    
    Bco01400_RecalculoLancamentoContaCorrente_Service.validateCheckedRow(checkedRow).then().catch(function(error) {
      
      if (error.data.messages[0].variables.CODE == LANC_FORA_DO_PERIODO_CODE) {
        ScreenService.showMessage("Lançamento fora do período contábil: __DTINICCTBL__ a __DTFINACCTBL__", null, null, error.data.messages[0].variables);
      } else {
        ScreenService.showMessage("Período contábil inexistente");
      }
      
      unselectRow(widget, checkedRow);
      
    });
    
  };
  
  this.validateCheckedRows = function(widget) {
    
    var checkedRows = widget.dataSource.checkedRows;
    
    Bco01400_RecalculoLancamentoContaCorrente_Service.validateCheckedRows(checkedRows).then(function(response) {
      
      unselectGivenRows(widget, response.dataset.rows_to_be_unchecked);
      
    });
    
    
  };
  
  this.recontabilizarLancamentos = function(widget) {
    
    var checkedRows = widget.dataSource.checkedRows;
    
    Bco01400_RecalculoLancamentoContaCorrente_Service.recontabilizarLancamentos(checkedRows).then(function(success){
      ScreenService.showMessage("Recontabilizacao feita com sucesso.");   
    }).catch(function(error) {
      
      if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os lançamentos devido a erros na contablização.")) {
        var message = "Timeout";
        if (error.data.error) {
          message = error.data.error;
        }  
        ScreenService.showMessage(message);
      }
      
    });
    
  };
  
  this.enableTipos = function(widget) {
    
    if (widget.currentRow.P_DEBCRED === 'T') {
      widget.getField('P_TIPOCRED').isVisible = true;
      widget.getField('P_TIPODEB').isVisible = true;
    }
    if (widget.currentRow.P_DEBCRED === 'D') {
      widget.getField('P_TIPOCRED').isVisible = false;
      widget.currentRow.P_TIPOCRED = [];
      widget.getField('P_TIPODEB').isVisible = true;
    }
    if (widget.currentRow.P_DEBCRED === 'C') {
      widget.getField('P_TIPOCRED').isVisible = true;
      widget.getField('P_TIPODEB').isVisible = false;
      widget.P_TIPODEB = [];
    }
    
  };
  
  var unselectRow = function (widget, row) {
    var index = widget.getCheckedRows().indexOf(row);
    widget.dataSource.checkedRows.splice(index, 1);
    row.__isSelected = false;
  };
  
  var unselectGivenRows = function (widget, rows) {
    
    rows.forEach(function(row) {
      
      var removedRow;
      
      widget.dataSource.checkedRows.forEach(function(checkedRow, index, checkedRows) {
        if (checkedRow.NUMERO === row.NUMERO && checkedRow.DEBCRED === row.DEBCRED)
          removedRow = checkedRows.splice(index, 1)[0];
      });
      
      removedRow.__isSelected = false;
      
    });
    
    
  };
  
  
  
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco01400_RecalculoLancamentoContaCorrente_Controller', Bco01400_RecalculoLancamentoContaCorrente_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco01500_RecontabilizacaoDeTransferencia_Controller.js
var Bco01500_RecontabilizacaoDeTransferencia_Controller = function(templateManager, ValidationEngine, ScreenService, contabilizacaoController, Bco01500_RecontabilizacaoDeTransferencia_Service) {

    var mensagens = {
      vlr: 'Valor inicial deve ser menor do que o valor final.'
    };
    
  /*  this.formatFilter = function(widget, preference) {
        widget.dataSourceFilter = Bco01500_RecontabilizacaoDeTransferencia_Service.buildMainGridDataSourceFilter(widget, preference);
        widget.activate();
    };*/
    
    this.formatFilter = function(widget, preference) {
        var dataInicial = null;
        var dataFinal = null;
        var origem = null;
        var destino = null;
        var filterRow = preference.FILTER_ROW;
        if (filterRow.hasOwnProperty('DATA')){
            if (filterRow.DATA.hasOwnProperty('start')){
                dataInicial = filterRow.DATA.start;
                dataFinal = filterRow.DATA.end;
            }
        }
        if (filterRow.hasOwnProperty('ORIGEM')){
            if (filterRow.ORIGEM.length > 0){
                origem = filterRow.ORIGEM.join('_');
            }
        }
        if (filterRow.hasOwnProperty('DESTINO')){
            if (filterRow.DESTINO.length > 0){
                destino = filterRow.DESTINO.join('_');
            }
        }
        widget.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA': filter.value = filterRow.CDEMPRESA;
                  break;
                case 'ORIGEM': filter.value = (origem) ? origem : null;
                  break;
                case 'DESTINO': filter.value = (destino) ? destino : null; 
                  break;
                case 'DATAINI': filter.value = (dataInicial) ? dataInicial : null;
                  break;
                case 'DATAFIN': filter.value = (dataFinal) ? dataFinal : null;
                  break;
                case 'VRINI': filter.value = (filterRow.VRINI) ? filterRow.VRINI : null;
                  break;
                case 'VRFIN': filter.value = (filterRow.VRFIN) ? filterRow.VRFIN : null;
                  break;
                case 'DSHISTTBANC': filter.value = (filterRow.DSHISTTBANC) ? filterRow.DSHISTTBANC : null;
                  break;
            }  
        });      
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('DATA')){
            delete preference.CUSTOM_FILTER_ROW.DATA;
        }
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('ORIGEM'))
            if (preference.CUSTOM_FILTER_ROW.ORIGEM.length > 0)
                preference.CUSTOM_FILTER_ROW.ORIGEM = preference.CUSTOM_FILTER_ROW.ORIGEM.join('_');
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('DESTINO'))
            if (preference.CUSTOM_FILTER_ROW.DESTINO.length > 0)
                preference.CUSTOM_FILTER_ROW.DESTINO = preference.CUSTOM_FILTER_ROW.DESTINO.join('_');
        widget.activate();
    };
    
    this.loadGridTransferencias = function(widget) {
      widget.widgets[0].dataSourceFilter = [
        {
          name: "NRTRANBANC",
          operator: "=",
          value: widget.currentRow.NRTRANBANC
        },
        {
          name: "CDEMPRESA",
          operator: "=",
          value: widget.currentRow.CDEMPRESA
        }
      ];
      
      widget.widgets[0].reload();
      
    };
    
    this.onChangeValue = function(field, widget) {
        if(widget.currentRow.VRINI && widget.currentRow.VRFIN) {
          if(widget.currentRow.VRINI >= widget.currentRow.VRFIN) {
            if(field === 'VRINI') widget.currentRow.VRINI = undefined;
            else widget.currentRow.VRFIN = undefined;
            ScreenService.showMessage(mensagens.vlr);
          }
        }
        if(widget.currentRow.VRINI || widget.currentRow.VRFIN) {
          widget.getField('VRINI').validations = {required: true};
          widget.getField('VRFIN').validations = {required: true};
        }
        else {
          widget.getField('VRINI').validations = {};
          widget.getField('VRFIN').validations = {};
        }
    };
    
    this.consultaContabilizacao = function(widget){

        var checkedRows = widget.getCheckedRows();
        var contabil = [];

        checkedRows.forEach(function(checkedRow) {
            contabil.push({
            	CDEMPRESA : checkedRow.CDEMPRESA,	
	        	NRLANCCONT: checkedRow.NRLCTBTRAN
            });
        });
        
        if(contabil.length > 0)
            contabilizacaoController.exibeContabilizacao(contabil);
        else 
        	ScreenService.showMessage('Título(s) marcado(s) não possui(em) contabilização(ões).');
    };
    
    this.recontabilizarTransferencias = function(widget) {    
        var checkedRows = widget.dataSource.checkedRows;
        
        Bco01500_RecontabilizacaoDeTransferencia_Service.recontabilizarTransferencias(checkedRows).then(function(success){
          ScreenService.showMessage("Recontabilizacao feita com sucesso.");   
        }).catch(function(error) {
          
          if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar todos os lançamentos devido a erros na contablização.")) {
            var message = "Timeout";
            if (error.data.error) {
              message = error.data.error;
            }  
            ScreenService.showMessage(message);
          }
          
        });
    };
    
};

Configuration(function(ContextRegister) {
    ContextRegister.register('Bco01500_RecontabilizacaoDeTransferencia_Controller', Bco01500_RecontabilizacaoDeTransferencia_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco11100_extratoContaCorrente_Controller.js
var Bco11100_extratoContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;

    self.openFilter = function(extrato){
        self.iniciaExtratoFiltro(extrato).then(function(filterRow){
            self.abrirFiltro(extrato, filterRow);
        });
    };

    self.abrirFiltro = function(extrato, filterRow){
        ZhFilterPreferences.setOriginalFilter(extrato.id, filterRow);
        ZhFilterPreferences.openFilterWidget(extrato, filterRow);
    };

    self.iniciaExtratoFiltro = function(extrato){
        var deferred = ZHPromise.defer();
        filterRow = {};
        var dtmovbanc = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'DTMOVBANC';
        });
        if (dtmovbanc.length == 0){
            var start = moment().subtract(1, 'Month').format('DD/MM/YYYY');
            var end   = moment().format('DD/MM/YYYY');
            var periodoInicial = {
                "name": "DTMOVBANC",
                "value": [start, end],
                "operator": "BETWEEN",
                "isCustomFilter": true
            };
            extrato.dataSourceFilter.push(periodoInicial);
            filterRow.DTMOVBANC = {start: start, end: end};
        }
        deferred.resolve(filterRow);
        return deferred.promise;
    };

    self.preparaFiltroExtrato = function(preference){
        var extrato = templateManager.container.getWidget('extrato');
        var dataInicial = null;
        var dataFinal = null;
        if (preference.FILTER_ROW.hasOwnProperty('DTMOVBANC')){
            if (preference.FILTER_ROW.DTMOVBANC.hasOwnProperty('start')){
                dataInicial = preference.FILTER_ROW.DTMOVBANC.start;
                dataFinal = preference.FILTER_ROW.DTMOVBANC.end;
                
            }
        }
        extrato.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDREDUCONT': filtro.value = (preference.FILTER_ROW.CDREDUCONT) ? preference.FILTER_ROW.CDREDUCONT : null;
                    break;
                case 'DATAINI': filtro.value = (dataInicial) ? dataInicial : null;
                    break;
                case 'DATAFIN': filtro.value = (dataFinal) ? dataFinal : null;
                    break;
            }
        });
        var dtmovbanc = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'DTMOVBANC';
        });
        var empresa = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'CDEMPRESA';
        });
        
        preference.CUSTOM_FILTER_ROW = preference.FILTER_ROW;
        
        delete preference.CUSTOM_FILTER_ROW.CDEMPRESA;
        //delete preference.CUSTOM_FILTER_ROW.DTMOVBANC;
    };

    self.habilitaContaCorrente = function(filtro){
        var extrato = templateManager.container.getWidget('extrato');
        if (filtro.currentRow.NMEMPRESA) {
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('CONTACORRENTE').readOnly = false;
        }else {
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = null;
            filtro.getField('CONTACORRENTE').readOnly = true;
        }
        filtro.currentRow.CONTACORRENTE = null;
    };

    self.chooseTipoDeMovimentacaoBancaria = function(movimentacaoTabbed){
        var id = movimentacaoTabbed.getParent().currentRow.IDORIGMBANC;
        var movimentacaoWidget = templateManager.container.getWidget("tipoDeMovimentacaoBancaria_"+id);
        this.switchWidgetChild(movimentacaoTabbed, movimentacaoWidget);
        movimentacaoTabbed.widgets.forEach(
            function(widget){
                widget.isVisible = (widget.name === movimentacaoWidget.name);
            }
        );
        this.updateTipoDeMovimentacaoBancariaFilter(movimentacaoWidget);
    };

    self.openDetalhesFromExtrato = function(extratoWidget){
        if(extratoWidget.currentRow.IDMOVBANC != 'X'){
            var movimentacaoTabbed = extratoWidget.widgets[0];
            ScreenService.openSwipe(movimentacaoTabbed).then(
                function(promise){
                    self.chooseTipoDeMovimentacaoBancaria(movimentacaoTabbed);
                }
            );
        }else ScreenService.closeSwipe();
    };

    self.updateTipoDeMovimentacaoBancariaFilter = function(movimentacaoWidget){
        var extratoWidget = movimentacaoWidget.getParent().getParent();
        var codigoReduzido  = extratoWidget.currentRow.CDREDUCONT;
        var numeroMovimentacao = extratoWidget.currentRow.NRMOVBANC;
        codigoReduzido = (codigoReduzido) ? codigoReduzido : null;
        numeroMovimentacao = (numeroMovimentacao) ? numeroMovimentacao : null;
        var paramenterNames = ["CDREDUCONT","NRMOVBANC"];
        var values = [codigoReduzido, numeroMovimentacao];
        movimentacaoWidget.dataSourceFilter = [];
        this.setFilterParameters(movimentacaoWidget, paramenterNames , values);
        movimentacaoWidget.reload();
    };

    self.updateTipoDeMovimentacaoBancariaGridFilter = function(movimentacaoGridWidget){
        var movimentacaoWidget = movimentacaoGridWidget.getParent();
        var id = movimentacaoWidget.getParent().getParent().currentRow.IDORIGMBANC;
        var paramenterNames, values;
        var codigoUnidade;
        switch (id) {
            case 'T':
                var numeroTransferencia  = movimentacaoWidget.currentRow.NRTRANBANC;
                var codigoEmpresa = movimentacaoWidget.currentRow.CDEMPRESA;
                numeroTransferencia = (numeroTransferencia) ? numeroTransferencia : null;
                codigoEmpresa = (codigoEmpresa) ? codigoEmpresa : null;
                paramenterNames = ["NRTRANBANC","CDEMPRESA"];
                values = [numeroTransferencia, codigoEmpresa];
                break;
            case 'P':
                codigoUnidade = movimentacaoWidget.currentRow.CDFILIAL;
                var numeroBaixaPagar  = movimentacaoWidget.currentRow.NRBAIXAPAG;
                codigoUnidade = (codigoUnidade) ? codigoUnidade : null;
                numeroBaixaPagar = (numeroBaixaPagar) ? numeroBaixaPagar : null;
                paramenterNames = ["CDFILIAL","NRBAIXAPAG"];
                values = [codigoUnidade, numeroBaixaPagar];
                break;
            case 'R':
                codigoUnidade = movimentacaoWidget.currentRow.CDFILIAL;
                var numeroBaixaReceber  = movimentacaoWidget.currentRow.NRBAIXAREC;
                codigoUnidade = (codigoUnidade) ? codigoUnidade : null;
                numeroBaixaReceber = (numeroBaixaReceber) ? numeroBaixaReceber : null;
                paramenterNames = ["CDFILIAL","NRBAIXAREC"];
                values = [codigoUnidade, numeroBaixaReceber];
        }
        movimentacaoGridWidget.dataSourceFilter = [movimentacaoGridWidget.dataSourceFilter[0]];
        this.setFilterParameters(movimentacaoGridWidget, paramenterNames , values);
        movimentacaoGridWidget.reload();
        movimentacaoGridWidget.activate();
    };

    self.setFilterParameters = function(object, parameterNames, values){
        var dataSourceFilter = object.dataSourceFilter;
        for (var i = 0; i< parameterNames.length; i++){
            dataSourceFilter = dataSourceFilter.concat(
                {
                    "name": parameterNames[i],
                    "operator": "=",
                    "value": values[i]
                }
            );
        }
        object.dataSourceFilter = dataSourceFilter;
    };

    self.switchWidgetChild = function(widget , widgetChild){
        widget.currentWidget = widgetChild;
        widgetChild.activate();
    };
    
    self.calcSaldo = function(widget){
        var saldo = widget.dataSource.data[0].SALDOINIC;
        
        widget.dataSource.data.forEach(function(data){
            if(data.IDMOVBANC != 'X')
                saldo += data.VRMOVBANCSIMB;
            else saldo = data.VRMOVBANCSIMB;
            data.SALDO = saldo;
        });
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_extratoContaCorrente_Controller', Bco11100_extratoContaCorrente_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco11100_lancamentoContaCorrente_Controller.js
var Bco11100_lancamentoContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;

    self.forceActivate = function(){
        templateManager.container.getWidget('lancamentoContaCorrente').activate();
    };

    self.openFilter = function(lancamento){
        self.iniciaLancamentoFiltro(lancamento)
        .then(function(filterRow){
            self.buscaPadraoEmpresa(filterRow).then(function(filterRow){
                self.abrirFiltro(lancamento, filterRow);
            });

        });
    };
    
    self.buscaPadraoEmpresa = function(filterRow) {
        var deferred = ZHPromise.defer();
        var lancamentoContaCorrente = templateManager.container.getWidget('lancamentoContaCorrente');
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial)
        .then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                filterRow.CDEMPRESA = empresa.CDEMPRESA;
                filterRow.NMEMPRESA = empresa.NMEMPRESA;
                lancamentoContaCorrente.getField('DSCONTCORR').dataSourceFilter[0].value = cdempresa;
                deferred.resolve(filterRow);
            });
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    self.abrirFiltro = function(lancamento, rowFilter){
        var deferred = ZHPromise.defer();
        ZhFilterPreferences.setOriginalFilter(lancamento.id, rowFilter);
        ZhFilterPreferences.openFilterWidget(lancamento, rowFilter);
        deferred.resolve();
        return deferred.promise;
    };

    self.iniciaLancamentoFiltro = function(lancamento){
        var deferred = ZHPromise.defer();
        var route = 'fin#/crud/geral_bco';
        var rowFilter = {};
        //rowFilter.VRMOVI_INI = 0;
        //rowFilter.VRMOVI_FIN = 0;
        rowFilter.DEBCRED = 'T';
        General.callRestEngine(route, 'FilterData', [])
            .then(function(response){
                self.setDataLancamentoNoFiltro(lancamento, response, 'DTMOVICRED')
                .then(function(intervalo){
                    rowFilter.DTMOVICRED = intervalo;
                })
                .catch(function(){});
                lancamento.dataSourceFilter.forEach(function(filtro){
                    switch(filtro.name){
                        case 'VRTODOS': filtro.value = 'T';
                            break;
                        case 'DEBCRED': filtro.value = 'T';
                            break;
                        case 'TTIPODEBCRED': filtro.value = 'T';
                            break;
                    }
                });
                lancamento.getField('TIPODEBCRED').filterProperties.isVisible = true;
                deferred.resolve(rowFilter);
            })
            .catch(function(erro){
                deferred.resolve(rowFilter);
            });
        return deferred.promise;
    };

    self.setDataLancamentoNoFiltro = function(lancamento, response, data){
        var deferred = ZHPromise.defer();
        var dtmovimento = lancamento.dataSourceFilter.filter(function(filtro){
            return filtro.name == data;
        });
        if (dtmovimento.length == 0){
            var diasAnt  = response.dataset.geral_bco[0].QTDIAANTLANC;
            var diasPos  = response.dataset.geral_bco[0].QTDIAAPOLANC;
            var start = moment().subtract(diasAnt, 'days').format('DD/MM/YYYY');
            var end   = moment().add(diasPos, 'days').format('DD/MM/YYYY');
            var periodoInicial = {
                "name": data,
                "value": [start, end],
                "operator": "BETWEEN",
                "static": true
            };
            //lancamento.dataSourceFilter.push(periodoInicial);
            var intervalo = {start: start, end: end};
            deferred.resolve(intervalo);
            //lancamento.getField(data).filterProperties.newLine = true;
        }else{
            deferred.reject();
        }
        return deferred.promise;
    };

    /**
     * Prepara Filtro de Lançamento
     */
    self.preparaFiltroLancamento = function(args){
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        
        var filtroLancamento = args.e.preference;
        var valorInicial = filtroLancamento.FILTER_ROW.VRMOVI_INI;
        var valorFinal = filtroLancamento.FILTER_ROW.VRMOVI_FIN;
        
        lancamento.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'VRMOVI_INI' || filtro.name == 'VRMOVI_FIN'){
                filtro.value = self.formataValoresDecimais(filtro.value);
            }
        });
        
        var fieldTipo = lancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.filterProperties.isVisible == true);
        });
        
        var tpdebcred_concat = '';
        if (filtroLancamento.FILTER_ROW.DEBCRED == 'C')
            tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'C');
        else if (filtroLancamento.FILTER_ROW.DEBCRED == 'D')
            tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'D');
        else if (filtroLancamento.FILTER_ROW.DEBCRED == 'T')
            tpdebcred_concat = self.preparaFiltroTipoLancamento(fieldTipo[0], 'T');
        
        lancamento.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'VRTODOS': filtro.value = ((valorInicial >= 0) && (valorFinal > 0) && (valorFinal >= valorInicial)) ? null : 'T';
                    break;
                case 'TIPODEBCRED': filtro.value = tpdebcred_concat;
                    break;
                case 'TTIPODEBCRED': filtro.value = (tpdebcred_concat) ? null : 'T';
                    break;
            }
        });
    };

    /**
     * Prepara Filtro de Tipo de Lançamento 
     */
    self.preparaFiltroTipoLancamento = function(tipo, id){
        var tipo_checked     = tipo.dataSource.getCheckedRows();
        var primeiro         = true;
        var tpdebcred_concat = '';
        
        if (tipo_checked.length > 0){
            tipo_checked.forEach(function(row){
                if (id == 'C'){
                    if (primeiro){
                        tpdebcred_concat += id + row.CDTPCREDBANC;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + id + row.CDTPCREDBANC;
                }else if (id == 'D'){
                    if (primeiro){
                        tpdebcred_concat += id + row.CDTPDEBIBANC;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + id + row.CDTPDEBIBANC;
                }else if (id == 'T'){
                    if (primeiro){
                        tpdebcred_concat += row.IDTIPO + row.TIPODEBCRED;
                        primeiro = false;
                    }else tpdebcred_concat += '_' + row.IDTIPO + row.TIPODEBCRED;
                }
            });
        }
        
        return tpdebcred_concat;
    };

    self.abrirPopupCheque = function (){
        var emissaoCheque = templateManager.container.getWidget('emissaoCheque');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        ScreenService.openPopup(emissaoCheque).then(
            function(){
                emissaoCheque.edit();
                emissaoCheque.getField('TIPOCHEQUE').setValue('A');
                emissaoCheque.getField('NRCHEQUE').dataSourceFilter[0].value = lancamento.currentRow.CDREDUCONT;
            }
        );
    };

    self.validaTipoCheque = function (widget){
        if (widget.currentRow.TIPOCHEQUE == 'A'){
            widget.getField('NOMINAL').readOnly = false;
        }else{
            widget.getField('NOMINAL').readOnly = true;
            widget.getField('NOMINAL').setValue('');
        }
    };

    self.confirmarEmissaoCheque = function(){
        var route = "fin_movimentacaoBancaria#/emissaoCheque";
        var emissaoCheque = templateManager.container.getWidget('emissaoCheque');
        if (emissaoCheque.isValid()){
            var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
            var nominal = (lancamento.currentRow.TIPOCHEQUE == 'A') ? emissaoCheque.currentRow.NOMINAL : null;
            var nrcheque = emissaoCheque.currentRow.NRCHEQUE;
            var row = {
                'VRCHEQUE': lancamento.currentRow.VRMOVICRED,
                'DTEMISCHEQ': lancamento.currentRow.DTMOVBANC,
                'NRMOVIDEBI': lancamento.currentRow.CODIGOLANCAMENTO,
                'CDREDUCONT': lancamento.currentRow.CDREDUCONT,
                'DSNOMCHEQ': nominal,
                'CDEMPRESA': lancamento.currentRow.CDEMPRESA,
                'IDCTBLBANC': lancamento.currentRow.IDCTBLBANC,
                'NRCHEQUE': emissaoCheque.currentRow.NRCHEQUE
            };
            General.callRestEngine(route, 'Row', row)
                .then(
                    function(){
                        self.setValoresDefaultCheque(lancamento, emissaoCheque).then(function(){
                            ScreenService.showMessage('Cheque salvo com sucesso.').then(function(){
                                if (emissaoCheque.currentRow.TIPOCHEQUE == "A"){
                                    ScreenService.confirmMessage('Cheque inicial '+lancamento.getField('NRCHEQUE').value()+'. Confirmar emissão do cheque?', 'question', function () {
                                        Bco11100_movimentacaoContaCorrente_Service.buscaMunicipio(getLocalVar('CDFILIAL'), lancamento.currentRow.NRORG).then(function (response) {
                                            var municipio = response.dataset.municipio_endefili[0].NMMUNICIPIO;
                                            self.getReport(municipio, lancamento.currentRow.CDREDUCONT, nrcheque);
                                        });
                                    });
                                }
                            });
                        });
                    }
                )
                .catch(function(erro){
                    if (contabilizacaoController.checkErrosContabilizacao(erro)){
                        ScreenService.showMessage(erro.data.error);
                    }else{
                        lancamento.activate();
                    }
                });
        }
    };

    self.setValoresDefaultCheque = function(lancamento, emissaoCheque){
        var deferred = ZHPromise.defer();
        ScreenService.closePopup();
        lancamento.activate();
        lancamento.getField('NRCHEQUE').isVisible = true;
        lancamento.getField('NOMINAL').isVisible = true;
        lancamento.getField('NRCHEQUE').setValue(emissaoCheque.getField('NRCHEQUE').value());
        lancamento.getField('NOMINAL').setValue(emissaoCheque.getField('NOMINAL').value());
        emissaoCheque.getField('NOMINAL').setValue('');
        emissaoCheque.getField('NRCHEQUE').setValue('');
        lancamento.getAction('EMITIRCHEQUE').isVisible = false;
        deferred.resolve();
        return deferred.promise;
    };

    self.getReport = function(municipio, cdreducont, nrcheque){
        var routes = 'fin_movimentacaoBancaria#/ModeloCheque';
        var row = {
            P_TESTE   : 'N',
            P_STLOCAL : municipio,
            P_CODREDU : cdreducont,
            P_NUMERO  : nrcheque
        };
        General.openReport(routes, 'ModeloCheque', row);
    };

    self.cancelLancamento = function (widget){
        General.widgetCancelForm(widget);
        widget.getField('NRCHEQUE').isVisible = true;
        widget.getField('NOMINAL').isVisible = true;
    };

    self.validaVisibilidadeCheque = function(widget){
        if (widget.currentRow.DEBCRED == 'D'){
            if (widget.currentRow.NRCHEQUE){
                widget.getField('NRCHEQUE').isVisible = true;
                widget.getField('NOMINAL').isVisible = true;
            }else{
                widget.getField('NRCHEQUE').isVisible = false;
                widget.getField('NOMINAL').isVisible = false;
            }
        }else{
            widget.getField('NRCHEQUE').isVisible = false;
            widget.getField('NOMINAL').isVisible = false;
        }
    };

    self.limitaValorDesconto = function(widget){
        var superiorDisponivel, superiorLancamento = false;
        var valorLancamento = parseFloat(templateManager.container.getWidget('lancamentoContaCorrente').currentRow.VRMOVICRED);
        var totalDesconto = 0.00;
        var totalDisponivel = 0.00;
        var indice = widget.currentRow.__index;
        valorDesconto = parseFloat(widget.currentRow.VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
        totalDisponivel = parseFloat(widget.currentRow.VRTOTDISP.toString().replace('.', '').replace(',', '.'));
        var data = widget.dataSource.data;
        for(var i = 0; i < data.length; i++){
            if (i != indice && data[i].__isSelected){
                totalDesconto += parseFloat(data[i].VRDESCONTO);
            }
        }
        if ((valorDesconto + totalDesconto) > totalDisponivel) {
            superiorDisponivel = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }
        if (valorDesconto > valorLancamento) {
            superiorLancamento = true;
            widget.dataSource.data[indice].VRDESCONTO_inline = 0;
        }
        if (widget.currentRow.__isSelected) {
            desconto = parseFloat(totalDesconto) + parseFloat(widget.dataSource.data[indice].VRDESCONTO_inline.toString().replace('.', '').replace(',', '.'));
            widget.getParent().currentRow.TOTALDESCONTO = desconto;
        }else{
            widget.getParent().currentRow.TOTALDESCONTO = parseFloat(totalDesconto);
        }
        if (superiorDisponivel){
            ScreenService.showMessage('Valor do desconto deve ser menor ou igual que o valor disponível.');
        }
        if (superiorLancamento){
            ScreenService.showMessage('Operação bloqueada. O valor a descontar não pode ser maior que o valor do lançamento.');
        }
    };

    self.setValorCliente = function(widget){
        if (widget.currentRow.CDFORNECED){
            widget.currentRow.CDCLIENTE = null;
            widget.currentRow.NRINSJURCLIE = null;
            widget.currentRow.NMRAZSOCCLIE = null;
        }
    };

    self.setValorFornecedor = function(widget){
        if (widget.currentRow.CDCLIENTE){
            widget.currentRow.CDFORNECED = null;
            widget.currentRow.NMRAZSOCFORN = null;
            widget.currentRow.NRINSJURFORN = null;
        }
    };

    self.setCurrentRowForm = function (widget, clifor){
        if (clifor == 'C') {
            widget.getParent().currentRow.NMRAZSOCCLIE = widget.getParent().getParent().currentRow.NMRAZSOCCLIE;
        }else{
            widget.getParent().currentRow.NMRAZSOCFORN = widget.getParent().getParent().currentRow.NMRAZSOCFORN;
        }
    };

    self.setDataSourcePopupCliente = function(popupCliente){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        popupCliente.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDCLIENTE') === 0){
                filtro.value = widget.currentRow.CDCLIENTE;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = widget.currentRow.CDEMPRESA;
            }
        });
        popupCliente.reload();
    };

    self.setDataSourcePopupFornecedor = function(popupFornecedor){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        popupFornecedor.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('CDFORNECED') === 0){
                filtro.value = widget.currentRow.CDFORNECED;
            }else if (filtro.name.localeCompare('CDEMPRESA') === 0) {
                filtro.value = widget.currentRow.CDEMPRESA;
            }else if (filtro.name.localeCompare('NRINSJURFORN') === 0) {
                filtro.value = widget.currentRow.NRINSJURFORN;
            }
        });
        popupFornecedor.reload();
    };

    self.newRowLancamento = function(widget){
        if(widget.appliedFilter){
            if(widget.appliedFilter.FILTER_ROW.CDREDUCONT){
                widget.getField('NRCHEQUE').isVisible = false;
                widget.getField('NOMINAL').isVisible = false;
                General.widgetNewRow(widget);
                widget.currentRow.CDREDUCONT = widget.appliedFilter.FILTER_ROW.CDREDUCONT;
                widget.currentRow.DSCONTCORR = widget.appliedFilter.FILTER_ROW.CONTACORRENTE;
                widget.currentRow.NMEMPRESA = widget.appliedFilter.FILTER_ROW.NMEMPRESA;
                widget.currentRow.CDEMPRESA = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
                widget.getField('NMFILIAL').dataSourceFilter[0].value = widget.appliedFilter.FILTER_ROW.CDEMPRESA;
                widget.getField('NMFILIAL').reload(); 
            }else ScreenService.showMessage("Operação bloqueada. Selecione a Conta Corrente no filtro.");
        }else ScreenService.showMessage("Operação bloqueada. Selecione a Conta Corrente no filtro.");
    };

    self.saveLancamentoContaCorrente = function(lancamento){
        if (lancamento.isValid()) {
            if (parseFloat(lancamento.currentRow.VRMOVICRED) != 0.00) {
                if (lancamento.currentRow.__is_new) {
                    var routeAdiantamento = '';
                    var adiantamentoPopup = '';
                    if (lancamento.currentRow.CDCLIENTE) {
                        routeAdiantamento = 'fin_movimentacaoBancaria#/validaAdiantamentoCliente';
                        adiantamentoPopup = templateManager.container.getWidget('adiantamentoClienteTabbed');
                        msg = 'Atenção! Cliente possui crédito de adiantamento. Deseja utilizá-lo?';
                        self.openAdiantamento(lancamento, routeAdiantamento, adiantamentoPopup, msg, 'C');
                    } else if (lancamento.currentRow.CDFORNECED) {
                        routeAdiantamento = 'fin_movimentacaoBancaria#/validaAdiantamentoFornecedor';
                        adiantamentoPopup = templateManager.container.getWidget('adiantamentoFornecedorTabbed');
                        msg = 'Atenção! Fornecedor possui crédito de adiantamento. Deseja utilizá-lo?';
                        self.openAdiantamento(lancamento, routeAdiantamento, adiantamentoPopup, msg, 'F');
                    } else {
                        self.confirmarLancamentoContaCorrente(lancamento.currentRow.DEBCRED);
                    }
                } else {
                    self.confirmarLancamentoContaCorrente(lancamento.currentRow.DEBCRED);
                }
            }else{
                ScreenService.showMessage("Valor deve ser diferente de zero.");
            }
        }
    };

    self.openAdiantamento = function(lancamento, routeAdiantamento, adiantamentoPopup, msg, clifor){
        var msgDev, routeDevolucao, devolucaoPopup = '';
        if (clifor == 'C') {
            msgDev = 'Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';
            devolucaoPopup = templateManager.container.getWidget('devolucaoClienteTabbed');
        }else{
            msgDev = 'Atenção! Fornecedor possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';
            devolucaoPopup = templateManager.container.getWidget('devolucaoFornecedorTabbed');
        }
        General.callRestEngine(routeAdiantamento, 'Row', lancamento.currentRow)
            .then(function(adiantamento) {
                if (adiantamento.dataset.ADIANTAMENTO.length > 0) {
                    ScreenService.confirmMessage(msg, 'question',
                        function() {
                            ScreenService.openPopup(adiantamentoPopup).then(function () {
                                self.setCurrentRowForm(adiantamentoPopup.widgets[0], clifor);
                                adiantamentoPopup.widgets[0].reload();
                            });
                        },
                        function(){
                            self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
                        });
                }else{
                    self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
                }
            })
            .catch(function(erro){
                self.confirmarLancamentoContaCorrente(clifor);
            });
    };

    self.openDevolucao = function(lancamento, routeDevolucao, devolucaoPopup, msg, clifor){
        General.callRestEngine(routeDevolucao, 'Row', lancamento.currentRow)
            .then(function(devolucao) {
                if (devolucao.dataset.DEVOLUCAO.length > 0) {
                    ScreenService.confirmMessage(msg, 'question', function(success){
                        ScreenService.openPopup(devolucaoPopup).then(function(){
                            self.setCurrentRowForm(devolucaoPopup.widgets[0], clifor);
                            devolucaoPopup.widgets[0].reload();
                        });
                    });
                }else{
                    self.confirmarLancamentoContaCorrente(clifor);
                }
            })
            .catch(function(erro){
                self.confirmarLancamentoContaCorrente(clifor);
            });
    };

    self.confirmarAdiantamento = function(widget, clifor){
        var msgDev, msgConfirme, routeDevolucao, devolucaoPopup = '';
        if (clifor == 'C') {
            msgDev = 'Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoCliente';
            devolucaoPopup = templateManager.container.getWidget('devolucaoClienteTabbed');
        }else{
            msgDev = 'Atenção! Fornecedor possui nota fiscal de devolução. Deseja utilizá-la?';
            routeDevolucao = 'fin_movimentacaoBancaria#/validaDevolucaoFornecedor';
            devolucaoPopup = templateManager.container.getWidget('devolucaoFornecedorTabbed');
        }
        msgConfirme = 'É necessário selecionar os registros para confirmação do Adiantamento.';
        if (self.validacaoConfirmePopup(widget.widgets[0])) {
            lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
            self.openDevolucao(lancamento, routeDevolucao, devolucaoPopup, msgDev, clifor);
        }else{
            ScreenService.showMessage(msgConfirme);
        }
    };

    self.confirmarDevolucao = function(devolucao, clifor) {
        msgConfirme = 'É necessário selecionar os registros para confirmação da Devolução.';
        if (self.validacaoConfirmePopup(devolucao.widgets[0])) {
            self.confirmarLancamentoContaCorrente(clifor);
        } else {
            ScreenService.showMessage(msgConfirme);
        }
    };

    self.cancelarAdiantamento = function(routeDevolucao, clifor, widget){
        var devolucaoPopup = (clifor == 'C') ? templateManager.container.getWidget('devolucaoClienteTabbed') : templateManager.container.getWidget('devolucaoFornecedorTabbed');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
        General.callRestEngine(routeDevolucao, 'Row', lancamento.currentRow)
            .then(function(devolucao) {
                if (devolucao.length > 0) {
                    ScreenService.confirmMessage('Atenção! Cliente possui nota fiscal de devolução. Deseja utilizá-la?', 'question', function(success){
                        ScreenService.openPopup(devolucaoPopup).then(function(){
                            self.setCurrentRowForm(devolucaoPopup.widgets[0], clifor);
                        });
                    });
                }else{
                    self.confirmarLancamentoContaCorrente(clifor);
                }
            });
    };

    self.cancelarDevolucao = function(clifor, widget){
        widget.widgets[0].dataSource.unSelectAll();
        ScreenService.closePopup();
        self.confirmarLancamentoContaCorrente(clifor);
    };

    /**
     * Ao confirmar um novo lancamento em Conta Corrente são
     * realizadas validações em uma popup de adiantamento ou
     * desconto para clientes e fornecedores. Após validação
     * a ação de lançamento é continuada.
     */
    self.confirmarLancamentoContaCorrente = function(clifor){
        var adiantamento, devolucao = '';
        var routeSave = 'fin_movimentacaoBancaria#/saveLancamentoContaCorrente';
        //ScreenService.closePopup();
        //ScreenService.openWindow('fin_movimentacaoBancaria#/bco11100_movimentacaoContaCorrente');
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        if (clifor == 'C') {
            adiantamento = templateManager.container.getWidget('adiantamentoCliente');
            devolucao = templateManager.container.getWidget('devolucaoCliente');
        }else{
            adiantamento = templateManager.container.getWidget('adiantamentoFornecedor');
            devolucao = templateManager.container.getWidget('devolucaoFornecedor');
        }
        var dataSet = [];
        lancamento.currentRow.CDCLIENTE = (lancamento.currentRow.hasOwnProperty('CDCLIENTE')) ? lancamento.currentRow.CDCLIENTE : '';
        lancamento.currentRow.CDFORNECED = (lancamento.currentRow.hasOwnProperty('CDFORNECED')) ? lancamento.currentRow.CDFORNECED : '';
        dataSet[0] = lancamento.currentRow;
        var data = (adiantamento.dataSource.data) ? adiantamento.dataSource.data : [];
        for(var i = 0; i < data.length; i++){
            if (data[i].__isSelected){
                dataSet[dataSet.length] = data[i];
            }
        }
        var dataDev = (devolucao.dataSource.data) ? devolucao.dataSource.data : [];
        for(i = 0; i < dataDev.length; i++){
            if (dataDev[i].__isSelected){
                dataSet[data.length + dataSet.length] = dataDev[i];
            }
        }
        dataSet[0].QTDEADIANTAMENTO = adiantamento.getCheckedRows().length;
        
        localStorage.setItem('lancBancRow',JSON.stringify(dataSet));
        localStorage.setItem('TIPOBAIXA','LANCBANC');
        
        General.callRestEngine(routeSave, 'DataSet', dataSet)
            .then(function (response) {
                ScreenService.closeSwipe().then(function(){
                    lancamento.reload();
                    ScreenService.showMessage('Registro salvo com sucesso');
                    ScreenService.closePopup();
                });
                lancamento.activate();
                lancamento.currentRow.__is_new = false;
            })
            .catch(function(erro){
                if (contabilizacaoController.checkErrosContabilizacao(erro)){
                    ScreenService.showMessage(erro.data.error);
                }else{
                    lancamento.activate();
                }
            });
    };

    //Calcula o total disponível de contas do tipo adiantamento para cliente ou fornecedor
    self.calculaTotalAdiantamentoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += data.VRTOTDISP;
        });
        widget.getParent().currentRow.TOTALADIANTAMENTO = totalDisponivel;
        widget.getParent().currentRow.TOTALDESCONTO = 0;
    };

    //Calcula o total disponível de contas do tipo devolucao para cliente ou fornecedor
    self.calculaTotalDevolucaoDisponivel = function(widget){
        var totalDisponivel = 0;
        widget.dataSource.data.forEach(function(data){
            totalDisponivel += data.VRDISPONIVEL;
        });
        widget.getParent().currentRow.TOTALDEVOLUCAO = totalDisponivel;
    };

    self.selecionaTodosRegistros = function(widget){
        self.atualizaValorAdiantaDev(widget).then(
            function() {},
            function(error){
                widget.unSelectAll();
            }
        );
    };

    self.atualizaValorAdiantaDev = function(widget) {
        var deferred = ZHPromise.defer();
        var totalDesconto = 0;
        widget.getCheckedRows().forEach(function (data) {
            if (data.__isSelected) {
                totalDesconto += parseFloat(data.VRDESCONTO);
            }
        });
        var valorLancamento = parseFloat(templateManager.container.getWidget('lancamentoContaCorrente').currentRow.VRMOVICRED);
        if (totalDesconto > valorLancamento){
            ScreenService.showMessage('O valor a descontar não pode ser maior que o valor do lançamento.');
            widget.getLastSelectedRow().__isSelected = false;
            deferred.reject();
        }else {
            deferred.resolve();
            widget.getParent().currentRow.TOTALDESCONTO = totalDesconto;
        }
        return deferred.promise;
    };

    self.validacaoConfirmePopup = function(widget){
        var checked = false;
        var data = widget.dataSource.data;
        for (var i = 0; i < data.length && !checked; i++) {
            if (data[i].__isSelected){
                checked = true;
            }
        }
        return checked;
    };

    self.setTipoLancamento = function(widget){
        if (widget.currentRow.DEBCRED == 'C'){
            widget.currentRow.TIPOCREDITO = widget.currentRow.TIPO;
            self.validaLancamento(widget);
            widget.getAction('EMITIRCHEQUE').isVisible = false;
        }
        else{
            widget.currentRow.TIPODEBITO = widget.currentRow.TIPO;
            self.validaLancamento(widget);
            if (widget.currentRow.NRCHEQUE)
                 widget.getAction('EMITIRCHEQUE').isVisible = false;
            else widget.getAction('EMITIRCHEQUE').isVisible = true;
        }
    };

    self.deleteLancamentoContaCorrente = function(lancamento){
        var route = 'fin_movimentacaoBancaria#/deleteLancamentoContaCorrente';
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question',
            function (success) {
                General.callRestEngine(route, 'Row', lancamento.currentRow).then(function (mensagem) {
                    ScreenService.closeSwipe().then(function () {
                        lancamento.activate();
                        lancamento.reload();
                        ScreenService.showMessage(mensagem.messages[0].message);
                    });
                }).catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }
        );

    };

    self.validaTipoLancamento = function(widget){
        var tipocredito = widget.getField('TIPOCREDITO');
        var tipodebito = widget.getField('TIPODEBITO');
        var tipodebcred = widget.getField('TIPODEBCRED');
        if (widget.getField('DEBCRED').value() == 'C') {
            tipodebito.setValue('');
            tipodebito.isVisible = false;
            tipodebito.showOnForm = false;
            tipodebito.showOnView = false;
            tipodebito.class = "0";
            tipocredito.isVisible = true;
            tipocredito.showOnForm = true;
            tipocredito.showOnView = true;
            tipocredito.class = "6";
        }else if (widget.getField('DEBCRED').value() == 'D'){
            tipocredito.setValue('');
            tipocredito.isVisible = false;
            tipocredito.showOnForm = false;
            tipocredito.showOnView = false;
            tipocredito.class = "0";
            tipodebito.isVisible = true;
            tipodebito.showOnForm = true;
            tipodebito.showOnView = true;
            tipodebito.class = "6";
        }
        widget.getField('DESCRICAO').setValue('');
        tipodebito.reload();
        tipocredito.reload();
    };

    self.validaTipoLancamentoFiltro = function(filtroLancamento){
        var tipoCred = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.descriptionField == 'DSTPCREDBANC');
        })[0];
        var tipoDeb = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.descriptionField == 'DSTPDEBIBANC');
        })[0];
        var tipoDebCred = filtroLancamento.fields.filter(function(filtro){
            return (filtro.name == 'TIPODEBCRED' && filtro.descriptionField == 'DESCRICAO');
        })[0];
        if (filtroLancamento.getField('DEBCRED').value() == 'C') {
            tipoDeb.setValue('');
            tipoDebCred.setValue('');
            tipoDeb.isVisible = false;
            tipoDeb.showOnForm = false;
            tipoDeb.showOnView = false;
            tipoDeb.class = "0";
            tipoCred.isVisible = true;
            tipoCred.showOnForm = true;
            tipoCred.showOnView = true;
            tipoCred.class = "6";
            tipoDebCred.isVisible = false;
            tipoDebCred.showOnForm = false;
            tipoDebCred.showOnView = false;
            tipoDebCred.class = "0";
        }else if (filtroLancamento.getField('DEBCRED').value() == 'D') {
            tipoCred.setValue('');
            tipoDebCred.setValue('');
            tipoDeb.isVisible = true;
            tipoDeb.showOnForm = true;
            tipoDeb.showOnView = true;
            tipoDeb.class = "6";
            tipoCred.isVisible = false;
            tipoCred.showOnForm = false;
            tipoCred.showOnView = false;
            tipoCred.class = "0";
            tipoDebCred.isVisible = false;
            tipoDebCred.showOnForm = false;
            tipoDebCred.showOnView = false;
            tipoDebCred.class = "0";
        }else if (filtroLancamento.getField('DEBCRED').value() == 'T') {
            tipoCred.setValue('');
            tipoCred.setValue('');
            tipoDeb.isVisible = false;
            tipoDeb.showOnForm = false;
            tipoDeb.showOnView = false;
            tipoDeb.class = "0";
            tipoCred.isVisible = false;
            tipoCred.showOnForm = false;
            tipoCred.showOnView = false;
            tipoCred.class = "0";
            tipoDebCred.isVisible = true;
            tipoDebCred.showOnForm = true;
            tipoDebCred.showOnView = true;
            tipoDebCred.class = "6";
        }
        tipoDeb.reload();
        tipoCred.reload();
        tipoDebCred.reload();
    };

    self.validaLancamento = function(widget){
        var tipocredito = widget.getField('TIPOCREDITO');
        var tipodebito = widget.getField('TIPODEBITO');
        if (widget.getField('DEBCRED').value() == 'C') {
            tipodebito.isVisible = false;
            tipodebito.showOnForm = false;
            tipodebito.showOnView = false;
            tipodebito.class = "0";
            tipocredito.isVisible = true;
            tipocredito.showOnForm = true;
            tipocredito.showOnView = true;
            tipocredito.class = "6";
        }else{
            tipocredito.isVisible = false;
            tipocredito.showOnForm = false;
            tipocredito.showOnView = false;
            tipocredito.class = "0";
            tipodebito.isVisible = true;
            tipodebito.showOnForm = true;
            tipodebito.showOnView = true;
            tipodebito.class = "6";
        }
    };

    self.setTipoFiltro = function(widget){
        var tipoLancamento = widget.getField('DEBCRED');
        widget.getField('TIPOFILTRO').dataSourceFilter[0].value = tipoLancamento.value();
    };

    self.atualizaValoresSelectMultiple = function(widget){
        if (widget.getField('TIPO').filterProperties.copyValueSelectMultiple) {
            var valorConcatenado = widget.appliedFilter.FILTER_ROW.TIPO;
            widget.getField('TIPO').setValue(widget.getField('TIPO').filterProperties.copyValueSelectMultiple);
            widget.appliedFilter.FILTER_ROW.TIPO = valorConcatenado;
        }
    };

    self.setDescricaoLancamento = function(widget){
        widget.currentRow.DESCRICAO = (widget.currentRow.DEBCRED == 'C')? widget.currentRow.TIPOCREDITO : widget.currentRow.TIPODEBITO;
        widget.currentRow.TIPO = widget.currentRow.DESCRICAO;
    };

    self.setDefaultValue = function(fields){
        fields.forEach(function(field){
            if (field.value().length == 0){
                field.setValue("0,00");
            }
        });
    };

    self.validaActionContabilizacaoLancamento = function(widget){
        if (widget.currentRow.NRLANCCONT)
             widget.getAction('CONTABILIZACAO').isVisible = true;
        else widget.getAction('CONTABILIZACAO').isVisible = false;
    };

    self.consultarContabilizacaoLancamento = function(){
        var widget = templateManager.container.getWidget('lancamentoContaCorrente');
        if (widget.currentRow.NRLANCCONT){
            var contabil = [];
            contabil.push({});
            contabil[0].CDEMPRESA  = widget.currentRow.CDEMPRESA;
            contabil[0].NRLANCCONT = widget.currentRow.NRLANCCONT;
            contabilizacaoController.exibeContabilizacao(contabil);
        }
    };
    
    /**
     * Formata Valores Decimais
     */
    self.formataValoresDecimais = function(valor){
        var valorFormatado = 
            (valor.toString().search(',') != '-1') ?
                ((parseFloat(valor.toString().split(',')[0]) > 0) ?
                    parseFloat(valor.toString().replace('.', '').replace(',', '.')) 
                 : parseFloat(valor.toString().replace(',', '.')))
            : parseFloat(valor);
            
        return valorFormatado;
    };
    
    self.habilitaContaCorrente = function(filtro){
        var lancamentoContaCorrente = templateManager.container.getWidget('lancamentoContaCorrente');
        
        filtro.getField('CONTACORRENTE').setValue('');
        
        if (filtro.currentRow.NMEMPRESA) {
            //lancamentoContaCorrente.getField('ORIGEM').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('CONTACORRENTE').readOnly = false;
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
        }else {
            filtro.getField('CONTACORRENTE').readOnly = true;
            filtro.getField('CONTACORRENTE').dataSourceFilter[0].value = null;
        }
    };
    
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_lancamentoContaCorrente_Controller', Bco11100_lancamentoContaCorrente_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco11100_movimentacaoContaCorrente_Controller.js
var Bco11100_movimentacaoContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;
    
    /**
     * Preenche Valores do Stripe
     */
    self.preencheValoresStripe = function(widget){
        var stripeLancamento    = templateManager.container.getWidget('stripeLancamento');
        var stripeTransferencia = templateManager.container.getWidget('stripeTransferencia');
        var stripeExtrato       = templateManager.container.getWidget('stripeExtrato');

        if (widget.name == 'lancamentoContaCorrente'){
            var filtroLancamento = templateManager.container.getWidget('lancamentoContaCorrente').appliedFilter;
            
            stripeLancamento.currentRow.EMPRESA    = filtroLancamento.FILTER_ROW.CDEMPRESA + ' - ' + filtroLancamento.FILTER_ROW.NMEMPRESA;
            stripeLancamento.currentRow.CDCONTCORR = filtroLancamento.FILTER_ROW.CDCONTCORR;
            stripeLancamento.currentRow.DSCONTCORR = filtroLancamento.FILTER_ROW.CONTACORRENTE;
        }else if (widget.name == 'transferenciaContaCorrente'){
            var filtroTransferencia = templateManager.container.getWidget('transferenciaContaCorrente').appliedFilter;
            
            stripeTransferencia.currentRow.EMPRESA = filtroTransferencia.FILTER_ROW.CDEMPRESA + ' - ' + filtroTransferencia.FILTER_ROW.NMEMPRESA;
            
            if (filtroTransferencia.FILTER_ROW.CONTAORIGEM)
                 stripeTransferencia.currentRow.ORIGEM = filtroTransferencia.FILTER_ROW.CONTAORIGEM + ' - ' + filtroTransferencia.FILTER_ROW.ORIGEM;
            else stripeTransferencia.currentRow.ORIGEM = '';
            
            if (filtroTransferencia.FILTER_ROW.CONTADESTINO)
                 stripeTransferencia.currentRow.DESTINO = filtroTransferencia.FILTER_ROW.CONTADESTINO + ' - ' + filtroTransferencia.FILTER_ROW.DESTINO;
            else stripeTransferencia.currentRow.DESTINO = '';
        }else{
            var filtroExtrato = templateManager.container.getWidget('extrato').appliedFilter;
            
            stripeExtrato.currentRow.EMPRESA       = filtroExtrato.FILTER_ROW.CDEMPRESA + ' - ' + filtroExtrato.FILTER_ROW.NMEMPRESA;
            stripeExtrato.currentRow.CONTACORRENTE = filtroExtrato.FILTER_ROW.CDCONTCORR + ' - ' + filtroExtrato.FILTER_ROW.CONTACORRENTE;
            stripeExtrato.currentRow.PERIODO       = 'De ' + filtroExtrato.FILTER_ROW.DTMOVBANC.start + ' até ' + filtroExtrato.FILTER_ROW.DTMOVBANC.end;
        }
    };
    
    /**
     * Altera Stripe
     * Alteração de stripe de acordo com a tela ativada
     */ 
    self.alteraStripe = function(widget){
        var stripeLancamento    = templateManager.container.getWidget('stripeLancamento');
        var stripeTransferencia = templateManager.container.getWidget('stripeTransferencia');
        var stripeExtrato       = templateManager.container.getWidget('stripeExtrato');

        if (widget.name == 'lancamentoContaCorrente'){
            stripeLancamento.isVisible    = true;
            stripeTransferencia.isVisible = false;
            stripeExtrato.isVisible       = false;
        }else if (widget.name == 'transferenciaContaCorrente'){
            stripeLancamento.isVisible    = false;
            stripeTransferencia.isVisible = true;
            stripeExtrato.isVisible       = false;
        }else{
            stripeLancamento.isVisible    = false;
            stripeTransferencia.isVisible = false;
            stripeExtrato.isVisible       = true;
        }
    };

    self.validaActionContabilizacaoTransferencia = function(widget){
        if (widget.currentRow.NRLCTBTRAN){
            widget.getAction('CONTABILIZACAO').isVisible = true;
        }else{
            widget.getAction('CONTABILIZACAO').isVisible = false;
        }
    };

    self.consultarContabilizacaoTransferencia = function(){
        var widget = templateManager.container.getWidget('transferenciaContaCorrente');
        if (widget.currentRow.NRLANCCONT){
            var contabil = [];
            contabil.push({});
            contabil[0].CDEMPRESA  = widget.currentRow.CDEMPRESA;
            contabil[0].NRLANCCONT = widget.currentRow.NRLCTBTRAN;
            contabilizacaoController.exibeContabilizacao(contabil);
        }
    };



    this.setValorFinal = function(widget, valorInicial, valorFinal){
        var novoValorInicial = (valorInicial.value()) ? valorInicial.value() : 0;
        valorInicial.value(novoValorInicial);
        if (widget.getField(valorFinal).value() < novoValorInicial){
            widget.getField(valorFinal).value(valorInicial.value());
        }
    };

    self.setStripeData = function(gridWidget) {
        var currentRow = gridWidget.appliedFilter.FILTER_ROW;
        var stripeWidget = templateManager.container.getWidget("stripe");
        stripeWidget.currentRow = {};
        stripeWidget.currentRow.EMPRESA = currentRow.CDEMPRESA + ' - ' + currentRow.NMEMPRESA;
        stripeWidget.currentRow.CDEMPRESA = currentRow.CDEMPRESA;
        stripeWidget.currentRow.NMEMPRESA = currentRow.NMEMPRESA;
    };

    self.concatenaCamposSelectMultiple = function(widget, fields, codigos){
        var field = null;
        for (i = 0; i < fields.length; i++) {
            field = widget.getField(fields[i]);
            self.setValorConcatenado(widget, field, field.filterProperties.name, codigos[i], 'T'+field.filterProperties.name);
        }
    };

    self.setValorConcatenado = function(widget, field, fieldName, codigo, todos){
        var valorConcatenado = '_';

        if (field.value()) {
            if (field.value().length > 0) {
                field.dataSource.data.forEach(function (data) {
                    if (data.__isSelected) {
                        valorConcatenado += data[codigo] + "_";
                    }
                });
                valorConcatenado = (valorConcatenado != '_') ? valorConcatenado : '';
                widget.dataSourceFilter.forEach(function (filtro) {
                    if (filtro.name.localeCompare(fieldName) === 0) {
                        filtro.value = valorConcatenado;
                    }else if (filtro.name.localeCompare(todos) === 0){
                        filtro.value = '';
                    }
                });
            }else{
                widget.dataSourceFilter.forEach(function (filtro) {
                    if (filtro.name.localeCompare(todos) === 0){
                        filtro.value = 'T';
                    }
                });
            }
        }else{
            if (field.name.localeCompare(fieldName) === 0){
                widget.dataSourceFilter.forEach(function(filtro){
                    if (filtro.name.localeCompare(todos) === 0){
                        filtro.value = 'T';
                    }
                });
            }
        }
    };

    self.setDataSourceFilhos = function(){
        var lancamento = templateManager.container.getWidget('lancamentoContaCorrente');
        var transferencia = templateManager.container.getWidget('transferenciaContaCorrente');
        var extrato = templateManager.container.getWidget('extrato');
        ScreenService.openSwipe(lancamento.getParent()).then(function(){
            self.setLancamentoContaCorrente(lancamento).then(function(){
                lancamento.activate().then(function(){
                    self.iniciaLancamentoFiltro(lancamento);

                });
            });
            self.setTransferenciaContaCorrente(transferencia).then(function(){
                self.iniciaTransferenciaFiltro(transferencia);
            });
            self.setExtratoContaCorrente(extrato).then(function(){
                self.iniciaExtratoFiltro(extrato);
            });
        });
    };

    self.iniciaTransferenciaFiltro = function(transferencia){
        var route = 'fin#/crud/geral_bco';
        General.callRestEngine(route, 'FilterData', [])
            .then(function(response){
                var dttranbanc = transferencia.dataSourceFilter.filter(function(filtro){
                    return filtro.name == 'DTTRANBANC';
                });
                if (dttranbanc.length == 0){
                    var diasAnt  = response.dataset.geral_bco[0].QTDIAANTTRAN;
                    var diasPos  = response.dataset.geral_bco[0].QTDIAAPOTRAN;
                    var start = moment().subtract(diasAnt, 'days').format('DD/MM/YYYY');
                    var end   = moment().add(diasPos, 'days').format('DD/MM/YYYY');
                    var periodoInicial = {
                        "name": "DTTRANBANC",
                        "value": [start, end],
                        "operator": "BETWEEN",
                        "static": true
                    };
                    transferencia.dataSourceFilter.push(periodoInicial);

                    transferencia.getField('DTTRANBANC').filterProperties.defaultValue = {
                        start: start,
                        end: end
                    };
                    transferencia.getField('DTTRANBANC').filterProperties.newLine = true;
                }
                var vrtranbanc_ini = transferencia.dataSourceFilter.filter(function(filtro){
                    return filtro.name == 'VRTRANBANC_INI';
                });
                if (vrtranbanc_ini.length == 0){
                    transferencia.getField('VRTRANBANC_INI').filterProperties.defaultValue = 0;
                    transferencia.getField('VRTRANBANC_INI').filterProperties.newLine = true;
                }
                var vrtranbanc_fin = transferencia.dataSourceFilter.filter(function(filtro){
                    return filtro.name == 'VRTRANBANC_FIN';
                });
                if (vrtranbanc_ini.length == 0){
                    transferencia.getField('VRTRANBANC_FIN').filterProperties.defaultValue = 0;
                    transferencia.getField('VRTRANBANC_FIN').filterProperties.newLine = true;
                }
                transferencia.dataSourceFilter.forEach(function(filtro){
                    switch(filtro.name){
                        case 'VRTODOS': filtro.value = 'T';
                            break;
                        case 'THISTORICO': filtro.value = 'T';
                            break;
                    }
                });
                transferencia.reload();
            })
            .catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
    };

    self.setExtratoContaCorrente = function(extrato){
        var deferred = ZHPromise.defer();
        var widgetMovimentacao = templateManager.container.getWidget('movimentacaoContaCorrente');
        extrato.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDREDUCONT': filtro.value = (widgetMovimentacao.currentRow.CDREDUCONT) ? widgetMovimentacao.currentRow.CDREDUCONT : null;
                    break;
                default: break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };


    self.iniciaExtratoFiltro = function(extrato){
        var dtmovbanc = extrato.dataSourceFilter.filter(function(filtro){
            return filtro.name == 'DTMOVBANC';
        });
        if (dtmovbanc.length == 0){
            var start = moment().subtract(1, 'Month').format('DD/MM/YYYY');
            var end   = moment().format('DD/MM/YYYY');
            var periodoInicial = {
                "name": "DTMOVBANC",
                "value": [start, end],
                "operator": "BETWEEN",
                "static": true
            };
            extrato.dataSourceFilter.push(periodoInicial);
            extrato.getField('DTMOVBANC').filterProperties.defaultValue = {
                start: start,
                end: end
            };
            extrato.getField('DTMOVBANC').filterProperties.newLine = true;
        }
        extrato.reload();
    };

    self.setLancamentoContaCorrente = function(lancamento){
        var deferred = ZHPromise.defer();
        var widgetMovimentacao = templateManager.container.getWidget('movimentacaoContaCorrente');
        lancamento.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDREDUCONT': filtro.value = widgetMovimentacao.currentRow.CDREDUCONT;
                    break;
                default: break;
            }
        });
        deferred.resolve();
        return deferred.promise;
    };


};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_movimentacaoContaCorrente_Controller', Bco11100_movimentacaoContaCorrente_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco11100_transferenciaContaCorrente_Controller.js
var Bco11100_transferenciaContaCorrente_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, contabilizacaoController, Bco11100_movimentacaoContaCorrente_Service, ZhFilterPreferences){
    var self = this;

    self.consultarContabilizacaoTransferencia = function(){
        var widget = templateManager.container.getWidget('transferenciaContaCorrente');
        if (widget.currentRow.NRLANCCONT){
            var contabil = [];
            contabil.push({});
            contabil[0].CDEMPRESA  = widget.currentRow.CDEMPRESA;
            contabil[0].NRLANCCONT = widget.currentRow.NRLCTBTRAN;
            contabilizacaoController.exibeContabilizacao(contabil);
        }
    };
    
    self.consultaFiltroTransferencia = function(widget){
        if(widget.appliedFilter){
            if(widget.appliedFilter.FILTER_ROW.CDEMPRESA){
                self.adicionaTransferencia(widget);
            }else{
                ScreenService.showMessage("Filtre uma empresa para continuar");
            }
        }else{
            ScreenService.showMessage("Filtre uma empresa para continuar");
        }
    };

    self.openFilter = function(transferencia){
        self.iniciaTransferenciaFiltro(transferencia)
        .then(function(filterRow){
            self.buscaPadraoEmpresa(filterRow).then(function(filterRow){
                self.abrirFiltro(transferencia, filterRow);
            });
        });
    };

    self.abrirFiltro = function(transferencia, filterRow){
        ZhFilterPreferences.setOriginalFilter(transferencia.id, filterRow);
        ZhFilterPreferences.openFilterWidget(transferencia, filterRow);
    };

    self.buscaPadraoEmpresa = function(filterRow) {
        var deferred = ZHPromise.defer();
        var itens = templateManager.container.getWidget('itensTransferencia');
        var transferencia = templateManager.container.getWidget('transferenciaContaCorrente');
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial)
        .then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                filterRow.CDEMPRESA = empresa.CDEMPRESA;
                filterRow.NMEMPRESA = empresa.NMEMPRESA;
                transferencia.getField('ORIGEM').dataSourceFilter[0].value = cdempresa;
                transferencia.getField('DESTINO').dataSourceFilter[0].value = cdempresa;
                itens.getField('CDREDUCONT').dataSourceFilter[0].value = cdempresa;
                deferred.resolve(filterRow);
            });
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

    self.preparaFiltroTransferencia = function(transferencia, FILTER_ROW){
        var cdempresa = FILTER_ROW.CDEMPRESA;
        var valorInicial = FILTER_ROW.VRTRANBANC_INI;
        var valorFinal = FILTER_ROW.VRTRANBANC_FIN;
        var data = FILTER_ROW.DTTRANBANC;
        var dataIni = data.start;
        var dataFin = data.end;
        var contaOrigem = FILTER_ROW.CONTAORIGEM;
        var contaDestino = FILTER_ROW.CONTADESTINO;
        var historico = FILTER_ROW.HISTORICO;
        transferencia.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDEMPRESA': filtro.value = (cdempresa) ? cdempresa : null;
                    break;
                case 'VRTODOS': filtro.value = ((valorInicial == 0) && (valorFinal == 0)) ? 'T' : null;
                    break;
                case 'THISTORICO': filtro.value = (historico) ? null : 'T';
                    break;
                case 'CONTAORIGEM': filtro.value = (contaOrigem) ? contaOrigem : null;
                    break;
                case 'TCONTAORIGEM': filtro.value = (contaOrigem) ? null : 'T';
                    break;
                case 'CONTADESTINO': filtro.value = (contaDestino) ? contaDestino : null;
                    break;
                case 'TCONTADESTINO': filtro.value = (contaDestino) ? null : 'T';
                    break;
                case 'DATAINI': filtro.value = (dataIni) ? dataIni : null;
                    break;
                case 'DATAFIN': filtro.value = (dataFin) ? dataFin : null;
                    break;
                case 'DTTODOS': filtro.value = (!dataIni || !dataFin) ? 'T' : null;
                    break;
            }
        });
    };


    self.habilitaContaCorrente = function(filtro){
        var transferenciaContaCorrente = templateManager.container.getWidget('transferenciaContaCorrente');
        if (filtro.currentRow.NMEMPRESA) {
            transferenciaContaCorrente.getField('ORIGEM').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('ORIGEM').readOnly = false;
            filtro.getField('DESTINO').dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
            filtro.getField('DESTINO').readOnly = false;
        }else {
            filtro.getField('ORIGEM').dataSourceFilter[0].value = null;
            filtro.getField('ORIGEM').readOnly = true;
            filtro.currentRow.ORIGEM = null;
            filtro.getField('DESTINO').dataSourceFilter[0].value = null;
            filtro.getField('DESTINO').readOnly = true;
            filtro.currentRow.DESTINO = null;
        }
    };

    self.validaStatusAction = function(){

        var itens = templateManager.container.getWidget('itensTransferencia');
        if (itens.mode == 'edit'){
            self.setGridNaoClicavel();
            itens.getAction('EDIT').isVisible = true;
        }else{
            itens.getAction('EDIT').isVisible = false;
        }
    };

    self.setGridNaoClicavel = function(){
        /*var swipe = document.getElementsByClassName('zh-swipe-inner zh-content ng-scope open');
        var div = document.createElement('div');
        div.setAttribute('class', 'swipe-overlay ng-scope');
        div.setAttribute('data-ng-if', 'swipe.editing || swipe.currentWidget.editing');
        swipe[0].appendChild(div);*/
    };

    self.cancelaGridNaoClicavel = function(){
        /*var swipe = document.getElementsByClassName('zh-swipe-inner zh-content ng-scope open');
        var divFilha = document.getElementsByClassName('swipe-overlay ng-scope');
        swipe[0].removeChild(divFilha[divFilha.length - 1]);*/
    };

    self.setTransferenciaContaCorrente = function(transferencia){
        var deferred = ZHPromise.defer();
        //var widgetMovimentacao = templateManager.container.getWidget('movimentacaoContaCorrente');
        transferencia.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDEMPRESA': filtro.value = widgetMovimentacao.appliedFilter.FILTER_ROW.CDEMPRESA;
                    break;
                /*case 'CDREDUCONT': filtro.value = (widgetMovimentacao.currentRow.CDREDUCONT) ? widgetMovimentacao.currentRow.CDREDUCONT : null;
                    break;*/
                default: break;
            }
        });
        transferencia.reload().then(function(){
            deferred.resolve();
        });
        return deferred.promise;
    };

    self.iniciaTransferenciaFiltro = function(transferencia){
        var deferred = ZHPromise.defer();
        var route = 'fin#/crud/geral_bco';
        var rowFilter = {};
        rowFilter.VRTRANBANC_INI = 0;
        rowFilter.VRTRANBANC_FIN = 0;
        rowFilter.DEBCRED = 'T';
        General.callRestEngine(route, 'FilterData', [])
            .then(function(response){
                var dttranbanc = transferencia.dataSourceFilter.filter(function(filtro){
                    return filtro.name == 'DTTRANBANC';
                });
                if (dttranbanc.length == 0){
                    var diasAnt  = response.dataset.geral_bco[0].QTDIAANTTRAN;
                    var diasPos  = response.dataset.geral_bco[0].QTDIAAPOTRAN;
                    var start = moment().subtract(diasAnt, 'days').format('DD/MM/YYYY');
                    var end   = moment().add(diasPos, 'days').format('DD/MM/YYYY');
                    var periodoInicial = {
                        "name": "DTTRANBANC",
                        "value": [start, end],
                        "operator": "BETWEEN",
                        "isCustomFilter": true
                    };
                    transferencia.dataSourceFilter.push(periodoInicial);
                    rowFilter.DTTRANBANC = {start: start, end: end};
                }
                transferencia.dataSourceFilter.forEach(function(filtro){
                    switch(filtro.name){
                        case 'VRTODOS': filtro.value = 'T';
                            break;
                        case 'THISTORICO': filtro.value = 'T';
                            break;
                    }
                });
                deferred.resolve(rowFilter);
            })
            .catch(function(erro){
                deferred.resolve(rowFilter);
            });
        return deferred.promise;
    };

    self.adicionaTransferencia = function(transferencia){
        if(transferencia.appliedFilter){
            if(transferencia.appliedFilter.FILTER_ROW.CDEMPRESA){
                var tabbed_itens = templateManager.container.getWidget('tabbed_itensTransferencia');
                ScreenService.openSwipe(tabbed_itens).then(function(){
                    var itens = templateManager.container.getWidget('itensTransferencia');
                    itens.dataSource.data = [];
                    itens.getAction('ADD').isVisible = true;
                    itens.getAction('SAVE').isVisible = false;
                    itens.getAction('DELETE').isVisible = false;
                    itens.getAction('BACK_LIST').isVisible = false;
                    itens.getAction('CANCEL_LIST').isVisible = true;
                    itens.edit();
                    tabbed_itens.currentRow.__is_new = true;
                    self.habilitaFieldsTransferencia(tabbed_itens);
                    tabbed_itens.getField('DTTRANBANC').setValue('');
                    tabbed_itens.getField('VRTRANBANC').setValue('');
                    tabbed_itens.getField('DSHISTTBANC').setValue('');
                    itens.getField('CDREDUCONT').dataSourceFilter[0].value = transferencia.appliedFilter.FILTER_ROW.CDEMPRESA;
                    transferencia.currentRow.IDPERTRANSCCOR = self.requestGeralBco();
                    //self.validaEstadoTabItem(transferencia);
                });
            }else ScreenService.showMessage("Operação bloqueada. Selecione a Empresa no filtro.");
        }else ScreenService.showMessage("Operação bloqueada. Selecione a Empresa no filtro.");

    };

    self.requestGeralBco = function(){
        var route = 'fin#/crud/geral_bco';
        var requestType = {'requestType':'FilterData', "filter": []};
        restEngine.post(route, requestType).then(function(response) {
            return response.dataset.geral_bco[0].IDPERTRANSCCOR;
        });
    };

    self.validaInclusaoContaTransferencia = function(itens){
        var transferencia = templateManager.container.getWidget('transferenciaContaCorrente');
        self.contaJaInformada(itens).then(function(){
            if (transferencia.currentRow.IDPERTRANSCCOR == 'N') {
                var route = 'fin_movimentacaoBancaria#/validaAcessoOperador';
                var row = itens.currentRow;
                General.callRestEngine(route, 'Row', row).then(
                    function(){},
                    function(erro){
                        ScreenService.showMessage(erro.data.error);
                        itens.currentRow.CDREDUCONT = '';
                        itens.currentRow.CDBANCO = '';
                        itens.currentRow.CDAGENCIA = '';
                        itens.currentRow.CDCONTCORR = '';
                        itens.currentRow.DSCONTCORR = '';
                    }
                );
            }
        });
    };

    self.cancelTransferencia = function(){
        var transferencia = templateManager.container.getWidget("transferenciaContaCorrente");
        var itens = templateManager.container.getWidget("itensTransferencia");
        itens.getAction('ADD').isVisible = false;
        itens.getAction('EDIT').isVisible = false;
        itens.getAction('SAVELIST').isVisible = false;
        itens.getAction('DELETE').isVisible = true;
        itens.getAction('BACK_LIST').isVisible = true;
        itens.getAction('CANCEL_LIST').isVisible = false;
        if (transferencia.currentRow.__is_new){
            var indexOfCurrentRow = transferencia.dataSource.data.indexOf(transferencia.currentRow);
            if(indexOfCurrentRow !== -1) {
                transferencia.dataSource.data.pop();
            }
            transferencia.widgets[0].enabled = true;
            transferencia.list();
            ScreenService.closeSwipe();
        } else { //caso a transferencia estiver em modo de edição, recupera o backup e entra no modo de visualização
            Object.keys(transferencia.backupRow).forEach(function(key) {
                transferencia.currentRow[key] = transferencia.backupRow[key];
            }); //copia dados coluna por coluna para não perder referência do objeto da currentRow
            transferencia.view();
        }
    };

    self.editarItensTransferencia = function(itens){
        General.widgetEdit(itens);
        itens.getAction('SAVE').isVisible = true;
        itens.getAction('SAVELIST').isVisible = false;
        itens.getAction('ADD').isVisible = false;
        self.cancelaGridNaoClicavel();
    };

    self.editTransferenciaContaCorrente = function(){
        var tabbed_itens = templateManager.container.getWidget("tabbed_itensTransferencia");
        var itens = templateManager.container.getWidget("itensTransferencia");
        itens.edit();
        self.habilitaFieldsTransferencia(tabbed_itens);
        itens.getAction('EDITTRANSFERENCIA').isVisible = false;
        itens.getAction('SAVE').isVisible = false;
        itens.getAction('ADD').isVisible = true;
        itens.getAction('EDIT').isVisible = true;
        itens.getAction('CANCEL_LIST').isVisible = true;
        var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
        tabbed_itens.backupRow = Util.clone(tabbed_itens.currentRow, ignoredKeys);
        itens.dataSource.dataBackup = Util.clone(itens.dataSource.data);
        tabbed_itens.currentRow.__is_new = false;
        tabbed_itens.currentRow.edit = true;
    };

    self.voltarItens = function(){
        ScreenService.closeSwipe().then(function(){
            templateManager.container.getWidget('transferenciaContaCorrente').activate();
        });
    };

    self.voltarListaItens = function(){
        var tabbed_itens = templateManager.container.getWidget("tabbed_itensTransferencia");
        if (tabbed_itens.currentRow.edit || tabbed_itens.currentRow.__is_new){
            ScreenService.closeSwipe().then(function(){
                templateManager.container.getWidget('itensTransferencia').edit();
                self.cancelaGridNaoClicavel();
            });
        }else{
            ScreenService.closeSwipe();
        }
    };

    self.cancelarItens = function(itens){
        var tabbed_itens = templateManager.container.getWidget("tabbed_itensTransferencia");
        if (tabbed_itens.currentRow.edit){
            if (itens.getAction('ADD').isVisible){
                self.cancelaItensModoList(itens, tabbed_itens);
                Object.keys(tabbed_itens.backupRow).forEach(function(key) {
                    tabbed_itens.currentRow[key] = tabbed_itens.backupRow[key];
                });
                itens.dataSource.data = [];
                itens.dataSource.data = itens.dataSource.dataBackup;
                tabbed_itens.currentRow.edit = false;
            }else{
                if (itens.mode == 'edit' && !itens.currentRow.__is_new){
                    self.cancelaModoEdicao(itens).then(function(){
                        self.validaActionFinalizar(itens);
                        self.setGridNaoClicavel();
                    });
                }else if (itens.mode == 'edit' && itens.currentRow.__is_new){
                    ScreenService.closeSwipe().then(function(){
                        self.cancelaGridNaoClicavel();
                        itens.dataSource.data.pop();
                        self.validaActionFinalizar(itens);
                        itens.edit();
                        itens.getAction('ADD').isVisible = true;
                        itens.getAction('SAVE').isVisible = false;
                    });
                }
            }
        }else if (tabbed_itens.currentRow.__is_new){
            if (itens.getAction('ADD').isVisible){
                self.cancelaItensModoList(itens, tabbed_itens);
                ScreenService.closeSwipe().then(function(){
                    templateManager.container.getWidget('transferenciaContaCorrente').activate();
                    itens.dataSource.data = [];
                });
                tabbed_itens.currentRow.__is_new = false;
            }else{
                if (!itens.currentRow.hasOwnProperty('INSERT')){
                    self.cancelaModoEdicao(itens).then(function(){
                        self.validaActionFinalizar(itens);
                        self.setGridNaoClicavel();
                    });
                }else if (itens.currentRow.hasOwnProperty('INSERT')){
                    ScreenService.closeSwipe().then(function(){
                        self.cancelaGridNaoClicavel();
                        itens.dataSource.data.pop();
                        self.validaActionFinalizar(itens);
                        itens.edit();
                        itens.getAction('ADD').isVisible = true;
                        itens.getAction('SAVE').isVisible = false;
                    });
                }
            }
        }
        delete itens.currentRow.INSERT;
    };

    self.cancelaItensModoList = function(itens, tabbed_itens){
        itens.getAction('SAVELIST').isVisible = false;
        itens.getAction('BACK_LIST').isVisible = true;
        itens.getAction('CANCEL_LIST').isVisible = false;
        itens.getAction('ADD').isVisible = false;
        itens.getAction('EDITTRANSFERENCIA').isVisible = true;
        self.desabilitaFieldsTransferencia(tabbed_itens);
        itens.list();
    };

    self.desabilitaFieldsTransferencia = function(tabbed_itens){
        tabbed_itens.getField('DTTRANBANC').template = 'field/span-field.html';
        tabbed_itens.getField('VRTRANBANC').template = 'field/span-field.html';
        tabbed_itens.getField('DSHISTTBANC').template = 'field/span-field.html';
    };

    self.habilitaFieldsTransferencia = function(tabbed_itens){
        tabbed_itens.getField('DTTRANBANC').template = 'field/calendar.html';
        tabbed_itens.getField('VRTRANBANC').template = 'field/text-edit.html';
        tabbed_itens.getField('DSHISTTBANC').template = 'field/text-edit.html';
    };

    self.validaActionFinalizar = function(itens){
        var novoItem = itens.dataSource.data.filter(function(data){
            return data.__is_new;
        });
        if (novoItem.length > 0){
            itens.getAction('SAVELIST').isVisible = true;
        }else{
            itens.getAction('SAVELIST').isVisible = false;
        }
    };

    self.cancelaModoEdicao = function(itens){
        var deferred = ZHPromise.defer();
        itens.view();
        Object.keys(itens.backupRow).forEach(function(key) {
            itens.currentRow[key] = itens.backupRow[key];
        });
        itens.getAction('ADD').isVisible = true;
        itens.getAction('SAVE').isVisible = false;
        deferred.resolve();
        return deferred.promise;
    };

    self.atualizaValoresSelectMultiple = function(widget){
        var fieldsSearch = [];
        fieldsSearch.push(widget.getField('ORIGEM'));
        fieldsSearch.push(widget.getField('DESTINO'));
        fieldsSearch.forEach(function(field) {
            if (field.filterProperties.copyValueSelectMultiple) {
                var valorConcatenado = field.value();
                field.setValue(field.filterProperties.copyValueSelectMultiple);
                field.filterProperties.copyValueSelectMultiple = valorConcatenado;
            }
        });
    };

    self.setDataSourceInterval = function(){
        var widget = templateManager.container.getWidget('transferenciaContaCorrente');
        widget.dataSourceFilter.forEach(function(filtro){
            if (filtro.name.localeCompare('DATAINI') === 0){
                filtro.value = widget.appliedFilter.FILTER_ROW.DTTRANBANC.start;
            }else if (filtro.name.localeCompare('DATAFIN') === 0){
                filtro.value = widget.appliedFilter.FILTER_ROW.DTTRANBANC.end;
            }
        });
    };

    self.validaHistorico = function(widget){
        var valor = (widget.appliedFilter.FILTER_ROW.DSHISMBANC0)? "" : "T";
        widget.dataSourceFilter.forEach(function(filtro) {
            if (filtro.name.localeCompare('THISTORICO') === 0){
                filtro.value = valor;
            }
        });
    };

    self.salvarTransferenciaContaCorrente = function(widget){
        var itens = templateManager.container.getWidget('itensTransferencia');
        var transferencia = templateManager.container.getWidget('tabbed_itensTransferencia');
        var valorTransferencia = parseFloat(transferencia.currentRow.VRTRANBANC.toString().replace('.', '').replace(',', '.'));
        if (itens.getAction('ADD').isVisible) {
            if (self.validaCondicoesParaTransferencia(itens, transferencia)){
                var filtroSaldo = self.preparaFiltroSaldoCC(itens, transferencia);
                var route = 'fin#/crud/sdoccorr';
                General.callRestEngine(route, 'FilterData', filtroSaldo)
                .then(function(response){
                    var contasConcatenadas = self.preparaMensagem(response.dataset.sdoccorr, transferencia);
                    if (contasConcatenadas){
                        ScreenService.confirmMessage('Saldo atual da(s) conta(s) de débito<br>' + contasConcatenadas + ' não comportam o valor da transferencia. Deseja lançar a transferência?', 'question',
                            function (sim) {
                                self.saveTransferencia(transferencia, itens);
                            },
                            function (nao) {
                            }
                        );
                    }else{
                        self.saveTransferencia(transferencia, itens);
                    }
                })
                .catch(function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }
        }else{
            self.salvaLocalmente(itens);
        }
    };

    self.salvaLocalmente = function(itens){
        if (itens.isValid()) {
            if (parseFloat(itens.currentRow.VRMOVITBANC.toString().replace('.', '').replace(',', '.')) != 0.00) {
                ScreenService.goBack().then(function(){
                    if (itens.currentRow.INSERT){
                        //self.cancelaGridNaoClicavel();
                    }
                    itens.edit();
                    delete itens.currentRow.INSERT;
                    itens.getAction('SAVELIST').isVisible = true;
                    itens.getAction('SAVE').isVisible = false;
                    itens.getAction('ADD').isVisible = true;
                });
            }else{
                ScreenService.showMessage('Valor deve ser diferente de zero.');
            }
        }
    };

    self.preparaMensagem = function(saldos, transferencia){
        var vrsaldo = (transferencia.currentRow.__is_new) ? parseFloat(transferencia.currentRow.VRTRANBANC) : parseFloat(transferencia.currentRow.VRTRANBANC) - parseFloat(transferencia.backupRow.VRTRANBANC);
        var contasConcatenadas = '';
        saldos.forEach(function(saldo){
            vrsdoccorr = saldo.VRSDOCCORR;
            if ((vrsdoccorr - vrsaldo) <= 0) {
                vrsdoccorr = (vrsdoccorr == 0) ? "0.00" : vrsdoccorr;
                contasConcatenadas = contasConcatenadas + "Conta: " + saldo.CDREDUCONT + " - Saldo: " + vrsdoccorr + ",<br>";
            }
        });
        return contasConcatenadas;
    };

    self.validaCondicoesParaTransferencia = function(itens, transferencia){
        var soma = 0;
        var vrmovitbanc = 0;
        var vrtranbanc = transferencia.currentRow.VRTRANBANC;
        var valorTransferencia = self.formataValoresDecimais(vrtranbanc);
        itens.dataSource.data.forEach(function(data){
            if (data.IDMOVITBANC == 'D'){
                vrmovitbanc = self.formataValoresDecimais(data.VRMOVITBANC);
                soma = soma + vrmovitbanc;
            }
        });
        var transferenciaValida = true;
        if (!transferencia.isValid() && transferenciaValida) {
            transferenciaValida = false;
        }
        if (valorTransferencia == 0.00 && transferenciaValida) {
            transferenciaValida = false;
            ScreenService.showMessage('Valor da transferência deve ser diferente de zero.');
        }
        if (self.existeDiferencaDebitoCredito(itens) && transferenciaValida) {
            transferenciaValida = false;
            ScreenService.showMessage('Soma dos débitos deve ser igual à soma dos créditos.');
        }
        if (valorTransferencia != soma && transferenciaValida) {
            transferenciaValida = false;
            ScreenService.showMessage('Total dos lançamentos difere do valor total da transferência.');
        }

        return transferenciaValida;
    };

    self.preparaFiltroSaldoCC = function(itens, transferencia){
        var contasDebito = itens.dataSource.data
        .filter(function(f){
            return f.IDMOVITBANC == 'D';
        })
        .map(function(m){
            return m.CDREDUCONT;
        });
        var vrsdoccorr = '';
        var  filtroSaldo = [
            {
                "name": "CDREDUCONT",
                "operator": "IN",
                "value": contasDebito
            },
            {
                "name": "DTSDOCCORR",
                "operator": "=",
                "value": transferencia.currentRow.DTTRANBANC
            }
        ];
        return filtroSaldo;
    };

    self.saveTransferencia = function(transferencia, itens){
        var dataSet = self.montaDataSet(itens, transferencia);
        var route = 'fin_movimentacaoBancaria#/saveTransferenciaSaldo';
        
        localStorage.setItem('lancTransRow',JSON.stringify(dataSet));
        localStorage.setItem('TIPOBAIXA','LANCTRANS');
        
        General.callRestEngine(route, 'DataSet', dataSet).then(function (response) {
                transferencia.currentRow.__is_new = false;
                itens.enabled = true;
                itens.getAction('ADD').isVisible = false;
                itens.getAction('SAVELIST').isVisible = false;
                itens.getAction('DELETE').isVisible = true;
                itens.getAction('BACK_LIST').isVisible = true;
                itens.getAction('CANCEL_LIST').isVisible = false;
                self.desabilitaFieldsTransferencia(transferencia);
                itens.list();
                ScreenService.closeSwipe().then(function(){
                    transferencia.getParent().reload();
                    transferencia.getParent().activate();
                    ScreenService.showMessage('Registro salvo com sucesso.');
                });
            },
            function(erro){
                if (contabilizacaoController.checkErrosContabilizacao(erro)){
                    ScreenService.showMessage(erro.data.error);
                }
            }
        );
    };

    self.setSugestaoValor = function(widget){
        var soma = 0;
        widget.dataSource.data.forEach(function(data){
            soma += (data.VRMOVITBANC == '')? 0 : parseFloat(data.VRMOVITBANC);
        });
        var dif = parseFloat(widget.getParent().currentRow.VRTRANBANC) - soma;
        if (dif >= 0) {
            widget.currentRow.VRMOVITBANC = dif;
        }else{
            widget.currentRow.VRMOVITBANC = 0;
        }
    };

    self.contaJaInformada = function(itens){
        var deferred = ZHPromise.defer();
        var count = 0;
        var difcreddeb = false;
        itens.dataSource.data.forEach(function(item){
            if (item.CDREDUCONT == itens.currentRow.CDREDUCONT){
                count++;
            }
        });
        if (count > 1) {
            ScreenService.showMessage('Conta já informada.').then(function(){
                itens.currentRow.CDREDUCONT = '';
                itens.currentRow.CDBANCO = '';
                itens.currentRow.CDAGENCIA = '';
                itens.currentRow.CDCONTCORR = '';
                itens.currentRow.DSCONTCORR = '';
            });
        }else{
            deferred.resolve();
        }
        return deferred.promise;
    };

    self.existeDiferencaDebitoCredito = function(itens){
        var somaCredito = 0;
        var somaDebito = 0;
        var credito = 0;
        var debito = 0;
        var vrmovitbanc = 0;
        itens.dataSource.data.forEach(function(data){
            vrmovitbanc = self.formataValoresDecimais(data.VRMOVITBANC);
            if (data.IDMOVITBANC == 'C'){
                credito = self.formataValoresDecimais(somaCredito);
                somaCredito = credito + vrmovitbanc;
            }
            if (data.IDMOVITBANC == 'D'){
                debito = self.formataValoresDecimais(somaDebito);
                somaDebito = debito + vrmovitbanc;
            }
        });
        return (somaCredito != somaDebito)? true : false;
    };

    self.formataValoresDecimais = function(valor){
        var valorFormatado = 0;
        valorFormatado = (valor.toString().search(',') != '-1') ?
            ((parseFloat(valor.toString().split(',')[0]) > 0) ? parseFloat(valor.toString().replace('.', '').replace(',', '.')) : parseFloat(valor.toString().replace(',', '.'))) :
            parseFloat(valor);
        return valorFormatado;
    };

    self.deleteTransferenciaContaCorrente = function(){
        var transferencia  = templateManager.container.getWidget('transferenciaContaCorrente');
        var itens = templateManager.container.getWidget('itensTransferencia');
        var route = 'fin_movimentacaoBancaria#/deleteTransferenciaContaCorrente';
        var dataSet = self.geraDataDeleteTransferencia(transferencia, itens);
        //var idpertransccor = self.requestGeralBco();
        ScreenService.confirmMessage(ScreenService.i18n('Delete Register?'), 'question',
            function (success) {
                General.callRestEngine(route, 'DataSet', dataSet).then(function (mensagem) {
                    transferencia.reload();
                    ScreenService.closeSwipe().then(function () {
                        transferencia.activate();
                        ScreenService.showMessage(mensagem.messages[0].message);
                    });
                },
                function(erro){
                    ScreenService.showMessage(erro.data.error);
                });
            }
        );
    };

    self.geraDataDeleteTransferencia = function(transferencia, itens){
        var dataSet = [];
        dataSet[0] = {};
        dataSet[0].CDEMPRESA = transferencia.currentRow.CDEMPRESA;
        dataSet[0].VRMOVITBANC = transferencia.currentRow.VRTRANBANC;
        dataSet[0].NRORG = transferencia.currentRow.NRORG;
        dataSet[0].NRTRANBANC = transferencia.currentRow.NRTRANBANC;
        dataSet[0].NRLCTBTRAN = transferencia.currentRow.NRLCTBTRAN;
        dataSet[0].CDFILIAL = transferencia.currentRow.CDFILIAL;
        dataSet[0].CDCLASSFINA = transferencia.currentRow.CDCLASSFINA;
        var itensTransferencia = itens.dataSource.data;
        for(var i = 0; i < itensTransferencia.length; i++){
            dataSet[i + 1]={};
            dataSet[i + 1].CDREDUCONT = itensTransferencia[i].CDREDUCONT;
            dataSet[i + 1].CDEMPRESA = transferencia.currentRow.CDEMPRESA;
            dataSet[i + 1].NRMOVBANC = itensTransferencia[i].NRMOVBANC;
            dataSet[i + 1].VRMOVITBANC = itensTransferencia[i].VRMOVITBANC;
            dataSet[i + 1].NRORG = itensTransferencia[i].NRORG;
            dataSet[i + 1].NRTRANBANC = itensTransferencia[i].NRTRANBANC;
            dataSet[i + 1].IDMOVBANC = itensTransferencia[i].IDMOVITBANC;
        }
        dataSet.forEach(function(data){
            data.DTSDOCCORR = transferencia.currentRow.DTTRANBANC.substr(0, 10);
        });
        return dataSet;
    };

    self.adicionaValorAnteriorDataSet = function(widget){
        widget.dataSource.data.forEach(function(data){
            if (widget.currentRow.CDREDUCONT == data.CDREDUCONT){
                data.VALORANTERIOR = widget.backupRow.VRTRANBANC;
            }
        });
    };

    /**
     * Prepara Row antes de envia-la ao backend.
     * @param itens
     * @param transferencia
     * @returns {Array}
     */
    self.montaDataSet = function(itens, transferencia){
        var historico = transferencia.currentRow.DSHISTTBANC;
        var empresa = transferencia.getParent().appliedFilter.FILTER_ROW.CDEMPRESA;
        var data = transferencia.currentRow.DTTRANBANC;
        var reduzido = transferencia.getParent().appliedFilter.FILTER_ROW.CDREDUCONT;
        var nrtranbanc = transferencia.getParent().currentRow.NRTRANBANC;
        var dataSet = [];
        dataSet[0]={};
        dataSet[0].__is_new = transferencia.currentRow.__is_new;
        dataSet[0].changed = transferencia.currentRow.hasOwnProperty('changed');
        dataSet[0].CDREDUCONT=reduzido;
        dataSet[0].VRMOVITBANC=self.formataValoresDecimais(transferencia.currentRow.VRTRANBANC);
        if (!transferencia.currentRow.__is_new) {
            dataSet[0].DIFERENCASALDO = transferencia.backupRow.VRTRANBANC - transferencia.currentRow.VRTRANBANC;
        }
        dataSet[0].DSHISTTBANC = historico;
        dataSet[0].CDEMPRESA = empresa;
        dataSet[0].DTTRANBANC = data;
        dataSet[0].NRTRANBANC = nrtranbanc;
        dataSet[0].NRMOVBANC = transferencia.currentRow.NRMOVBANC;
        var creditos = itens.dataSource.data;
        for(var i = 0; i < creditos.length; i++){
            dataSet[i + 1]={};
            dataSet[i + 1].__is_new = creditos[i].__is_new;
            dataSet[i + 1].changed = creditos[i].hasOwnProperty('changed');
            dataSet[i + 1].CDREDUCONT=creditos[i].CDREDUCONT;
            dataSet[i + 1].VRMOVITBANC=self.formataValoresDecimais(creditos[i].VRMOVITBANC);
            if (!transferencia.currentRow.__is_new) {
                dataSet[i + 1].DIFERENCASALDO = creditos[i].VRMOVITBANC - creditos[i].SALDOANTERIOR;
            }
            dataSet[i + 1].IDMOVBANC=creditos[i].IDMOVITBANC;
            dataSet[i + 1].DSHISTTBANC = historico;
            dataSet[i + 1].CDEMPRESA = empresa;
            dataSet[i + 1].DTTRANBANC = data;
            dataSet[i + 1].NRTRANBANC = nrtranbanc;
            dataSet[i + 1].NRMOVBANC = creditos[i].NRMOVBANC;
        }
        return dataSet;
    };

    self.openItensTransferencia = function(){
        var tabbed_itens = templateManager.container.getWidget('tabbed_itensTransferencia');
        self.setDadosTransferencia().then(function(){
            ScreenService.openSwipe(tabbed_itens);
        });
    };

    self.setDadosTransferencia = function(){
        var deferred = ZHPromise.defer();
        var transferencia = templateManager.container.getWidget('transferenciaContaCorrente');
        var tabbed_itens = templateManager.container.getWidget('tabbed_itensTransferencia');
        var itens = templateManager.container.getWidget('itensTransferencia');
        itens.dataSourceFilter.forEach(function(filtro){
            if (filtro.name == 'NRTRANBANC'){
                filtro.value = transferencia.currentRow.NRTRANBANC;
            }
            if (filtro.name == 'CDEMPRESA'){
                filtro.value = transferencia.currentRow.CDEMPRESA;
            }
        });
        itens.reload();
        tabbed_itens.currentRow.DTTRANBANC = transferencia.currentRow.DTTRANBANC;
        tabbed_itens.currentRow.VRTRANBANC = transferencia.currentRow.VRTRANBANC;
        tabbed_itens.currentRow.DSHISTTBANC = transferencia.currentRow.DSHISTTBANC;
        deferred.resolve();
        return deferred.promise;
    };

    self.adicionarConta = function(itens){
        itens.newRow();
        //ScreenService.openSwipe(itens).then(function(){
            itens.currentRow.INSERT = true;
            itens.getAction('ADD').isVisible = false;
            itens.getAction('SAVE').isVisible = true;
            itens.getAction('SAVELIST').isVisible = false;
        //});
    };

    self.habilitaSaveList = function(){
        var itens = templateManager.container.getWidget('itensTransferencia');
        if (itens.getParent().currentRow.edit)
            itens.getAction('SAVELIST').isVisible = true;
    };

    self.concatenaCamposSelectMultiple = function(widget, fields, codigos){
        var field = null;
        for (i = 0; i < fields.length; i++) {
            field = widget.appliedFilter.getField(fields[i]);
            self.setValorConcatenado(widget, field, field.name, codigos[i], 'T'+field.name);
        }
    };

};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco11100_transferenciaContaCorrente_Controller', Bco11100_transferenciaContaCorrente_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco21200_RelacaoSaldosContaCorrente_Controller.js
var Bco21200_RelacaoSaldosContaCorrente_Controller = function(Bco21200_RelacaoSaldosContaCorrente_Service) {
  
  this.updateContCorrFilter = function(filter) {
    filter.currentRow.CDREDUCONT = [];
    filter.currentRow.CDCONTCORRMASK = [];
    
    if (filter.currentRow.CDEMPRESA instanceof Array)
      if (filter.currentRow.CDEMPRESA.length == 0)
        filter.getField('CDREDUCONT').dataSourceFilter[0].value = 'T';
      else filter.getField('CDREDUCONT').dataSourceFilter[0].value = filter.currentRow.CDEMPRESA.join(" ");
    else filter.getField('CDREDUCONT').dataSourceFilter[0].value = 'T';
  };
  
  this.formatFilterRelacaoSaldo = function(widget, preference) {
    var currentFilter    = preference.FILTER_ROW;
    var dataSourceFilter = Bco21200_RelacaoSaldosContaCorrente_Service.buildDataSourceFilterFromFilterRow(currentFilter);
    
    widget.dataSourceFilter = dataSourceFilter;
    
    widget.dataSourceFilter.push(
        {
          "name": "CDOPERADOR",
          "operator": "=",
          "value": getLocalVar('CDOPERADOR')
        }
    );
    
    preference.FILTER_ROW = dataSourceFilter;
    
  };
  
};

Configuration(function (ContextRegister){
    ContextRegister.register('Bco21200_RelacaoSaldosContaCorrente_Controller', Bco21200_RelacaoSaldosContaCorrente_Controller);
});


// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco33001_RelatorioGestaoFluxoCaixa_Controller.js
var bco33001_RelatorioGestaoFluxoCaixa_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, ZhFilterPreferences){

    this.openFilterRelatorio = function(widget) {
	    ZhFilterPreferences.openFilterWidget(widget);
	    
	};
	
	this.setFilterFluxoCaixa = function(preference){
        var relatorioGestaoFluxoCaixaGrid = templateManager.container.getWidget('relatorioGestaoFluxoCaixaGrid');
        var filtro = preference.FILTER_ROW;
        
		var cdempresa   = filtro.CDEMPRESA;
		var datainicial = filtro.DTENTRSAID.start;
		var datafinal   = filtro.DTENTRSAID.end;
		var cdreducont  = filtro.hasOwnProperty('CDREDUCONT')  ? filtro.CDREDUCONT.join('_')  : null;
		var cdcliente   = filtro.hasOwnProperty('CDCLIENTE')   ? filtro.CDCLIENTE.join('_')   : null;
		var cdtpctreceb = filtro.hasOwnProperty('CDTPCTRECEB') ? filtro.CDTPCTRECEB.join('_') : null;
		var cdforneced  = filtro.hasOwnProperty('CDFORNECED')  ? filtro.CDFORNECED.join('_')  : null;
		var cdtpctpagar = filtro.hasOwnProperty('CDTPCTPAGAR') ? filtro.CDTPCTPAGAR.join('_') : null;
		var idstatb     = filtro.hasOwnProperty('IDSTATB')     ? filtro.IDSTATB.join('_')     : null;
        
        relatorioGestaoFluxoCaixaGrid.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA'   : filter.value = (cdempresa)   ? cdempresa   : 'T';
                    break;
                case 'DTINICIAL'   : filter.value = (datainicial) ? datainicial : null;
                    break;
                case 'DTFINAL'     : filter.value = (datafinal)   ? datafinal   : null;
                    break;
                case 'CDREDUCONT'  : filter.value = (cdreducont)  ? cdreducont  : 'T';
                    break;
                case 'CDCLIENTE'   : filter.value = (cdcliente)   ? cdcliente   : 'T';
                    break;
                case 'CDTPCTRECEB' : filter.value = (cdtpctreceb) ? cdtpctreceb : 'T';
                    break;
                case 'CDFORNECED'  : filter.value = (cdforneced)  ? cdforneced  : 'T';
                    break;
                case 'CDTPCTPAGAR' : filter.value = (cdtpctpagar) ? cdtpctpagar : 'T';
                    break;
                case 'IDSTATB'     : filter.value = (idstatb)     ? idstatb     : 'T';
                    break;
            }
        });
        
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        
        delete preference.CUSTOM_FILTER_ROW.CDEMPRESA;
        delete preference.CUSTOM_FILTER_ROW.NMEMPRESA;
        delete preference.CUSTOM_FILTER_ROW.DTENTRSAID;
        delete preference.CUSTOM_FILTER_ROW.CDREDUCONT;
        delete preference.CUSTOM_FILTER_ROW.DSCONTCORR;
        delete preference.CUSTOM_FILTER_ROW.CDCLIENTE;
        delete preference.CUSTOM_FILTER_ROW.CDTPCTRECEB;
        delete preference.CUSTOM_FILTER_ROW.CDFORNECED;
        delete preference.CUSTOM_FILTER_ROW.CDTPCTPAGAR;
        delete preference.CUSTOM_FILTER_ROW.IDSTATB;
        
        
        var saldoContaCorrenteGrid = templateManager.container.getWidget('saldoContaCorrenteGrid');
        
        saldoContaCorrenteGrid.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDREDUCONT'  : filter.value = (cdreducont)  ? cdreducont  : 'T';
                    break;
            }
        });
        
        
        var titulosReceberVencidosGrid = templateManager.container.getWidget('titulosReceberVencidosGrid');
        
        titulosReceberVencidosGrid.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA'   : filter.value = (cdempresa)   ? cdempresa   : 'T';
                    break;
            }
        });
    };
	
	this.setStripeRelatorio = function(widget, preference){
        var stripe       = templateManager.container.getWidget('stripe');
        
        stripe.currentRow.EMPRESA    = preference.FILTER_ROW.CDEMPRESA + ' - ' + preference.FILTER_ROW.NMEMPRESA;
        stripe.currentRow.DTINICIAL  = preference.FILTER_ROW.DTENTRSAID.start;
        stripe.currentRow.DTFINAL    = preference.FILTER_ROW.DTENTRSAID.end;
        stripe.currentRow.CDBANCO    = preference.FILTER_ROW.CDBANCO;
        stripe.currentRow.CDAGENCIA  = preference.FILTER_ROW.CDAGENCIAMASK;
        stripe.currentRow.CDCONTCORR = preference.FILTER_ROW.CDCONTCORRMASK;
        stripe.currentRow.DSCONTCORR = preference.FILTER_ROW.DSCONTCORR;
    };
    
    this.calcSaldoContas = function(filtro){
        var soma = 0;
        
        filtro.VRSDOCCORR.forEach(function(data){
            soma = parseFloat(parseFloat(parseFloat(soma).toFixed(2)) + parseFloat(parseFloat(data).toFixed(2))).toFixed(2);
        });
        
        return soma;
    };
	
    this.setMascCnpjCpf = function(widget) {
	    var data = widget.dataSource.data;
	    var inscFormatada;
	    
	    data.forEach(function(value){
	        if(value.NRINSJURFORNCLIE){
    	        if(value.NRINSJURFORNCLIE.length === 14)      inscFormatada = value.NRINSJURFORNCLIE.substr(0, 2) + '.' + value.NRINSJURFORNCLIE.substr(2, 3) + '.' + value.NRINSJURFORNCLIE.substr(5, 3) + '/' + value.NRINSJURFORNCLIE.substr(8, 4) + '-' + value.NRINSJURFORNCLIE.substr(12, 2);
                else if(value.NRINSJURFORNCLIE.length === 11) inscFormatada = value.NRINSJURFORNCLIE.substr(0, 3) + '.' + value.NRINSJURFORNCLIE.substr(3, 3) + '.' + value.NRINSJURFORNCLIE.substr(6, 3) + '-' + value.NRINSJURFORNCLIE.substr(9, 2);
                else inscFormatada = null;
	        }else inscFormatada = null;
            
            value.NRINSJURFORNCLIE = inscFormatada?inscFormatada:value.NRINSJURFORNCLIE;
	    });
	};
	
    this.changeEmpresa = function(args) {
        var widget = args.owner.field.widget;
	    var cdempresa  = widget.currentRow.CDEMPRESA;
        var cdreducont = widget.getField('CDREDUCONT');
        
	    
	    if(cdreducont.dataSourceFilter[0].value != cdempresa){
            cdreducont.dataSourceFilter[0].value = cdempresa;
            cdreducont.setValue("");
            cdreducont.readOnly = cdempresa ? false : true;
            
            cdreducont.reload();
            templateManager.updateTemplate();
	    }
    };
    
       
};
    
    


Configuration(function (ContextRegister){
    ContextRegister.register('Bco33001_RelatorioGestaoFluxoCaixa_Controller', bco33001_RelatorioGestaoFluxoCaixa_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco51100_TipodeDebitoBancario_Controller.js
var Bco51100_TipodeDebitoBancario_Controller = function(ZhFilterPreferences,ScreenService, templateManager,Bco51100_TipodeDebitoBancario_Service){

    this.findCompleteLevel = function (widget) {
        var verificaNivel = widget.getField('CDPICTCCTB').dataSource.data[0].CDPICTCCTB;
        verificaNivel = verificaNivel.split('.');
        verificaNivel = verificaNivel.length;
        widget.getField('CDCONTCTBL').dataSourceFilter[0].value = verificaNivel;
        widget.getField('CDCONTCTBL').reload();
    };
    this.codeAlreadyRegistered = function (widget) {
        Bco51100_TipodeDebitoBancario_Service.validarCodigo(widget.currentRow).then(
            null,
            function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
			}
        );
    };
    
    this.filter = function(widget){
        if(widget.appliedFilter){
            widget.appliedFilter.reload();
        }  
        ZhFilterPreferences.openFilterWidget(widget);
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco51100_TipodeDebitoBancario_Controller', Bco51100_TipodeDebitoBancario_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco51200_TipoCreditoBancario_Controller.js
var Bco51200_TipoDeCreditoBancario_Controller = function(ZhFilterPreferences,ScreenService,Bco51200_TipoDeCreditoBancario_Service){

    this.findCompleteLevel = function (widget) {
        var verificaNivel = widget.getField('CDPICTCCTB').dataSource.data[0].CDPICTCCTB;
        verificaNivel = verificaNivel.split('.');
        verificaNivel = verificaNivel.length;
        widget.getField('CDCONTCTBL').dataSourceFilter[0].value = verificaNivel;
        widget.getField('CDCONTCTBL').reload();
    };
    
    this.codeAlreadyRegistered = function (widget) {
        Bco51200_TipoDeCreditoBancario_Service.validarCodigo(widget.currentRow).then(
            null,
            function (error) {
				var errorMessage = 'Timeout';
				if (error.data) {
					errorMessage = error.data.error;
				}
				ScreenService.showMessage(errorMessage);
				widget.currentRow.CDTPCREDBANC = "";
			}
        );
    };
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco51200_TipoDeCreditoBancario_Controller', Bco51200_TipoDeCreditoBancario_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco53100_ParametrosControleBancario_Controller.js
var Bco53100_ParametrosControleBancario_Controller = function(ScreenService, General, templateManager, Bco53100_ParametrosDeControleBancario_Repository){
    var widget;
    var checa = true;
    
    this.validaGeralBcoPopulada = function(){
        var route = 'fin_movimentacaoBancaria#/validaGeralBcoPopulada';
        General.callRestEngine(route, 'Row', {}).then(function(){
            if(checa){
                widget = templateManager.container.getWidget('bco53100_parametros_controle_bancario_form');
                Bco53100_ParametrosDeControleBancario_Repository.consultaGeralBco().then(function(consulta){
                    var resultado = consulta.dataset.geral_bco[0];
                    if (resultado){
                        widget.currentRow.CDGERALBCO = resultado.CDGERALBCO;
                        this.construct(resultado, widget);
                        widget.currentRow.NRORG = resultado.NRORG;
                    }
                    this.setZero(widget.getField('QTDIAANTLANC'));
                    this.setZero(widget.getField('QTDIAAPOLANC'));
                    this.setZero(widget.getField('QTDIAANTTRAN'));
                    this.setZero(widget.getField('QTDIAAPOTRAN'));
                    Bco53100_ParametrosDeControleBancario_Repository.consultaParamGeral().then(function(consulta2){
                        var resultado2 = consulta2.dataset.paramgeral[0];
                        if (resultado2){
                            widget.currentRow.IDUTLPROCBCO = resultado2.IDUTLPROCBCO;
                            widget.currentRow.CDGROPERPROB = resultado2.CDGROPERPROB;
                            widget.currentRow.CDPARGERAL = resultado2.CDPARGERAL;
                            widget.currentRow.NRORGPARAM = resultado2.NRORG;
                        }
                        checa = false;
                    });
                }.bind(this));
                Bco53100_ParametrosDeControleBancario_Repository.consultaGeralFcx().then(function(consulta3){
                    var geral_fcx = consulta3.dataset.geral_fcx[0];
                    if (geral_fcx){
                        var cdclassfina = geral_fcx.CDCLASSFINA;
                        widget.currentRow.CDGERALFCX  = geral_fcx.CDGERALFCX;
                        if (cdclassfina){
                            Bco53100_ParametrosDeControleBancario_Repository.consultaClassfinanc(cdclassfina).then(function(consulta4){
                                widget.currentRow.CDCLASSFINA = consulta4.dataset.classfinanc[0].CDCLASSFINA;
                                widget.currentRow.DSCLASSFINA = consulta4.dataset.classfinanc[0].DSCLASSFINA;
                            });
                        }
                    }
                });
            }
        }.bind(this));
    };

    this.construct = function(resultado, widget){
        for (var i=0; i < widget.fields.length; i++) {
            var fieldName = widget.fields[i].name;
            widget.getField(fieldName).setValue(resultado[fieldName]);
        }
    };

    this.salvar = function(){
        if(widget.isValid()){
            widget.currentRow.QTDIAANTLANC = parseInt(widget.currentRow.QTDIAANTLANC);
            widget.currentRow.QTDIAAPOLANC = parseInt(widget.currentRow.QTDIAAPOLANC);
            widget.currentRow.QTDIAANTTRAN = parseInt(widget.currentRow.QTDIAANTTRAN);
            widget.currentRow.QTDIAAPOTRAN = parseInt(widget.currentRow.QTDIAAPOTRAN);
            var data = [
                {
                    __is_new : false,
                    changed: true,
                    IDUTLPROCBCO : widget.currentRow.IDUTLPROCBCO ? widget.currentRow.IDUTLPROCBCO : 'N',
                    CDGROPERPROB : widget.currentRow.CDGROPERPROB,
                    CDPARGERAL : widget.currentRow.CDPARGERAL,
                    NRORG : widget.currentRow.NRORGPARAM
                },
                {
                    __is_new : false,
                    changed: true,
                    CDGERALBCO : widget.currentRow.CDGERALBCO,
                    IDCTBTRANS : widget.currentRow.IDCTBTRANS ? widget.currentRow.IDCTBTRANS : 'N',
                    IDTPIMPRUTL : widget.currentRow.IDTPIMPRUTL ? widget.currentRow.IDTPIMPRUTL : '0',
                    IDTPPORTIMPR : widget.currentRow.IDTPPORTIMPR ? widget.currentRow.IDTPPORTIMPR : null,
                    QTDIAANTLANC : widget.currentRow.QTDIAANTLANC ? widget.currentRow.QTDIAANTLANC : null,
                    QTDIAAPOLANC : widget.currentRow.QTDIAAPOLANC ? widget.currentRow.QTDIAAPOLANC : null,
                    QTDIAANTTRAN : widget.currentRow.QTDIAANTTRAN ? widget.currentRow.QTDIAANTTRAN : null,
                    QTDIAAPOTRAN : widget.currentRow.QTDIAAPOTRAN ? widget.currentRow.QTDIAAPOTRAN : null,
                    IDPERTRANSCCOR : widget.currentRow.IDPERTRANSCCOR ? widget.currentRow.IDPERTRANSCCOR : 'S',
                    CDCLASSFINA : widget.currentRow.CDCLASSFINA ? widget.currentRow.CDCLASSFINA : null,
                    NRORG : widget.currentRow.NRORG ? widget.currentRow.NRORG : null
                },
                {
                    __is_new : false,
                    changed: true,
                    CDGERALFCX : widget.currentRow.CDGERALFCX,
                    CDCLASSFINA : widget.currentRow.CDCLASSFINA ? widget.currentRow.CDCLASSFINA : null,
                    NRORG : widget.currentRow.NRORG ? widget.currentRow.NRORG : null
                }
            ];
            Bco53100_ParametrosDeControleBancario_Repository.salvarGeralBco(data).then(function(){
                widget.template = widget.templateToView;
                widget.view();
                widget.activate();
                widget.reload();
            }, function(error) {
                ScreenService.showMessage(error.data.error);
            });
        }
    };

    this.editar = function(){
        widget.template = widget.templateToForm;
        General.edit(widget);
    };

    this.cancelar = function(){
        for (var i in widget.backupRow) {
            widget.currentRow[i] = widget.backupRow[i];
        }
        widget.template = widget.templateToView;
        widget.view();
    };

    this.setZero = function(valor){
        if (!valor.value()){
            valor.setValue(0);
        }
    };

};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco53100_ParametrosControleBancario_Controller', Bco53100_ParametrosControleBancario_Controller);
});


// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco54200_ExtratoBancario_Controller.js
var Bco54200_ExtratoBancario_Controller = function(ScreenService, General,restEngine, templateManager, Bco54200_ExtratoBancario_Service, ZHPromise, ZhFilterPreferences){
    /**************************************** Exception *****************************************************/

    var self = this;
    
    var data = [];
    function exception(message) {
        this.message = message;
    }

    this.openFilter = function(){
        var extratoBancario = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var data = moment().format('DD/MM/YYYY');
        var filterRow = {};
        filterRow.DTMOVREAL = {start : data, end : data};
        ZhFilterPreferences.setOriginalFilter(extratoBancario.id, filterRow);
        ZhFilterPreferences.openFilterWidget(extratoBancario, filterRow);
    };

    this.validateKeys = function(key, fieldName) {
        var integerNumbers = [48, 49, 50, 51,  52,  53,  54,  55,  56,  57];
        var validKeys = {};

        //build validation cases
        validKeys.integer = integerNumbers;
        validKeys.MSKINSJURFILI = integerNumbers.concat([45, 46, 47]); //'-', '.', '/'

        //check if there is any special validation
        if(fieldName && validKeys.hasOwnProperty(fieldName))
            return validKeys[fieldName].indexOf(key) < 0;
        else //returns generic one
            return validKeys.integer.indexOf(key) < 0;
    };

    /**************************************** Extrato Bancário *************************************************/

    /**
     * Essa função tem como finalidade fazer as validações necessárias  para a exclusão de
     * uma movimentação.
     *
     * @param  {Object} widget widget do extrato bancario (bco54200_extrato_bancario_grid)
     */
    this.delete = function(widget) {
        var currentRow = widget.currentRow;
        
        var param = [{
                'name': 'CDREDUCONT',
                'operator': '=',
                'value': currentRow.CDREDUCONT
            },
            {
                'name': 'CDSEQUEXTR',
                'operator': '=',
                'value': currentRow.CDSEQUEXTR
            },
            {
                'name': 'NRSEQMOVREAL',
                'operator': '=',
                'value': currentRow.NRSEQMOVREAL
            }
        ];
        
        restEngine.post('fin_movimentacaoBancaria#/buscaExtratoImportado', {
            "requestType": "Row",
            "row": param
        }).then((response) => {
            var retorno = response.dataset.conciliacao.conciliacao;
            if (retorno[0].IDSTCONMVBAN === '0') {
                if (retorno[0].IDTPMOVREAL != '1') {
                    ScreenService.confirmMessage(ScreenService.i18n('Atenção! Extrato bancário importado pelo sistema. Deseja excluir?'), 'question', function() {
                        widget.removeRow().then(function() {
                            ScreenService.closeSwipe().then(function() {
                                widget.activate();
                                widget.reload();
                                ScreenService.showMessage('Registro excluído com sucesso.');
                            });
                        }, function(error) {
                            ScreenService.showMessage(error.data.error);
                        });
                    });
                } else {
                    General.deleteCrud(widget).then(function() {
                        ScreenService.showMessage('Registro excluído com sucesso.');
                    });
                }
            } else {
                ScreenService.showMessage('Operação bloqueada! Extrato bancário está conciliado.');
            }
        });
    };

    /**
     * Essa função tem como finalidade fazer as validações necessárias para deixar, ou não,
     * uma movimentação ser editada.
     *
     * @param  {Object} widget widget do extrato bancario (bco54200_extrato_bancario_grid)
     */
    this.editar = function(widget) {
        var currentRow = widget.currentRow;
        if(currentRow.IDSTCONMVBAN != '0'){
            ScreenService.showMessage('Operação bloqueada! Extrato bancário está conciliado.');
        } else if(currentRow.IDTPMOVREAL != '1'){
            ScreenService.showMessage('Operação bloqueada! Extrato bancário foi importado.');
        } else {

            var ignoredKeys = ["__rightActions", "__leftActions", "_children", "_parent", "__groupHeaders", "__index", "__isSelected", "__notSelectable", "$$hashKey"];
            widget.backupRow = Util.clone(widget.currentRow, ignoredKeys);
            widget.edit();
        }
    };

    /**
     * Essa função tem como objetivo chamar as funções que geram o link para o relátorio,
     * abri-lo ou monstrar na tela o erro voltado pela requisição.
     * FUNÇÃO DESATUALIZADA. SUBSTITUIDO PELO DATAEXPORTER DO ZEEDHI
     */
    this.geraReport = function() {
        var widget = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var widgetFilter = widget.appliedFilter;
        Bco54200_ExtratoBancario_Service.geraReport(widgetFilter.currentRow).then(function(birtURL){
            window.open(birtURL.messages[0].message);
        }, function(error) {
            if(error.data){
                ScreenService.showMessage(error.data.error);
            } else {
                ScreenService.showMessage(error.message);
            }
        });
    };

    /**
     * Essa função tem como finalidade adicinar em um field uma validação no seu keypress.
     * Essa validação adiciona so vai permitir que esse field rereba numeros inteiros.
     *
     * @param  {String} fieldName nome do filed que vai receber a regra
     */
    this.interger = function(fieldName) {
        var element = $("#" + fieldName);
        element.keypress(function(e) {
            if(this.validateKeys(e.which, fieldName)){
                e.preventDefault();
            }
        }.bind(this));
    };

    /************************************ Importação de Extrato Bancário ******************************************/

    /**
     * Essa função tem como objetivo verificar o valor do campo de banco da widget de
     * importação de extrato bancario. Quando o valor for alterado para null ou vazio
     * os dados das widgets de importação devem ser apenas zerados. Quando o campo tiver
     * o valor é preciso, além de zerar os valores anteriores da widget, chamar a rotina
     * de leitura de arquivo.
     *
     * @param  {Object} field field do banco
     */
    this.callFuncitionBancoField = function(field) {
        this.clearInfosImport();
        if(field.value() !== null || field.value() !== "") {
            this.uploadContentFile(field.widget);
        }
    };

    /**
     * Essa função tem como finalidade resetar o estado das widgets responsaveis pelo
     * processo de importação de extrato bancario, terem os valores dos campos, que
     * se referem as lidas no arquivo, resetados.
     */
    this.clearInfosImport = function() {
        var widgetExtrato = templateManager.container.getWidget('importacaoExtratoBancario');
        var widgetMovimentacoes = templateManager.container.getWidget('importacaoGrid');

        //Limpando campos de informações sobre o extrato a ser importado
        widgetExtrato.getField('CDCONTCORRMASK').setValue("");
        widgetExtrato.getField('DTSALDINIC').setValue("");
        widgetExtrato.getField('DTSALDFIN').setValue("");
        widgetExtrato.getField('SALDOINICIAL').setValue("");
        widgetExtrato.getField('SALDOFINAL').setValue("");
        widgetExtrato.getField('DTMOVIMENTO').setValue({start: null, end: null,  formattedStart: '__/__/____', formattedEnd: '__/__/____'});

        //Limpando as movimentações do grid
        widgetMovimentacoes.dataSource.data = [];
        widgetMovimentacoes.redraw();
        
    };

    /**
     * Essa função tem como finalidade retornar todas as widgets envolvidas no processo de
     * importação de extrato bancario para a configurar default da widget. Campo de banco
     * desabilitado para edição e sem requerimento; todos os campos limpos; data do grid
     * zerado.
     */
    this.defaultStatusWidget = function() {
        var widget = templateManager.container.getWidget('importacaoExtratoBancario');
        widget.getField('NMBANCO').readOnly = true;
        delete widget.getField('NMBANCO').validations.required;
        widget.currentRow.NMBANCO = null;
        widget.currentRow.CDBANCO = null;
        this.clearInfosImport();
    };

    /**
     * Essa função tem como finalidade deletar uma movimentação (no grid de importação), quando não é mais necessária que
     * aquela movimentação, do extrato importado, não seja salva. Paro isso, as linhas selecionadas são excluidas do data
     * do dataSource da widget (importacaoGrid), e o novo valor final é calculado e colocado na currentRow da widget
     * importacaoExtratoBancario.
     *
     * @param  {Object} widget widget importacaoGrid, que contém as movimentações do extrato que será importado
     */
    this.deletMovImport = function(widget) {
        var checkedRows = widget.dataSource.checkedRows;
        var data = widget.dataSource.data;
        var saldoini = widget.getParent().currentRow.SALDOINICIAL;
        data = data.filter(function(movimentacao){
            return checkedRows.indexOf(movimentacao) === -1;
        });
        var newVrfin = this.recalcVrfinal(data, saldoini);
        widget.dataSource.data = data;
        widget.getParent().currentRow.SALDOFINAL = newVrfin;
    };

    /**
     * Está função é chamado no evento da action (importar) da widget extrato bancário (bco54200_extrato_bancario_grid).
     * Ela torna a widget para importação de extrato (importacaoExtratoBancario) bancário visível e a coloca no modo de
     * edição. Além disso usa tal widget para abrir um swipe, onde sera feita a importação do extrato bancário.
     *
     * @param  {Object} widget widget que sera usada para abrir o swipe (importacaoExtratoBancario)
     */
    this.openSwipeForImport = function() {
        var widgetImportacao = templateManager.container.getWidget('importacaoExtratoBancario');
        var widgetGrid = templateManager.container.getWidget('importacaoGrid');
        
        widgetImportacao.currentRow = {};
        widgetImportacao.currentRow.__is_new = true;
        
        ScreenService.openSwipe(widgetImportacao).then(function(){
            self.viewMode(widgetGrid);
			
			widgetImportacao.templateToForm = "widget/tabbedWidget.html";
			widgetImportacao.templateToView = "widget/tabbedWidget.html";
			widgetImportacao.formTemplate = "widget/tabbedWidget.html";
			
			widgetGrid.dataSource.data = [];
			setTimeout(function(){ 
			
    			widgetGrid.list();
    			widgetImportacao.childTemplate = "widget/form.html";
    			templateManager.updateTemplate(); widgetGrid.activate();
    			
    			$($('section.zh-widget.ng-scope')[1]).children().css('top', (parseInt($('section.zh-field-group.ng-scope').css('height').replace('px', '')) + 40 ) + 'px');
    			
    			$(window).resize();
			
			}, 1000);
		});
    };
    
    this.viewMode = function (widget) {
        widget.view();
        templateManager.updateTemplate();
    };
    
    /* Função antiga para abrir a importação
    
    this.openSwipeForImport = function() {
        var widgetImport = templateManager.container.getWidget('importacaoExtratoBancario');
        widgetImport.isVisible = true;
        ScreenService.openSwipe(widgetImport).then(function(){
            widgetImport.editing = true;
            //widgetImport.widgets[0].list();
        }.bind(this));
    };*/

    /**defaultStatusWidget
     * Essa função é chamada no evneto da action (cancelar) da widget grid de importação da extrato bancário (importacaoGrid).
     * Ela fecha todos os swipes abertos e torna a widget (bco54200_extrato_bancario_grid) de extrato bancário ativa.
     *
     * @param  {String} widget widget pai da widget importacao de extrato bancário (bco54200_extrato_bancario_grid)
     */
    this.backToExtrato = function() {
        var tabbed = templateManager.container.getWidget('importacaoExtratoBancario');
        ScreenService.goBack();
        ScreenService.closeSwipe().then(function(){
            tabbed.isVisible = false;
            this.defaultStatusWidget();
            //tabbed.list();
            //tabbed.widgets[0].list();
            tabbed.getParent().activate();
        }.bind(this));
    };

    /**
     * Essa função te como objetivo, receber um data de um widget e recalcular o saldo final do extrato. Para isso é calcular
     * a soma dos valores de cada movimentação passa e subitrair do saldo inicial do extrato
     *
     * @param  {Array} data  objetos com os dados para semre recalculados
     * @param  {String} saldo valor com o saldo inicial do extrato
     */
    this.recalcVrfinal = function(data, saldo) {
        var saldoMovimentacao = 0;
        data.forEach(function(movimentacao) {
            if (movimentacao.IDMOVREAL === 'D') { // D = debito, é preciso passar o valor para < 0
                saldoMovimentacao += (movimentacao.VRMOVREAL * (-1));
            } else if (movimentacao.IDMOVREAL === 'C'){ // C = crédito, é preciso apenas somar
                saldoMovimentacao += movimentacao.VRMOVREAL;
            }
        });
        var newSaldo = saldo + saldoMovimentacao;
        return newSaldo;
    };

    this.setCdreducont = function(extrato, conta, widget, tipo) {
        var movis = extrato.movreal;
        movis.forEach(function(movi){
            movi.CDREDUCONT = movi.CDREDUCONT[conta[1]];
        });
        this.setDataOfGrid(widget, extrato, tipo);
    };

    /**
     * Essa função manda todos o dados do data do dataSource da widget de grid de importação de extrato bancario, tratando
     * todos os erros.
     *
     * @param  {Object} widget widget grid de importação de extrat bancário
     */
    this.saveImport = function(widget) {
        if(widget.getParent().getField('UPLOADFILE').value()) {
            var data = widget.dataSource.data;
            var container = templateManager.container.getWidget('importacaoExtratoBancario');
            var empresa = container.getField('NMEMPRESA');
            if (data.length !== 0) {
                Bco54200_ExtratoBancario_Service.saveImport(data).then(
                    function(response){
                        //Limpa os dados da widget, para possibilitar uma nova importação
                        this.defaultStatusWidget();
                        widget.getParent().getField('UPLOADFILE').fileList[0].name = '';
                        ScreenService.closeSwipe().then(function(){
                            //Recarrega a widget de movimentações
                            widget.getParent().getParent().reload().then(function(){
                                if (response.dataset.MOVDUPLICADOS.length > 0){
                                    var movimentosDuplicados = templateManager.container.getWidget('movimentosDuplicados');
                                    movimentosDuplicados.dataSource.data = response.dataset.MOVDUPLICADOS;
                                    ScreenService.confirmMessage(ScreenService.i18n('Existem movimentações com o mesmo número, valor e data. Deseja realizar a importação?'),'question',function(){
                                        ScreenService.openPopup(movimentosDuplicados);
                                    }, function(){
                                       templateManager.container.getWidget('bco54200_extrato_bancario_grid').reload();
                                       ScreenService.closePopup(); 
                                    });
                                }else{
                                    ScreenService.showMessage('Importação realizada com sucesso.');
                                }
                            });
                            widget.getParent().getParent().activate();
                            widget.getParent().isVisible = false;
                        });

                    }.bind(this),
                    function(error){
                        ScreenService.showMessage(error.data.error);
                    }
                );
            } else {
                ScreenService.showMessage('Operação bloqueada. Não há dado(s) a ser(em) importado(s).');
            }
        } else {
            ScreenService.showMessage('Operação bloqueada. Informe o arquivo para importação.');
        }
    };

    /**
     * Essa fução tem como finalidade pegar o resultado do dataset que ela receber e satar o data do dataSource da widget
     * filha da importacaoExtratoBancario, e fazer os dados, lidos pelo backend serem mostrados no grid
     *
     * @param {Oject} widget   widget filha da widget de iporatação e de extrato bancario
     * @param {Array} dataSets dataset retornado da leitura do arquivo pelo backned
     */
    this.setDataOfGrid = function(widget, dataSets, tipo) {
        this.changeFieldDtMovimentacao(dataSets.extrbanco.BANCO);
        widget.getField('CDCONTCORRMASK').setValue(dataSets.extrbanco.CDCONTCORRMASK);
        widget.getField('SALDOINICIAL').setValue(dataSets.extrbanco.SALDOINICIAL);
        widget.getField('SALDOFINAL').setValue(dataSets.extrbanco.SALDOFINAL);
        widget.getField('DTSALDINIC').setValue(dataSets.extrbanco.DTSALDINIC);
        widget.getField('DTSALDFIN').setValue(dataSets.extrbanco.DTSALDFIN);
        widget.getField('NMBANCO').setValue(dataSets.extrbanco.NMBANCO);
        if (tipo === "OFX") {
            if(dataSets.extrbanco.BANCO != '237' && dataSets.extrbanco.BANCO != '0237'){
                widget.widgets[0].dataSource.data = dataSets.movreal;
            } else {
                this.data = dataSets.movreal;
            }
            //widget.widgets[0].redraw();
        }
    };

    /**COMPLETAR DOCUMENTAÇÃO
     * Essa função tem como finalidade enviar ao seviço o conteudo do arquivo (convertido em base 64) e, se a extenção
     * do arquivo for TXT, informar de qual banco é o arquivo.
     *
     * @param  {Object} widget recebe a widget de importacao de e xtrato bancário (importacaoExtratoBancario)
     */
    this.uploadContentFile = function(widget) {
        try {
            if(widget.isValid()){
                var valueField = widget.currentRow.UPLOADFILE[0];
                var banco = "";
                var agencia = "";
                var tipo = "OFX";
                var empresa;
                
                if(widget.currentRow.NMEMPRESA){
                    empresa = widget.currentRow.CDEMPRESA;
                }else{
                    empresa = "T";
                }
                
                if(valueField.type === 'text/plain'){
                    agencia = widget.currentRow.CDAGENCIA;
                    banco = widget.currentRow.CDBANCCNAB;
                    tipo = "TXT";
                }
                Bco54200_ExtratoBancario_Service.uploadContentFile(valueField.b64File, banco, tipo, empresa).then(function(callback){
                    var extrato = callback.dataset;
                    var contas = extrato.extrbanco.CONTAS;
                    if (contas > 1) {
                        this.validaConta(extrato, widget, tipo);
                    } else {
                        this.setDataOfGrid(widget, extrato, tipo);
                    }
                }.bind(this), function(error){
                    ScreenService.showMessage(error.data.error);
                });
            }
        } catch (e) {
            ScreenService.showMessage(e.message);
        }
    };

    this.validaConta = function(extrato, widget, tipo) {
        var tpContas = extrato.extrbanco.TPCONTAS;
        var actions = [];
        var self = this;
        tpContas.forEach(function(conta){
            var optionLabel = conta[1] === 'C' ? `Circulante - ${conta[0]}` : conta[1] === 'L' ? `Aplicação - ${conta[0]}` : `Garantida - ${conta[0]}`;
            var option = {
                label: ScreenService.i18n(optionLabel),
                "default": true,
                code: function(e, deferred) {
                    self.setCdreducont(extrato, conta, widget, tipo);
                }
            };
            actions.push(option);
        });
        ScreenService.showCustomDialog('Importar extrato bancário para a conta do tipo ?', 'CONFIRMATION', 'question', actions);
    };

    /**
     * Essa função recebe tem a finalidade de verificar qual a extenssão do arquivo que o usuário selecionou, e quando
     * do tipo TXT, habilidar o campo de banco para que o usuário informe o banco, dessa forma o backend sabera qual o
     * layout que devera ser utilizado para a leitura do arquivo. Quando OFX o layout utiliado é padrão para todos os
     * bancos
     *
     * @param  {[type]} widget recebe a widget de importacao de e xtrato bancário (importacaoExtratoBancario)
     */
    this.validTypeFile = function(widget) {
        var valueField = widget.currentRow.UPLOADFILE[0];
        var file = valueField.b64File.split(',');
        if(file[1]) {
            var fieldBanco = widget.getField('NMBANCO');
            if(valueField.type === 'text/plain'){
                fieldBanco.readOnly = false;
                fieldBanco.validations.required = true;
                ScreenService.showMessage('Informe o banco referente ao extrato selecionado.');
            } else {
                fieldBanco.readOnly = true;
                delete fieldBanco.validations.required;
                this.uploadContentFile(widget);
            }
        } else {
            valueField.setValue(null);
            widget.currentRow.valueField = null;
            ScreenService.showMessage('Arquivo vazio.');
        }
    };
    
    
    /**
     * Novo Extrato Bancário
     */
    this.newExtrato = function() {
        var widget = templateManager.container.getWidget('bco54200_extrato_bancario_grid');
        var filtro = widget.appliedFilter;
        
        widget.newRow();
        
        if(filtro){
            widget.currentRow.CDREDUCONT = filtro.FILTER_ROW.CDREDUCONT;
            widget.currentRow.CDBANCO    = filtro.FILTER_ROW.CDBANCO;
            widget.currentRow.CDAGENCIA  = filtro.FILTER_ROW.CDAGENCIA;
            widget.currentRow.CDCONTCORR = filtro.FILTER_ROW.CDCONTCORR;
            widget.currentRow.DSCONTCORR = filtro.FILTER_ROW.DSCONTCORR;
        }
    };
    
    /**
     * Essa função tem como finalidade verificar o banco que esta sendo passado
     * no arquivo OFX e caso for BRADESCO habilitar o campo de filtro de data
    */
     this.changeFieldDtMovimentacao = function(banco) {
         var widgetExtrato = templateManager.container.getWidget('importacaoExtratoBancario');
         var dtMovimento = widgetExtrato.getField('DTMOVIMENTO');
         if(banco == '237' || banco == '0237'){
             dtMovimento.readOnly =  false;
         } else {
             dtMovimento.readOnly =  true;
         }
     };
     
     /**
      * Essa Função tem como obetivo filtrar o datasource da importacao bancaria
      * de acordo com o intervalo de datas informado pelo usuário na tela
      * SOMENTE BRADESCO
     */
     this.setDataOfGridBradesco = function() {
         var widgetExtrato = templateManager.container.getWidget('importacaoExtratoBancario');
         var widgetGrid = templateManager.container.getWidget('importacaoGrid');
         var dataFilterDatasource = widgetExtrato.getField('DTMOVIMENTO').value();
         let objetosFiltrados = this.data.filter(result => {
             return this.convertData(result.DTMOVREAL) >= this.convertData(dataFilterDatasource.start) &&
                    this.convertData(result.DTMOVREAL) <= this.convertData(dataFilterDatasource.end);
          });
         widgetGrid.dataSource.data = objetosFiltrados;
         //widgetGrid.redraw();
     };
     
     /**
      * Essa função converte as datas passadas pelo filtro em objetoDate para comparações
     */
     this.convertData = function converteData(dataConvert) {
        let date = dataConvert.split("/");
        let objectDate = new Date(parseInt(date[2], 10),
                      parseInt(date[1], 10) - 1,
                      parseInt(date[0], 10));
        return objectDate;
    };
    
    
    /*
     * Função criada para inserir extratos duplicados.
    */
    this.saveImportDulicados =  function(widget){
        var checkedrows = widget.getCheckedRows();
        //var data = widget.dataSource.data;
        if(checkedrows.length > 0){
            General.callRestEngine('fin_movimentacaoBancaria#/saveDuplicado', 'Row', checkedrows).then(function() {
                templateManager.container.getWidget('bco54200_extrato_bancario_grid').reload();
                ScreenService.closePopup(widget);
            }, function (error) {
              ScreenService.showMessage('Ocorreu um erro ao salvar o registro');
            });
        } else {
            ScreenService.showMessage('favor selecionar aos registros');
        }
    };
    
};
Configuration(function (ContextRegister){
    ContextRegister.register('Bco54200_ExtratoBancario_Controller', Bco54200_ExtratoBancario_Controller);
});


// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco54500_Conciliacao_Controller.js
var Bco54500_Conciliacao_Controller = function(ScreenService, General, Bco54500_Conciliacao_Service, templateManager, ZHPromise, ZhFilterPreferences) {

    var self = this;
    var statusMovbancario = '';
    var statusExtrato = '';
    
    self.openFilterConciliacao = function(widget){
        var filterRow = {};
        self.buscaPadraoEmpresa(filterRow).then(function(filterRow){
            widget.activate();
            ZhFilterPreferences.setOriginalFilter(widget.id, filterRow);
            ZhFilterPreferences.openFilterWidget(widget, filterRow);
        });
    };

    self.atualizaTabbeds = function(conciliacao, preference, widget_name) {
        self.setDataSourceFilterTabbeds(preference).then(function() {
			self.setContaCorrente(preference);
			var extrato   = templateManager.container.getWidget('extrato_bancario');
            var movimento = templateManager.container.getWidget('movimentacao_bancaria');
            
            extrato.activate();
            extrato.getAction('CONCILIAR').isVisible = false;
            extrato.getAction('DESCONCILIAR').isVisible = false;
            
            movimento.activate();
            movimento.getAction('CONCILIAR').isVisible = false;
            movimento.getAction('DESCONCILIAR').isVisible = false;
        });
        
        self.atualizaWidgets(widget_name);
    };
    
    self.atualizaWidgets = function(widget_name) {
        var widget = templateManager.container.getWidget(widget_name);
        widget.reload();
    };

    self.setContaCorrente = function(popup) {
        var widget = templateManager.container.getWidget('conciliacao');
        
        widget.currentRow.CDREDUCONT = popup.FILTER_ROW.CDREDUCONT;
        widget.currentRow.CDCONTCORR = popup.FILTER_ROW.CDCONTCORR;
        widget.currentRow.CDBANCO    = popup.FILTER_ROW.CDBANCO;
        widget.currentRow.CDAGENCIA  = popup.FILTER_ROW.CDAGENCIA;
        widget.currentRow.DSCONTCORR = popup.FILTER_ROW.DSCONTCORR;
        
        var currentRow = {
            cdreducont : widget.currentRow.CDREDUCONT,
            datarefe   : moment().format('DD/MM/YYYY')
        };
        
        Bco54500_Conciliacao_Service.getSaldoContaCorr(currentRow).then(function(saldo_contacorr) {
            widget.currentRow.SALDO = saldo_contacorr.dataset.getSaldoContaCorr.SALDO;
        });
    };

    self.setDataSourceFilterTabbeds = function (popupFilter) {
        var deferred = ZHPromise.defer();
        var extrato = templateManager.container.getWidget('extrato_bancario');
        var sistema = templateManager.container.getWidget('movimentacao_bancaria');
        var dtmovreal    = false;
        var dtmovbanc    = false;
        
        delete popupFilter.FILTER_ROW.CDEMPRESA;
        delete popupFilter.FILTER_ROW.NMEMPRESA;
        
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTMOVREAL'))
            if (popupFilter.FILTER_ROW.DTMOVREAL.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTMOVREAL.start))
                dtmovreal = true;
                
        if (popupFilter.FILTER_ROW.hasOwnProperty('DTMOVBANC'))
            if (popupFilter.FILTER_ROW.DTMOVBANC.hasOwnProperty('start') && (popupFilter.FILTER_ROW.DTMOVBANC.start))
                dtmovbanc = true;
                
        for(var i = extrato.dataSourceFilter.length - 1; i >= 0; i--){
            if (extrato.dataSourceFilter[i].name === 'NRMOVBARE')
                extrato.dataSourceFilter.splice(i, 1);
            else if (extrato.dataSourceFilter[i].name === 'NRCONCIMOVB')
                extrato.dataSourceFilter.splice(i, 1);
        }
        extrato.dataSource.clearCheckedRows();
        
        for(var y = sistema.dataSourceFilter.length - 1; y >= 0; y--){
            if (sistema.dataSourceFilter[y].name === 'NRSEQMOVREBA')
                sistema.dataSourceFilter.splice(y, 1);
            else if (sistema.dataSourceFilter[y].name === 'CDSEQUEXBA')
                sistema.dataSourceFilter.splice(y, 1);
            else if (sistema.dataSourceFilter[y].name === 'NRCONCIMOVB')
                sistema.dataSourceFilter.splice(y, 1);
        }
        sistema.dataSource.clearCheckedRows();
        
        extrato.dataSourceFilter.forEach(function(filter) {
            switch(filter.name) {
                case 'CDREDUCONT': filter.value = (popupFilter.FILTER_ROW.CDREDUCONT) ? popupFilter.FILTER_ROW.CDREDUCONT : null;
                    break;
                case 'DTINICIAL': filter.value = (dtmovreal) ? popupFilter.FILTER_ROW.DTMOVREAL.start : null;
                    break;
                case 'DTFINAL': filter.value = (dtmovreal) ? popupFilter.FILTER_ROW.DTMOVREAL.end : null;
                    break;
                case 'TDATAS': filter.value = (dtmovreal) ? null : 'T';
                    break;
                case 'NRDOC': filter.value = (popupFilter.FILTER_ROW.NRDOC) ? popupFilter.FILTER_ROW.NRDOC : 'T';
                    break;
                case 'VALOR': filter.value = (popupFilter.FILTER_ROW.VALOR == 0) ? null : popupFilter.FILTER_ROW.VALOR;
                    break;
                case 'TVALOR': filter.value = ((popupFilter.FILTER_ROW.VALOR == 0) || (popupFilter.FILTER_ROW.VALOR == null)) ? 'T' : null;
                    break;
                case 'IDSTATUSEXTRATO': filter.value = (popupFilter.FILTER_ROW.IDSTATUSEXTRATO) ? popupFilter.FILTER_ROW.IDSTATUSEXTRATO : 'T';
                    break;
                case 'IDMOVREAL': filter.value = (popupFilter.FILTER_ROW.IDMOVREAL) ? popupFilter.FILTER_ROW.IDMOVREAL : 'T';
                    break;
                default: break;
            }
        });

        sistema.dataSourceFilter.forEach(function(filter) {
            switch(filter.name) {
                case 'CDREDUCONT': filter.value = (popupFilter.FILTER_ROW.CDREDUCONT) ? popupFilter.FILTER_ROW.CDREDUCONT : null;
                    break;
                case 'DTINICIAL': filter.value = (dtmovbanc) ? popupFilter.FILTER_ROW.DTMOVBANC.start : null;
                    break;
                case 'DTFINAL': filter.value = (dtmovbanc) ? popupFilter.FILTER_ROW.DTMOVBANC.end : null;
                    break;
                case 'NRDOCMBANC': filter.value = (popupFilter.FILTER_ROW.NRDOCMBANC) ? popupFilter.FILTER_ROW.NRDOCMBANC : null;
                    break;
                case 'VALORBANC': filter.value = (popupFilter.FILTER_ROW.VALORBANC == 0) ? null : popupFilter.FILTER_ROW.VALORBANC;
                    break;
                case 'IDSTATUSBANC': filter.value = (popupFilter.FILTER_ROW.IDSTATUSBANC) ? popupFilter.FILTER_ROW.IDSTATUSBANC : 'T';
                    break;
                case 'IDMOVBANC': filter.value = (popupFilter.FILTER_ROW.IDMOVBANC) ? popupFilter.FILTER_ROW.IDMOVBANC : 'T';
                    break;
                default: break;
            }
        });
        
        deferred.resolve();
        return deferred.promise;
    };
    
    self.retornaAoFiltro = function (reload) {
        var deferred = ZHPromise.defer();
        
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var extrato     = templateManager.container.getWidget('extrato_bancario');
        var movimento   = templateManager.container.getWidget('movimentacao_bancaria');
        
        conciliacao.currentRow.VALORMOVBANC = 0;
        conciliacao.currentRow.VALOREXTRATO = 0;
        
        extrato.getAction('DESCONCILIAR').isVisible = false;
        movimento.getAction('DESCONCILIAR').isVisible = false;
        
        for(var i = extrato.dataSourceFilter.length - 1; i >= 0; i--){
            if (extrato.dataSourceFilter[i].name === 'NRMOVBARE')
                extrato.dataSourceFilter.splice(i, 1);
            else if (extrato.dataSourceFilter[i].name === 'NRCONCIMOVB')
                extrato.dataSourceFilter.splice(i, 1);
        }
        extrato.dataSource.clearCheckedRows();
        
        for(var y = movimento.dataSourceFilter.length - 1; y >= 0; y--){
            if (movimento.dataSourceFilter[y].name === 'NRSEQMOVREBA')
                movimento.dataSourceFilter.splice(y, 1);
            else if (movimento.dataSourceFilter[y].name === 'CDSEQUEXBA')
                movimento.dataSourceFilter.splice(y, 1);
            else if (movimento.dataSourceFilter[y].name === 'NRCONCIMOVB')
                movimento.dataSourceFilter.splice(y, 1);
        }
        movimento.dataSource.clearCheckedRows();
        
        if(reload){
            movimento.reload();
            extrato.reload();
        }
        
        deferred.resolve();
        return deferred;
    };

    self.conciliarExtratosMovimentos = function() {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var valorExtrato = parseFloat(conciliacao.getField('VALOREXTRATO').value().toFixed(2));
        var valorMovbanc = parseFloat(conciliacao.getField('VALORMOVBANC').value().toFixed(2));
        
        if (valorExtrato == valorMovbanc) {
            
            var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
            var extrato = templateManager.container.getWidget('extrato_bancario');
            var dataSet = [];
            var route = 'fin_movimentacaoBancaria#/conciliarExtratosMovimentos';
            var dataExtrato = extrato.getCheckedRows();
            var dataMovimentos = movimentos.getCheckedRows();
            var message = '';
            
            if ((dataExtrato.length + dataMovimentos.length) == 2) {
                if (dataExtrato[0].NRDOCMOVREAL != dataMovimentos[0].NRDOCMBANC)
                    message += 'Nº do documento do extrato está diferente do nº do documento da movimentação bancária.\n';
                    
                if (dataExtrato[0].DTMOVREAL != dataMovimentos[0].DTMOVBANC)
                    message += 'Data do extrato está diferente da data da movimentação bancária.';
            }
            
            if (message != '') {
                ScreenService.confirmMessage(message + '\nDeseja confirmar?', 'question', function (success) {
                    self.request(dataSet, dataExtrato, dataMovimentos, route).then(function() {
                        extrato.unSelectAll();
                        movimentos.unSelectAll();
                        ScreenService.showMessage('Conciliação realizada com sucesso.');
                        ScreenService.closePopup();
                        self.retornaAoFiltro(true);
                    });
                });
            }else{
                self.request(dataSet, dataExtrato, dataMovimentos, route).then(function() {
                    extrato.unSelectAll();
                    movimentos.unSelectAll();
                    ScreenService.showMessage('Conciliação realizada com sucesso.');
                    ScreenService.closePopup();
                    
                    self.retornaAoFiltro(true);
                });
            }

        }else ScreenService.showMessage('Operação bloqueada. Valores totais de extratos e movimentações diferentes.');
    };

    self.desconciliarExtratosMovimentos = function(widget) {
        ScreenService.confirmMessage('Deseja realmente desconciliar os extratos e movimentações exibidos?', 'question',
            function () {
                var conciliacao = templateManager.container.getWidget('conciliacao');
                var extrato     = templateManager.container.getWidget('extrato_bancario');
                var movimentos  = templateManager.container.getWidget('movimentacao_bancaria');
                
                var dataSet = [];
                var route   = 'fin_movimentacaoBancaria#/desconciliarExtratosMovimentos';
                
                self.request(dataSet, extrato.dataSource.data, movimentos.dataSource.data, route).then(function() {
                    ScreenService.showMessage('Desconciliação realizada com sucesso.');
                    ScreenService.closePopup();
                    
                    self.setDataSourceFilterTabbeds(movimentos.appliedFilter).then(function() {
                        movimentos.reload();
                        extrato.reload();
                        
                        extrato.getAction('DESCONCILIAR').isVisible = false;
                        movimentos.getAction('DESCONCILIAR').isVisible = false;
                    });
                });
            }
        );
        

    };

    self.request = function(dataSet, dataExtrato, dataMovimentos, route) {
        var deferred = ZHPromise.defer();
        
        for (var i = 0; i < dataExtrato.length; i++)
            dataSet[dataSet.length] = dataExtrato[i];
            
        for (var j = 0; j < dataMovimentos.length; j++)
            dataSet[dataSet.length] = dataMovimentos[j];
            
        dataSet[0].QTDEEXTRATO = dataExtrato.length;
        
        General.callRestEngine(route, 'DataSet', dataSet)
            .then(function () {
                deferred.resolve();
            })
            .catch(function (erro) {
                deferred.reject();
            }
        );

        return deferred;
    };

    self.validaCheckExtrato = function() {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var extrato     = templateManager.container.getWidget('extrato_bancario');
        var movimentos  = templateManager.container.getWidget('movimentacao_bancaria');
        
        var hash = null;
        var soma = 0;
        
        var idstconext = (movimentos.getCheckedRows().length > 0)?movimentos.getCheckedRows()[0].IDSTCONEXT:extrato.getLastSelectedRow().IDSTCONMVBAN;
        
        if ((extrato.getCheckedRows().length > 1) && (extrato.getCheckedRows()[0].IDSTCONMVBAN != extrato.getLastSelectedRow().IDSTCONMVBAN)) {
            ScreenService.showMessage('Operação bloqueada. Extratos devem possuir o mesmo status.');
            extrato.getLastSelectedRow().__isSelected = false;
            hash = self.getHashKey(extrato, extrato.getLastSelectedRow());
			delete extrato.dataSource.checkedRowsCache[hash];
            extrato.dataSource.updateCheckedRows();
        } else if ((extrato.getCheckedRows()) && (movimentos.getCheckedRows()) && (idstconext != extrato.getLastSelectedRow().IDSTCONMVBAN)) {
            ScreenService.showMessage('Operação bloqueada. Extratos devem possuir o mesmo status das movimentações.');
            extrato.getLastSelectedRow().__isSelected = false;
            hash = self.getHashKey(extrato, extrato.getLastSelectedRow());
            delete extrato.dataSource.checkedRowsCache[hash];
            extrato.dataSource.updateCheckedRows();
        } else {
            statusExtrato = extrato.getLastSelectedRow().IDSTCONMVBAN;

            // Cálculo do Valor Total de Extratos
            extrato.getCheckedRows().forEach(function (data) {
                if (data.IDSTCONMVBAN.toString() == '0') {
                    soma += (data.IDMOVREAL == 'C')?parseFloat(data.VRMOVREAL):parseFloat(data.VRMOVREAL)*-1;
                }
            });
            conciliacao.currentRow.VALOREXTRATO = soma;
            
            if (extrato.getLastSelectedRow().IDSTCONMVBAN == '1') {
                extrato.filterData = angular.copy(extrato.dataSource.data);
                if((extrato.getCheckedRows().length < 2) && ((movimentos.getCheckedRows().length == 0) || movimentos.getCheckedRows().length == 'undefined'))
                    self.reloadExtratoMovimentos(extrato, extrato.getLastSelectedRow().NRCONCIMOVB);
            }else{
                if(movimentos.getCheckedRows().length > 0){
                    extrato.activate();
                    extrato.getAction('CONCILIAR').isVisible = true;
                    extrato.getAction('DESCONCILIAR').isVisible = false;
                }
            }
        }
    };

    self.getHashKey = function(widget, row) {
        var VO = widget.dataSource.getVO();
        var hash = "";
        if (VO && VO.localKey) {
            hash = VO.localKey.reduce(function(hash, key) {
                return row[key] ? hash + "_" + row[key].toString() : hash;
            }, "");
        }
        return hash;
    };

    self.validaCheckMovimentos = function() {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var movimentos  = templateManager.container.getWidget('movimentacao_bancaria');
        var extrato     = templateManager.container.getWidget('extrato_bancario');
        
        var soma = 0;
        var hash;
        
        var idstconmvban = (extrato.getCheckedRows().length > 0)?extrato.getCheckedRows()[0].IDSTCONMVBAN:movimentos.getLastSelectedRow().IDSTCONEXT;
        
        if ((movimentos.getCheckedRows().length > 1) && (movimentos.getCheckedRows()[0].IDSTCONEXT != movimentos.getLastSelectedRow().IDSTCONEXT)) {
            ScreenService.showMessage('Operação bloqueada. Movimentos devem possuir o mesmo status');
            movimentos.getLastSelectedRow().__isSelected = false;
            hash = self.getHashKey(movimentos, movimentos.getLastSelectedRow());
            delete movimentos.dataSource.checkedRowsCache[hash];
            movimentos.dataSource.updateCheckedRows();
        } else if ((extrato.getCheckedRows()) && (movimentos.getCheckedRows()) && (idstconmvban != movimentos.getLastSelectedRow().IDSTCONEXT)) {
            ScreenService.showMessage('Operação bloqueada. Movimentações devem possuir o mesmo status dos extratos.');
            movimentos.getLastSelectedRow().__isSelected = false;
            hash = self.getHashKey(movimentos, movimentos.getLastSelectedRow());
            delete movimentos.dataSource.checkedRowsCache[hash];
            movimentos.dataSource.updateCheckedRows();
        } else {
            statusMovbancario = movimentos.getLastSelectedRow().IDSTCONEXT;

            movimentos.getCheckedRows().forEach(function (data) {
                if (data.IDSTCONEXT.toString() == '0')
                    soma += (data.IDMOVBANC == 'C')?parseFloat(data.VRMOVBANC):parseFloat(data.VRMOVBANC)*-1;
            });
            conciliacao.currentRow.VALORMOVBANC = soma;
            
            if (movimentos.getLastSelectedRow().IDSTCONEXT == '1') {
                movimentos.filterData = angular.copy(movimentos.dataSource.data);
                if((movimentos.getCheckedRows().length < 2) && ((extrato.getCheckedRows().length == 0) || extrato.getCheckedRows().length == 'undefined'))
                    self.reloadExtratoMovimentos(movimentos, movimentos.getLastSelectedRow().NRCONCIMOVB);
            }else{
                if(extrato.getCheckedRows().length > 0){
                    movimentos.activate();
                    movimentos.getAction('CONCILIAR').isVisible = true;
                    movimentos.getAction('DESCONCILIAR').isVisible = false;
                }
            }
        }
    };

    self.reloadExtratoMovimentos = function(widget, numeroConciliacao) {
        var extrato    = templateManager.container.getWidget('extrato_bancario');
        var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
        
        extrato.getAction('CONCILIAR').isVisible = false;
        extrato.getAction('DESCONCILIAR').isVisible = true;
        movimentos.getAction('CONCILIAR').isVisible = false;
        movimentos.getAction('DESCONCILIAR').isVisible = true;
        
        var extCamposNulos = ['DTINICIAL', 'DTFINAL', 'VALOR'];
        extrato.filtroAntigo = angular.copy(extrato.dataSourceFilter);
        extrato.dataSourceFilter.forEach(function(filter) {
            if (filter.name != 'CDREDUCONT') {
                filter.value = (extCamposNulos.indexOf(filter.name) != -1) ? null : 'T';
            }
        });
        
        var nrmovbare = (widget.name == 'extrato_bancario') ? widget.getLastSelectedRow().NRMOVBARE : widget.getLastSelectedRow().NRMOVBANC;
        
        if (!numeroConciliacao) {
            extrato.dataSourceFilter.push({name: 'NRMOVBARE', operator: '=', value: nrmovbare});
        }else{
            extrato.dataSourceFilter.push({name: 'NRCONCIMOVB', operator: '=', value: widget.getLastSelectedRow().NRCONCIMOVB});
        }
        
        var movCamposNulos = ['DTINICIAL', 'DTFINAL', 'VALORBANC'];
        movimentos.filtroAntigo = angular.copy(movimentos.dataSourceFilter);
        movimentos.dataSourceFilter.forEach(function(filter) {
            if (filter.name != 'CDREDUCONT') {
                filter.value = (movCamposNulos.indexOf(filter.name) != -1) ? null : 'T';
            }
        });
        
        var nrseqmovreba = (widget.name == 'extrato_bancario') ? widget.getLastSelectedRow().NRSEQMOVREAL : widget.getLastSelectedRow().NRSEQMOVREBA;
        var cdsequexba = (widget.name == 'extrato_bancario') ? widget.getLastSelectedRow().CDSEQUEXTR : widget.getLastSelectedRow().CDSEQUEXBA;
        
        if (!numeroConciliacao) {
            movimentos.dataSourceFilter.push({name: 'NRSEQMOVREBA', operator: '=', value: nrseqmovreba});
            movimentos.dataSourceFilter.push({name: 'CDSEQUEXBA', operator: '=', value: cdsequexba});
        }else{
            movimentos.dataSourceFilter.push({name: 'NRCONCIMOVB', operator: '=', value: widget.getLastSelectedRow().NRCONCIMOVB});
        }
    
        movimentos.reload();
        extrato.reload();
    };

    self.validaExtratoSelectAll = function(extrato) {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
        var naoconciliado = false;
        var conciliado = false;
        var valorextrato = 0;
        
        extrato.getCheckedRows().forEach(function (data) {
            if(data.IDSTCONMVBAN == 0){
                naoconciliado = true;
                valorextrato += (data.IDMOVREAL == 'C')?parseFloat(data.VRMOVREAL):parseFloat(data.VRMOVREAL)*-1;
            }else conciliado = true;
        });
        
        if(conciliado && naoconciliado){
            extrato.dataSource.unSelectAll();
            ScreenService.confirmMessage('Somente é possivel marcar extratos não conciliados. Deseja marcá-los?', 'question',
                function () {
                    var a = extrato.dataSource.data.filter(function(data){
                        return data.IDSTCONMVBAN == 0;
                    });
                    extrato.dataSource.selectAll(a);
                    statusExtrato = '0';
                    conciliacao.currentRow.VALOREXTRATO = valorextrato;
                    if(movimentos.getCheckedRows().length > 0){
                        extrato.activate();
                        extrato.getAction('CONCILIAR').isVisible = true;
                        movimentos.getAction('CONCILIAR').isVisible = true;
                    }
                    templateManager.updateTemplate();
                }
                
            );
        }else if(conciliado && !naoconciliado){
            extrato.dataSource.unSelectAll();
            ScreenService.showMessage('Operação bloqueada. Não há extratos não conciliados para marcação.');
        }else{
            statusExtrato = '0';
            if(movimentos.getCheckedRows().length > 0){
                extrato.activate();
                extrato.getAction('CONCILIAR').isVisible = true;
                movimentos.getAction('CONCILIAR').isVisible = true;
            }
            conciliacao.currentRow.VALOREXTRATO = valorextrato;
        }
    };
    
    self.validaMovimentosSelectAll = function(movimentos) {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var extrato    = templateManager.container.getWidget('extrato_bancario');
        var naoconciliado = false;
        var conciliado = false;
        var valormovbanc = 0;
        
        movimentos.getCheckedRows().forEach(function (data) {
            if(data.IDSTCONEXT == 0){
                naoconciliado = true;
                valormovbanc += (data.IDMOVBANC == 'C')?parseFloat(data.VRMOVBANC):parseFloat(data.VRMOVBANC)*-1;
            }else conciliado = true;
        });
        
        if(conciliado && naoconciliado){
            movimentos.dataSource.unSelectAll();
            ScreenService.confirmMessage('Somente é possivel marcar movimentos não conciliados. Deseja marcá-los?', 'question',
                function () {
                    var a = movimentos.dataSource.data.filter(function(data){
                        return data.IDSTCONEXT == 0;
                    });
                    movimentos.dataSource.selectAll(a);
                    statusMovbancario = '0';
                    if(extrato.getCheckedRows().length > 0){
                        movimentos.activate();
                        extrato.getAction('CONCILIAR').isVisible = true;
                        movimentos.getAction('CONCILIAR').isVisible = true;
                    }
                    conciliacao.currentRow.VALORMOVBANC = valormovbanc;
                    templateManager.updateTemplate();
                }
                
            );
        }else if(conciliado && !naoconciliado){
            movimentos.dataSource.unSelectAll();
            ScreenService.showMessage('Operação bloqueada. Não há movimentos não conciliados para marcação.');
        }else{
            statusMovbancario = '0';
            if(extrato.getCheckedRows().length > 0){
                movimentos.activate();
                extrato.getAction('CONCILIAR').isVisible = true;
                movimentos.getAction('CONCILIAR').isVisible = true;
            }
            conciliacao.currentRow.VALORMOVBANC = valormovbanc;
        }
    };

    self.validaUnselectAll = function(widget) {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var extrato = templateManager.container.getWidget('extrato_bancario');
        var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
        var widget_active = templateManager.container.getActiveWidget();
        
        if((widget_active.name == 'extrato_bancario') || (widget_active.name == 'movimentacao_bancaria')){
            
            extrato.getAction('CONCILIAR').isVisible = false;
            extrato.getAction('DESCONCILIAR').isVisible = false;
            movimentos.getAction('CONCILIAR').isVisible = false;
            movimentos.getAction('DESCONCILIAR').isVisible = false;
            
            if(widget.name == 'extrato_bancario'){
                statusExtrato = '';
                conciliacao.currentRow.VALOREXTRATO = 0;
            }else{
                statusMovbancario = '';
                conciliacao.currentRow.VALORMOVBANC = 0;
            }
            
            if((extrato.getLastSelectedRow().IDCONCREAL) || (movimentos.getLastSelectedRow().IDCONCBANC)){
                var checkeds = extrato.getCheckedRows().length + movimentos.getCheckedRows().length;
                if (checkeds == 0) {
                    self.setDataSourceFilterTabbeds(movimentos.appliedFilter).then(function() {
                        movimentos.reload();
                        extrato.reload();
                    });
                }
            }
        }
    };

    self.validaUnselectRow = function(widget) {
        var conciliacao = templateManager.container.getWidget('conciliacao');
        var extrato = templateManager.container.getWidget('extrato_bancario');
        var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
        var somaExt = 0;
        var somaMov = 0;
        
        // Status do Extrato e Movimentação
        if (widget.getCheckedRows().length == 0) {
            statusExtrato = (widget.name == extrato.name) ? '' : statusExtrato;
            statusMovbancario = (widget.name == movimentos.name) ? '' : statusMovbancario;
        }
        
        // Filtro de Conciliação
        if((extrato.getLastSelectedRow().IDCONCREAL) || (movimentos.getLastSelectedRow().IDCONCBANC)){
            var checkeds = extrato.getCheckedRows().length + movimentos.getCheckedRows().length;
            if (checkeds == 0) {
                self.setDataSourceFilterTabbeds(movimentos.appliedFilter).then(function() {
                    movimentos.reload();
                    extrato.reload();
                });
                
                extrato.getAction('CONCILIAR').isVisible = false;
                extrato.getAction('DESCONCILIAR').isVisible = false;
                movimentos.getAction('CONCILIAR').isVisible = false;
                movimentos.getAction('DESCONCILIAR').isVisible = false;
            }
        }else{

            // Calcula Valor
            if (widget.name == movimentos.name) {
                movimentos.getCheckedRows().forEach(function (data) {
                    somaMov += (data.IDMOVBANC == 'C')?parseFloat(data.VRMOVBANC):parseFloat(data.VRMOVBANC)*-1;
                });
                conciliacao.currentRow.VALORMOVBANC = somaMov;
            }
            if (widget.name == extrato.name) {
                extrato.getCheckedRows().forEach(function (data) {
                    somaExt += (data.IDMOVREAL == 'C')?parseFloat(data.VRMOVREAL):parseFloat(data.VRMOVREAL)*-1;
                });
                conciliacao.currentRow.VALOREXTRATO = somaExt;
            }
            
            if ((extrato.getCheckedRows().length == 0) || (movimentos.getCheckedRows().length == 0)){
                extrato.getAction('CONCILIAR').isVisible = false;
                extrato.getAction('DESCONCILIAR').isVisible = false;
                movimentos.getAction('CONCILIAR').isVisible = false;
                movimentos.getAction('DESCONCILIAR').isVisible = false;
            }
            
            
        }
    };

    ////////////////////////////// Conciliação Automática //////////////////////////////////////////

    /**
     * Essa função é responsável por chamar a requisição que fara a conciliação automática.
     * É montado um array de um objeto que contém os parametros passado pelo usuário. Quando
     * o usuário não seleciona uma data, a propriedade TDTCONCILIA é passada como 'T', quando
     * ele seleciona uma data é passado null. Ela é responsável, também, por tratar o retorno
     * deste request.
     *
     * @param  {Object} widget widget do popup da cociliação automática
     */
    self.conciliaMovs = function(widget) {
        if(widget.isValid()) {
            var movimentos = templateManager.container.getWidget('movimentacao_bancaria');
            var extratos   = templateManager.container.getWidget('extrato_bancario');
            
            var row = [{
                "NRDIA": widget.currentRow.NRDIA,
                "CDREDUCONT": widget.currentRow.CDREDUCONT,
                "DTCONCILIA": widget.currentRow.DTCONCILIA ? widget.currentRow.DTCONCILIA : null,
                "TDTCONCILIA": widget.currentRow.DTCONCILIA ? null : 'T'
            }];
            General.callRestEngine('fin_movimentacaoBancaria#/conciliacaoAutomatica', 'Row', row).then(function (success) {
                ScreenService.showMessage(success.messages[0].message);
                self.setDataSourceFilterTabbeds(movimentos.appliedFilter).then(function() {
                    movimentos.reload();
                    extratos.reload();
                    ScreenService.closePopup();
                });
            }).catch(function (error) {
                if(error.data)
                     ScreenService.showMessage(error.data.error);
                else ScreenService.showMessage(error);
            }
        );
        }
    };

    /**
     * Essa função tem como objetivo setar a correnteRow da widget, da rotina de conciliação automática,
     * com os campos de CDREDUCONT, necessário para a rotina, e DSCONTCORR que é necessário para exibição
     * no campo de conta corrente. Esses valores só são setado se o usuário tiver filtrado a tela, e por
     * isso a widget de conciliação manual tiver o campo de CDREDUCONT, que também é necessário para o
     * seu processo.
     *
     * @param  {Object} widgetPopup widget do popup de conciliação automática
     */
    self.getContaCorrente = function(widgetPopup) {
        var widgetConciliManual = templateManager.container.getWidget('conciliacao');
        var currentRow = widgetConciliManual.currentRow;

        if(currentRow.CDREDUCONT) {
            widgetPopup.currentRow.CDREDUCONT = currentRow.CDREDUCONT;
            widgetPopup.currentRow.DSCONTCORR = currentRow.DSCONTCORR;
        }
    };
    
    self.habilitaContaCorrente = function(filtro){
        var contacorrente = filtro.getField('DSCONTCORR');
        
        contacorrente.setValue('');
        
        if (filtro.currentRow.NMEMPRESA) {
            contacorrente.readOnly = false;
            contacorrente.dataSourceFilter[0].value = filtro.currentRow.CDEMPRESA;
        }else {
            contacorrente.readOnly = true;
            contacorrente.dataSourceFilter[0].value = null;
        }
    };
    
    self.buscaPadraoEmpresa = function(filterRow) {
        var deferred = ZHPromise.defer();
        var movimentacao_bancaria = templateManager.container.getWidget('movimentacao_bancaria');
        var filtroFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filtroFilial)
        .then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var cdempresa = filial.CDEMPRESA;
            var routeEmpresa = 'fin#/crud/empresa';
            var filtroEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": cdempresa}];
            General.callRestEngine(routeEmpresa, 'FilterData', filtroEmpresa).then(function (response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                filterRow.CDEMPRESA = empresa.CDEMPRESA;
                filterRow.NMEMPRESA = empresa.NMEMPRESA;
                movimentacao_bancaria.getField('DSCONTCORR').dataSourceFilter[0].value = cdempresa;
                deferred.resolve(filterRow);
            });
        })
        .catch(function(){
            deferred.resolve(filterRow);
        });
        return deferred.promise;
    };

};
Configuration(function (ContextRegister) {
    ContextRegister.register('Bco54500_Conciliacao_Controller', Bco54500_Conciliacao_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco56100_TipoDeDespesa_Controller.js
var  Bco56100_TipoDeDespesa_Controller = function(ScreenService, General, templateManager, ZHPromise, localStorageEngine, contabilizacaoController, restEngine, ZhFilterPreferences){
    
    var self = this;
    
    
    this.geraCodigoDespesa = function(widget) {
        var codigo = 0;
		widget.newRow();
		
		General.callRestEngine('fin#/crud/tipodespesas', 'FilterData').then(function (result) {
			var datasetTipoDespesas = result.dataset.tipodespesas;
			
			
			if(result.dataset.tipodespesas.length > 0){
    			datasetTipoDespesas.forEach(function(data){
    			    if(data.CDTIPODESP >= codigo)
    			        codigo = data.CDTIPODESP;
    			});
				// var cdgrupo = ('000000000' + (parseInt(codigo.CDTIPODESP) + 1)).slice(-10);
				widget.getField('CDTIPODESP').value(codigo+1);
			}else widget.getField('CDTIPODESP').value(1);
		}).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
	};
	
    /*self.salvarDespesa = function(widget){
        if(widget.isValid()){
            General.callRestEngine('fin_movimentacaoBancaria#/saveTipoDeDespesa', 'Row', widget.currentRow).then(function(result){
                ScreenService.showMessage("Tipo de Despesa Salva com sucesso.");
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function(error){
                if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar o lançamento devido a erros na contablização.")) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }
    };*/
    
    /*self.seveDespesas = function(widget){
        General.saveCrud(widget);
    };
    
    self.deletarLancamento = function(widget){
        General.callRestEngine('fin_movimentacaoBancaria#/removeLancamentoCredito', 'Row', widget.currentRow).then(function(result){
            ScreenService.showMessage("Lançamento Deletado com sucesso.");
            ScreenService.closeSwipe();
            widget.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };*/
};

Configuration(function(ContextRegister){
    ContextRegister.register('Bco56100_TipoDeDespesa_Controller', Bco56100_TipoDeDespesa_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/Bco56200_LancamentoDespesa_Controller.js
var bco56200_LancamentoDespesa_Controller = function(ScreenService, General, templateManager, contabilizacaoController){

    
    this.newLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        widget.newRow();
        
        widget.getField('CDCENTCUST').setValue('');
        widget.getField('NMCENTCUST').setValue('');
        widget.getField('NMCENTCUST').dataSourceFilter[0].value = '';
        widget.getField('NMCENTCUST').readOnly = true;
        widget.getField('NMCENTCUST').reload();
        
        templateManager.updateTemplate();
        
    };
    
    this.setDescLancDesp = function (args) {
        args.row.DSLANCDESP = args.row.NMDESPESA;
    };
    
    this.saveLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        
        if(widget.isValid()){
            
            if(widget.currentRow.VRLANCDESP <= 0){
                ScreenService.showMessage('Operação bloqueada. Valor deve ser maior que zero.');
                return;
            }
    
            General.callRestEngine('fin_movimentacaoBancaria#/saveLancDespCaixa', 'Row', widget.currentRow).then(function() {
                ScreenService.showMessage('Registros salvo com sucesso.');
                ScreenService.closeSwipe();
                
                widget.activate();
                widget.reload();
            }, function (error) {
                if (contabilizacaoController.checkErrosContabilizacao(error)) {
                    ScreenService.showMessage(error.data.error);
                }
            });
            
        }
    };
    
    this.deleteLancDespCaixa = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        
        ScreenService.confirmMessage('Deseja realmente excluir o registro?', 'question', function() {

            General.callRestEngine('fin_movimentacaoBancaria#/deleteLancDespCaixa', 'Row', widget.currentRow).then(function() {
                ScreenService.showMessage('Registros excluído com sucesso.');
                ScreenService.closeSwipe();
                
                widget.activate();
                widget.reload();
            });

        });
        
    };
    
    this.setCentCustFil = function (args) {
        var widget = templateManager.container.getWidget('LancamentoDespesasGrid');
        var value = '';
        var readOnly = true;
        
        var unidade = widget.getField('CDFILIAL').value();
        
        if(unidade){
            value = unidade;
            readOnly = false;
        }
        
        widget.getField('CDCENTCUST').setValue('');
        widget.getField('NMCENTCUST').setValue('');
        widget.getField('NMCENTCUST').dataSourceFilter[0].value = value;
        widget.getField('NMCENTCUST').readOnly = readOnly;
        widget.getField('NMCENTCUST').reload();
        
        templateManager.updateTemplate();
    };
    
};
/*global Configuration*/
Configuration(function (ContextRegister){
    ContextRegister.register('Bco56200_LancamentoDespesa_Controller', bco56200_LancamentoDespesa_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/bco33000_gestaoFluxoCaixa_Controller.js
var bco33000_gestaoFluxoCaixa_Controller = function(ScreenService, General, templateManager, ZHPromise, restEngine, ZhFilterPreferences){

    var self = this;

    this.alteraEdicaoEFiltroContaCorrente = function(filtro){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        
        var empresa = filtro.getField('CDEMPRESA').value();
        
        if (empresa)
            if (empresa.length > 0){
                empresa = empresa.join('_');
            }else empresa = null;
        
        var dtsdoccorr = filtro.getField('DTSDOCCORR');
        var dataInicial = dtsdoccorr.value()?dtsdoccorr.value().start:dtsdoccorr.value();
        var cdreducont = filtro.getField('CDREDUCONT');
        
        cdreducont.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDEMPRESA': filtro.value = (empresa) ? empresa : 'T';
                    break;
                case 'DTSDOCCORR': filtro.value = (dataInicial) ? dataInicial : null;
                    break;
            }
        });
        
        if (dataInicial && empresa){
            cdreducont.readOnly = false;
        }else{
            cdreducont.readOnly = true;
            cdreducont.setValue('');
        }
        
        cdreducont.reload();
    };

    this.preencheValoresStripe = function(widget, preference){
        var saldoTotal = this.somaSaldoContas(preference.FILTER_ROW);
        var contas = preference.FILTER_ROW.CDREDUCONT.join(' - ');
        var stripe = templateManager.container.getWidget('stripe');
        var qtdeEmpresas = null;
        if (preference.FILTER_ROW.CDEMPRESA){
            qtdeEmpresas = preference.FILTER_ROW.CDEMPRESA.length;
        }
        var empresas = '';
        for(var i = 0; qtdeEmpresas > i; i++){
            empresas = empresas /*+ preference.FILTER_ROW.CDEMPRESA[i] + ' '*/ + preference.FILTER_ROW.NMEMPRESA[i];
            if (qtdeEmpresas - 1 != i)
                empresas = empresas + ' - ';
        }
        stripe.currentRow.EMPRESA = (empresas) ? empresas : 'Todos';
        stripe.currentRow.DTINICIAL = preference.FILTER_ROW.DTSDOCCORR.start;
        stripe.currentRow.DTFINAL = preference.FILTER_ROW.DTSDOCCORR.end;
        stripe.currentRow.SALDOTOTAL = saldoTotal;
        stripe.currentRow.CONTAS = contas;
    };

    this.somaSaldoContas = function(filtro){
        var soma = 0;
        filtro.VRSDOCCORR.forEach(function(data){
            soma += self.formataValor(data);
        });
        return self.formataValor(soma).toLocaleString('pt-BR', {minimumFractionDigits: 2});
    };
    
    /**
     * Formata Valor
     * @param valor
     */
    self.formataValor = function(valor){
        if(typeof valor == 'string')
            if(valor.indexOf(',') >= 0)
                valor = valor.replace('.','').replace(',','.');
                
        valor = parseFloat(parseFloat(valor).toFixed(2));

        return valor;
    };

    /*this.validaContaCorrente = function(){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        var filtro = gestaoFluxoCaixa.__filterPopup;
        if (filtro.getField('DTINICIAL').value()){
            filtro.getField('DSCONTCORR').readOnly = true;
        }else{
            filtro.getField('DSCONTCORR').readOnly = false;
        }
    };*/

    this.preparaFiltro = function(){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        var dataAtual = moment().format('DD/MM/YYYY');
        var filterRow = {
            'DTSDOCCORR': {start : dataAtual, end : dataAtual},
            'CONSIDVENCSCC': 'N'
        };
        ZhFilterPreferences.setOriginalFilter(gestaoFluxoCaixa.id, filterRow);
        ZhFilterPreferences.openFilterWidget(gestaoFluxoCaixa, filterRow);
    };

    this.preparaFiltroFluxoCaixa = function(preference){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        var filtro = preference.FILTER_ROW;
        var cdempresa = null;
        var cdreducont = null;
        var dataInicial = null;
        var dataFinal = null;
        if (filtro.hasOwnProperty('CDEMPRESA')){
            if (filtro.CDEMPRESA.length > 0){
                cdempresa = filtro.CDEMPRESA.join('_');
            }
        }
        if (filtro.hasOwnProperty('CDREDUCONT')){
            if (filtro.CDREDUCONT.length > 0){
                cdreducont = filtro.CDREDUCONT.join('_');
            }
        }
        if (filtro.hasOwnProperty('DTSDOCCORR')){
            if (filtro.DTSDOCCORR.hasOwnProperty('start') && filtro.DTSDOCCORR.start){
                dataInicial = filtro.DTSDOCCORR.start;
                dataFinal   = filtro.DTSDOCCORR.end;
            }
        }
        var posData = null;
        var i = 0;
        gestaoFluxoCaixa.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA': filter.value = (cdempresa) ? cdempresa : 'T';
                    break;
                ///case 'CDREDUCONT': filter.value = (cdreducont) ? cdreducont : 'T';
                    //break;
                case 'DTINICIAL': filter.value = (dataInicial) ? dataInicial : null;
                    break;
                case 'DTFINAL': filter.value = (dataFinal) ? dataFinal : null;
                    break;
                case 'DTSDOCCORR': posData = i;
                    break;
            }
            i++;
        });
        if (posData)
            gestaoFluxoCaixa.dataSourceFilter.splice(posData, 1);
        preference.CUSTOM_FILTER_ROW = Util.clone(preference.FILTER_ROW);
        preference.CUSTOM_FILTER_ROW.CDEMPRESA = cdempresa;
        preference.CUSTOM_FILTER_ROW.CDREDUCONT = cdreducont;
        if (preference.CUSTOM_FILTER_ROW.hasOwnProperty('DTSDOCCORR')){
            delete preference.CUSTOM_FILTER_ROW.DTSDOCCORR;
        } 
    };

    this.preparaFiltroOrigemFluxoCaixa = function(widget){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        var filtro = gestaoFluxoCaixa.appliedFilter;
        var empresa = null;
        if (filtro.FILTER_ROW.hasOwnProperty('CDEMPRESA')){
            if (filtro.FILTER_ROW.CDEMPRESA.value().length > 0){
                empresa = empresa.join('_');
            }
        }
        var cdreducont = filtro.FILTER_ROW.CDREDUCONT;
        var dtsdoccorr = gestaoFluxoCaixa.getField('DATAFLUXO').value();
        var considvencscc = filtro.FILTER_ROW.CONSIDVENCSCC;
        widget.dataSourceFilter.forEach(function(filter){
            switch(filter.name){
                case 'CDEMPRESA': filter.value = (empresa) ? empresa : 'T';
                    break;
                case 'CDREDUCONT': filter.value = (cdreducont) ? cdreducont.join('_') : 'T';
                    break;
                case 'DATA': filter.value = dtsdoccorr;
                    break;
                case 'CONSIDVENCSCC': filter.value = considvencscc;
                    break;

            }
        });
    };

    this.getColumnName = function(field){
        var gestaoFluxoCaixa = field.widget;
        gestaoFluxoCaixa.ORIGEMFLUXO = field.name;
    };

    this.abrirDadosPorOrigem = function(gestaoFluxoCaixa){
        var dataFluxo = gestaoFluxoCaixa.currentRow.DATAFLUXO;
        var titulosReceber = ['VRRECEBER', 'VRRECPREV', 'VRRECEFET'];
        var titulosPagar = ['VRPAGAR', 'VRPAGPREV', 'VRPAGEFET'];
        var movimentacoes = ['VRCREDITO', 'VRDEBITO'];
        if (gestaoFluxoCaixa.ORIGEMFLUXO){
            if (gestaoFluxoCaixa.getField(gestaoFluxoCaixa.ORIGEMFLUXO).value() != 0){
                if (titulosReceber.indexOf(gestaoFluxoCaixa.ORIGEMFLUXO) != '-1'){
                    this.openSwipeTitulosReceber(gestaoFluxoCaixa);
                }else if (titulosPagar.indexOf(gestaoFluxoCaixa.ORIGEMFLUXO) != '-1'){
                    this.openSwipeTitulosPagar(gestaoFluxoCaixa);
                }else if (movimentacoes.indexOf(gestaoFluxoCaixa.ORIGEMFLUXO) != '-1'){
                    this.openSwipeMovimentacoes(gestaoFluxoCaixa);
                }
            }
        }
    };

    this.openSwipeTitulosReceber = function(gestaoFluxoCaixa){
        var titulosReceber = templateManager.container.getWidget('titulosReceber');
        if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRRECEBER'){
            titulosReceber.dataSourceFilter.push({"name": "CDREDUCONT","operator": "=","value": ""});
            titulosReceber.dataSourceFilter.push({"name": "CONSIDVENCSCC","operator": "=","value": ""});
            titulosReceber.label = 'Títulos a Receber Abertos';
            titulosReceber.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletreceber_fluxoCaixaTitulosAbertos';
        }else if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRRECPREV'){
            titulosReceber.getField('CONTACORRENTE').isVisible = false;
            titulosReceber.getField('CONTACORRENTE').grouped = false;
            titulosReceber.getField('CLASSIFICACAO').isVisible = false;
            titulosReceber.getField('CLASSIFICACAO').grouped = false;
            titulosReceber.getField('VALOR').expression = "";
            titulosReceber.label = 'Títulos a Receber Previstos';
            titulosReceber.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletreceber_fluxoCaixaTitulosPrevistos';
        }else if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRRECEFET'){
            titulosReceber.dataSourceFilter.push({"name": "CDREDUCONT","operator": "=","value": ""});
            titulosReceber.label = 'Títulos a Receber Efetivados';
            titulosReceber.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletreceber_fluxoCaixaTitulosEfetivados';
        }
        this.preparaFiltroOrigemFluxoCaixa(titulosReceber);
        ScreenService.openSwipe(titulosReceber.getParent())
        .then(function(){
            titulosReceber.reload();
        })
        .catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    this.voltar = function(widget, dataSourceAberto, dataSourceEfetivado, dataSourcePrevisto){
        var dataSourceTitulosAbertos = 'fin_movimentacaoBancaria#/crud/' + dataSourceAberto;
        var dataSourceTitulosEfetivados = 'fin_movimentacaoBancaria#/crud/' + dataSourceEfetivado;
        var dataSourceTitulosPrevistos = 'fin_movimentacaoBancaria#/crud/' + dataSourcePrevisto;
        var indices = [];
        var i = 0;
        if (widget.dataSource.name == dataSourceTitulosAbertos){
            widget.dataSourceFilter.forEach(function(filtro){
                if (filtro.name == 'CDREDUCONT'){
                    indices.push(i);
                }else if (filtro.name == 'CONSIDVENCSCC'){
                    indices.push(i);
                }
                i++;
            });
            this.deleteFiltro(widget, indices);
        }
        if (widget.dataSource.name == dataSourceTitulosEfetivados){
            widget.dataSourceFilter.forEach(function(filtro){
                if (filtro.name == 'CDREDUCONT'){
                    indices.push(i);
                }
                i++;
            });
            this.deleteFiltro(widget, indices);
        }
        if (widget.dataSource.name == dataSourceTitulosPrevistos){
            widget.getField('CONTACORRENTE').isVisible = true;
            widget.getField('CONTACORRENTE').grouped = true;
            widget.getField('CLASSIFICACAO').isVisible = true;
            widget.getField('CLASSIFICACAO').grouped = true;
            widget.getField('VALOR').expression = "SUM";
        }
        widget.dataSource.data = [];
        General.widgetGoBackToList(widget);
    };

    this.deleteFiltro = function(widget, indices){
        for (var i = indices.length - 1; i >= 0; i--){
            widget.dataSourceFilter.splice(indices[i], 1);
        }
    };

    this.openSwipeTitulosPagar = function(gestaoFluxoCaixa){
        var titulosPagar = templateManager.container.getWidget('titulosPagar');
        if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRPAGAR'){
            titulosPagar.dataSourceFilter.push({"name": "CDREDUCONT","operator": "=","value": ""});
            titulosPagar.dataSourceFilter.push({"name": "CONSIDVENCSCC","operator": "=","value": ""});
            titulosPagar.label = 'Títulos a Pagar Abertos';
            titulosPagar.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletpagar_fluxoCaixaTitulosAbertos';
        }else if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRPAGPREV'){
            titulosPagar.getField('CONTACORRENTE').isVisible = false;
            titulosPagar.getField('CONTACORRENTE').grouped = false;
            titulosPagar.getField('CLASSIFICACAO').isVisible = false;
            titulosPagar.getField('CLASSIFICACAO').grouped = false;
            titulosPagar.getField('VALOR').expression = "";
            titulosPagar.label = 'Títulos a Pagar Previstos';
            titulosPagar.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletpagar_fluxoCaixaTitulosPrevistos';
        }else if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRPAGEFET'){
            titulosPagar.dataSourceFilter.push({"name": "CDREDUCONT","operator": "=","value": ""});
            titulosPagar.label = 'Títulos a Pagar Efetivados';
            titulosPagar.dataSource.name = 'fin_movimentacaoBancaria#/crud/boletpagar_fluxoCaixaTitulosEfetivados';
        }
        this.preparaFiltroOrigemFluxoCaixa(titulosPagar);
        ScreenService.openSwipe(titulosPagar.getParent())
        .then(function(){
            titulosPagar.reload();
        })
        .catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    this.openSwipeMovimentacoes = function(gestaoFluxoCaixa){
        var movimentacoes = templateManager.container.getWidget('movimentacoes');
        if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRCREDITO'){
            movimentacoes.label = 'Movimentações de Crédito';
            movimentacoes.dataSource.name = 'fin_movimentacaoBancaria#/crud/movbancario_fluxoCaixaMovimentosCredito';
        }else if (gestaoFluxoCaixa.ORIGEMFLUXO == 'VRDEBITO'){
            movimentacoes.label = 'Movimentações de Débito';
            movimentacoes.dataSource.name = 'fin_movimentacaoBancaria#/crud/movbancario_fluxoCaixaMovimentosDebito';
        }
        this.preparaFiltroOrigemFluxoCaixa(movimentacoes);
        ScreenService.openSwipe(movimentacoes.getParent())
        .then(function(){
            movimentacoes.reload().then(function(){
                /*this.atualizarWidget(movimentacoes).then(function(){
                    movimentacoes.groupedData.forEach(function(group){
                        group.__groupOpened = false;
                    });
                }.bind(this));*/
            }.bind(this));
        }.bind(this))
        .catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };

    this.atualizarWidget = function(movimentacoes){
        var deferred = ZHPromise.defer();
        movimentacoes.redraw();
        deferred.resolve();
        return deferred.promise;
    };

    this.calculaSaldoDiaEAcumulado = function(widget){
        var gestaoFluxoCaixa = templateManager.container.getWidget('gestaoFluxoCaixa');
        var saldoTotalContas = this.somaSaldoContas(gestaoFluxoCaixa.appliedFilter.FILTER_ROW);
        var saldoAnterior = saldoTotalContas;
        var saldoAnteriorReal = saldoTotalContas;
        var saldoDia = 0;
        var saldoDiaReal = 0;
        var acumulativo = 0;
        var acumulativoReal = 0;
        widget.dataSource.data.forEach(function(data){
            saldoDiaReal = parseFloat(((data.VRRECEFET + data.VRCREDITO) - (data.VRPAGEFET + data.VRDEBITO)).toFixed(2));
            saldoDia = parseFloat(((data.VRRECEBER + data.VRRECPREV + data.VRRECEFET + data.VRCREDITO) - (data.VRPAGAR + data.VRPAGPREV + data.VRPAGEFET + data.VRDEBITO)).toFixed(2));
            acumulativo = parseFloat((saldoAnterior + saldoDia).toFixed(2));
            acumulativoReal = parseFloat((saldoAnteriorReal + saldoDiaReal).toFixed(2));
            data.SALDODIA = saldoDia;
            data.SALDODIAREAL = saldoDiaReal;
            data.ACUMULATIVO = acumulativo;
            data.ACUMULATIVOREAL = acumulativoReal;
            saldoAnterior = acumulativo;
            saldoAnteriorReal = acumulativoReal;
        });
    };
    
    this.executaEvolucaoFluxoCaixa = function(widget) {
        if(widget.isValid()){
            General.callRestEngine('fin#/evolucaoFluxoCaixa', 'Row', widget.currentRow).then(function (result) {
                ScreenService.closePopup();
                ScreenService.showMessage('Evolução de fluxo de caixa efetuada com sucesso.');
            }).catch(function(erro){
                ScreenService.showMessage(erro.data.error);
            });
        }
    };
};

Configuration(function (ContextRegister){
    ContextRegister.register('bco33000_gestaoFluxoCaixa_Controller', bco33000_gestaoFluxoCaixa_Controller);
});

// FILE: ./assets/modules/movimentacaoBancaria/assets/js/controllers/lancamentoDeCredito_Controller.js
var  lancamentoDeCredito_Controller = function(ScreenService, General, templateManager, ZHPromise, localStorageEngine, contabilizacaoController, restEngine, ZhFilterPreferences){
    
    var self = this;
    
    self.adicionaNovoLanc = function(){
        var widget = templateManager.container.getWidget('widgetLancamento');
        widget.newRow();
    };
    
    self.activeCentCust = function(){
        var widget = templateManager.container.getWidget('widgetLancamento');
        if(widget.currentRow.CDFILIAL){
            widget.currentRow.NMCENTCUST = "";
            widget.getField('NMCENTCUST').dataSourceFilter[0].value = widget.currentRow.CDFILIAL;
            widget.getField('NMCENTCUST').readOnly = false;
        }
        else{
            widget.currentRow.CDFILIAL = "";
            widget.currentRow.NMCENTCUST = "";
            widget.getField('NMCENTCUST').readOnly = true;
        }
    };
    
    self.formatFilter = function(widget, currentRowFiltro) {
    
        var cdfilial = currentRowFiltro.CDFILIAL;
        var dtlancsald = currentRowFiltro.DTLANCSALD;
        
        widget.dataSourceFilter.forEach(function(filtro){
            switch(filtro.name){
                case 'CDFILIAL': filtro.value = cdfilial;
                    break;
                case 'DATAINI': filtro.value = dtlancsald == null ? dtlancsald : dtlancsald.start;
					break;
				case 'DATAFIN': filtro.value = dtlancsald == null ? dtlancsald : dtlancsald.end;
				    break;
				case 'DTTODOS': filtro.value = dtlancsald == null ? 'T': null;
            }       
        });
    };
    
    self.salvarLancamento = function(widget){
        if(widget.isValid()){
            General.callRestEngine('fin_movimentacaoBancaria#/saveLancamentoCredito', 'Row', widget.currentRow).then(function(result){
                ScreenService.showMessage("Lançamento Salvo com sucesso.");
                ScreenService.closeSwipe();
                widget.reload();
            }).catch(function(error){
                if (contabilizacaoController.checkErrosContabilizacao(error, "Não foi possível realizar o lançamento devido a erros na contablização.")) {
                    ScreenService.showMessage(error.data.error);
                }
            });
        }
    };
    
    self.deletarLancamento = function(widget){
        General.callRestEngine('fin_movimentacaoBancaria#/removeLancamentoCredito', 'Row', widget.currentRow).then(function(result){
            ScreenService.showMessage("Lançamento Deletado com sucesso.");
            ScreenService.closeSwipe();
            widget.reload();
        }).catch(function(erro){
            ScreenService.showMessage(erro.data.error);
        });
    };
};

Configuration(function(ContextRegister){
    ContextRegister.register('lancamentoDeCredito_Controller', lancamentoDeCredito_Controller);
});

// FILE: ./assets/modules/relatoriosPagar/assets/js/controllers/Pag11001_RelatorioCustomizavelTitulosPagarPagos_Controller.js
function Pag11001_RelatorioCustomizavelTitulosPagarPagos_Controller(General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    var self = this;
    
    self.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };
     
    self.openFilterTitulosAPagar = function(widgetTitulosAPagar){
        self.buscaDadosIniciaisFiltro(widgetTitulosAPagar).then(function(){
            var rowSetFilter = {
                'DTVENBOLPAG' : widgetTitulosAPagar.DTVENBOLPAG,
                'CDEMPRESA'   : [widgetTitulosAPagar.empresa.CDEMPRESA]
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAPagar) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAPagar).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAPagar).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralPag = function(){
        return General.callRestEngine(
            'fin#/crud/geral_pag',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        ); 
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralpag     = self.buscaGeralPag();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralpag, paramestoque]).then(function(response){
            geralpag     = response[0].dataset.geral_pag[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralpag?geralpag.QTDIAANTBOLP?geralpag.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralpag?geralpag.QTDIAAPOBOLP?geralpag.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLPAG = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    
    self.buildFilter = function (widget, preference) {
        var filtro = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;
        
        validateStartEndValues(filterRow);
        
        filtro.forEach(function(fil, index) {
            switch (fil.name) {
                case 'CDEMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_") : null;
                    break;
                case 'DTVENBOLPAGINI':
                    fil.value = filterRow.DTVENBOLPAG ? filterRow.DTVENBOLPAG.start : null;
                    break;
                case 'DTVENBOLPAGFIN':
                    fil.value = filterRow.DTVENBOLPAG ? filterRow.DTVENBOLPAG.end : null;
                    break;
                case 'DTEMISSAOINI':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.start : null;
                    break;
                case 'DTEMISSAOFIN':
                    fil.value = filterRow.DTEMISSAO ? filterRow.DTEMISSAO.end : null;
                    break;
                case 'DTENTRSAIDINI':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.start : null;
                    break;
                case 'DTENTRSAIDFIN':
                    fil.value = filterRow.DTENTRSAID ? filterRow.DTENTRSAID.end : null;
                    break;
                case 'DTCNABPAGINI':
                    fil.value = filterRow.DTCNABPAG ? filterRow.DTCNABPAG.start : null;
                    break;
                case 'DTCNABPAGFIN':
                    fil.value = filterRow.DTCNABPAG ? filterRow.DTCNABPAG.end : null;
                    break;
                case 'DTBAXINICIAL':
                    fil.value = filterRow.DTBAIXAPAG ? filterRow.DTBAIXAPAG.start : null;
                    break;
                case 'DTBAXFINAL':
                    fil.value = filterRow.DTBAIXAPAG ? filterRow.DTBAIXAPAG.end : null;
                    break;
                case 'TDTBAIXA':
                    fil.value = filterRow.DTBAIXAPAG ? filterRow.DTBAIXAPAG.start ? null : 'T' : 'T';
                    break;
                case 'DTLBAINICIAL':
                    fil.value = filterRow.DTLANCBAIXAP ? filterRow.DTLANCBAIXAP.start : null;
                    break;
                case 'DTLBAFINAL':
                    fil.value = filterRow.DTLANCBAIXAP ? filterRow.DTLANCBAIXAP.end : null;
                    break;
                case 'TDTLBAIXA':
                    fil.value = filterRow.DTLANCBAIXAP ? filterRow.DTLANCBAIXAP.start ? null : 'T' : 'T';
                    break;
                case 'VRBAIXAPAGINI':
                    fil.value = filterRow.VRBAIXAPAGINI && filterRow.VRBAIXAPAGFIN ? filterRow.VRBAIXAPAGINI : null;
                    break;
                case 'VRBAIXAPAGFIN':
                    fil.value = filterRow.VRBAIXAPAGINI && filterRow.VRBAIXAPAGFIN ? filterRow.VRBAIXAPAGFIN : null;
                    break;
                case 'TVRBAIXA':
                    fil.value = filterRow.VRBAIXAPAGINI && filterRow.VRBAIXAPAGFIN ? null : 'T';
                    break;
            }
        });
        filtro = getDataSourceWithoutIntervalFieldNames(filtro);
        
        preference.CUSTOM_FILTER_ROW = {
            'CDFORNECED'   : preference.FILTER_ROW.CDFORNECED,
            'IDSTATBPAG'   : preference.FILTER_ROW.IDSTATBPAG,
            'IDHABIBPAG'   : preference.FILTER_ROW.IDHABIBPAG,
            'CDFILIRECE'   : preference.FILTER_ROW.CDFILIRECE,
            'CDFILIALNF'   : preference.FILTER_ROW.CDFILIALNF,
            'CDTPCTPAGAR'  : preference.FILTER_ROW.CDTPCTPAGAR,
            'CDCLASSFINA'  : preference.FILTER_ROW.CDCLASSFINA,
            'CDTIPOOPER'   : preference.FILTER_ROW.CDTIPOOPER,
            'IDSTATUS'     : preference.FILTER_ROW.IDSTATUS,
            'IDSITUCNABP'  : preference.FILTER_ROW.IDSITUCNABP,
            'CDTPBAIXAPAG' : preference.FILTER_ROW.CDTPBAIXAPAG
        };
    };
    
    self.changeInputValues = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        validateStartEndValues(filterRow);
    };
    
    //Limpa os nomes dos campos invterval do dataSource
    var getDataSourceWithoutIntervalFieldNames = function(filtro){
        for(var i = 0; i < filtro.length; i++) {
            if(filtro[i].name === 'DTEMISSAO' ||
               filtro[i].name === 'DTENTRSAID' ||
               filtro[i].name === 'DTVENBOLPAG' ||
               filtro[i].name === 'DTCNABPAG' ||
               filtro[i].name === 'DTBAIXAPAG' ||
               filtro[i].name === 'DTLANCBAIXAR' ) {
                filtro.splice(i, 1);
                i--;
            }
        }
        return filtro;
    };
    
    /*
     * Verificação para igualar valores iniciais e finais do filtro, no caso
     * do usuário preencher um e deixar o outro em branco. Isso irá fazer
     * com que a query retorne somente resultados que sejam exatamente os  
     * mesmos valores inseridos em um dos campos.
     */
    var validateStartEndValues = function(filterRow) {
        // Verifica valor do boleto inicial e final.
        filterRow.VRBOLPAGINI = filterRow.VRBOLPAGINI || filterRow.VRBOLPAGFIN || null;
        filterRow.VRBOLPAGFIN = filterRow.VRBOLPAGFIN || filterRow.VRBOLPAGINI;
        
        // Verifica valor líquido inicial e final.
        filterRow.VRFINALBPAGINI = filterRow.VRFINALBPAGINI || filterRow.VRFINALBPAGFIN || null;
        filterRow.VRFINALBPAGFIN = filterRow.VRFINALBPAGFIN || filterRow.VRFINALBPAGINI;
        
        // Verifica nota fiscal inicial e final.
        filterRow.NRNOTAFISCINI = filterRow.NRNOTAFISCINI || filterRow.NRNOTAFISCFIN || null;
        filterRow.NRNOTAFISCFIN = filterRow.NRNOTAFISCFIN || filterRow.NRNOTAFISCINI;
        
        // Verifica valor da baixa inicial e final.
        filterRow.VRBAIXAPAGINI = filterRow.VRBAIXAPAGINI || filterRow.VRBAIXAPAGFIN || null;
        filterRow.VRBAIXAPAGFIN = filterRow.VRBAIXAPAGFIN || filterRow.VRBAIXAPAGINI;
        
        // Verifica número remessa inicial e final.
        filterRow.NRCNABPAGREMINI = filterRow.NRCNABPAGREMINI || filterRow.NRCNABPAGREMFIN || null;
        filterRow.NRCNABPAGREMFIN = filterRow.NRCNABPAGREMFIN || filterRow.NRCNABPAGREMINI;
        
        // Verifica número retorno inicial e final.
        filterRow.NRCNABPAGRETINI = filterRow.NRCNABPAGRETINI || filterRow.NRCNABPAGRETFIN || null;
        filterRow.NRCNABPAGRETFIN = filterRow.NRCNABPAGRETFIN || filterRow.NRCNABPAGRETINI;
        
        // Verifica valor remessa inicial e final.
        filterRow.VRTOTBOLCNABPAGINI = filterRow.VRTOTBOLCNABPAGINI || filterRow.VRTOTBOLCNABPAGFIN || null;
        filterRow.VRTOTBOLCNABPAGFIN = filterRow.VRTOTBOLCNABPAGFIN || filterRow.VRTOTBOLCNABPAGINI;
    };
    
    this.changeHeaderValues = function(widget){
        /*var codigo; 
        var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "is equal to".concat(codigo).concat(' - ').concat(nmempresa)
        };*/
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        //newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "é igual a."){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
        
    };
    
    
    self.onChangeEmpresaFilter = function(widget){
        var cdempresa = widget.currentRow.CDEMPRESA;
        var cdtpctpagar = widget.getField('CDTPCTPAGAR');
        
        cdtpctpagar.setValue("");
        if(Array.isArray(cdempresa)){
            cdtpctpagar.dataSourceFilter[0].value = "";
            cdempresa.forEach(function(empresa){
                cdtpctpagar.dataSourceFilter[0].value = cdtpctpagar.dataSourceFilter[0].value+empresa+'_';
            });
            cdtpctpagar.dataSourceFilter[0].value = '_'+cdtpctpagar.dataSourceFilter[0].value;
        } else {
            cdtpctpagar.dataSourceFilter[0].value = cdempresa;
        }
        cdtpctpagar.reload();
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Pag11001_RelatorioCustomizavelTitulosPagarPagos_Controller', Pag11001_RelatorioCustomizavelTitulosPagarPagos_Controller);
});

// FILE: ./assets/modules/relatoriosPagar/assets/js/controllers/Pag11002_RelatorioCustomizavelTitulosPagarPagos_Controller.js
function Pag11002_RelatorioCustomizavelTitulosPagarPagos_Controller (General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    var self = this; 
    
    self.setNameLayout = function(args){
        var widget = args.owner;
        if (!widget.hasOwnProperty('labelPrincipal')){
            widget.labelPrincipal = widget.label;
        }
        if (args.e.layout.ID == 'ORIGINAL'){
            widget.container.label = widget.labelPrincipal;
            widget.label = widget.labelPrincipal;
        }else{
            widget.container.label = args.e.layout.LABEL_TO_PREFERENCE;
            widget.label = args.e.layout.LABEL_TO_PREFERENCE;
        }
    };
     
    self.openFilterTitulosAPagar = function(widgetTitulosAPagar){
        self.buscaDadosIniciaisFiltro(widgetTitulosAPagar).then(function(){
            var rowSetFilter = {
                //'DTVENBOLPAG' : widgetTitulosAPagar.DTVENBOLPAG,
                'CDEMPRESA'   : widgetTitulosAPagar.empresa.CDEMPRESA,
                'NMEMPRESA'   : widgetTitulosAPagar.empresa.NMEMPRESA
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAPagar) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAPagar).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAPagar).then(function(){
                deferred.resolve();        
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa';
            var filterEmpresa = [{"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA}];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralPag = function(){
        return General.callRestEngine(
            'fin#/crud/geral_pag',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        );
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralpag     = self.buscaGeralPag();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralpag, paramestoque]).then(function(response){
            geralpag     = response[0].dataset.geral_pag[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralpag?geralpag.QTDIAANTBOLP?geralpag.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralpag?geralpag.QTDIAAPOBOLP?geralpag.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            //widget.DTVENBOLPAG = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
    
    self.buildFilter = function (widget, preference) {
        var filtro = widget.dataSourceFilter;
        var filterRow = preference.FILTER_ROW;
        filtro.forEach(function(fil) {
            if (Array.isArray(fil.value)) {
                fil.value = fil.value.join("_");
            }
            switch (fil.name) {
                
                case 'VALORINI':
                    fil.value = filterRow.VALORINI ? filterRow.VALORINI : null;
                    break;
                
                case 'VALORFIN':
                    fil.value = filterRow.VALORFIN ? filterRow.VALORFIN : null;
                    break;
                
                case 'DATENTR':
                    fil.value = filterRow.ENTRADA ? filterRow.ENTRADA.start : null;
                    break;
                
                case 'DATSAID':
                    fil.value = filterRow.ENTRADA ? filterRow.ENTRADA.end : null;
                    break;
                
                case 'DATEMISINI':
                    fil.value = filterRow.EMISSAO ? filterRow.EMISSAO.start : null;
                    break;
                
                case 'DATEMIFIN':
                    fil.value = filterRow.EMISSAO ? filterRow.EMISSAO.end : null;
                    break;
                
                case 'DTATUAVENPAGFIN':
                    fil.value = filterRow.VENCIMENTO ? filterRow.VENCIMENTO.start : null;
                    break;
                
                case 'DTATUAVENPAGINI':
                    fil.value = filterRow.VENCIMENTO ? filterRow.VENCIMENTO.end : null;
                    break;
                
                case 'IDTPCTPAGAR':
                    fil.value = filterRow.IDTPCTPAGAR ? filterRow.IDTPCTPAGAR : null;
                    break;
                
                case 'CODFILILANC':
                    fil.value = filterRow.CDFILIAL ? filterRow.CDFILIAL : 'T';
                    break;
                
                case 'STATUS':
                    fil.value = filterRow.STATUS ? filterRow.STATUS : 'T';
                    break;
                
                case 'TIPO':
                    fil.value = filterRow.IDSTATUS ? filterRow.IDSTATUS : 'A_E'; 
                    break;
                
                case 'TIPOCONTA':
                    fil.value = filterRow.CDTPCTPAGAR ? filterRow.CDTPCTPAGAR : 'T';
                    break;
                
                case 'P_CLASSFINA':
                    fil.value = filterRow.CDCLASSFINA ? filterRow.CDCLASSFINA: 'T';
                    break;
                
                case 'IDBLOQNFFIN':
                    if(filterRow.IDBLOQNFFIN == null){
                        filterRow.IDBLOQNFFIN = 'L';
                    }
                    fil.value = filterRow.IDBLOQNFFIN ? filterRow.IDBLOQNFFIN : 'L';
                    break;
                
                case 'HABILITA':
                    fil.value = filterRow.IDHABIBPAG ? filterRow.IDHABIBPAG : 'T';
                    break;
                
                case 'P_FORMA':
                    fil.value = filterRow.CDTPBAIXAPAG ? filterRow.CDTPBAIXAPAG : 'T';
                    break;
                
                case 'P_CCORR':
                    fil.value = filterRow.CDREDUCONT ? filterRow.CDREDUCONT : 'T';
                    break;
                                
                case 'P_TOPERLANC':
                    fil.value = filterRow.CDOPERACTPG ? null : 'T';
                    break;
                
                case 'P_OPERLANC':
                    fil.value = filterRow.CDOPERACTPG ? filterRow.CDOPERACTPG : null;
                    break;
                
                 case 'P_TCDTIPOOPER':
                    fil.value = filterRow.CDTIPOOPER ? null : 'T';
                    break;
                
                case 'P_CDTIPOOPER':
                    fil.value = filterRow.CDTIPOOPER ? filterRow.CDTIPOOPER : null;
                    break;
                    
                case 'P_PRIPUB':
                    fil.value = filterRow.P_PRIPUB ? filterRow.P_PRIPUB : 'T';
                    break;
                    
                case 'IDSISTEMA':
                    fil.value = filterRow.IDSISTEMA ? filterRow.IDSISTEMA : 'T';
                    break;
                    
                case 'P_EMPRESA':
                    fil.value = filterRow.CDEMPRESA ? filterRow.CDEMPRESA.join("_"): null;
                    break;
            }
            
        });
        delete filterRow.CDEMPRESA;
        delete filterRow.NMEMPRESA;
        filtro = getDataSourceWithoutIntervalFieldNames(filtro);
    };
    
    self.changeInputValues = function(widget, preference) {
        var filterRow = preference.FILTER_ROW;
        validateStartEndValues(filterRow);
    };
    
    //Limpa os nomes dos campos invterval do dataSource
    var getDataSourceWithoutIntervalFieldNames = function(filtro){
        for(var i = 0; i < filtro.length; i++) {
            if(filtro[i].name === 'DTEMISSAO' ||
               filtro[i].name === 'DTENTRSAID' ||
               filtro[i].name === 'DTVENBOLPAG' ||
               filtro[i].name === 'DTBAIXAPAG' ) {
                filtro.splice(i, 1);
                i--;
            }
        } 
        return filtro;
    };
    
    /*
     * Verificação para igualar valores iniciais e finais do filtro, no caso
     * do usuário preencher um e deixar o outro em branco. Isso irá fazer
     * com que a query retorne somente resultados que sejam exatamente os  
     * mesmos valores inseridos em um dos campos.
     */
    var validateStartEndValues = function(filterRow) {
        // Verifica valor líquido inicial e final.
        filterRow.VRFINALBPAGINI = filterRow.VRFINALBPAGINI || filterRow.VRFINALBPAGFIN || null;
        filterRow.VRFINALBPAGFIN = filterRow.VRFINALBPAGFIN || filterRow.VRFINALBPAGINI;
        
        // Verifica nota fiscal inicial e final.
        filterRow.NRNOTAFISCINI = filterRow.NRNOTAFISCINI || filterRow.NRNOTAFISCFIN || null;
        filterRow.NRNOTAFISCFIN = filterRow.NRNOTAFISCFIN || filterRow.NRNOTAFISCINI;
        
        // Verifica valor da baixa inicial e final.
        filterRow.VRBAIXAPAGINI = filterRow.VRBAIXAPAGINI || filterRow.VRBAIXAPAGFIN || null;
        filterRow.VRBAIXAPAGFIN = filterRow.VRBAIXAPAGFIN || filterRow.VRBAIXAPAGINI;
        
        // Verifica número remessa inicial e final.
        // Verifica valor remessa inicial e final.
        filterRow.VRTOTBOLCNABPAGINI = filterRow.VRTOTBOLCNABPAGINI || filterRow.VRTOTBOLCNABPAGFIN || null;
        filterRow.VRTOTBOLCNABPAGFIN = filterRow.VRTOTBOLCNABPAGFIN || filterRow.VRTOTBOLCNABPAGINI;
    };
    
    
    
    this.changeHeaderValues = function(widget){
        /*var codigo; 
        var nmempresa = widget.owner.dataSource.data[0].NMEMPRESA;
        widget.owner.dataSourceFilter.forEach(function(empresa){
           if(empresa.name == 'CDEMPRESA'){
               codigo = empresa.value;
           } 
        });
        
        var cdempresa = {
            'label' : "Empresa",
            'value' : "is equal to".concat(codigo).concat(' - ').concat(nmempresa)
        };*/
        
        var filter = widget.e.metadata.filter;
        var newFilter = [];
        //newFilter.push(cdempresa);
        filter.forEach(function(x){
            if(x.value != "é igual a."){
                newFilter.push(x);
            }
        });
        
        var correctFilter = [];
        var value;
        newFilter.forEach(function(aux){
            if(aux.value.match(/é igual a/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }else if(aux.value.match(/está entre/)){
                value = aux.value.substr(11);
                aux.value = value;
                correctFilter.push(aux);
            }
        });
        
        widget.e.metadata.filter = correctFilter;    
        
        
    };
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Pag11002_RelatorioCustomizavelTitulosPagarPagos_Controller', Pag11002_RelatorioCustomizavelTitulosPagarPagos_Controller);
});

// FILE: ./assets/modules/relatoriosPagar/assets/js/controllers/Pag11004_RelatorioCustomizavelRateiosTitulosPagarPagos_Controller.js
function Pag11004_RelatorioCustomizavelRateiosTitulosPagarPagos_Controller(General, ScreenService, templateManager, ZHPromise, ZhFilterPreferences, restEngine) {
    
    var self = this;
    
    self.openFilterTitulosAPagar = function(widgetTitulosAPagar){
        self.buscaDadosIniciaisFiltro(widgetTitulosAPagar).then(function(){
            var rowSetFilter = {
                'DTVENBOLPAG' : widgetTitulosAPagar.DTVENBOLPAG,
                'CDEMPRESA'   : widgetTitulosAPagar.empresa.CDEMPRESA,
                'NMEMPRESA'   : widgetTitulosAPagar.empresa.NMEMPRESA
            };
            ZhFilterPreferences.setOriginalFilter(widgetTitulosAPagar.id, rowSetFilter);
            ZhFilterPreferences.openFilterWidget(widgetTitulosAPagar, rowSetFilter);
            
        });

    };
    
    self.buscaDadosIniciaisFiltro = function(widgetTitulosAPagar) {
        var deferred = ZHPromise.defer();
        self.buscaPadraoEmpresa(widgetTitulosAPagar).then(function(){
            self.buscaPadraoIntervaloVencimento(widgetTitulosAPagar).then(function(){
                deferred.resolve();
            });    
        });
        
        return deferred.promise;
    };
    
    self.buscaPadraoEmpresa = function(widget) {
        var deferred = ZHPromise.defer();
        var filterFilial = [{"name": "CDFILIAL", "operator": "=", "value": "FILIAL_LOGADA"}];
        var routeFilial = 'fin#/crud/filial';
        General.callRestEngine(routeFilial, 'FilterData', filterFilial).then(function(response_filial){
            var filial   = response_filial.dataset.filial[0];
            var routeEmpresa = 'fin#/crud/empresa_filial_filioper';
            var filterEmpresa = [
                {"name": "CDEMPRESA", "operator": "=", "value": filial.CDEMPRESA},
                {"name": "CDOPERADOR", "operator": "=", "value": getLocalVar('CDOPERADOR')}
            ];
            General.callRestEngine(routeEmpresa, 'FilterData', filterEmpresa).then(function(response_empresa){
                var empresa = response_empresa.dataset.empresa_filial_filioper[0];
                widget.empresa = empresa;
                deferred.resolve();
            });
        });
        
        return deferred.promise;
    };
    
    self.buscaGeralPag = function(){
        return General.callRestEngine(
            'fin#/crud/geral_pag',
            'FilterData',
            []
        );
    };
    
    self.buscaParamEstoque = function(cdFilialOpcional) {
        var filter = [{"name": "CDFILIAL", "operator": "=", "value": 'FILIAL_LOGADA'}];
        return General.callRestEngine(
            'fin#/crud/paramestoque_processamento',
            'FilterData',
            filter
        ); 
    };
    
    self.buscaPadraoIntervaloVencimento = function (widget){
        var deferred = ZHPromise.defer();
        var geralpag     = self.buscaGeralPag();
        var paramestoque = self.buscaParamEstoque();

        ZHPromise.all([geralpag, paramestoque]).then(function(response){
            geralpag     = response[0].dataset.geral_pag[0];
            paramestoque = response[1].dataset.paramestoque_processamento[0];

            var qtdiaantlanr = geralpag?geralpag.QTDIAANTBOLP?geralpag.QTDIAANTBOLP:0:0;
            var qtdiaapolanr = geralpag?geralpag.QTDIAAPOBOLP?geralpag.QTDIAAPOBOLP:0:0;
            var dtprocessa   = paramestoque?paramestoque.DTPROCESSA?paramestoque.DTPROCESSA:moment():moment();

            var start = moment(dtprocessa, 'DD/MM/YYYY').subtract(qtdiaantlanr, 'days').format('DD/MM/YYYY');
            var end   = moment(dtprocessa, 'DD/MM/YYYY').add(qtdiaapolanr, 'days').format('DD/MM/YYYY');

            widget.DTVENBOLPAG = {start : start, end : end};
            deferred.resolve();
        });
        return deferred.promise;
    };
    
}

Configuration(function (ContextRegister) {
	ContextRegister.register('Pag11004_RelatorioCustomizavelRateiosTitulosPagarPagos_Controller', Pag11004_RelatorioCustomizavelRateiosTitulosPagarPagos_Controller);
});
